@@ -1,6 +1,11 @@
-# $Id: init.tcl 6040 2007-02-03 19:46:06Z m9710797 $
+# $Id: init.tcl 8378 2008-12-01 17:53:53Z m9710797 $
 
-# internal proc to make help function available to TCL procs
+# Only execute this script once. Below we source other Tcl script,
+# so this makes sure we don't get in an infinte loop.
+if [info exists ::__init_tcl_executed] return
+set ::__init_tcl_executed true
+
+# internal proc to make help function available to Tcl procs
 proc __help { args } {
 	set command [lindex $args 0]
 	if [info exists ::__help_proc($command)] {
@@ -16,8 +21,10 @@ proc __help { args } {
 proc set_help_text { command help } {
 	set ::__help_text($command) $help
 }
+set_help_text set_help_text \
+{Associate a help-text with a Tcl proc. This is normally only used in Tcl scripts.}
 
-# internal proc to make tabcompletion available to TCL procs
+# internal proc to make tabcompletion available to Tcl procs
 proc __tabcompletion { args } {
 	set command [lindex $args 0]
 	set result ""
@@ -37,6 +44,8 @@ proc set_tabcompletion_proc { command proc {case_sensitive true} } {
 		set ::__tabcompletion_proc_insensitive($command) $proc
 	}
 }
+set_help_text set_tabcompletion_proc \
+{Provide a way to do tab-completion for a certain Tcl proc. For details look at the numerous examples in the share/scripts directory. This is normally only used in Tcl scripts.}
 
 set_help_text data_file \
 "Resolve data file. First try user directory, if the file doesn't exist
@@ -50,15 +59,11 @@ proc data_file { file } {
 
 # Source all .tcl files in user and system scripts directory. Prefer
 # the version in the user directory in case a script exists in both
-set user_scripts [glob -dir $env(OPENMSX_USER_DATA)/scripts -tails -nocomplain *.tcl]
-set system_scripts [glob -dir $env(OPENMSX_SYSTEM_DATA)/scripts -tails -nocomplain *.tcl]
-foreach script [lsort -unique [concat $user_scripts $system_scripts]] {
-	set script [data_file scripts/$script]
-	if {[catch {source $script}]} {
-		puts stderr "Error while executing $script\n$errorInfo"
+set __user_scripts [glob -dir $env(OPENMSX_USER_DATA)/scripts -tails -nocomplain *.tcl]
+set __system_scripts [glob -dir $env(OPENMSX_SYSTEM_DATA)/scripts -tails -nocomplain *.tcl]
+foreach __script [lsort -unique [concat $__user_scripts $__system_scripts]] {
+	set __script [data_file scripts/$__script]
+	if {[catch {source $__script}]} {
+		puts stderr "Error while executing $__script\n$errorInfo"
 	}
 }
-
-# Execute the init.tcl file in the user's directory (if it exists)
-set user_init_tcl $env(OPENMSX_USER_DATA)/init.tcl
-if [file exists $user_init_tcl] { source $user_init_tcl }
