@@ -1,10 +1,16 @@
+set_help_text guess_title \
+{Guess the title of the current game.
+This proc uses some heuristics to guess the name of the current game, based on
+the inserted ROM cartridges or disks.
+}
+
 proc guess_rom_title { ps ss } {
 	# check device name at address #4000 in given slot
 	set slots [split [slotmap] \n]
 	set slot_name "$ps"
 	if [machine_info issubslotted $ps] { append slot_name ".$ss" }
 	set index [lsearch $slots "slot $slot_name:"]
-	if {$index != -1} { 
+	if {$index != -1} {
 		set rom [lrange [lindex $slots [expr $index + 2]] 1 end]
 		if { $rom != "empty" } { return $rom }
 	}
@@ -30,7 +36,7 @@ proc guess_title { { fallback "" } } {
 			}
 		}
 	}
-	
+
 	# then try disks
 	foreach drive [list "diska" "diskb"] {
 		set title [guess_disk_title $drive]
