@@ -1,4 +1,4 @@
-# $Id: text_echo.tcl 5354 2006-05-03 21:22:27Z mthuurne $
+# $Id: text_echo.tcl 8378 2008-12-01 17:53:53Z m9710797 $
 
 set_help_text text_echo \
 {Echoes all characters printed in text mode to stderr, meaning they will appear on the command line that openMSX was started from.
@@ -8,12 +8,12 @@ proc text_echo {} {
 	set ::text_echo_graph 0
 	set ::text_echo_escape 0
 	set ::text_echo_escape_count 0
-	debug set_bp 0x0018 {} { text_echo_print }
-	debug set_bp 0x00A2 {} { text_echo_print }
+	debug set_bp 0x0018 {} { __text_echo_print }
+	debug set_bp 0x00A2 {} { __text_echo_print }
 	return ""
 }
 
-proc text_echo_print { } {
+proc __text_echo_print { } {
 	set slot [ get_selected_slot 0 ]
 	if { $slot == "0 0" || $slot == "0 X" } {
 		set char [ reg A ]
@@ -29,7 +29,7 @@ proc text_echo_print { } {
 					set ::text_echo_escape_count 1
 				}
 			} else {
-				decr ::text_echo_escape_count
+				incr ::text_echo_escape_count -1
 				if { $::text_echo_escape_count == 0 } {
 					set ::text_echo_escape 0
 				}
