@@ -1,8 +1,8 @@
-// $Id: CartridgeSlotManager.cc 6821 2007-08-08 08:27:27Z m9710797 $
+// $Id: CartridgeSlotManager.cc 8412 2008-12-20 19:41:39Z m9710797 $
 
 #include "CartridgeSlotManager.hh"
 #include "MSXMotherBoard.hh"
-#include "ExtensionConfig.hh"
+#include "HardwareConfig.hh"
 #include "RecordedCommand.hh"
 #include "CommandException.hh"
 #include "CommandController.hh"
@@ -11,7 +11,7 @@
 #include "MSXException.hh"
 #include "StringOp.hh"
 #include "openmsx.hh"
-#include "MSXCliComm.hh"
+#include "CliComm.hh"
 #include <cassert>
 
 using std::string;
@@ -25,16 +25,38 @@ class CartCmd : public RecordedCommand
 public:
 	CartCmd(CartridgeSlotManager& manager, MSXMotherBoard& motherBoard,
 	        const std::string& commandName);
-	virtual string execute(const vector<string>& tokens, const EmuTime& time);
+	virtual string execute(const vector<string>& tokens, EmuTime::param time);
 	virtual string help(const vector<string>& tokens) const;
 	virtual void tabCompletion(vector<string>& tokens) const;
 private:
-	const ExtensionConfig* getExtensionConfig(const string& cartname);
+	const HardwareConfig* getExtensionConfig(const string& cartname);
 	CartridgeSlotManager& manager;
-	MSXCliComm& cliComm;
+	CliComm& cliComm;
 };
 
 
+// CartridgeSlotManager::Slot
+CartridgeSlotManager::Slot::Slot()
+	: ps(0), ss(0), config(NULL)
+{
+}
+
+CartridgeSlotManager::Slot::~Slot()
+{
+}
+
+bool CartridgeSlotManager::Slot::exists() const
+{
+	return command.get();
+}
+
+bool CartridgeSlotManager::Slot::used(const HardwareConfig* allowed) const
+{
+	return config && (config != allowed);
+}
+
+
+// CartridgeSlotManager
 CartridgeSlotManager::CartridgeSlotManager(MSXMotherBoard& motherBoard_)
 	: motherBoard(motherBoard_)
 	, cartCmd(new CartCmd(*this, motherBoard, "cart"))
@@ -61,7 +83,7 @@ int CartridgeSlotManager::getSlotNum(const string& slot)
 			throw MSXException(
 				"Invalid slot specification: " + slot);
 		}
-		return result-128;
+		return result - 128;
 	} else {
 		int result = StringOp::stringToInt(slot);
 		if ((result < 0) || (4 <= result)) {
@@ -90,7 +112,8 @@ void CartridgeSlotManager::createExternalSlot(int ps, int ss)
 			slotName[4] += slot;
 			motherBoard.getMSXCliComm().update(
 				CliComm::HARDWARE, slotName, "add");
-			slots[slot].command = new CartCmd(*this, motherBoard, slotName);
+			slots[slot].command.reset(
+				new CartCmd(*this, motherBoard, slotName));
 			return;
 		}
 	}
@@ -137,8 +160,7 @@ void CartridgeSlotManager::removeExternalSlot(int ps, int ss)
 	const string& slotName = slots[slot].command->getName();
 	motherBoard.getMSXCliComm().update(
 		CliComm::HARDWARE, slotName, "remove");
-	delete slots[slot].command;
-	slots[slot].command = NULL;
+	slots[slot].command.reset();
 }
 
 int CartridgeSlotManager::getSpecificSlot(int slot, int& ps, int& ss,
@@ -188,7 +210,7 @@ int CartridgeSlotManager::getAnyFreeSlot(int& ps, int& ss,
 }
 
 int CartridgeSlotManager::getFreePrimarySlot(
-		int &ps, const HardwareConfig& hwConfig)
+		int& ps, const HardwareConfig& hwConfig)
 {
 	for (int slot = 0; slot < MAX_SLOTS; ++slot) {
 		ps = slots[slot].ps;
@@ -223,7 +245,7 @@ bool CartridgeSlotManager::isExternalSlot(int ps, int ss, bool convert) const
 // CartCmd
 CartCmd::CartCmd(CartridgeSlotManager& manager_, MSXMotherBoard& motherBoard,
                  const string& commandName)
-	: RecordedCommand(motherBoard.getMSXCommandController(),
+	: RecordedCommand(motherBoard.getCommandController(),
 	                  motherBoard.getMSXEventDistributor(),
 	                  motherBoard.getScheduler(),
 	                  commandName)
@@ -232,18 +254,16 @@ CartCmd::CartCmd(CartridgeSlotManager& manager_, MSXMotherBoard& motherBoard,
 {
 }
 
-const ExtensionConfig* CartCmd::getExtensionConfig(const string& cartname)
+const HardwareConfig* CartCmd::getExtensionConfig(const string& cartname)
 {
 	if (cartname.size() != 5) {
 		throw SyntaxError();
 	}
 	int slot = cartname[4] - 'a';
-	const HardwareConfig* conf = manager.slots[slot].config;
-	assert(!conf || dynamic_cast<const ExtensionConfig*>(conf));
-	return static_cast<const ExtensionConfig*>(conf);
+	return manager.slots[slot].config;
 }
 
-string CartCmd::execute(const vector<string>& tokens, const EmuTime& /*time*/)
+string CartCmd::execute(const vector<string>& tokens, EmuTime::param /*time*/)
 {
 	string result;
 	string cartname = tokens[0];
@@ -258,7 +278,7 @@ string CartCmd::execute(const vector<string>& tokens, const EmuTime& /*time*/)
 	}
 	if (tokens.size() == 1) {
 		// query name of cartridge
-		const ExtensionConfig* extConf = getExtensionConfig(cartname);
+		const HardwareConfig* extConf = getExtensionConfig(cartname);
 		TclObject object(getCommandController().getInterpreter());
 		object.addListElement(cartname + ':');
 		object.addListElement(extConf ? extConf->getName() : "");
@@ -275,8 +295,7 @@ string CartCmd::execute(const vector<string>& tokens, const EmuTime& /*time*/)
 		if (tokens[1] == "-eject") {
 			result += "Warning: use of '-eject' is deprecated, instead use the 'eject' subcommand";
 		}
-		const ExtensionConfig* extConf = getExtensionConfig(cartname);
-		if (extConf) {
+		if (const HardwareConfig* extConf = getExtensionConfig(cartname)) {
 			try {
 				manager.motherBoard.removeExtension(*extConf);
 				cliComm.update(CliComm::MEDIA, cartname, "");
@@ -304,11 +323,19 @@ string CartCmd::execute(const vector<string>& tokens, const EmuTime& /*time*/)
 			options.push_back(tokens[i]);
 		}
 		try {
-			ExtensionConfig& extension =
-				manager.motherBoard.loadRom(
-					tokens[extensionNameToken], slotname, options);
-			cliComm.update(CliComm::MEDIA, cartname, tokens[extensionNameToken]);
-			result = extension.getName();
+			string romname = tokens[extensionNameToken];
+			std::auto_ptr<HardwareConfig> extension(
+				HardwareConfig::createRomConfig(
+					manager.motherBoard, romname, slotname, options));
+			if (slotname != "any") {
+				if (const HardwareConfig* extConf =
+					       getExtensionConfig(cartname)) {
+					// still a cartridge inserted, (try to) remove it now
+					manager.motherBoard.removeExtension(*extConf);
+				}
+			}
+			result = manager.motherBoard.insertExtension("ROM", extension);
+			cliComm.update(CliComm::MEDIA, cartname, romname);
 		} catch (MSXException& e) {
 			throw CommandException(e.getMessage());
 		}
@@ -331,8 +358,8 @@ void CartCmd::tabCompletion(vector<string>& tokens) const
 		extra.insert("eject");
 		extra.insert("insert");
 	}
-	UserFileContext context(getCommandController());
-	completeFileName(tokens, context, extra);
+	UserFileContext context;
+	completeFileName(getCommandController(), tokens, context, extra);
 }
 
 } // namespace openmsx
