@@ -1,4 +1,4 @@
-// $Id: CommandLineParser.cc 6835 2007-08-14 09:03:28Z m9710797 $
+// $Id: CommandLineParser.cc 8351 2008-11-12 18:20:29Z m9710797 $
 
 #include "CommandLineParser.hh"
 #include "CLIOption.hh"
@@ -19,6 +19,9 @@
 #include "ConfigException.hh"
 #include "FileException.hh"
 #include "EnumSetting.hh"
+#include "XMLLoader.hh"
+#include "XMLElement.hh"
+#include "XMLException.hh"
 #include "HostCPU.hh"
 #include "GLUtil.hh"
 #include "Reactor.hh"
@@ -28,7 +31,7 @@
 
 using std::cout;
 using std::endl;
-using std::list;
+using std::deque;
 using std::map;
 using std::set;
 using std::string;
@@ -40,9 +43,8 @@ class HelpOption : public CLIOption
 {
 public:
 	explicit HelpOption(CommandLineParser& parser);
-	virtual bool parseOption(const std::string& option,
-		std::list<std::string>& cmdLine);
-	virtual const std::string& optionHelp() const;
+	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual const string& optionHelp() const;
 private:
 	CommandLineParser& parser;
 };
@@ -51,9 +53,8 @@ class VersionOption : public CLIOption
 {
 public:
 	explicit VersionOption(CommandLineParser& parser);
-	virtual bool parseOption(const std::string& option,
-		std::list<std::string>& cmdLine);
-	virtual const std::string& optionHelp() const;
+	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual const string& optionHelp() const;
 private:
 	CommandLineParser& parser;
 };
@@ -62,9 +63,8 @@ class ControlOption : public CLIOption
 {
 public:
 	explicit ControlOption(CommandLineParser& parser);
-	virtual bool parseOption(const std::string& option,
-		std::list<std::string>& cmdLine);
-	virtual const std::string& optionHelp() const;
+	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual const string& optionHelp() const;
 private:
 	CommandLineParser& parser;
 };
@@ -73,9 +73,8 @@ class ScriptOption : public CLIOption
 {
 public:
 	const CommandLineParser::Scripts& getScripts() const;
-	virtual bool parseOption(const std::string& option,
-		std::list<std::string>& cmdLine);
-	virtual const std::string& optionHelp() const;
+	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual const string& optionHelp() const;
 private:
 	CommandLineParser::Scripts scripts;
 };
@@ -84,9 +83,8 @@ class MachineOption : public CLIOption
 {
 public:
 	explicit MachineOption(CommandLineParser& parser);
-	virtual bool parseOption(const std::string& option,
-		std::list<std::string>& cmdLine);
-	virtual const std::string& optionHelp() const;
+	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual const string& optionHelp() const;
 private:
 	CommandLineParser& parser;
 };
@@ -95,9 +93,8 @@ class SettingOption : public CLIOption
 {
 public:
 	explicit SettingOption(CommandLineParser& parser);
-	virtual bool parseOption(const std::string& option,
-		std::list<std::string>& cmdLine);
-	virtual const std::string& optionHelp() const;
+	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual const string& optionHelp() const;
 private:
 	CommandLineParser& parser;
 };
@@ -105,39 +102,34 @@ private:
 class NoMMXOption : public CLIOption
 {
 public:
-	virtual bool parseOption(const std::string& option,
-		std::list<std::string>& cmdLine);
-	virtual const std::string& optionHelp() const;
+	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual const string& optionHelp() const;
 };
 
 class NoSSEOption : public CLIOption {
 public:
-	virtual bool parseOption(const std::string& option,
-		std::list<std::string>& cmdLine);
-	virtual const std::string& optionHelp() const;
+	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual const string& optionHelp() const;
 };
 
 class NoSSE2Option : public CLIOption {
 public:
-	virtual bool parseOption(const std::string& option,
-		std::list<std::string>& cmdLine);
-	virtual const std::string& optionHelp() const;
+	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual const string& optionHelp() const;
 };
 
 class NoPBOOption : public CLIOption {
 public:
-	virtual bool parseOption(const std::string& option,
-		std::list<std::string>& cmdLine);
-	virtual const std::string& optionHelp() const;
+	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual const string& optionHelp() const;
 };
 
 class TestConfigOption : public CLIOption
 {
 public:
 	explicit TestConfigOption(CommandLineParser& parser);
-	virtual bool parseOption(const std::string& option,
-		std::list<std::string>& cmdLine);
-	virtual const std::string& optionHelp() const;
+	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual const string& optionHelp() const;
 private:
 	CommandLineParser& parser;
 };
@@ -148,7 +140,7 @@ private:
 CommandLineParser::CommandLineParser(Reactor& reactor_)
 	: reactor(reactor_)
 	, settingsConfig(reactor.getCommandController().getSettingsConfig())
-	, output(reactor.getGlobalCliComm())
+	, output(reactor.getCliComm())
 	, helpOption(new HelpOption(*this))
 	, versionOption(new VersionOption(*this))
 	, controlOption(new ControlOption(*this))
@@ -215,8 +207,8 @@ void CommandLineParser::registerFileClass(
 
 void CommandLineParser::registerFileTypes()
 {
-	const XMLElement* config = settingsConfig.findChild("FileTypes");
-	if (config) {
+	if (const XMLElement* config =
+			settingsConfig.getXMLElement().findChild("FileTypes")) {
 		for (FileClassMap::const_iterator i = fileClassMap.begin();
 		     i != fileClassMap.end(); ++i) {
 			XMLElement::Children extensions;
@@ -229,6 +221,7 @@ void CommandLineParser::registerFileTypes()
 	} else {
 		map<string, string> fileExtMap;
 		fileExtMap["rom"] = "romimage";
+		fileExtMap["ri"]  = "romimage";
 		fileExtMap["dsk"] = "diskimage";
 		fileExtMap["di1"] = "diskimage";
 		fileExtMap["di2"] = "diskimage";
@@ -246,7 +239,7 @@ void CommandLineParser::registerFileTypes()
 }
 
 bool CommandLineParser::parseOption(
-	const string& arg, list<string>& cmdLine, unsigned priority)
+	const string& arg, deque<string>& cmdLine, unsigned priority)
 {
 	map<string, OptionData>::const_iterator it1 = optionMap.find(arg);
 	if (it1 != optionMap.end()) {
@@ -263,12 +256,12 @@ bool CommandLineParser::parseOption(
 	return false; // unknown
 }
 
-bool CommandLineParser::parseFileName(const string& arg, list<string>& cmdLine)
+bool CommandLineParser::parseFileName(const string& arg, deque<string>& cmdLine)
 {
 	string originalName(arg);
 	try {
-		UserFileContext context(getReactor().getCommandController());
-		File file(context.resolve(arg));
+		UserFileContext context;
+		File file(context.resolve(getCommandController(), arg));
 		originalName = file.getOriginalName();
 	} catch (FileException& e) {
 		// ignore
@@ -296,8 +289,8 @@ void CommandLineParser::parse(int argc, char** argv)
 {
 	parseStatus = RUN;
 
-	list<string> cmdLine;
-	list<string> backupCmdLine;
+	deque<string> cmdLine;
+	deque<string> backupCmdLine;
 	for (int i = 1; i < argc; i++) {
 		cmdLine.push_back(FileOperations::getConventionalPath(argv[i]));
 	}
@@ -306,11 +299,17 @@ void CommandLineParser::parse(int argc, char** argv)
 		switch (priority) {
 		case 3:
 			if (!haveSettings) {
-				// Load default settings file in case the user didn't specify
-				// one.
+				// Load default settings file in case the user
+				// didn't specify one.
+				SystemFileContext context;
+				string filename = "settings.xml";
 				try {
-					SystemFileContext context;
-					settingsConfig.loadSetting(context, "settings.xml");
+					settingsConfig.loadSetting(context, filename);
+				} catch (XMLException& e) {
+					output.printWarning(
+						"Loading of settings failed: " +
+						e.getMessage() + "\n"
+						"Reverting to default settings.");
 				} catch (FileException& e) {
 					// settings.xml not found
 				} catch (ConfigException& e) {
@@ -319,6 +318,9 @@ void CommandLineParser::parse(int argc, char** argv)
 				}
 				// Consider an attempt to load the settings good enough.
 				haveSettings = true;
+				// Even if parsing failed, use this file for saving,
+				// this forces overwriting a non-setting file.
+				settingsConfig.setSaveFilename(context, filename);
 			}
 			break;
 		case 5: {
@@ -328,7 +330,7 @@ void CommandLineParser::parse(int argc, char** argv)
 					reactor.getMachineSetting().getValueString();
 				output.printInfo("Using default machine: " + machine);
 				try {
-					reactor.createMotherBoard(machine);
+					reactor.switchMachine(machine);
 				} catch (MSXException& e) {
 					output.printInfo(
 						"Failed to initialize default machine: " + e.getMessage()
@@ -338,7 +340,7 @@ void CommandLineParser::parse(int argc, char** argv)
 						reactor.getMachineSetting().getRestoreValueString();
 					output.printInfo("Using fallback machine: " + fallbackMachine);
 					try {
-						reactor.createMotherBoard(fallbackMachine);
+						reactor.switchMachine(fallbackMachine);
 					} catch (MSXException& e2) {
 						// Fallback machine failed as well; we're out of options.
 						throw FatalError(e2.getMessage());
@@ -403,16 +405,17 @@ const CommandLineParser::Scripts& CommandLineParser::getStartupScripts() const
 	return scriptOption->getScripts();
 }
 
-Reactor& CommandLineParser::getReactor() const
-{
-	return reactor;
-}
-
 MSXMotherBoard* CommandLineParser::getMotherBoard() const
 {
-	return getReactor().getMotherBoard();
+	return reactor.getMotherBoard();
 }
 
+CommandController& CommandLineParser::getCommandController() const
+{
+	return reactor.getCommandController();
+}
+
+
 // Control option
 
 ControlOption::ControlOption(CommandLineParser& parser_)
@@ -420,10 +423,9 @@ ControlOption::ControlOption(CommandLineParser& parser_)
 {
 }
 
-bool ControlOption::parseOption(const string& option, list<string>& cmdLine)
+bool ControlOption::parseOption(const string& option, deque<string>& cmdLine)
 {
-	parser.getReactor().getGlobalCliComm().startInput(
-		getArgument(option, cmdLine));
+	parser.reactor.getGlobalCliComm().startInput(getArgument(option, cmdLine));
 	parser.parseStatus = CommandLineParser::CONTROL;
 	return true;
 }
@@ -442,8 +444,7 @@ const CommandLineParser::Scripts& ScriptOption::getScripts() const
 	return scripts;
 }
 
-bool ScriptOption::parseOption(const string& option,
-		list<string>& cmdLine)
+bool ScriptOption::parseOption(const string& option, deque<string>& cmdLine)
 {
 	scripts.push_back(getArgument(option, cmdLine));
 	return true;
@@ -527,7 +528,7 @@ HelpOption::HelpOption(CommandLineParser& parser_)
 }
 
 bool HelpOption::parseOption(const string& /*option*/,
-		list<string>& /*cmdLine*/)
+                             deque<string>& /*cmdLine*/)
 {
 	if (!parser.haveSettings) {
 		return false; // not parsed yet, load settings first
@@ -577,7 +578,7 @@ VersionOption::VersionOption(CommandLineParser& parser_)
 }
 
 bool VersionOption::parseOption(const string& /*option*/,
-		list<string>& /*cmdLine*/)
+                                deque<string>& /*cmdLine*/)
 {
 	cout << Version::FULL_VERSION << endl;
 	cout << "flavour: " << BUILD_FLAVOUR << endl;
@@ -600,7 +601,7 @@ MachineOption::MachineOption(CommandLineParser& parser_)
 {
 }
 
-bool MachineOption::parseOption(const string& option, list<string>& cmdLine)
+bool MachineOption::parseOption(const string& option, deque<string>& cmdLine)
 {
 	if (parser.haveConfig) {
 		throw FatalError("Only one machine option allowed");
@@ -608,7 +609,7 @@ bool MachineOption::parseOption(const string& option, list<string>& cmdLine)
 	string machine(getArgument(option, cmdLine));
 	parser.output.printInfo("Using specified machine: " + machine);
 	try {
-		parser.getReactor().createMotherBoard(machine);
+		parser.reactor.switchMachine(machine);
 	} catch (MSXException& e) {
 		throw FatalError(e.getMessage());
 	}
@@ -629,15 +630,13 @@ SettingOption::SettingOption(CommandLineParser& parser_)
 {
 }
 
-bool SettingOption::parseOption(const string &option, list<string> &cmdLine)
+bool SettingOption::parseOption(const string& option, deque<string>& cmdLine)
 {
 	if (parser.haveSettings) {
 		throw FatalError("Only one setting option allowed");
 	}
 	try {
-		UserFileContext context(
-		        parser.getReactor().getCommandController(),
-		        "", true); // skip user directories
+		CurrentDirFileContext context;
 		parser.settingsConfig.loadSetting(
 			context, getArgument(option, cmdLine));
 		parser.haveSettings = true;
@@ -659,7 +658,7 @@ const string& SettingOption::optionHelp() const
 // class NoMMXOption
 
 bool NoMMXOption::parseOption(const string& /*option*/,
-		list<string>& /*cmdLine*/)
+                              deque<string>& /*cmdLine*/)
 {
 	cout << "Disabling MMX" << endl;
 	HostCPU::getInstance().forceDisableMMX();
@@ -677,7 +676,7 @@ const string& NoMMXOption::optionHelp() const
 // class NoSSEOption
 
 bool NoSSEOption::parseOption(const string& /*option*/,
-		list<string>& /*cmdLine*/)
+                              deque<string>& /*cmdLine*/)
 {
 	cout << "Disabling SSE" << endl;
 	HostCPU::getInstance().forceDisableSSE();
@@ -695,7 +694,7 @@ const string& NoSSEOption::optionHelp() const
 // class NoSSE2Option
 
 bool NoSSE2Option::parseOption(const string& /*option*/,
-		list<string>& /*cmdLine*/)
+                               deque<string>& /*cmdLine*/)
 {
 	cout << "Disabling SSE2" << endl;
 	HostCPU::getInstance().forceDisableSSE2();
@@ -713,7 +712,7 @@ const string& NoSSE2Option::optionHelp() const
 // class NoPBOOption
 
 bool NoPBOOption::parseOption(const string& /*option*/,
-		list<string>& /*cmdLine*/)
+                              deque<string>& /*cmdLine*/)
 {
 	#ifdef COMPONENT_GL
 	cout << "Disabling PBO" << endl;
@@ -738,7 +737,7 @@ TestConfigOption::TestConfigOption(CommandLineParser& parser_)
 }
 
 bool TestConfigOption::parseOption(const string& /*option*/,
-		list<string>& /*cmdLine*/)
+                                   deque<string>& /*cmdLine*/)
 {
 	parser.parseStatus = CommandLineParser::TEST;
 	return true;
