@@ -1,4 +1,4 @@
-// $Id: DeviceFactory.cc 7175 2007-10-21 20:32:35Z m9710797 $
+// $Id: DeviceFactory.cc 8257 2008-09-21 17:07:23Z manuelbi $
 
 #include "DeviceFactory.hh"
 #include "XMLElement.hh"
@@ -16,6 +16,7 @@
 #include "MSXFmPac.hh"
 #include "MSXAudio.hh"
 #include "MSXMoonSound.hh"
+#include "MSXOPL3Cartridge.hh"
 #include "MSXYamahaSFG.hh"
 #include "MC6850.hh"
 #include "MSXKanji.hh"
@@ -51,110 +52,109 @@
 #include "MSXMapperIO.hh"
 #include "VDPIODelay.hh"
 #include "MSXMotherBoard.hh"
-#include "MachineConfig.hh"
 #include "EmuTime.hh"
-#include "FileContext.hh"
+#include "MSXException.hh"
 
 namespace openmsx {
 
 std::auto_ptr<MSXDevice> DeviceFactory::create(
 	MSXMotherBoard& motherBoard, const HardwareConfig& hwConf,
-	const XMLElement& conf, const EmuTime& time)
+	const XMLElement& conf)
 {
 	std::auto_ptr<MSXDevice> result;
 	const std::string& type = conf.getName();
 	if (type == "PPI") {
-		result.reset(new MSXPPI(motherBoard, conf, time));
+		result.reset(new MSXPPI(motherBoard, conf));
 	} else if (type == "RAM") {
-		result.reset(new MSXRam(motherBoard, conf, time));
+		result.reset(new MSXRam(motherBoard, conf));
 	} else if (type == "VDP") {
-		result.reset(new VDP(motherBoard, conf, time));
+		result.reset(new VDP(motherBoard, conf));
 	} else if (type == "E6Timer") {
-		result.reset(new MSXE6Timer(motherBoard, conf, time));
+		result.reset(new MSXE6Timer(motherBoard, conf));
 	} else if (type == "F4Device") {
-		result.reset(new MSXF4Device(motherBoard, conf, time));
-	} else if (type == "TurboRLeds") {
-		// deprecated, remove in next version
+		result.reset(new MSXF4Device(motherBoard, conf));
 	} else if (type == "TurboRPause") {
-		result.reset(new MSXTurboRPause(motherBoard, conf, time));
+		result.reset(new MSXTurboRPause(motherBoard, conf));
 	} else if (type == "TurboRPCM") {
-		result.reset(new MSXTurboRPCM(motherBoard, conf, time));
+		result.reset(new MSXTurboRPCM(motherBoard, conf));
 	} else if (type == "S1985") {
-		result.reset(new MSXS1985(motherBoard, conf, time));
+		result.reset(new MSXS1985(motherBoard, conf));
 	} else if (type == "S1990") {
-		result.reset(new MSXS1990(motherBoard, conf, time));
+		result.reset(new MSXS1990(motherBoard, conf));
 	} else if (type == "PSG") {
-		result.reset(new MSXPSG(motherBoard, conf, time));
+		result.reset(new MSXPSG(motherBoard, conf));
 	} else if (type == "MSX-MUSIC") {
-		result.reset(new MSXMusic(motherBoard, conf, time));
+		result.reset(new MSXMusic(motherBoard, conf));
 	} else if (type == "FMPAC") {
-		result.reset(new MSXFmPac(motherBoard, conf, time));
+		result.reset(new MSXFmPac(motherBoard, conf));
 	} else if (type == "MSX-AUDIO") {
-		result.reset(new MSXAudio(motherBoard, conf, time));
+		result.reset(new MSXAudio(motherBoard, conf));
 	} else if (type == "MusicModuleMIDI") {
-		result.reset(new MC6850(motherBoard, conf, time));
+		result.reset(new MC6850(motherBoard, conf));
 	} else if (type == "YamahaSFG") {
-		result.reset(new MSXYamahaSFG(motherBoard, conf, time));
+		result.reset(new MSXYamahaSFG(motherBoard, conf));
 	} else if (type == "MoonSound") {
-		result.reset(new MSXMoonSound(motherBoard, conf, time));
+		result.reset(new MSXMoonSound(motherBoard, conf));
+	} else if (type == "OPL3Cartridge") {
+		result.reset(new MSXOPL3Cartridge(motherBoard, conf));
 	} else if (type == "Kanji") {
-		result.reset(new MSXKanji(motherBoard, conf, time));
+		result.reset(new MSXKanji(motherBoard, conf));
 	} else if (type == "Bunsetsu") {
-		result.reset(new MSXBunsetsu(motherBoard, conf, time));
+		result.reset(new MSXBunsetsu(motherBoard, conf));
 	} else if (type == "MemoryMapper") {
-		result.reset(new MSXMemoryMapper(motherBoard, conf, time));
+		result.reset(new MSXMemoryMapper(motherBoard, conf));
 	} else if (type == "PanasonicRAM") {
-		result.reset(new PanasonicRam(motherBoard, conf, time));
+		result.reset(new PanasonicRam(motherBoard, conf));
 	} else if (type == "RTC") {
-		result.reset(new MSXRTC(motherBoard, conf, time));
+		result.reset(new MSXRTC(motherBoard, conf));
 	} else if (type == "PasswordCart") {
-		result.reset(new PasswordCart(motherBoard, conf, time));
+		result.reset(new PasswordCart(motherBoard, conf));
 	} else if (type == "ROM") {
-		result = RomFactory::create(motherBoard, conf, time);
+		result = RomFactory::create(motherBoard, conf);
 	} else if (type == "PrinterPort") {
-		result.reset(new MSXPrinterPort(motherBoard, conf, time));
+		result.reset(new MSXPrinterPort(motherBoard, conf));
 	} else if (type == "SCCplus") { // Note: it's actually called SCC-I
-		result.reset(new MSXSCCPlusCart(motherBoard, conf, time));
+		result.reset(new MSXSCCPlusCart(motherBoard, conf));
 	} else if (type == "WD2793") {
-		result.reset(new PhilipsFDC(motherBoard, conf, time));
+		result.reset(new PhilipsFDC(motherBoard, conf));
 	} else if (type == "Microsol") {
-		result.reset(new MicrosolFDC(motherBoard, conf, time));
+		result.reset(new MicrosolFDC(motherBoard, conf));
 	} else if (type == "MB8877A") {
-		result.reset(new NationalFDC(motherBoard, conf, time));
+		result.reset(new NationalFDC(motherBoard, conf));
 	} else if (type == "TC8566AF") {
-		result.reset(new TurboRFDC(motherBoard, conf, time));
+		result.reset(new TurboRFDC(motherBoard, conf));
 	} else if (type == "SunriseIDE") {
-		result.reset(new SunriseIDE(motherBoard, conf, time));
+		result.reset(new SunriseIDE(motherBoard, conf));
 	} else if (type == "GoudaSCSI") {
-		result.reset(new GoudaSCSI(motherBoard, conf, time));
+		result.reset(new GoudaSCSI(motherBoard, conf));
 	} else if (type == "MegaSCSI") {
-		result.reset(new MegaSCSI(motherBoard, conf, time));
+		result.reset(new MegaSCSI(motherBoard, conf));
 	} else if (type == "ESERAM") {
-		result.reset(new ESE_RAM(motherBoard, conf, time));
+		result.reset(new ESE_RAM(motherBoard, conf));
 	} else if (type == "WaveSCSI") {
-		result.reset(new ESE_SCC(motherBoard, conf, time, true));
+		result.reset(new ESE_SCC(motherBoard, conf, true));
 	} else if (type == "ESESCC") {
-		result.reset(new ESE_SCC(motherBoard, conf, time, false));
+		result.reset(new ESE_SCC(motherBoard, conf, false));
 	} else if (type == "Matsushita") {
-		result.reset(new MSXMatsushita(motherBoard, conf, time));
+		result.reset(new MSXMatsushita(motherBoard, conf));
 	} else if (type == "Kanji12") {
-		result.reset(new MSXKanji12(motherBoard, conf, time));
+		result.reset(new MSXKanji12(motherBoard, conf));
 	} else if (type == "MSX-MIDI") {
-		result.reset(new MSXMidi(motherBoard, conf, time));
+		result.reset(new MSXMidi(motherBoard, conf));
 	} else if (type == "MSX-RS232") {
-		result.reset(new MSXRS232(motherBoard, conf, time));
+		result.reset(new MSXRS232(motherBoard, conf));
 	} else if (type == "MegaRam") {
-		result.reset(new MSXMegaRam(motherBoard, conf, time));
+		result.reset(new MSXMegaRam(motherBoard, conf));
 	} else if (type == "PAC") {
-		result.reset(new MSXPac(motherBoard, conf, time));
+		result.reset(new MSXPac(motherBoard, conf));
 	} else if (type == "HBI55") {
-		result.reset(new MSXHBI55(motherBoard, conf, time));
+		result.reset(new MSXHBI55(motherBoard, conf));
 	} else if (type == "DebugDevice") {
-		result.reset(new DebugDevice(motherBoard, conf, time));
+		result.reset(new DebugDevice(motherBoard, conf));
 	} else if (type == "V9990") {
-		result.reset(new V9990(motherBoard, conf, time));
+		result.reset(new V9990(motherBoard, conf));
 	} else if (type == "ADVram") {
-		result.reset(new ADVram(motherBoard, conf, time));
+		result.reset(new ADVram(motherBoard, conf));
 	} else {
 		throw MSXException("Unknown device \"" + type +
 		                   "\" specified in configuration");
@@ -169,8 +169,6 @@ static XMLElement createConfig(const std::string& name, const std::string& id)
 {
 	XMLElement config(name);
 	config.addAttribute("id", id);
-	config.setFileContext(std::auto_ptr<FileContext>(
-		new SystemFileContext()));
 	return config;
 }
 
@@ -179,8 +177,8 @@ std::auto_ptr<DummyDevice> DeviceFactory::createDummyDevice(
 {
 	static XMLElement config(createConfig("Dummy", "empty"));
 	std::auto_ptr<DummyDevice> result(
-		new DummyDevice(motherBoard, config, EmuTime::zero));
-	result->init(motherBoard.getMachineConfig());
+		new DummyDevice(motherBoard, config));
+	result->init(*motherBoard.getMachineConfig());
 	return result;
 }
 
@@ -189,8 +187,8 @@ std::auto_ptr<MSXDeviceSwitch> DeviceFactory::createDeviceSwitch(
 {
 	static XMLElement config(createConfig("DeviceSwitch", "DeviceSwitch"));
 	std::auto_ptr<MSXDeviceSwitch> result(
-		new MSXDeviceSwitch(motherBoard, config, EmuTime::zero));
-	result->init(motherBoard.getMachineConfig());
+		new MSXDeviceSwitch(motherBoard, config));
+	result->init(*motherBoard.getMachineConfig());
 	return result;
 }
 
@@ -200,7 +198,7 @@ std::auto_ptr<MSXMapperIO> DeviceFactory::createMapperIO(
 	static XMLElement config(createConfig("MapperIO", "MapperIO"));
 	std::auto_ptr<MSXMapperIO> result(
 		new MSXMapperIO(motherBoard, config, EmuTime::zero));
-	result->init(motherBoard.getMachineConfig());
+	result->init(*motherBoard.getMachineConfig());
 	return result;
 }
 
@@ -210,7 +208,7 @@ std::auto_ptr<VDPIODelay> DeviceFactory::createVDPIODelay(
 	static XMLElement config(createConfig("VDPIODelay", "VDPIODelay"));
 	std::auto_ptr<VDPIODelay> result(
 		new VDPIODelay(motherBoard, config, EmuTime::zero));
-	result->init(motherBoard.getMachineConfig());
+	result->init(*motherBoard.getMachineConfig());
 	return result;
 }
 
