@@ -1,10 +1,11 @@
-// $Id: GlobalSettings.cc 6681 2007-06-30 15:17:18Z m9710797 $
+// $Id: GlobalSettings.cc 8346 2008-11-08 08:37:26Z m9710797 $
 
 #include "GlobalSettings.hh"
 #include "IntegerSetting.hh"
 #include "BooleanSetting.hh"
 #include "StringSetting.hh"
 #include "SettingsConfig.hh"
+#include "CommandController.hh"
 #include "ThrottleManager.hh"
 #include "EnumSetting.hh"
 
@@ -28,7 +29,15 @@ GlobalSettings::GlobalSettings(CommandController& commandController_)
 	userDirSetting.reset(new StringSetting(commandController,
 	        "user_directories", "list of user directories", ""));
 	umrCallBackSetting.reset(new StringSetting(commandController,
-	        "umr_callback", "TCL proc to call when an UMR is detected", ""));
+	        "umr_callback", "Tcl proc to call when an UMR is detected", ""));
+	EnumSetting<SyncMode>::Map syncDirAsDSKMap;
+	syncDirAsDSKMap["read_only"] = SYNC_READONLY;
+	syncDirAsDSKMap["cached_write"] = SYNC_CACHEDWRITE;
+	syncDirAsDSKMap["nodelete"] = SYNC_NODELETE;
+	syncDirAsDSKMap["full"] = SYNC_FULL;
+	syncDirAsDSKSetting.reset(new EnumSetting<SyncMode>(commandController,
+		"DirAsDSKmode", "type of syncronisation between host directory and dir-as-dsk diskimage",
+		SYNC_FULL, syncDirAsDSKMap));
 	EnumSetting<bool>::Map bootsectorMap;
 	bootsectorMap["DOS1"] = false;
 	bootsectorMap["DOS2"] = true;
@@ -95,6 +104,11 @@ StringSetting& GlobalSettings::getUserDirSetting()
 	return *userDirSetting.get();
 }
 
+EnumSetting<GlobalSettings::SyncMode>& GlobalSettings::getSyncDirAsDSKSetting()
+{
+	return *syncDirAsDSKSetting.get();
+}
+
 EnumSetting<bool>& GlobalSettings::getBootSectorSetting()
 {
 	return *bootSectorSetting.get();
@@ -112,7 +126,7 @@ void GlobalSettings::update(const Setting& setting)
 		// automatically unpause after a power off/on cycle
 		// this solved a bug, but apart from that this behaviour also
 		// makes more sense
-		getPauseSetting().setValue(false);
+		getPauseSetting().changeValue(false);
 	}
 }
 
