@@ -1,7 +1,8 @@
-// $Id: IPSPatch.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: IPSPatch.cc 8157 2008-08-12 17:58:55Z m9710797 $
 
 #include "IPSPatch.hh"
 #include "File.hh"
+#include "Filename.hh"
 #include "MSXException.hh"
 #include <cstring>
 #include <cassert>
@@ -15,16 +16,17 @@ static unsigned getStop(const IPSPatch::PatchMap::const_iterator& it)
 	return it->first + it->second.size();
 }
 
-IPSPatch::IPSPatch(const std::string& filename,
+IPSPatch::IPSPatch(const Filename& filename_,
                    std::auto_ptr<const PatchInterface> parent_)
-	: parent(parent_)
+	: filename(filename_)
+	, parent(parent_)
 {
 	File ipsFile(filename);
 
 	byte buf[5];
 	ipsFile.read(buf, 5);
 	if (memcmp(buf, "PATCH", 5) != 0) {
-		throw MSXException("Invalid IPS file: " + filename);
+		throw MSXException("Invalid IPS file: " + filename.getOriginal());
 	}
 	ipsFile.read(buf, 3);
 	while (memcmp(buf, "EOF", 3) != 0) {
@@ -125,4 +127,10 @@ unsigned IPSPatch::getSize() const
 	return size;
 }
 
+void IPSPatch::getFilenames(std::vector<Filename>& result) const
+{
+	parent->getFilenames(result);
+	result.push_back(filename);
+}
+
 } // namespace openmsx
