@@ -1,13 +1,16 @@
-// $Id: MSXDevice.cc 6807 2007-08-06 17:08:53Z m9710797 $
+// $Id: MSXDevice.cc 8374 2008-11-29 20:30:41Z m9710797 $
 
 #include "MSXDevice.hh"
 #include "XMLElement.hh"
 #include "MSXMotherBoard.hh"
 #include "CartridgeSlotManager.hh"
 #include "MSXCPUInterface.hh"
+#include "MSXCPU.hh"
 #include "CacheLine.hh"
+#include "TclObject.hh"
 #include "StringOp.hh"
 #include "MSXException.hh"
+#include "serialize.hh"
 #include <set>
 #include <algorithm>
 #include <cassert>
@@ -23,19 +26,18 @@ byte MSXDevice::unmappedWrite[0x10000];
 
 
 MSXDevice::MSXDevice(MSXMotherBoard& motherBoard_, const XMLElement& config,
-                     const EmuTime& /*time*/, const string& name)
-	: deviceConfig(config), motherBoard(motherBoard_)
+                     const string& name)
+	: motherBoard(motherBoard_), deviceConfig(config)
 	, hardwareConfig(NULL), externalSlotID(-1)
 {
 	initName(name);
 }
 
-MSXDevice::MSXDevice(MSXMotherBoard& motherBoard_, const XMLElement& config,
-                     const EmuTime& /*time*/)
-	: deviceConfig(config), motherBoard(motherBoard_)
+MSXDevice::MSXDevice(MSXMotherBoard& motherBoard_, const XMLElement& config)
+	: motherBoard(motherBoard_), deviceConfig(config)
 	, hardwareConfig(NULL), externalSlotID(-1)
 {
-	initName(deviceConfig.getId());
+	initName(getDeviceConfig().getId());
 }
 
 void MSXDevice::initName(const string& name)
@@ -57,8 +59,8 @@ void MSXDevice::init(const HardwareConfig& hwConf)
 	staticInit();
 
 	lockDevices();
-	registerSlots(deviceConfig);
-	registerPorts(deviceConfig);
+	registerSlots();
+	registerPorts();
 }
 
 MSXDevice::~MSXDevice()
@@ -111,7 +113,7 @@ void MSXDevice::lockDevices()
 	// needed we can implement something more sophisticated later without
 	// changing the format of the config files.
 	XMLElement::Children refConfigs;
-	deviceConfig.getChildren("device", refConfigs);
+	getDeviceConfig().getChildren("device", refConfigs);
 	for (XMLElement::Children::const_iterator it = refConfigs.begin();
 	     it != refConfigs.end(); ++it) {
 		string name = (*it)->getAttribute("idref");
@@ -145,11 +147,16 @@ const MSXDevice::Devices& MSXDevice::getReferences() const
 	return references;
 }
 
-void MSXDevice::registerSlots(const XMLElement& config)
+EmuTime::param MSXDevice::getCurrentTime() const
+{
+	return getMotherBoard().getCurrentTime();
+}
+
+void MSXDevice::registerSlots()
 {
 	MemRegions tmpMemRegions;
 	XMLElement::Children memConfigs;
-	config.getChildren("mem", memConfigs);
+	getDeviceConfig().getChildren("mem", memConfigs);
 	for (XMLElement::Children::const_iterator it = memConfigs.begin();
 	     it != memConfigs.end(); ++it) {
 		unsigned base = (*it)->getAttributeAsInt("base");
@@ -176,28 +183,39 @@ void MSXDevice::registerSlots(const XMLElement& config)
 	CartridgeSlotManager& slotManager = getMotherBoard().getSlotManager();
 	ps = 0;
 	ss = 0;
-	const XMLElement* parent = config.getParent();
-	while (true) {
-		const string& name = parent->getName();
-		if (name == "secondary") {
-			const string& secondSlot = parent->getAttribute("slot");
+	const XMLElement& config = getDeviceConfig();
+	if (config.hasAttribute("primary_slot")) {
+		const string& primSlot = config.getAttribute("primary_slot");
+		ps = slotManager.getSlotNum(primSlot);
+		if (config.hasAttribute("secondary_slot")) {
+			const string& secondSlot = config.getAttribute("secondary_slot");
 			ss = slotManager.getSlotNum(secondSlot);
-		} else if (name == "primary") {
-			const string& primSlot = parent->getAttribute("slot");
-			ps = slotManager.getSlotNum(primSlot);
-			break;
 		}
-		parent = parent->getParent();
-		if (!parent) {
-			throw MSXException("Invalid memory specification");
+	} else {
+		const XMLElement* parent = config.getParent();
+		while (true) {
+			const string& name = parent->getName();
+			if (name == "secondary") {
+				const string& secondSlot = parent->getAttribute("slot");
+				ss = slotManager.getSlotNum(secondSlot);
+			} else if (name == "primary") {
+				const string& primSlot = parent->getAttribute("slot");
+				ps = slotManager.getSlotNum(primSlot);
+				break;
+			}
+			parent = parent->getParent();
+			if (!parent) {
+				throw MSXException("Invalid memory specification");
+			}
 		}
 	}
-	if ( (0 > ss ) && (ss >= -128) ) {
-		if ( (0 <= ps) && (ps<4)
-			 && motherBoard.getCPUInterface().isExpanded(ps))
+	if ((-128 <= ss) && (ss < 0)) {
+		if ((0 <= ps) && (ps < 4) &&
+		    motherBoard.getCPUInterface().isExpanded(ps)) {
 			ss += 128;
-		else
+		} else {
 			ss = 0;
+		}
 	}
 
 	if (ps == -256) {
@@ -211,6 +229,11 @@ void MSXDevice::registerSlots(const XMLElement& config)
 		// numerical specified slot (0, 1, 2, 3)
 	}
 
+	// store actual slot in config
+	XMLElement& writableConfig = const_cast<XMLElement&>(config);
+	writableConfig.setAttribute("primary_slot",   StringOp::toString(ps));
+	writableConfig.setAttribute("secondary_slot", StringOp::toString(ss));
+
 	for (MemRegions::const_iterator it = tmpMemRegions.begin();
 	     it != tmpMemRegions.end(); ++it) {
 		getMotherBoard().getCPUInterface().registerMemDevice(
@@ -232,10 +255,10 @@ void MSXDevice::unregisterSlots()
 	}
 }
 
-void MSXDevice::registerPorts(const XMLElement& config)
+void MSXDevice::registerPorts()
 {
 	XMLElement::Children ios;
-	config.getChildren("io", ios);
+	getDeviceConfig().getChildren("io", ios);
 	for (XMLElement::Children::const_iterator it = ios.begin();
 	     it != ios.end(); ++it) {
 		unsigned base = StringOp::stringToInt((*it)->getAttribute("base"));
@@ -272,7 +295,7 @@ void MSXDevice::unregisterPorts()
 }
 
 
-void MSXDevice::reset(const EmuTime& /*time*/)
+void MSXDevice::reset(EmuTime::param /*time*/)
 {
 	// nothing
 }
@@ -282,12 +305,12 @@ byte MSXDevice::readIRQVector()
 	return 0xFF;
 }
 
-void MSXDevice::powerDown(const EmuTime& /*time*/)
+void MSXDevice::powerDown(EmuTime::param /*time*/)
 {
 	// nothing
 }
 
-void MSXDevice::powerUp(const EmuTime& time)
+void MSXDevice::powerUp(EmuTime::param time)
 {
 	reset(time);
 }
@@ -297,8 +320,19 @@ string MSXDevice::getName() const
 	return deviceName;
 }
 
+void MSXDevice::getDeviceInfo(TclObject& result) const
+{
+	result.addListElement(getDeviceConfig().getName());
+	getExtraDeviceInfo(result);
+}
 
-byte MSXDevice::readIO(word port, const EmuTime& /*time*/)
+void MSXDevice::getExtraDeviceInfo(TclObject& /*result*/) const
+{
+	// nothing
+}
+
+
+byte MSXDevice::readIO(word port, EmuTime::param /*time*/)
 {
 	(void)port;
 	PRT_DEBUG("MSXDevice::readIO (0x" << std::hex << int(port & 0xFF)
@@ -306,7 +340,7 @@ byte MSXDevice::readIO(word port, const EmuTime& /*time*/)
 	return 0xFF;
 }
 
-void MSXDevice::writeIO(word port, byte value, const EmuTime& /*time*/)
+void MSXDevice::writeIO(word port, byte value, EmuTime::param /*time*/)
 {
 	(void)port;
 	(void)value;
@@ -316,13 +350,13 @@ void MSXDevice::writeIO(word port, byte value, const EmuTime& /*time*/)
 	// do nothing
 }
 
-byte MSXDevice::peekIO(word /*port*/, const EmuTime& /*time*/) const
+byte MSXDevice::peekIO(word /*port*/, EmuTime::param /*time*/) const
 {
 	return 0xFF;
 }
 
 
-byte MSXDevice::readMem(word address, const EmuTime& /*time*/)
+byte MSXDevice::readMem(word address, EmuTime::param /*time*/)
 {
 	(void)address;
 	PRT_DEBUG("MSXDevice: read from unmapped memory " << std::hex <<
@@ -332,11 +366,11 @@ byte MSXDevice::readMem(word address, const EmuTime& /*time*/)
 
 const byte* MSXDevice::getReadCacheLine(word /*start*/) const
 {
-	return NULL;	// uncacheable
+	return NULL; // uncacheable
 }
 
 void MSXDevice::writeMem(word address, byte /*value*/,
-                            const EmuTime& /*time*/)
+                            EmuTime::param /*time*/)
 {
 	(void)address;
 	PRT_DEBUG("MSXDevice: write to unmapped memory " << std::hex <<
@@ -344,7 +378,7 @@ void MSXDevice::writeMem(word address, byte /*value*/,
 	// do nothing
 }
 
-byte MSXDevice::peekMem(word address, const EmuTime& /*time*/) const
+byte MSXDevice::peekMem(word address, EmuTime::param /*time*/) const
 {
 	word base = address & CacheLine::HIGH;
 	const byte* cache = getReadCacheLine(base);
@@ -358,14 +392,26 @@ byte MSXDevice::peekMem(word address, const EmuTime& /*time*/) const
 }
 
 void MSXDevice::globalWrite(word /*address*/, byte /*value*/,
-                            const EmuTime& /*time*/)
+                            EmuTime::param /*time*/)
 {
 	assert(false);
 }
 
 byte* MSXDevice::getWriteCacheLine(word /*start*/) const
 {
-	return NULL;	// uncacheable
+	return NULL; // uncacheable
 }
 
+void MSXDevice::invalidateMemCache(word start, unsigned size)
+{
+	getMotherBoard().getCPU().invalidateMemCache(start, size);
+}
+
+template<typename Archive>
+void MSXDevice::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.serialize("name", deviceName);
+}
+INSTANTIATE_SERIALIZE_METHODS(MSXDevice);
+
 } // namespace openmsx
