@@ -1,37 +1,19 @@
-// $Id: MSXDeviceSwitch.cc 6805 2007-08-06 12:01:49Z m9710797 $
+// $Id: MSXDeviceSwitch.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "MSXDeviceSwitch.hh"
+#include "MSXSwitchedDevice.hh"
 #include "MSXCPUInterface.hh"
 #include "MSXMotherBoard.hh"
 #include "MSXException.hh"
 #include "StringOp.hh"
+#include "serialize.hh"
 #include <cassert>
 
 namespace openmsx {
 
-/// class MSXSwitchedDevice ///
-
-MSXSwitchedDevice::MSXSwitchedDevice(MSXMotherBoard& motherBoard_, byte id_)
-	: motherBoard(motherBoard_), id(id_)
-{
-	motherBoard.getDeviceSwitch().registerDevice(id, this);
-}
-
-MSXSwitchedDevice::~MSXSwitchedDevice()
-{
-	motherBoard.getDeviceSwitch().unregisterDevice(id);
-}
-
-void MSXSwitchedDevice::reset(const EmuTime& /*time*/)
-{
-}
-
-
-/// class MSXDeviceSwitch ///
-
 MSXDeviceSwitch::MSXDeviceSwitch(MSXMotherBoard& motherBoard,
-                                 const XMLElement& config, const EmuTime& time)
-	: MSXDevice(motherBoard, config, time)
+                                 const XMLElement& config)
+	: MSXDevice(motherBoard, config)
 {
 	for (int i = 0; i < 256; ++i) {
 		devices[i] = NULL;
@@ -42,17 +24,15 @@ MSXDeviceSwitch::MSXDeviceSwitch(MSXMotherBoard& motherBoard,
 
 MSXDeviceSwitch::~MSXDeviceSwitch()
 {
-	for (int i = 0; i < 256; i++) {
+	for (int i = 0; i < 256; ++i) {
 		// all devices must be unregistered
 		assert(devices[i] == NULL);
 	}
 	assert(count == 0);
 }
 
-
 void MSXDeviceSwitch::registerDevice(byte id, MSXSwitchedDevice* device)
 {
-	//PRT_DEBUG("Switch: register device with id " << (int)id);
 	if (devices[id]) {
 		// TODO implement multiplexing
 		throw MSXException("Already have a switched device with id " +
@@ -83,50 +63,52 @@ void MSXDeviceSwitch::unregisterDevice(byte id)
 	devices[id] = NULL;
 }
 
-
-void MSXDeviceSwitch::reset(const EmuTime& time)
+bool MSXDeviceSwitch::hasRegisteredDevices() const
 {
-	for (int i = 0; i < 256; i++) {
-		if (devices[i]) {
-			devices[i]->reset(time);
-		}
-	}
+	return count;
+}
 
+void MSXDeviceSwitch::reset(EmuTime::param /*time*/)
+{
 	selected = 0;
 }
 
-byte MSXDeviceSwitch::readIO(word port, const EmuTime& time)
+byte MSXDeviceSwitch::readIO(word port, EmuTime::param time)
 {
 	if (devices[selected]) {
-		//PRT_DEBUG("Switch read device " << (int)selected << " port " << (int)port);
-		return devices[selected]->readIO(port, time);
-	} else {
-		//PRT_DEBUG("Switch read no device");
-		return 0xFF;
-	}
-}
-
-byte MSXDeviceSwitch::peekIO(word port, const EmuTime& time) const
-{
-	if (devices[selected]) {
-		return devices[selected]->peekIO(port, time);
+		return devices[selected]->readSwitchedIO(port, time);
 	} else {
 		return 0xFF;
 	}
 }
 
-void MSXDeviceSwitch::writeIO(word port, byte value, const EmuTime& time)
+byte MSXDeviceSwitch::peekIO(word port, EmuTime::param time) const
 {
-	port &= 0x0F;
-	if (port == 0x00) {
+	if (devices[selected]) {
+		return devices[selected]->peekSwitchedIO(port, time);
+	} else {
+		return 0xFF;
+	}
+}
+
+void MSXDeviceSwitch::writeIO(word port, byte value, EmuTime::param time)
+{
+	if ((port & 0x0F) == 0x00) {
 		selected = value;
-		PRT_DEBUG("Switch " << int(selected));
 	} else if (devices[selected]) {
-		//PRT_DEBUG("Switch write device " << (int)selected << " port " << (int)port);
-		devices[selected]->writeIO(port, value, time);
+		devices[selected]->writeSwitchedIO(port, value, time);
 	} else {
-		//ignore
+		// ignore
 	}
 }
 
+
+template<typename Archive>
+void MSXDeviceSwitch::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<MSXDevice>(*this);
+	ar.serialize("selected", selected);
+}
+INSTANTIATE_SERIALIZE_METHODS(MSXDeviceSwitch);
+
 } // namespace openmsx
