@@ -1,10 +1,10 @@
-// $Id: MSXMotherBoard.cc 6821 2007-08-08 08:27:27Z m9710797 $
+// $Id: MSXMotherBoard.cc 8412 2008-12-20 19:41:39Z m9710797 $
 
 #include "MSXMotherBoard.hh"
 #include "Reactor.hh"
 #include "MSXDevice.hh"
-#include "MachineConfig.hh"
-#include "ExtensionConfig.hh"
+#include "HardwareConfig.hh"
+#include "XMLElement.hh"
 #include "MSXCliComm.hh"
 #include "GlobalCliComm.hh"
 #include "MSXCommandController.hh"
@@ -19,9 +19,10 @@
 #include "MSXCPU.hh"
 #include "PanasonicMemory.hh"
 #include "MSXDeviceSwitch.hh"
+#include "MSXMapperIO.hh"
 #include "CassettePort.hh"
 #include "RenShaTurbo.hh"
-#include "LedEvent.hh"
+#include "LedStatus.hh"
 #include "MSXEventDistributor.hh"
 #include "EventDelay.hh"
 #include "EventTranslator.hh"
@@ -30,10 +31,16 @@
 #include "BooleanSetting.hh"
 #include "GlobalSettings.hh"
 #include "Command.hh"
+#include "CommandException.hh"
 #include "RecordedCommand.hh"
 #include "InfoTopic.hh"
 #include "FileException.hh"
+#include "TclObject.hh"
 #include "Observer.hh"
+#include "StringOp.hh"
+#include "serialize.hh"
+#include "serialize_stl.hh"
+#include "ref.hh"
 #include <cassert>
 #include <map>
 #include <iostream>
@@ -48,10 +55,12 @@ namespace openmsx {
 
 class AddRemoveUpdate;
 class ResetCmd;
+class LoadMachineCmd;
 class ListExtCmd;
 class ExtCmd;
 class RemoveExtCmd;
 class MachineNameInfo;
+class DeviceInfo;
 
 class MSXMotherBoardImpl : private Observer<Setting>, private noncopyable
 {
@@ -71,25 +80,29 @@ public:
 	void unpause();
 	void powerUp();
 	void schedulePowerDown();
-	void doPowerDown(const EmuTime& time);
+	void doPowerDown(EmuTime::param time);
+	void activate(bool active);
+	bool isActive() const;
 	void scheduleReset();
-	void doReset(const EmuTime& time);
+	void doReset(EmuTime::param time);
 	byte readIRQVector();
 
-	const MachineConfig& getMachineConfig() const;
+	const HardwareConfig* getMachineConfig() const;
+	void setMachineConfig(HardwareConfig* machineConfig);
+	bool isTurboR() const;
 	void loadMachine(const string& machine);
 	const MSXMotherBoard::Extensions& getExtensions() const;
-	ExtensionConfig* findExtension(const string& extensionName);
-	ExtensionConfig& loadExtension(const string& extensionName);
-	ExtensionConfig& loadRom(
-		const string& romname, const string& slotname,
-		const vector<string>& options);
-	void removeExtension(const ExtensionConfig& extension);
+	HardwareConfig* findExtension(const string& extensionName);
+	string loadExtension(const string& extensionName);
+	string insertExtension(const std::string& name,
+	                       std::auto_ptr<HardwareConfig> extension);
+	void removeExtension(const HardwareConfig& extension);
 
-	MSXCliComm& getMSXCliComm();
+	CliComm& getMSXCliComm();
+	CliComm* getMSXCliCommIfAvailable();
+	MSXEventDistributor& getMSXEventDistributor();
 	MSXCommandController& getMSXCommandController();
 	Scheduler& getScheduler();
-	MSXEventDistributor& getMSXEventDistributor();
 	CartridgeSlotManager& getSlotManager();
 	EventDelay& getEventDelay();
 	EventTranslator& getEventTranslator();
@@ -104,6 +117,7 @@ public:
 	MSXDeviceSwitch& getDeviceSwitch();
 	CassettePortInterface& getCassettePort();
 	RenShaTurbo& getRenShaTurbo();
+	LedStatus& getLedStatus();
 	EventDistributor& getEventDistributor();
 	Display& getDisplay();
 	DiskManipulator& getDiskManipulator();
@@ -111,16 +125,23 @@ public:
 	GlobalSettings& getGlobalSettings();
 	GlobalCliComm& getGlobalCliComm();
 	CommandController& getCommandController();
+	InfoCommand& getMachineInfoCommand();
+	EmuTime::param getCurrentTime();
 
 	void addDevice(MSXDevice& device);
 	void removeDevice(MSXDevice& device);
 	MSXDevice* findDevice(const string& name);
 
 	MSXMotherBoard::SharedStuff& getSharedStuff(const string& name);
+	MSXMapperIO* createMapperIO();
+	void destroyMapperIO();
 
 	string getUserName(const string& hwName);
 	void freeUserName(const string& hwName, const string& userName);
 
+	template<typename Archive>
+	void serialize(Archive& ar, unsigned version);
+
 private:
 	void deleteMachine();
 
@@ -134,21 +155,31 @@ private:
 	string machineName;
 
 	typedef vector<MSXDevice*> Devices;
-	Devices availableDevices;
+	Devices availableDevices; // no ownership
 
 	typedef map<string, MSXMotherBoard::SharedStuff> SharedStuffMap;
 	SharedStuffMap sharedStuffMap;
 	map<string, set<string> > userNames;
 
-	auto_ptr<MachineConfig> machineConfig;
+	auto_ptr<MSXMapperIO> mapperIO;
+	unsigned mapperIOCounter;
+
+	// These two should normally be the same, only during savestate loading
+	// machineConfig will already be filled in, but machineConfig2 not yet.
+	// This is important when an exception happens during loading of
+	// machineConfig2 (otherwise machineConfig2 gets deleted twice).
+	// See also HardwareConfig::serialize() and setMachineConfig()
+	auto_ptr<HardwareConfig> machineConfig2;
+	HardwareConfig* machineConfig;
+
 	MSXMotherBoard::Extensions extensions;
 
 	// order of auto_ptr's is important!
 	auto_ptr<AddRemoveUpdate> addRemoveUpdate;
 	auto_ptr<MSXCliComm> msxCliComm;
+	auto_ptr<MSXEventDistributor> msxEventDistributor;
 	auto_ptr<MSXCommandController> msxCommandController;
 	auto_ptr<Scheduler> scheduler;
-	auto_ptr<MSXEventDistributor> msxEventDistributor;
 	auto_ptr<CartridgeSlotManager> slotManager;
 	auto_ptr<EventDelay> eventDelay;
 	auto_ptr<EventTranslator> eventTranslator;
@@ -163,18 +194,24 @@ private:
 	auto_ptr<MSXDeviceSwitch> deviceSwitch;
 	auto_ptr<CassettePortInterface> cassettePort;
 	auto_ptr<RenShaTurbo> renShaTurbo;
+	auto_ptr<LedStatus> ledStatus;
 
 	auto_ptr<ResetCmd>     resetCommand;
+	auto_ptr<LoadMachineCmd> loadMachineCommand;
 	auto_ptr<ListExtCmd>   listExtCommand;
 	auto_ptr<ExtCmd>       extCommand;
 	auto_ptr<RemoveExtCmd> removeExtCommand;
 	auto_ptr<MachineNameInfo> machineNameInfo;
+	auto_ptr<DeviceInfo>   deviceInfo;
+	friend class DeviceInfo;
+
 	BooleanSetting& powerSetting;
 
 	int blockedCounter;
 	bool powered;
 	bool needReset;
 	bool needPowerDown;
+	bool active;
 };
 
 
@@ -192,12 +229,23 @@ class ResetCmd : public RecordedCommand
 public:
 	ResetCmd(MSXMotherBoardImpl& motherBoard);
 	virtual string execute(const vector<string>& tokens,
-	                       const EmuTime& time);
+	                       EmuTime::param time);
 	virtual string help(const vector<string>& tokens) const;
 private:
 	MSXMotherBoardImpl& motherBoard;
 };
 
+class LoadMachineCmd : public SimpleCommand
+{
+public:
+	LoadMachineCmd(MSXMotherBoardImpl& motherBoard);
+	virtual string execute(const vector<string>& tokens);
+	virtual string help(const vector<string>& tokens) const;
+	virtual void tabCompletion(vector<string>& tokens) const;
+private:
+	MSXMotherBoardImpl& motherBoard;
+};
+
 class ListExtCmd : public Command
 {
 public:
@@ -214,7 +262,7 @@ class ExtCmd : public RecordedCommand
 public:
 	ExtCmd(MSXMotherBoardImpl& motherBoard);
 	virtual string execute(const vector<string>& tokens,
-	                       const EmuTime& time);
+	                       EmuTime::param time);
 	virtual string help(const vector<string>& tokens) const;
 	virtual void tabCompletion(vector<string>& tokens) const;
 private:
@@ -226,7 +274,7 @@ class RemoveExtCmd : public RecordedCommand
 public:
 	RemoveExtCmd(MSXMotherBoardImpl& motherBoard);
 	virtual string execute(const vector<string>& tokens,
-	                       const EmuTime& time);
+	                       EmuTime::param time);
 	virtual string help(const vector<string>& tokens) const;
 	virtual void tabCompletion(vector<string>& tokens) const;
 private:
@@ -244,28 +292,47 @@ private:
 	MSXMotherBoardImpl& motherBoard;
 };
 
+class DeviceInfo : public InfoTopic
+{
+public:
+	DeviceInfo(MSXMotherBoardImpl& motherBoard);
+	virtual void execute(const vector<TclObject*>& tokens,
+	                     TclObject& result) const;
+	virtual string help(const vector<string>& tokens) const;
+	virtual void tabCompletion(vector<string>& tokens) const;
+private:
+	MSXMotherBoardImpl& motherBoard;
+};
+
 
 MSXMotherBoardImpl::MSXMotherBoardImpl(MSXMotherBoard& self_, Reactor& reactor_)
 	: self(self_)
 	, reactor(reactor_)
+	, mapperIOCounter(0)
+	, machineConfig(NULL)
 	, powerSetting(getGlobalSettings().getPowerSetting())
 	, blockedCounter(0)
 	, powered(false)
 	, needReset(false)
 	, needPowerDown(false)
+	, active(false)
 {
 	self.pimple.reset(this);
 
 	resetCommand.reset(new ResetCmd(*this));
+	loadMachineCommand.reset(new LoadMachineCmd(*this));
 	listExtCommand.reset(new ListExtCmd(*this));
 	extCommand.reset(new ExtCmd(*this));
 	removeExtCommand.reset(new RemoveExtCmd(*this));
 	machineNameInfo.reset(new MachineNameInfo(*this));
+	deviceInfo.reset(new DeviceInfo(*this));
 
 	getMSXMixer().mute(); // powered down
 	getRealTime(); // make sure it's instantiated
 	getEventTranslator();
 	powerSetting.attach(*this);
+
+	addRemoveUpdate.reset(new AddRemoveUpdate(*this));
 }
 
 MSXMotherBoardImpl::~MSXMotherBoardImpl()
@@ -273,25 +340,28 @@ MSXMotherBoardImpl::~MSXMotherBoardImpl()
 	powerSetting.detach(*this);
 	deleteMachine();
 
+	assert(mapperIOCounter == 0);
 	assert(availableDevices.empty());
 	assert(extensions.empty());
-	assert(!machineConfig.get());
+	assert(!machineConfig2.get());
+	assert(!getMachineConfig());
 }
 
 void MSXMotherBoardImpl::deleteMachine()
 {
 	while (!extensions.empty()) {
 		try {
-	 		removeExtension(*extensions.back());
+			removeExtension(*extensions.back());
 		} catch (MSXException& e) {
 			std::cerr << "Internal error: failed to remove "
 			             "extension while deleting a machine: "
-			          << e.getMessage() << std::endl;   
+			          << e.getMessage() << std::endl;
 			assert(false);
 		}
 	}
 
-	machineConfig.reset();
+	machineConfig2.reset();
+	machineConfig = NULL;
 }
 
 const string& MSXMotherBoardImpl::getMachineID()
@@ -308,21 +378,34 @@ const string& MSXMotherBoardImpl::getMachineName() const
 	return machineName;
 }
 
-const MachineConfig& MSXMotherBoardImpl::getMachineConfig() const
+const HardwareConfig* MSXMotherBoardImpl::getMachineConfig() const
 {
-	assert(machineConfig.get());
-	return *machineConfig;
+	return machineConfig;
+}
+
+void MSXMotherBoardImpl::setMachineConfig(HardwareConfig* machineConfig_)
+{
+	assert(!getMachineConfig());
+	machineConfig = machineConfig_;
+}
+
+bool MSXMotherBoardImpl::isTurboR() const
+{
+	const HardwareConfig* config = getMachineConfig();
+	assert(config);
+	return config->getConfig().getChild("devices").findChild("S1990");
 }
 
 void MSXMotherBoardImpl::loadMachine(const string& machine)
 {
 	assert(machineName.empty());
 	assert(extensions.empty());
-	assert(!machineConfig.get());
-	machineName = machine;
+	assert(!machineConfig2.get());
+	assert(!getMachineConfig());
 
 	try {
-		machineConfig.reset(new MachineConfig(self, machine));
+		machineConfig2 = HardwareConfig::createMachineConfig(self, machine);
+		setMachineConfig(machineConfig2.get());
 	} catch (FileException& e) {
 		throw MSXException("Machine \"" + machine + "\" not found: " +
 		                   e.getMessage());
@@ -330,7 +413,6 @@ void MSXMotherBoardImpl::loadMachine(const string& machine)
 		throw MSXException("Error in \"" + machine + "\" machine: " +
 		                   e.getMessage());
 	}
-	addRemoveUpdate.reset(new AddRemoveUpdate(*this));
 	try {
 		machineConfig->parseSlots();
 		machineConfig->createDevices();
@@ -341,50 +423,41 @@ void MSXMotherBoardImpl::loadMachine(const string& machine)
 	if (powerSetting.getValue()) {
 		powerUp();
 	}
+	machineName = machine;
 }
 
-ExtensionConfig& MSXMotherBoardImpl::loadExtension(const string& name)
+string MSXMotherBoardImpl::loadExtension(const string& name)
 {
-	auto_ptr<ExtensionConfig> extension;
+	auto_ptr<HardwareConfig> extension;
 	try {
-		extension.reset(new ExtensionConfig(self, name));
+		extension = HardwareConfig::createExtensionConfig(self, name);
 	} catch (FileException& e) {
 		throw MSXException(
-			"Extension \"" + name + "\" not found: " + e.getMessage()
-			);
+			"Extension \"" + name + "\" not found: " + e.getMessage());
 	} catch (MSXException& e) {
 		throw MSXException(
-			"Error in \"" + name + "\" extension: " + e.getMessage()
-			);
+			"Error in \"" + name + "\" extension: " + e.getMessage());
 	}
+	return insertExtension(name, extension);
+}
+
+string MSXMotherBoardImpl::insertExtension(
+	const string& name, auto_ptr<HardwareConfig> extension)
+{
 	try {
 		extension->parseSlots();
 		extension->createDevices();
 	} catch (MSXException& e) {
 		throw MSXException(
-			"Error in \"" + name + "\" extension: " + e.getMessage()
-			);
+			"Error in \"" + name + "\" extension: " + e.getMessage());
 	}
-	ExtensionConfig& result = *extension;
+	string result = extension->getName();
 	extensions.push_back(extension.release());
-	self.getMSXCliComm().update(CliComm::EXTENSION, result.getName(), "add");
+	self.getMSXCliComm().update(CliComm::EXTENSION, result, "add");
 	return result;
 }
 
-ExtensionConfig& MSXMotherBoardImpl::loadRom(
-		const string& romname, const string& slotname,
-		const vector<string>& options)
-{
-	auto_ptr<ExtensionConfig> extension(
-		new ExtensionConfig(self, romname, slotname, options));
-	extension->parseSlots();
-	extension->createDevices();
-	ExtensionConfig& result = *extension;
-	extensions.push_back(extension.release());
-	return result;
-}
-
-ExtensionConfig* MSXMotherBoardImpl::findExtension(const string& extensionName)
+HardwareConfig* MSXMotherBoardImpl::findExtension(const string& extensionName)
 {
 	for (MSXMotherBoard::Extensions::const_iterator it = extensions.begin();
 	     it != extensions.end(); ++it) {
@@ -400,7 +473,7 @@ const MSXMotherBoard::Extensions& MSXMotherBoardImpl::getExtensions() const
 	return extensions;
 }
 
-void MSXMotherBoardImpl::removeExtension(const ExtensionConfig& extension)
+void MSXMotherBoardImpl::removeExtension(const HardwareConfig& extension)
 {
 	extension.testRemove();
 	MSXMotherBoard::Extensions::iterator it =
@@ -411,15 +484,27 @@ void MSXMotherBoardImpl::removeExtension(const ExtensionConfig& extension)
 	extensions.erase(it);
 }
 
-MSXCliComm& MSXMotherBoardImpl::getMSXCliComm()
+CliComm& MSXMotherBoardImpl::getMSXCliComm()
 {
 	if (!msxCliComm.get()) {
-		msxCliComm.reset(new MSXCliComm(
-			self, reactor.getGlobalCliComm()));
+		msxCliComm.reset(new MSXCliComm(self, getGlobalCliComm()));
 	}
 	return *msxCliComm;
 }
 
+CliComm* MSXMotherBoardImpl::getMSXCliCommIfAvailable()
+{
+	return msxCliComm.get();
+}
+
+MSXEventDistributor& MSXMotherBoardImpl::getMSXEventDistributor()
+{
+	if (!msxEventDistributor.get()) {
+		msxEventDistributor.reset(new MSXEventDistributor());
+	}
+	return *msxEventDistributor;
+}
+
 MSXCommandController& MSXMotherBoardImpl::getMSXCommandController()
 {
 	if (!msxCommandController.get()) {
@@ -437,14 +522,6 @@ Scheduler& MSXMotherBoardImpl::getScheduler()
 	return *scheduler;
 }
 
-MSXEventDistributor& MSXMotherBoardImpl::getMSXEventDistributor()
-{
-	if (!msxEventDistributor.get()) {
-		msxEventDistributor.reset(new MSXEventDistributor());
-	}
-	return *msxEventDistributor;
-}
-
 CartridgeSlotManager& MSXMotherBoardImpl::getSlotManager()
 {
 	if (!slotManager.get()) {
@@ -475,9 +552,7 @@ EventTranslator& MSXMotherBoardImpl::getEventTranslator()
 RealTime& MSXMotherBoardImpl::getRealTime()
 {
 	if (!realTime.get()) {
-		realTime.reset(new RealTime(
-			getScheduler(), getEventDistributor(),
-			getEventDelay(), getGlobalSettings()));
+		realTime.reset(new RealTime(self));
 	}
 	return *realTime;
 }
@@ -517,6 +592,9 @@ DummyDevice& MSXMotherBoardImpl::getDummyDevice()
 
 MSXCPU& MSXMotherBoardImpl::getCPU()
 {
+	// because CPU needs to know if we're emulating turbor or not
+	assert(getMachineConfig());
+
 	if (!msxCpu.get()) {
 		msxCpu.reset(new MSXCPU(self));
 	}
@@ -526,9 +604,8 @@ MSXCPU& MSXMotherBoardImpl::getCPU()
 MSXCPUInterface& MSXMotherBoardImpl::getCPUInterface()
 {
 	if (!msxCpuInterface.get()) {
-		// TODO assert hw config already loaded
-		msxCpuInterface = MSXCPUInterface::create(
-			self, getMachineConfig().getConfig());
+		assert(getMachineConfig());
+		msxCpuInterface.reset(new MSXCPUInterface(self));
 	}
 	return *msxCpuInterface;
 }
@@ -552,7 +629,8 @@ MSXDeviceSwitch& MSXMotherBoardImpl::getDeviceSwitch()
 CassettePortInterface& MSXMotherBoardImpl::getCassettePort()
 {
 	if (!cassettePort.get()) {
-		if (getMachineConfig().getConfig().findChild("CassettePort")) {
+		assert(getMachineConfig());
+		if (getMachineConfig()->getConfig().findChild("CassettePort")) {
 			cassettePort.reset(new CassettePort(self));
 		} else {
 			cassettePort.reset(new DummyCassettePort());
@@ -564,12 +642,22 @@ CassettePortInterface& MSXMotherBoardImpl::getCassettePort()
 RenShaTurbo& MSXMotherBoardImpl::getRenShaTurbo()
 {
 	if (!renShaTurbo.get()) {
+		assert(getMachineConfig());
 		renShaTurbo.reset(new RenShaTurbo(
-			getCommandController(), getMachineConfig().getConfig()));
+			getCommandController(),
+			getMachineConfig()->getConfig()));
 	}
 	return *renShaTurbo;
 }
 
+LedStatus& MSXMotherBoardImpl::getLedStatus()
+{
+	if (!ledStatus.get()) {
+		ledStatus.reset(new LedStatus(self));
+	}
+	return *ledStatus;
+}
+
 EventDistributor& MSXMotherBoardImpl::getEventDistributor()
 {
 	return reactor.getEventDistributor();
@@ -605,19 +693,30 @@ CommandController& MSXMotherBoardImpl::getCommandController()
 	return getMSXCommandController();
 }
 
+InfoCommand& MSXMotherBoardImpl::getMachineInfoCommand()
+{
+	return getMSXCommandController().getMachineInfoCommand();
+}
+
+EmuTime::param MSXMotherBoardImpl::getCurrentTime()
+{
+	return getScheduler().getCurrentTime();
+}
+
 bool MSXMotherBoardImpl::execute()
 {
+	if (!powered || blockedCounter) {
+		return false;
+	}
+	assert(getMachineConfig()); // otherwise powered cannot be true
+
 	if (needReset) {
 		needReset = false;
-		doReset(getScheduler().getCurrentTime());
+		doReset(getCurrentTime());
 	}
 	if (needPowerDown) {
 		needPowerDown = false;
-		doPowerDown(getScheduler().getCurrentTime());
-	}
-
-	if (!powered || blockedCounter) {
-		return false;
+		doPowerDown(getCurrentTime());
 	}
 
 	getCPU().execute();
@@ -640,13 +739,17 @@ void MSXMotherBoardImpl::unblock()
 
 void MSXMotherBoardImpl::pause()
 {
-	getCPU().setPaused(true);
+	if (getMachineConfig()) {
+		getCPU().setPaused(true);
+	}
 	getMSXMixer().mute();
 }
 
 void MSXMotherBoardImpl::unpause()
 {
-	getCPU().setPaused(false);
+	if (getMachineConfig()) {
+		getCPU().setPaused(false);
+	}
 	getMSXMixer().unmute();
 }
 
@@ -670,8 +773,10 @@ void MSXMotherBoardImpl::scheduleReset()
 	exitCPULoopSync();
 }
 
-void MSXMotherBoardImpl::doReset(const EmuTime& time)
+void MSXMotherBoardImpl::doReset(EmuTime::param time)
 {
+	assert(getMachineConfig());
+
 	getCPUInterface().reset();
 	for (Devices::iterator it = availableDevices.begin();
 	     it != availableDevices.end(); ++it) {
@@ -697,6 +802,7 @@ byte MSXMotherBoardImpl::readIRQVector()
 void MSXMotherBoardImpl::powerUp()
 {
 	if (powered) return;
+	if (!getMachineConfig()) return;
 
 	powered = true;
 	// TODO: If our "powered" field is always equal to the power setting,
@@ -704,13 +810,12 @@ void MSXMotherBoardImpl::powerUp()
 	// TODO: assert disabled see note in Reactor::run() where this method
 	//       is called
 	//assert(powerSetting.getValue() == powered);
-	powerSetting.setValue(true);
+	powerSetting.changeValue(true);
 	// TODO: We could make the power LED a device, so we don't have to handle
 	//       it separately here.
-	getEventDistributor().distributeEvent(
-		new LedEvent(LedEvent::POWER, true, self));
+	getLedStatus().setLed(LedStatus::POWER, true);
 
-	const EmuTime& time = getScheduler().getCurrentTime();
+	EmuTime::param time = getCurrentTime();
 	getCPUInterface().reset();
 	for (Devices::iterator it = availableDevices.begin();
 	     it != availableDevices.end(); ++it) {
@@ -730,7 +835,7 @@ void MSXMotherBoardImpl::schedulePowerDown()
 	exitCPULoopSync();
 }
 
-void MSXMotherBoardImpl::doPowerDown(const EmuTime& time)
+void MSXMotherBoardImpl::doPowerDown(EmuTime::param time)
 {
 	if (!powered) return;
 
@@ -739,9 +844,8 @@ void MSXMotherBoardImpl::doPowerDown(const EmuTime& time)
 	//       How do we want the Reactor to shutdown: immediately or after
 	//       handling all pending commands/events/updates?
 	//assert(powerSetting.getValue() == powered);
-	powerSetting.setValue(false);
-	getEventDistributor().distributeEvent(
-		new LedEvent(LedEvent::POWER, false, self));
+	powerSetting.changeValue(false);
+	getLedStatus().setLed(LedStatus::POWER, false);
 
 	getMSXMixer().mute();
 
@@ -751,14 +855,34 @@ void MSXMotherBoardImpl::doPowerDown(const EmuTime& time)
 	}
 }
 
+void MSXMotherBoardImpl::activate(bool active_)
+{
+	active = active_;
+	MSXEventDistributor::EventPtr event = active
+		? MSXEventDistributor::EventPtr(new SimpleEvent<OPENMSX_MACHINE_ACTIVATED>())
+		: MSXEventDistributor::EventPtr(new SimpleEvent<OPENMSX_MACHINE_DEACTIVATED>());
+	getMSXEventDistributor().distributeEvent(event, getScheduler().getCurrentTime());
+	if (active) {
+		getRealTime().resync();
+	}
+}
+bool MSXMotherBoardImpl::isActive() const
+{
+	return active;
+}
+
 void MSXMotherBoardImpl::exitCPULoopSync()
 {
-	getCPU().exitCPULoopSync();
+	if (getMachineConfig()) {
+		getCPU().exitCPULoopSync();
+	}
 }
 
 void MSXMotherBoardImpl::exitCPULoopAsync()
 {
-	getCPU().exitCPULoopAsync();
+	if (getMachineConfig()) {
+		getCPU().exitCPULoopAsync();
+	}
 }
 
 // Observer<Setting>
@@ -792,6 +916,45 @@ MSXMotherBoard::SharedStuff& MSXMotherBoardImpl::getSharedStuff(
 	return sharedStuffMap[name];
 }
 
+MSXMapperIO* MSXMotherBoardImpl::createMapperIO()
+{
+	if (mapperIOCounter == 0) {
+		mapperIO = DeviceFactory::createMapperIO(self);
+
+		MSXCPUInterface& cpuInterface = getCPUInterface();
+		cpuInterface.register_IO_Out(0xFC, mapperIO.get());
+		cpuInterface.register_IO_Out(0xFD, mapperIO.get());
+		cpuInterface.register_IO_Out(0xFE, mapperIO.get());
+		cpuInterface.register_IO_Out(0xFF, mapperIO.get());
+		cpuInterface.register_IO_In (0xFC, mapperIO.get());
+		cpuInterface.register_IO_In (0xFD, mapperIO.get());
+		cpuInterface.register_IO_In (0xFE, mapperIO.get());
+		cpuInterface.register_IO_In (0xFF, mapperIO.get());
+	}
+	++mapperIOCounter;
+	return mapperIO.get();
+}
+
+void MSXMotherBoardImpl::destroyMapperIO()
+{
+	assert(mapperIO.get());
+	assert(mapperIOCounter);
+	--mapperIOCounter;
+	if (mapperIOCounter == 0) {
+		MSXCPUInterface& cpuInterface = getCPUInterface();
+		cpuInterface.unregister_IO_Out(0xFC, mapperIO.get());
+		cpuInterface.unregister_IO_Out(0xFD, mapperIO.get());
+		cpuInterface.unregister_IO_Out(0xFE, mapperIO.get());
+		cpuInterface.unregister_IO_Out(0xFF, mapperIO.get());
+		cpuInterface.unregister_IO_In (0xFC, mapperIO.get());
+		cpuInterface.unregister_IO_In (0xFD, mapperIO.get());
+		cpuInterface.unregister_IO_In (0xFE, mapperIO.get());
+		cpuInterface.unregister_IO_In (0xFF, mapperIO.get());
+
+		mapperIO.reset();
+	}
+}
+
 string MSXMotherBoardImpl::getUserName(const string& hwName)
 {
 	set<string>& s = userNames[hwName];
@@ -831,7 +994,7 @@ AddRemoveUpdate::~AddRemoveUpdate()
 
 // ResetCmd
 ResetCmd::ResetCmd(MSXMotherBoardImpl& motherBoard_)
-	: RecordedCommand(motherBoard_.getMSXCommandController(),
+	: RecordedCommand(motherBoard_.getCommandController(),
 	                  motherBoard_.getMSXEventDistributor(),
 	                  motherBoard_.getScheduler(),
 	                  "reset")
@@ -840,7 +1003,7 @@ ResetCmd::ResetCmd(MSXMotherBoardImpl& motherBoard_)
 }
 
 string ResetCmd::execute(const vector<string>& /*tokens*/,
-                         const EmuTime& /*time*/)
+                         EmuTime::param /*time*/)
 {
 	motherBoard.scheduleReset();
 	return "";
@@ -848,13 +1011,45 @@ string ResetCmd::execute(const vector<string>& /*tokens*/,
 
 string ResetCmd::help(const vector<string>& /*tokens*/) const
 {
-	return "Resets the MSX.\n";
+	return "Resets the MSX.";
+}
+
+
+// LoadMachineCmd
+LoadMachineCmd::LoadMachineCmd(MSXMotherBoardImpl& motherBoard_)
+	: SimpleCommand(motherBoard_.getCommandController(), "load_machine")
+	, motherBoard(motherBoard_)
+{
+}
+
+string LoadMachineCmd::execute(const vector<string>& tokens)
+{
+	if (tokens.size() != 2) {
+		throw SyntaxError();
+	}
+	if (motherBoard.getMachineConfig()) {
+		throw CommandException("Already loaded a config in this machine.");
+	}
+	motherBoard.loadMachine(tokens[1]);
+	return motherBoard.getMachineName();
+}
+
+string LoadMachineCmd::help(const vector<string>& /*tokens*/) const
+{
+	return "Load a msx machine configuration into an empty machine.";
+}
+
+void LoadMachineCmd::tabCompletion(vector<string>& tokens) const
+{
+	set<string> machines;
+	Reactor::getHwConfigs("machines", machines);
+	completeString(tokens, machines);
 }
 
 
 // ListExtCmd
 ListExtCmd::ListExtCmd(MSXMotherBoardImpl& motherBoard_)
-	: Command(motherBoard_.getMSXCommandController(), "list_extensions")
+	: Command(motherBoard_.getCommandController(), "list_extensions")
 	, motherBoard(motherBoard_)
 {
 }
@@ -877,7 +1072,7 @@ string ListExtCmd::help(const vector<string>& /*tokens*/) const
 
 // ExtCmd
 ExtCmd::ExtCmd(MSXMotherBoardImpl& motherBoard_)
-	: RecordedCommand(motherBoard_.getMSXCommandController(),
+	: RecordedCommand(motherBoard_.getCommandController(),
 	                  motherBoard_.getMSXEventDistributor(),
 	                  motherBoard_.getScheduler(),
 	                  "ext")
@@ -885,15 +1080,13 @@ ExtCmd::ExtCmd(MSXMotherBoardImpl& motherBoard_)
 {
 }
 
-string ExtCmd::execute(const vector<string>& tokens, const EmuTime& /*time*/)
+string ExtCmd::execute(const vector<string>& tokens, EmuTime::param /*time*/)
 {
 	if (tokens.size() != 2) {
 		throw SyntaxError();
 	}
 	try {
-		ExtensionConfig& extension =
-			motherBoard.loadExtension(tokens[1]);
-		return extension.getName();
+		return motherBoard.loadExtension(tokens[1]);
 	} catch (MSXException& e) {
 		throw CommandException(e.getMessage());
 	}
@@ -914,7 +1107,7 @@ void ExtCmd::tabCompletion(vector<string>& tokens) const
 
 // RemoveExtCmd
 RemoveExtCmd::RemoveExtCmd(MSXMotherBoardImpl& motherBoard_)
-	: RecordedCommand(motherBoard_.getMSXCommandController(),
+	: RecordedCommand(motherBoard_.getCommandController(),
 	                  motherBoard_.getMSXEventDistributor(),
 	                  motherBoard_.getScheduler(),
 	                  "remove_extension")
@@ -922,12 +1115,12 @@ RemoveExtCmd::RemoveExtCmd(MSXMotherBoardImpl& motherBoard_)
 {
 }
 
-string RemoveExtCmd::execute(const vector<string>& tokens, const EmuTime& /*time*/)
+string RemoveExtCmd::execute(const vector<string>& tokens, EmuTime::param /*time*/)
 {
 	if (tokens.size() != 2) {
 		throw SyntaxError();
 	}
-	ExtensionConfig* extension = motherBoard.findExtension(tokens[1]);
+	HardwareConfig* extension = motherBoard.findExtension(tokens[1]);
 	if (!extension) {
 		throw CommandException("No such extension: " + tokens[1]);
 	}
@@ -962,8 +1155,7 @@ void RemoveExtCmd::tabCompletion(vector<string>& tokens) const
 // MachineNameInfo
 
 MachineNameInfo::MachineNameInfo(MSXMotherBoardImpl& motherBoard_)
-	: InfoTopic(motherBoard_.getMSXCommandController().getMachineInfoCommand(),
-	            "config_name")
+	: InfoTopic(motherBoard_.getMachineInfoCommand(), "config_name")
 	, motherBoard(motherBoard_)
 {
 }
@@ -980,6 +1172,109 @@ string MachineNameInfo::help(const vector<string>& /*tokens*/) const
 }
 
 
+// DeviceInfo
+
+DeviceInfo::DeviceInfo(MSXMotherBoardImpl& motherBoard_)
+	: InfoTopic(motherBoard_.getMachineInfoCommand(), "device")
+	, motherBoard(motherBoard_)
+{
+}
+
+void DeviceInfo::execute(const vector<TclObject*>& tokens,
+                         TclObject& result) const
+{
+	switch (tokens.size()) {
+	case 2:
+		for (MSXMotherBoardImpl::Devices::const_iterator it =
+		        motherBoard.availableDevices.begin();
+		     it != motherBoard.availableDevices.end(); ++it) {
+			result.addListElement((*it)->getName());
+		}
+		break;
+	case 3: {
+		string name = tokens[2]->getString();
+		MSXDevice* device = motherBoard.findDevice(name);
+		if (!device) {
+			throw CommandException("No such device: " + name);
+		}
+		device->getDeviceInfo(result);
+		break;
+	}
+	default:
+		throw SyntaxError();
+	}
+}
+
+string DeviceInfo::help(const vector<string>& /*tokens*/) const
+{
+	return "Without any arguments, returns the list of used device names.\n"
+	       "With a device name as argument, returns the type (and for some "
+	       "devices the subtype) of the given device.";
+}
+
+void DeviceInfo::tabCompletion(vector<string>& tokens) const
+{
+	if (tokens.size() == 3) {
+		set<string> names;
+		for (MSXMotherBoardImpl::Devices::const_iterator it =
+		         motherBoard.availableDevices.begin();
+		     it != motherBoard.availableDevices.end(); ++it) {
+			names.insert((*it)->getName());
+		}
+		completeString(tokens, names);
+	}
+}
+
+// serialize
+template<typename Archive>
+void MSXMotherBoardImpl::serialize(Archive& ar, unsigned /*version*/)
+{
+	// don't serialize:
+	//    machineID, userNames, availableDevices, addRemoveUpdate,
+	//    sharedStuffMap, msxCliComm, msxEventDistributor,
+	//    msxCommandController, slotManager, eventDelay, eventTranslator,
+	//    debugger, msxMixer, dummyDevice, panasonicMemory, renShaTurbo,
+	//    ledStatus
+
+	// Scheduler must come early so that devices can query current time
+	ar.serialize("scheduler", getScheduler());
+	// MSXMixer has already set syncpoints, those are invalid now
+	// the following call will fix this
+	getMSXMixer().reschedule();
+
+	ar.serialize("name", machineName);
+	ar.serialize("config", machineConfig2, ref(self));
+	assert(getMachineConfig() == machineConfig2.get());
+	ar.serialize("extensions", extensions, ref(self));
+
+	if (mapperIO.get()) {
+		ar.serialize("mapperIO", *mapperIO);
+	}
+
+	MSXDeviceSwitch& devSwitch = getDeviceSwitch();
+	if (devSwitch.hasRegisteredDevices()) {
+		ar.serialize("deviceSwitch", devSwitch);
+	}
+
+	if (getMachineConfig()) {
+		ar.serialize("cpu", getCPU());
+	}
+	ar.serialize("cpuInterface", getCPUInterface());
+
+	if (CassettePort* port = dynamic_cast<CassettePort*>(&getCassettePort())) {
+		ar.serialize("cassetteport", *port);
+	}
+
+	if (ar.isLoader()) {
+		if (powerSetting.getValue()) {
+			powered = true;
+			getLedStatus().setLed(LedStatus::POWER, true);
+			getMSXMixer().unmute();
+		}
+	}
+}
+
+
 // MSXMotherBoard
 
 MSXMotherBoard::MSXMotherBoard(Reactor& reactor)
@@ -1033,15 +1328,23 @@ void MSXMotherBoard::schedulePowerDown()
 {
 	pimple->schedulePowerDown();
 }
-void MSXMotherBoard::doPowerDown(const EmuTime& time)
+void MSXMotherBoard::doPowerDown(EmuTime::param time)
 {
 	pimple->doPowerDown(time);
 }
+void MSXMotherBoard::activate(bool active)
+{
+	pimple->activate(active);
+}
+bool MSXMotherBoard::isActive() const
+{
+	return pimple->isActive();
+}
 void MSXMotherBoard::scheduleReset()
 {
 	pimple->scheduleReset();
 }
-void MSXMotherBoard::doReset(const EmuTime& time)
+void MSXMotherBoard::doReset(EmuTime::param time)
 {
 	pimple->doReset(time);
 }
@@ -1049,10 +1352,18 @@ byte MSXMotherBoard::readIRQVector()
 {
 	return pimple->readIRQVector();
 }
-const MachineConfig& MSXMotherBoard::getMachineConfig() const
+const HardwareConfig* MSXMotherBoard::getMachineConfig() const
 {
 	return pimple->getMachineConfig();
 }
+void MSXMotherBoard::setMachineConfig(HardwareConfig* machineConfig)
+{
+	pimple->setMachineConfig(machineConfig);
+}
+bool MSXMotherBoard::isTurboR() const
+{
+	return pimple->isTurboR();
+}
 void MSXMotherBoard::loadMachine(const string& machine)
 {
 	pimple->loadMachine(machine);
@@ -1061,28 +1372,32 @@ const MSXMotherBoard::Extensions& MSXMotherBoard::getExtensions() const
 {
 	return pimple->getExtensions();
 }
-ExtensionConfig* MSXMotherBoard::findExtension(const string& extensionName)
+HardwareConfig* MSXMotherBoard::findExtension(const string& extensionName)
 {
 	return pimple->findExtension(extensionName);
 }
-ExtensionConfig& MSXMotherBoard::loadExtension(const string& extensionName)
+string MSXMotherBoard::loadExtension(const string& extensionName)
 {
 	return pimple->loadExtension(extensionName);
 }
-ExtensionConfig& MSXMotherBoard::loadRom(
-	const string& romname, const string& slotname,
-	const vector<string>& options)
+string MSXMotherBoard::insertExtension(
+	const string& name, auto_ptr<HardwareConfig> extension)
 {
-	return pimple->loadRom(romname, slotname, options);
+	return pimple->insertExtension(name, extension);
 }
-void MSXMotherBoard::removeExtension(const ExtensionConfig& extension)
+void MSXMotherBoard::removeExtension(const HardwareConfig& extension)
 {
 	pimple->removeExtension(extension);
 }
-MSXCliComm& MSXMotherBoard::getMSXCliComm()
+CliComm& MSXMotherBoard::getMSXCliComm()
 {
+	// note: return-type is CliComm instead of MSXCliComm
 	return pimple->getMSXCliComm();
 }
+CliComm* MSXMotherBoard::getMSXCliCommIfAvailable()
+{
+	return pimple->getMSXCliCommIfAvailable();
+}
 MSXCommandController& MSXMotherBoard::getMSXCommandController()
 {
 	return pimple->getMSXCommandController();
@@ -1151,6 +1466,10 @@ RenShaTurbo& MSXMotherBoard::getRenShaTurbo()
 {
 	return pimple->getRenShaTurbo();
 }
+LedStatus& MSXMotherBoard::getLedStatus()
+{
+	return pimple->getLedStatus();
+}
 EventDistributor& MSXMotherBoard::getEventDistributor()
 {
 	return pimple->getEventDistributor();
@@ -1171,14 +1490,23 @@ GlobalSettings& MSXMotherBoard::getGlobalSettings()
 {
 	return pimple->getGlobalSettings();
 }
-GlobalCliComm& MSXMotherBoard::getGlobalCliComm()
+CliComm& MSXMotherBoard::getGlobalCliComm()
 {
+	// note: return-type is CliComm instead of GlobalCliComm
 	return pimple->getGlobalCliComm();
 }
 CommandController& MSXMotherBoard::getCommandController()
 {
 	return pimple->getCommandController();
 }
+InfoCommand& MSXMotherBoard::getMachineInfoCommand()
+{
+	return pimple->getMachineInfoCommand();
+}
+EmuTime::param MSXMotherBoard::getCurrentTime()
+{
+	return pimple->getCurrentTime();
+}
 void MSXMotherBoard::addDevice(MSXDevice& device)
 {
 	pimple->addDevice(device);
@@ -1195,6 +1523,14 @@ MSXMotherBoard::SharedStuff& MSXMotherBoard::getSharedStuff(const string& name)
 {
 	return pimple->getSharedStuff(name);
 }
+MSXMapperIO* MSXMotherBoard::createMapperIO()
+{
+	return pimple->createMapperIO();
+}
+void MSXMotherBoard::destroyMapperIO()
+{
+	pimple->destroyMapperIO();
+}
 string MSXMotherBoard::getUserName(const string& hwName)
 {
 	return pimple->getUserName(hwName);
@@ -1205,4 +1541,11 @@ void MSXMotherBoard::freeUserName(const string& hwName,
 	pimple->freeUserName(hwName, userName);
 }
 
+template<typename Archive>
+void MSXMotherBoard::serialize(Archive& ar, unsigned version)
+{
+	pimple->serialize(ar, version);
+}
+INSTANTIATE_SERIALIZE_METHODS(MSXMotherBoard)
+
 } // namespace openmsx
