@@ -1,4 +1,4 @@
-// $Id: MSXS1990.cc 6805 2007-08-06 12:01:49Z m9710797 $
+// $Id: MSXS1990.cc 8353 2008-11-13 18:38:44Z m9710797 $
 
 #include "MSXS1990.hh"
 #include "MSXCPU.hh"
@@ -6,6 +6,7 @@
 #include "PanasonicMemory.hh"
 #include "FirmwareSwitch.hh"
 #include "SimpleDebuggable.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
@@ -20,32 +21,31 @@ private:
 };
 
 
-MSXS1990::MSXS1990(MSXMotherBoard& motherBoard, const XMLElement& config,
-                   const EmuTime& time)
-	: MSXDevice(motherBoard, config, time)
+MSXS1990::MSXS1990(MSXMotherBoard& motherBoard, const XMLElement& config)
+	: MSXDevice(motherBoard, config)
 	, firmwareSwitch(
 	      new FirmwareSwitch(motherBoard.getCommandController(), config))
 	, debuggable(new S1990Debuggable(motherBoard, *this))
 {
-	reset(time);
+	reset(EmuTime::dummy());
 }
 
 MSXS1990::~MSXS1990()
 {
 }
 
-void MSXS1990::reset(const EmuTime& /*time*/)
+void MSXS1990::reset(EmuTime::param /*time*/)
 {
-	registerSelect = 0;	// TODO check this
+	registerSelect = 0; // TODO check this
 	setCPUStatus(96);
 }
 
-byte MSXS1990::readIO(word port, const EmuTime& time)
+byte MSXS1990::readIO(word port, EmuTime::param time)
 {
 	return peekIO(port, time);
 }
 
-byte MSXS1990::peekIO(word port, const EmuTime& /*time*/) const
+byte MSXS1990::peekIO(word port, EmuTime::param /*time*/) const
 {
 	switch (port & 0x01) {
 	case 0:
@@ -58,7 +58,7 @@ byte MSXS1990::peekIO(word port, const EmuTime& /*time*/) const
 	}
 }
 
-void MSXS1990::writeIO(word port, byte value, const EmuTime& /*time*/)
+void MSXS1990::writeIO(word port, byte value, EmuTime::param /*time*/)
 {
 	switch (port & 0x01) {
 	case 0:
@@ -76,27 +76,27 @@ byte MSXS1990::readRegister(byte reg) const
 {
 	PRT_DEBUG("S1990: read reg " << int(reg));
 	switch (reg) {
-		case 5:
-			return firmwareSwitch->getStatus() ? 0x40 : 0x00;
-		case 6:
-			return cpuStatus;
-		case 13:
-			return 0x03;	//TODO
-		case 14:
-			return 0x2F;	//TODO
-		case 15:
-			return 0x8B;	//TODO
-		default:
-			return 0xFF;
+	case 5:
+		return firmwareSwitch->getStatus() ? 0x40 : 0x00;
+	case 6:
+		return cpuStatus;
+	case 13:
+		return 0x03; // TODO
+	case 14:
+		return 0x2F; // TODO
+	case 15:
+		return 0x8B; // TODO
+	default:
+		return 0xFF;
 	}
 }
 
 void MSXS1990::writeRegister(byte reg, byte value)
 {
 	switch (reg) {
-		case 6:
-			setCPUStatus(value);
-			break;
+	case 6:
+		setCPUStatus(value);
+		break;
 	}
 }
 
@@ -130,4 +130,18 @@ void S1990Debuggable::write(unsigned address, byte value)
 	s1990.writeRegister(address, value);
 }
 
+
+template<typename Archive>
+void MSXS1990::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<MSXDevice>(*this);
+	ar.serialize("registerSelect", registerSelect);
+	ar.serialize("cpuStatus", cpuStatus);
+	if (ar.isLoader()) {
+		setCPUStatus(cpuStatus);
+	}
+}
+INSTANTIATE_SERIALIZE_METHODS(MSXS1990);
+REGISTER_MSXDEVICE(MSXS1990, "S1990");
+
 } // namespace openmsx
