@@ -1,16 +1,16 @@
-// $Id: MSXTurboRPause.cc 5854 2006-11-06 18:24:21Z m9710797 $
+// $Id: MSXTurboRPause.cc 8353 2008-11-13 18:38:44Z m9710797 $
 
 #include "MSXTurboRPause.hh"
-#include "LedEvent.hh"
-#include "EventDistributor.hh"
+#include "LedStatus.hh"
 #include "MSXMotherBoard.hh"
 #include "BooleanSetting.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
 MSXTurboRPause::MSXTurboRPause(MSXMotherBoard& motherBoard,
-                               const XMLElement& config, const EmuTime& time)
-	: MSXDevice(motherBoard, config, time)
+                               const XMLElement& config)
+	: MSXDevice(motherBoard, config)
 	, pauseSetting(new BooleanSetting(motherBoard.getCommandController(),
 	               "turborpause", "status of the TurboR pause", false))
 	, status(255)
@@ -19,7 +19,7 @@ MSXTurboRPause::MSXTurboRPause(MSXMotherBoard& motherBoard,
 	, hwPause(false)
 {
 	pauseSetting->attach(*this);
-	reset(time);
+	reset(EmuTime::dummy());
 }
 
 MSXTurboRPause::~MSXTurboRPause()
@@ -27,36 +27,34 @@ MSXTurboRPause::~MSXTurboRPause()
 	pauseSetting->detach(*this);
 }
 
-void MSXTurboRPause::reset(const EmuTime& time)
+void MSXTurboRPause::reset(EmuTime::param dummy)
 {
-	pauseSetting->setValue(false);
-	writeIO(0, 0, time);
+	pauseSetting->changeValue(false);
+	writeIO(0, 0, dummy);
 }
 
-void MSXTurboRPause::powerDown(const EmuTime& time)
+void MSXTurboRPause::powerDown(EmuTime::param dummy)
 {
-	writeIO(0, 0, time);
+	writeIO(0, 0, dummy);
 }
 
-byte MSXTurboRPause::readIO(word port, const EmuTime& time)
+byte MSXTurboRPause::readIO(word port, EmuTime::param time)
 {
 	return peekIO(port, time);
 }
 
-byte MSXTurboRPause::peekIO(word /*port*/, const EmuTime& /*time*/) const
+byte MSXTurboRPause::peekIO(word /*port*/, EmuTime::param /*time*/) const
 {
 	return pauseSetting->getValue() ? 1 : 0;
 }
 
-void MSXTurboRPause::writeIO(word /*port*/, byte value, const EmuTime& /*time*/)
+void MSXTurboRPause::writeIO(word /*port*/, byte value, EmuTime::param /*time*/)
 {
 	status = value;
 	bool newTurboLed = (status & 0x80);
 	if (newTurboLed != turboLed) {
 		turboLed = newTurboLed;
-		getMotherBoard().getEventDistributor().distributeEvent(
-			new LedEvent(LedEvent::TURBO, turboLed,
-			             getMotherBoard()));
+		getMotherBoard().getLedStatus().setLed(LedStatus::TURBO, turboLed);
 	}
 	updatePause();
 }
@@ -81,10 +79,20 @@ void MSXTurboRPause::updatePause()
 	bool newPauseLed = (status & 0x01) || hwPause;
 	if (newPauseLed != pauseLed) {
 		pauseLed = newPauseLed;
-		getMotherBoard().getEventDistributor().distributeEvent(
-			new LedEvent(LedEvent::PAUSE, pauseLed,
-			             getMotherBoard()));
+		getMotherBoard().getLedStatus().setLed(LedStatus::PAUSE, pauseLed);
 	}
 }
 
+template<typename Archive>
+void MSXTurboRPause::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<MSXDevice>(*this);
+	ar.serialize("status", status);
+	if (ar.isLoader()) {
+		writeIO(0, status, EmuTime::dummy());
+	}
+}
+INSTANTIATE_SERIALIZE_METHODS(MSXTurboRPause);
+REGISTER_MSXDEVICE(MSXTurboRPause, "TurboRPause");
+
 } // namespace openmsx
