@@ -1,12 +1,14 @@
-// $Id: Printer.cc 6807 2007-08-06 17:08:53Z m9710797 $
+// $Id: Printer.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "Printer.hh"
 #include "ScreenShotSaver.hh"
 #include "FileOperations.hh"
-#include "FloatSetting.hh"
+#include "IntegerSetting.hh"
 #include "MSXMotherBoard.hh"
-#include "MSXCliComm.hh"
+#include "CliComm.hh"
+#include "MSXException.hh"
 #include "Math.hh"
+#include "serialize.hh"
 #include <algorithm>
 #include <vector>
 #include <cassert>
@@ -53,12 +55,12 @@ PrinterCore::~PrinterCore()
 {
 }
 
-bool PrinterCore::getStatus(const EmuTime& /*time*/)
+bool PrinterCore::getStatus(EmuTime::param /*time*/)
 {
-	return false;	// false = low = ready
+	return false; // false = low = ready
 }
 
-void PrinterCore::setStrobe(bool strobe, const EmuTime& /*time*/)
+void PrinterCore::setStrobe(bool strobe, EmuTime::param /*time*/)
 {
 	if (!strobe && prevStrobe) {
 		// falling edge
@@ -67,17 +69,17 @@ void PrinterCore::setStrobe(bool strobe, const EmuTime& /*time*/)
 	prevStrobe = strobe;
 }
 
-void PrinterCore::writeData(byte data, const EmuTime& /*time*/)
+void PrinterCore::writeData(byte data, EmuTime::param /*time*/)
 {
 	toPrint = data;
 }
 
-void PrinterCore::plugHelper(Connector& /*connector*/, const EmuTime& /*time*/)
+void PrinterCore::plugHelper(Connector& /*connector*/, EmuTime::param /*time*/)
 {
 	// nothing
 }
 
-void PrinterCore::unplugHelper(const EmuTime& /*time*/)
+void PrinterCore::unplugHelper(EmuTime::param /*time*/)
 {
 	forceFormFeed();
 }
@@ -122,13 +124,13 @@ ImagePrinter::ImagePrinter(MSXMotherBoard& motherBoard_)
 		motherBoard.getSharedStuff("print-resolution");
 	if (info.counter == 0) {
 		assert(info.stuff == NULL);
-		info.stuff = new FloatSetting(
-	        	motherBoard.getCommandController(), "print-resolution",
-	        	"resolution of the ouput image of emulated dot matrix printer in DPI",
-	        	300, 150, 1200);
+		info.stuff = new IntegerSetting(
+		    motherBoard.getCommandController(), "print-resolution",
+		    "resolution of the ouput image of emulated dot matrix printer in DPI",
+		    300, 72, 1200);
 	}
 	++info.counter;
-	dpiSetting = reinterpret_cast<FloatSetting*>(info.stuff);
+	dpiSetting = reinterpret_cast<IntegerSetting*>(info.stuff);
 
 	letterQuality = false;
 	bold = false;
@@ -311,7 +313,7 @@ void ImagePrinter::flushEmulatedPrinter()
 static unsigned compress9(unsigned a)
 {
 	unsigned result = 0;
-	for (int i = 0; i < 9; ++i) {
+	for (unsigned i = 0; i < 9; ++i) {
 		if (a & (1 << i)) {
 			result |= 1 << (i / 2);
 		}
@@ -829,7 +831,7 @@ void ImagePrinterMSX::processEscSequence()
 					break;
 			}
 			break;
-		case '/': //Right margin
+		case '/': // Right margin
 			break;
 		case 'L':
 			leftBorder = parseNumber(1, 3);
@@ -955,6 +957,14 @@ void ImagePrinterMSX::processCharacter(byte data)
 	}
 }
 
+template<typename Archive>
+void ImagePrinterMSX::serialize(Archive& /*ar*/, unsigned /*version*/)
+{
+	// TODO is this worth it?
+}
+INSTANTIATE_SERIALIZE_METHODS(ImagePrinterMSX);
+REGISTER_POLYMORPHIC_INITIALIZER(Pluggable, ImagePrinterMSX, "ImagePrinterMSX");
+
 
 // class ImagePrinterEpson
 
@@ -1661,6 +1671,14 @@ void ImagePrinterEpson::processCharacter(byte data)
 	}
 }
 
+template<typename Archive>
+void ImagePrinterEpson::serialize(Archive& /*ar*/, unsigned /*version*/)
+{
+	// TODO is this worth it?
+}
+INSTANTIATE_SERIALIZE_METHODS(ImagePrinterEpson);
+REGISTER_POLYMORPHIC_INITIALIZER(Pluggable, ImagePrinterEpson, "ImagePrinterEpson");
+
 
 // class Paper
 
@@ -1680,11 +1698,11 @@ string Paper::save() const
 {
 	string filename = FileOperations::getNextNumberedFileName(
 		"prints", "page", ".png");
-	std::vector<byte*> row_pointers(sizeY);
+	const void* rowPointers[sizeY];
 	for (unsigned y = 0; y < sizeY; ++y) {
-		row_pointers[y] = &buf[sizeX * y];
+		rowPointers[y] = &buf[sizeX * y];
 	}
-	ScreenShotSaver::saveGrayscale(sizeX, sizeY, &row_pointers[0], filename);
+	ScreenShotSaver::saveGrayscale(sizeX, sizeY, rowPointers, filename);
 	return filename;
 }
 
