@@ -1,15 +1,16 @@
-// $Id: RP5C01.cc 6593 2007-06-03 23:12:19Z mthuurne $
+// $Id: RP5C01.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "RP5C01.hh"
 #include "EnumSetting.hh"
 #include "SRAM.hh"
+#include "serialize.hh"
 #include <cassert>
 #include <ctime>
 
 namespace openmsx {
 
-//TODO  ALARM is not implemented        (not connected on MSX)
-//TODO  1Hz 16Hz output not implemented (not connected on MSX)
+// TODO  ALARM is not implemented        (not connected on MSX)
+// TODO  1Hz 16Hz output not implemented (not connected on MSX)
 
 const nibble MODE_REG  = 13;
 const nibble TEST_REG  = 14;
@@ -39,17 +40,22 @@ static const nibble mask[4][13] = {
 	{ 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf}
 };
 
+static EnumSetting<RP5C01::RTCMode>* createModeSetting(
+	CommandController& commandController)
+{
+	EnumSetting<RP5C01::RTCMode>::Map modeMap;
+	modeMap["EmuTime"]  = RP5C01::EMUTIME;
+	modeMap["RealTime"] = RP5C01::REALTIME;
+	return new EnumSetting<RP5C01::RTCMode>(commandController,
+		"rtcmode", "Real Time Clock mode", RP5C01::EMUTIME, modeMap);
+}
 
 RP5C01::RP5C01(CommandController& commandController, SRAM& regs_,
-               const EmuTime& time)
-	: regs(regs_), reference(time)
+               EmuTime::param time)
+	: regs(regs_)
+	, modeSetting(createModeSetting(commandController))
+	, reference(time)
 {
-	EnumSetting<RTCMode>::Map modeMap;
-	modeMap["EmuTime"] = EMUTIME;
-	modeMap["RealTime"] = REALTIME;
-	modeSetting.reset(new EnumSetting<RTCMode>(commandController,
-		"rtcmode", "Real Time Clock mode", EMUTIME, modeMap));
-
 	initializeTime();
 	reset(time);
 }
@@ -58,7 +64,7 @@ RP5C01::~RP5C01()
 {
 }
 
-void RP5C01::reset(const EmuTime& time)
+void RP5C01::reset(EmuTime::param time)
 {
 	modeReg = MODE_TIMERENABLE;
 	testReg = 0;
@@ -66,51 +72,56 @@ void RP5C01::reset(const EmuTime& time)
 	updateTimeRegs(time);
 }
 
-nibble RP5C01::readPort(nibble port, const EmuTime& time)
+nibble RP5C01::readPort(nibble port, EmuTime::param time)
 {
-	assert (port<=0x0f);
+	assert(port <= 0x0f);
 	switch (port) {
-		case MODE_REG:
-			return modeReg;
-		case TEST_REG:
-		case RESET_REG:
-			// write only
-			return 0x0f;	// TODO check this
-		default:
-			int block = modeReg & MODE_BLOKSELECT;
-			if (block == TIME_BLOCK)
-				updateTimeRegs(time);
-			nibble tmp = regs[block * 13 + port];
-			return tmp & mask[block][port];
+	case MODE_REG:
+		return modeReg;
+	case TEST_REG:
+	case RESET_REG:
+		// write only
+		return 0x0f; // TODO check this
+	default:
+		unsigned block = modeReg & MODE_BLOKSELECT;
+		if (block == TIME_BLOCK) {
+			updateTimeRegs(time);
+		}
+		nibble tmp = regs[block * 13 + port];
+		return tmp & mask[block][port];
 	}
 }
 
-void RP5C01::writePort(nibble port, nibble value, const EmuTime& time)
+void RP5C01::writePort(nibble port, nibble value, EmuTime::param time)
 {
 	assert (port<=0x0f);
 	switch (port) {
-		case MODE_REG:
+	case MODE_REG:
+		updateTimeRegs(time);
+		modeReg = value;
+		break;
+	case TEST_REG:
+		updateTimeRegs(time);
+		testReg = value;
+		break;
+	case RESET_REG:
+		resetReg = value;
+		if (value & RESET_ALARM) {
+			resetAlarm();
+		}
+		if (value & RESET_FRACTION) {
+			fraction = 0;
+		}
+		break;
+	default:
+		unsigned block = modeReg & MODE_BLOKSELECT;
+		if (block == TIME_BLOCK) {
 			updateTimeRegs(time);
-			modeReg = value;
-			break;
-		case TEST_REG:
-			updateTimeRegs(time);
-			testReg = value;
-			break;
-		case RESET_REG:
-			resetReg = value;
-			if (value & RESET_ALARM)
-				resetAlarm();
-			if (value & RESET_FRACTION)
-				fraction = 0;
-			break;
-		default:
-			int block = modeReg & MODE_BLOKSELECT;
-			if (block == TIME_BLOCK)
-				updateTimeRegs(time);
-			regs.write(block * 13 + port, value & mask[block][port]);
-			if (block == TIME_BLOCK)
-				regs2Time();
+		}
+		regs.write(block * 13 + port, value & mask[block][port]);
+		if (block == TIME_BLOCK) {
+			regs2Time();
+		}
 	}
 }
 
@@ -149,7 +160,7 @@ void RP5C01::regs2Time()
 
 void RP5C01::time2Regs()
 {
-	int hours_ = hours;
+	unsigned hours_ = hours;
 	if (!regs[ALARM_BLOCK * 13 + 10]) {
 		// 12 hours mode
 		if (hours >= 12) hours_ = (hours - 12) + 20;
@@ -162,8 +173,8 @@ void RP5C01::time2Regs()
 	regs.write(TIME_BLOCK  * 13 +  4,  hours_    % 10);
 	regs.write(TIME_BLOCK  * 13 +  5,  hours_    / 10);
 	regs.write(TIME_BLOCK  * 13 +  6,  dayWeek);
-	regs.write(TIME_BLOCK  * 13 +  7, (days+1)   % 10);	// 0-30 -> 1-31
-	regs.write(TIME_BLOCK  * 13 +  8, (days+1)   / 10);	// 0-11 -> 1-12
+	regs.write(TIME_BLOCK  * 13 +  7, (days+1)   % 10); // 0-30 -> 1-31
+	regs.write(TIME_BLOCK  * 13 +  8, (days+1)   / 10); // 0-11 -> 1-12
 	regs.write(TIME_BLOCK  * 13 +  9, (months+1) % 10);
 	regs.write(TIME_BLOCK  * 13 + 10, (months+1) / 10);
 	regs.write(TIME_BLOCK  * 13 + 11,  years     % 10);
@@ -171,9 +182,9 @@ void RP5C01::time2Regs()
 	regs.write(ALARM_BLOCK * 13 + 11,  leapYear);
 }
 
-static int daysInMonth(int month, int leapYear)
+static int daysInMonth(int month, unsigned leapYear)
 {
-	const int daysInMonths[12] = {
+	const unsigned daysInMonths[12] = {
 		31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
 	};
 
@@ -181,7 +192,7 @@ static int daysInMonth(int month, int leapYear)
 	return ((month == 1) && (leapYear == 0)) ? 29 : daysInMonths[month];
 }
 
-void RP5C01::updateTimeRegs(const EmuTime& time)
+void RP5C01::updateTimeRegs(EmuTime::param time)
 {
 	if (modeSetting->getValue() == EMUTIME) {
 		// sync with EmuTime, perfect emulation
@@ -190,15 +201,15 @@ void RP5C01::updateTimeRegs(const EmuTime& time)
 
 		// in test mode increase sec/min/.. at a rate of 16384Hz
 		fraction += (modeReg & MODE_TIMERENABLE) ? elapsed : 0;
-		int carrySeconds = (testReg & TEST_SECONDS)
-		                 ? elapsed : fraction / FREQ;
+		unsigned carrySeconds = (testReg & TEST_SECONDS)
+		                      ? elapsed : fraction / FREQ;
 		seconds  += carrySeconds;
-		int carryMinutes = (testReg & TEST_MINUTES)
-		                 ? elapsed : seconds / 60;
+		unsigned carryMinutes = (testReg & TEST_MINUTES)
+		                      ? elapsed : seconds / 60;
 		minutes  += carryMinutes;
 		hours    += minutes / 60;
-		int carryDays = (testReg & TEST_DAYS)
-		              ? elapsed : hours / 24;
+		unsigned carryDays = (testReg & TEST_DAYS)
+		                   ? elapsed : hours / 24;
 		days     += carryDays;
 		dayWeek  += carryDays;
 		while (days >= daysInMonth(months, leapYear)) {
@@ -208,8 +219,8 @@ void RP5C01::updateTimeRegs(const EmuTime& time)
 			days -= daysInMonth(months, leapYear);
 			months++;
 		}
-		int carryYears = (testReg & TEST_YEARS)
-		               ? elapsed : months / 12;
+		unsigned carryYears = (testReg & TEST_YEARS)
+		                    ? elapsed : months / 12;
 		years    += carryYears;
 		leapYear += carryYears;
 
@@ -232,9 +243,28 @@ void RP5C01::updateTimeRegs(const EmuTime& time)
 
 void RP5C01::resetAlarm()
 {
-	for (int i = 2; i <= 8; i++) {
+	for (unsigned i = 2; i <= 8; ++i) {
 		regs.write(ALARM_BLOCK * 13 + i, 0);
 	}
 }
 
+template<typename Archive>
+void RP5C01::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.serialize("reference", reference);
+	ar.serialize("fraction",  fraction);
+	ar.serialize("seconds",   seconds);
+	ar.serialize("minutes",   minutes);
+	ar.serialize("hours",     hours);
+	ar.serialize("dayWeek",   dayWeek);
+	ar.serialize("years",     years);
+	ar.serialize("leapYear",  leapYear);
+	ar.serialize("days",      days);
+	ar.serialize("months",    months);
+	ar.serialize("modeReg",   modeReg);
+	ar.serialize("testReg",   testReg);
+	ar.serialize("resetReg",  resetReg);
+}
+INSTANTIATE_SERIALIZE_METHODS(RP5C01);
+
 } // namespace openmsx
