@@ -1,6 +1,8 @@
-// $Id: CassettePlayer.cc 7239 2007-11-06 20:28:03Z m9710797 $
+// $Id: CassettePlayer.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
-//TODO:
+// TODO:
+// - improve consistency when a reset occurs: tape is removed when you were
+//   recording, but it is not removed when you were playing
 // - specify prefix for auto file name generation when recording (setting?)
 // - append to existing wav files when recording (record command), but this is
 //   basically a special case (pointer at the end) of:
@@ -24,16 +26,15 @@
 #include "BooleanSetting.hh"
 #include "Connector.hh"
 #include "CassettePort.hh"
-#include "MSXCommandController.hh"
+#include "CommandController.hh"
 #include "RecordedCommand.hh"
 #include "GlobalSettings.hh"
 #include "XMLElement.hh"
 #include "FileContext.hh"
 #include "WavImage.hh"
 #include "CasImage.hh"
-#include "MSXCliComm.hh"
+#include "CliComm.hh"
 #include "CommandException.hh"
-#include "Scheduler.hh"
 #include "EventDistributor.hh"
 #include "FileOperations.hh"
 #include "WavWriter.hh"
@@ -41,8 +42,10 @@
 #include "TclObject.hh"
 #include "DynamicClock.hh"
 #include "Clock.hh"
+#include "StringOp.hh"
 #include <algorithm>
 #include <cassert>
+#include <unistd.h>
 
 using std::auto_ptr;
 using std::string;
@@ -62,11 +65,11 @@ enum SyncType {
 class TapeCommand : public RecordedCommand
 {
 public:
-	TapeCommand(MSXCommandController& msxCommandController,
+	TapeCommand(CommandController& commandController,
 	            MSXEventDistributor& msxEventDistributor,
 	            Scheduler& scheduler,
 	            CassettePlayer& cassettePlayer);
-	virtual string execute(const vector<string>& tokens, const EmuTime& time);
+	virtual string execute(const vector<string>& tokens, EmuTime::param time);
 	virtual string help(const vector<string>& tokens) const;
 	virtual void tabCompletion(vector<string>& tokens) const;
 private:
@@ -75,33 +78,32 @@ private:
 
 
 CassettePlayer::CassettePlayer(
-		MSXCommandController& msxCommandController_,
+		CommandController& commandController_,
 		MSXMixer& mixer, Scheduler& scheduler,
 		MSXEventDistributor& msxEventDistributor,
 		EventDistributor& eventDistributor_,
-		MSXCliComm& cliComm_)
+		CliComm& cliComm_)
 	: SoundDevice(mixer, getName(), getDescription(), 1)
-	, Resample(msxCommandController_.getGlobalSettings(), 1)
+	, Resample(commandController_.getGlobalSettings(), 1)
 	, Schedulable(scheduler)
-	, tapeTime(EmuTime::zero)
-	, recTime(EmuTime::zero)
-	, prevTime(EmuTime::zero)
-	, playPos(0)
-	, msxCommandController(msxCommandController_)
+	, tapePos(EmuTime::zero)
+	, prevSyncTime(EmuTime::zero)
+	, audioPos(0)
+	, commandController(commandController_)
 	, cliComm(cliComm_)
 	, eventDistributor(eventDistributor_)
-	, tapeCommand(new TapeCommand(msxCommandController, msxEventDistributor,
+	, tapeCommand(new TapeCommand(commandController, msxEventDistributor,
 	                              scheduler, *this))
 	, loadingIndicator(new LoadingIndicator(
-	       msxCommandController.getGlobalSettings().getThrottleManager()))
+	       commandController.getGlobalSettings().getThrottleManager()))
+	, autoRunSetting(new BooleanSetting(commandController,
+		"autoruncassettes", "automatically try to run cassettes", false))
 	, sampcnt(0)
 	, state(STOP)
 	, lastOutput(false)
 	, motor(false), motorControl(true)
 	, syncScheduled(false)
 {
-	autoRunSetting.reset(new BooleanSetting(msxCommandController,
-		"autoruncassettes", "automatically try to run cassettes", false));
 	removeTape(EmuTime::zero);
 
 	static XMLElement cassettePlayerConfig("cassetteplayer");
@@ -121,7 +123,7 @@ CassettePlayer::~CassettePlayer()
 {
 	unregisterSound();
 	if (Connector* connector = getConnector()) {
-		connector->unplug(getScheduler().getCurrentTime());
+		connector->unplug(getCurrentTime());
 	}
 	eventDistributor.unregisterEventListener(OPENMSX_BOOT_EVENT, *this);
 	cliComm.update(CliComm::HARDWARE, getName(), "remove");
@@ -129,6 +131,8 @@ CassettePlayer::~CassettePlayer()
 
 void CassettePlayer::autoRun()
 {
+	if (!playImage.get()) return;
+
 	// try to automatically run the tape, if that's set
 	CassetteImage::FileType type = playImage->getFirstFileType();
 	if (!autoRunSetting->getValue() || type == CassetteImage::UNKNOWN) {
@@ -155,7 +159,7 @@ void CassettePlayer::autoRun()
 		"after time 2 \"if $" + var + "==\\$" + var + " { "
 		"type " + loadingInstruction + "\\\\r }\"";
 	try {
-		msxCommandController.executeCommand(command);
+		commandController.executeCommand(command);
 	} catch (CommandException& e) {
 		cliComm.printWarning(
 			"Error executing loading instruction for AutoRun: " +
@@ -168,20 +172,9 @@ CassettePlayer::State CassettePlayer::getState() const
 	return state;
 }
 
-bool CassettePlayer::isRolling() const
+string CassettePlayer::getStateString() const
 {
-	// TODO implement end-of-tape
-	// Is the tape 'rolling'?
-	// is true when:
-	//  not in stop mode (there is a tape inserted and not at end-of-tape)
-	//  AND [ user forced playing (motorcontrol=off) OR motor enabled by
-	//        software (motor=on) ]
-	return (getState() != STOP) && (motor || !motorControl);
-}
-
-string CassettePlayer::getStateString(State state)
-{
-	switch (state) {
+	switch (getState()) {
 		case PLAY:   return "play";
 		case RECORD: return "record";
 		case STOP:   return "stop";
@@ -190,11 +183,44 @@ string CassettePlayer::getStateString(State state)
 	return "";
 }
 
+bool CassettePlayer::isRolling() const
+{
+	// Is the tape 'rolling'?
+	// is true when:
+	//  not in stop mode (there is a tape inserted and not at end-of-tape)
+	//  AND [ user forced playing (motorcontrol=off) OR motor enabled by
+	//        software (motor=on) ]
+	return (getState() != STOP) && (motor || !motorControl);
+}
+
+double CassettePlayer::getTapePos(EmuTime::param time)
+{
+	sync(time);
+	return (tapePos - EmuTime::zero).toDouble();
+}
+
+double CassettePlayer::getTapeLength(EmuTime::param time)
+{
+	if (playImage.get()) {
+		return (playImage->getEndTime() - EmuTime::zero).toDouble();
+	} else if (getState() == RECORD) {
+		return getTapePos(time);
+	} else {
+		return 0.0;
+	}
+}
+
 void CassettePlayer::checkInvariants() const
 {
 	switch (getState()) {
 	case STOP:
 		assert(!recordImage.get());
+		if (playImage.get()) {
+			// we're at end-of tape
+			assert(!getImageName().empty());
+		} else {
+			// no tape inserted, imageName may or may not be empty
+		}
 		break;
 	case PLAY:
 		assert(!getImageName().empty());
@@ -211,209 +237,237 @@ void CassettePlayer::checkInvariants() const
 	}
 }
 
-void CassettePlayer::setState(State newState, const EmuTime& time)
+void CassettePlayer::setState(State newState, const Filename& newImage,
+                              EmuTime::param time)
 {
-	if (getImageName().empty()) {
-		// no image, always STOP state
-		newState = STOP;
-	}
+	sync(time);
 
 	// set new state if different from old state
 	State oldState = getState();
 	if (oldState == newState) return;
-	state = newState;
+
+	// cannot directly switch from PLAY to RECORD or vice-versa,
+	// (should always go via STOP)
+	assert(!((oldState == PLAY)   && (newState == RECORD)));
+	assert(!((oldState == RECORD) && (newState == PLAY)));
 
 	// stuff for leaving the old state
 	if (oldState == RECORD) {
-		setSignal(lastOutput, time);
 		flushOutput();
+		bool empty = recordImage.get()->isEmpty();
 		recordImage.reset();
+		if (empty) {
+			// delete the created WAV file, as it is useless
+			unlink(getImageName().getResolved().c_str());
+			setImageName(Filename());
+		}
 	}
 
+	// actually switch state
+	state = newState;
+	setImageName(newImage);
+
 	// stuff for entering the new state
 	if (newState == RECORD) {
 		partialOut = 0.0;
 		partialInterval = 0.0;
 		lastX = lastOutput ? OUTPUT_AMP : -OUTPUT_AMP;
 		lastY = 0.0;
-	} else if (newState == PLAY) {
-		prevTime = time;
 	}
-	cliComm.update(CliComm::STATUS, "cassetteplayer",
-	               getStateString(newState));
+	cliComm.update(CliComm::STATUS, "cassetteplayer", getStateString());
 
 	updateLoadingState(time); // sets SP for tape-end detection
 
 	checkInvariants();
 }
 
-void CassettePlayer::updateLoadingState(const EmuTime& time)
+void CassettePlayer::updateLoadingState(EmuTime::param time)
 {
 	// TODO also set loadingIndicator for RECORD?
 	// note: we don't use isRolling()
 	loadingIndicator->update(motor && (getState() == PLAY));
 
 	removeSyncPoint(END_OF_TAPE);
-	if ((motor || !motorControl) && (getState() == PLAY)) {
-		setSyncPoint(time + (playImage->getEndTime() - tapeTime), END_OF_TAPE);
+	if (isRolling() && (getState() == PLAY)) {
+		setSyncPoint(time + (playImage->getEndTime() - tapePos), END_OF_TAPE);
 	}
 }
 
-void CassettePlayer::setImageName(const string& newImage)
+void CassettePlayer::setImageName(const Filename& newImage)
 {
 	casImage = newImage;
-	cliComm.update(CliComm::MEDIA, "cassetteplayer", casImage);
+	cliComm.update(CliComm::MEDIA, "cassetteplayer", casImage.getResolved());
 }
 
-const string& CassettePlayer::getImageName() const
+const Filename& CassettePlayer::getImageName() const
 {
 	return casImage;
 }
 
-void CassettePlayer::playTape(const string& filename, const EmuTime& time)
+void CassettePlayer::insertTape(const Filename& filename)
 {
-	try {
-		// first try WAV
-		playImage.reset(new WavImage(filename));
-	} catch (MSXException& e) {
-		// if that fails use CAS
-		playImage.reset(new CasImage(filename, cliComm));
+	if (!filename.empty()) {
+		try {
+			// first try WAV
+			playImage.reset(new WavImage(filename));
+		} catch (MSXException& e) {
+			try {
+				// if that fails use CAS
+				playImage.reset(new CasImage(filename, cliComm));
+			} catch (MSXException& e2) {
+				throw MSXException(
+					"Failed to insert WAV image: \"" +
+					e.getMessage() +
+					"\" and also failed to insert CAS image: \"" +
+					e2.getMessage() + "\"");
+			}
+		}
+	} else {
+		// This is a bit tricky, consider this scenario: we switch from
+		// RECORD->PLAY, but we didn't actually record anything: The
+		// removeTape() call above (indirectly) deletes the empty
+		// recorded wav image and also clears imageName. Now because
+		// the 'filename' parameter is passed by reference, and because
+		// getImageName() returns a reference, this 'filename'
+		// parameter now also is an empty string.
 	}
 	setImageName(filename);
+}
+
+void CassettePlayer::playTape(const Filename& filename, EmuTime::param time)
+{
+	if (getState() == RECORD) {
+		// First close the recorded image. Otherwise it goes wrong
+		// if you switch from RECORD->PLAY on the same image.
+		setState(STOP, getImageName(), time); // keep current image
+	}
+	insertTape(filename);
 	rewind(time); // sets PLAY mode
 	autoRun();
-	setOutputRate(outputRate);
+	setOutputRate(outputRate); // recalculate resample stuff
 }
 
-void CassettePlayer::rewind(const EmuTime& time)
+void CassettePlayer::rewind(EmuTime::param time)
 {
-	tapeTime = EmuTime::zero;
-	playPos = 0;
-	setState(PLAY, time);
+	assert(getState() != RECORD);
+	tapePos = EmuTime::zero;
+	audioPos = 0;
+
+	if (getImageName().empty()) {
+		// no image inserted, do nothing
+		assert(getState() == STOP);
+	} else {
+		// keep current image
+		setState(PLAY, getImageName(), time);
+	}
 }
 
-void CassettePlayer::recordTape(const string& filename, const EmuTime& time)
+void CassettePlayer::recordTape(const Filename& filename, EmuTime::param time)
 {
 	removeTape(time); // flush (possible) previous recording
 	recordImage.reset(new WavWriter(filename, 1, 8, RECORD_FREQ));
-	setImageName(filename);
-	setState(RECORD, time);
-	recTime = time;
+	tapePos = EmuTime::zero;
+	setState(RECORD, filename, time);
 }
 
-void CassettePlayer::removeTape(const EmuTime& time)
+void CassettePlayer::removeTape(EmuTime::param time)
 {
 	playImage.reset();
-	setImageName("");
-	setState(STOP, time);
+	tapePos = EmuTime::zero;
+	setState(STOP, Filename(), time);
 }
 
-void CassettePlayer::setMotor(bool status, const EmuTime& time)
+void CassettePlayer::setMotor(bool status, EmuTime::param time)
 {
 	if (status != motor) {
-		updateAll(time);
+		sync(time);
 		motor = status;
 		updateLoadingState(time);
 	}
 }
 
-void CassettePlayer::setMotorControl(bool status, const EmuTime& time)
+void CassettePlayer::setMotorControl(bool status, EmuTime::param time)
 {
 	if (status != motorControl) {
-		updateAll(time);
+		sync(time);
 		motorControl = status;
 		updateLoadingState(time);
 	}
 }
 
-short CassettePlayer::getSample(const EmuTime& time)
-{
-	assert(getState() == PLAY);
-	return isRolling() ? playImage->getSampleAt(time) : 0;
-}
-
-short CassettePlayer::readSample(const EmuTime& time)
+short CassettePlayer::readSample(EmuTime::param time)
 {
 	if (getState() == PLAY) {
 		// playing
-		updatePlayPosition(time);
-		return getSample(tapeTime);
+		sync(time);
+		return isRolling() ? playImage->getSampleAt(tapePos) : 0;
 	} else {
 		// record or stop
 		return 0;
 	}
 }
 
-void CassettePlayer::updatePlayPosition(const EmuTime& time)
+void CassettePlayer::setSignal(bool output, EmuTime::param time)
 {
-	assert(getState() == PLAY);
-	if (isRolling()) {
-		tapeTime += (time - prevTime);
-
-		if (!syncScheduled) {
-			// don't sync too often, this improves sound quality
-			syncScheduled = true;
-			Clock<1> next(time);
-			next += 1;
-			setSyncPoint(next.getTime(), SYNC_AUDIO_EMU);
-		}
-	}
-	prevTime = time;
-}
-
-void CassettePlayer::updateAll(const EmuTime& time)
-{
-	switch (getState()) {
-	case PLAY:
-		updatePlayPosition(time);
-		break;
-	case RECORD:
-		if (isRolling()) {
-			// was already recording, update output
-			setSignal(lastOutput, time);
-		} else {
-			// (possibly) restart recording, reset parameters
-			recTime = time;
-		}
-		flushOutput();
-		break;
-	default:
-		// nothing
-		break;
-	}
-}
-
-void CassettePlayer::setSignal(bool output, const EmuTime& time)
-{
-	if (recordImage.get() && isRolling()) {
-		double out = output ? OUTPUT_AMP : -OUTPUT_AMP;
-		double samples = (time - recTime).toDouble() * RECORD_FREQ;
-		double rest = 1.0 - partialInterval;
-		if (rest <= samples) {
-			// enough to fill next interval
-			partialOut += out * rest;
-			fillBuf(1, int(partialOut));
-			samples -= rest;
-
-			// fill complete intervals
-			int count = int(samples);
-			if (count > 0) {
-				fillBuf(count, int(out));
-			}
-			samples -= count;
-
-			// partial last interval
-			partialOut = samples * out;
-			partialInterval = 0.0;
-		} else {
-			partialOut += samples * out;
-			partialInterval += samples;
-		}
-	}
-	recTime = time;
+	sync(time);
 	lastOutput = output;
 }
 
+void CassettePlayer::sync(EmuTime::param time)
+{
+	EmuDuration duration = time - prevSyncTime;
+	prevSyncTime = time;
+
+	updateTapePosition(duration, time);
+	generateRecordOutput(duration);
+}
+
+void CassettePlayer::updateTapePosition(
+	EmuDuration::param duration, EmuTime::param time)
+{
+	if (!isRolling()) return;
+
+	tapePos += duration;
+
+	// synchronize audio with actual tape position
+	if ((getState() == PLAY) && !syncScheduled) {
+		// don't sync too often, this improves sound quality
+		syncScheduled = true;
+		Clock<1> next(time);
+		next += 1;
+		setSyncPoint(next.getTime(), SYNC_AUDIO_EMU);
+	}
+}
+
+void CassettePlayer::generateRecordOutput(EmuDuration::param duration)
+{
+	if (!recordImage.get() || !isRolling()) return;
+
+	double out = lastOutput ? OUTPUT_AMP : -OUTPUT_AMP;
+	double samples = duration.toDouble() * RECORD_FREQ;
+	double rest = 1.0 - partialInterval;
+	if (rest <= samples) {
+		// enough to fill next interval
+		partialOut += out * rest;
+		fillBuf(1, int(partialOut));
+		samples -= rest;
+
+		// fill complete intervals
+		int count = int(samples);
+		if (count > 0) {
+			fillBuf(count, int(out));
+		}
+		samples -= count;
+
+		// partial last interval
+		partialOut = samples * out;
+		partialInterval = 0.0;
+	} else {
+		partialOut += samples * out;
+		partialInterval += samples;
+	}
+}
+
 void CassettePlayer::fillBuf(size_t length, double x)
 {
 	assert(recordImage.get());
@@ -461,15 +515,15 @@ const string& CassettePlayer::getDescription() const
 	return desc;
 }
 
-void CassettePlayer::plugHelper(Connector& connector, const EmuTime& time)
+void CassettePlayer::plugHelper(Connector& connector, EmuTime::param time)
 {
-	lastOutput = static_cast<CassettePortInterface&>(connector).lastOut();
-	updateAll(time);
+	sync(time);
+	lastOutput = static_cast<CassettePort&>(connector).lastOut();
 }
 
-void CassettePlayer::unplugHelper(const EmuTime& time)
+void CassettePlayer::unplugHelper(EmuTime::param time)
 {
-	setState(STOP, time);
+	setState(STOP, getImageName(), time); // keep current image
 }
 
 
@@ -488,8 +542,11 @@ void CassettePlayer::generateChannels(int** buffers, unsigned num)
 		buffers[0] = 0;
 		return;
 	}
-	playImage->fillBuffer(playPos, buffers, num);
-	playPos += num;
+	// Note: fillBuffer() replaces the values in the buffer. It should add
+	//       to the existing values in the buffer. But because there is only
+	//       one channel this doesn't matter (buffer contains all zeros).
+	playImage->fillBuffer(audioPos, buffers, num);
+	audioPos += num;
 }
 
 bool CassettePlayer::generateInput(int* buffer, unsigned num)
@@ -498,7 +555,7 @@ bool CassettePlayer::generateInput(int* buffer, unsigned num)
 }
 
 bool CassettePlayer::updateBuffer(unsigned length, int* buffer,
-     const EmuTime& /*time*/, const EmuDuration& /*sampDur*/)
+     EmuTime::param /*time*/, EmuDuration::param /*sampDur*/)
 {
 	return generateOutput(buffer, length);
 }
@@ -510,8 +567,7 @@ bool CassettePlayer::signalEvent(shared_ptr<const Event> event)
 		if (!getImageName().empty()) {
 			// Reinsert tape to make sure everything is reset.
 			try {
-				playTape(getImageName(),
-				         getScheduler().getCurrentTime());
+				playTape(getImageName(), getCurrentTime());
 			} catch (MSXException& e) {
 				cliComm.printWarning(
 					"Failed to insert tape: " + e.getMessage());
@@ -527,7 +583,7 @@ const std::string& CassettePlayer::schedName() const
 	return schedName;
 }
 
-void CassettePlayer::executeUntil(const EmuTime& time, int userData)
+void CassettePlayer::executeUntil(EmuTime::param time, int userData)
 {
 	switch (userData) {
 	case END_OF_TAPE:
@@ -537,15 +593,15 @@ void CassettePlayer::executeUntil(const EmuTime& time, int userData)
 			"You may need to insert another tape image "
 			"that contains side B. (Or you used the wrong "
 			"loading command.)");
-		setState(STOP, time);
+		setState(STOP, getImageName(), time); // keep current image
 		break;
 	case SYNC_AUDIO_EMU:
 		if (getState() == PLAY) {
 			updateStream(time);
-			updatePlayPosition(time);
+			sync(time);
 			DynamicClock clk(EmuTime::zero);
 			clk.setFreq(playImage->getFrequency());
-			playPos = clk.getTicksTill(tapeTime);
+			audioPos = clk.getTicksTill(tapePos);
 		}
 		syncScheduled = false;
 		break;
@@ -555,29 +611,28 @@ void CassettePlayer::executeUntil(const EmuTime& time, int userData)
 
 // class TapeCommand
 
-TapeCommand::TapeCommand(MSXCommandController& msxCommandController,
+TapeCommand::TapeCommand(CommandController& commandController,
                          MSXEventDistributor& msxEventDistributor,
                          Scheduler& scheduler,
                          CassettePlayer& cassettePlayer_)
-	: RecordedCommand(msxCommandController, msxEventDistributor,
+	: RecordedCommand(commandController, msxEventDistributor,
 	                  scheduler, "cassetteplayer")
 	, cassettePlayer(cassettePlayer_)
 {
 }
 
-string TapeCommand::execute(const vector<string>& tokens, const EmuTime& time)
+string TapeCommand::execute(const vector<string>& tokens, EmuTime::param time)
 {
 	string result;
 	if (tokens.size() == 1) {
-		// Returning TCL lists here, similar to the disk commands in
+		// Returning Tcl lists here, similar to the disk commands in
 		// DiskChanger
 		TclObject tmp(getCommandController().getInterpreter());
 		tmp.addListElement(getName() + ':');
-		tmp.addListElement(cassettePlayer.getImageName());
+		tmp.addListElement(cassettePlayer.getImageName().getResolved());
 
 		TclObject options(getCommandController().getInterpreter());
-		options.addListElement(cassettePlayer.getStateString(
-		                           cassettePlayer.getState()));
+		options.addListElement(cassettePlayer.getStateString());
 		tmp.addListElement(options);
 		result += tmp.getString();
 
@@ -586,15 +641,15 @@ string TapeCommand::execute(const vector<string>& tokens, const EmuTime& time)
 		                ? tokens[2]
 		                : FileOperations::getNextNumberedFileName(
 		                         "taperecordings", "openmsx", ".wav");
-		cassettePlayer.recordTape(filename, time);
+		cassettePlayer.recordTape(Filename(filename), time);
 		result += "Created new cassette image file: " + filename
 		       + ", inserted it and set recording mode.";
 
 	} else if (tokens[1] == "insert" && tokens.size() == 3) {
 		try {
 			result += "Changing tape";
-			UserFileContext context(getCommandController());
-			cassettePlayer.playTape(context.resolve(tokens[2]), time);
+			Filename filename(tokens[2], getCommandController());
+			cassettePlayer.playTape(filename, time);
 		} catch (MSXException& e) {
 			throw CommandException(e.getMessage());
 		}
@@ -629,8 +684,10 @@ string TapeCommand::execute(const vector<string>& tokens, const EmuTime& time)
 			} catch (MSXException& e) {
 				throw CommandException(e.getMessage());
 			}
+		} else if (cassettePlayer.getState() == CassettePlayer::STOP) {
+			throw CommandException("No tape inserted or tape at end!");
 		} else {
-			// both PLAY and STOP
+			// PLAY mode
 			result += "Already in play mode.";
 		}
 
@@ -651,11 +708,17 @@ string TapeCommand::execute(const vector<string>& tokens, const EmuTime& time)
 		cassettePlayer.rewind(time);
 		result += "Tape rewound";
 
+	} else if (tokens[1] == "getpos") {
+		result += StringOp::toString(cassettePlayer.getTapePos(time));
+
+	} else if (tokens[1] == "getlength") {
+		result += StringOp::toString(cassettePlayer.getTapeLength(time));
+
 	} else {
 		try {
 			result += "Changing tape";
-			UserFileContext context(getCommandController());
-			cassettePlayer.playTape(context.resolve(tokens[1]), time);
+			Filename filename(tokens[1], getCommandController());
+			cassettePlayer.playTape(filename, time);
 		} catch (MSXException& e) {
 			throw CommandException(e.getMessage());
 		}
@@ -710,6 +773,13 @@ string TapeCommand::help(const vector<string>& tokens) const
 			    "used to be able to resume recording to an "
 			    "existing cassette image, previously inserted with "
 			    "the insert command.";
+		} else if (tokens[1] == "getpos") {
+			helptext =
+			    "Return the position of the tape, in seconds from "
+			    "the beginning of the tape.";
+		} else if (tokens[1] == "getlength") {
+			helptext =
+			    "Return the length of the tape in seconds.";
 		}
 	} else {
 		helptext =
@@ -727,6 +797,10 @@ string TapeCommand::help(const vector<string>& tokens) const
 		    ": create and insert new tape image file and go to record mode\n"
 		    "cassetteplayer insert <filename> "
 		    ": insert (a different) tape file\n"
+		    "cassetteplayer getpos            "
+		    ": query the position of the tape\n"
+		    "cassetteplayer getlength         "
+		    ": query the total length of the tape\n"
 		    "cassetteplayer <filename>        "
 		    ": insert (a different) tape file\n";
 	}
@@ -744,11 +818,13 @@ void TapeCommand::tabCompletion(vector<string>& tokens) const
 		extra.insert("new");
 		extra.insert("play");
 	//	extra.insert("record");
-		UserFileContext context(getCommandController());
-		completeFileName(tokens, context, extra);
+		extra.insert("getpos");
+		extra.insert("getlength");
+		UserFileContext context;
+		completeFileName(getCommandController(), tokens, context, extra);
 	} else if ((tokens.size() == 3) && (tokens[1] == "insert")) {
-		UserFileContext context(getCommandController());
-		completeFileName(tokens, context);
+		UserFileContext context;
+		completeFileName(getCommandController(), tokens, context);
 	} else if ((tokens.size() == 3) && (tokens[1] == "motorcontrol")) {
 		set<string> extra;
 		extra.insert("on");
@@ -757,4 +833,53 @@ void TapeCommand::tabCompletion(vector<string>& tokens) const
 	}
 }
 
+
+static enum_string<CassettePlayer::State> stateInfo[] = {
+	{ "PLAY",   CassettePlayer::PLAY   },
+	{ "RECORD", CassettePlayer::RECORD },
+	{ "STOP",   CassettePlayer::STOP   }
+};
+SERIALIZE_ENUM(CassettePlayer::State, stateInfo);
+
+template<typename Archive>
+void CassettePlayer::serialize(Archive& ar, unsigned /*version*/)
+{
+	if (recordImage.get()) {
+		// buf, sampcnt
+		flushOutput();
+	}
+	if (state == RECORD) {
+		// TODO we don't support savestates in RECORD mode yet
+		setState(STOP, getImageName(), getCurrentTime());
+	}
+
+	ar.serialize("casImage", casImage);
+	if (ar.isLoader()) {
+		removeTape(getCurrentTime());
+		casImage.updateAfterLoadState(commandController);
+		insertTape(casImage);
+	}
+
+	// only for RECORD
+	//double lastX;
+	//double lastY;
+	//double partialOut;
+	//double partialInterval;
+	//std::auto_ptr<WavWriter> recordImage;
+
+	ar.serialize("tapePos", tapePos);
+	ar.serialize("prevSyncTime", prevSyncTime);
+	ar.serialize("audioPos", audioPos);
+	ar.serialize("state", state);
+	ar.serialize("lastOutput", lastOutput);
+	ar.serialize("motor", motor);
+	ar.serialize("motorControl", motorControl);
+
+	if (ar.isLoader()) {
+		updateLoadingState(getCurrentTime());
+	}
+}
+INSTANTIATE_SERIALIZE_METHODS(CassettePlayer);
+REGISTER_POLYMORPHIC_INITIALIZER(Pluggable, CassettePlayer, "CassettePlayer");
+
 } // namespace openmsx
