@@ -1,4 +1,4 @@
-// $Id: Completer.cc 6463 2007-05-12 08:27:34Z m9710797 $
+// $Id: Completer.cc 8351 2008-11-12 18:20:29Z m9710797 $
 
 #include "Completer.hh"
 #include "InterpreterOutput.hh"
@@ -72,7 +72,7 @@ static vector<string> format(const set<string>& input, unsigned columnLimit)
 	return result;
 }
 
-bool Completer::completeString2(string &str, set<string>& st,
+bool Completer::completeString2(string& str, set<string>& st,
                                 bool caseSensitive)
 {
 	set<string>::iterator it = st.begin();
@@ -99,14 +99,14 @@ bool Completer::completeString2(string &str, set<string>& st,
 		it = st.begin();
 		if (equal(str, *it, caseSensitive)) {
 			// match is as long as first word
-			goto out;	// TODO rewrite this
+			goto out; // TODO rewrite this
 		}
 		// expand with one char and check all strings
 		string string2 = (*it).substr(0, str.size() + 1);
 		for (;  it != st.end(); it++) {
 			if (!equal(string2, (*it).substr(0, string2.size()),
 				   caseSensitive)) {
-				goto out;	// TODO rewrite this
+				goto out; // TODO rewrite this
 			}
 		}
 		// no conflict found
@@ -133,18 +133,20 @@ void Completer::completeString(vector<string>& tokens, set<string>& st,
 	}
 }
 
-void Completer::completeFileName(vector<string>& tokens,
+void Completer::completeFileName(CommandController& controller,
+                                 vector<string>& tokens,
                                  const FileContext& context)
 {
 	set<string> empty;
-	completeFileName(tokens, context, empty);
+	completeFileName(controller, tokens, context, empty);
 }
 
-void Completer::completeFileName(vector<string>& tokens,
+void Completer::completeFileName(CommandController& controller,
+                                 vector<string>& tokens,
                                  const FileContext& context,
                                  const set<string>& extra)
 {
-	vector<string> paths(context.getPaths());
+	vector<string> paths(context.getPaths(controller));
 
 	string& filename = tokens.back();
 	filename = FileOperations::expandTilde(filename);
@@ -158,12 +160,12 @@ void Completer::completeFileName(vector<string>& tokens,
 	for (vector<string>::const_iterator it = paths.begin();
 	     it != paths.end();
 	     ++it) {
-		string dirname = *it + basename;
+		string dirname = FileOperations::join(*it, basename);
 		ReadDir dir(FileOperations::getNativePath(dirname));
 		while (dirent* de = dir.getEntry()) {
-			string name = dirname + de->d_name;
+			string name = FileOperations::join(dirname, de->d_name);
 			if (FileOperations::exists(name)) {
-				string nm = basename + de->d_name;
+				string nm = FileOperations::join(basename, de->d_name);
 				if (FileOperations::isDirectory(name)) {
 					nm += '/';
 				}
