@@ -1,4 +1,4 @@
-// $Id: GlobalCommandController.cc 6824 2007-08-08 15:20:19Z m9710797 $
+// $Id: GlobalCommandController.cc 8377 2008-11-30 20:56:25Z manuelbi $
 
 #include "GlobalCommandController.hh"
 #include "Command.hh"
@@ -6,7 +6,7 @@
 #include "ProxyCommand.hh"
 #include "ProxySetting.hh"
 #include "InfoTopic.hh"
-#include "File.hh"
+#include "LocalFileReference.hh"
 #include "openmsx.hh"
 #include "CliComm.hh"
 #include "HotKey.hh"
@@ -20,6 +20,7 @@
 #include "TclObject.hh"
 #include "Version.hh"
 #include "ScopedAssign.hh"
+#include "openmsx.hh"
 #include <cassert>
 #include <cstdlib>
 
@@ -82,13 +83,21 @@ GlobalCommandController::~GlobalCommandController()
 	// all this reset() stuff is also done automatically by the destructor,
 	// but we need it slightly earlier to test the assertions.
 	// TODO find a cleaner way to do this
+	PRT_DEBUG("~GlobalCommandController: reset romInfoTopic");
 	romInfoTopic.reset();
+	PRT_DEBUG("~GlobalCommandController: reset versionInfo");
 	versionInfo.reset();
+	PRT_DEBUG("~GlobalCommandController: reset tabCompletionCmd");
 	tabCompletionCmd.reset();
+	PRT_DEBUG("~GlobalCommandController: reset helpCmd");
 	helpCmd.reset();
+	PRT_DEBUG("~GlobalCommandController: reset globalSettings");
 	globalSettings.reset();
+	PRT_DEBUG("~GlobalCommandController: reset settingsConfig");
 	settingsConfig.reset();
+	PRT_DEBUG("~GlobalCommandController: reset hotkey");
 	hotKey.reset();
+	PRT_DEBUG("~GlobalCommandController: reset openMSXinfocmd");
 	openMSXInfoCommand.reset();
 
 	assert(commands.empty());
@@ -169,6 +178,11 @@ CliComm& GlobalCommandController::getCliComm()
 	return *cliComm;
 }
 
+CliComm* GlobalCommandController::getCliCommIfAvailable()
+{
+	return cliComm;
+}
+
 CliConnection* GlobalCommandController::getConnection() const
 {
 	return connection;
@@ -182,7 +196,7 @@ Interpreter& GlobalCommandController::getInterpreter()
 	return *interpreter;
 }
 
-Reactor& GlobalCommandController::getReactor()
+Reactor& GlobalCommandController::getReactor() const
 {
 	return reactor;
 }
@@ -265,16 +279,27 @@ void GlobalCommandController::unregisterSetting(Setting& setting)
 	getSettingsConfig().getSettingsManager().unregisterSetting(setting, name);
 }
 
+Setting* GlobalCommandController::findSetting(const std::string& name)
+{
+	return getSettingsConfig().getSettingsManager().findSetting(name);
+}
+
+void GlobalCommandController::changeSetting(
+	const std::string& name, const string& value)
+{
+	getInterpreter().setVariable(name, value);
+}
+
+void GlobalCommandController::changeSetting(Setting& setting, const string& value)
+{
+	changeSetting(setting.getName(), value);
+}
+
 string GlobalCommandController::makeUniqueSettingName(const string& name)
 {
 	return getSettingsConfig().getSettingsManager().makeUnique(name);
 }
 
-GlobalCommandController& GlobalCommandController::getGlobalCommandController()
-{
-	return *this;
-}
-
 bool GlobalCommandController::hasCommand(const string& command) const
 {
 	return commands.find(command) != commands.end();
@@ -439,8 +464,11 @@ void GlobalCommandController::splitList(
 void GlobalCommandController::source(const string& script)
 {
 	try {
-		File file(script);
-		getInterpreter().executeFile(file.getLocalName());
+		PRT_DEBUG("GlobalCommandController::source: " << script << ": getting local file reference...");
+		LocalFileReference file(script);
+		PRT_DEBUG("GlobalCommandController::source: " << script << ": executing file...");
+		getInterpreter().executeFile(file.getFilename());
+		PRT_DEBUG("GlobalCommandController::source: " << script << ": executing file... DONE!");
 	} catch (CommandException& e) {
 		getCliComm().printWarning(
 			 "While executing init.tcl: " + e.getMessage());
