@@ -1,14 +1,15 @@
-// $Id: MSXCommandController.cc 6127 2007-03-01 09:30:39Z m9710797 $
+// $Id: MSXCommandController.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "MSXCommandController.hh"
 #include "GlobalCommandController.hh"
+#include "MSXEventDistributor.hh"
 #include "MSXMotherBoard.hh"
 #include "SettingsConfig.hh"
 #include "SettingsManager.hh"
 #include "InfoCommand.hh"
 #include "Interpreter.hh"
 #include "Setting.hh"
-#include "MSXCliComm.hh"
+#include "Event.hh"
 #include <iostream>
 
 using std::string;
@@ -25,10 +26,14 @@ MSXCommandController::MSXCommandController(
 	getInterpreter().createNamespace(motherboard.getMachineID());
 
 	machineInfoCommand.reset(new InfoCommand(*this, "machine_info"));
+
+	motherboard.getMSXEventDistributor().registerEventListener(*this);
 }
 
 MSXCommandController::~MSXCommandController()
 {
+	motherboard.getMSXEventDistributor().unregisterEventListener(*this);
+
 	machineInfoCommand.reset();
 
 	#ifndef NDEBUG
@@ -47,16 +52,16 @@ MSXCommandController::~MSXCommandController()
 	getInterpreter().deleteNamespace(motherboard.getMachineID());
 }
 
-InfoCommand& MSXCommandController::getMachineInfoCommand()
-{
-	return *machineInfoCommand;
-}
-
 GlobalCommandController& MSXCommandController::getGlobalCommandController()
 {
 	return globalCommandController;
 }
 
+InfoCommand& MSXCommandController::getMachineInfoCommand()
+{
+	return *machineInfoCommand;
+}
+
 void MSXCommandController::registerCommand(Command& command, const string& str)
 {
 	assert(!hasCommand(str));
@@ -115,6 +120,12 @@ void MSXCommandController::unregisterSetting(Setting& setting)
 	getSettingsConfig().getSettingsManager().unregisterSetting(setting, fullname);
 }
 
+void MSXCommandController::changeSetting(Setting& setting, const string& value)
+{
+	string fullname = motherboard.getMachineID() + "::" + setting.getName();
+	globalCommandController.changeSetting(fullname, value);
+}
+
 string MSXCommandController::makeUniqueSettingName(const string& name)
 {
 	//return getSettingsConfig().getSettingsManager().makeUnique(name);
@@ -127,7 +138,7 @@ Command* MSXCommandController::findCommand(const std::string& name) const
 	return (it != commandMap.end()) ? it->second : NULL;
 }
 
-Setting* MSXCommandController::findSetting(const std::string& name) const
+Setting* MSXCommandController::findSetting(const std::string& name)
 {
 	SettingMap::const_iterator it = settingMap.find(name);
 	return (it != settingMap.end()) ? it->second : NULL;
@@ -144,6 +155,16 @@ string MSXCommandController::executeCommand(const string& command,
 	return globalCommandController.executeCommand(command, connection);
 }
 
+void MSXCommandController::tabCompletion(std::string& command)
+{
+	globalCommandController.tabCompletion(command);
+}
+
+bool MSXCommandController::isComplete(const std::string& command)
+{
+	return globalCommandController.isComplete(command);
+}
+
 void MSXCommandController::splitList(const string& list,
 	                             vector<string>& result)
 {
@@ -155,6 +176,11 @@ CliComm& MSXCommandController::getCliComm()
 	return motherboard.getMSXCliComm();
 }
 
+CliComm* MSXCommandController::getCliCommIfAvailable()
+{
+	return motherboard.getMSXCliCommIfAvailable();
+}
+
 GlobalSettings& MSXCommandController::getGlobalSettings()
 {
 	return globalCommandController.getGlobalSettings();
@@ -175,4 +201,21 @@ CliConnection* MSXCommandController::getConnection() const
 	return globalCommandController.getConnection();
 }
 
+Reactor& MSXCommandController::getReactor() const
+{
+	return globalCommandController.getReactor();
+}
+
+void MSXCommandController::signalEvent(
+	shared_ptr<const Event> event, EmuTime::param /*time*/)
+{
+	if (event->getType() != OPENMSX_MACHINE_ACTIVATED) return;
+
+	// simple way to synchronize proxy settings
+	for (SettingMap::const_iterator it = settingMap.begin();
+	     it != settingMap.end(); ++it) {
+		changeSetting(*it->second, it->second->getValueString());
+	}
+}
+
 } // namespace openmsx
