@@ -1,4 +1,4 @@
-// $Id: TclObject.cc 5676 2006-09-21 19:01:22Z m9710797 $
+// $Id: TclObject.cc 8157 2008-08-12 17:58:55Z m9710797 $
 
 #include "TclObject.hh"
 #include "Interpreter.hh"
@@ -23,6 +23,12 @@ TclObject::TclObject(Tcl_Interp* interp_, const string& value)
 	init(Tcl_NewStringObj(value.data(), value.size()));
 }
 
+TclObject::TclObject(const string& value)
+	: interp(0)
+{
+	init(Tcl_NewStringObj(value.data(), value.size()));
+}
+
 TclObject::TclObject(Tcl_Interp* interp_)
 	: interp(interp_)
 {
@@ -61,6 +67,18 @@ TclObject::~TclObject()
 	}
 }
 
+TclObject& TclObject::operator=(const TclObject& other)
+{
+	if (&other != this) {
+		if (owned) {
+			Tcl_DecrRefCount(obj);
+		}
+		interp = other.interp;
+		init(other.obj);
+	}
+	return *this;
+}
+
 Tcl_Interp* TclObject::getInterpreter() const
 {
 	return interp;
@@ -101,6 +119,12 @@ void TclObject::setInt(int value)
 	Tcl_SetIntObj(obj, value);
 }
 
+void TclObject::setBoolean(bool value)
+{
+	unshare();
+	Tcl_SetBooleanObj(obj, value);
+}
+
 void TclObject::setDouble(double value)
 {
 	unshare();
@@ -150,6 +174,15 @@ int TclObject::getInt() const
 	return result;
 }
 
+bool TclObject::getBoolean() const
+{
+	int result;
+	if (Tcl_GetBooleanFromObj(interp, obj, &result) != TCL_OK) {
+		throwException();
+	}
+	return result;
+}
+
 double TclObject::getDouble() const
 {
 	double result;
