@@ -1,8 +1,10 @@
-// $Id: SettingsConfig.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: SettingsConfig.cc 8377 2008-11-30 20:56:25Z manuelbi $
 
 #include "SettingsConfig.hh"
 #include "SettingsManager.hh"
+#include "XMLElement.hh"
 #include "XMLLoader.hh"
+#include "LocalFileReference.hh"
 #include "File.hh"
 #include "FileContext.hh"
 #include "FileException.hh"
@@ -11,7 +13,9 @@
 #include "CommandException.hh"
 #include "GlobalCommandController.hh"
 #include "Command.hh"
+#include "openmsx.hh"
 #include <memory>
+#include <cassert>
 
 using std::auto_ptr;
 using std::string;
@@ -44,24 +48,27 @@ private:
 };
 
 
-SettingsConfig::SettingsConfig(GlobalCommandController& commandController_,
-                               HotKey& hotKey_)
-	: XMLElement("settings")
-	, commandController(commandController_)
+SettingsConfig::SettingsConfig(
+		GlobalCommandController& globalCommandController,
+		HotKey& hotKey_)
+	: commandController(globalCommandController)
 	, saveSettingsCommand(new SaveSettingsCommand(commandController, *this))
 	, loadSettingsCommand(new LoadSettingsCommand(commandController, *this))
+	, settingsManager(new SettingsManager(globalCommandController))
+	, xmlElement(new XMLElement("settings"))
 	, hotKey(hotKey_)
 	, mustSaveSettings(false)
 {
-	setFileContext(auto_ptr<FileContext>(new SystemFileContext()));
-	settingsManager.reset(new SettingsManager(commandController));
 }
 
 SettingsConfig::~SettingsConfig()
 {
+	PRT_DEBUG("~SettingsConfig...");
 	if (mustSaveSettings) {
 		try {
+			PRT_DEBUG("Saving Settings...");
 			saveSetting();
+			PRT_DEBUG("Saving Settings... DONE");
 		} catch (FileException& e) {
 			commandController.getCliComm().printWarning(
 				"Auto-saving of settings failed: " + e.getMessage() );
@@ -71,19 +78,20 @@ SettingsConfig::~SettingsConfig()
 
 void SettingsConfig::loadSetting(FileContext& context, const string& filename)
 {
-	try {
-		saveName = context.resolveCreate(filename);
-		File file(context.resolve(filename));
-		auto_ptr<XMLElement> doc(XMLLoader::loadXML(
-			file.getLocalName(), "settings.dtd"));
-		XMLElement::operator=(*doc);
-		getSettingsManager().loadSettings(*this);
-		hotKey.loadBindings(*this);
-	} catch (XMLException& e) {
-		commandController.getCliComm().printWarning(
-			"Loading of settings failed: " + e.getMessage() + "\n"
-			"Reverting to default settings.");
-	}
+	LocalFileReference file(context.resolve(commandController, filename));
+	xmlElement = XMLLoader::load(file.getFilename(), "settings.dtd");
+	xmlElement->setFileContext(
+		auto_ptr<FileContext>(new SystemFileContext()));
+	getSettingsManager().loadSettings(*xmlElement);
+	hotKey.loadBindings(*xmlElement);
+
+	// only set saveName after file was successfully parsed
+	setSaveFilename(context, filename);
+}
+
+void SettingsConfig::setSaveFilename(FileContext& context, const string& filename)
+{
+	saveName = context.resolveCreate(filename);
 }
 
 void SettingsConfig::saveSetting(const string& filename)
@@ -91,11 +99,12 @@ void SettingsConfig::saveSetting(const string& filename)
 	const string& name = filename.empty() ? saveName : filename;
 	if (name.empty()) return;
 
-	getSettingsManager().saveSettings(*this);
-	hotKey.saveBindings(*this);
+	getSettingsManager().saveSettings(*xmlElement);
+	hotKey.saveBindings(*xmlElement);
 
 	File file(name, File::TRUNCATE);
-	string data = "<!DOCTYPE settings SYSTEM 'settings.dtd'>\n" + dump();
+	string data = "<!DOCTYPE settings SYSTEM 'settings.dtd'>\n" +
+	              xmlElement->dump();
 	file.write(data.c_str(), data.size());
 }
 
@@ -109,6 +118,12 @@ SettingsManager& SettingsConfig::getSettingsManager()
 	return *settingsManager; // ***
 }
 
+XMLElement& SettingsConfig::getXMLElement()
+{
+	assert(xmlElement.get());
+	return *xmlElement;
+}
+
 
 // class SaveSettingsCommand
 
@@ -124,14 +139,14 @@ string SaveSettingsCommand::execute(const vector<string>& tokens)
 {
 	try {
 		switch (tokens.size()) {
-			case 1:
-				settingsConfig.saveSetting();
-				break;
-			case 2:
-				settingsConfig.saveSetting(tokens[1]);
-				break;
-			default:
-				throw SyntaxError();
+		case 1:
+			settingsConfig.saveSetting();
+			break;
+		case 2:
+			settingsConfig.saveSetting(tokens[1]);
+			break;
+		default:
+			throw SyntaxError();
 		}
 	} catch (FileException& e) {
 		throw CommandException(e.getMessage());
@@ -148,7 +163,7 @@ void SaveSettingsCommand::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
 		SystemFileContext context;
-		completeFileName(tokens, context);
+		completeFileName(getCommandController(), tokens, context);
 	}
 }
 
@@ -182,7 +197,7 @@ void LoadSettingsCommand::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
 		SystemFileContext context;
-		completeFileName(tokens, context);
+		completeFileName(getCommandController(), tokens, context);
 	}
 }
 
