@@ -1,16 +1,16 @@
-// $Id: CommandConsole.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: CommandConsole.cc 8093 2008-07-26 12:12:54Z manuelbi $
 
 #include "CommandConsole.hh"
 #include "CommandException.hh"
-#include "GlobalCommandController.hh"
+#include "CommandController.hh"
 #include "Completer.hh"
 #include "Interpreter.hh"
 #include "Keys.hh"
 #include "FileContext.hh"
-#include "FileOperations.hh"
 #include "FileException.hh"
 #include "CliComm.hh"
 #include "SettingsConfig.hh"
+#include "XMLElement.hh"
 #include "InputEvents.hh"
 #include "Display.hh"
 #include "EventDistributor.hh"
@@ -24,7 +24,6 @@
 
 using std::min;
 using std::max;
-using std::list;
 using std::string;
 
 namespace openmsx {
@@ -35,7 +34,7 @@ const char* const PROMPT1 = "> ";
 const char* const PROMPT2 = "| ";
 
 CommandConsole::CommandConsole(
-		GlobalCommandController& commandController_,
+		CommandController& commandController_,
 		EventDistributor& eventDistributor_,
 		Display& display_)
 	: commandController(commandController_)
@@ -50,7 +49,7 @@ CommandConsole::CommandConsole(
 	maxHistory = 100;
 	removeDoubles = true;
 	if (const XMLElement* config = commandController.getSettingsConfig().
-	                                                 findChild("Console")) {
+		                        getXMLElement().findChild("Console")) {
 		maxHistory = config->getChildDataAsInt(
 			"historysize", maxHistory);
 		removeDoubles = config->getChildDataAsBool(
@@ -80,20 +79,19 @@ CommandConsole::~CommandConsole()
 	eventDistributor.unregisterEventListener(OPENMSX_KEY_DOWN_EVENT, *this);
 	commandController.getInterpreter().setOutput(NULL);
 	Completer::setOutput(NULL);
-	saveHistory();
 }
 
 void CommandConsole::saveHistory()
 {
 	try {
-		UserFileContext context(commandController, "console");
-		std::ofstream outputfile(FileOperations::expandTilde(
-		        context.resolveCreate("history.txt")).c_str());
+		UserFileContext context("console");
+		std::ofstream outputfile(
+		        context.resolveCreate("history.txt").c_str());
 		if (!outputfile) {
 			throw FileException(
 				"Error while saving the console history.");
 		}
-		for (list<string>::iterator it = history.begin();
+		for (History::const_iterator it = history.begin();
 		     it != history.end(); ++it) {
 			outputfile << it->substr(prompt.length()) << std::endl;
 		}
@@ -105,9 +103,9 @@ void CommandConsole::saveHistory()
 void CommandConsole::loadHistory()
 {
 	try {
-		UserFileContext context(commandController, "console");
-		std::ifstream inputfile(FileOperations::expandTilde(
-		        context.resolveCreate("history.txt")).c_str());
+		UserFileContext context("console");
+		std::ifstream inputfile(
+		        context.resolveCreate("history.txt").c_str());
 		if (!inputfile) {
 			throw FileException(
 				"Error while loading the console history.");
@@ -277,6 +275,7 @@ void CommandConsole::commandExecute()
 {
 	resetScrollBack();
 	putCommandHistory(lines[0]);
+	saveHistory(); // save at this point already, so that we don't lose history in case of a crash
 
 	commandBuffer += lines[0].substr(prompt.length()) + '\n';
 	newLineConsole(lines[0]);
@@ -329,7 +328,7 @@ void CommandConsole::prevCommand()
 		return; // no elements
 	}
 	bool match = false;
-	list<string>::iterator tempScrollBack = commandScrollBack;
+	History::const_iterator tempScrollBack = commandScrollBack;
 	while ((tempScrollBack != history.begin()) && !match) {
 		tempScrollBack--;
 		match = ((tempScrollBack->length() >= currentLine.length()) &&
@@ -349,7 +348,7 @@ void CommandConsole::nextCommand()
 		return; // don't loop !
 	}
 	bool match = false;
-	list<string>::iterator tempScrollBack = commandScrollBack;
+	History::const_iterator tempScrollBack = commandScrollBack;
 	while ((++tempScrollBack != history.end()) && !match) {
 		match = ((tempScrollBack->length() >= currentLine.length()) &&
 		         (tempScrollBack->substr(0, currentLine.length()) == currentLine));
