@@ -1,4 +1,4 @@
-// $Id: SDLFont.cc 6914 2007-09-08 21:22:05Z m9710797 $
+// $Id: SDLFont.cc 7904 2008-06-17 16:51:18Z m9710797 $
 
 /*  Written By: Garrett Banuk <mongoose@mongeese.org>
  *  This is free, just be sure to give me credit when using it
@@ -6,13 +6,13 @@
  */
 
 #include "SDLFont.hh"
-#include "openmsx.hh"
+#include "OutputSurface.hh"
+#include "LocalFileReference.hh"
 #include "MSXException.hh"
-#include "File.hh"
+#include "openmsx.hh"
 #include <SDL.h>
 #include <SDL_image.h>
 
-
 namespace openmsx {
 
 const int NUM_CHRS = 256;
@@ -20,15 +20,15 @@ const int CHARS_PER_ROW = 16;
 const int CHARS_PER_COL = NUM_CHRS / CHARS_PER_ROW;
 
 
-SDLFont::SDLFont(const std::string& filePath, SDL_Surface* surface)
-	: outputScreen(surface)
+SDLFont::SDLFont(const std::string& filename, OutputSurface& output_)
+	: output(output_)
 {
 	// load the font bitmap
+	LocalFileReference file(filename);
 	SDL_Surface* image1;
-	if (!(image1 = IMG_Load(filePath.c_str()))) {
+	if (!(image1 = IMG_Load(file.getFilename().c_str()))) {
 		throw MSXException(
-			"Can't load font " + filePath + ": " + IMG_GetError()
-			);
+			"Can't load font " + filename + ": " + IMG_GetError());
 	}
 	fontSurface = SDL_DisplayFormatAlpha(image1);
 	SDL_FreeSurface(image1);
@@ -42,7 +42,7 @@ SDLFont::SDLFont(const std::string& filePath, SDL_Surface* surface)
 		format->Rmask, format->Gmask, format->Bmask, 0);
 	if (!workImage) {
 		SDL_FreeSurface(fontSurface);
-		throw MSXException("Can't create surface for font " + filePath);
+		throw MSXException("Can't create surface for font " + filename);
 	}
 }
 
@@ -55,12 +55,12 @@ SDLFont::~SDLFont()
 void SDLFont::drawText(const std::string& str, int x, int y, byte alpha)
 {
 	// see how many characters can fit on the screen
-	if ((outputScreen->w <= x) || (outputScreen->h <= y)) {
+	if ((int(output.getWidth()) <= x) || (int(output.getHeight()) <= y)) {
 		return;
 	}
 	unsigned characters = str.length();
-	if (characters > ((outputScreen->w - x) / charWidth)) {
-		characters = (outputScreen->w - x) / charWidth;
+	if (characters > ((output.getWidth() - x) / charWidth)) {
+		characters = (output.getWidth() - x) / charWidth;
 	}
 	SDL_Rect destRect;
 	destRect.x = x;
@@ -73,25 +73,25 @@ void SDLFont::drawText(const std::string& str, int x, int y, byte alpha)
 	sourceRect.h = charHeight;
 
 	// Now draw it
-	if (SDL_MUSTLOCK(outputScreen)) SDL_UnlockSurface(outputScreen);
+	output.unlock();
 	for (unsigned loop = 0; loop < characters; loop++) {
 		sourceRect.x = (str[loop] % CHARS_PER_ROW) * charWidth;
 		sourceRect.y = (str[loop] / CHARS_PER_ROW) * charHeight;
+		SDL_Surface* outputSurface = output.getSDLWorkSurface();
 		if (alpha == 255) {
 			SDL_BlitSurface(fontSurface, &sourceRect,
-			                outputScreen, &destRect);
+			                outputSurface, &destRect);
 		} else {
-			SDL_BlitSurface(outputScreen, &destRect,
+			SDL_BlitSurface(outputSurface, &destRect,
 			                workImage, NULL);
 			SDL_BlitSurface(fontSurface, &sourceRect,
 			                workImage, NULL);
 			SDL_SetAlpha(workImage, SDL_SRCALPHA, alpha);
 			SDL_BlitSurface(workImage, NULL,
-			                outputScreen, &destRect);
+			                outputSurface, &destRect);
 		}
 		destRect.x += charWidth;
 	}
-	if (SDL_MUSTLOCK(outputScreen)) SDL_LockSurface(outputScreen);
 }
 
 } // namespace openmsx
