@@ -1,21 +1,23 @@
-// $Id: CPU.cc 5965 2006-12-27 12:16:54Z m9710797 $
+// $Id: CPU.cc 8341 2008-11-05 19:41:15Z m9710797 $
 
 #include "CPU.hh"
 #include "BreakPoint.hh"
+#include "serialize.hh"
+#include <cassert>
 
 namespace openmsx {
 
 byte CPU::ZSTable[256];
 byte CPU::ZSXYTable[256];
-byte CPU::ZSPXYTable[256];
 byte CPU::ZSPTable[256];
+byte CPU::ZSPXYTable[256];
+byte CPU::ZSPHTable[256];
 word CPU::DAATable[0x800];
 
 CPU::BreakPoints CPU::breakPoints;
 bool CPU::breaked = false;
 bool CPU::continued = false;
 bool CPU::step = false;
-bool CPU::paused = false;
 
 word CPU::start_pc;
 
@@ -36,15 +38,22 @@ CPU::CPU()
 		for (int v = 128; v != 0; v >>= 1) {
 			if (i & v) vFlag ^= V_FLAG;
 		}
-		ZSTable[i]    = zFlag | sFlag;
-		ZSXYTable[i]  = zFlag | sFlag | xFlag | yFlag;
+		ZSTable   [i] = zFlag | sFlag;
+		ZSXYTable [i] = zFlag | sFlag | xFlag | yFlag;
+		ZSPTable  [i] = zFlag | sFlag |                 vFlag;
 		ZSPXYTable[i] = zFlag | sFlag | xFlag | yFlag | vFlag;
-		ZSPTable[i]   = zFlag | sFlag |                 vFlag;
+		ZSPHTable [i] = zFlag | sFlag |                 vFlag | H_FLAG;
 	}
+	assert(ZSTable   [  0] == ZS0);
+	assert(ZSXYTable [  0] == ZSXY0);
+	assert(ZSPTable  [  0] == ZSP0);
+	assert(ZSPXYTable[  0] == ZSPXY0);
+	assert(ZSTable   [255] == ZS255);
+	assert(ZSXYTable [255] == ZSXY255);
 
 	for (int x = 0; x < 0x800; ++x) {
-		bool nf = x & 0x400;
-		bool hf = x & 0x200;
+		bool hf = x & 0x400;
+		bool nf = x & 0x200;
 		bool cf = x & 0x100;
 		byte a = x & 0xFF;
 		byte hi = a / 16;
@@ -117,9 +126,9 @@ const CPU::BreakPoints& CPU::getBreakPoints() const
 	return breakPoints;
 }
 
-void CPU::setPaused(bool paused_)
+void CPU::setPaused(bool paused)
 {
-	paused = paused_;
+	getRegisters().setExtHALT(paused);
 	exitCPULoopSync();
 }
 
@@ -132,5 +141,32 @@ void CPU::checkBreakPoints(std::pair<BreakPoints::const_iterator,
 	}
 }
 
+template<typename Archive>
+void CPU::CPURegs::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.serialize("af",  AF_.w);
+	ar.serialize("bc",  BC_.w);
+	ar.serialize("de",  DE_.w);
+	ar.serialize("hl",  HL_.w);
+	ar.serialize("af2", AF2_.w);
+	ar.serialize("bc2", BC2_.w);
+	ar.serialize("de2", DE2_.w);
+	ar.serialize("hl2", HL2_.w);
+	ar.serialize("ix",  IX_.w);
+	ar.serialize("iy",  IY_.w);
+	ar.serialize("pc",  PC_.w);
+	ar.serialize("sp",  SP_.w);
+	ar.serialize("i",   I_);
+	byte r = getR();
+	ar.serialize("r",   r);  // combined R_ and R2_
+	setR(r);
+	ar.serialize("im",  IM_);
+	ar.serialize("iff1", IFF1_);
+	ar.serialize("iff2", IFF2_);
+	ar.serialize("afterEI", afterEI_);
+	ar.serialize("halt", HALT_);
+}
+INSTANTIATE_SERIALIZE_METHODS(CPU::CPURegs);
+
 } // namespace openmsx
 
