@@ -1,4 +1,4 @@
-// $Id: Dasm.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: Dasm.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "Dasm.hh"
 #include "DasmTables.hh"
@@ -17,11 +17,11 @@ static int abs(unsigned char a)
 	return (a & 128) ? (256 - a) : a;
 }
 
-int dasm(const MSXCPUInterface& interf, word pc, byte buf[4], std::string& dest,
-         const EmuTime& time)
+unsigned dasm(const MSXCPUInterface& interf, word pc, byte buf[4],
+              std::string& dest, EmuTime::param time)
 {
 	const char* s;
-	int i = 0;
+	unsigned i = 0;
 	const char* r = 0;
 
 	buf[0] = interf.peekMem(pc, time);
