@@ -1,16 +1,18 @@
-// $Id: MSXCPU.cc 6602 2007-06-06 20:27:31Z m9710797 $
+// $Id: MSXCPU.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "MSXCPU.hh"
 #include "MSXMotherBoard.hh"
 #include "Debugger.hh"
+#include "Scheduler.hh"
 #include "SimpleDebuggable.hh"
-#include "MSXCommandController.hh"
 #include "BooleanSetting.hh"
 #include "CPUCore.hh"
 #include "Z80.hh"
 #include "R800.hh"
 #include "BreakPoint.hh"
 #include "InfoTopic.hh"
+#include "TclObject.hh"
+#include "serialize.hh"
 #include <cassert>
 
 using std::string;
@@ -48,36 +50,44 @@ MSXCPU::MSXCPU(MSXMotherBoard& motherboard_)
 	                              Setting::DONT_SAVE))
 	, z80 (new CPUCore<Z80TYPE> (motherboard, "z80",  *traceSetting,
 	                             EmuTime::zero))
-	, r800(new CPUCore<R800TYPE>(motherboard, "r800", *traceSetting,
-	                             EmuTime::zero))
+	, r800(motherboard.isTurboR()
+	       ? new CPUCore<R800TYPE>(motherboard, "r800", *traceSetting,
+	                               EmuTime::zero)
+	       : NULL)
 	, reference(EmuTime::zero)
 	, timeInfo(new TimeInfoTopic(
-	     motherboard.getMSXCommandController().getMachineInfoCommand(), *this))
+		motherboard.getMachineInfoCommand(), *this))
 	, debuggable(new MSXCPUDebuggable(motherboard_, *this))
 {
-	activeCPU = z80.get();	// setActiveCPU(CPU_Z80);
+	activeCPU = z80.get(); // setActiveCPU(CPU_Z80);
 	newCPU = 0;
 
 	motherboard.getDebugger().setCPU(this);
+	motherboard.getScheduler().setCPU(this);
 	traceSetting->attach(*this);
 }
 
 MSXCPU::~MSXCPU()
 {
 	traceSetting->detach(*this);
+	motherboard.getScheduler().setCPU(0);
 	motherboard.getDebugger().setCPU(0);
 }
 
 void MSXCPU::setInterface(MSXCPUInterface* interface)
 {
 	z80 ->setInterface(interface);
-	r800->setInterface(interface);
+	if (r800.get()) {
+		r800->setInterface(interface);
+	}
 }
 
-void MSXCPU::doReset(const EmuTime& time)
+void MSXCPU::doReset(EmuTime::param time)
 {
 	z80 ->doReset(time);
-	r800->doReset(time);
+	if (r800.get()) {
+		r800->doReset(time);
+	}
 
 	reference = time;
 }
@@ -92,11 +102,12 @@ void MSXCPU::setActiveCPU(CPUType cpu)
 			break;
 		case CPU_R800:
 			PRT_DEBUG("Active CPU: R800");
+			assert(r800.get());
 			tmp = r800.get();
 			break;
 		default:
 			assert(false);
-			tmp = NULL;	// prevent warning
+			tmp = NULL; // prevent warning
 	}
 	if (tmp != activeCPU) {
 		exitCPULoopSync();
@@ -106,6 +117,7 @@ void MSXCPU::setActiveCPU(CPUType cpu)
 
 void MSXCPU::setDRAMmode(bool dram)
 {
+	assert(r800.get());
 	r800->setDRAMmode(dram);
 }
 
@@ -130,16 +142,23 @@ void MSXCPU::exitCPULoopAsync()
 }
 
 
-const EmuTime& MSXCPU::getCurrentTime() const
+EmuTime::param MSXCPU::getCurrentTime() const
 {
 	return activeCPU->getCurrentTime();
 }
 
+void MSXCPU::setNextSyncPoint(EmuTime::param time)
+{
+	activeCPU->setNextSyncPoint(time);
+}
+
 
 void MSXCPU::updateVisiblePage(byte page, byte primarySlot, byte secondarySlot)
 {
 	invalidateMemCache(page * 0x4000, 0x4000);
-	r800->updateVisiblePage(page, primarySlot, secondarySlot);
+	if (r800.get()) {
+		r800->updateVisiblePage(page, primarySlot, secondarySlot);
+	}
 }
 
 void MSXCPU::invalidateMemCache(word start, unsigned size)
@@ -150,22 +169,30 @@ void MSXCPU::invalidateMemCache(word start, unsigned size)
 void MSXCPU::raiseIRQ()
 {
 	z80 ->raiseIRQ();
-	r800->raiseIRQ();
+	if (r800.get()) {
+		r800->raiseIRQ();
+	}
 }
 void MSXCPU::lowerIRQ()
 {
 	z80 ->lowerIRQ();
-	r800->lowerIRQ();
+	if (r800.get()) {
+		r800->lowerIRQ();
+	}
 }
 void MSXCPU::raiseNMI()
 {
 	z80 ->raiseNMI();
-	r800->raiseNMI();
+	if (r800.get()) {
+		r800->raiseNMI();
+	}
 }
 void MSXCPU::lowerNMI()
 {
 	z80 ->lowerNMI();
-	r800->lowerNMI();
+	if (r800.get()) {
+		r800->lowerNMI();
+	}
 }
 
 bool MSXCPU::isR800Active()
@@ -178,11 +205,16 @@ void MSXCPU::setZ80Freq(unsigned freq)
 	z80->setFreq(freq);
 }
 
-void MSXCPU::wait(const EmuTime& time)
+void MSXCPU::wait(EmuTime::param time)
 {
 	activeCPU->wait(time);
 }
 
+void MSXCPU::waitCycles(unsigned cycles)
+{
+	activeCPU->waitCycles(cycles);
+}
+
 void MSXCPU::update(const Setting& setting)
 {
 	(void)setting;
@@ -346,12 +378,11 @@ void MSXCPUDebuggable::write(unsigned address, byte value)
 	case 23: regs.setSPl(value); break;
 	case 24: regs.setI(value); break;
 	case 25: regs.setR(value); break;
-	case 26: 
+	case 26:
 		if (value < 3) regs.setIM(value);
 		break;
 	case 27:
 		regs.setIFF1(value & 0x01);
-		regs.setNextIFF1(value & 0x01);
 		regs.setIFF2(value & 0x02);
 		break;
 	default:
@@ -360,4 +391,18 @@ void MSXCPUDebuggable::write(unsigned address, byte value)
 	}
 }
 
+
+template<typename Archive>
+void MSXCPU::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.serialize("z80",  *z80);
+	if (r800.get()) {
+		ar.serialize("r800", *r800);
+	}
+	ar.serializePointerID("activeCPU", activeCPU);
+	ar.serializePointerID("newCPU",    newCPU);
+	ar.serialize("resetTime", reference);
+}
+INSTANTIATE_SERIALIZE_METHODS(MSXCPU);
+
 } // namespace openmsx
