@@ -1,23 +1,22 @@
-// $Id: MSXCPUInterface.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: MSXCPUInterface.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "MSXCPUInterface.hh"
 #include "DummyDevice.hh"
 #include "SimpleDebuggable.hh"
-#include "MSXCommandController.hh"
 #include "InfoTopic.hh"
 #include "CommandException.hh"
 #include "TclObject.hh"
 #include "MSXMotherBoard.hh"
 #include "MSXCPU.hh"
-#include "XMLElement.hh"
 #include "VDPIODelay.hh"
-#include "MSXCliComm.hh"
+#include "CliComm.hh"
 #include "MSXMultiIODevice.hh"
 #include "MSXMultiMemDevice.hh"
 #include "MSXWatchIODevice.hh"
 #include "MSXException.hh"
 #include "CartridgeSlotManager.hh"
 #include "DeviceFactory.hh"
+#include "serialize.hh"
 #include "StringOp.hh"
 #include "checked_cast.hh"
 #include <cstdio>
@@ -45,8 +44,8 @@ class MemoryDebug : public SimpleDebuggable
 public:
 	MemoryDebug(MSXCPUInterface& interface,
 	            MSXMotherBoard& motherBoard);
-	virtual byte read(unsigned address, const EmuTime& time);
-	virtual void write(unsigned address, byte value, const EmuTime& time);
+	virtual byte read(unsigned address, EmuTime::param time);
+	virtual void write(unsigned address, byte value, EmuTime::param time);
 private:
 	MSXCPUInterface& interface;
 };
@@ -56,8 +55,8 @@ class SlottedMemoryDebug : public SimpleDebuggable
 public:
 	SlottedMemoryDebug(MSXCPUInterface& interface,
 	                   MSXMotherBoard& motherBoard);
-	virtual byte read(unsigned address, const EmuTime& time);
-	virtual void write(unsigned address, byte value, const EmuTime& time);
+	virtual byte read(unsigned address, EmuTime::param time);
+	virtual void write(unsigned address, byte value, EmuTime::param time);
 private:
 	MSXCPUInterface& interface;
 };
@@ -67,8 +66,8 @@ class IODebug : public SimpleDebuggable
 public:
 	IODebug(MSXCPUInterface& interface,
 	        MSXMotherBoard& motherBoard);
-	virtual byte read(unsigned address, const EmuTime& time);
-	virtual void write(unsigned address, byte value, const EmuTime& time);
+	virtual byte read(unsigned address, EmuTime::param time);
+	virtual void write(unsigned address, byte value, EmuTime::param time);
 private:
 	MSXCPUInterface& interface;
 };
@@ -123,36 +122,30 @@ private:
 };
 
 
-auto_ptr<MSXCPUInterface> MSXCPUInterface::create(
-		MSXMotherBoard& motherBoard, const XMLElement& machineConfig)
-{
-	if (machineConfig.getChild("devices").findChild("S1990")) {
-		return auto_ptr<MSXCPUInterface>(
-			new TurborCPUInterface(motherBoard));
-	} else {
-		return auto_ptr<MSXCPUInterface>(
-			new MSXCPUInterface(motherBoard));
-	}
-}
+// Bitfields used in the disallowReadCache and disallowWriteCache arrays
+static const byte SECUNDARY_SLOT_BIT = 0x01;
+static const byte MEMORY_WATCH_BIT   = 0x02;
+static const byte GLOBAL_WRITE_BIT   = 0x04;
+
 
 MSXCPUInterface::MSXCPUInterface(MSXMotherBoard& motherBoard_)
 	: memoryDebug       (new MemoryDebug       (*this, motherBoard_))
 	, slottedMemoryDebug(new SlottedMemoryDebug(*this, motherBoard_))
 	, ioDebug           (new IODebug           (*this, motherBoard_))
 	, slotInfo        (new SlotInfo(
-		motherBoard_.getMSXCommandController().getMachineInfoCommand(), *this))
+		motherBoard_.getMachineInfoCommand(), *this))
 	, subSlottedInfo  (new SubSlottedInfo(
-		motherBoard_.getMSXCommandController().getMachineInfoCommand(), *this))
+		motherBoard_.getMachineInfoCommand(), *this))
 	, externalSlotInfo(new ExternalSlotInfo(
-		motherBoard_.getMSXCommandController().getMachineInfoCommand(),
+		motherBoard_.getMachineInfoCommand(),
 		motherBoard_.getSlotManager()))
 	, inputPortInfo (new IOInfo(
-	        motherBoard_.getMSXCommandController().getMachineInfoCommand(), *this, true))
+	        motherBoard_.getMachineInfoCommand(), *this, true))
 	, outputPortInfo(new IOInfo(
-	        motherBoard_.getMSXCommandController().getMachineInfoCommand(), *this, false))
+	        motherBoard_.getMachineInfoCommand(), *this, false))
 	, dummyDevice(motherBoard_.getDummyDevice())
 	, msxcpu(motherBoard_.getCPU())
-	, cliCommOutput(motherBoard_.getMSXCliComm())
+	, cliComm(motherBoard_.getMSXCliComm())
 	, motherBoard(motherBoard_)
 {
 	for (int port = 0; port < 256; ++port) {
@@ -173,16 +166,40 @@ MSXCPUInterface::MSXCPUInterface(MSXMotherBoard& motherBoard_)
 	for (int page = 0; page < 4; ++page) {
 		visibleDevices[page] = &dummyDevice;
 	}
-	setAllowedCache();
+
+	// initially allow all regions to be cached
+	memset(disallowReadCache,  0, sizeof(disallowReadCache));
+	memset(disallowWriteCache, 0, sizeof(disallowWriteCache));
 
 	// Note: SlotState is initialised at reset
 
 	msxcpu.setInterface(this);
+
+	if (motherBoard.isTurboR()) {
+		// TODO also MSX2+ needs (slightly different) VDPIODelay
+		delayDevice = DeviceFactory::createVDPIODelay(motherBoard);
+		for (int port = 0x98; port <= 0x9B; ++port) {
+			assert(IO_In [port] == &dummyDevice);
+			assert(IO_Out[port] == &dummyDevice);
+			IO_In [port] = delayDevice.get();
+			IO_Out[port] = delayDevice.get();
+		}
+	}
 }
 
 MSXCPUInterface::~MSXCPUInterface()
 {
 	removeAllWatchPoints();
+
+	if (delayDevice.get()) {
+		for (int port = 0x98; port <= 0x9B; ++port) {
+			assert(IO_In [port] == delayDevice.get());
+			assert(IO_Out[port] == delayDevice.get());
+			IO_In [port] = &dummyDevice;
+			IO_Out[port] = &dummyDevice;
+		}
+	}
+
 	msxcpu.setInterface(NULL);
 
 	#ifndef NDEBUG
@@ -217,10 +234,10 @@ void MSXCPUInterface::removeAllWatchPoints()
 
 }
 
-byte MSXCPUInterface::readMemSlow(word address, const EmuTime& time)
+byte MSXCPUInterface::readMemSlow(word address, EmuTime::param time)
 {
 	// something special in this region?
-	if (unlikely(!allowReadCache[address >> CacheLine::BITS])) {
+	if (unlikely(disallowReadCache[address >> CacheLine::BITS])) {
 		// execute read watches before actual read
 		if (readWatchSet[address >> CacheLine::BITS]
 		                [address &  CacheLine::LOW]) {
@@ -234,7 +251,7 @@ byte MSXCPUInterface::readMemSlow(word address, const EmuTime& time)
 	}
 }
 
-void MSXCPUInterface::writeMemSlow(word address, byte value, const EmuTime& time)
+void MSXCPUInterface::writeMemSlow(word address, byte value, EmuTime::param time)
 {
 	if (unlikely((address == 0xFFFF) && isExpanded(primarySlotState[3]))) {
 		setSubSlot(primarySlotState[3], value);
@@ -242,7 +259,7 @@ void MSXCPUInterface::writeMemSlow(word address, byte value, const EmuTime& time
 		visibleDevices[address>>14]->writeMem(address, value, time);
 	}
 	// something special in this region?
-	if (unlikely(!allowWriteCache[address >> CacheLine::BITS])) {
+	if (unlikely(disallowWriteCache[address >> CacheLine::BITS])) {
 		// slot-select-ignore writes (Super Lode Runner)
 		for (GlobalWrites::const_iterator it = globalWrites.begin();
 		     it != globalWrites.end(); ++it) {
@@ -260,24 +277,6 @@ void MSXCPUInterface::writeMemSlow(word address, byte value, const EmuTime& time
 	}
 }
 
-void MSXCPUInterface::setAllowedCache()
-{
-	memset(allowReadCache,  1, sizeof(allowReadCache));
-	memset(allowWriteCache, 1, sizeof(allowWriteCache));
-	if (isExpanded(primarySlotState[3])) {
-		allowReadCache [255] = false;
-		allowWriteCache[255] = false;
-	}
-	for (unsigned i = 0; i < CacheLine::NUM; ++i) {
-		if (readWatchSet [i].any()) allowReadCache [i] = false;
-		if (writeWatchSet[i].any()) allowWriteCache[i] = false;
-	}
-	for (GlobalWrites::const_iterator it = globalWrites.begin();
-	     it != globalWrites.end(); ++it) {
-		allowWriteCache[it->addr >> CacheLine::BITS] = false;
-	}
-}
-
 void MSXCPUInterface::setExpanded(int ps)
 {
 	if (expanded[ps] == 0) {
@@ -384,7 +383,7 @@ void MSXCPUInterface::register_IO(int port, bool isIn,
 			devicePtr = multi;
 		}
 		if (isIn) {
-			cliCommOutput.printWarning(
+			cliComm.printWarning(
 				"Conflicting input port 0x" +
 				StringOp::toHexString(port, 2) +
 				" for devices " + devicePtr->getName());
@@ -511,7 +510,9 @@ void MSXCPUInterface::registerGlobalWrite(MSXDevice& device, word address)
 {
 	GlobalWriteInfo info = { &device, address };
 	globalWrites.push_back(info);
-	setAllowedCache();
+
+	disallowWriteCache[address >> CacheLine::BITS] |= GLOBAL_WRITE_BIT;
+	msxcpu.invalidateMemCache(address & CacheLine::HIGH, 0x100);
 }
 
 void MSXCPUInterface::unregisterGlobalWrite(MSXDevice& device, word address)
@@ -521,7 +522,17 @@ void MSXCPUInterface::unregisterGlobalWrite(MSXDevice& device, word address)
 		find(globalWrites.begin(), globalWrites.end(), info);
 	assert(it != globalWrites.end());
 	globalWrites.erase(it);
-	setAllowedCache();
+
+	for (GlobalWrites::const_iterator it = globalWrites.begin();
+	     it != globalWrites.end(); ++it) {
+		if ((it->addr >> CacheLine::BITS) ==
+		    (address  >> CacheLine::BITS)) {
+			// there is still a global write in this region
+			return;
+		}
+	}
+	disallowWriteCache[address >> CacheLine::BITS] &= ~GLOBAL_WRITE_BIT;
+	msxcpu.invalidateMemCache(address & CacheLine::HIGH, 0x100);
 }
 
 void MSXCPUInterface::updateVisible(int page)
@@ -565,7 +576,14 @@ void MSXCPUInterface::setPrimarySlots(byte value)
 		// Change the visible devices
 		updateVisible(page);
 	}
-	setAllowedCache();
+	if (isExpanded(primarySlotState[3])) {
+		disallowReadCache [0xFF] |=  SECUNDARY_SLOT_BIT;
+		disallowWriteCache[0xFF] |=  SECUNDARY_SLOT_BIT;
+	} else {
+		disallowReadCache [0xFF] &= ~SECUNDARY_SLOT_BIT;
+		disallowWriteCache[0xFF] &= ~SECUNDARY_SLOT_BIT;
+	}
+	msxcpu.invalidateMemCache(0xFFFF & CacheLine::HIGH, 0x100);
 }
 
 void MSXCPUInterface::setSubSlot(byte primSlot, byte value)
@@ -580,7 +598,7 @@ void MSXCPUInterface::setSubSlot(byte primSlot, byte value)
 	}
 }
 
-byte MSXCPUInterface::peekMem(word address, const EmuTime& time) const
+byte MSXCPUInterface::peekMem(word address, EmuTime::param time) const
 {
 	if ((address == 0xFFFF) && isExpanded(primarySlotState[3])) {
 		return 0xFF ^ subSlotRegister[primarySlotState[3]];
@@ -589,7 +607,7 @@ byte MSXCPUInterface::peekMem(word address, const EmuTime& time) const
 	}
 }
 
-byte MSXCPUInterface::peekSlottedMem(unsigned address, const EmuTime& time) const
+byte MSXCPUInterface::peekSlottedMem(unsigned address, EmuTime::param time) const
 {
 	byte primSlot = (address & 0xC0000) >> 18;
 	byte subSlot = (address & 0x30000) >> 16;
@@ -607,7 +625,7 @@ byte MSXCPUInterface::peekSlottedMem(unsigned address, const EmuTime& time) cons
 }
 
 void MSXCPUInterface::writeSlottedMem(unsigned address, byte value,
-                                      const EmuTime& time)
+                                      EmuTime::param time)
 {
 	byte primSlot = (address & 0xC0000) >> 18;
 	byte subSlot = (address & 0x30000) >> 16;
@@ -732,7 +750,18 @@ void MSXCPUInterface::updateMemWatch(WatchPoint::Type type)
 			}
 		}
 	}
-	setAllowedCache();
+	for (unsigned i = 0; i < CacheLine::NUM; ++i) {
+		if (readWatchSet [i].any()) {
+			disallowReadCache [i] |=  MEMORY_WATCH_BIT;
+		} else {
+			disallowReadCache [i] &= ~MEMORY_WATCH_BIT;
+		}
+		if (writeWatchSet[i].any()) {
+			disallowWriteCache[i] |=  MEMORY_WATCH_BIT;
+		} else {
+			disallowWriteCache[i] &= ~MEMORY_WATCH_BIT;
+		}
+	}
 	msxcpu.invalidateMemCache(0x0000, 0x10000);
 }
 
@@ -758,13 +787,13 @@ MemoryDebug::MemoryDebug(
 {
 }
 
-byte MemoryDebug::read(unsigned address, const EmuTime& time)
+byte MemoryDebug::read(unsigned address, EmuTime::param time)
 {
 	return interface.peekMem(address, time);
 }
 
 void MemoryDebug::write(unsigned address, byte value,
-                                         const EmuTime& time)
+                                         EmuTime::param time)
 {
 	return interface.writeMem(address, value, time);
 }
@@ -780,13 +809,13 @@ SlottedMemoryDebug::SlottedMemoryDebug(
 {
 }
 
-byte SlottedMemoryDebug::read(unsigned address, const EmuTime& time)
+byte SlottedMemoryDebug::read(unsigned address, EmuTime::param time)
 {
 	return interface.peekSlottedMem(address, time);
 }
 
 void SlottedMemoryDebug::write(unsigned address, byte value,
-                                                const EmuTime& time)
+                                                EmuTime::param time)
 {
 	return interface.writeSlottedMem(address, value, time);
 }
@@ -900,12 +929,12 @@ IODebug::IODebug(MSXCPUInterface& interface_,
 {
 }
 
-byte IODebug::read(unsigned address, const EmuTime& time)
+byte IODebug::read(unsigned address, EmuTime::param time)
 {
 	return interface.IO_In[address & 0xFF]->peekIO(address, time);
 }
 
-void IODebug::write(unsigned address, byte value, const EmuTime& time)
+void IODebug::write(unsigned address, byte value, EmuTime::param time)
 {
 	interface.writeIO(word(address), value, time);
 }
@@ -940,30 +969,31 @@ string IOInfo::help(const vector<string>& /*tokens*/) const
 }
 
 
-// class TurborCPUInterface
-
-TurborCPUInterface::TurborCPUInterface(MSXMotherBoard& motherBoard)
-	: MSXCPUInterface(motherBoard)
+template<typename Archive>
+void MSXCPUInterface::serialize(Archive& ar, unsigned /*version*/)
 {
-	delayDevice = DeviceFactory::createVDPIODelay(motherBoard);
+	// TODO watchPoints ???
 
-	for (int port = 0x98; port <= 0x9B; ++port) {
-		assert(IO_In [port] == &dummyDevice);
-		assert(IO_Out[port] == &dummyDevice);
-		IO_In [port] = delayDevice.get();
-		IO_Out[port] = delayDevice.get();
+	// primary and 4 secundary slot select registers
+	byte prim = 0;
+	if (!ar.isLoader()) {
+		for (int i = 0; i < 4; ++i) {
+			prim |= primarySlotState[i] << (2 * i);
+		}
+	}
+	ar.serialize("primarySlots", prim);
+	ar.serialize("subSlotRegs", subSlotRegister);
+	if (ar.isLoader()) {
+		setPrimarySlots(prim);
+		for (int i = 0; i < 4; ++i) {
+			setSubSlot(i, subSlotRegister[i]);
+		}
 	}
-}
 
-TurborCPUInterface::~TurborCPUInterface()
-{
-	removeAllWatchPoints();
-	for (int port = 0x98; port <= 0x9B; ++port) {
-		assert(IO_In [port] == delayDevice.get());
-		assert(IO_Out[port] == delayDevice.get());
-		IO_In [port] = &dummyDevice;
-		IO_Out[port] = &dummyDevice;
+	if (delayDevice.get()) {
+		ar.serialize("vdpDelay", *delayDevice);
 	}
 }
+INSTANTIATE_SERIALIZE_METHODS(MSXCPUInterface);
 
 } // namespace openmsx
