@@ -1,4 +1,4 @@
-// $Id: MSXWatchIODevice.cc 6593 2007-06-03 23:12:19Z mthuurne $
+// $Id: MSXWatchIODevice.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "MSXWatchIODevice.hh"
 #include "MSXMotherBoard.hh"
@@ -34,13 +34,13 @@ std::string MSXWatchIODevice::getName() const
 	return device->getName();
 }
 
-byte MSXWatchIODevice::peekIO(word port, const EmuTime& time) const
+byte MSXWatchIODevice::peekIO(word port, EmuTime::param time) const
 {
 	assert(device);
 	return device->peekIO(port, time);
 }
 
-byte MSXWatchIODevice::readIO(word port, const EmuTime& time)
+byte MSXWatchIODevice::readIO(word port, EmuTime::param time)
 {
 	assert(device);
 	//std::cout << "Watch readIO " << port << std::endl;
@@ -49,7 +49,7 @@ byte MSXWatchIODevice::readIO(word port, const EmuTime& time)
 	return result;
 }
 
-void MSXWatchIODevice::writeIO(word port, byte value, const EmuTime& time)
+void MSXWatchIODevice::writeIO(word port, byte value, EmuTime::param time)
 {
 	assert(device);
 	//std::cout << "Watch writeIO " << port << " " << value << std::endl;
