@@ -1,4 +1,4 @@
-// $Id: WatchPoint.cc 6366 2007-04-24 19:05:58Z m9710797 $
+// $Id: WatchPoint.cc 8182 2008-08-28 18:05:52Z m9710797 $
 
 #include "WatchPoint.hh"
 #include "TclObject.hh"
@@ -8,15 +8,15 @@ namespace openmsx {
 
 unsigned WatchPoint::lastId = 0;
 
-WatchPoint::WatchPoint(MSXCliComm& cliComm,
+WatchPoint::WatchPoint(CliComm& cliComm,
                        std::auto_ptr<TclObject> command,
                        std::auto_ptr<TclObject> condition,
                        Type type_, unsigned beginAddr_, unsigned endAddr_)
 	: BreakPointBase(cliComm, command, condition)
+	, id(++lastId)
 	, beginAddr(beginAddr_), endAddr(endAddr_), type(type_)
 {
 	assert(beginAddr <= endAddr);
-	id = ++lastId;
 }
 
 WatchPoint::~WatchPoint()
