@@ -1,7 +1,9 @@
-// $Id: Debugger.cc 6593 2007-06-03 23:12:19Z mthuurne $
+// $Id: Debugger.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
-#include "Debuggable.hh"
 #include "Debugger.hh"
+#include "Debuggable.hh"
+#include "Probe.hh"
+#include "ProbeBreakPoint.hh"
 #include "MSXMotherBoard.hh"
 #include "MSXCPU.hh"
 #include "MSXCPUInterface.hh"
@@ -25,13 +27,13 @@ namespace openmsx {
 class DebugCmd : public RecordedCommand
 {
 public:
-	DebugCmd(MSXCommandController& msxCommandController,
+	DebugCmd(CommandController& commandController,
 	         MSXEventDistributor& msxEventDistributor,
-	         Scheduler& scheduler, MSXCliComm& cliComm,
+	         Scheduler& scheduler, CliComm& cliComm,
 	         Debugger& debugger);
 	virtual bool needRecord(const vector<TclObject*>& tokens) const;
 	virtual void execute(const vector<TclObject*>& tokens,
-	                     TclObject& result, const EmuTime& time);
+	                     TclObject& result, EmuTime::param time);
 	virtual string help(const vector<string>& tokens) const;
 	virtual void tabCompletion(vector<string>& tokens) const;
 
@@ -63,15 +65,29 @@ private:
 	                      TclObject& result);
 	void listWatchPoints(const vector<TclObject*>& tokens,
 	                     TclObject& result);
+	void probe(const vector<TclObject*>& tokens,
+	           TclObject& result);
+	void probeList(const vector<TclObject*>& tokens,
+	               TclObject& result);
+	void probeDesc(const vector<TclObject*>& tokens,
+	               TclObject& result);
+	void probeRead(const vector<TclObject*>& tokens,
+	               TclObject& result);
+	void probeSetBreakPoint(const vector<TclObject*>& tokens,
+	                        TclObject& result);
+	void probeRemoveBreakPoint(const vector<TclObject*>& tokens,
+	                           TclObject& result);
+	void probeListBreakPoints(const vector<TclObject*>& tokens,
+	                          TclObject& result);
 
-	MSXCliComm& cliComm;
+	CliComm& cliComm;
 	Debugger& debugger;
 };
 
 
 Debugger::Debugger(MSXMotherBoard& motherBoard_)
 	: motherBoard(motherBoard_)
-	, debugCmd(new DebugCmd(motherBoard.getMSXCommandController(),
+	, debugCmd(new DebugCmd(motherBoard.getCommandController(),
 	                        motherBoard.getMSXEventDistributor(),
 	                        motherBoard.getScheduler(),
 	                        motherBoard.getMSXCliComm(), *this))
@@ -81,6 +97,11 @@ Debugger::Debugger(MSXMotherBoard& motherBoard_)
 
 Debugger::~Debugger()
 {
+	for (ProbeBreakPoints::const_iterator it = probeBreakPoints.begin();
+	     it != probeBreakPoints.end(); ++it) {
+		delete *it;
+	}
+
 	assert(!cpu);
 	assert(debuggables.empty());
 }
@@ -99,35 +120,116 @@ void Debugger::registerDebuggable(const string& name, Debuggable& debuggable)
 void Debugger::unregisterDebuggable(const string& name, Debuggable& debuggable)
 {
 	(void)debuggable;
-	map<string, Debuggable*>::iterator it = debuggables.find(name);
+	Debuggables::iterator it = debuggables.find(name);
 	assert(it != debuggables.end() && (it->second == &debuggable));
 	debuggables.erase(it);
 }
 
 Debuggable* Debugger::findDebuggable(const string& name)
 {
-	map<string, Debuggable*>::iterator it = debuggables.find(name);
+	Debuggables::iterator it = debuggables.find(name);
 	return (it != debuggables.end()) ? it->second : NULL;
 }
 
-Debuggable* Debugger::getDebuggable(const string& name)
+Debuggable& Debugger::getDebuggable(const string& name)
 {
 	Debuggable* result = findDebuggable(name);
 	if (!result) {
-		throw CommandException("No such debuggable.");
+		throw CommandException("No such debuggable: " + name);
 	}
-	return result;
+	return *result;
 }
 
 void Debugger::getDebuggables(set<string>& result) const
 {
-	for (map<string, Debuggable*>::const_iterator it = debuggables.begin();
+	for (Debuggables::const_iterator it = debuggables.begin();
 	     it != debuggables.end(); ++it) {
 		result.insert(it->first);
 	}
 }
 
 
+void Debugger::registerProbe(const std::string& name, ProbeBase& probe)
+{
+	assert(probes.find(name) == probes.end());
+	probes[name] = &probe;
+}
+
+void Debugger::unregisterProbe(const std::string& name, ProbeBase& probe)
+{
+	(void)probe;
+	Probes::iterator it = probes.find(name);
+	assert(it != probes.end() && (it->second == &probe));
+	probes.erase(it);
+}
+
+ProbeBase* Debugger::findProbe(const std::string& name)
+{
+	Probes::iterator it = probes.find(name);
+	return (it != probes.end()) ? it->second : NULL;
+}
+
+ProbeBase& Debugger::getProbe(const std::string& name)
+{
+	ProbeBase* result = findProbe(name);
+	if (!result) {
+		throw CommandException("No such probe: " + name);
+	}
+	return *result;
+}
+
+void Debugger::getProbes(std::set<std::string>& result) const
+{
+	for (Probes::const_iterator it = probes.begin();
+	     it != probes.end(); ++it) {
+		result.insert(it->first);
+	}
+}
+
+
+void Debugger::insertProbeBreakPoint(auto_ptr<ProbeBreakPoint> bp)
+{
+	probeBreakPoints.push_back(bp.release());
+}
+
+void Debugger::removeProbeBreakPoint(const string& name)
+{
+	if (StringOp::startsWith(name, "pp#")) {
+		// remove by id
+		unsigned id = StringOp::stringToInt(name.substr(3));
+		for (ProbeBreakPoints::iterator it = probeBreakPoints.begin();
+		     it != probeBreakPoints.end(); ++it) {
+			if ((*it)->getId() == id) {
+				delete *it;
+				probeBreakPoints.erase(it);
+				return;
+			}
+		}
+		throw CommandException("No such breakpoint: " + name);
+	} else {
+		// remove by probe, only works for unconditional bp
+		for (ProbeBreakPoints::iterator it = probeBreakPoints.begin();
+		     it != probeBreakPoints.end(); ++it) {
+			if ((*it)->getProbe().getName() == name) {
+				delete *it;
+				probeBreakPoints.erase(it);
+				return;
+			}
+		}
+		throw CommandException(
+			"No (unconditional) breakpoint for probe: " + name);
+	}
+}
+
+void Debugger::removeProbeBreakPoint(ProbeBreakPoint& bp)
+{
+	ProbeBreakPoints::iterator it =
+		find(probeBreakPoints.begin(), probeBreakPoints.end(), &bp);
+	assert(it != probeBreakPoints.end());
+	delete *it;
+	probeBreakPoints.erase(it);
+}
+
 // class DebugCmd
 
 static word getAddress(const vector<TclObject*>& tokens)
@@ -142,11 +244,11 @@ static word getAddress(const vector<TclObject*>& tokens)
 	return addr;
 }
 
-DebugCmd::DebugCmd(MSXCommandController& msxCommandController,
+DebugCmd::DebugCmd(CommandController& commandController,
                    MSXEventDistributor& msxEventDistributor,
-                   Scheduler& scheduler, MSXCliComm& cliComm_,
+                   Scheduler& scheduler, CliComm& cliComm_,
                    Debugger& debugger_)
-	: RecordedCommand(msxCommandController, msxEventDistributor,
+	: RecordedCommand(commandController, msxEventDistributor,
 	                  scheduler, "debug")
 	, cliComm(cliComm_)
 	, debugger(debugger_)
@@ -161,7 +263,7 @@ bool DebugCmd::needRecord(const vector<TclObject*>& tokens) const
 }
 
 void DebugCmd::execute(const vector<TclObject*>& tokens,
-                       TclObject& result, const EmuTime& /*time*/)
+                       TclObject& result, EmuTime::param /*time*/)
 {
 	if (tokens.size() < 2) {
 		throw CommandException("Missing argument");
@@ -203,6 +305,8 @@ void DebugCmd::execute(const vector<TclObject*>& tokens,
 		removeWatchPoint(tokens, result);
 	} else if (subCmd == "list_watchpoints") {
 		listWatchPoints(tokens, result);
+	} else if (subCmd == "probe") {
+		probe(tokens, result);
 	} else {
 		throw SyntaxError();
 	}
@@ -210,11 +314,9 @@ void DebugCmd::execute(const vector<TclObject*>& tokens,
 
 void DebugCmd::list(TclObject& result)
 {
-	for (map<string, Debuggable*>::const_iterator it =
-	       debugger.debuggables.begin();
-	     it != debugger.debuggables.end(); ++it) {
-		result.addListElement(it->first);
-	}
+	set<string> debuggables;
+	debugger.getDebuggables(debuggables);
+	result.addListElements(debuggables.begin(), debuggables.end());
 }
 
 void DebugCmd::desc(const vector<TclObject*>& tokens, TclObject& result)
@@ -222,8 +324,8 @@ void DebugCmd::desc(const vector<TclObject*>& tokens, TclObject& result)
 	if (tokens.size() != 3) {
 		throw SyntaxError();
 	}
-	Debuggable* device = debugger.getDebuggable(tokens[2]->getString());
-	result.setString(device->getDescription());
+	Debuggable& device = debugger.getDebuggable(tokens[2]->getString());
+	result.setString(device.getDescription());
 }
 
 void DebugCmd::size(const vector<TclObject*>& tokens, TclObject& result)
@@ -231,8 +333,8 @@ void DebugCmd::size(const vector<TclObject*>& tokens, TclObject& result)
 	if (tokens.size() != 3) {
 		throw SyntaxError();
 	}
-	Debuggable* device = debugger.getDebuggable(tokens[2]->getString());
-	result.setInt(device->getSize());
+	Debuggable& device = debugger.getDebuggable(tokens[2]->getString());
+	result.setInt(device.getSize());
 }
 
 void DebugCmd::read(const vector<TclObject*>& tokens, TclObject& result)
@@ -240,12 +342,12 @@ void DebugCmd::read(const vector<TclObject*>& tokens, TclObject& result)
 	if (tokens.size() != 4) {
 		throw SyntaxError();
 	}
-	Debuggable* device = debugger.getDebuggable(tokens[2]->getString());
+	Debuggable& device = debugger.getDebuggable(tokens[2]->getString());
 	unsigned addr = tokens[3]->getInt();
-	if (addr >= device->getSize()) {
+	if (addr >= device.getSize()) {
 		throw CommandException("Invalid address");
 	}
-	result.setInt(device->read(addr));
+	result.setInt(device.read(addr));
 }
 
 void DebugCmd::readBlock(const vector<TclObject*>& tokens, TclObject& result)
@@ -253,8 +355,8 @@ void DebugCmd::readBlock(const vector<TclObject*>& tokens, TclObject& result)
 	if (tokens.size() != 5) {
 		throw SyntaxError();
 	}
-	Debuggable* device = debugger.getDebuggable(tokens[2]->getString());
-	unsigned size = device->getSize();
+	Debuggable& device = debugger.getDebuggable(tokens[2]->getString());
+	unsigned size = device.getSize();
 	unsigned addr = tokens[3]->getInt();
 	if (addr >= size) {
 		throw CommandException("Invalid address");
@@ -266,7 +368,7 @@ void DebugCmd::readBlock(const vector<TclObject*>& tokens, TclObject& result)
 
 	byte* buf = new byte[num];
 	for (unsigned i = 0; i < num; ++i) {
-		buf[i] = device->read(addr + i);
+		buf[i] = device.read(addr + i);
 	}
 	result.setBinary(buf, num);
 	delete[] buf;
@@ -278,9 +380,9 @@ void DebugCmd::write(const vector<TclObject*>& tokens,
 	if (tokens.size() != 5) {
 		throw SyntaxError();
 	}
-	Debuggable* device = debugger.getDebuggable(tokens[2]->getString());
+	Debuggable& device = debugger.getDebuggable(tokens[2]->getString());
 	unsigned addr = tokens[3]->getInt();
-	if (addr >= device->getSize()) {
+	if (addr >= device.getSize()) {
 		throw CommandException("Invalid address");
 	}
 	unsigned value = tokens[4]->getInt();
@@ -288,7 +390,7 @@ void DebugCmd::write(const vector<TclObject*>& tokens,
 		throw CommandException("Invalid value");
 	}
 
-	device->write(addr, value);
+	device.write(addr, value);
 }
 
 void DebugCmd::writeBlock(const vector<TclObject*>& tokens,
@@ -297,8 +399,8 @@ void DebugCmd::writeBlock(const vector<TclObject*>& tokens,
 	if (tokens.size() != 5) {
 		throw SyntaxError();
 	}
-	Debuggable* device = debugger.getDebuggable(tokens[2]->getString());
-	unsigned size = device->getSize();
+	Debuggable& device = debugger.getDebuggable(tokens[2]->getString());
+	unsigned size = device.getSize();
 	unsigned addr = tokens[3]->getInt();
 	if (addr >= size) {
 		throw CommandException("Invalid address");
@@ -310,7 +412,7 @@ void DebugCmd::writeBlock(const vector<TclObject*>& tokens,
 	}
 
 	for (unsigned i = 0; i < num; ++i) {
-		device->write(addr + i, static_cast<byte>(buf[i]));
+		device.write(addr + i, static_cast<byte>(buf[i]));
 	}
 }
 
@@ -547,6 +649,114 @@ void DebugCmd::listWatchPoints(const vector<TclObject*>& /*tokens*/,
 	result.setString(res);
 }
 
+void DebugCmd::probe(const vector<TclObject*>& tokens,
+                     TclObject& result)
+{
+	if (tokens.size() < 3) {
+		throw CommandException("Missing argument");
+	}
+	string subCmd = tokens[2]->getString();
+	if (subCmd == "list") {
+		probeList(tokens, result);
+	} else if (subCmd == "desc") {
+		probeDesc(tokens, result);
+	} else if (subCmd == "read") {
+		probeRead(tokens, result);
+	} else if (subCmd == "set_bp") {
+		probeSetBreakPoint(tokens, result);
+	} else if (subCmd == "remove_bp") {
+		probeRemoveBreakPoint(tokens, result);
+	} else if (subCmd == "list_bp") {
+		probeListBreakPoints(tokens, result);
+	} else {
+		throw SyntaxError();
+	}
+}
+void DebugCmd::probeList(const vector<TclObject*>& /*tokens*/,
+                         TclObject& result)
+{
+	set<string> probes;
+	debugger.getProbes(probes);
+	result.addListElements(probes.begin(), probes.end());
+}
+void DebugCmd::probeDesc(const vector<TclObject*>& tokens,
+                         TclObject& result)
+{
+	if (tokens.size() != 4) {
+		throw SyntaxError();
+	}
+	ProbeBase& probe = debugger.getProbe(tokens[3]->getString());
+	result.setString(probe.getDescription());
+}
+void DebugCmd::probeRead(const vector<TclObject*>& tokens,
+                         TclObject& result)
+{
+	if (tokens.size() != 4) {
+		throw SyntaxError();
+	}
+	ProbeBase& probe = debugger.getProbe(tokens[3]->getString());
+	result.setString(probe.getValue());
+}
+void DebugCmd::probeSetBreakPoint(const vector<TclObject*>& tokens,
+                                  TclObject& result)
+{
+	auto_ptr<ProbeBreakPoint> bp;
+	auto_ptr<TclObject> command(
+		new TclObject(result.getInterpreter(), "debug break"));
+	auto_ptr<TclObject> condition;
+	switch (tokens.size()) {
+	case 6: // command
+		command->setString(tokens[5]->getString());
+		command->checkCommand();
+		// fall-through
+	case 5: // condition
+		if (!tokens[4]->getString().empty()) {
+			condition.reset(new TclObject(*tokens[4]));
+			condition->checkExpression();
+		}
+		// fall-through
+	case 4: { // probe
+		ProbeBase& probe = debugger.getProbe(tokens[3]->getString());
+		bp.reset(new ProbeBreakPoint(cliComm, command, condition,
+		                             debugger, probe));
+		break;
+	}
+	default:
+		if (tokens.size() < 4) {
+			throw CommandException("Too few arguments.");
+		} else {
+			throw CommandException("Too many arguments.");
+		}
+	}
+	result.setString("pp#" + StringOp::toString(bp->getId()));
+	debugger.insertProbeBreakPoint(bp);
+}
+void DebugCmd::probeRemoveBreakPoint(const vector<TclObject*>& tokens,
+                                     TclObject& /*result*/)
+{
+	if (tokens.size() != 4) {
+		throw SyntaxError();
+	}
+	debugger.removeProbeBreakPoint(tokens[3]->getString());
+}
+void DebugCmd::probeListBreakPoints(const vector<TclObject*>& /*tokens*/,
+                                    TclObject& result)
+{
+	string res;
+	for (Debugger::ProbeBreakPoints::const_iterator it =
+	         debugger.probeBreakPoints.begin();
+	     it != debugger.probeBreakPoints.end(); ++it) {
+		const ProbeBreakPoint& bp = **it;
+		TclObject line(result.getInterpreter());
+		line.addListElement("pp#" + StringOp::toString(bp.getId()));
+		line.addListElement(bp.getProbe().getName());
+		line.addListElement(bp.getCondition());
+		line.addListElement(bp.getCommand());
+		res += line.getString() + '\n';
+	}
+	result.setString(res);
+}
+
 string DebugCmd::help(const vector<string>& tokens) const
 {
 	static const string generalHelp =
@@ -565,6 +775,7 @@ string DebugCmd::help(const vector<string>& tokens) const
 		"    set_watchpoint    insert a new watchpoint\n"
 		"    remove_watchpoint remove a certain watchpoint\n"
 		"    list_watchpoints  list the active watchpoints\n"
+		"    probe             probe related subcommands\n"
 		"    cont              continue execution after break\n"
 		"    step              execute one instruction\n"
 		"    break             break CPU at current position\n"
@@ -588,7 +799,7 @@ string DebugCmd::help(const vector<string>& tokens) const
 		"  Read a byte at offset <addr> from the given debuggable.\n"
 		"  The offset must be smaller then the value returned from the "
 		"'size' subcommand\n"
-		"  Note that openMSX comes with a bunch of TCL scripts that make "
+		"  Note that openMSX comes with a bunch of Tcl scripts that make "
 		"some of the debug reads much more convient (e.g. reading from "
 		"Z80 or VDP registers). See the 'commands.txt' document for more "
 		"details about these.\n";
@@ -601,7 +812,7 @@ string DebugCmd::help(const vector<string>& tokens) const
 		"debug read_block <name> <addr> <size>\n"
 		"  Read a whole block at once. This is equivalent with repeated "
 		"invokations of the 'read' subcommand, but using this subcommand "
-		"may be faster. The result is a TCL binary string (see TCL manual).\n"
+		"may be faster. The result is a Tcl binary string (see Tcl manual).\n"
 		"  The block is specified as size/offset in the debuggable. The "
 		"complete block must fit in the debuggable (see the 'size' "
 		"subcommand).\n";
@@ -609,8 +820,8 @@ string DebugCmd::help(const vector<string>& tokens) const
 		"debug write_block <name> <addr> <values>\n"
 		"  Write a whole block at once. This is equivalent with repeated "
 		"invokations of the 'write' subcommand, but using this subcommand "
-		"may be faster. The <values> arugment must be a TCL binary string "
-		"(see TCL manual).\n"
+		"may be faster. The <values> arugment must be a Tcl binary string "
+		"(see Tcl manual).\n"
 		"  The block has a size and an offset in the debuggable. The "
 		"complete block must fit in the debuggable (see the 'size' "
 		"subcommand).\n";
@@ -623,7 +834,7 @@ string DebugCmd::help(const vector<string>& tokens) const
 		"  Optionally you can specify a condition. When the CPU reaches "
 		"the breakpoint this condition is evaluated, only when the condition "
 		"evaluated to true execution will be breaked.\n"
-		"  A condition must be specified as a TCL expression. For example\n"
+		"  A condition must be specified as a Tcl expression. For example\n"
 		"     debug set_bp 0xf37d {[reg C] == 0x2F}\n"
 		"  This break on address on address 0xf37d but only when Z80 "
 		"register C has the value 0x2F.\n"
@@ -669,6 +880,15 @@ string DebugCmd::help(const vector<string>& tokens) const
 		"  Lists all active breakpoints. The result is similar to the "
 		"'list_bp' subcommand, but there is an extra column (2nd column) "
 		"that contains the type of the watchpoint.\n";
+	static const string probeHelp =
+		"debug probe <subcommand> [<arguments>]\n"
+		"  Possible subcommands are:\n"
+		"    list                             returns a list of all probes\n"
+		"    desc   <probe>                   returns a description of this probe\n"
+		"    read   <probe>                   returns the current value of this probe\n"
+		"    set_bp <probe> [<cond>] [<cmd>]  set a breakpoint on the given probe\n"
+		"    remove_bp <id>                   remove the given reakpoint\n"
+		"    list_bp                          returns a list of breakpoints that are set on probes\n";
 	static const string contHelp =
 		"debug cont\n"
 		"  Continue execution after CPU was breaked.\n";
@@ -687,12 +907,12 @@ string DebugCmd::help(const vector<string>& tokens) const
 	static const string disasmHelp =
 		"debug disasm <addr>\n"
 		"  Disassemble the instruction at the given address. The result "
-		"is a TCL list. The first element in the list contains a textual "
+		"is a Tcl list. The first element in the list contains a textual "
 		"representation of the instruction, the next elements contain the "
 		"bytes that make up this instruction (thus the length of the "
 		"resulting list can be used to derive the number of bytes in the "
 		"instruction).\n"
-		"  Note that openMSX comes with a 'disasm' TCL script that is much "
+		"  Note that openMSX comes with a 'disasm' Tcl script that is much "
 		"more convient to use then this subcommand.";
 	static const string unknownHelp =
 		"Unknown subcommand, use 'help debug' to see a list of valid "
@@ -726,6 +946,8 @@ string DebugCmd::help(const vector<string>& tokens) const
 		return removeWatchPointHelp;
 	} else if (tokens[1] == "list_watchpoints") {
 		return listWatchPointsHelp;
+	} else if (tokens[1] == "probe") {
+		return probeHelp;
 	} else if (tokens[1] == "cont") {
 		return contHelp;
 	} else if (tokens[1] == "step") {
@@ -786,45 +1008,62 @@ void DebugCmd::tabCompletion(vector<string>& tokens) const
 	otherCmds.insert("remove_bp");
 	otherCmds.insert("set_watchpoint");
 	otherCmds.insert("remove_watchpoint");
+	otherCmds.insert("probe");
 	switch (tokens.size()) {
-		case 2: {
-			set<string> cmds;
-			cmds.insert(singleArgCmds.begin(), singleArgCmds.end());
-			cmds.insert(debuggableArgCmds.begin(), debuggableArgCmds.end());
-			cmds.insert(otherCmds.begin(), otherCmds.end());
-			completeString(tokens, cmds);
-			break;
-		}
-		case 3: {
-			if (singleArgCmds.find(tokens[1]) ==
-					singleArgCmds.end()) {
-				// this command takes (an) argument(s)
-				if (debuggableArgCmds.find(tokens[1]) !=
-					debuggableArgCmds.end()) {
-					// it takes a debuggable here
-					set<string> debuggables;
-					debugger.getDebuggables(debuggables);
-					completeString(tokens, debuggables);
-				} else if (tokens[1] == "remove_bp") {
-					// this one takes a bp id
-					set<string> bpids = getBreakPointIdsAsStringSet();
-					completeString(tokens, bpids);
-				} else if (tokens[1] == "remove_watchpoint") {
-					// this one takes a wp id
-					set<string> wpids = getWatchPointIdsAsStringSet();
-					completeString(tokens, wpids);
-				} else if (tokens[1] == "set_watchpoint") {
-					set<string> types;
-					types.insert("write_io");
-					types.insert("write_mem");
-					types.insert("read_io");
-					types.insert("read_mem");
-					completeString(tokens, types);
-				}
+	case 2: {
+		set<string> cmds;
+		cmds.insert(singleArgCmds.begin(), singleArgCmds.end());
+		cmds.insert(debuggableArgCmds.begin(), debuggableArgCmds.end());
+		cmds.insert(otherCmds.begin(), otherCmds.end());
+		completeString(tokens, cmds);
+		break;
+	}
+	case 3:
+		if (singleArgCmds.find(tokens[1]) ==
+				singleArgCmds.end()) {
+			// this command takes (an) argument(s)
+			if (debuggableArgCmds.find(tokens[1]) !=
+				debuggableArgCmds.end()) {
+				// it takes a debuggable here
+				set<string> debuggables;
+				debugger.getDebuggables(debuggables);
+				completeString(tokens, debuggables);
+			} else if (tokens[1] == "remove_bp") {
+				// this one takes a bp id
+				set<string> bpids = getBreakPointIdsAsStringSet();
+				completeString(tokens, bpids);
+			} else if (tokens[1] == "remove_watchpoint") {
+				// this one takes a wp id
+				set<string> wpids = getWatchPointIdsAsStringSet();
+				completeString(tokens, wpids);
+			} else if (tokens[1] == "set_watchpoint") {
+				set<string> types;
+				types.insert("write_io");
+				types.insert("write_mem");
+				types.insert("read_io");
+				types.insert("read_mem");
+				completeString(tokens, types);
+			} else if (tokens[1] == "probe") {
+				set<string> subCmds;
+				subCmds.insert("list");
+				subCmds.insert("desc");
+				subCmds.insert("read");
+				subCmds.insert("set_bp");
+				subCmds.insert("remove_bp");
+				subCmds.insert("list_bp");
+				completeString(tokens, subCmds);
 			}
-			break;
-		// other stuff doesn't really make sense to complete
 		}
+		break;
+	case 4:
+		if ((tokens[1] == "probe") &&
+		    ((tokens[2] == "desc") || (tokens[2] == "read") ||
+		     (tokens[2] == "set_bp"))) {
+			set<string> probes;
+			debugger.getProbes(probes);
+			completeString(tokens, probes);
+		}
+		break;
 	}
 }
 
