@@ -1,12 +1,10 @@
-// $Id: SimpleDebuggable.cc 5161 2006-02-11 17:44:09Z manuelbi $
+// $Id: SimpleDebuggable.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
 #include "Debugger.hh"
-#include "Scheduler.hh"
 #include <cassert>
 
-
 namespace openmsx {
 
 SimpleDebuggable::SimpleDebuggable(
@@ -42,10 +40,10 @@ const std::string& SimpleDebuggable::getDescription() const
 
 byte SimpleDebuggable::read(unsigned address)
 {
-	return read(address, motherBoard.getScheduler().getCurrentTime());
+	return read(address, motherBoard.getCurrentTime());
 }
 
-byte SimpleDebuggable::read(unsigned /*address*/, const EmuTime& /*time*/)
+byte SimpleDebuggable::read(unsigned /*address*/, EmuTime::param /*time*/)
 {
 	assert(false);
 	return 0;
@@ -53,11 +51,11 @@ byte SimpleDebuggable::read(unsigned /*address*/, const EmuTime& /*time*/)
 
 void SimpleDebuggable::write(unsigned address, byte value)
 {
-	write(address, value, motherBoard.getScheduler().getCurrentTime());
+	write(address, value, motherBoard.getCurrentTime());
 }
 
 void SimpleDebuggable::write(unsigned /*address*/, byte /*value*/,
-                             const EmuTime& /*time*/)
+                             EmuTime::param /*time*/)
 {
 	// does nothing
 }
