@@ -1,15 +1,17 @@
-// $Id: AfterCommand.cc 6623 2007-06-15 11:27:19Z m9710797 $
+// $Id: AfterCommand.cc 8377 2008-11-30 20:56:25Z manuelbi $
 
 #include "AfterCommand.hh"
 #include "CommandController.hh"
 #include "CliComm.hh"
-#include "Scheduler.hh"
 #include "Schedulable.hh"
 #include "EventDistributor.hh"
 #include "Event.hh"
 #include "Reactor.hh"
 #include "MSXMotherBoard.hh"
+#include "Alarm.hh"
+#include "EmuTime.hh"
 #include "CommandException.hh"
+#include "openmsx.hh"
 #include <cstdlib>
 #include <sstream>
 
@@ -48,7 +50,7 @@ protected:
 		      AfterCommand& afterCommand,
 		      const std::string& command, double time);
 private:
-	virtual void executeUntil(const EmuTime& time, int userData);
+	virtual void executeUntil(EmuTime::param time, int userData);
 	virtual void schedulerDeleted();
 	virtual const std::string& schedName() const;
 
@@ -85,6 +87,22 @@ private:
 	const std::string type;
 };
 
+class AfterRealTimeCmd : public AfterCmd, private Alarm
+{
+public:
+	AfterRealTimeCmd(AfterCommand& afterCommand,
+	                 EventDistributor& eventDistributor,
+	                 const std::string& command, double time);
+	virtual ~AfterRealTimeCmd();
+	virtual const std::string& getType() const;
+	bool hasExpired() const;
+
+private:
+	virtual bool alarm();
+
+	EventDistributor& eventDistributor;
+	bool expired;
+};
 
 
 AfterCommand::AfterCommand(Reactor& reactor_,
@@ -120,6 +138,8 @@ AfterCommand::AfterCommand(Reactor& reactor_,
 		OPENMSX_BOOT_EVENT, *this);
 	eventDistributor.registerEventListener(
 		OPENMSX_MACHINE_LOADED_EVENT, *this);
+	eventDistributor.registerEventListener(
+		OPENMSX_AFTER_REALTIME_EVENT, *this);
 }
 
 AfterCommand::~AfterCommand()
@@ -128,6 +148,8 @@ AfterCommand::~AfterCommand()
 		delete afterCmds.begin()->second; // removes itself from map
 	}
 
+	eventDistributor.unregisterEventListener(
+		OPENMSX_AFTER_REALTIME_EVENT, *this);
 	eventDistributor.unregisterEventListener(
 		OPENMSX_MACHINE_LOADED_EVENT, *this);
 	eventDistributor.unregisterEventListener(
@@ -171,6 +193,8 @@ string AfterCommand::execute(const vector<string>& tokens)
 	}
 	if (tokens[1] == "time") {
 		return afterTime(tokens);
+	} else if (tokens[1] == "realtime") {
+		return afterRealTime(tokens);
 	} else if (tokens[1] == "idle") {
 		return afterIdle(tokens);
 	} else if (tokens[1] == "frame") {
@@ -216,6 +240,17 @@ string AfterCommand::afterTime(const vector<string>& tokens)
 	return cmd->getId();
 }
 
+string AfterCommand::afterRealTime(const vector<string>& tokens)
+{
+	if (tokens.size() != 4) {
+		throw SyntaxError();
+	}
+	double time = getTime(tokens[2]);
+	AfterRealTimeCmd* cmd = new AfterRealTimeCmd(
+		*this, eventDistributor, tokens[3], time);
+	return cmd->getId();
+}
+
 string AfterCommand::afterIdle(const vector<string>& tokens)
 {
 	if (tokens.size() != 4) {
@@ -266,12 +301,13 @@ string AfterCommand::afterCancel(const vector<string>& tokens)
 
 string AfterCommand::help(const vector<string>& /*tokens*/) const
 {
-	return "after time <seconds> <command>  execute a command after some time\n"
-	       "after idle <seconds> <command>  execute a command after some time being idle\n"
-	       "after frame <command>           execute a command after a new frame is drawn\n"
-	       "after break <command>           execute a command after a breakpoint is reached\n"
-	       "after info                      list all postponed commands\n"
-	       "after cancel <id>               cancel the postponed command with given id\n";
+	return "after time     <seconds> <command>  execute a command after some time (MSX time)\n"
+	       "after realtime <seconds> <command>  execute a command after some time (realtime)\n"
+	       "after idle     <seconds> <command>  execute a command after some time being idle\n"
+	       "after frame <command>               execute a command after a new frame is drawn\n"
+	       "after break <command>               execute a command after a breakpoint is reached\n"
+	       "after info                          list all postponed commands\n"
+	       "after cancel <id>                   cancel the postponed command with given id\n";
 }
 
 void AfterCommand::tabCompletion(vector<string>& tokens) const
@@ -279,6 +315,7 @@ void AfterCommand::tabCompletion(vector<string>& tokens) const
 	if (tokens.size()==2) {
 		set<string> cmds;
 		cmds.insert("time");
+		cmds.insert("realtime");
 		cmds.insert("idle");
 		cmds.insert("frame");
 		cmds.insert("break");
@@ -307,6 +344,23 @@ template<EventType T> static void executeEvents(
 	}
 }
 
+void AfterCommand::executeRealTime()
+{
+	vector<AfterCmd*> tmp; // make copy because map will change
+	for (AfterCmdMap::const_iterator it = afterCmds.begin();
+	     it != afterCmds.end(); ++it) {
+		if (AfterRealTimeCmd* realtimeCmd =
+		              dynamic_cast<AfterRealTimeCmd*>(it->second)) {
+			if (realtimeCmd->hasExpired()) {
+				tmp.push_back(it->second);
+			}
+		}
+	}
+	for (vector<AfterCmd*>::iterator it = tmp.begin();
+	     it != tmp.end(); ++it) {
+		(*it)->execute();
+	}
+}
 
 bool AfterCommand::signalEvent(shared_ptr<const Event> event)
 {
@@ -318,6 +372,8 @@ bool AfterCommand::signalEvent(shared_ptr<const Event> event)
 		executeEvents<OPENMSX_BOOT_EVENT>(afterCmds);
 	} else if (event->getType() == OPENMSX_MACHINE_LOADED_EVENT) {
 		executeEvents<OPENMSX_MACHINE_LOADED_EVENT>(afterCmds);
+	} else if (event->getType() == OPENMSX_AFTER_REALTIME_EVENT) {
+		executeRealTime();
 	} else {
 		for (AfterCmdMap::const_iterator it = afterCmds.begin();
 		     it != afterCmds.end(); ++it) {
@@ -394,14 +450,15 @@ double AfterTimedCmd::getTime() const
 void AfterTimedCmd::reschedule()
 {
 	removeSyncPoint();
-	EmuTime t = getScheduler().getCurrentTime() + EmuDuration(time);
-	setSyncPoint(t);
+	setSyncPoint(getCurrentTime() + EmuDuration(time));
 }
 
-void AfterTimedCmd::executeUntil(const EmuTime& /*time*/,
-                                               int /*userData*/)
+void AfterTimedCmd::executeUntil(EmuTime::param /*time*/,
+                                 int /*userData*/)
 {
+	PRT_DEBUG("Going to execute after command....");
 	execute();
+	PRT_DEBUG("Going to execute after command.... DONE!");
 }
 
 void AfterTimedCmd::schedulerDeleted()
@@ -466,4 +523,46 @@ const string& AfterEventCmd<T>::getType() const
 	return type;
 }
 
+
+// class AfterRealTimeCmd
+
+AfterRealTimeCmd::AfterRealTimeCmd(
+		AfterCommand& afterCommand, EventDistributor& eventDistributor_,
+		const std::string& command, double time)
+	: AfterCmd(afterCommand, command)
+	, eventDistributor(eventDistributor_)
+	, expired(false)
+{
+	schedule(unsigned(time * 1000000)); // micro seconds
+}
+
+AfterRealTimeCmd::~AfterRealTimeCmd()
+{
+	prepareDelete();
+}
+
+const std::string& AfterRealTimeCmd::getType() const
+{
+	static const string type("realtime");
+	return type;
+}
+
+bool AfterRealTimeCmd::alarm()
+{
+	PRT_DEBUG("AfterRealTimeCmd::alarm()...");
+	// this runs in a different thread, so we can't directly execute the
+	// command here
+	expired = true;
+	PRT_DEBUG("AfterRealTimeCmd::alarm()... distribute AFTER REALTIME EVENT");
+	eventDistributor.distributeEvent(
+			new SimpleEvent<OPENMSX_AFTER_REALTIME_EVENT>());
+	PRT_DEBUG("AfterRealTimeCmd::alarm()... distribute AFTER REALTIME EVENT, DONE! Don't repeat.");
+	return false; // don't repeat alarm
+}
+
+bool AfterRealTimeCmd::hasExpired() const
+{
+	return expired;
+}
+
 } // namespace openmsx
