@@ -1,4 +1,4 @@
-// $Id: CliServer.cc 6789 2007-08-04 12:04:54Z m9710797 $
+// $Id: CliServer.cc 8205 2008-09-08 07:49:38Z m9710797 $
 
 #include "CliServer.hh"
 #include "GlobalCliComm.hh"
@@ -6,6 +6,7 @@
 #include "StringOp.hh"
 #include "FileOperations.hh"
 #include "MSXException.hh"
+#include "build-info.hh"
 #include <string>
 #include <memory>
 #include <sys/stat.h>
@@ -14,7 +15,7 @@
 #ifdef _WIN32
 #include <fstream>
 #include <cstdlib>
-#include <time.h>
+#include <ctime>
 #else
 #include <pwd.h>
 #endif
@@ -26,7 +27,9 @@ namespace openmsx {
 
 static string getUserName()
 {
-#ifdef _WIN32
+#if defined(_WIN32) || PLATFORM_GP2X
+	// GP2X: For some reason getpwuid() cannot be used in statically
+	//       linked applications.
 	return "default";
 #else
 	struct passwd* pw = getpwuid(getuid());
