@@ -1,4 +1,4 @@
-// $Id: Event.cc 5711 2006-09-29 11:53:12Z m9710797 $
+// $Id: Event.cc 8365 2008-11-25 19:26:40Z m9710797 $
 
 #include "Event.hh"
 #include "TclObject.hh"
@@ -38,6 +38,10 @@ bool Event::operator==(const Event& other) const
 {
 	return !(*this < other) && !(other < *this);
 }
+bool Event::operator!=(const Event& other) const
+{
+	return !(*this == other);
+}
 
 void Event::toStringImpl(TclObject& /*result*/) const
 {
