@@ -1,10 +1,9 @@
-// $Id: GlobalCliComm.cc 7312 2007-11-25 09:28:03Z m9710797 $
+// $Id: GlobalCliComm.cc 8240 2008-09-18 17:56:31Z m9710797 $
 
 #include "GlobalCliComm.hh"
 #include "CommandException.hh"
 #include "XMLElement.hh"
 #include "EventDistributor.hh"
-#include "LedEvent.hh"
 #include "CliConnection.hh"
 #include "Socket.hh"
 #include "GlobalCommandController.hh"
@@ -48,8 +47,6 @@ GlobalCliComm::GlobalCliComm(GlobalCommandController& commandController_,
 	, xmlOutput(false)
 {
 	commandController.setCliComm(this);
-
-	eventDistributor.registerEventListener(OPENMSX_LED_EVENT, *this);
 }
 
 GlobalCliComm::~GlobalCliComm()
@@ -60,8 +57,6 @@ GlobalCliComm::~GlobalCliComm()
 		delete *it;
 	}
 
-	eventDistributor.unregisterEventListener(OPENMSX_LED_EVENT, *this);
-
 	commandController.setCliComm(0);
 }
 
@@ -104,8 +99,8 @@ void GlobalCliComm::addConnection(auto_ptr<CliConnection> connection)
 }
 
 const char* const updateStr[CliComm::NUM_UPDATES] = {
-	"led", "setting", "hardware", "plug", "unplug", "media", "status", 
-		"extension", "sounddevice", "connector"
+	"led", "setting", "setting-info", "hardware", "plug", "unplug",
+	"media", "status", "extension", "sounddevice", "connector"
 };
 void GlobalCliComm::log(LogLevel level, const string& message)
 {
@@ -130,23 +125,23 @@ void GlobalCliComm::log(LogLevel level, const string& message)
 
 void GlobalCliComm::update(UpdateType type, const string& name,
                            const string& value)
-{
-	update(type, "", name, value);
-}
-
-void GlobalCliComm::update(UpdateType type, const string& machine,
-                           const string& name, const string& value)
 {
 	assert(type < NUM_UPDATES);
-	map<string, string>::iterator it = prevValues[type][machine].find(name);
-	if (it != prevValues[type][machine].end()) {
+	map<string, string>::iterator it = prevValues[type].find(name);
+	if (it != prevValues[type].end()) {
 		if (it->second == value) {
 			return;
 		}
 		it->second = value;
 	} else {
-		prevValues[type][machine][name] = value;
+		prevValues[type][name] = value;
 	}
+	updateHelper(type, "", name, value);
+}
+
+void GlobalCliComm::updateHelper(UpdateType type, const string& machine,
+                                 const string& name, const string& value)
+{
 	ScopedLock lock(sem);
 	if (!connections.empty()) {
 		string str = string("<update type=\"") + updateStr[type] + '\"';
@@ -167,29 +162,6 @@ void GlobalCliComm::update(UpdateType type, const string& machine,
 	}
 }
 
-bool GlobalCliComm::signalEvent(shared_ptr<const Event> event)
-{
-	static const string ON = "on";
-	static const string OFF = "off";
-	static map<LedEvent::Led, string> ledName;
-	static bool init = false;
-	if (!init) {
-		init = true;
-		ledName[LedEvent::POWER] = "power";
-		ledName[LedEvent::CAPS]  = "caps";
-		ledName[LedEvent::KANA]  = "kana";
-		ledName[LedEvent::PAUSE] = "pause";
-		ledName[LedEvent::TURBO] = "turbo";
-		ledName[LedEvent::FDD]   = "FDD";
-	}
-
-	assert(event->getType() == OPENMSX_LED_EVENT);
-	const LedEvent& ledEvent = static_cast<const LedEvent&>(*event);
-	update(LED, ledEvent.getMachine(), ledName[ledEvent.getLed()],
-	       ledEvent.getStatus() ? ON : OFF);
-	return true;
-}
-
 
 // class UpdateCmd
 
@@ -244,18 +216,17 @@ string UpdateCmd::help(const vector<string>& /*tokens*/) const
 void UpdateCmd::tabCompletion(vector<string>& tokens) const
 {
 	switch (tokens.size()) {
-		case 2: {
-			set<string> ops;
-			ops.insert("enable");
-			ops.insert("disable");
-			completeString(tokens, ops);
-			break;
-		}
-		case 3: {
-			set<string> types(updateStr,
-			                  updateStr + CliComm::NUM_UPDATES);
-			completeString(tokens, types);
-		}
+	case 2: {
+		set<string> ops;
+		ops.insert("enable");
+		ops.insert("disable");
+		completeString(tokens, ops);
+		break;
+	}
+	case 3: {
+		set<string> types(updateStr, updateStr + CliComm::NUM_UPDATES);
+		completeString(tokens, types);
+	}
 	}
 }
 
