@@ -1,4 +1,4 @@
-// $Id: InputEventFactory.cc 6421 2007-04-30 13:51:56Z m9710797 $
+// $Id: InputEventFactory.cc 7414 2007-12-12 22:12:56Z m9710797 $
 
 #include "InputEventFactory.hh"
 #include "InputEvents.hh"
@@ -13,26 +13,31 @@ namespace openmsx {
 
 namespace InputEventFactory {
 
-static EventPtr parseKeyEvent(const string& str)
+static EventPtr parseKeyEvent(const string& str, const int unicode)
 {
 	Keys::KeyCode keyCode = Keys::getCode(str);
 	if (keyCode == Keys::K_NONE) {
 		throw CommandException("Invalid keycode: " + str);
 	}
 	if (keyCode & Keys::KD_RELEASE) {
-		return EventPtr(new KeyUpEvent(keyCode));
+		return EventPtr(new KeyUpEvent(keyCode, unicode));
 	} else {
-		return EventPtr(new KeyDownEvent(keyCode));
+		return EventPtr(new KeyDownEvent(keyCode, unicode));
 	}
 }
 
 static EventPtr parseKeyEvent(
 		const string& str, const vector<string>& components)
 {
-	if (components.size() != 2) {
+	if (components.size() == 2) {
+		return parseKeyEvent(components[1], 0);
+	} else if ((components.size() == 3) &&
+	           (StringOp::startsWith(components[2], "unicode"))) {
+		return parseKeyEvent(components[1],
+		                     StringOp::stringToInt(components[2].substr(7)));
+	} else {
 		throw CommandException("Invalid keyboard event: " + str);
 	}
-	return parseKeyEvent(components[1]);
 }
 
 static bool upDown(const string& str)
@@ -160,7 +165,7 @@ EventPtr createInputEvent(const string& str)
 		return parseCommandEvent(str, components);
 	} else {
 		// fall back
-		return parseKeyEvent(components[0]);
+		return parseKeyEvent(components[0], 0);
 	}
 }
 
