@@ -1,4 +1,4 @@
-// $Id: InputEvents.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: InputEvents.cc 7414 2007-12-12 22:12:56Z m9710797 $
 
 #include "InputEvents.hh"
 #include "Keys.hh"
@@ -50,6 +50,9 @@ void KeyEvent::toStringImpl(TclObject& result) const
 {
 	result.addListElement("keyb");
 	result.addListElement(Keys::getName(getKeyCode()));
+	if (getUnicode() != 0) {
+		result.addListElement("unicode" + StringOp::toString(getUnicode()));
+	}
 }
 
 bool KeyEvent::lessImpl(const InputEvent& other) const
@@ -63,7 +66,7 @@ bool KeyEvent::lessImpl(const InputEvent& other) const
 // class KeyUpEvent
 
 KeyUpEvent::KeyUpEvent(Keys::KeyCode keyCode)
-	: KeyEvent(OPENMSX_KEY_UP_EVENT, keyCode, word(-1))
+	: KeyEvent(OPENMSX_KEY_UP_EVENT, keyCode, word(0))
 {
 }
 
@@ -76,7 +79,7 @@ KeyUpEvent::KeyUpEvent(Keys::KeyCode keyCode, word unicode)
 // class KeyDownEvent
 
 KeyDownEvent::KeyDownEvent(Keys::KeyCode keyCode)
-	: KeyEvent(OPENMSX_KEY_DOWN_EVENT, keyCode, word(-1))
+	: KeyEvent(OPENMSX_KEY_DOWN_EVENT, keyCode, word(0))
 {
 }
 
