@@ -1,36 +1,34 @@
-// $Id: DSKDiskImage.cc 6131 2007-03-02 12:00:24Z m9710797 $
+// $Id: DSKDiskImage.cc 8170 2008-08-21 20:36:38Z m9710797 $
 
 #include "DSKDiskImage.hh"
 #include "File.hh"
 
-using std::string;
-
 namespace openmsx {
 
-DSKDiskImage::DSKDiskImage(const string& fileName)
+DSKDiskImage::DSKDiskImage(const Filename& fileName)
 	: SectorBasedDisk(fileName)
 	, file(new File(fileName, File::PRE_CACHE))
 {
-	nbSectors = file->getSize() / SECTOR_SIZE;
+	setNbSectors(file->getSize() / SECTOR_SIZE);
 }
 
 DSKDiskImage::~DSKDiskImage()
 {
 }
 
-void DSKDiskImage::readLogicalSector(unsigned sector, byte* buf)
+void DSKDiskImage::readSectorSBD(unsigned sector, byte* buf)
 {
 	file->seek(sector * SECTOR_SIZE);
 	file->read(buf, SECTOR_SIZE);
 }
 
-void DSKDiskImage::writeLogicalSector(unsigned sector, const byte* buf)
+void DSKDiskImage::writeSectorSBD(unsigned sector, const byte* buf)
 {
 	file->seek(sector * SECTOR_SIZE);
 	file->write(buf, SECTOR_SIZE);
 }
 
-bool DSKDiskImage::writeProtected()
+bool DSKDiskImage::isWriteProtectedImpl() const
 {
 	return file->isReadOnly();
 }
