@@ -1,4 +1,4 @@
-// $Id: DiskChanger.cc 6624 2007-06-15 13:26:37Z m9710797 $
+// $Id: DiskChanger.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "DiskChanger.hh"
 #include "DummyDisk.hh"
@@ -11,14 +11,17 @@
 #include "MSXEventDistributor.hh"
 #include "InputEvents.hh"
 #include "Scheduler.hh"
+#include "MSXMotherBoard.hh"
 #include "DiskManipulator.hh"
 #include "FileContext.hh"
 #include "FileException.hh"
 #include "CommandException.hh"
-#include "MSXCliComm.hh"
+#include "CliComm.hh"
 #include "TclObject.hh"
 #include "EmuTime.hh"
 #include "checked_cast.hh"
+#include "serialize.hh"
+#include "serialize_stl.hh"
 
 using std::set;
 using std::string;
@@ -40,20 +43,18 @@ private:
 };
 
 DiskChanger::DiskChanger(const string& driveName_,
-                         CommandController& commandController,
+                         CommandController& controller_,
                          DiskManipulator& manipulator_,
-                         MSXEventDistributor* msxEventDistributor_,
-                         Scheduler* scheduler_)
-	: cliComm(commandController.getCliComm())
-	, globalSettings(commandController.getGlobalSettings())
-	, msxEventDistributor(msxEventDistributor_)
-	, scheduler(scheduler_)
+                         MSXMotherBoard* board)
+	: controller(controller_)
+	, msxEventDistributor(board ? &board->getMSXEventDistributor() : NULL)
+	, scheduler(board ? &board->getScheduler() : NULL)
 	, manipulator(manipulator_)
 	, driveName(driveName_)
-	, diskCommand(new DiskCommand(commandController, *this))
+	, diskCommand(new DiskCommand(controller, *this))
 {
 	ejectDisk();
-	manipulator.registerDrive(*this);
+	manipulator.registerDrive(*this, board);
 	if (msxEventDistributor) {
 		msxEventDistributor->registerEventListener(*this);
 	}
@@ -72,7 +73,7 @@ const string& DiskChanger::getDriveName() const
 	return driveName;
 }
 
-const string& DiskChanger::getDiskName() const
+const Filename& DiskChanger::getDiskName() const
 {
 	return disk->getName();
 }
@@ -117,7 +118,7 @@ void DiskChanger::sendChangeDiskEvent(const vector<string>& args)
 }
 
 void DiskChanger::signalEvent(
-	shared_ptr<const Event> event, const EmuTime& /*time*/)
+	shared_ptr<const Event> event, EmuTime::param /*time*/)
 {
 	if (event->getType() != OPENMSX_MSX_COMMAND_EVENT) return;
 
@@ -140,26 +141,30 @@ void DiskChanger::insertDisk(const vector<TclObject*>& args)
 	if (diskImage == "ramdsk") {
 		newDisk.reset(new RamDSKDiskImage());
 	} else {
+		Filename filename(diskImage, controller);
 		try {
 			// first try XSA
-			newDisk.reset(new XSADiskImage(diskImage));
+			newDisk.reset(new XSADiskImage(filename));
 		} catch (MSXException& e) {
 			try {
-				//First try the fake disk, because a DSK will always
-				//succeed if diskImage can be resolved
-				//It is simply stat'ed, so even a directory name
-				//can be resolved and will be accepted as dsk name
+				// First try the fake disk, because a DSK will always
+				// succeed if diskImage can be resolved
+				// It is simply stat'ed, so even a directory name
+				// can be resolved and will be accepted as dsk name
 				// try to create fake DSK from a dir on host OS
 				newDisk.reset(new DirAsDSK(
-					cliComm, globalSettings, diskImage));
+					controller.getCliComm(),
+					controller.getGlobalSettings(),
+					filename));
 			} catch (MSXException& e) {
 				// then try normal DSK
-				newDisk.reset(new DSKDiskImage(diskImage));
+				newDisk.reset(new DSKDiskImage(filename));
 			}
 		}
 	}
 	for (unsigned i = 2; i < args.size(); ++i) {
-		newDisk->applyPatch(args[i]->getString());
+		Filename filename(args[i]->getString(), controller);
+		newDisk->applyPatch(filename);
 	}
 
 	// no errors, only now replace original disk
@@ -175,7 +180,8 @@ void DiskChanger::changeDisk(std::auto_ptr<Disk> newDisk)
 {
 	disk = newDisk;
 	diskChangedFlag = true;
-	cliComm.update(CliComm::MEDIA, getDriveName(), getDiskName());
+	controller.getCliComm().update(CliComm::MEDIA, getDriveName(),
+	                               getDiskName().getResolved());
 }
 
 
@@ -193,7 +199,7 @@ void DiskCommand::execute(const vector<TclObject*>& tokens, TclObject& result)
 	int firstFileToken = 1;
 	if (tokens.size() == 1) {
 		result.addListElement(diskChanger.getDriveName() + ':');
-		result.addListElement(diskChanger.getDiskName());
+		result.addListElement(diskChanger.getDiskName().getResolved());
 
 		TclObject options(result.getInterpreter());
 		if (dynamic_cast<DummyDisk*>(diskChanger.disk.get())) {
@@ -203,7 +209,7 @@ void DiskCommand::execute(const vector<TclObject*>& tokens, TclObject& result)
 		} else if (dynamic_cast<RamDSKDiskImage*>(diskChanger.disk.get())) {
 			options.addListElement("ramdsk");
 		}
-		if (diskChanger.disk->writeProtected()) {
+		if (diskChanger.disk->isWriteProtected()) {
 			options.addListElement("readonly");
 		}
 		if (options.getListLength() != 0) {
@@ -243,12 +249,20 @@ void DiskCommand::execute(const vector<TclObject*>& tokens, TclObject& result)
 			}
 		}
 		try {
-			UserFileContext context(getCommandController());
 			vector<string> args;
 			args.push_back(diskChanger.getDriveName());
 			for (unsigned i = firstFileToken; i < tokens.size(); ++i) {
-				args.push_back(context.resolve(
-					tokens[i]->getString()));
+				string option = tokens[i]->getString();
+				if (option == "-ips") {
+					if (++i == tokens.size()) {
+						throw MSXException(
+							"Missing argument for option \"" + option + "\"");
+					}
+					args.push_back(tokens[i]->getString());
+				} else {
+					// backwards compatibility
+					args.push_back(option);
+				}
 			}
 			diskChanger.sendChangeDiskEvent(args);
 		} catch (FileException& e) {
@@ -274,9 +288,81 @@ void DiskCommand::tabCompletion(vector<string>& tokens) const
 		extra.insert("eject");
 		extra.insert("ramdsk");
 		extra.insert("insert");
-		UserFileContext context(getCommandController());
-		completeFileName(tokens, context, extra);
+		UserFileContext context;
+		completeFileName(getCommandController(), tokens, context, extra);
 	}
 }
 
+static string calcSha1(SectorAccessibleDisk* disk)
+{
+	return disk ? disk->getSHA1Sum() : "";
+}
+
+template<typename Archive>
+void DiskChanger::serialize(Archive& ar, unsigned /*version*/)
+{
+	Filename diskname = disk->getName();
+	ar.serializeNoID("disk", diskname);
+
+	vector<Filename> patches;
+	if (!ar.isLoader()) {
+		disk->getPatches(patches);
+	}
+	ar.serializeNoID("patches", patches);
+
+	if (ar.isLoader()) {
+		diskname.updateAfterLoadState(controller);
+		string name = diskname.getResolved(); // TODO use Filename
+		if (!name.empty()) {
+			vector<TclObject> objs;
+			objs.push_back(TclObject("dummy"));
+			objs.push_back(TclObject(name));
+			for (vector<Filename>::iterator it = patches.begin();
+			     it != patches.end(); ++it) {
+				it->updateAfterLoadState(controller);
+				objs.push_back(TclObject(it->getResolved())); // TODO
+			}
+
+			vector<TclObject*> args;
+			for (vector<TclObject>::iterator it = objs.begin();
+			     it != objs.end(); ++it) {
+				args.push_back(&(*it));
+			}
+
+			try {
+				insertDisk(args);
+			} catch (MSXException& e) {
+				throw MSXException(
+					"Couldn't reinsert disk in drive " +
+					getDriveName() + ": " + e.getMessage());
+				// Alternative: Print warning and continue
+				//   without diskimage. Is this better?
+			}
+		}
+	}
+
+	ar.serialize("diskChanged", diskChangedFlag);
+
+	string oldChecksum;
+	if (!ar.isLoader()) {
+		oldChecksum = calcSha1(getSectorAccessibleDisk());
+	}
+	ar.serialize("checksum", oldChecksum);
+	if (ar.isLoader()) {
+		string newChecksum = calcSha1(getSectorAccessibleDisk());
+		if (oldChecksum != newChecksum) {
+			controller.getCliComm().printWarning(
+				"The content of the diskimage " +
+				diskname.getResolved() +
+				" has changed since the time this savestate was "
+				"created. This might result in emulation problems "
+				"or even diskcorruption. To prevent the latter, "
+				"the disk is now write-protected (eject and "
+				"reinsert the disk if you want to override this).");
+			disk->forceWriteProtect();
+		}
+	}
+}
+INSTANTIATE_SERIALIZE_METHODS(DiskChanger);
+
 } // namespace openmsx
