@@ -1,7 +1,7 @@
-// $Id: DiskDrive.cc 5756 2006-10-05 21:34:18Z m9710797 $
+// $Id: DiskDrive.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "DiskDrive.hh"
-#include "Disk.hh"
+#include "DiskExceptions.hh"
 
 namespace openmsx {
 
@@ -14,68 +14,68 @@ DiskDrive::~DiskDrive()
 
 // class DummyDrive
 
-bool DummyDrive::ready()
+bool DummyDrive::isReady() const
 {
 	return false;
 }
 
-bool DummyDrive::writeProtected()
+bool DummyDrive::isWriteProtected() const
 {
 	return true;
 }
 
-bool DummyDrive::doubleSided()
+bool DummyDrive::isDoubleSided() const
 {
 	return false;
 }
 
+bool DummyDrive::isTrack00() const
+{
+	return false; // National_FS-5500F1 2nd drive detection depends on this
+}
+
 void DummyDrive::setSide(bool /*side*/)
 {
 	// ignore
 }
 
-void DummyDrive::step(bool /*direction*/, const EmuTime& /*time*/)
+void DummyDrive::step(bool /*direction*/, EmuTime::param /*time*/)
 {
 	// ignore
 }
 
-bool DummyDrive::track00(const EmuTime& /*time*/)
-{
-	return false; // National_FS-5500F1 2nd drive detection depends on this
-}
-
-void DummyDrive::setMotor(bool /*status*/, const EmuTime& /*time*/)
+void DummyDrive::setMotor(bool /*status*/, EmuTime::param /*time*/)
 {
 	// ignore
 }
 
-bool DummyDrive::indexPulse(const EmuTime& /*time*/)
+bool DummyDrive::indexPulse(EmuTime::param /*time*/)
 {
 	return false;
 }
 
-int DummyDrive::indexPulseCount(const EmuTime& /*begin*/,
-                                const EmuTime& /*end*/)
+int DummyDrive::indexPulseCount(EmuTime::param /*begin*/,
+                                EmuTime::param /*end*/)
 {
 	return 0;
 }
 
-EmuTime DummyDrive::getTimeTillSector(byte /*sector*/, const EmuTime& time)
+EmuTime DummyDrive::getTimeTillSector(byte /*sector*/, EmuTime::param time)
 {
 	return time;
 }
 
-EmuTime DummyDrive::getTimeTillIndexPulse(const EmuTime& time)
+EmuTime DummyDrive::getTimeTillIndexPulse(EmuTime::param time)
 {
 	return time;
 }
 
-void DummyDrive::setHeadLoaded(bool /*status*/, const EmuTime& /*time*/)
+void DummyDrive::setHeadLoaded(bool /*status*/, EmuTime::param /*time*/)
 {
 	// ignore
 }
 
-bool DummyDrive::headLoaded(const EmuTime& /*time*/)
+bool DummyDrive::headLoaded(EmuTime::param /*time*/)
 {
 	return false;
 }
@@ -104,12 +104,7 @@ void DummyDrive::getTrackHeader(byte* /*buf*/)
 	throw DriveEmptyException("No drive connected");
 }
 
-void DummyDrive::initWriteTrack()
-{
-	// ignore ???
-}
-
-void DummyDrive::writeTrackData(byte /*data*/)
+void DummyDrive::writeTrackData(const byte* /*data*/)
 {
 	// ignore ???
 }
@@ -124,7 +119,7 @@ bool DummyDrive::peekDiskChanged() const
 	return false;
 }
 
-bool DummyDrive::dummyDrive()
+bool DummyDrive::isDummyDrive() const
 {
 	return true;
 }
