@@ -1,19 +1,18 @@
-// $Id: DiskImageCLI.cc 6131 2007-03-02 12:00:24Z m9710797 $
+// $Id: DiskImageCLI.cc 7731 2008-04-03 19:58:10Z m9710797 $
 
 #include "DiskImageCLI.hh"
-#include "Reactor.hh"
 #include "CommandController.hh"
 #include "CommandLineParser.hh"
 #include "TclObject.hh"
 #include "MSXException.hh"
 
-using std::list;
+using std::deque;
 using std::string;
 
 namespace openmsx {
 
 DiskImageCLI::DiskImageCLI(CommandLineParser& commandLineParser)
-	: commandController(commandLineParser.getReactor().getCommandController())
+	: commandController(commandLineParser.getCommandController())
 {
 	commandLineParser.registerOption("-diska", *this);
 	commandLineParser.registerOption("-diskb", *this);
@@ -22,8 +21,7 @@ DiskImageCLI::DiskImageCLI(CommandLineParser& commandLineParser)
 	driveLetter = 'a';
 }
 
-bool DiskImageCLI::parseOption(const string& option,
-                               list<string>& cmdLine)
+bool DiskImageCLI::parseOption(const string& option, deque<string>& cmdLine)
 {
 	string filename = getArgument(option, cmdLine);
 	parse(option.substr(1), filename, cmdLine);
@@ -35,8 +33,7 @@ const string& DiskImageCLI::optionHelp() const
 	return text;
 }
 
-void DiskImageCLI::parseFileType(const string& filename,
-                                 list<string>& cmdLine)
+void DiskImageCLI::parseFileType(const string& filename, deque<string>& cmdLine)
 {
 	parse(string("disk") + driveLetter, filename, cmdLine);
 	++driveLetter;
@@ -49,7 +46,7 @@ const string& DiskImageCLI::fileTypeHelp() const
 }
 
 void DiskImageCLI::parse(const string& drive, const string& image,
-                         list<string>& cmdLine)
+                         deque<string>& cmdLine)
 {
 	if (!commandController.hasCommand(drive)) { // TODO WIP
 		throw MSXException("No drive named '" + drive + "'.");
