@@ -1,13 +1,17 @@
-// $Id: DiskManipulator.cc 5881 2006-11-15 17:08:34Z m9710797 $
+// $Id: DiskManipulator.cc 8201 2008-09-04 17:06:27Z m9710797 $
 
 #include "DiskManipulator.hh"
 #include "DiskContainer.hh"
 #include "DiskChanger.hh"
 #include "MSXtar.hh"
+#include "DiskImageUtils.hh"
 #include "DSKDiskImage.hh"
 #include "CommandController.hh"
 #include "CommandException.hh"
+#include "MSXMotherBoard.hh"
+#include "Reactor.hh"
 #include "File.hh"
+#include "Filename.hh"
 #include "FileContext.hh"
 #include "FileException.hh"
 #include "FileOperations.hh"
@@ -19,68 +23,70 @@
 using std::set;
 using std::string;
 using std::vector;
+using std::auto_ptr;
 
 namespace openmsx {
 
 DiskManipulator::DiskManipulator(CommandController& commandController)
 	: SimpleCommand(commandController, "diskmanipulator")
+	, virtualDrive(new DiskChanger("virtual_drive", commandController,
+	                               *this, NULL))
 {
-	virtualDrive.reset(new DiskChanger("virtual_drive", commandController,
-	                                   *this));
 }
 
 DiskManipulator::~DiskManipulator()
 {
 	virtualDrive.reset();
-
-	assert(diskImages.empty()); // all DiskContainers must be unregistered
+	assert(drives.empty()); // all DiskContainers must be unregistered
 }
 
-
-
-void DiskManipulator::registerDrive(DiskContainer& drive)
+string DiskManipulator::getMachinePrefix() const
 {
-	assert(findDriveSettings(drive) == diskImages.end());
+	string id = getCommandController().getReactor().getMachineID();
+	return id.empty() ? id : id + "::";
+}
+
+void DiskManipulator::registerDrive(DiskContainer& drive, MSXMotherBoard* board)
+{
+	assert(findDriveSettings(drive) == drives.end());
 	DriveSettings driveSettings;
 	driveSettings.drive = &drive;
+	string prefix = board ? (board->getMachineID() + "::") : "";
+	driveSettings.driveName = prefix + drive.getContainerName();
 	driveSettings.partition = 0;
-	for (int i = 0; i < 31; ++i) {
+	for (int i = 0; i < 32; ++i) {
 		driveSettings.workingDir[i] = "/";
 	}
-	diskImages.push_back(driveSettings);
+	drives.push_back(driveSettings);
 }
 
 void DiskManipulator::unregisterDrive(DiskContainer& drive)
 {
-	DiskImages::iterator it = findDriveSettings(drive);
-	assert(it != diskImages.end());
-	diskImages.erase(it);
+	Drives::iterator it = findDriveSettings(drive);
+	assert(it != drives.end());
+	drives.erase(it);
 }
 
-DiskManipulator::DiskImages::iterator DiskManipulator::findDriveSettings(
+DiskManipulator::Drives::iterator DiskManipulator::findDriveSettings(
 	DiskContainer& drive)
 {
-	for (DiskImages::iterator it = diskImages.begin();
-	     it != diskImages.end(); ++it) {
+	for (Drives::iterator it = drives.begin(); it != drives.end(); ++it) {
 		if (it->drive == &drive) {
 			return it;
 		}
 	}
-	return diskImages.end();
+	return drives.end();
 }
 
-DiskManipulator::DiskImages::iterator DiskManipulator::findDriveSettings(
+DiskManipulator::Drives::iterator DiskManipulator::findDriveSettings(
 	const string& name)
 {
-	// there might be multiple with the same name, pick the first
-	// improve this once we have multiple simultaneous machines
-	for (DiskImages::iterator it = diskImages.begin();
-	     it != diskImages.end(); ++it) {
-		if (it->drive->getContainerName() == name) {
+	for (Drives::iterator it = drives.begin(); it != drives.end(); ++it) {
+		if (it->driveName == name) {
 			return it;
 		}
 	}
-	return diskImages.end();
+	return drives.end();
 }
 
 DiskManipulator::DriveSettings& DiskManipulator::getDriveSettings(
@@ -88,33 +94,44 @@ DiskManipulator::DriveSettings& DiskManipulator::getDriveSettings(
 {
 	// first split of the end numbers if present
 	// these will be used as partition indication
-	string::size_type pos = diskname.find_first_of("0123456789");
+	string::size_type pos = diskname.find("::");
+	pos = diskname.find_first_of("0123456789", ((pos != string::npos) ? pos : 0));
 	string tmp = diskname.substr(0, pos);
 
-	DiskImages::iterator it = findDriveSettings(tmp);
-	if (it == diskImages.end()) {
-		throw CommandException("Unknown drive: "  + tmp);
-	}
-
-	it->partition = 0;
-	if (pos != string::npos) {
-		int i = strtol(diskname.substr(pos).c_str(), NULL, 10);
-		if (i <= 0 || i > 31) {
-			throw CommandException("Invalid partition specified.");
+	Drives::iterator it = findDriveSettings(tmp);
+	if (it == drives.end()) {
+		it = findDriveSettings(getMachinePrefix() + tmp);
+		if (it == drives.end()) {
+			throw CommandException("Unknown drive: "  + tmp);
 		}
-		it->partition = i - 1;
 	}
-	return *it;
-}
 
-SectorAccessibleDisk& DiskManipulator::getDisk(const DriveSettings& driveData)
-{
-	SectorAccessibleDisk* disk = driveData.drive->getSectorAccessibleDisk();
+	SectorAccessibleDisk* disk = it->drive->getSectorAccessibleDisk();
 	if (!disk) {
 		// not a SectorBasedDisk
 		throw CommandException("Unsupported disk type.");
 	}
-	return *disk;
+
+	if (pos == string::npos) {
+		// whole disk
+		it->partition = 0;
+	} else {
+		string num = diskname.substr(pos);
+		SectorAccessibleDisk* disk =
+			it->drive->getSectorAccessibleDisk();
+		int partition = strtol(num.c_str(), NULL, 10);
+		DiskImageUtils::checkValidPartition(*disk, partition);
+		it->partition = partition;
+	}
+	return *it;
+}
+
+auto_ptr<SectorAccessibleDisk> DiskManipulator::getPartition(
+	const DriveSettings& driveData)
+{
+	SectorAccessibleDisk* disk = driveData.drive->getSectorAccessibleDisk();
+	assert(disk);
+	return DiskImageUtils::getPartition(*disk, driveData.partition);
 }
 
 
@@ -125,13 +142,13 @@ string DiskManipulator::execute(const vector<string>& tokens)
 		throw CommandException("Missing argument");
 
 	} else if ((tokens.size() != 4 && ( tokens[1] == "savedsk"
-	                                || tokens[1] == "mkdir"
-	                                || tokens[1] == "export"))
+	                                || tokens[1] == "mkdir"))
 	        || (tokens.size() != 3 && (tokens[1] == "dir"
 	                                || tokens[1] == "format"))
 	        || ((tokens.size() < 3 || tokens.size() > 4) &&
 	                                  (tokens[1] == "chdir"))
-	        || (tokens.size() < 4 && tokens[1] == "import")
+	        || (tokens.size() < 4 && ( tokens[1] == "export"
+					|| tokens[1] == "import"))
 	        || (tokens.size() <= 3 && (tokens[1] == "create"))) {
 		throw CommandException("Incorrect number of parameters");
 
@@ -140,7 +157,8 @@ string DiskManipulator::execute(const vector<string>& tokens)
 			throw CommandException(tokens[3] + " is not a directory");
 		}
 		DriveSettings& settings = getDriveSettings(tokens[2]);
-		exprt (settings, tokens[3]);
+		vector<string> lists(tokens.begin() + 4, tokens.end());
+		exprt(settings, tokens[3], lists);
 
 	} else if (tokens[1] == "import" ) {
 		DriveSettings& settings = getDriveSettings(tokens[2]);
@@ -269,29 +287,29 @@ void DiskManipulator::tabCompletion(vector<string>& tokens) const
 		completeString(tokens, cmds);
 
 	} else if ((tokens.size() == 3) && (tokens[1] == "create")) {
-		UserFileContext context(getCommandController());
-		completeFileName(tokens, context);
+		UserFileContext context;
+		completeFileName(getCommandController(), tokens, context);
 
 	} else if (tokens.size() == 3) {
 		set<string> names;
-		for (DiskImages::const_iterator it = diskImages.begin();
-		     it != diskImages.end(); ++it) {
-			string name = it->drive->getContainerName();
-			names.insert(name);
+		for (Drives::const_iterator it = drives.begin();
+		     it != drives.end(); ++it) {
+			string name1 = it->driveName; // with prexix
+			string name2 = it->drive->getContainerName(); // without prefix
+			names.insert(name1);
+			names.insert(name2);
 			// if it has partitions then we also add the partition
 			// numbers to the autocompletion
-			SectorAccessibleDisk* sectorDisk =
-				it->drive->getSectorAccessibleDisk();
-			if (sectorDisk != NULL) {
-				try {
-					MSXtar workhorse(*sectorDisk);
-					for (int i = 0; i < 31; ++i) {
-						if (workhorse.hasPartition(i)) {
-							names.insert(name + StringOp::toString(i + 1));
-						}
+			if (SectorAccessibleDisk* disk =
+				it->drive->getSectorAccessibleDisk()) {
+				for (int i = 1; i <= 31; ++i) {
+					try {
+						DiskImageUtils::checkValidPartition(*disk, i);
+						names.insert(name1 + StringOp::toString(i));
+						names.insert(name2 + StringOp::toString(i));
+					} catch (MSXException& e) {
+						// skip invalid partition
 					}
-				} catch (MSXException& e) {
-					// ignore
 				}
 			}
 		}
@@ -301,8 +319,8 @@ void DiskManipulator::tabCompletion(vector<string>& tokens) const
 		if ((tokens[1] == "savedsk") ||
 		    (tokens[1] == "import")  ||
 		    (tokens[1] == "export")) {
-			UserFileContext context(getCommandController());
-			completeFileName(tokens, context);
+			UserFileContext context;
+			completeFileName(getCommandController(), tokens, context);
 		} else if (tokens[1] == "create") {
 			set<string> cmds;
 			cmds.insert("360");
@@ -316,12 +334,12 @@ void DiskManipulator::tabCompletion(vector<string>& tokens) const
 void DiskManipulator::savedsk(const DriveSettings& driveData,
                               const string& filename)
 {
-	SectorAccessibleDisk& disk = getDisk(driveData);
-	unsigned nrsectors = disk.getNbSectors();
+	auto_ptr<SectorAccessibleDisk> partition = getPartition(driveData);
+	unsigned nrsectors = partition->getNbSectors();
 	byte buf[SectorBasedDisk::SECTOR_SIZE];
 	File file(filename, File::CREATE);
 	for (unsigned i = 0; i < nrsectors; ++i) {
-		disk.readLogicalSector(i, buf);
+		partition->readSector(i, buf);
 		file.write(buf, SectorBasedDisk::SECTOR_SIZE);
 	}
 }
@@ -331,6 +349,10 @@ void DiskManipulator::create(const vector<string>& tokens)
 	vector<unsigned> sizes;
 	unsigned totalSectors = 0;
 	for (unsigned i = 3; i < tokens.size(); ++i) {
+		if (sizes.size() >= 31) {
+			throw CommandException(
+				"Maximum number of partitions is 31.");
+		}
 		char* q;
 		int sectors = strtol(tokens[i].c_str(), &q, 0);
 		int scale = 1024; // default is kilobytes
@@ -379,50 +401,42 @@ void DiskManipulator::create(const vector<string>& tokens)
 	}
 
 	// create file with correct size
+	Filename filename(tokens[2]);
 	try {
-		File file(tokens[2], File::CREATE);
+		File file(filename, File::CREATE);
 		file.truncate(totalSectors * SectorBasedDisk::SECTOR_SIZE);
 	} catch (FileException& e) {
 		throw CommandException("Couldn't create image: " + e.getMessage());
 	}
 
 	// initialize (create partition tables and format partitions)
-	DSKDiskImage image(tokens[2]);
-	MSXtar workhorse(image);
-	workhorse.createDiskFile(sizes);
+	DSKDiskImage image(filename);
+	if (sizes.size() > 1) {
+		DiskImageUtils::partition(image, sizes);
+	} else {
+		// only one partition specified, don't create partition table
+		DiskImageUtils::format(image);
+	}
 }
 
 void DiskManipulator::format(DriveSettings& driveData)
 {
-	MSXtar workhorse(getDisk(driveData));
-	try {
-		/*bool partitionExists = */workhorse.usePartition(driveData.partition);
-	} catch (MSXException& e) {
-		// ignore, because only the partition selection part is
-		// interesting for the format command, so ignore other
-		// errors (see MSXtar code)
-	}
-	// TODO: check if the partition actually exists (now: use return value)
-	// doesn't work yet, because we can't discern exception error or return
-	// value error... (exception prevents setting of retval)
-	//if (!partitionExists) {
-	//	throw MSXException("The selected partition " + StringOp::toString(driveData.partition) +  " does not exist, can't format");
-	//}
-	workhorse.format();
+	auto_ptr<SectorAccessibleDisk> partition = getPartition(driveData);
+	DiskImageUtils::format(*partition);
 	driveData.workingDir[driveData.partition] = "/";
 }
 
-void DiskManipulator::restoreCWD(MSXtar& workhorse, DriveSettings& driveData)
+auto_ptr<MSXtar> DiskManipulator::getMSXtar(
+	SectorAccessibleDisk& disk, DriveSettings& driveData)
 {
-	if (!workhorse.hasPartitionTable()) {
-		workhorse.usePartition(0); //read the bootsector
-	} else if (!workhorse.usePartition(driveData.partition)) {
+	if (DiskImageUtils::hasPartitionTable(disk)) {
 		throw CommandException(
-		    "Partition " + StringOp::toString(1 + driveData.partition) +
-		    " doesn't exist on this device. Command aborted, please retry.");
+			"Please select partition number.");
 	}
+
+	auto_ptr<MSXtar> result(new MSXtar(disk));
 	try {
-		workhorse.chdir(driveData.workingDir[driveData.partition]);
+		result->chdir(driveData.workingDir[driveData.partition]);
 	} catch (MSXException& e) {
 		driveData.workingDir[driveData.partition] = "/";
 		throw CommandException(
@@ -430,22 +444,23 @@ void DiskManipulator::restoreCWD(MSXtar& workhorse, DriveSettings& driveData)
 		    " doesn't exist anymore. Went back to root "
 		    "directory. Command aborted, please retry.");
 	}
+	return result;
 }
 
 void DiskManipulator::dir(DriveSettings& driveData, string& result)
 {
-	MSXtar workhorse(getDisk(driveData));
-	restoreCWD(workhorse, driveData);
-	result += workhorse.dir();
+	auto_ptr<SectorAccessibleDisk> partition = getPartition(driveData);
+	auto_ptr<MSXtar> workhorse = getMSXtar(*partition, driveData);
+	result += workhorse->dir();
 }
 
 void DiskManipulator::chdir(DriveSettings& driveData,
                             const string& filename, string& result)
 {
-	MSXtar workhorse(getDisk(driveData));
-	restoreCWD(workhorse, driveData);
+	auto_ptr<SectorAccessibleDisk> partition = getPartition(driveData);
+	auto_ptr<MSXtar> workhorse = getMSXtar(*partition, driveData);
 	try {
-		workhorse.chdir(filename);
+		workhorse->chdir(filename);
 	} catch (MSXException& e) {
 		throw CommandException("chdir failed: " + e.getMessage());
 	}
@@ -462,10 +477,10 @@ void DiskManipulator::chdir(DriveSettings& driveData,
 
 void DiskManipulator::mkdir(DriveSettings& driveData, const string& filename)
 {
-	MSXtar workhorse(getDisk(driveData));
-	restoreCWD(workhorse, driveData);
+	auto_ptr<SectorAccessibleDisk> partition = getPartition(driveData);
+	auto_ptr<MSXtar> workhorse = getMSXtar(*partition, driveData);
 	try {
-		workhorse.mkdir(filename);
+		workhorse->mkdir(filename);
 	} catch (MSXException& e) {
 		throw CommandException(e.getMessage());
 	}
@@ -474,10 +489,10 @@ void DiskManipulator::mkdir(DriveSettings& driveData, const string& filename)
 string DiskManipulator::import(DriveSettings& driveData,
                              const vector<string>& lists)
 {
-	string messages;
-	MSXtar workhorse(getDisk(driveData));
-	restoreCWD(workhorse, driveData);
+	auto_ptr<SectorAccessibleDisk> partition = getPartition(driveData);
+	auto_ptr<MSXtar> workhorse = getMSXtar(*partition, driveData);
 
+	string messages;
 	for (vector<string>::const_iterator it = lists.begin();
 	     it != lists.end(); ++it) {
 		vector<string> list;
@@ -487,9 +502,9 @@ string DiskManipulator::import(DriveSettings& driveData,
 		     it != list.end(); ++it) {
 			try {
 				if (FileOperations::isDirectory(*it)) {
-					messages += workhorse.addDir(*it);
+					messages += workhorse->addDir(*it);
 				} else if (FileOperations::isRegularFile(*it)) {
-					messages += workhorse.addFile(*it);
+					messages += workhorse->addFile(*it);
 				} else {
 					// ignore other stuff (sockets, device nodes, ..)
 					messages += "Ignoring " + *it + '\n';
@@ -502,12 +517,21 @@ string DiskManipulator::import(DriveSettings& driveData,
 	return messages;
 }
 
-void DiskManipulator::exprt(DriveSettings& driveData, const string& dirname)
+void DiskManipulator::exprt(DriveSettings& driveData, const string& dirname,
+                            const vector<string>& lists)
 {
+	auto_ptr<SectorAccessibleDisk> partition = getPartition(driveData);
+	auto_ptr<MSXtar> workhorse = getMSXtar(*partition, driveData);
 	try {
-		MSXtar workhorse(getDisk(driveData));
-		restoreCWD(workhorse, driveData);
-		workhorse.getDir(dirname);
+		if (lists.empty()) {
+			// export all
+			workhorse->getDir(dirname);
+		} else {
+			for (vector<string>::const_iterator it = lists.begin();
+			     it != lists.end(); ++it) {
+				workhorse->getItemFromDir(dirname, *it);
+			}
+		}
 	} catch (MSXException& e) {
 		throw CommandException(e.getMessage());
 	}
