@@ -1,6 +1,7 @@
-// $Id: DriveMultiplexer.cc 4839 2005-11-11 22:00:07Z m9710797 $
+// $Id: DriveMultiplexer.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "DriveMultiplexer.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
@@ -21,7 +22,7 @@ DriveMultiplexer::~DriveMultiplexer()
 	delete drive[NO_DRIVE];
 }
 
-void DriveMultiplexer::selectDrive(DriveNum num, const EmuTime& time)
+void DriveMultiplexer::selectDrive(DriveNum num, EmuTime::param time)
 {
 	if (selected != num) {
 		drive[selected]->setMotor(false, time);
@@ -31,19 +32,19 @@ void DriveMultiplexer::selectDrive(DriveNum num, const EmuTime& time)
 	}
 }
 
-bool DriveMultiplexer::ready()
+bool DriveMultiplexer::isReady() const
 {
-	return drive[selected]->ready();
+	return drive[selected]->isReady();
 }
 
-bool DriveMultiplexer::writeProtected()
+bool DriveMultiplexer::isWriteProtected() const
 {
-	return drive[selected]->writeProtected();
+	return drive[selected]->isWriteProtected();
 }
 
-bool DriveMultiplexer::doubleSided()
+bool DriveMultiplexer::isDoubleSided() const
 {
-	return drive[selected]->doubleSided();
+	return drive[selected]->isDoubleSided();
 }
 
 void DriveMultiplexer::setSide(bool side_)
@@ -52,49 +53,49 @@ void DriveMultiplexer::setSide(bool side_)
 	drive[selected]->setSide(side);
 }
 
-void DriveMultiplexer::step(bool direction, const EmuTime& time)
+void DriveMultiplexer::step(bool direction, EmuTime::param time)
 {
 	drive[selected]->step(direction, time);
 }
 
-bool DriveMultiplexer::track00(const EmuTime& time)
+bool DriveMultiplexer::isTrack00() const
 {
-	return drive[selected]->track00(time);
+	return drive[selected]->isTrack00();
 }
 
-void DriveMultiplexer::setMotor(bool status, const EmuTime& time)
+void DriveMultiplexer::setMotor(bool status, EmuTime::param time)
 {
 	motor = status;
 	drive[selected]->setMotor(status, time);
 }
 
-bool DriveMultiplexer::indexPulse(const EmuTime& time)
+bool DriveMultiplexer::indexPulse(EmuTime::param time)
 {
 	return drive[selected]->indexPulse(time);
 }
 
-int DriveMultiplexer::indexPulseCount(const EmuTime& begin,
-                                      const EmuTime& end)
+int DriveMultiplexer::indexPulseCount(EmuTime::param begin,
+                                      EmuTime::param end)
 {
 	return drive[selected]->indexPulseCount(begin, end);
 }
 
-EmuTime DriveMultiplexer::getTimeTillSector(byte sector, const EmuTime& time)
+EmuTime DriveMultiplexer::getTimeTillSector(byte sector, EmuTime::param time)
 {
 	return drive[selected]->getTimeTillSector(sector, time);
 }
 
-EmuTime DriveMultiplexer::getTimeTillIndexPulse(const EmuTime& time)
+EmuTime DriveMultiplexer::getTimeTillIndexPulse(EmuTime::param time)
 {
 	return drive[selected]->getTimeTillIndexPulse(time);
 }
 
-void DriveMultiplexer::setHeadLoaded(bool status, const EmuTime& time)
+void DriveMultiplexer::setHeadLoaded(bool status, EmuTime::param time)
 {
 	drive[selected]->setHeadLoaded(status, time);
 }
 
-bool DriveMultiplexer::headLoaded(const EmuTime& time)
+bool DriveMultiplexer::headLoaded(EmuTime::param time)
 {
 	return drive[selected]->headLoaded(time);
 }
@@ -125,12 +126,7 @@ void DriveMultiplexer::getTrackHeader(byte* buf)
 	drive[selected]->getTrackHeader(buf);
 }
 
-void DriveMultiplexer::initWriteTrack()
-{
-	drive[selected]->initWriteTrack();
-}
-
-void DriveMultiplexer::writeTrackData(byte data)
+void DriveMultiplexer::writeTrackData(const byte* data)
 {
 	drive[selected]->writeTrackData(data);
 }
@@ -145,9 +141,28 @@ bool DriveMultiplexer::peekDiskChanged() const
 	return drive[selected]->peekDiskChanged();
 }
 
-bool DriveMultiplexer::dummyDrive()
+bool DriveMultiplexer::isDummyDrive() const
 {
-	return drive[selected]->dummyDrive();
+	return drive[selected]->isDummyDrive();
 }
 
+
+static enum_string<DriveMultiplexer::DriveNum> driveNumInfo[] = {
+	{ "A",    DriveMultiplexer::DRIVE_A },
+	{ "B",    DriveMultiplexer::DRIVE_B },
+	{ "C",    DriveMultiplexer::DRIVE_C },
+	{ "D",    DriveMultiplexer::DRIVE_D },
+	{ "none", DriveMultiplexer::NO_DRIVE }
+};
+SERIALIZE_ENUM(DriveMultiplexer::DriveNum, driveNumInfo);
+
+template<typename Archive>
+void DriveMultiplexer::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.serialize("selected", selected);
+	ar.serialize("motor", motor);
+	ar.serialize("side", side);
+}
+INSTANTIATE_SERIALIZE_METHODS(DriveMultiplexer);
+
 } // namespace openmsx
