@@ -1,4 +1,4 @@
-// $Id: EmptyDiskPatch.cc 5074 2006-01-24 18:03:35Z m9710797 $
+// $Id: EmptyDiskPatch.cc 8170 2008-08-21 20:36:38Z m9710797 $
 
 #include "EmptyDiskPatch.hh"
 #include "SectorBasedDisk.hh"
@@ -15,7 +15,7 @@ void EmptyDiskPatch::copyBlock(unsigned src, byte* dst, unsigned num) const
 {
 	(void)num;
 	assert(num == SectorBasedDisk::SECTOR_SIZE);
-	disk.readLogicalSector(src / SectorBasedDisk::SECTOR_SIZE, dst);
+	disk.readSectorSBD(src / SectorBasedDisk::SECTOR_SIZE, dst);
 }
 
 unsigned EmptyDiskPatch::getSize() const
@@ -23,5 +23,10 @@ unsigned EmptyDiskPatch::getSize() const
 	return disk.getNbSectors() * SectorBasedDisk::SECTOR_SIZE;
 }
 
+void EmptyDiskPatch::getFilenames(std::vector<Filename>& /*result*/) const
+{
+	// nothing
+}
+
 } // namespace openmsx
 
