@@ -1,4 +1,4 @@
-// $Id: MSXtar.cc 7401 2007-12-09 21:02:23Z mthuurne $
+// $Id: MSXtar.cc 8351 2008-11-12 18:20:29Z m9710797 $
 
 #include "MSXtar.hh"
 #include "ReadDir.hh"
@@ -6,22 +6,22 @@
 #include "FileOperations.hh"
 #include "MSXException.hh"
 #include "StringOp.hh"
-#include "BootBlocks.hh"
-#include <sys/stat.h>
-#include <time.h>
-#include <utime.h>
-#include <unistd.h>
-#include <cstdlib>
+#include "File.hh"
 #include <cstring>
 #include <algorithm>
 #include <cassert>
+#include <ctime>
+#include <utime.h>
+#include <sys/stat.h>
+#include <ctype.h>
 
 using std::string;
 
 namespace openmsx {
 
-static const unsigned EOF_FAT = 0x0FFF; // signals EOF in FAT12
-static const unsigned SECTOR_SIZE = 512;
+static const unsigned BAD_FAT = 0xFF7;
+static const unsigned EOF_FAT = 0xFFF; // actually 0xFF8-0xFFF, signals EOF in FAT12
+static const unsigned SECTOR_SIZE = SectorAccessibleDisk::SECTOR_SIZE;
 
 static const byte T_MSX_REG  = 0x00; // Normal file
 static const byte T_MSX_READ = 0x01; // Read-Only file
@@ -117,30 +117,9 @@ void MSXtar::parseBootSector(const byte* buf)
 	maxCluster = sectorToCluster(nbSectors);
 }
 
-void MSXtar::parseBootSectorFAT(const byte* buf)
-{
-	// empty FAT buffer before filling it again (or for the first time)
-	writeCachedFAT();
-
-	try {
-		parseBootSector(buf);
-	} catch (MSXException& e) {
-		throw MSXException("Bad disk image: " + e.getMessage());
-	}
-
-	// cache complete FAT
-	assert(fatBuffer.empty()); // cache must be empty at this point
-	fatCacheDirty = false;
-	fatBuffer.resize(SECTOR_SIZE * sectorsPerFat);
-	for (unsigned i = 0; i < sectorsPerFat; ++i) {
-		disk.readLogicalSector(i + 1 + partitionOffset,
-		                       &fatBuffer[SECTOR_SIZE * i]);
-	}
-}
-
 void MSXtar::writeLogicalSector(unsigned sector, const byte* buf)
 {
-	assert(sector < partitionNbSectors);
+	assert(sector < disk.getNbSectors());
 	unsigned fatSector = sector - 1;
 	if ((fatSector < sectorsPerFat) && !fatBuffer.empty()) {
 		// we have a cache and this is a sector of the 1st FAT
@@ -148,192 +127,63 @@ void MSXtar::writeLogicalSector(unsigned sector, const byte* buf)
 		memcpy(&fatBuffer[SECTOR_SIZE * fatSector], buf, SECTOR_SIZE);
 		fatCacheDirty = true;
 	} else {
-		disk.writeLogicalSector(sector + partitionOffset, buf);
+		disk.writeSector(sector, buf);
 	}
 }
 
 void MSXtar::readLogicalSector(unsigned sector, byte* buf)
 {
-	assert(sector < partitionNbSectors);
+	assert(sector < disk.getNbSectors());
 	unsigned fatSector = sector - 1;
 	if ((fatSector < sectorsPerFat) && !fatBuffer.empty()) {
 		// we have a cache and this is a sector of the 1st FAT
 		//   --> read from cache
 		memcpy(buf, &fatBuffer[SECTOR_SIZE * fatSector], SECTOR_SIZE);
 	} else {
-		disk.readLogicalSector(sector + partitionOffset, buf);
+		disk.readSector(sector, buf);
 	}
 }
 
 MSXtar::MSXtar(SectorAccessibleDisk& sectordisk)
 	: disk(sectordisk)
 {
-	static bool init = false;
-	if (!init) {
-		init = true;
-		srand(time(0));
+	if (disk.getNbSectors() == 0) {
+		throw MSXException("No disk inserted.");
+	}
+	try {
+		byte bootSector[SECTOR_SIZE];
+		disk.readSector(0, bootSector);
+		parseBootSector(bootSector);
+	} catch (MSXException& e) {
+		throw MSXException("Bad disk image: " + e.getMessage());
 	}
 
+	// cache complete FAT
 	fatCacheDirty = false;
-	partitionOffset = 0;
-	partitionNbSectors = disk.getNbSectors();
-	if (partitionNbSectors == 0) {
-		throw MSXException("No disk inserted.");
+	fatBuffer.resize(SECTOR_SIZE * sectorsPerFat);
+	for (unsigned i = 0; i < sectorsPerFat; ++i) {
+		disk.readSector(i + 1, &fatBuffer[SECTOR_SIZE * i]);
 	}
 }
 
 MSXtar::~MSXtar()
-{
-	try {
-		writeCachedFAT();
-	} catch (MSXException& e) {
-		// nothing
-	}
-}
-
-// write cached FAT back to disk image
-void MSXtar::writeCachedFAT()
 {
 	if (fatCacheDirty) {
 		for (unsigned i = 0; i < fatBuffer.size() / SECTOR_SIZE; ++i) {
-			disk.writeLogicalSector(i + 1 + partitionOffset,
-			                        &fatBuffer[SECTOR_SIZE * i]);
+			try {
+				disk.writeSector(i + 1, &fatBuffer[SECTOR_SIZE * i]);
+			} catch (MSXException& e) {
+				// nothing
+			}
 		}
-		fatCacheDirty = false;
-	}
-	fatBuffer.clear();
-}
-
-// Create a correct bootsector depending on the required size of the filesystem
-void MSXtar::setBootSector(byte* buf, unsigned nbSectors)
-{
-	// variables set to single sided disk by default
-	word nbSides = 1;
-	byte nbFats = 2;
-	byte nbReservedSectors = 1; // Just copied from a 32MB IDE partition
-	byte nbSectorsPerFat = 2;
-	byte nbHiddenSectors = 1;
-	word nbDirEntry = 112;
-	byte descriptor = 0xF8;
-
-	// now set correct info according to size of image (in sectors!)
-	// and using the same layout as used by Jon in IDEFDISK v 3.1
-	if (nbSectors >= 32733) {
-		nbSides = 2;		// unknown yet
-		nbFats = 2;		// unknown yet
-		nbSectorsPerFat = 12;	// copied from a partition from an IDE HD
-		nbSectorsPerCluster = 16;
-		nbDirEntry = 256;
-		nbSides = 32;		// copied from a partition from an IDE HD
-		nbHiddenSectors = 16;
-		descriptor = 0xF0;
-	} else if (nbSectors >= 16389) {
-		nbSides = 2;		// unknown yet
-		nbFats = 2;		// unknown yet
-		nbSectorsPerFat = 3;	// unknown yet
-		nbSectorsPerCluster = 8;
-		nbDirEntry = 256;
-		descriptor = 0XF0;
-	} else if (nbSectors >= 8213) {
-		nbSides = 2;		// unknown yet
-		nbFats = 2;		// unknown yet
-		nbSectorsPerFat = 3;	// unknown yet
-		nbSectorsPerCluster = 4;
-		nbDirEntry = 256;
-		descriptor = 0xF0;
-	} else if (nbSectors >= 4127) {
-		nbSides = 2;		// unknown yet
-		nbFats = 2;		// unknown yet
-		nbSectorsPerFat = 3;	// unknown yet
-		nbSectorsPerCluster = 2;
-		nbDirEntry = 256;
-		descriptor = 0xF0;
-	} else if (nbSectors >= 2880) {
-		nbSides = 2;		// unknown yet
-		nbFats = 2;		// unknown yet
-		nbSectorsPerFat = 3;	// unknown yet
-		nbSectorsPerCluster = 1;
-		nbDirEntry = 224;
-		descriptor = 0xF0;
-	} else if (nbSectors >= 1441) {
-		nbSides = 2;		// unknown yet
-		nbFats = 2;		// unknown yet
-		nbSectorsPerFat = 3;	// unknown yet
-		nbSectorsPerCluster = 2;
-		nbDirEntry = 112;
-		descriptor = 0xF0;
-
-	} else if (nbSectors <= 720) {
-		// normal single sided disk
-		nbSectors = 720;
-	} else {
-		// normal double sided disk
-		nbSectors = 1440;
-		nbSides = 2;
-		nbFats = 2;
-		nbSectorsPerFat = 3;
-		nbSectorsPerCluster = 2;
-		nbDirEntry = 112;
-		descriptor = 0xF9;
-	}
-	MSXBootSector* boot = reinterpret_cast<MSXBootSector*>(buf);
-
-	setsh(boot->nrsectors, nbSectors);
-	setsh(boot->nrsides, nbSides);
-	boot->spcluster[0] = nbSectorsPerCluster;
-	boot->nrfats[0] = nbFats;
-	setsh(boot->sectorsfat, nbSectorsPerFat);
-	setsh(boot->direntries, nbDirEntry);
-	boot->descriptor[0] = descriptor;
-	setsh(boot->reservedsectors, nbReservedSectors);
-	setsh(boot->hiddensectors, nbHiddenSectors);
-
-	// set random volume id
-	for (int i = 0x27; i < 0x2B; ++i) {
-		buf[i] = rand() & 0x7F;
 	}
 }
 
-// Format a diskimage with correct bootsector, FAT etc.
-void MSXtar::format()
+// transform BAD_FAT (0xFF7) and EOF_FAT-range (0xFF8-0xFFF)
+// to a single value: EOF_FAT (0xFFF)
+static unsigned normalizeFAT(unsigned cluster)
 {
-	// first create a bootsector and start from the default bootblock
-	byte sectorbuf[512];
-	const byte* defaultBootBlock = BootBlocks::dos2BootBlock;
-	memcpy(sectorbuf, defaultBootBlock, SECTOR_SIZE);
-	setBootSector(sectorbuf, partitionNbSectors);
-	try {
-		parseBootSector(sectorbuf); //set object variables to correct values
-	} catch (MSXException& e) {
-		throw MSXException("Internal error, invalid defaultBootBlock: " +
-		                   e.getMessage());
-	}
-	writeLogicalSector(0, sectorbuf);
-
-	MSXBootSector* boot = reinterpret_cast<MSXBootSector*>(sectorbuf);
-	byte descriptor = boot->descriptor[0];
-
-	// Assign default empty values to disk
-	memset(sectorbuf, 0x00, SECTOR_SIZE);
-	for (unsigned i = 2; i <= rootDirLast; ++i) {
-		writeLogicalSector(i, sectorbuf);
-	}
-	// for some reason the first 3 bytes are used to indicate the end of a
-	// cluster, making the first available cluster nr 2 some sources say
-	// that this indicates the disk format and FAT[0] should 0xF7 for
-	// single sided disk, and 0xF9 for double sided disks
-	// TODO: check this :-)
-	// for now I simply repeat the media descriptor here
-	sectorbuf[0] = descriptor;
-	sectorbuf[1] = 0xFF;
-	sectorbuf[2] = 0xFF;
-
-	writeLogicalSector(1, sectorbuf);
-
-	memset(sectorbuf, 0xE5, SECTOR_SIZE);
-	for (unsigned i = 1 + rootDirLast; i < partitionNbSectors; ++i) {
-		writeLogicalSector(i, sectorbuf);
-	}
+	return (cluster < BAD_FAT) ? cluster : EOF_FAT;
 }
 
 // Get the next clusternumber from the FAT chain
@@ -341,9 +191,10 @@ unsigned MSXtar::readFAT(unsigned clnr) const
 {
 	assert(!fatBuffer.empty()); // FAT must already be cached
 	const byte* p = &fatBuffer[(clnr * 3) / 2];
-	return (clnr & 1)
-	       ? (p[0] >> 4) + (p[1] << 4)
-	       : p[0] + ((p[1] & 0x0F) << 8);
+	unsigned result = (clnr & 1)
+	                ? (p[0] >> 4) + (p[1] << 4)
+	                : p[0] + ((p[1] & 0x0F) << 8);
+	return normalizeFAT(result);
 }
 
 // Write an entry to the FAT
@@ -393,6 +244,13 @@ unsigned MSXtar::getNextSector(unsigned sector)
 	}
 }
 
+// get start cluster from a directory entry,
+// also takes care of BAD_FAT and EOF_FAT-range.
+unsigned MSXtar::getStartCluster(const MSXDirEntry& entry)
+{
+	return normalizeFAT(rdsh(entry.startcluster));
+}
+
 // If there are no more free entries in a subdirectory, the subdir is
 // expanded with an extra cluster. This function gets the free cluster,
 // clears it and updates the fat for the subdir
@@ -401,20 +259,20 @@ unsigned MSXtar::getNextSector(unsigned sector)
 unsigned MSXtar::appendClusterToSubdir(unsigned sector)
 {
 	unsigned nextcl = findFirstFreeCluster();
-	unsigned logicalSector = clusterToSector(nextcl);
+	unsigned nextSector = clusterToSector(nextcl);
 
 	// clear this cluster
 	byte buf[SECTOR_SIZE];
 	memset(buf, 0, SECTOR_SIZE);
 	for (unsigned i = 0; i < sectorsPerCluster; ++i) {
-		writeLogicalSector(i + logicalSector, buf);
+		writeLogicalSector(i + nextSector, buf);
 	}
 
 	unsigned curcl = sectorToCluster(sector);
 	assert(readFAT(curcl) == EOF_FAT);
 	writeFAT(curcl, nextcl);
 	writeFAT(nextcl, EOF_FAT);
-	return logicalSector;
+	return nextSector;
 }
 
 
@@ -465,10 +323,11 @@ MSXtar::DirEntry MSXtar::addEntryToDir(unsigned sector)
 			if (result.index != unsigned(-1)) {
 				return result;
 			}
-			result.sector = getNextSector(result.sector);
-			if (result.sector == 0) {
-				result.sector = appendClusterToSubdir(result.sector);
+			unsigned nextSector = getNextSector(result.sector);
+			if (nextSector == 0) {
+				nextSector = appendClusterToSubdir(result.sector);
 			}
+			result.sector = nextSector;
 		}
 	}
 }
@@ -544,7 +403,7 @@ unsigned MSXtar::addSubdir(
 	// save the sector again
 	writeLogicalSector(result.sector, buf);
 
-	//clear this cluster
+	// clear this cluster
 	unsigned logicalSector = clusterToSector(curcl);
 	memset(buf, 0, SECTOR_SIZE);
 	for (unsigned i = 0; i < sectorsPerCluster; ++i) {
@@ -627,14 +486,11 @@ void MSXtar::alterFileInDSK(MSXDirEntry& msxdirentry, const string& hostName)
 	unsigned remaining = hostSize;
 
 	// open host file for reading
-	FILE* file = fopen(hostName.c_str(), "rb");
-	if (!file) {
-		throw MSXException("Error reading host file: " + hostName);
-	}
+	File file(hostName, "rb");
 
 	// copy host file to image
 	unsigned prevcl = 0;
-	unsigned curcl = rdsh(msxdirentry.startcluster);
+	unsigned curcl = getStartCluster(msxdirentry);
 	while (remaining) {
 		// allocate new cluster if needed
 		try {
@@ -659,7 +515,7 @@ void MSXtar::alterFileInDSK(MSXDirEntry& msxdirentry, const string& hostName)
 			byte buf[SECTOR_SIZE];
 			memset(buf, 0, SECTOR_SIZE);
 			unsigned chunkSize = std::min(SECTOR_SIZE, remaining);
-			fread(buf, 1, chunkSize, file);
+			file.read(buf, chunkSize);
 			writeLogicalSector(logicalSector + j, buf);
 			remaining -= chunkSize;
 		}
@@ -668,7 +524,6 @@ void MSXtar::alterFileInDSK(MSXDirEntry& msxdirentry, const string& hostName)
 		prevcl = curcl;
 		curcl = readFAT(curcl);
 	}
-	fclose(file);
 
 	// terminate FAT chain
 	if (prevcl == 0) {
@@ -785,7 +640,7 @@ string MSXtar::recurseDirFill(const string& dirName, unsigned sector)
 				if (msxdirentry.attrib & T_MSX_DIR) {
 					// .. and is a directory
 					unsigned nextSector = clusterToSector(
-						rdsh(msxdirentry.startcluster));
+						getStartCluster(msxdirentry));
 					messages += recurseDirFill(fullName, nextSector);
 				} else {
 					// .. but is NOT a directory
@@ -853,14 +708,14 @@ string MSXtar::dir()
 				string tmp = condensName(direntry[i]);
 				tmp.resize(13, ' ');
 				result += tmp;
-				//attributes
+				// attributes
 				result += (direntry[i].attrib & T_MSX_DIR  ? "d" : "-");
 				result += (direntry[i].attrib & T_MSX_READ ? "r" : "-");
 				result += (direntry[i].attrib & T_MSX_HID  ? "h" : "-");
-				result += (direntry[i].attrib & T_MSX_VOL  ? "v" : "-"); //todo check if this is the output of files,l
-				result += (direntry[i].attrib & T_MSX_ARC  ? "a" : "-"); //todo check if this is the output of files,l
+				result += (direntry[i].attrib & T_MSX_VOL  ? "v" : "-"); // TODO check if this is the output of files,l
+				result += (direntry[i].attrib & T_MSX_ARC  ? "a" : "-"); // TODO check if this is the output of files,l
 				result += "  ";
-				//filesize
+				// filesize
 				result += StringOp::toString(rdlg(direntry[i].size)) + "\n";
 			}
 		}
@@ -916,7 +771,7 @@ void MSXtar::chroot(const string& newRootDir, bool createDir)
 			if (!(direntry.attrib & T_MSX_DIR)) {
 				throw MSXException(firstpart + " is not a directory.");
 			}
-			chrootSector = clusterToSector(rdsh(direntry.startcluster));
+			chrootSector = clusterToSector(getStartCluster(direntry));
 		}
 	}
 }
@@ -924,25 +779,54 @@ void MSXtar::chroot(const string& newRootDir, bool createDir)
 void MSXtar::fileExtract(string resultFile, MSXDirEntry& direntry)
 {
 	unsigned size = rdlg(direntry.size);
-	unsigned sector = clusterToSector(rdsh(direntry.startcluster));
+	unsigned sector = clusterToSector(getStartCluster(direntry));
 
-	FILE* file = fopen(resultFile.c_str(), "wb");
-	if (!file) {
-		throw MSXException("Couldn't open file for writing!");
-	}
+	File file(resultFile, "wb");
 	while (size && sector) {
 		byte buf[SECTOR_SIZE];
 		readLogicalSector(sector, buf);
 		unsigned savesize = std::min(size, SECTOR_SIZE);
-		fwrite(buf, 1, savesize, file);
+		file.write(buf, savesize);
 		size -= savesize;
 		sector = getNextSector(sector);
 	}
-	fclose(file);
 	// now change the access time
 	changeTime(resultFile, direntry);
 }
 
+// extracts a single item (file or directory) from the msximage to the host OS
+string MSXtar::singleItemExtract(const string& dirName, const string& itemName,
+                                 unsigned sector)
+{
+	// first find out if the filename exists in current dir
+	byte dummy[SECTOR_SIZE];
+	string msxName = makeSimpleMSXFileName(itemName);
+	DirEntry entry = findEntryInDir(msxName, sector, dummy);
+	if (entry.sector == 0) {
+		return itemName + " not found!\n";
+	}
+
+	MSXDirEntry* direntries = reinterpret_cast<MSXDirEntry*>(dummy);
+	MSXDirEntry& msxdirentry = direntries[entry.index];
+	// create full name for loacl filesystem
+	string fullname = dirName + '/' + condensName(msxdirentry);
+
+	// ...and extract
+	if  (msxdirentry.attrib & T_MSX_DIR) {
+		// recursive extract this subdir
+		FileOperations::mkdirp(fullname);
+		recurseDirExtract(
+		    fullname,
+		    clusterToSector(getStartCluster(msxdirentry)));
+	} else {
+		// it is a file
+		fileExtract(fullname, msxdirentry);
+	}
+	return "";
+}
+
+
+// extracts the contents of the directory (at sector) and all its subdirs to the host OS
 void MSXtar::recurseDirExtract(const string& dirName, unsigned sector)
 {
 	for (/* */ ; sector != 0; sector = getNextSector(sector)) {
@@ -967,146 +851,28 @@ void MSXtar::recurseDirExtract(const string& dirName, unsigned sector)
 					changeTime(fullname, direntry[i]);
 					recurseDirExtract(
 					    fullname,
-					    clusterToSector(rdsh(direntry[i].startcluster)));
+					    clusterToSector(getStartCluster(direntry[i])));
 				}
 			}
 		}
 	}
 }
 
-static const char* const PARTAB_HEADER= "\353\376\220MSX_IDE ";
-
-/** returns: true if succesfull, false if partition isn't valid
-  */
-static bool isPartitionTableSector(byte* buf)
-{
-	return memcmp(buf, PARTAB_HEADER, 11) == 0;
-}
-
-bool MSXtar::hasPartitionTable()
-{
-	byte buf[SECTOR_SIZE];
-	disk.readLogicalSector(0, buf);
-	return isPartitionTableSector(buf);
-}
-
-bool MSXtar::hasPartition(unsigned partition)
-{
-	byte buf[SECTOR_SIZE];
-	disk.readLogicalSector(0, buf);
-	if (!isPartitionTableSector(buf)) {
-		return false;
-	}
-	Partition* p = reinterpret_cast<Partition*>(buf + 14 + (30 - partition) * 16);
-	if (rdlg(p->start4) == 0) {
-		return false;
-	}
-	return true;
-}
-
-bool MSXtar::usePartition(unsigned partition)
-{
-	// TODO: separate partition selection from boot sector parsing
-	// (format only needs the former, and the latter can throw exceptions)
-	byte partitionTable[SECTOR_SIZE];
-	partitionOffset = 0;
-	partitionNbSectors = disk.getNbSectors();
-	disk.readLogicalSector(0, partitionTable);
-	bool hasPartitionTable = isPartitionTableSector(partitionTable);
-	if (hasPartitionTable) {
-		Partition* p = reinterpret_cast<Partition*>
-			(partitionTable + 14 + (30 - partition) * 16);
-		if (rdlg(p->start4) != 0) {
-			partitionOffset = rdlg(p->start4);
-			partitionNbSectors = rdlg(p->size4);
-			if (p->sys_ind != 0x01) {
-				throw MSXException("Not a FAT12 partition");
-			}
-		} else {
-			hasPartitionTable = false;
-		}
-	}
-
-	byte bootSector[SECTOR_SIZE];
-	disk.readLogicalSector(partitionOffset, bootSector);
-	parseBootSectorFAT(bootSector);
-	return hasPartitionTable;
-}
-
-static void logicalToCHS(unsigned logical, unsigned& cylinder,
-                         unsigned& head, unsigned& sector)
-{
-	// This is made to fit the openMSX harddisk configuration:
-	//  32 sectors/track   16 heads
-	unsigned tmp = logical + 1;
-	sector = tmp % 32;
-	if (sector == 0) sector = 32;
-	tmp = (tmp - sector) / 32;
-	head = tmp % 16;
-	cylinder = tmp / 16;
-}
-
-void MSXtar::createDiskFile(std::vector<unsigned> sizes)
-{
-	byte buf[SECTOR_SIZE];
-	assert(fatBuffer.empty()); //when creating disks we shouldn't have any fatBuffer
-
-	// create the partition table if needed
-	if (sizes.size() > 1) {
-		memset(buf, 0, SECTOR_SIZE);
-		memcpy(buf, PARTAB_HEADER, 11);
-		buf[SECTOR_SIZE - 2] = 0x55;
-		buf[SECTOR_SIZE - 1] = 0xAA;
-
-		partitionOffset = 1;
-		for (unsigned i = 0; (i < sizes.size()) && (i < 30); ++i) {
-			Partition* p = reinterpret_cast<Partition*>
-				(buf + (14 + (30 - i) * 16));
-			unsigned startCylinder, startHead, startSector;
-			unsigned endCylinder, endHead, endSector;
-			logicalToCHS(partitionOffset,
-			             startCylinder, startHead, startSector);
-			logicalToCHS(partitionOffset + sizes[i] - 1,
-			             endCylinder, endHead, endSector);
-			p->boot_ind = (i == 0) ? 0x80 : 0x00; // bootflag
-			p->head = startHead;
-			p->sector = startSector;
-			p->cyl = startCylinder;
-			p->sys_ind = 0x01; // FAT12
-			p->end_head = endHead;
-			p->end_sector = endSector;
-			p->end_cyl = endCylinder;
-			setlg(p->start4, partitionOffset);
-			setlg(p->size4, sizes[i]);
-			partitionNbSectors = sizes[i];
-			format();
-			partitionOffset += partitionNbSectors;
-		}
-		partitionOffset = 0;
-		partitionNbSectors = disk.getNbSectors();
-		writeLogicalSector(0, buf);
-	} else {
-		assert(partitionOffset == 0);
-		setBootSector(buf, sizes[0]);
-		writeLogicalSector(0, buf); //allow usePartition to read the bootsector!
-		usePartition(0);
-		format();
-	}
-}
-
-// temporary way to test import MSXtar functionality
 string MSXtar::addDir(const string& rootDirName)
 {
 	return recurseDirFill(rootDirName, chrootSector);
 }
 
-// add file into fake dsk
 string MSXtar::addFile(const string& filename)
 {
 	return addFileToDSK(filename, chrootSector);
 }
 
-//temporary way to test export MSXtar functionality
+string MSXtar::getItemFromDir(const string& rootDirName, const string& itemName)
+{
+	return singleItemExtract(rootDirName, itemName, chrootSector);
+}
+
 void MSXtar::getDir(const string& rootDirName)
 {
 	recurseDirExtract(rootDirName, chrootSector);
