@@ -1,18 +1,18 @@
-// $Id: MicrosolFDC.cc 6805 2007-08-06 12:01:49Z m9710797 $
+// $Id: MicrosolFDC.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "MicrosolFDC.hh"
 #include "DriveMultiplexer.hh"
 #include "WD2793.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
-MicrosolFDC::MicrosolFDC(MSXMotherBoard& motherBoard, const XMLElement& config,
-                         const EmuTime& time)
-	: WD2793BasedFDC(motherBoard, config, time)
+MicrosolFDC::MicrosolFDC(MSXMotherBoard& motherBoard, const XMLElement& config)
+	: WD2793BasedFDC(motherBoard, config)
 {
 }
 
-byte MicrosolFDC::readIO(word port, const EmuTime& time)
+byte MicrosolFDC::readIO(word port, EmuTime::param time)
 {
 	byte value;
 	switch (port & 0x07) {
@@ -42,7 +42,7 @@ byte MicrosolFDC::readIO(word port, const EmuTime& time)
 	return value;
 }
 
-byte MicrosolFDC::peekIO(word port, const EmuTime& time) const
+byte MicrosolFDC::peekIO(word port, EmuTime::param time) const
 {
 	byte value;
 	switch (port & 0x07) {
@@ -70,7 +70,7 @@ byte MicrosolFDC::peekIO(word port, const EmuTime& time) const
 	return value;
 }
 
-void MicrosolFDC::writeIO(word port, byte value, const EmuTime& time)
+void MicrosolFDC::writeIO(word port, byte value, EmuTime::param time)
 {
 	PRT_DEBUG("MicrosolFDC: write 0x" << std::hex << int(port) << " 0x"
 	          << int(value) << std::dec);
@@ -133,4 +133,13 @@ void MicrosolFDC::writeIO(word port, byte value, const EmuTime& time)
 	}
 }
 
+
+template<typename Archive>
+void MicrosolFDC::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<WD2793BasedFDC>(*this);
+}
+INSTANTIATE_SERIALIZE_METHODS(MicrosolFDC);
+REGISTER_MSXDEVICE(MicrosolFDC, "MicrosolFDC");
+
 } // namespace openmsx
