@@ -1,19 +1,19 @@
-// $Id: NationalFDC.cc 5751 2006-10-05 19:34:54Z m9710797 $
+// $Id: NationalFDC.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "NationalFDC.hh"
 #include "CacheLine.hh"
 #include "DriveMultiplexer.hh"
 #include "WD2793.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
-NationalFDC::NationalFDC(MSXMotherBoard& motherBoard, const XMLElement& config,
-                         const EmuTime& time)
-	: WD2793BasedFDC(motherBoard, config, time)
+NationalFDC::NationalFDC(MSXMotherBoard& motherBoard, const XMLElement& config)
+	: WD2793BasedFDC(motherBoard, config)
 {
 }
 
-byte NationalFDC::readMem(word address, const EmuTime& time)
+byte NationalFDC::readMem(word address, EmuTime::param time)
 {
 	byte value;
 	switch (address & 0x3FC7) {
@@ -44,7 +44,7 @@ byte NationalFDC::readMem(word address, const EmuTime& time)
 	return value;
 }
 
-byte NationalFDC::peekMem(word address, const EmuTime& time) const
+byte NationalFDC::peekMem(word address, EmuTime::param time) const
 {
 	byte value;
 	// According to atarulum:
@@ -101,7 +101,7 @@ const byte* NationalFDC::getReadCacheLine(word start) const
 	}
 }
 
-void NationalFDC::writeMem(word address, byte value, const EmuTime& time)
+void NationalFDC::writeMem(word address, byte value, EmuTime::param time)
 {
 	//PRT_DEBUG("NationalFDC write 0x" << hex << (int)address << " 0x" << (int)value << dec);
 	switch (address & 0x3FC7) {
@@ -121,10 +121,10 @@ void NationalFDC::writeMem(word address, byte value, const EmuTime& time)
 	case 0x3F85:
 	case 0x3F86:
 	case 0x3F87:
-		//bit 0 -> select drive 0
-		//bit 1 -> select drive 1
-		//bit 2 -> side select
-		//bit 3 -> motor on
+		// bit 0 -> select drive 0
+		// bit 1 -> select drive 1
+		// bit 2 -> side select
+		// bit 3 -> motor on
 		DriveMultiplexer::DriveNum drive;
 		switch (value & 3) {
 			case 1:
@@ -153,4 +153,13 @@ byte* NationalFDC::getWriteCacheLine(word address) const
 	}
 }
 
+
+template<typename Archive>
+void NationalFDC::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<WD2793BasedFDC>(*this);
+}
+INSTANTIATE_SERIALIZE_METHODS(NationalFDC);
+REGISTER_MSXDEVICE(NationalFDC, "NationalFDC");
+
 } // namespace openmsx
