@@ -1,29 +1,29 @@
-// $Id: PhilipsFDC.cc 6023 2007-01-29 21:16:49Z m9710797 $
+// $Id: PhilipsFDC.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "PhilipsFDC.hh"
 #include "CacheLine.hh"
 #include "DriveMultiplexer.hh"
 #include "WD2793.hh"
 #include "XMLElement.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
-PhilipsFDC::PhilipsFDC(MSXMotherBoard& motherBoard, const XMLElement& config,
-                       const EmuTime& time)
-	: WD2793BasedFDC(motherBoard, config, time)
+PhilipsFDC::PhilipsFDC(MSXMotherBoard& motherBoard, const XMLElement& config)
+	: WD2793BasedFDC(motherBoard, config)
+	, brokenFDCread(config.getChildDataAsBool("broken_fdc_read", false))
 {
-	brokenFDCread = config.getChildDataAsBool("broken_fdc_read", false);
-	reset(time);
+	reset(getCurrentTime());
 }
 
-void PhilipsFDC::reset(const EmuTime& time)
+void PhilipsFDC::reset(EmuTime::param time)
 {
 	WD2793BasedFDC::reset(time);
 	writeMem(0x3FFC, 0x00, time);
 	writeMem(0x3FFD, 0x00, time);
 }
 
-byte PhilipsFDC::readMem(word address, const EmuTime& time)
+byte PhilipsFDC::readMem(word address, EmuTime::param time)
 {
 	byte value;
 	switch (address & 0x3FFF) {
@@ -51,7 +51,7 @@ byte PhilipsFDC::readMem(word address, const EmuTime& time)
 	return value;
 }
 
-byte PhilipsFDC::peekMem(word address, const EmuTime& time) const
+byte PhilipsFDC::peekMem(word address, EmuTime::param time) const
 {
 	byte value;
 	// FDC registers are mirrored in
@@ -77,19 +77,19 @@ byte PhilipsFDC::peekMem(word address, const EmuTime& time) const
 		value = controller->peekDataReg(time);
 		break;
 	case 0x3FFC:
-		//bit 0 = side select
-		//TODO check other bits !!
+		// bit 0 = side select
+		// TODO check other bits !!
 		value = sideReg; // value = multiplexer.getSideSelect();
 		break;
 	case 0x3FFD:
-		//bit 1,0 -> drive number
-		//(00 or 10: drive A, 01: drive B, 11: nothing)
-		//bit 7 -> motor on
-		//TODO check other bits !!
+		// bit 1,0 -> drive number
+		// (00 or 10: drive A, 01: drive B, 11: nothing)
+		// bit 7 -> motor on
+		// TODO check other bits !!
 		value = driveReg; // multiplexer.getSelectedDrive();
 		break;
 	case 0x3FFE:
-		//not used
+		// not used
 		value = 255;
 		break;
 	case 0x3FFF:
@@ -97,7 +97,7 @@ byte PhilipsFDC::peekMem(word address, const EmuTime& time) const
 		// interrupt request
 		// bit 6: !intrq
 		// bit 7: !dtrq
-		//TODO check other bits !!
+		// TODO check other bits !!
 		value = 0xC0;
 		if (controller->peekIRQ(time)) value &= ~0x40;
 		if (controller->peekDTRQ(time)) value &= ~0x80;
@@ -131,7 +131,7 @@ const byte* PhilipsFDC::getReadCacheLine(word start) const
 	}
 }
 
-void PhilipsFDC::writeMem(word address, byte value, const EmuTime& time)
+void PhilipsFDC::writeMem(word address, byte value, EmuTime::param time)
 {
 	//PRT_DEBUG("PhilipsFDC write 0x" << hex << (int)address <<
 	//          " 0x" << (int)value << dec);
@@ -149,17 +149,17 @@ void PhilipsFDC::writeMem(word address, byte value, const EmuTime& time)
 		controller->setDataReg(value, time);
 		break;
 	case 0x3FFC:
-		//bit 0 = side select
-		//TODO check other bits !!
+		// bit 0 = side select
+		// TODO check other bits !!
 		sideReg = value;
 		multiplexer->setSide(value & 1);
 		break;
 	case 0x3FFD:
-		//bit 1,0 -> drive number
-		//(00 or 10: drive A, 01: drive B, 11: nothing)
-		//TODO bit 6 -> drive LED (0 -> off, 1 -> on)
-		//bit 7 -> motor on
-		//TODO check other bits !!
+		// bit 1,0 -> drive number
+		// (00 or 10: drive A, 01: drive B, 11: nothing)
+		// TODO bit 6 -> drive LED (0 -> off, 1 -> on)
+		// bit 7 -> motor on
+		// TODO check other bits !!
 		driveReg = value;
 		DriveMultiplexer::DriveNum drive;
 		switch (value & 3) {
@@ -189,4 +189,15 @@ byte* PhilipsFDC::getWriteCacheLine(word address) const
 	}
 }
 
+
+template<typename Archive>
+void PhilipsFDC::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<WD2793BasedFDC>(*this);
+	ar.serialize("sideReg", sideReg);
+	ar.serialize("driveReg", driveReg);
+}
+INSTANTIATE_SERIALIZE_METHODS(PhilipsFDC);
+REGISTER_MSXDEVICE(PhilipsFDC, "PhilipsFDC");
+
 } // namespace openmsx
