@@ -1,15 +1,16 @@
-// $Id: RealDrive.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: RealDrive.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "RealDrive.hh"
 #include "Disk.hh"
 #include "DiskChanger.hh"
 #include "MSXMotherBoard.hh"
-#include "EventDistributor.hh"
-#include "LedEvent.hh"
+#include "LedStatus.hh"
 #include "CommandController.hh"
 #include "ThrottleManager.hh"
-#include "MSXCliComm.hh"
+#include "CliComm.hh"
 #include "GlobalSettings.hh"
+#include "MSXException.hh"
+#include "serialize.hh"
 #include <bitset>
 
 using std::string;
@@ -17,19 +18,19 @@ using std::vector;
 
 namespace openmsx {
 
-/// class RealDrive ///
-
 static const unsigned MAX_DRIVES = 26; // a-z
 typedef std::bitset<MAX_DRIVES> DrivesInUse;
 
-RealDrive::RealDrive(MSXMotherBoard& motherBoard_, const EmuTime& time)
+RealDrive::RealDrive(MSXMotherBoard& motherBoard_, bool doubleSided)
 	: Schedulable(motherBoard_.getScheduler())
-	, motorTimer(time), headLoadTimer(time)
-	, headPos(0), motorStatus(false), headLoadStatus(false)
 	, motherBoard(motherBoard_)
 	, loadingIndicator(new LoadingIndicator(
 	         motherBoard.getGlobalSettings().getThrottleManager()))
+	, motorTimer(motherBoard.getCurrentTime())
+	, headLoadTimer(motherBoard.getCurrentTime())
+	, headPos(0), side(0), motorStatus(false), headLoadStatus(false)
 	, timeOut(false)
+	, doubleSizedDrive(doubleSided)
 {
 	MSXMotherBoard::SharedStuff& info =
 		motherBoard.getSharedStuff("drivesInUse");
@@ -55,9 +56,7 @@ RealDrive::RealDrive(MSXMotherBoard& motherBoard_, const EmuTime& time)
 	motherBoard.getMSXCliComm().update(CliComm::HARDWARE, driveName, "add");
 	changer.reset(new DiskChanger(
 		driveName, motherBoard.getCommandController(),
-		motherBoard.getDiskManipulator(),
-		&motherBoard.getMSXEventDistributor(),
-		&motherBoard.getScheduler()));
+		motherBoard.getDiskManipulator(), &motherBoard));
 }
 
 RealDrive::~RealDrive()
@@ -82,19 +81,34 @@ RealDrive::~RealDrive()
 	}
 }
 
-bool RealDrive::ready()
+bool RealDrive::isReady() const
 {
-	return changer->getDisk().ready();
+	return changer->getDisk().isReady();
 }
 
-bool RealDrive::writeProtected()
+bool RealDrive::isWriteProtected() const
 {
 	// write protected bit is always 0 when motor is off
 	// verified on NMS8280
-	return motorStatus && changer->getDisk().writeProtected();
+	return motorStatus && changer->getDisk().isWriteProtected();
 }
 
-void RealDrive::step(bool direction, const EmuTime& time)
+bool RealDrive::isDoubleSided() const
+{
+	return doubleSizedDrive ? changer->getDisk().isDoubleSided()
+	                        : false;
+}
+
+void RealDrive::setSide(bool side_)
+{
+	if (doubleSizedDrive) {
+		side = side_ ? 1 : 0;
+	} else {
+		assert(side == 0);
+	}
+}
+
+void RealDrive::step(bool direction, EmuTime::param time)
 {
 	if (direction) {
 		// step in
@@ -111,15 +125,14 @@ void RealDrive::step(bool direction, const EmuTime& time)
 	resetTimeOut(time);
 }
 
-bool RealDrive::track00(const EmuTime& time)
+bool RealDrive::isTrack00() const
 {
-	resetTimeOut(time);
 	// track00 bit is always 0 when motor is off
 	// verified on NMS8280
 	return motorStatus && (headPos == 0);
 }
 
-void RealDrive::setMotor(bool status, const EmuTime& time)
+void RealDrive::setMotor(bool status, EmuTime::param time)
 {
 	if (motorStatus != status) {
 		motorStatus = status;
@@ -127,26 +140,25 @@ void RealDrive::setMotor(bool status, const EmuTime& time)
 		/* The following is a hack to emulate the drive LED behaviour.
 		 * This is in real life dependent on the FDC and should be
 		 * investigated in detail to implement it properly... TODO */
-		motherBoard.getEventDistributor().distributeEvent(
-			new LedEvent(LedEvent::FDD, motorStatus, motherBoard));
+		motherBoard.getLedStatus().setLed(LedStatus::FDD, motorStatus);
 		updateLoadingState();
 	}
 
 }
 
-bool RealDrive::indexPulse(const EmuTime& time)
+bool RealDrive::indexPulse(EmuTime::param time)
 {
-	if (!motorStatus && changer->getDisk().ready()) {
+	if (!motorStatus && isReady()) {
 		return false;
 	}
 	int angle = motorTimer.getTicksTill(time) % TICKS_PER_ROTATION;
 	return angle < INDEX_DURATION;
 }
 
-int RealDrive::indexPulseCount(const EmuTime& begin,
-                               const EmuTime& end)
+int RealDrive::indexPulseCount(EmuTime::param begin,
+                               EmuTime::param end)
 {
-	if (!motorStatus && changer->getDisk().ready()) {
+	if (!motorStatus && isReady()) {
 		return 0;
 	}
 	int t1 = motorTimer.before(begin) ? motorTimer.getTicksTill(begin) : 0;
@@ -154,9 +166,9 @@ int RealDrive::indexPulseCount(const EmuTime& begin,
 	return (t2 / TICKS_PER_ROTATION) - (t1 / TICKS_PER_ROTATION);
 }
 
-EmuTime RealDrive::getTimeTillSector(byte sector, const EmuTime& time)
+EmuTime RealDrive::getTimeTillSector(byte sector, EmuTime::param time)
 {
-	if (!motorStatus || !changer->getDisk().ready()) { // TODO is this correct?
+	if (!motorStatus || !isReady()) { // TODO is this correct?
 		return time;
 	}
 	// TODO this really belongs in the Disk class
@@ -167,18 +179,15 @@ EmuTime RealDrive::getTimeTillSector(byte sector, const EmuTime& time)
 	int delta = sectorAngle - angle;
 	if (delta < 0) delta += TICKS_PER_ROTATION;
 	assert((0 <= delta) && (delta < TICKS_PER_ROTATION));
-	//std::cout << "DEBUG a1: " << angle
-	//          << " a2: " << sectorAngle
-	//          << " delta: " << delta << std::endl;
 
 	EmuDuration dur = Clock<TICKS_PER_ROTATION * ROTATIONS_PER_SECOND>::
 	                      duration(delta);
 	return time + dur;
 }
 
-EmuTime RealDrive::getTimeTillIndexPulse(const EmuTime& time)
+EmuTime RealDrive::getTimeTillIndexPulse(EmuTime::param time)
 {
-	if (!motorStatus || !changer->getDisk().ready()) { // TODO is this correct?
+	if (!motorStatus || !isReady()) { // TODO is this correct?
 		return time;
 	}
 	int delta = TICKS_PER_ROTATION -
@@ -188,7 +197,7 @@ EmuTime RealDrive::getTimeTillIndexPulse(const EmuTime& time)
 	return time + dur;
 }
 
-void RealDrive::setHeadLoaded(bool status, const EmuTime& time)
+void RealDrive::setHeadLoaded(bool status, EmuTime::param time)
 {
 	if (headLoadStatus != status) {
 		headLoadStatus = status;
@@ -196,12 +205,49 @@ void RealDrive::setHeadLoaded(bool status, const EmuTime& time)
 	}
 }
 
-bool RealDrive::headLoaded(const EmuTime& time)
+bool RealDrive::headLoaded(EmuTime::param time)
 {
 	return headLoadStatus &&
 	       (headLoadTimer.getTicksTill(time) > 10);
 }
 
+void RealDrive::read(byte sector, byte* buf,
+                     byte& onDiskTrack, byte& onDiskSector,
+                     byte& onDiskSide,  int&  onDiskSize)
+{
+	onDiskTrack = headPos;
+	onDiskSector = sector;
+	onDiskSide = side;
+	onDiskSize = 512;
+	changer->getDisk().read(headPos, sector, side, 512, buf);
+}
+
+void RealDrive::write(byte sector, const byte* buf,
+                      byte& onDiskTrack, byte& onDiskSector,
+                      byte& onDiskSide,  int&  onDiskSize)
+{
+	onDiskTrack = headPos;
+	onDiskSector = sector;
+	onDiskSide = side;
+	onDiskSize = 512;
+	changer->getDisk().write(headPos, sector, side, 512, buf);
+}
+
+void RealDrive::getSectorHeader(byte sector, byte* buf)
+{
+	changer->getDisk().getSectorHeader(headPos, sector, side, buf);
+}
+
+void RealDrive::getTrackHeader(byte* buf)
+{
+	changer->getDisk().getTrackHeader(headPos, side, buf);
+}
+
+void RealDrive::writeTrackData(const byte* data)
+{
+	changer->getDisk().writeTrackData(headPos, side, data);
+}
+
 bool RealDrive::diskChanged()
 {
 	return changer->diskChanged();
@@ -212,12 +258,12 @@ bool RealDrive::peekDiskChanged() const
 	return changer->peekDiskChanged();
 }
 
-bool RealDrive::dummyDrive()
+bool RealDrive::isDummyDrive() const
 {
 	return false;
 }
 
-void RealDrive::executeUntil(const EmuTime& /*time*/, int /*userData*/)
+void RealDrive::executeUntil(EmuTime::param /*time*/, int /*userData*/)
 {
 	timeOut = true;
 	updateLoadingState();
@@ -235,7 +281,7 @@ const std::string& RealDrive::schedName() const
 	return schedName;
 }
 
-void RealDrive::resetTimeOut(const EmuTime& time)
+void RealDrive::resetTimeOut(EmuTime::param time)
 {
 	timeOut = false;
 	updateLoadingState();
@@ -244,126 +290,31 @@ void RealDrive::resetTimeOut(const EmuTime& time)
 	setSyncPoint(now + 1000);
 }
 
-/// class SingleSidedDrive ///
-
-SingleSidedDrive::SingleSidedDrive(
-		MSXMotherBoard& motherBoard, const EmuTime& time)
-	: RealDrive(motherBoard, time)
+template<typename Archive>
+void RealDrive::serialize(Archive& ar, unsigned /*version*/)
 {
+	ar.template serializeBase<Schedulable>(*this);
+	ar.serialize("motorTimer", motorTimer);
+	ar.serialize("headLoadTimer", headLoadTimer);
+	ar.serialize("changer", *changer);
+	ar.serialize("headPos", headPos);
+	ar.serialize("side", side);
+	ar.serialize("motorStatus", motorStatus);
+	ar.serialize("headLoadStatus", headLoadStatus);
+	ar.serialize("timeOut", timeOut);
+	if (ar.isLoader()) {
+		updateLoadingState();
+
+		// This is a workaround for the fact that we can have multiple drives
+		// (and only one is on), in which case the 2nd drive will turn off the
+		// LED again which the first drive just turned on. TODO: fix by modelling
+		// individual drive LEDs properly. See also
+		// http://sourceforge.net/tracker/index.php?func=detail&aid=1540929&group_id=38274&atid=421864
+		if (motorStatus) {
+			motherBoard.getLedStatus().setLed(LedStatus::FDD, true);
+		}
+	}
 }
-
-bool SingleSidedDrive::doubleSided()
-{
-	return false;
-}
-
-void SingleSidedDrive::setSide(bool /*side*/)
-{
-	// ignore
-}
-
-void SingleSidedDrive::read(byte sector, byte* buf,
-                            byte& onDiskTrack, byte& onDiskSector,
-                            byte& onDiskSide,  int&  onDiskSize)
-{
-	onDiskTrack = headPos;
-	onDiskSector = sector;
-	onDiskSide = 0;
-	onDiskSize = 512;
-	changer->getDisk().read(headPos, sector, 0, 512, buf);
-}
-
-void SingleSidedDrive::write(byte sector, const byte* buf,
-                             byte& onDiskTrack, byte& onDiskSector,
-                             byte& onDiskSide,  int&  onDiskSize)
-{
-	onDiskTrack = headPos;
-	onDiskSector = sector;
-	onDiskSide = 0;
-	onDiskSize = 512;
-	changer->getDisk().write(headPos, sector, 0, 512, buf);
-}
-
-void SingleSidedDrive::getSectorHeader(byte sector, byte* buf)
-{
-	changer->getDisk().getSectorHeader(headPos, sector, 0, buf);
-}
-
-void SingleSidedDrive::getTrackHeader(byte* buf)
-{
-	changer->getDisk().getTrackHeader(headPos, 0, buf);
-}
-
-void SingleSidedDrive::initWriteTrack()
-{
-	changer->getDisk().initWriteTrack(headPos, 0);
-}
-
-void SingleSidedDrive::writeTrackData(byte data)
-{
-	changer->getDisk().writeTrackData(data);
-}
-
-
-/// class DoubleSidedDrive ///
-
-DoubleSidedDrive::DoubleSidedDrive(
-		MSXMotherBoard& motherBoard, const EmuTime& time)
-	: RealDrive(motherBoard, time)
-{
-	side = 0;
-}
-
-bool DoubleSidedDrive::doubleSided()
-{
-	return changer->getDisk().doubleSided();
-}
-
-void DoubleSidedDrive::setSide(bool side_)
-{
-	side = side_ ? 1 : 0;
-}
-
-void DoubleSidedDrive::read(byte sector, byte* buf,
-                            byte& onDiskTrack, byte& onDiskSector,
-                            byte& onDiskSide,  int&  onDiskSize)
-{
-	onDiskTrack = headPos;
-	onDiskSector = sector;
-	onDiskSide = side;
-	onDiskSize = 512;
-	changer->getDisk().read(headPos, sector, side, 512, buf);
-}
-
-void DoubleSidedDrive::write(byte sector, const byte* buf,
-                             byte& onDiskTrack, byte& onDiskSector,
-                             byte& onDiskSide,  int&  onDiskSize)
-{
-	onDiskTrack = headPos;
-	onDiskSector = sector;
-	onDiskSide = side;
-	onDiskSize = 512;
-	changer->getDisk().write(headPos, sector, side, 512, buf);
-}
-
-void DoubleSidedDrive::getSectorHeader(byte sector, byte* buf)
-{
-	changer->getDisk().getSectorHeader(headPos, sector, side, buf);
-}
-
-void DoubleSidedDrive::getTrackHeader(byte* buf)
-{
-	changer->getDisk().getTrackHeader(headPos, side, buf);
-}
-
-void DoubleSidedDrive::initWriteTrack()
-{
-	changer->getDisk().initWriteTrack(headPos, side);
-}
-
-void DoubleSidedDrive::writeTrackData(byte data)
-{
-	changer->getDisk().writeTrackData(data);
-}
+INSTANTIATE_SERIALIZE_METHODS(RealDrive);
 
 } // namespace openmsx
