@@ -1,4 +1,4 @@
-// $Id: TC8566AF.cc 6593 2007-06-03 23:12:19Z mthuurne $
+// $Id: TC8566AF.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 /*
  * Based on code from NLMSX written by Frits Hilderink
@@ -8,6 +8,7 @@
 #include "TC8566AF.hh"
 #include "DiskDrive.hh"
 #include "MSXException.hh"
+#include "serialize.hh"
 #include <cstring>
 
 namespace openmsx {
@@ -55,9 +56,12 @@ static const byte ST3_WP  = 0x40;
 static const byte ST3_FLT = 0x80;
 
 
-TC8566AF::TC8566AF(DiskDrive* drv[4], const EmuTime& time)
+TC8566AF::TC8566AF(DiskDrive* drv[4], EmuTime::param time)
 	: delayTime(EmuTime::zero)
 {
+	// avoid UMR (on savestate)
+	memset(sectorBuf, 0, sizeof(sectorBuf));
+
 	drive[0] = drv[0];
 	drive[1] = drv[1];
 	drive[2] = drv[2];
@@ -65,7 +69,7 @@ TC8566AF::TC8566AF(DiskDrive* drv[4], const EmuTime& time)
 	reset(time);
 }
 
-void TC8566AF::reset(const EmuTime& time)
+void TC8566AF::reset(EmuTime::param time)
 {
 	drive[0]->setMotor(false, time);
 	drive[1]->setMotor(false, time);
@@ -97,7 +101,7 @@ void TC8566AF::reset(const EmuTime& time)
 	//interrupt = false;
 }
 
-byte TC8566AF::peekReg(int reg, const EmuTime& /*time*/) const
+byte TC8566AF::peekReg(int reg, EmuTime::param /*time*/) const
 {
 	switch (reg) {
 	case 4: // Main Status Register
@@ -108,7 +112,7 @@ byte TC8566AF::peekReg(int reg, const EmuTime& /*time*/) const
 	return 0xff;
 }
 
-byte TC8566AF::readReg(int reg, const EmuTime& time)
+byte TC8566AF::readReg(int reg, EmuTime::param time)
 {
 	switch (reg) {
 	case 4: // Main Status Register
@@ -126,7 +130,7 @@ byte TC8566AF::peekStatus() const
 	return mainStatus | (dma ? STM_NDM : 0);
 }
 
-byte TC8566AF::readStatus(const EmuTime& time)
+byte TC8566AF::readStatus(EmuTime::param time)
 {
 	if (delayTime.before(time)) {
 		mainStatus |= STM_RQM;
@@ -146,7 +150,7 @@ byte TC8566AF::peekDataPort() const
 	}
 }
 
-byte TC8566AF::readDataPort(const EmuTime& time)
+byte TC8566AF::readDataPort(EmuTime::param time)
 {
 	//interrupt = false;
 	switch (phase) {
@@ -281,7 +285,7 @@ byte TC8566AF::resultsPhaseRead()
 	return result;
 }
 
-void TC8566AF::writeReg(int reg, byte data, const EmuTime& time)
+void TC8566AF::writeReg(int reg, byte data, EmuTime::param time)
 {
 	switch (reg) {
 	case 2: // control register 0
@@ -304,7 +308,7 @@ void TC8566AF::writeReg(int reg, byte data, const EmuTime& time)
 	}
 }
 
-void TC8566AF::writeDataPort(byte value, const EmuTime& time)
+void TC8566AF::writeDataPort(byte value, EmuTime::param time)
 {
 	switch (phase) {
 	case PHASE_IDLE:
@@ -322,7 +326,7 @@ void TC8566AF::writeDataPort(byte value, const EmuTime& time)
 		mainStatus &= ~STM_RQM;
 		break;
 	default:
-		//nothing
+		// nothing
 		break;
 	}
 }
@@ -384,28 +388,28 @@ void TC8566AF::idlePhaseWrite(byte value)
 	}
 }
 
-void TC8566AF::commandPhase1(byte value, const EmuTime& time)
+void TC8566AF::commandPhase1(byte value)
 {
 	drive[driveSelect]->setSide(value & 0x04);
 	status0 &= ~(ST0_DS0 | ST0_DS1 | ST0_IC0 | ST0_IC1);
-	status0 |= //(drive[driveSelect]->ready() ? 0 : ST0_DS0) |
+	status0 |= //(drive[driveSelect]->isReady() ? 0 : ST0_DS0) |
 	           (value & (ST0_DS0 | ST0_DS1)) |
-	           (drive[driveSelect]->dummyDrive() ? ST0_IC1 : 0);
+	           (drive[driveSelect]->isDummyDrive() ? ST0_IC1 : 0);
 	status3  = (value & (ST3_DS0 | ST3_DS1)) |
-	           (drive[driveSelect]->track00(time)    ? ST3_TK0 : 0) |
-	           (drive[driveSelect]->doubleSided()    ? ST3_HD  : 0) |
-	           (drive[driveSelect]->writeProtected() ? ST3_WP  : 0) |
-	           (drive[driveSelect]->ready()          ? ST3_RDY : 0);
+	           (drive[driveSelect]->isTrack00()        ? ST3_TK0 : 0) |
+	           (drive[driveSelect]->isDoubleSided()    ? ST3_HD  : 0) |
+	           (drive[driveSelect]->isWriteProtected() ? ST3_WP  : 0) |
+	           (drive[driveSelect]->isReady()          ? ST3_RDY : 0);
 }
 
-void TC8566AF::commandPhaseWrite(byte value, const EmuTime& time)
+void TC8566AF::commandPhaseWrite(byte value, EmuTime::param time)
 {
 	switch (command) {
 	case CMD_READ_DATA:
 	case CMD_WRITE_DATA:
 		switch (phaseStep++) {
 		case 0:
-			commandPhase1(value, time);
+			commandPhase1(value);
 			break;
 		case 1:
 			cylinderNumber = value;
@@ -459,7 +463,7 @@ void TC8566AF::commandPhaseWrite(byte value, const EmuTime& time)
 	case CMD_FORMAT:
 		switch (phaseStep++) {
 		case 0:
-			commandPhase1(value, time);
+			commandPhase1(value);
 			//sectorSize = diskGetSectorSize(driveSelect, side, currentTrack, 0);
 			sectorSize = 512;
 			break;
@@ -484,7 +488,7 @@ void TC8566AF::commandPhaseWrite(byte value, const EmuTime& time)
 	case CMD_SEEK:
 		switch (phaseStep++) {
 		case 0:
-			commandPhase1(value, time);
+			commandPhase1(value);
 			break;
 		case 1:
 			while (value > currentTrack) {
@@ -507,10 +511,10 @@ void TC8566AF::commandPhaseWrite(byte value, const EmuTime& time)
 	case CMD_RECALIBRATE:
 		switch (phaseStep++) {
 		case 0: {
-			commandPhase1(value, time);
+			commandPhase1(value);
 
 			unsigned maxSteps = 255;
-			while (!drive[driveSelect]->track00(time) && maxSteps--) {
+			while (!drive[driveSelect]->isTrack00() && maxSteps--) {
 				drive[driveSelect]->step(false, time);
 			}
 			currentTrack = 0;
@@ -536,7 +540,7 @@ void TC8566AF::commandPhaseWrite(byte value, const EmuTime& time)
 	case CMD_SENSE_DEVICE_STATUS:
 		switch (phaseStep++) {
 		case 0:
-			commandPhase1(value, time);
+			commandPhase1(value);
 
 			mainStatus |= STM_DIO;
 			phase       = PHASE_RESULT;
@@ -546,7 +550,7 @@ void TC8566AF::commandPhaseWrite(byte value, const EmuTime& time)
 		}
 		break;
 	default:
-		//nothing
+		// nothing
 		break;
 	}
 }
@@ -610,7 +614,7 @@ void TC8566AF::executionPhaseWrite(byte value)
 		}
 		break;
 	default:
-		//nothing
+		// nothing
 		break;
 	}
 }
@@ -627,4 +631,60 @@ bool TC8566AF::peekDiskChanged(unsigned driveNum) const
 	return drive[driveNum]->peekDiskChanged();
 }
 
+
+static enum_string<TC8566AF::Command> commandInfo[] = {
+	{ "UNKNOWN",                TC8566AF::CMD_UNKNOWN                },
+	{ "READ_DATA",              TC8566AF::CMD_READ_DATA              },
+	{ "WRITE_DATA",             TC8566AF::CMD_WRITE_DATA             },
+	{ "WRITE_DELETED_DATA",     TC8566AF::CMD_WRITE_DELETED_DATA     },
+	{ "READ_DELETED_DATA",      TC8566AF::CMD_READ_DELETED_DATA      },
+	{ "READ_DIAGNOSTIC",        TC8566AF::CMD_READ_DIAGNOSTIC        },
+	{ "READ_ID",                TC8566AF::CMD_READ_ID                },
+	{ "FORMAT",                 TC8566AF::CMD_FORMAT                 },
+	{ "SCAN_EQUAL",             TC8566AF::CMD_SCAN_EQUAL             },
+	{ "SCAN_LOW_OR_EQUAL",      TC8566AF::CMD_SCAN_LOW_OR_EQUAL      },
+	{ "SCAN_HIGH_OR_EQUAL",     TC8566AF::CMD_SCAN_HIGH_OR_EQUAL     },
+	{ "SEEK",                   TC8566AF::CMD_SEEK                   },
+	{ "RECALIBRATE",            TC8566AF::CMD_RECALIBRATE            },
+	{ "SENSE_INTERRUPT_STATUS", TC8566AF::CMD_SENSE_INTERRUPT_STATUS },
+	{ "SPECIFY",                TC8566AF::CMD_SPECIFY                },
+	{ "SENSE_DEVICE_STATUS",    TC8566AF::CMD_SENSE_DEVICE_STATUS    }
+};
+SERIALIZE_ENUM(TC8566AF::Command, commandInfo);
+
+static enum_string<TC8566AF::Phase> phaseInfo[] = {
+	{ "IDLE",         TC8566AF::PHASE_IDLE         },
+	{ "COMMAND",      TC8566AF::PHASE_COMMAND      },
+	{ "DATATRANSFER", TC8566AF::PHASE_DATATRANSFER },
+	{ "RESULT",       TC8566AF::PHASE_RESULT       }
+};
+SERIALIZE_ENUM(TC8566AF::Phase, phaseInfo);
+
+template<typename Archive>
+void TC8566AF::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.serialize("delayTime", delayTime);
+	ar.serialize("command", command);
+	ar.serialize("phase", phase);
+	ar.serialize("phaseStep", phaseStep);
+	ar.serialize("sectorSize", sectorSize);
+	ar.serialize("sectorOffset", sectorOffset);
+	ar.serialize_blob("sectorBuf", sectorBuf, sizeof(sectorBuf));
+	ar.serialize("driveSelect", driveSelect);
+	ar.serialize("mainStatus", mainStatus);
+	ar.serialize("status0", status0);
+	ar.serialize("status1", status1);
+	ar.serialize("status2", status2);
+	ar.serialize("status3", status3);
+	ar.serialize("commandCode", commandCode);
+	ar.serialize("cylinderNumber", cylinderNumber);
+	ar.serialize("headNumber", headNumber);
+	ar.serialize("sectorNumber", sectorNumber);
+	ar.serialize("number", number);
+	ar.serialize("currentTrack", currentTrack);
+	ar.serialize("sectorsPerCylinder", sectorsPerCylinder);
+	ar.serialize("fillerByte", fillerByte);
+};
+INSTANTIATE_SERIALIZE_METHODS(TC8566AF);
+
 } // namespace openmsx
