@@ -1,8 +1,10 @@
-// $Id: WD2793.cc 6805 2007-08-06 12:01:49Z m9710797 $
+// $Id: WD2793.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "WD2793.hh"
 #include "DiskDrive.hh"
 #include "MSXException.hh"
+#include "serialize.hh"
+#include <cstring>
 
 namespace openmsx {
 
@@ -35,16 +37,21 @@ static const int IMM_IRQ    = 0x08;
 // Sync point types
 enum SyncPointType { SCHED_FSM, SCHED_IDX_IRQ };
 
-WD2793::WD2793(Scheduler& scheduler, DiskDrive& drive_, const EmuTime& time)
+WD2793::WD2793(Scheduler& scheduler, DiskDrive& drive_, EmuTime::param time)
 	: Schedulable(scheduler)
 	, drive(drive_)
 	, commandStart(time)
 	, DRQTimer(time)
 {
+	// avoid (harmless) UMR in serialize()
+	memset(dataBuffer, 0, sizeof(dataBuffer));
+	dataCurrent = 0;
+	dataAvailable = 0;
+
 	reset(time);
 }
 
-void WD2793::reset(const EmuTime& time)
+void WD2793::reset(EmuTime::param time)
 {
 	removeSyncPoint(SCHED_FSM);
 	removeSyncPoint(SCHED_IDX_IRQ);
@@ -59,7 +66,6 @@ void WD2793::reset(const EmuTime& time)
 	resetIRQ();
 	immediateIRQ = false;
 
-	needInitWriteTrack = false;
 	formatting = false;
 	transferring = false;
 
@@ -68,7 +74,7 @@ void WD2793::reset(const EmuTime& time)
 	setCommandReg(0x03, time);
 }
 
-bool WD2793::getDTRQ(const EmuTime& time)
+bool WD2793::getDTRQ(EmuTime::param time)
 {
 	if (((commandReg & 0xC0) == 0x80) && (statusReg & BUSY)) {
 		// read/write sector cmd busy
@@ -80,40 +86,37 @@ bool WD2793::getDTRQ(const EmuTime& time)
 		}
 	} else if (((commandReg & 0xF0) == 0xF0) && (statusReg & BUSY)) {
 		// WRITE TRACK cmd busy
-		if (transferring) {
-			int ticks = DRQTimer.getTicksTill(time);
-			if (ticks >= 15) { // TODO found by trial and error
+		switch (drive.indexPulseCount(commandStart, time)) {
+		case 0: // no index pulse yet
+			break;
+		case 1: // first index pulse passed
+			if (DRQTimer.getTicksTill(time) >= 16) {
+				// '16' found by trial and error
+				// TODO implement something based on RAWTRACK_SIZE
 				DRQ = true;
 			}
-		} else {
-			int pulses = drive.indexPulseCount(commandStart, time);
-			if (pulses == 1) {
-				transferring = true;
-			}
-		}
-		if (drive.indexPulseCount(commandStart, time) >= 2) {
-			dataAvailable = 0;
-			dataCurrent = 0;
-			DRQ = false;
-			endCmd();
+			break;
+		default: // next indexpulse passed
+			endWriteTrackCmd();
+			break;
 		}
 	}
 	//PRT_DEBUG("WD2793::getDTRQ() " << DRQ);
 	return DRQ;
 }
 
-bool WD2793::peekDTRQ(const EmuTime& /*time*/)
+bool WD2793::peekDTRQ(EmuTime::param /*time*/)
 {
 	return DRQ; // TODO can be improved
 }
 
-bool WD2793::getIRQ(const EmuTime& /*time*/)
+bool WD2793::getIRQ(EmuTime::param /*time*/)
 {
 	//PRT_DEBUG("WD2793::getIRQ() " << INTRQ);
 	return INTRQ | immediateIRQ;
 }
 
-bool WD2793::peekIRQ(const EmuTime& time)
+bool WD2793::peekIRQ(EmuTime::param time)
 {
 	return getIRQ(time);
 }
@@ -128,13 +131,13 @@ void WD2793::resetIRQ()
 	INTRQ = false;
 }
 
-void WD2793::setDRQ(bool drq, const EmuTime& time)
+void WD2793::setDRQ(bool drq, EmuTime::param time)
 {
 	DRQ = drq;
 	DRQTimer.advance(time);
 }
 
-void WD2793::setCommandReg(byte value, const EmuTime& time)
+void WD2793::setCommandReg(byte value, EmuTime::param time)
 {
 	//PRT_DEBUG("WD2793::setCommandReg() 0x" << std::hex << (int)value);
 	removeSyncPoint(SCHED_FSM);
@@ -173,7 +176,7 @@ void WD2793::setCommandReg(byte value, const EmuTime& time)
 	}
 }
 
-byte WD2793::getStatusReg(const EmuTime& time)
+byte WD2793::getStatusReg(EmuTime::param time)
 {
 	if (((commandReg & 0x80) == 0) || ((commandReg & 0xF0) == 0xD0)) {
 		// Type I or type IV command
@@ -181,13 +184,13 @@ byte WD2793::getStatusReg(const EmuTime& time)
 		if (drive.indexPulse(time)) {
 			statusReg |=  INDEX;
 		}
-		if (drive.track00(time)) {
+		if (drive.isTrack00()) {
 			statusReg |=  TRACK00;
 		}
 		if (drive.headLoaded(time)) {
 			statusReg |=  HEAD_LOADED;
 		}
-		if (drive.writeProtected()) {
+		if (drive.isWriteProtected()) {
 			statusReg |=  WRITE_PROTECTED;
 		}
 	} else {
@@ -199,7 +202,7 @@ byte WD2793::getStatusReg(const EmuTime& time)
 		}
 	}
 
-	if (drive.ready()) {
+	if (drive.isReady()) {
 		statusReg &= ~NOT_READY;
 	} else {
 		statusReg |=  NOT_READY;
@@ -210,50 +213,50 @@ byte WD2793::getStatusReg(const EmuTime& time)
 	return statusReg;
 }
 
-byte WD2793::peekStatusReg(const EmuTime& time)
+byte WD2793::peekStatusReg(EmuTime::param time)
 {
 	return getStatusReg(time);
 }
 
-void WD2793::setTrackReg(byte value, const EmuTime& /*time*/)
+void WD2793::setTrackReg(byte value, EmuTime::param /*time*/)
 {
 	//PRT_DEBUG("WD2793::setTrackReg() 0x" << std::hex << (int)value);
 	trackReg = value;
 }
 
-byte WD2793::getTrackReg(const EmuTime& /*time*/)
+byte WD2793::getTrackReg(EmuTime::param /*time*/)
 {
 	return trackReg;
 }
 
-byte WD2793::peekTrackReg (const EmuTime& time)
+byte WD2793::peekTrackReg (EmuTime::param time)
 {
 	return getTrackReg(time);
 }
 
-void WD2793::setSectorReg(byte value, const EmuTime& /*time*/)
+void WD2793::setSectorReg(byte value, EmuTime::param /*time*/)
 {
 	//PRT_DEBUG("WD2793::setSectorReg() 0x" << std::hex << (int)value);
 	sectorReg = value;
 }
 
-byte WD2793::getSectorReg(const EmuTime& /*time*/)
+byte WD2793::getSectorReg(EmuTime::param /*time*/)
 {
 	return sectorReg;
 }
 
-byte WD2793::peekSectorReg(const EmuTime& time)
+byte WD2793::peekSectorReg(EmuTime::param time)
 {
 	return getSectorReg(time);
 }
 
-void WD2793::setDataReg(byte value, const EmuTime& time)
+void WD2793::setDataReg(byte value, EmuTime::param time)
 {
 	//PRT_DEBUG("WD2793::setDataReg() 0x" << std::hex << (int)value);
 	// TODO Is this also true in case of sector write?
 	//      Not so according to ASM of brMSX
 	dataReg = value;
-	if ((commandReg & 0xE0) == 0xA0) {
+	if (((commandReg & 0xE0) == 0xA0) && (statusReg & BUSY)) {
 		// WRITE SECTOR
 		dataBuffer[dataCurrent] = value;
 		dataCurrent++;
@@ -295,38 +298,23 @@ void WD2793::setDataReg(byte value, const EmuTime& time)
 				return;
 			}
 		}
-	} else if ((commandReg & 0xF0) == 0xF0) {
+	} else if (((commandReg & 0xF0) == 0xF0) && (statusReg & BUSY)) {
 		// WRITE TRACK
 		if (!formatting) {
 			return;
 		}
-		if (needInitWriteTrack) {
-			// cannot do this in writeTrackCmd() because then
-			// the correct drive is not yet selected
-			PRT_DEBUG("WD2793: initWriteTrack()");
-			needInitWriteTrack = false;
-			drive.initWriteTrack();
-		}
 		setDRQ(false, time);
 
-		//indexmark related timing
-		int pulses = drive.indexPulseCount(commandStart, time);
-		switch (pulses) {
+		// indexmark related timing
+		switch (drive.indexPulseCount(commandStart, time)) {
 		case 0: // no index pulse yet
 			break;
 		case 1: // first index pulse passed
-			try {
-				drive.writeTrackData(value);
-			} catch (MSXException& e) {
-				// ignore
-			}
+			assert(dataCurrent < Disk::RAWTRACK_SIZE);
+			dataBuffer[dataCurrent++] = value;
 			break;
 		default: // next indexpulse passed
-			dataAvailable = 0; //return correct DTR
-			dataCurrent = 0;
-			DRQ = false;
-			formatting = false;
-			endCmd();
+			endWriteTrackCmd();
 			break;
 		}
 		/* followin switch stement belongs in the backend, since
@@ -358,17 +346,17 @@ void WD2793::setDataReg(byte value, const EmuTime& time)
 			PRT_DEBUG("two CRC characters");
 			break;
 		 default:
-			//Normal write to track
+			// Normal write to track
 			break;
 		 }
-		 //shouldn't been done here !!
-		 statusReg &= ~0x03;	// reset status on Busy and DRQ
+		 // shouldn't be done here!!
+		 statusReg &= ~0x03; // reset status on Busy and DRQ
 		 */
 
 
 		/*
 		   if (indexmark) {
-		   statusReg &= ~0x03;	// reset status on Busy and DRQ
+		   statusReg &= ~0x03; // reset status on Busy and DRQ
 		   setIRQ();
 		   DRQ = false;
 		   }
@@ -376,7 +364,7 @@ void WD2793::setDataReg(byte value, const EmuTime& time)
 	}
 }
 
-byte WD2793::getDataReg(const EmuTime& time)
+byte WD2793::getDataReg(EmuTime::param time)
 {
 	if (((commandReg & 0xE0) == 0x80) && (statusReg & BUSY)) {
 		// READ SECTOR
@@ -400,7 +388,7 @@ byte WD2793::getDataReg(const EmuTime& time)
 	return dataReg;
 }
 
-byte WD2793::peekDataReg(const EmuTime& /*time*/)
+byte WD2793::peekDataReg(EmuTime::param /*time*/)
 {
 	if (((commandReg & 0xE0) == 0x80) && (statusReg & BUSY)) {
 		// READ SECTOR
@@ -430,20 +418,20 @@ void WD2793::tryToReadSector()
 		transferring = true;
 	} catch (MSXException& e) {
 		PRT_DEBUG("WD2793: read sector failed: " << e.getMessage());
-		DRQ = false;	// TODO data not ready (read error)
-		statusReg = 0;	// reset flags
+		DRQ = false; // TODO data not ready (read error)
+		statusReg = 0; // reset flags
 	}
 }
 
 
-void WD2793::schedule(FSMState state, const EmuTime& time)
+void WD2793::schedule(FSMState state, EmuTime::param time)
 {
 	assert(!pendingSyncPoint(SCHED_FSM));
 	fsmState = state;
 	setSyncPoint(time, SCHED_FSM);
 }
 
-void WD2793::executeUntil(const EmuTime& time, int userData)
+void WD2793::executeUntil(EmuTime::param time, int userData)
 {
 	if (userData == SCHED_IDX_IRQ) {
 		INTRQ = true;
@@ -503,7 +491,7 @@ const std::string& WD2793::schedName() const
 	return name;
 }
 
-void WD2793::startType1Cmd(const EmuTime& time)
+void WD2793::startType1Cmd(EmuTime::param time)
 {
 	statusReg &= ~(SEEK_ERROR | CRC_ERROR);
 	statusReg |= BUSY;
@@ -541,7 +529,7 @@ void WD2793::startType1Cmd(const EmuTime& time)
 	}
 }
 
-void WD2793::seek(const EmuTime& time)
+void WD2793::seek(EmuTime::param time)
 {
 	if (trackReg == dataReg) {
 		endType1Cmd();
@@ -551,7 +539,7 @@ void WD2793::seek(const EmuTime& time)
 	}
 }
 
-void WD2793::step(const EmuTime& time)
+void WD2793::step(EmuTime::param time)
 {
 	const int timePerStep[4] = {
 		// in ms, in case a 1MHz clock is used (as in MSX)
@@ -566,18 +554,18 @@ void WD2793::step(const EmuTime& time)
 			trackReg--;
 		}
 	}
-	if (!directionIn && drive.track00(time)) {
+	if (!directionIn && drive.isTrack00()) {
 		trackReg = 0;
 		endType1Cmd();
 	} else {
 		drive.step(directionIn, time);
-		Clock<1000> next(time);	// ms
+		Clock<1000> next(time); // ms
 		next += timePerStep[commandReg & STEP_SPEED];
 		schedule(FSM_SEEK, next.getTime());
 	}
 }
 
-void WD2793::seekNext(const EmuTime& time)
+void WD2793::seekNext(EmuTime::param time)
 {
 	if ((commandReg & 0xE0) == 0x00) {
 		// Restore or seek
@@ -597,22 +585,22 @@ void WD2793::endType1Cmd()
 }
 
 
-void WD2793::startType2Cmd(const EmuTime& time)
+void WD2793::startType2Cmd(EmuTime::param time)
 {
 	statusReg &= ~(LOST_DATA   | RECORD_NOT_FOUND |
 	               RECORD_TYPE | WRITE_PROTECTED);
 	statusReg |= BUSY;
 	setDRQ(false, time);
 
-	if (!drive.ready()) {
+	if (!drive.isReady()) {
 		endCmd();
 	} else {
 		// WD2795/WD2797 would now set SSO output
 		drive.setHeadLoaded(true, time);
 
 		if (commandReg & E_FLAG) {
-			Clock<1000> next(time);	// ms
-			next += 30;	// when 1MHz clock
+			Clock<1000> next(time); // ms
+			next += 30; // when 1MHz clock
 			schedule(FSM_TYPE2_WAIT_LOAD, next.getTime());
 		} else {
 			type2WaitLoad(time);
@@ -620,7 +608,7 @@ void WD2793::startType2Cmd(const EmuTime& time)
 	}
 }
 
-void WD2793::type2WaitLoad(const EmuTime& time)
+void WD2793::type2WaitLoad(EmuTime::param time)
 {
 	// TODO wait till head loaded, I arbitrarily took 1ms delay
 	Clock<1000> next(time);
@@ -628,9 +616,9 @@ void WD2793::type2WaitLoad(const EmuTime& time)
 	schedule(FSM_TYPE2_LOADED, next.getTime());
 }
 
-void WD2793::type2Loaded(const EmuTime& time)
+void WD2793::type2Loaded(EmuTime::param time)
 {
-	if (((commandReg & 0xE0) == 0xA0) && (drive.writeProtected())) {
+	if (((commandReg & 0xE0) == 0xA0) && (drive.isWriteProtected())) {
 		// write command and write protected
 		PRT_DEBUG("WD2793: write protected");
 		statusReg |= WRITE_PROTECTED;
@@ -644,22 +632,22 @@ void WD2793::type2Loaded(const EmuTime& time)
 void WD2793::type2Rotated()
 {
 	switch (commandReg & 0xF0) {
-		case 0x80: //read sector
-		case 0x90: //read sector (multi)
+		case 0x80: // read sector
+		case 0x90: // read sector (multi)
 			tryToReadSector();
 			break;
 
 		case 0xA0: // write sector
 		case 0xB0: // write sector (multi)
 			dataCurrent = 0;
-			dataAvailable = 512;	// TODO should come from sector header
-			DRQ = true;	// data ready to be written
+			dataAvailable = 512; // TODO should come from sector header
+			DRQ = true; // data ready to be written
 			transferring = true;
 			break;
 	}
 }
 
-void WD2793::startType3Cmd(const EmuTime& time)
+void WD2793::startType3Cmd(EmuTime::param time)
 {
 	//PRT_DEBUG("WD2793 start type 3 command");
 	statusReg &= ~(LOST_DATA | RECORD_NOT_FOUND | RECORD_TYPE);
@@ -667,15 +655,15 @@ void WD2793::startType3Cmd(const EmuTime& time)
 	setDRQ(false, time);
 	commandStart = time; // done again later
 
-	if (!drive.ready()) {
+	if (!drive.isReady()) {
 		endCmd();
 	} else {
 		drive.setHeadLoaded(true, time);
 		// WD2795/WD2797 would now set SSO output
 
 		if (commandReg & E_FLAG) {
-			Clock<1000> next(time);	// ms
-			next += 30;	// when 1MHz clock
+			Clock<1000> next(time); // ms
+			next += 30; // when 1MHz clock
 			schedule(FSM_TYPE3_WAIT_LOAD, next.getTime());
 		} else {
 			type3WaitLoad(time);
@@ -683,7 +671,7 @@ void WD2793::startType3Cmd(const EmuTime& time)
 	}
 }
 
-void WD2793::type3WaitLoad(const EmuTime& time)
+void WD2793::type3WaitLoad(EmuTime::param time)
 {
 	// TODO wait till head loaded, I arbitrarily took 1ms delay
 	Clock<1000> next(time);
@@ -691,7 +679,7 @@ void WD2793::type3WaitLoad(const EmuTime& time)
 	schedule(FSM_TYPE3_LOADED, next.getTime());
 }
 
-void WD2793::type3Loaded(const EmuTime& time)
+void WD2793::type3Loaded(EmuTime::param time)
 {
 
 	// TODO TG43 update
@@ -722,25 +710,42 @@ void WD2793::readTrackCmd()
 	endCmd();
 }
 
-void WD2793::writeTrackCmd(const EmuTime& time)
+void WD2793::writeTrackCmd(EmuTime::param time)
 {
 	PRT_DEBUG("WD2793 command: write track");
 
-	if (drive.writeProtected()) {
+	if (drive.isWriteProtected()) {
 		// write track command and write protected
 		PRT_DEBUG("WD2793: write protected");
 		statusReg |= WRITE_PROTECTED;
 		endCmd();
 	} else {
 		// TODO wait for indexPulse
-
-		needInitWriteTrack = true;
 		formatting = true;
+		dataCurrent = 0;
+		memset(dataBuffer, 0, Disk::RAWTRACK_SIZE);
 		setDRQ(true, time);
 	}
 }
 
-void WD2793::startType4Cmd(const EmuTime& time)
+void WD2793::endWriteTrackCmd()
+{
+	try {
+		drive.writeTrackData(dataBuffer);
+	} catch (MSXException& e) {
+		// Ignore. Should rarely happen, because
+		// write-protected is already checked at the
+		// beginning of write-track command (maybe
+		// when disk is swapped during format)
+	}
+	dataAvailable = 0; // return correct DTR
+	dataCurrent = 0;
+	DRQ = false;
+	formatting = false;
+	endCmd();
+}
+
+void WD2793::startType4Cmd(EmuTime::param time)
 {
 	// Force interrupt
 	PRT_DEBUG("WD2793 command: Force interrupt");
@@ -754,7 +759,7 @@ void WD2793::startType4Cmd(const EmuTime& time)
 	if (flags == 0x00) {
 		immediateIRQ = false;
 	}
-	if ((flags & IDX_IRQ) && drive.ready()) {
+	if ((flags & IDX_IRQ) && drive.isReady()) {
 		setSyncPoint(drive.getTimeTillIndexPulse(time), SCHED_IDX_IRQ);
 	} else {
 		removeSyncPoint(SCHED_IDX_IRQ);
@@ -764,7 +769,7 @@ void WD2793::startType4Cmd(const EmuTime& time)
 	}
 
 	setDRQ(false, time);
-	statusReg &= ~BUSY;	// reset status on Busy
+	statusReg &= ~BUSY; // reset status on Busy
 }
 
 void WD2793::endCmd()
@@ -773,5 +778,46 @@ void WD2793::endCmd()
 	statusReg &= ~BUSY;
 }
 
+
+static enum_string<WD2793::FSMState> fsmStateInfo[] = {
+	{ "NONE",            WD2793::FSM_NONE },
+	{ "SEEK",            WD2793::FSM_SEEK },
+	{ "TYPE2_WAIT_LOAD", WD2793::FSM_TYPE2_WAIT_LOAD },
+	{ "TYPE2_LOADED",    WD2793::FSM_TYPE2_LOADED },
+	{ "TYPE2_ROTATED",   WD2793::FSM_TYPE2_ROTATED },
+	{ "TYPE3_WAIT_LOAD", WD2793::FSM_TYPE3_WAIT_LOAD },
+	{ "TYPE3_LOADED",    WD2793::FSM_TYPE3_LOADED },
+	{ "IDX_IRQ",         WD2793::FSM_IDX_IRQ }
+};
+SERIALIZE_ENUM(WD2793::FSMState, fsmStateInfo);
+
+template<typename Archive>
+void WD2793::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<Schedulable>(*this);
+
+	ar.serialize("commandStart", commandStart);
+	ar.serialize("DRQTimer", DRQTimer);
+
+	ar.serialize("fsmState", fsmState);
+	ar.serialize("statusReg", statusReg);
+	ar.serialize("commandReg", commandReg);
+	ar.serialize("sectorReg", sectorReg);
+	ar.serialize("trackReg", trackReg);
+	ar.serialize("dataReg", dataReg);
+
+	ar.serialize("directionIn", directionIn);
+	ar.serialize("INTRQ", INTRQ);
+	ar.serialize("immediateIRQ", immediateIRQ);
+	ar.serialize("DRQ", DRQ);
+	ar.serialize("transferring", transferring);
+	ar.serialize("formatting", formatting);
+
+	ar.serialize_blob("dataBuffer", dataBuffer, sizeof(dataBuffer));
+	ar.serialize("dataCurrent", dataCurrent);
+	ar.serialize("dataAvailable", dataAvailable);
+}
+INSTANTIATE_SERIALIZE_METHODS(WD2793);
+
 } // namespace openmsx
 
