@@ -1,4 +1,4 @@
-// $Id: FileOperations.cc 6824 2007-08-08 15:20:19Z m9710797 $
+// $Id: FileOperations.cc 8351 2008-11-12 18:20:29Z m9710797 $
 
 #ifdef	_WIN32
 #define WIN32_LEAN_AND_MEAN
@@ -8,25 +8,33 @@
 #include <io.h>
 #include <direct.h>
 #include <ctype.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdlib>
+#include <cstring>
 #include <algorithm>
-#define	MAXPATHLEN	MAX_PATH
 #define	mode_t	unsigned short int
 #else
 #include <sys/types.h>
 #include <pwd.h>
 #endif
 
+#if defined(PATH_MAX)
+#define MAXPATHLEN PATH_MAX
+#elif defined(MAX_PATH)
+#define MAXPATHLEN MAX_PATH
+#else
+#define MAXPATHLEN 4096
+#endif
+
+
 #ifdef __APPLE__
 #include <Carbon/Carbon.h>
 #endif
 
 #include "ReadDir.hh"
-#include "build-info.hh"
 #include "FileOperations.hh"
 #include "FileException.hh"
 #include "StringOp.hh"
+#include "build-info.hh"
 #include <sstream>
 #include <cerrno>
 #include <cstdlib>
@@ -42,7 +50,8 @@ namespace FileOperations {
 
 #ifdef __APPLE__
 
-std::string findShareDir() {
+static std::string findShareDir()
+{
 	// Find bundle location:
 	// for an app folder, this is the outer directory,
 	// for an unbundled executable, it is the executable itself.
@@ -236,9 +245,30 @@ string getBaseName(const string& path)
 	}
 }
 
-string getNativePath(const string &path)
+string join(const string& part1, const string& part2)
 {
-#if	defined(_WIN32)
+	if (isAbsolutePath(part2)) {
+		return part2;
+	}
+	if (part1.empty() || (*part1.rbegin() == '/')) {
+		return part1 + part2;
+	}
+	return part1 + '/' + part2;
+}
+string join(const string& part1, const string& part2, const string& part3)
+{
+	return join(join(part1, part2), part3);
+}
+
+string join(const string& part1, const string& part2,
+            const string& part3, const string& part4)
+{
+	return join(join(join(part1, part2), part3), part4);
+}
+
+string getNativePath(const string& path)
+{
+#ifdef _WIN32
 	string result(path);
 	replace(result.begin(), result.end(), '/', '\\');
 	return result;
@@ -247,9 +277,9 @@ string getNativePath(const string &path)
 #endif
 }
 
-string getConventionalPath(const string &path)
+string getConventionalPath(const string& path)
 {
-#if	defined(_WIN32)
+#ifdef _WIN32
 	string result(path);
 	replace(result.begin(), result.end(), '\\', '/');
 	return result;
@@ -258,9 +288,29 @@ string getConventionalPath(const string &path)
 #endif
 }
 
+string getCurrentWorkingDirectory()
+{
+	char buf[MAXPATHLEN];
+	if (!getcwd(buf, MAXPATHLEN)) {
+		throw FileException("Couldn't get current working directory.");
+	}
+	return buf;
+}
+
+string getAbsolutePath(const string& path)
+{
+	// In rare cases getCurrentWorkingDirectory() can throw,
+	// so only call it when really necessary.
+	if (isAbsolutePath(path)) {
+		return path;
+	}
+	string currentDir = getCurrentWorkingDirectory();
+	return join(currentDir, path);
+}
+
 bool isAbsolutePath(const string& path)
 {
-#if	defined(_WIN32)
+#ifdef _WIN32
 	if ((path.size() >= 3) && (path[1] == ':') && (path[2] == '/')) {
 		char drive = tolower(path[0]);
 		if (('a' <= drive) && (drive <= 'z')) {
@@ -273,7 +323,7 @@ bool isAbsolutePath(const string& path)
 
 string getUserHomeDir(const string& username)
 {
-#if	defined(_WIN32)
+#ifdef _WIN32
 	if (username.empty()); // ignore parameter, avoid warning
 	static string userDir;
 	if (userDir.empty()) {
@@ -281,7 +331,7 @@ string getUserHomeDir(const string& username)
 		if (sh32dll) {
 			FARPROC funcp = GetProcAddress(sh32dll, "SHGetSpecialFolderPathA");
 			if (funcp) {
-				char p[MAX_PATH + 1];
+				char p[MAXPATHLEN + 1];
 				int res = reinterpret_cast<BOOL(*)(HWND, LPSTR, int, BOOL)>(funcp)(0, p, CSIDL_PERSONAL, 1);
 				if (res == TRUE) {
 					userDir = getConventionalPath(p);
@@ -292,7 +342,7 @@ string getUserHomeDir(const string& username)
 		if (userDir.empty()) {
 			// workaround for Win95 w/o IE4(||later)
 			userDir = getSystemDataDir();
-			userDir.erase(userDir.length() - 6, 6);	// "/share"
+			userDir.erase(userDir.length() - 6, 6); // "/share"
 		}
 		if ((userDir.length() == 3) && (userDir.substr(1) == ":/")){
 			char drive = tolower(userDir[0]);
@@ -302,6 +352,16 @@ string getUserHomeDir(const string& username)
 		}
 	}
 	return userDir;
+#elif PLATFORM_GP2X
+	return ""; // TODO figure out why stuff below doesn't work
+	// We cannot use generic implementation below, because for some
+	// reason getpwuid() and getpwnam() cannot be used in statically
+	// linked applications.
+	const char* dir = getenv("HOME");
+	if (!dir) {
+		dir = "/root";
+	}
+	return dir;
 #else
 	const char* dir = NULL;
 	struct passwd* pw = NULL;
@@ -322,7 +382,7 @@ string getUserHomeDir(const string& username)
 
 const string& getUserOpenMSXDir()
 {
-#if defined(_WIN32)
+#ifdef _WIN32
 	static const string OPENMSX_DIR = expandTilde("~/openMSX");
 #else
 	static const string OPENMSX_DIR = expandTilde("~/.openMSX");
@@ -346,10 +406,10 @@ string getSystemDataDir()
 	}
 
 	string newValue;
-#if defined(_WIN32)
-	char p[MAX_PATH + 1];
-	int res = GetModuleFileNameA(NULL, p, MAX_PATH);
-	if ((res == 0) || (res == MAX_PATH)) {
+#ifdef _WIN32
+	char p[MAXPATHLEN + 1];
+	int res = GetModuleFileNameA(NULL, p, MAXPATHLEN);
+	if ((res == 0) || (res == MAXPATHLEN)) {
 		throw FatalError("Cannot detect openMSX directory.");
 	}
 	if (!strrchr(p, '\\')) {
@@ -375,8 +435,8 @@ string expandCurrentDirFromDrive(const string& path)
 		// get current directory for this drive
 		unsigned char drive = tolower(path[0]);
 		if (('a' <= drive) && (drive <= 'z')) {
-			char buffer[MAX_PATH + 1];
-			if (_getdcwd(drive - 'a' + 1, buffer, MAX_PATH) != NULL) {
+			char buffer[MAXPATHLEN + 1];
+			if (_getdcwd(drive - 'a' + 1, buffer, MAXPATHLEN) != NULL) {
 				result = buffer;
 				result = getConventionalPath(result);
 				if (*result.rbegin() != '/') {
@@ -395,21 +455,21 @@ string expandCurrentDirFromDrive(const string& path)
 bool isRegularFile(const string& filename)
 {
 	struct stat st;
-	int ret = stat(filename.c_str(), &st);
+	int ret = stat(expandTilde(filename).c_str(), &st);
 	return (ret == 0) && S_ISREG(st.st_mode);
 }
 
 bool isDirectory(const string& directory)
 {
 	struct stat st;
-	int ret = stat(directory.c_str(), &st);
+	int ret = stat(expandTilde(directory).c_str(), &st);
 	return (ret == 0) && S_ISDIR(st.st_mode);
 }
 
 bool exists(const string& filename)
 {
 	struct stat st;
-	return stat(filename.c_str(), &st) == 0;
+	return stat(expandTilde(filename).c_str(), &st) == 0;
 }
 
 static int getNextNum(dirent* d, const string& prefix, const string& extension,
