@@ -1,4 +1,4 @@
-// $Id: FilePool.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: FilePool.cc 8178 2008-08-26 17:57:53Z m9710797 $
 
 #include "FilePool.hh"
 #include "File.hh"
@@ -8,6 +8,7 @@
 #include "ReadDir.hh"
 #include "Date.hh"
 #include "SettingsConfig.hh"
+#include "XMLElement.hh"
 #include "sha1.hh"
 #include <fstream>
 #include <cassert>
@@ -27,8 +28,8 @@ const char* const FILE_CACHE = "/.filecache";
 
 FilePool::FilePool(SettingsConfig& settingsConfig)
 {
-	const XMLElement* config = settingsConfig.findChild("RomPool");
-	if (config) {
+	if (const XMLElement* config =
+			settingsConfig.getXMLElement().findChild("RomPool")) {
 		XMLElement::Children dirs;
 		config->getChildren("directory", dirs);
 		for (XMLElement::Children::const_iterator it = dirs.begin();
@@ -38,10 +39,11 @@ FilePool::FilePool(SettingsConfig& settingsConfig)
 		}
 	}
 	SystemFileContext context;
-	const vector<string>& paths = context.getPaths();
+	CommandController* controller = NULL; // ok for SystemFileContext
+	vector<string> paths = context.getPaths(*controller);
 	for (vector<string>::const_iterator it = paths.begin();
 	     it != paths.end(); ++it) {
-		string dir = *it + "/systemroms";
+		string dir = FileOperations::join(*it, "systemroms");
 		directories.push_back(dir);
 	}
 
@@ -69,9 +71,7 @@ static bool parse(const string& line, string& sha1, time_t& time, string& filena
 static auto_ptr<File> calcSha1sum(const string& filename, string& sum)
 {
 	auto_ptr<File> file(new File(filename));
-	SHA1 sha1;
-	sha1.update(file->mmap(), file->getSize());
-	sum = sha1.hex_digest();
+	sum = SHA1::calc(file->mmap(), file->getSize());
 	return file;
 }
 
