@@ -1,4 +1,4 @@
-// $Id: LocalFile.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: LocalFile.cc 8229 2008-09-15 18:06:05Z m9710797 $
 
 #include "probed_defs.hh"
 #include <unistd.h>
@@ -11,6 +11,8 @@
 #include "FileOperations.hh"
 #include "FileException.hh"
 #include "PreCacheFile.hh"
+#include <cstring> // for strchr
+#include <cassert>
 
 using std::string;
 
@@ -58,6 +60,16 @@ LocalFile::LocalFile(const string& filename_, File::OpenMode mode)
 	}
 }
 
+LocalFile::LocalFile(const std::string& filename, const char* mode)
+{
+	assert(strchr(mode, 'b'));
+	const string name = FileOperations::getNativePath(filename);
+	file = fopen(name.c_str(), mode);
+	if (!file) {
+		throw FileException("Error opening file \"" + filename + "\"");
+	}
+}
+
 LocalFile::~LocalFile()
 {
 	munmap();
@@ -74,17 +86,19 @@ void LocalFile::read(void* buffer, unsigned num)
 		throw FileException("Read beyond end of file");
 	}
 
-	fread(buffer, 1, num, file);
-	if (ferror(file)) {
-		throw FileException("Error reading file");
+	if (fread(buffer, 1, num, file) != num) {
+		if (ferror(file)) {
+			throw FileException("Error reading file");
+		}
 	}
 }
 
 void LocalFile::write(const void* buffer, unsigned num)
 {
-	fwrite(buffer, 1, num, file);
-	if (ferror(file)) {
-		throw FileException("Error writing file");
+	if (fwrite(buffer, 1, num, file) != num) {
+		if (ferror(file)) {
+			throw FileException("Error writing file");
+		}
 	}
 }
 
@@ -93,8 +107,9 @@ byte* LocalFile::mmap(bool writeBack)
 {
 	if (!mmem) {
 		int flags = writeBack ? MAP_SHARED : MAP_PRIVATE;
-		mmem = ::mmap(0, getSize(), PROT_READ | PROT_WRITE,
-		              flags, fileno(file), 0);
+		mmem = static_cast<byte*>(
+		          ::mmap(0, getSize(), PROT_READ | PROT_WRITE,
+		                 flags, fileno(file), 0));
 		// MAP_FAILED is #define'd using an old-style cast, we
 		// have to redefine it ourselves to avoid a warning
 		void* MY_MAP_FAILED = reinterpret_cast<void*>(-1);
@@ -102,7 +117,7 @@ byte* LocalFile::mmap(bool writeBack)
 			throw FileException("Error mmapping file");
 		}
 	}
-	return static_cast<byte*>(mmem);
+	return mmem;
 }
 
 void LocalFile::munmap()
@@ -153,15 +168,10 @@ void LocalFile::flush()
 
 const string LocalFile::getURL() const
 {
-	// we could use the official file:// prefix, but we
-	// are dropping it, because it is also allowed to omit
-	// it when specifying a file. Maybe we want to drop
-	// support for URLs anyway?
-//	static const string prefix("file://");
-	return /*prefix + */filename;
+	return filename;
 }
 
-const string LocalFile::getLocalName()
+const string LocalFile::getLocalReference()
 {
 	return filename;
 }
