@@ -1,4 +1,4 @@
-// $Id: PreCacheFile.cc 5033 2006-01-11 19:30:28Z m9710797 $
+// $Id: PreCacheFile.cc 6786 2007-08-04 10:29:59Z m9710797 $
 
 #include "PreCacheFile.hh"
 #include <cstdio>
@@ -21,15 +21,15 @@ void PreCacheFile::run()
 		// don't pre-cache non regular files (e.g. /dev/fd0)
 		return;
 	}
-	
+
 	FILE* file = fopen(name.c_str(), "rb");
 	if (!file) return;
 
 	fseek(file, 0, SEEK_END);
-	unsigned size = (unsigned)ftell(file);
+	unsigned size = unsigned(ftell(file));
 	if (size < 1024 * 1024) {
 		// only pre-cache small files
-		
+
 		const unsigned BLOCK_SIZE = 4096;
 		unsigned block = 0;
 		unsigned repeat = 0;
