@@ -1,12 +1,13 @@
-// $Id: AbstractIDEDevice.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: AbstractIDEDevice.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "AbstractIDEDevice.hh"
 #include "MSXMotherBoard.hh"
-#include "EventDistributor.hh"
-#include "LedEvent.hh"
+#include "LedStatus.hh"
 #include "Version.hh"
+#include "serialize.hh"
 #include <cassert>
 #include <cstring>
+#include <cstdio>
 
 namespace openmsx {
 
@@ -15,6 +16,7 @@ AbstractIDEDevice::AbstractIDEDevice(MSXMotherBoard& motherBoard_)
 {
 	transferRead = false;
 	transferWrite = false;
+	transferIdx = 0;
 }
 
 AbstractIDEDevice::~AbstractIDEDevice()
@@ -52,7 +54,7 @@ void AbstractIDEDevice::createSignature(bool preserveDevice)
 	}
 }
 
-void AbstractIDEDevice::reset(const EmuTime& /*time*/)
+void AbstractIDEDevice::reset(EmuTime::param /*time*/)
 {
 	errorReg = diagnostic();
 	statusReg = DRDY | DSC;
@@ -62,7 +64,7 @@ void AbstractIDEDevice::reset(const EmuTime& /*time*/)
 	setTransferWrite(false);
 }
 
-byte AbstractIDEDevice::readReg(nibble reg, const EmuTime& /*time*/)
+byte AbstractIDEDevice::readReg(nibble reg, EmuTime::param /*time*/)
 {
 	switch (reg) {
 	case 1: // error register
@@ -96,17 +98,17 @@ byte AbstractIDEDevice::readReg(nibble reg, const EmuTime& /*time*/)
 	case 15:// not used
 		return 0x7F;
 
-	case 0:	// data register, converted to readData by IDE interface
+	case 0: // data register, converted to readData by IDE interface
 	case 14:// alternate status reg, converted to read from normal
 		// status register by IDE interface
 	default:
 		assert(false);
-		return 0x7F;	// avoid warning
+		return 0x7F; // avoid warning
 	}
 }
 
 void AbstractIDEDevice::writeReg(
-	nibble reg, byte value, const EmuTime& /*time*/
+	nibble reg, byte value, EmuTime::param /*time*/
 	)
 {
 	switch (reg) {
@@ -160,14 +162,16 @@ void AbstractIDEDevice::writeReg(
 	}
 }
 
-word AbstractIDEDevice::readData(const EmuTime& /*time*/)
+word AbstractIDEDevice::readData(EmuTime::param /*time*/)
 {
+	assert(transferIdx < sizeof(buffer));
 	if (!transferRead) {
 		// no read in progress
 		return 0x7F7F;
 	}
-	word result = transferPntr[0] + (transferPntr[1] << 8);
-	transferPntr += 2;
+	word result = (buffer[transferIdx + 0] << 0) +
+	              (buffer[transferIdx + 1] << 8);
+	transferIdx += 2;
 	bufferLeft -= 2;
 	if (bufferLeft == 0) {
 		if (transferCount == 0) {
@@ -188,22 +192,23 @@ void AbstractIDEDevice::readNextBlock()
 	bufferLeft = readBlockStart(
 		buffer, std::min<unsigned>(sizeof(buffer), transferCount));
 	assert((bufferLeft & 1) == 0);
-	transferPntr = buffer;
+	transferIdx = 0;
 	transferCount -= bufferLeft;
 }
 
-void AbstractIDEDevice::writeData(word value, const EmuTime& /*time*/)
+void AbstractIDEDevice::writeData(word value, EmuTime::param /*time*/)
 {
+	assert(transferIdx < sizeof(buffer));
 	if (!transferWrite) {
 		// no write in progress
 		return;
 	}
-	transferPntr[0] = value & 0xFF;
-	transferPntr[1] = value >> 8;
-	transferPntr += 2;
+	buffer[transferIdx + 0] = value & 0xFF;
+	buffer[transferIdx + 1] = value >> 8;
+	transferIdx += 2;
 	bufferLeft -= 2;
 	if (bufferLeft == 0) {
-		unsigned bytesInBuffer = transferPntr - buffer;
+		unsigned bytesInBuffer = transferIdx;
 		if (transferCount == 0) {
 			// End of transfer.
 			setTransferWrite(false);
@@ -220,7 +225,7 @@ void AbstractIDEDevice::writeData(word value, const EmuTime& /*time*/)
 
 void AbstractIDEDevice::writeNextBlock()
 {
-	transferPntr = buffer;
+	transferIdx = 0;
 	bufferLeft = std::min<unsigned>(sizeof(buffer), transferCount);
 	transferCount -= bufferLeft;
 }
@@ -343,7 +348,7 @@ byte* AbstractIDEDevice::startShortReadTransfer(unsigned count)
 	startReadTransfer();
 	transferCount = 0;
 	bufferLeft = count;
-	transferPntr = buffer;
+	transferIdx = 0;
 	memset(buffer, 0x00, count);
 	return buffer;
 }
@@ -388,9 +393,7 @@ void AbstractIDEDevice::setTransferRead(bool status)
 		transferRead = status;
 		if (!transferWrite) {
 			// (this is a bit of a hack!)
-			motherBoard.getEventDistributor().distributeEvent(
-				new LedEvent(LedEvent::FDD, transferRead,
-				             motherBoard));
+			motherBoard.getLedStatus().setLed(LedStatus::FDD, transferRead);
 		}
 	}
 }
@@ -401,9 +404,7 @@ void AbstractIDEDevice::setTransferWrite(bool status)
 		transferWrite = status;
 		if (!transferRead) {
 			// (this is a bit of a hack!)
-			motherBoard.getEventDistributor().distributeEvent(
-				new LedEvent(LedEvent::FDD, transferWrite,
-				             motherBoard));
+			motherBoard.getLedStatus().setLed(LedStatus::FDD, transferWrite);
 		}
 	}
 }
@@ -443,4 +444,27 @@ void AbstractIDEDevice::createIdentifyBlock(byte* buffer)
 	fillIdentifyBlock(buffer);
 }
 
+
+template<typename Archive>
+void AbstractIDEDevice::serialize(Archive& ar, unsigned /*version*/)
+{
+	// no need to serialize IDEDevice base class
+	ar.serialize_blob("buffer", buffer, sizeof(buffer));
+	ar.serialize("transferIdx", transferIdx);
+	ar.serialize("bufferLeft", bufferLeft);
+	ar.serialize("transferCount", transferCount);
+	ar.serialize("errorReg", errorReg);
+	ar.serialize("sectorCountReg", sectorCountReg);
+	ar.serialize("sectorNumReg", sectorNumReg);
+	ar.serialize("cylinderLowReg", cylinderLowReg);
+	ar.serialize("cylinderHighReg", cylinderHighReg);
+	ar.serialize("devHeadReg", devHeadReg);
+	ar.serialize("statusReg", statusReg);
+	ar.serialize("featureReg", featureReg);
+	ar.serialize("transferIdentifyBlock", transferIdentifyBlock);
+	ar.serialize("transferRead", transferRead);
+	ar.serialize("transferWrite", transferWrite);
+}
+INSTANTIATE_SERIALIZE_METHODS(AbstractIDEDevice);
+
 } // namespace openmsx
