@@ -1,25 +1,24 @@
-// $Id: HDImageCLI.cc 6572 2007-06-01 22:09:15Z manuelbi $
+// $Id: HDImageCLI.cc 7731 2008-04-03 19:58:10Z m9710797 $
 
 #include "HDImageCLI.hh"
-#include "Reactor.hh"
 #include "CommandController.hh"
 #include "CommandLineParser.hh"
 #include "TclObject.hh"
 #include "MSXException.hh"
 
-using std::list;
+using std::deque;
 using std::string;
 
 namespace openmsx {
 
 HDImageCLI::HDImageCLI(CommandLineParser& commandLineParser)
-	: commandController(commandLineParser.getReactor().getCommandController())
+	: commandController(commandLineParser.getCommandController())
 {
 	commandLineParser.registerOption("-hda", *this);
 	// TODO: offer more options in case you want to specify 2 hard disk images?
 }
 
-bool HDImageCLI::parseOption(const string& option, list<string>& cmdLine)
+bool HDImageCLI::parseOption(const string& option, deque<string>& cmdLine)
 {
 	string hd = option.substr(1); // hda
 	string filename = getArgument(option, cmdLine);
