@@ -1,4 +1,4 @@
-// $Id: IDECDROM.cc 6593 2007-06-03 23:12:19Z mthuurne $
+// $Id: IDECDROM.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "IDECDROM.hh"
 #include "MSXMotherBoard.hh"
@@ -8,10 +8,12 @@
 #include "RecordedCommand.hh"
 #include "CommandException.hh"
 #include "TclObject.hh"
-#include "MSXCliComm.hh"
+#include "CliComm.hh"
+#include "serialize.hh"
 #include <algorithm>
 #include <bitset>
 #include <cassert>
+#include <cstdio>
 
 using std::string;
 using std::vector;
@@ -22,11 +24,11 @@ namespace openmsx {
 class CDXCommand : public RecordedCommand
 {
 public:
-	CDXCommand(MSXCommandController& msxCommandController,
+	CDXCommand(CommandController& commandController,
 	           MSXEventDistributor& msxEventDistributor,
 	           Scheduler& scheduler, IDECDROM& cd);
 	virtual void execute(const std::vector<TclObject*>& tokens,
-		TclObject& result, const EmuTime& time);
+		TclObject& result, EmuTime::param time);
 	virtual string help(const vector<string>& tokens) const;
 	virtual void tabCompletion(vector<string>& tokens) const;
 private:
@@ -37,8 +39,7 @@ private:
 static const unsigned MAX_CD = 26;
 typedef std::bitset<MAX_CD> CDInUse;
 
-IDECDROM::IDECDROM(MSXMotherBoard& motherBoard_, const XMLElement& /*config*/,
-                   const EmuTime& /*time*/)
+IDECDROM::IDECDROM(MSXMotherBoard& motherBoard_, const XMLElement& /*config*/)
 	: AbstractIDEDevice(motherBoard_)
 	, motherBoard(motherBoard_)
 {
@@ -60,7 +61,7 @@ IDECDROM::IDECDROM(MSXMotherBoard& motherBoard_, const XMLElement& /*config*/,
 	}
 	name = string("cd") + char('a' + id);
 	cdInUse[id] = true;
-	cdxCommand.reset(new CDXCommand(motherBoard.getMSXCommandController(),
+	cdxCommand.reset(new CDXCommand(motherBoard.getCommandController(),
 	                                motherBoard.getMSXEventDistributor(),
 	                                motherBoard.getScheduler(), *this));
 
@@ -347,17 +348,17 @@ void IDECDROM::insert(const string& filename)
 
 // class CDXCommand
 
-CDXCommand::CDXCommand(MSXCommandController& msxCommandController,
+CDXCommand::CDXCommand(CommandController& commandController,
                        MSXEventDistributor& msxEventDistributor,
                        Scheduler& scheduler, IDECDROM& cd_)
-	: RecordedCommand(msxCommandController, msxEventDistributor,
+	: RecordedCommand(commandController, msxEventDistributor,
 	                  scheduler, cd_.name)
 	, cd(cd_)
 {
 }
 
 void CDXCommand::execute(const std::vector<TclObject*>& tokens, TclObject& result,
-				const EmuTime& /*time*/)
+				EmuTime::param /*time*/)
 {
 	if (tokens.size() == 1) {
 		File* file = cd.file.get();
@@ -382,8 +383,9 @@ void CDXCommand::execute(const std::vector<TclObject*>& tokens, TclObject& resul
 			}
 		}
 		try {
-			UserFileContext context(getCommandController());
-			string filename = context.resolve(tokens[fileToken]->getString());
+			UserFileContext context;
+			string filename = context.resolve(
+				getCommandController(), tokens[fileToken]->getString());
 			cd.insert(filename);
 			// return filename; // Note: the diskX command doesn't do this either, so this has not been converted to TclObject style here
 		} catch (FileException& e) {
@@ -408,8 +410,35 @@ void CDXCommand::tabCompletion(vector<string>& tokens) const
 	set<string> extra;
 	extra.insert("eject");
 	extra.insert("insert");
-	UserFileContext context(getCommandController());
-	completeFileName(tokens, context, extra);
+	UserFileContext context;
+	completeFileName(getCommandController(), tokens, context, extra);
 }
 
+
+template<typename Archive>
+void IDECDROM::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<AbstractIDEDevice>(*this);
+
+	string filename = file.get() ? file->getURL() : "";
+	ar.serialize("filename", filename);
+	if (ar.isLoader()) {
+		// re-insert CDROM before restoring 'mediaChanged', 'senseKey'
+		if (filename.empty()) {
+			eject();
+		} else {
+			insert(filename);
+		}
+	}
+
+	ar.serialize("byteCountLimit", byteCountLimit);
+	ar.serialize("transferOffset", transferOffset);
+	ar.serialize("senseKey", senseKey);
+	ar.serialize("readSectorData", readSectorData);
+	ar.serialize("remMedStatNotifEnabled", remMedStatNotifEnabled);
+	ar.serialize("mediaChanged", mediaChanged);
+}
+INSTANTIATE_SERIALIZE_METHODS(IDECDROM);
+REGISTER_POLYMORPHIC_INITIALIZER(IDEDevice, IDECDROM, "IDECDROM");
+
 } // namespace openmsx
