@@ -1,20 +1,20 @@
-// $Id: IDEHD.cc 6824 2007-08-08 15:20:19Z m9710797 $
+// $Id: IDEHD.cc 8237 2008-09-18 17:54:54Z m9710797 $
 
 #include "IDEHD.hh"
-#include "FileException.hh"
+#include "MSXException.hh"
 #include "MSXMotherBoard.hh"
 #include "DiskManipulator.hh"
+#include "serialize.hh"
 #include <cassert>
 
 namespace openmsx {
 
-IDEHD::IDEHD(MSXMotherBoard& motherBoard, const XMLElement& config,
-             const EmuTime& /*time*/)
+IDEHD::IDEHD(MSXMotherBoard& motherBoard, const XMLElement& config)
 	: HD(motherBoard, config)
 	, AbstractIDEDevice(motherBoard)
 	, diskManipulator(motherBoard.getDiskManipulator())
 {
-	diskManipulator.registerDrive(*this);
+	diskManipulator.registerDrive(*this, &motherBoard);
 }
 
 IDEHD::~IDEHD()
@@ -64,10 +64,10 @@ unsigned IDEHD::readBlockStart(byte* buffer, unsigned count)
 	try {
 		assert(count >= 512);
 		(void)count; // avoid warning
-		readLogicalSector(transferSectorNumber, buffer);
+		readSector(transferSectorNumber, buffer);
 		++transferSectorNumber;
 		return 512;
-	} catch (FileException& e) {
+	} catch (MSXException& e) {
 		abortReadTransfer(UNC);
 		return 0;
 	}
@@ -77,12 +77,12 @@ void IDEHD::writeBlockComplete(byte* buffer, unsigned count)
 {
 	try {
 		while (count != 0) {
-			writeLogicalSector(transferSectorNumber, buffer);
+			writeSector(transferSectorNumber, buffer);
 			++transferSectorNumber;
 			assert(count >= 512);
 			count -= 512;
 		}
-	} catch (FileException& e) {
+	} catch (MSXException& e) {
 		abortWriteTransfer(UNC);
 	}
 }
@@ -120,29 +120,16 @@ void IDEHD::executeCommand(byte cmd)
 	}
 }
 
-void IDEHD::readLogicalSector(unsigned sector, byte* buf)
-{
-	readFromImage(512 * sector, 512, buf);
-}
 
-void IDEHD::writeLogicalSector(unsigned sector, const byte* buf)
+template<typename Archive>
+void IDEHD::serialize(Archive& ar, unsigned /*version*/)
 {
-	writeToImage(512 * sector, 512, buf);
-}
-
-unsigned IDEHD::getNbSectors() const
-{
-	return getImageSize() / 512;
-}
-
-SectorAccessibleDisk* IDEHD::getSectorAccessibleDisk()
-{
-	return this;
-}
-
-const std::string& IDEHD::getContainerName() const
-{
-	return getName();
+	// don't serialize SectorAccessibleDisk, DiskContainer base classes
+	ar.template serializeBase<HD>(*this);
+	ar.template serializeBase<AbstractIDEDevice>(*this);
+	ar.serialize("transferSectorNumber", transferSectorNumber);
 }
+INSTANTIATE_SERIALIZE_METHODS(IDEHD);
+REGISTER_POLYMORPHIC_INITIALIZER(IDEDevice, IDEHD, "IDEHD");
 
 } // namespace openmsx
