@@ -1,4 +1,4 @@
-// $Id: MB89352.cc 6805 2007-08-06 12:01:49Z m9710797 $
+// $Id: MB89352.cc 8351 2008-11-12 18:20:29Z m9710797 $
 /* Ported from:
 ** Source: /cvsroot/bluemsx/blueMSX/Src/IoDevice/MB89352.c,v
 ** Revision: 1.9
@@ -21,6 +21,7 @@
 #include "XMLElement.hh"
 #include "MSXException.hh"
 #include "StringOp.hh"
+#include "serialize.hh"
 #include <cassert>
 #include <string>
 #include <cstring>
@@ -93,7 +94,7 @@ MB89352::MB89352(MSXMotherBoard& motherBoard, const XMLElement& config)
 	: buffer(SCSIDevice::BUFFER_SIZE)
 {
 	PRT_DEBUG("spc create");
-	//TODO: devBusy = false;
+	// TODO: devBusy = false;
 
 	// ALMOST COPY PASTED FROM WD33C93:
 
@@ -134,7 +135,6 @@ MB89352::MB89352(MSXMotherBoard& motherBoard, const XMLElement& config)
 
 MB89352::~MB89352()
 {
-    PRT_DEBUG("spc destroy");
 }
 
 void MB89352::disconnect()
@@ -165,7 +165,7 @@ void MB89352::softReset()
 	regs[15] = 0xFF;               // un mapped
 	memset(cdb, 0, 12);
 
-	pCdb   = cdb;
+	cdbIdx = 0;
 	bufIdx = 0;
 	phase  = SCSI::BUS_FREE;
 	disconnect();
@@ -219,7 +219,7 @@ void MB89352::setACKREQ(byte& value)
 		regs[REG_PSNS] = PSNS_ACK | PSNS_BSY | PSNS_DATAIN;
 		break;
 
-	case SCSI::DATA_OUT: //Transfer phase (data out)
+	case SCSI::DATA_OUT: // Transfer phase (data out)
 		buffer[bufIdx] = value;
 		++bufIdx;
 		regs[REG_PSNS] = PSNS_ACK | PSNS_BSY | PSNS_DATAOUT;
@@ -227,12 +227,12 @@ void MB89352::setACKREQ(byte& value)
 
 	case SCSI::COMMAND: // Command phase
 		if (counter < 0) {
-			//Initialize command routine
-			pCdb    = cdb;
+			// Initialize command routine
+			cdbIdx  = 0;
 			counter = (value < 0x20) ? 6 : ((value < 0xA0) ? 10 : 12);
 		}
-		*pCdb = value;
-		++pCdb;
+		cdb[cdbIdx] = value;
+		++cdbIdx;
 		regs[REG_PSNS] = PSNS_ACK | PSNS_BSY | PSNS_COMMAND;
 		break;
 
@@ -312,7 +312,7 @@ void MB89352::resetACKREQ()
 			regs[REG_PSNS] = PSNS_REQ | PSNS_BSY | PSNS_COMMAND;
 		} else {
 			bufIdx = 0; // reset buffer index
-			//TODO: devBusy = true;
+			// TODO: devBusy = true;
 			counter = dev[targetId]->executeCmd(cdb, phase, blockCounter);
 			switch (phase) {
 			case SCSI::DATA_IN:
@@ -331,7 +331,7 @@ void MB89352::resetACKREQ()
 				PRT_DEBUG("phase error");
 				break;
 			}
-			//TODO: devBusy = false;
+			// TODO: devBusy = false;
 		}
 		break;
 
@@ -754,5 +754,51 @@ byte MB89352::peekRegister(byte reg) const
 	}
 }
 
+
+// TODO duplicated in WD33C93.cc
+static enum_string<SCSI::Phase> phaseInfo[] = {
+	{ "UNDEFINED",   SCSI::UNDEFINED   },
+	{ "BUS_FREE",    SCSI::BUS_FREE    },
+	{ "ARBITRATION", SCSI::ARBITRATION },
+	{ "SELECTION",   SCSI::SELECTION   },
+	{ "RESELECTION", SCSI::RESELECTION },
+	{ "COMMAND",     SCSI::COMMAND     },
+	{ "EXECUTE",     SCSI::EXECUTE     },
+	{ "DATA_IN",     SCSI::DATA_IN     },
+	{ "DATA_OUT",    SCSI::DATA_OUT    },
+	{ "STATUS",      SCSI::STATUS      },
+	{ "MSG_OUT",     SCSI::MSG_OUT     },
+	{ "MSG_IN",      SCSI::MSG_IN      }
+};
+SERIALIZE_ENUM(SCSI::Phase, phaseInfo);
+
+template<typename Archive>
+void MB89352::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.serialize_blob("buffer", &buffer[0], buffer.size());
+	for (unsigned i = 0; i < MAX_DEV; ++i) {
+		std::string tag = std::string("device") + char('0' + i);
+		ar.serializePolymorphic(tag.c_str(), *dev[i]);
+	}
+	ar.serialize("bufIdx", bufIdx);
+	ar.serialize("msgin", msgin);
+	ar.serialize("counter", counter);
+	ar.serialize("blockCounter", blockCounter);
+	ar.serialize("tc", tc);
+	ar.serialize("phase", phase);
+	ar.serialize("nextPhase", nextPhase);
+	ar.serialize("myId", myId);
+	ar.serialize("targetId", targetId);
+	ar.serialize_blob("registers", regs, sizeof(regs));
+	ar.serialize("rst", rst);
+	ar.serialize("atn", atn);
+	ar.serialize("isEnabled", isEnabled);
+	ar.serialize("isBusy", isBusy);
+	ar.serialize("isTransfer", isTransfer);
+	ar.serialize("cdbIdx", cdbIdx);
+	ar.serialize_blob("cdb", cdb, sizeof(cdb));
+}
+INSTANTIATE_SERIALIZE_METHODS(MB89352);
+
 } // namespace openmsx
 
