@@ -1,4 +1,4 @@
-// $Id: SCSILS120.cc 6807 2007-08-06 17:08:53Z m9710797 $
+// $Id: SCSILS120.cc 8352 2008-11-12 18:39:08Z m9710797 $
 /* Ported from:
 ** Source: /cvsroot/bluemsx/blueMSX/Src/IoDevice/ScsiDevice.c,v
 ** Revision: 1.10
@@ -24,15 +24,15 @@
 #include "File.hh"
 #include "FileOperations.hh"
 #include "FileException.hh"
-#include "LedEvent.hh"
+#include "LedStatus.hh"
 #include "MSXMotherBoard.hh"
-#include "EventDistributor.hh"
 #include "XMLElement.hh"
 #include "RecordedCommand.hh"
-#include "MSXCliComm.hh"
+#include "CliComm.hh"
 #include "TclObject.hh"
 #include "CommandException.hh"
 #include "FileContext.hh"
+#include "serialize.hh"
 #include <algorithm>
 #include <vector>
 #include <bitset>
@@ -42,9 +42,9 @@
 //#undef PRT_DEBUG
 /*
 #define  PRT_DEBUG(mes)                          \
-        do {                                    \
-                std::cout << mes << std::endl;  \
-        } while (0)
+	do {                                    \
+	        std::cout << mes << std::endl;  \
+	} while (0)
 */
 using std::string;
 using std::vector;
@@ -66,40 +66,40 @@ static const byte MT_DOOR_OPEN = 0x71;
 static const byte MT_FMT_ERROR = 0x72;
 
 static const byte inqdata[36] = {
-       0,   // bit5-0 device type code.
-       0,   // bit7 = 1 removable device
-       2,   // bit7,6 ISO version. bit5,4,3 ECMA version.
-            // bit2,1,0 ANSI Version (001=SCSI1, 010=SCSI2)
-       2,   // bit7 AENC. bit6 TrmIOP.
-            // bit3-0 Response Data Format. (0000=SCSI1, 0001=CCS, 0010=SCSI2)
-      51,   // addtional length
-       0, 0,// reserved
-       0,   // bit7 RelAdr, bit6 WBus32, bit5 Wbus16, bit4 Sync, bit3 Linked,
-            // bit2 reseved bit1 CmdQue, bit0 SftRe
-     'o', 'p', 'e', 'n', 'M', 'S', 'X', ' ',    // vendor ID (8bytes)
-     'S', 'C', 'S', 'I', '2', ' ', 'L', 'S',    // product ID (16bytes)
-     '-', '1', '2', '0', 'd', 'i', 's', 'k',
-     '0', '1', '0', 'a'                         // product version (ASCII 4bytes)
+	  0,   // bit5-0 device type code.
+	  0,   // bit7 = 1 removable device
+	  2,   // bit7,6 ISO version. bit5,4,3 ECMA version.
+	       // bit2,1,0 ANSI Version (001=SCSI1, 010=SCSI2)
+	  2,   // bit7 AENC. bit6 TrmIOP.
+	       // bit3-0 Response Data Format. (0000=SCSI1, 0001=CCS, 0010=SCSI2)
+	 51,   // addtional length
+	  0, 0,// reserved
+	  0,   // bit7 RelAdr, bit6 WBus32, bit5 Wbus16, bit4 Sync, bit3 Linked,
+	       // bit2 reseved bit1 CmdQue, bit0 SftRe
+	'o', 'p', 'e', 'n', 'M', 'S', 'X', ' ',    // vendor ID (8bytes)
+	'S', 'C', 'S', 'I', '2', ' ', 'L', 'S',    // product ID (16bytes)
+	'-', '1', '2', '0', 'd', 'i', 's', 'k',
+	'0', '1', '0', 'a'                         // product version (ASCII 4bytes)
 };
 
 // for FDSFORM.COM
 static const char fds120[28 + 1]  = "IODATA  LS-120 COSM     0001";
 
-static const unsigned SECTOR_SIZE = 512; // same as harddisks
-static const unsigned BUFFER_BLOCK_SIZE = SCSIDevice::BUFFER_SIZE / SECTOR_SIZE;
+static const unsigned BUFFER_BLOCK_SIZE = SCSIDevice::BUFFER_SIZE /
+                                          SectorAccessibleDisk::SECTOR_SIZE;
 
 class LSXCommand : public RecordedCommand
 {
 public:
-        LSXCommand(MSXCommandController& msxCommandController,
-                   MSXEventDistributor& msxEventDistributor,
-                   Scheduler& scheduler, SCSILS120& ls);
-        virtual void execute(const std::vector<TclObject*>& tokens,
-                TclObject& result, const EmuTime& time);
-        virtual string help(const vector<string>& tokens) const;
-        virtual void tabCompletion(vector<string>& tokens) const;
+	LSXCommand(CommandController& commandController,
+	           MSXEventDistributor& msxEventDistributor,
+	           Scheduler& scheduler, SCSILS120& ls);
+	virtual void execute(const std::vector<TclObject*>& tokens,
+	                     TclObject& result, EmuTime::param time);
+	virtual string help(const vector<string>& tokens) const;
+	virtual void tabCompletion(vector<string>& tokens) const;
 private:
-        SCSILS120& ls;
+	SCSILS120& ls;
 };
 
 static const unsigned MAX_LS = 26;
@@ -113,26 +113,26 @@ SCSILS120::SCSILS120(MSXMotherBoard& motherBoard_, const XMLElement& targetconfi
 	, scsiId(targetconfig.getAttributeAsInt("id"))
 {
 	MSXMotherBoard::SharedStuff& info =
-                motherBoard.getSharedStuff("lsInUse");
-        if (info.counter == 0) {
-                assert(info.stuff == NULL);
-                info.stuff = new LSInUse();
-        }
-        ++info.counter;
-        LSInUse& lsInUse = *reinterpret_cast<LSInUse*>(info.stuff);
+		motherBoard.getSharedStuff("lsInUse");
+	if (info.counter == 0) {
+		assert(info.stuff == NULL);
+		info.stuff = new LSInUse();
+	}
+	++info.counter;
+	LSInUse& lsInUse = *reinterpret_cast<LSInUse*>(info.stuff);
 
-        unsigned id = 0;
-        while (lsInUse[id]) {
-                ++id;
-                if (id == MAX_LS) {
-                        throw MSXException("Too many LSs");
-                }
-        }
-        name = string("ls") + char('a' + id);
-        lsInUse[id] = true;
-        lsxCommand.reset(new LSXCommand(motherBoard.getMSXCommandController(),
-                                        motherBoard.getMSXEventDistributor(),
-                                        motherBoard.getScheduler(), *this));
+	unsigned id = 0;
+	while (lsInUse[id]) {
+		++id;
+		if (id == MAX_LS) {
+			throw MSXException("Too many LSs");
+		}
+	}
+	name = string("ls") + char('a' + id);
+	lsInUse[id] = true;
+	lsxCommand.reset(new LSXCommand(motherBoard.getCommandController(),
+	                                motherBoard.getMSXEventDistributor(),
+	                                motherBoard.getScheduler(), *this));
 
 	reset();
 }
@@ -141,22 +141,22 @@ SCSILS120::~SCSILS120()
 {
 	PRT_DEBUG("ls120 close for ls120 " << int(scsiId));
 	MSXMotherBoard::SharedStuff& info =
-                motherBoard.getSharedStuff("lsInUse");
-        assert(info.counter);
-        assert(info.stuff);
-        LSInUse& lsInUse = *reinterpret_cast<LSInUse*>(info.stuff);
+	        motherBoard.getSharedStuff("lsInUse");
+	assert(info.counter);
+	assert(info.stuff);
+	LSInUse& lsInUse = *reinterpret_cast<LSInUse*>(info.stuff);
 
-        motherBoard.getMSXCliComm().update(CliComm::HARDWARE, name, "remove");
-        unsigned id = name[2] - 'a';
-        assert(lsInUse[id]);
-        lsInUse[id] = false;
+	motherBoard.getMSXCliComm().update(CliComm::HARDWARE, name, "remove");
+	unsigned id = name[2] - 'a';
+	assert(lsInUse[id]);
+	lsInUse[id] = false;
 
-        --info.counter;
-        if (info.counter == 0) {
-                assert(lsInUse.none());
-                delete &lsInUse;
-                info.stuff = NULL;
-        }
+	--info.counter;
+	if (info.counter == 0) {
+		assert(lsInUse.none());
+		delete &lsInUse;
+		info.stuff = NULL;
+	}
 }
 
 void SCSILS120::reset()
@@ -176,8 +176,7 @@ void SCSILS120::busReset()
 
 void SCSILS120::disconnect()
 {
-	motherBoard.getEventDistributor().distributeEvent(
-			new LedEvent(LedEvent::FDD, false, motherBoard));
+	motherBoard.getLedStatus().setLed(LedStatus::FDD, false);
 }
 
 // Check the initiator in the call origin.
@@ -215,12 +214,12 @@ void SCSILS120::startStopUnit()
 		PRT_DEBUG("eject disk " << int(scsiId));
 		break;
 	case 3: // Insert  TODO: how can this happen?
-	//      if (!diskPresent(diskId)) {
-	//      	*disk = disk;
-	//      	updateExtendedDiskName(diskId, disk->fileName, disk->fileNameInZip);
-	//      	boardChangeDiskette(diskId, disk->fileName, disk->fileNameInZip);
+		//if (!diskPresent(diskId)) {
+		//	*disk = disk;
+		//	updateExtendedDiskName(diskId, disk->fileName, disk->fileNameInZip);
+		//	boardChangeDiskette(diskId, disk->fileName, disk->fileNameInZip);
 			PRT_DEBUG("insert ls120 " << int(scsiId));
-	//      }
+		//}
 		break;
 	}
 }
@@ -422,15 +421,14 @@ bool SCSILS120::checkAddress()
 // Execute scsiDeviceCheckAddress previously.
 unsigned SCSILS120::readSector(unsigned& blocks)
 {
-	motherBoard.getEventDistributor().distributeEvent(
-		new LedEvent(LedEvent::FDD, true, motherBoard));
+	motherBoard.getLedStatus().setLed(LedStatus::FDD, true);
 
 	unsigned numSectors = std::min(currentLength, BUFFER_BLOCK_SIZE);
 	unsigned counter = currentLength * SECTOR_SIZE;
 
 	PRT_DEBUG("ls120#" << int(scsiId) << " read sector: " << currentSector << " " << numSectors);
 	try {
-		//TODO: somehow map this to SectorAccessibleDisk::readLogicalSector?
+		// TODO: somehow map this to SectorAccessibleDisk::readSector?
 		file->seek(SECTOR_SIZE * currentSector);
 		file->read(buffer, SECTOR_SIZE * numSectors);
 		currentSector += numSectors;
@@ -460,13 +458,12 @@ unsigned SCSILS120::dataIn(unsigned& blocks)
 // Execute scsiDeviceCheckAddress and scsiDeviceCheckReadOnly previously.
 unsigned SCSILS120::writeSector(unsigned& blocks)
 {
-	motherBoard.getEventDistributor().distributeEvent(
-		new LedEvent(LedEvent::FDD, true, motherBoard));
+	motherBoard.getLedStatus().setLed(LedStatus::FDD, true);
 
 	unsigned numSectors = std::min(currentLength, BUFFER_BLOCK_SIZE);
 
 	PRT_DEBUG("ls120#" << int(scsiId) << " write sector: " << currentSector << " " << numSectors);
-	//TODO: somehow map this to SectorAccessibleDisk::writeLogicalSector?
+	// TODO: somehow map this to SectorAccessibleDisk::writeSector?
 	try {
 		file->seek(SECTOR_SIZE * currentSector);
 		file->write(buffer, SECTOR_SIZE * numSectors);
@@ -519,23 +516,23 @@ byte SCSILS120::getStatusCode()
 
 void SCSILS120::eject()
 {
-        file.reset();
-        mediaChanged = true;
+	file.reset();
+	mediaChanged = true;
 	if (mode & MODE_UNITATTENTION) {
 		unitAttention = true;
 	}
-        motherBoard.getMSXCliComm().update(CliComm::MEDIA, name, "");
+	motherBoard.getMSXCliComm().update(CliComm::MEDIA, name, "");
 }
 
 void SCSILS120::insert(const string& filename)
 {
-        std::auto_ptr<File> newFile(new File(filename));
-        file = newFile;
-        mediaChanged = true;
+	std::auto_ptr<File> newFile(new File(filename));
+	file = newFile;
+	mediaChanged = true;
 	if (mode & MODE_UNITATTENTION) {
 		unitAttention = true;
 	}
-        motherBoard.getMSXCliComm().update(CliComm::MEDIA, name, filename);
+	motherBoard.getMSXCliComm().update(CliComm::MEDIA, name, filename);
 }
 
 unsigned SCSILS120::executeCmd(const byte* cdb_, SCSI::Phase& phase, unsigned& blocks)
@@ -617,9 +614,7 @@ unsigned SCSILS120::executeCmd(const byte* cdb_, SCSI::Phase& phase, unsigned& b
 				currentLength = SECTOR_SIZE >> 1;
 			}
 			if (checkAddress() && !checkReadOnly()) {
-				motherBoard.getEventDistributor().distributeEvent(
-					new LedEvent(LedEvent::FDD, true,
-						motherBoard));
+				motherBoard.getLedStatus().setLed(LedStatus::FDD, true);
 				unsigned tmp = std::min(currentLength, BUFFER_BLOCK_SIZE);
 				blocks = currentLength - tmp;
 				unsigned counter = tmp * SECTOR_SIZE;
@@ -631,8 +626,7 @@ unsigned SCSILS120::executeCmd(const byte* cdb_, SCSI::Phase& phase, unsigned& b
 
 		case SCSI::OP_SEEK6:
 			PRT_DEBUG("Seek6: " << currentSector);
-			motherBoard.getEventDistributor().distributeEvent(
-				new LedEvent(LedEvent::FDD, true, motherBoard));
+			motherBoard.getLedStatus().setLed(LedStatus::FDD, true);
 			currentLength = 1;
 			checkAddress();
 			return 0;
@@ -703,8 +697,7 @@ unsigned SCSILS120::executeCmd(const byte* cdb_, SCSI::Phase& phase, unsigned& b
 		}
 		case SCSI::OP_SEEK10:
 			PRT_DEBUG("Seek10: " << currentSector);
-			motherBoard.getEventDistributor().distributeEvent(
-				new LedEvent(LedEvent::FDD, true, motherBoard));
+			motherBoard.getLedStatus().setLed(LedStatus::FDD, true);
 			currentLength = 1;
 			checkAddress();
 			return 0;
@@ -735,10 +728,10 @@ byte SCSILS120::msgIn()
 scsiDeviceMsgOut()
 Notes:
     [out]
-          -1: Busfree demand. (Please process it in the call origin.)
-        bit2: Status phase demand. Error happend.
-        bit1: Make it to a busfree if ATN has not been released.
-        bit0: There is a message(MsgIn).
+	  -1: Busfree demand. (Please process it in the call origin.)
+	bit2: Status phase demand. Error happend.
+	bit1: Make it to a busfree if ATN has not been released.
+	bit0: There is a message(MsgIn).
 */
 int SCSILS120::msgOut(byte value)
 {
@@ -768,7 +761,7 @@ int SCSILS120::msgOut(byte value)
 	return ((value >= 0x04) && (value <= 0x11)) ? 3 : 1;
 }
 
-unsigned SCSILS120::getNbSectors() const
+unsigned SCSILS120::getNbSectorsImpl() const
 {
 	if (file.get()) {
 		return file->getSize() / SECTOR_SIZE;
@@ -777,15 +770,18 @@ unsigned SCSILS120::getNbSectors() const
 	}
 }
 
-// NOTE: UNUSED FOR NOW!
-void SCSILS120::readLogicalSector(unsigned sector, byte* buf)
+bool SCSILS120::isWriteProtectedImpl() const
+{
+	return false;
+}
+
+void SCSILS120::readSectorImpl(unsigned sector, byte* buf)
 {
 	file->seek(512 * sector);
 	file->read(buf, 512);
 }
 
-// NOTE: UNUSED FOR NOW!
-void SCSILS120::writeLogicalSector(unsigned sector, const byte* buf)
+void SCSILS120::writeSectorImpl(unsigned sector, const byte* buf)
 {
 	file->seek(512 * sector);
 	file->write(buf, 512);
@@ -803,17 +799,17 @@ const std::string& SCSILS120::getContainerName() const
 
 // class LSXCommand
 
-LSXCommand::LSXCommand(MSXCommandController& msxCommandController,
+LSXCommand::LSXCommand(CommandController& commandController,
                        MSXEventDistributor& msxEventDistributor,
                        Scheduler& scheduler, SCSILS120& ls_)
-	: RecordedCommand(msxCommandController, msxEventDistributor,
+	: RecordedCommand(commandController, msxEventDistributor,
 	                  scheduler, ls_.name)
 	, ls(ls_)
 {
 }
 
 void LSXCommand::execute(const std::vector<TclObject*>& tokens, TclObject& result,
-				const EmuTime& /*time*/)
+				EmuTime::param /*time*/)
 {
 	if (tokens.size() == 1) {
 		File* file = ls.file.get();
@@ -838,8 +834,9 @@ void LSXCommand::execute(const std::vector<TclObject*>& tokens, TclObject& resul
 			}
 		}
 		try {
-			UserFileContext context(getCommandController());
-			string filename = context.resolve(tokens[fileToken]->getString());
+			UserFileContext context;
+			string filename = context.resolve(
+				getCommandController(), tokens[fileToken]->getString());
 			ls.insert(filename);
 			// return filename; // Note: the diskX command doesn't do this either, so this has not been converted to TclObject style here
 		} catch (FileException& e) {
@@ -864,8 +861,35 @@ void LSXCommand::tabCompletion(vector<string>& tokens) const
 	set<string> extra;
 	extra.insert("eject");
 	extra.insert("insert");
-	UserFileContext context(getCommandController());
-	completeFileName(tokens, context, extra);
+	UserFileContext context;
+	completeFileName(getCommandController(), tokens, context, extra);
 }
 
+
+template<typename Archive>
+void SCSILS120::serialize(Archive& ar, unsigned /*version*/)
+{
+	string filename = file.get() ? file->getURL() : "";
+	ar.serialize("filename", filename);
+	if (ar.isLoader()) {
+		// re-insert disk before restoring 'mediaChanged'
+		if (filename.empty()) {
+			eject();
+		} else {
+			insert(filename);
+		}
+	}
+
+	ar.serialize("keycode", keycode);
+	ar.serialize("currentSector", currentSector);
+	ar.serialize("currentLength", currentLength);
+	ar.serialize("unitAttention", unitAttention);
+	ar.serialize("mediaChanged", mediaChanged);
+	ar.serialize("message", message);
+	ar.serialize("lun", lun);
+	ar.serialize_blob("cdb", cdb, sizeof(cdb));
+}
+INSTANTIATE_SERIALIZE_METHODS(SCSILS120);
+REGISTER_POLYMORPHIC_INITIALIZER(SCSIDevice, SCSILS120, "SCSILS120");
+
 } // namespace openmsx
