@@ -1,4 +1,4 @@
-// $Id: WD33C93.cc 6805 2007-08-06 12:01:49Z m9710797 $
+// $Id: WD33C93.cc 8206 2008-09-08 20:19:45Z m9710797 $
 /* Ported from:
 ** Source: /cvsroot/bluemsx/blueMSX/Src/IoDevice/wd33c93.c,v
 ** Revision: 1.12
@@ -21,18 +21,10 @@
 #include "XMLElement.hh"
 #include "MSXException.hh"
 #include "StringOp.hh"
+#include "serialize.hh"
 #include <cassert>
 #include <cstring>
 
-//#undef PRT_DEBUG
-/*
-#include <iostream>
-#define  PRT_DEBUG(mes)                          \
-        do {                                    \
-                std::cout << mes << std::endl;  \
-        } while (0)
-*/
-
 namespace openmsx {
 
 static const unsigned MAX_DEV = 8;
@@ -156,7 +148,6 @@ WD33C93::WD33C93(MSXMotherBoard& motherBoard, const XMLElement& config)
 
 WD33C93::~WD33C93()
 {
-	PRT_DEBUG("WD33C93 destroy");
 }
 
 void WD33C93::disconnect()
@@ -471,38 +462,76 @@ void WD33C93::reset(bool scsireset)
 	}
 }
 
+
+static enum_string<SCSI::Phase> phaseInfo[] = {
+	{ "UNDEFINED",   SCSI::UNDEFINED   },
+	{ "BUS_FREE",    SCSI::BUS_FREE    },
+	{ "ARBITRATION", SCSI::ARBITRATION },
+	{ "SELECTION",   SCSI::SELECTION   },
+	{ "RESELECTION", SCSI::RESELECTION },
+	{ "COMMAND",     SCSI::COMMAND     },
+	{ "EXECUTE",     SCSI::EXECUTE     },
+	{ "DATA_IN",     SCSI::DATA_IN     },
+	{ "DATA_OUT",    SCSI::DATA_OUT    },
+	{ "STATUS",      SCSI::STATUS      },
+	{ "MSG_OUT",     SCSI::MSG_OUT     },
+	{ "MSG_IN",      SCSI::MSG_IN      }
+};
+SERIALIZE_ENUM(SCSI::Phase, phaseInfo);
+
+template<typename Archive>
+void WD33C93::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.serialize_blob("buffer", &buffer[0], buffer.size());
+	for (unsigned i = 0; i < MAX_DEV; ++i) {
+		std::string tag = std::string("device") + char('0' + i);
+		ar.serializePolymorphic(tag.c_str(), *dev[i]);
+	}
+	ar.serialize("bufIdx", bufIdx);
+	ar.serialize("counter", counter);
+	ar.serialize("blockCounter", blockCounter);
+	ar.serialize("tc", tc);
+	ar.serialize("phase", phase);
+	ar.serialize("myId", myId);
+	ar.serialize("targetId", targetId);
+	ar.serialize_blob("registers", regs, sizeof(regs));
+	ar.serialize("latch", latch);
+	ar.serialize("devBusy", devBusy);
+}
+INSTANTIATE_SERIALIZE_METHODS(WD33C93);
+
 /* Here is some info on the parameters for SCSI devices:
 static SCSIDevice* wd33c93ScsiDevCreate(WD33C93* wd33c93, int id)
 {
 #if 1
-    // CD_UPDATE: Use dynamic parameters instead of hard coded ones
-    int diskId, mode, type;
+	// CD_UPDATE: Use dynamic parameters instead of hard coded ones
+	int diskId, mode, type;
 
-    diskId = diskGetHdDriveId(hdId, id);
-    if (diskIsCdrom(diskId)) {
-        mode = MODE_SCSI1 | MODE_UNITATTENTION | MODE_REMOVABLE | MODE_NOVAXIS;
-        type = SDT_CDROM;
-    } else {
-        mode = MODE_SCSI1 | MODE_UNITATTENTION | MODE_FDS120 | MODE_REMOVABLE | MODE_NOVAXIS;
-        type = SDT_DirectAccess;
-    }
-    return scsiDeviceCreate(id, diskId, buffer, NULL, type, mode,
-                           (CdromXferCompCb)wd33c93XferCb, wd33c93);
+	diskId = diskGetHdDriveId(hdId, id);
+	if (diskIsCdrom(diskId)) {
+		mode = MODE_SCSI1 | MODE_UNITATTENTION | MODE_REMOVABLE | MODE_NOVAXIS;
+		type = SDT_CDROM;
+	} else {
+		mode = MODE_SCSI1 | MODE_UNITATTENTION | MODE_FDS120 | MODE_REMOVABLE | MODE_NOVAXIS;
+		type = SDT_DirectAccess;
+	}
+	return scsiDeviceCreate(id, diskId, buffer, NULL, type, mode,
+	                       (CdromXferCompCb)wd33c93XferCb, wd33c93);
 #else
-    SCSIDEVICE* dev;
-    int mode;
-    int type;
+	SCSIDEVICE* dev;
+	int mode;
+	int type;
 
-    if (id != 2) {
-        mode = MODE_SCSI1 | MODE_UNITATTENTION | MODE_FDS120 | MODE_REMOVABLE | MODE_NOVAXIS;
-        type = SDT_DirectAccess;
-    } else {
-        mode = MODE_SCSI1 | MODE_UNITATTENTION | MODE_REMOVABLE | MODE_NOVAXIS;
-        type = SDT_CDROM;
-    }
-    dev = scsiDeviceCreate(id, diskGetHdDriveId(hdId, id),
-            buffer, NULL, type, mode, (CdromXferCompCb)wd33c93XferCb, wd33c93);
-    return dev;
+	if (id != 2) {
+		mode = MODE_SCSI1 | MODE_UNITATTENTION | MODE_FDS120 | MODE_REMOVABLE | MODE_NOVAXIS;
+		type = SDT_DirectAccess;
+	} else {
+		mode = MODE_SCSI1 | MODE_UNITATTENTION | MODE_REMOVABLE | MODE_NOVAXIS;
+		type = SDT_CDROM;
+	}
+	dev = scsiDeviceCreate(id, diskGetHdDriveId(hdId, id),
+	        buffer, NULL, type, mode, (CdromXferCompCb)wd33c93XferCb, wd33c93);
+	return dev;
 #endif
 }
 */
