@@ -1,11 +1,10 @@
-// $Id: EventDelay.cc 5779 2006-10-14 16:17:32Z m9710797 $
+// $Id: EventDelay.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "EventDelay.hh"
 #include "MSXEventDistributor.hh"
 #include "FloatSetting.hh"
-#include "MSXEventDistributor.hh"
 #include "Timer.hh"
-#include <algorithm>
+#include "MSXException.hh"
 #include <cassert>
 
 namespace openmsx {
@@ -32,7 +31,7 @@ void EventDelay::queueEvent(EventPtr event)
 	toBeScheduledEvents.push_back(EventTime(event, Timer::getTime()));
 }
 
-void EventDelay::sync(const EmuTime& emuTime)
+void EventDelay::sync(EmuTime::param emuTime)
 {
 	unsigned long long curRealTime = Timer::getTime();
 	unsigned long long realDuration = curRealTime - prevReal;
@@ -62,7 +61,7 @@ void EventDelay::sync(const EmuTime& emuTime)
 	prevEmu = emuTime;
 }
 
-void EventDelay::executeUntil(const EmuTime& time, int /*userData*/)
+void EventDelay::executeUntil(EmuTime::param time, int /*userData*/)
 {
 	try {
 		eventDistributor.distributeEvent(scheduledEvents.front(), time);
