@@ -1,16 +1,20 @@
-// $Id: Joystick.cc 6807 2007-08-06 17:08:53Z m9710797 $
+// $Id: Joystick.cc 8353 2008-11-13 18:38:44Z m9710797 $
 
 #include "Joystick.hh"
 #include "PluggingController.hh"
+#include "PlugException.hh"
 #include "MSXEventDistributor.hh"
 #include "InputEvents.hh"
 #include "checked_cast.hh"
+#include "serialize.hh"
 #include <cassert>
 
 using std::string;
 
 namespace openmsx {
 
+static const int THRESHOLD = 32768 / 10;
+
 void Joystick::registerAll(MSXEventDistributor& eventDistributor,
                            PluggingController& controller)
 {
@@ -25,6 +29,10 @@ void Joystick::registerAll(MSXEventDistributor& eventDistributor,
 	}
 }
 
+// Note: It's OK to open/close the same SDL_Joystick multiple times (we open it
+// once per MSX machine). The SDL documentation doesn't state this, but I
+// checked the implementation and a SDL_Joystick uses a 'reference count' on
+// the open/close calls.
 Joystick::Joystick(MSXEventDistributor& eventDistributor_, unsigned joyNum_)
 	: eventDistributor(eventDistributor_)
 	, name(string("joystick") + char('1' + joyNum_))
@@ -45,7 +53,7 @@ Joystick::~Joystick()
 	}
 }
 
-//Pluggable
+// Pluggable
 const string& Joystick::getName() const
 {
 	return name;
@@ -56,7 +64,7 @@ const string& Joystick::getDescription() const
 	return desc;
 }
 
-void Joystick::plugHelper(Connector& /*connector*/, const EmuTime& /*time*/)
+void Joystick::plugHelper(Connector& /*connector*/, EmuTime::param /*time*/)
 {
 	if (!joystick) {
 		throw PlugException("Failed to open joystick device");
@@ -64,29 +72,61 @@ void Joystick::plugHelper(Connector& /*connector*/, const EmuTime& /*time*/)
 
 	eventDistributor.registerEventListener(*this);
 
-	status = JOY_UP | JOY_DOWN | JOY_LEFT | JOY_RIGHT |
-	         JOY_BUTTONA | JOY_BUTTONB;
+	calcInitialState();
 }
 
-void Joystick::unplugHelper(const EmuTime& /*time*/)
+void Joystick::unplugHelper(EmuTime::param /*time*/)
 {
 	eventDistributor.unregisterEventListener(*this);
 }
 
 
-//JoystickDevice
-byte Joystick::read(const EmuTime& /*time*/)
+// JoystickDevice
+byte Joystick::read(EmuTime::param /*time*/)
 {
 	return status;
 }
 
-void Joystick::write(byte /*value*/, const EmuTime& /*time*/)
+void Joystick::write(byte /*value*/, EmuTime::param /*time*/)
 {
-	//do nothing
+	// nothing
 }
 
-//EventListener
-void Joystick::signalEvent(shared_ptr<const Event> event, const EmuTime& /*time*/)
+void Joystick::calcInitialState()
+{
+	status = JOY_UP | JOY_DOWN | JOY_LEFT | JOY_RIGHT |
+	         JOY_BUTTONA | JOY_BUTTONB;
+
+	if (joystick) {
+		int xAxis = SDL_JoystickGetAxis(joystick, 0);
+		if (xAxis < -THRESHOLD) {
+			status &= ~JOY_LEFT;
+		} else if (xAxis > THRESHOLD) {
+			status &= ~JOY_RIGHT;
+		}
+
+		int yAxis = SDL_JoystickGetAxis(joystick, 1);
+		if (yAxis < -THRESHOLD) {
+			status &= ~JOY_UP;
+		} else if (yAxis > THRESHOLD) {
+			status &= ~JOY_DOWN;
+		}
+
+		int numButtons = SDL_JoystickNumButtons(joystick);
+		for (int button = 0; button < numButtons; ++button) {
+			if (SDL_JoystickGetButton(joystick, button)) {
+				if (button & 1) {
+					status &= ~JOY_BUTTONB;
+				} else {
+					status &= ~JOY_BUTTONA;
+				}
+			}
+		}
+	}
+}
+
+// EventListener
+void Joystick::signalEvent(shared_ptr<const Event> event, EmuTime::param /*time*/)
 {
 	const JoystickEvent* joyEvent =
 		dynamic_cast<const JoystickEvent*>(event.get());
@@ -139,32 +179,20 @@ void Joystick::signalEvent(shared_ptr<const Event> event, const EmuTime& /*time*
 	case OPENMSX_JOY_BUTTON_DOWN_EVENT: {
 		const JoystickButtonEvent& buttonEvent =
 			checked_cast<const JoystickButtonEvent&>(*event);
-		switch (buttonEvent.getButton() % 2) {
-		case 0:
-			status &= ~JOY_BUTTONA;
-			break;
-		case 1:
+		if (buttonEvent.getButton() & 1) {
 			status &= ~JOY_BUTTONB;
-			break;
-		default:
-			// ignore other buttons
-			break;
+		} else {
+			status &= ~JOY_BUTTONA;
 		}
 		break;
 	}
 	case OPENMSX_JOY_BUTTON_UP_EVENT: {
 		const JoystickButtonEvent& buttonEvent =
 			checked_cast<const JoystickButtonEvent&>(*event);
-		switch (buttonEvent.getButton() % 2) {
-		case 0:
-			status |= JOY_BUTTONA;
-			break;
-		case 1:
+		if (buttonEvent.getButton() & 1) {
 			status |= JOY_BUTTONB;
-			break;
-		default:
-			// ignore other buttons
-			break;
+		} else {
+			status |= JOY_BUTTONA;
 		}
 		break;
 	}
@@ -173,4 +201,19 @@ void Joystick::signalEvent(shared_ptr<const Event> event, const EmuTime& /*time*
 	}
 }
 
+template<typename Archive>
+void Joystick::serialize(Archive& ar, unsigned /*version*/)
+{
+	// don't serialize 'status':
+	// it should be based on the 'current' joystick state
+	if (ar.isLoader()) {
+		Connector* conn = getConnector();
+		if (joystick && conn) {
+			plugHelper(*conn, EmuTime::dummy());
+		}
+	}
+}
+INSTANTIATE_SERIALIZE_METHODS(Joystick);
+REGISTER_POLYMORPHIC_INITIALIZER(Pluggable, Joystick, "Joystick");
+
 } // namespace openmsx
