@@ -1,10 +1,11 @@
-// $Id: KeyJoystick.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: KeyJoystick.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "KeyJoystick.hh"
 #include "MSXEventDistributor.hh"
 #include "KeyCodeSetting.hh"
 #include "InputEvents.hh"
 #include "checked_cast.hh"
+#include "serialize.hh"
 #include <cassert>
 
 using std::string;
@@ -16,24 +17,23 @@ KeyJoystick::KeyJoystick(CommandController& commandController,
                          const string& name_)
 	: eventDistributor(eventDistributor_)
 	, name(name_)
+	, up   (new KeyCodeSetting(commandController, name + ".up",
+		"key for direction up",    Keys::K_UP))
+	, down (new KeyCodeSetting(commandController, name + ".down",
+		"key for direction down",  Keys::K_DOWN))
+	, left (new KeyCodeSetting(commandController, name + ".left",
+		"key for direction left",  Keys::K_LEFT))
+	, right(new KeyCodeSetting(commandController, name + ".right",
+		"key for direction right", Keys::K_RIGHT))
+	, trigA(new KeyCodeSetting(commandController, name + ".triga",
+		"key for trigger A",       Keys::K_SPACE))
+	, trigB(new KeyCodeSetting(commandController, name + ".trigb",
+		"key for trigger B",       Keys::K_M))
 {
 	eventDistributor.registerEventListener(*this);
 
 	status = JOY_UP | JOY_DOWN | JOY_LEFT | JOY_RIGHT |
 	         JOY_BUTTONA | JOY_BUTTONB;
-
-	up.reset   (new KeyCodeSetting(commandController, name + ".up",
-		"key for direction up",    Keys::K_UP));
-	down.reset (new KeyCodeSetting(commandController, name + ".down",
-		"key for direction down",  Keys::K_DOWN));
-	left.reset (new KeyCodeSetting(commandController, name + ".left",
-		"key for direction left",  Keys::K_LEFT));
-	right.reset(new KeyCodeSetting(commandController, name + ".right",
-		"key for direction right", Keys::K_RIGHT));
-	trigA.reset(new KeyCodeSetting(commandController, name + ".triga",
-		"key for trigger A",       Keys::K_SPACE));
-	trigB.reset(new KeyCodeSetting(commandController, name + ".trigb",
-		"key for trigger B",       Keys::K_M));
 }
 
 KeyJoystick::~KeyJoystick()
@@ -62,30 +62,30 @@ void KeyJoystick::allUp()
 	         JOY_BUTTONA | JOY_BUTTONB;
 }
 
-void KeyJoystick::plugHelper(Connector& /*connector*/, const EmuTime& /*time*/)
+void KeyJoystick::plugHelper(Connector& /*connector*/, EmuTime::param /*time*/)
 {
 }
 
-void KeyJoystick::unplugHelper(const EmuTime& /*time*/)
+void KeyJoystick::unplugHelper(EmuTime::param /*time*/)
 {
 }
 
 
 // KeyJoystickDevice
-byte KeyJoystick::read(const EmuTime& /*time*/)
+byte KeyJoystick::read(EmuTime::param /*time*/)
 {
 	return status;
 }
 
-void KeyJoystick::write(byte /*value*/, const EmuTime& /*time*/)
+void KeyJoystick::write(byte /*value*/, EmuTime::param /*time*/)
 {
-	//do nothing
+	// nothing
 }
 
 
 // EventListener
 void KeyJoystick::signalEvent(shared_ptr<const Event> event,
-                              const EmuTime& /*time*/)
+                              EmuTime::param /*time*/)
 {
 	switch (event->getType()) {
 	case OPENMSX_CONSOLE_ON_EVENT:
@@ -119,4 +119,12 @@ void KeyJoystick::signalEvent(shared_ptr<const Event> event,
 	}
 }
 
+template<typename Archive>
+void KeyJoystick::serialize(Archive& /*ar*/, unsigned /*version*/)
+{
+	// don't serialize 'status', is controlled by key events
+}
+INSTANTIATE_SERIALIZE_METHODS(KeyJoystick);
+REGISTER_POLYMORPHIC_INITIALIZER(Pluggable, KeyJoystick, "KeyJoystick");
+
 } // namespace openmsx
