@@ -1,4 +1,4 @@
-// $Id: MSXEventDistributor.cc 5605 2006-09-06 20:09:12Z m9710797 $
+// $Id: MSXEventDistributor.cc 8441 2009-01-06 09:54:24Z m9710797 $
 
 #include "MSXEventDistributor.hh"
 #include "MSXEventListener.hh"
@@ -30,10 +30,16 @@ void MSXEventDistributor::unregisterEventListener(MSXEventListener& listener)
 	listeners.erase(find(listeners.begin(), listeners.end(), &listener));
 }
 
-void MSXEventDistributor::distributeEvent(EventPtr event, const EmuTime& time)
+void MSXEventDistributor::distributeEvent(EventPtr event, EmuTime::param time)
 {
-	for (Listeners::const_iterator it = listeners.begin();
-	     it != listeners.end(); ++it) {
+	// Iterate over a copy because signalEvent() may indirect call back into
+	// registerEventListener().
+	//   e.g. signalEvent() -> .. -> PlugCmd::execute() -> .. ->
+	//        Connector::plug() -> .. -> Joystick::plugHelper() ->
+	//        registerEventListener()
+	Listeners copy = listeners;
+	for (Listeners::const_iterator it = copy.begin();
+	     it != copy.end(); ++it) {
 		(*it)->signalEvent(event, time);
 	}
 }
