@@ -1,4 +1,4 @@
-// $Id: MSXEventRecorder.cc 5633 2006-09-16 13:13:10Z m9710797 $
+// $Id: MSXEventRecorder.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "MSXEventRecorder.hh"
 #include "MSXEventDistributor.hh"
@@ -27,7 +27,7 @@ MSXEventRecorder::~MSXEventRecorder()
 }
 
 void MSXEventRecorder::signalEvent(shared_ptr<const Event> event,
-                                   const EmuTime& time)
+                                   EmuTime::param time)
 {
 	logFileStream << time << ' ' << event->toString() << std::endl;
 }
