@@ -1,4 +1,4 @@
-// $Id: MSXEventReplayer.cc 6593 2007-06-03 23:12:19Z mthuurne $
+// $Id: MSXEventReplayer.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "MSXEventReplayer.hh"
 #include "InputEventFactory.hh"
@@ -26,8 +26,8 @@ MSXEventReplayer::MSXEventReplayer(Scheduler& scheduler,
 
 const std::string& MSXEventReplayer::schedName() const
 {
-        static const string schedName = "MSXEventReplayer";
-        return schedName;
+	static const string schedName = "MSXEventReplayer";
+	return schedName;
 }
 
 void MSXEventReplayer::processLogEntry()
@@ -38,7 +38,7 @@ void MSXEventReplayer::processLogEntry()
 	if (logFileStream.good()) {
 		string emutimeStr;
 		StringOp::splitOnFirst(temp, " ", emutimeStr, eventString);
-	 	EmuTime nextEventTime = EmuTime::makeEmuTime(
+		EmuTime nextEventTime = EmuTime::makeEmuTime(
 		              StringOp::stringToUint64(emutimeStr));
 		setSyncPoint(nextEventTime);
 	} else {
@@ -47,10 +47,10 @@ void MSXEventReplayer::processLogEntry()
 	}
 }
 
-void MSXEventReplayer::executeUntil(const EmuTime& time, int /*userData*/)
+void MSXEventReplayer::executeUntil(EmuTime::param time, int /*userData*/)
 {
 	try {
-	        InputEventFactory::EventPtr eventPtr =
+		InputEventFactory::EventPtr eventPtr =
 			InputEventFactory::createInputEvent(eventString);
 		try {
 			eventDistributor.distributeEvent(eventPtr, time);
