@@ -1,9 +1,10 @@
-// $Id: Mouse.cc 5605 2006-09-06 20:09:12Z m9710797 $
+// $Id: Mouse.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "Mouse.hh"
 #include "MSXEventDistributor.hh"
 #include "InputEvents.hh"
 #include "checked_cast.hh"
+#include "serialize.hh"
 
 using std::string;
 
@@ -35,7 +36,7 @@ Mouse::~Mouse()
 }
 
 
-//Pluggable
+// Pluggable
 const string& Mouse::getName() const
 {
 	static const string name("mouse");
@@ -48,7 +49,7 @@ const string& Mouse::getDescription() const
 	return desc;
 }
 
-void Mouse::plugHelper(Connector& /*connector*/, const EmuTime& time)
+void Mouse::plugHelper(Connector& /*connector*/, EmuTime::param time)
 {
 	if (status & JOY_BUTTONA) {
 		// not pressed, mouse mode
@@ -60,27 +61,27 @@ void Mouse::plugHelper(Connector& /*connector*/, const EmuTime& time)
 	}
 }
 
-void Mouse::unplugHelper(const EmuTime& /*time*/)
+void Mouse::unplugHelper(EmuTime::param /*time*/)
 {
 }
 
 
-//JoystickDevice
-byte Mouse::read(const EmuTime& /*time*/)
+// JoystickDevice
+byte Mouse::read(EmuTime::param /*time*/)
 {
 	if (mouseMode) {
 		switch (faze) {
-			case FAZE_XHIGH:
-				return (((xrel / SCALE) >> 4) & 0x0F) | status;
-			case FAZE_XLOW:
-				return  ((xrel / SCALE)       & 0x0F) | status;
-			case FAZE_YHIGH:
-				return (((yrel / SCALE) >> 4) & 0x0F) | status;
-			case FAZE_YLOW:
-				return  ((yrel / SCALE)       & 0x0F) | status;
-			default:
-				assert(false);
-				return status;	// avoid warning
+		case FAZE_XHIGH:
+			return (((xrel / SCALE) >> 4) & 0x0F) | status;
+		case FAZE_XLOW:
+			return  ((xrel / SCALE)       & 0x0F) | status;
+		case FAZE_YHIGH:
+			return (((yrel / SCALE) >> 4) & 0x0F) | status;
+		case FAZE_YLOW:
+			return  ((yrel / SCALE)       & 0x0F) | status;
+		default:
+			assert(false);
+			return status; // avoid warning
 		}
 	} else {
 		emulateJoystick();
@@ -137,13 +138,13 @@ void Mouse::emulateJoystick()
 	}
 }
 
-void Mouse::write(byte value, const EmuTime& time)
+void Mouse::write(byte value, EmuTime::param time)
 {
 	if (mouseMode) {
 		// TODO figure out the timeout mechanism
 		//      does it exist at all?
 
-		const int TIMEOUT = 1000;	// TODO find a good value
+		const int TIMEOUT = 1000; // TODO find a good value
 		int delta = lastTime.getTicksTill(time);
 		lastTime.advance(time);
 		if (delta >= TIMEOUT) {
@@ -151,22 +152,22 @@ void Mouse::write(byte value, const EmuTime& time)
 		}
 
 		switch (faze) {
-			case FAZE_XHIGH:
-				if ((value & STROBE) == 0) faze = FAZE_XLOW;
-				break;
-			case FAZE_XLOW:
-				if ((value & STROBE) != 0) faze = FAZE_YHIGH;
-				break;
-			case FAZE_YHIGH:
-				if ((value & STROBE) == 0) faze = FAZE_YLOW;
-				break;
-			case FAZE_YLOW:
-				if ((value & STROBE) != 0) {
-					faze = FAZE_XHIGH;
-					xrel = curxrel; yrel = curyrel;
-					curxrel = 0; curyrel = 0;
-				}
-				break;
+		case FAZE_XHIGH:
+			if ((value & STROBE) == 0) faze = FAZE_XLOW;
+			break;
+		case FAZE_XLOW:
+			if ((value & STROBE) != 0) faze = FAZE_YHIGH;
+			break;
+		case FAZE_YHIGH:
+			if ((value & STROBE) == 0) faze = FAZE_YLOW;
+			break;
+		case FAZE_YLOW:
+			if ((value & STROBE) != 0) {
+				faze = FAZE_XHIGH;
+				xrel = curxrel; yrel = curyrel;
+				curxrel = 0; curyrel = 0;
+			}
+			break;
 		}
 	} else {
 		// ignore
@@ -174,8 +175,8 @@ void Mouse::write(byte value, const EmuTime& time)
 }
 
 
-//EventListener
-void Mouse::signalEvent(shared_ptr<const Event> event, const EmuTime& /*time*/)
+// EventListener
+void Mouse::signalEvent(shared_ptr<const Event> event, EmuTime::param /*time*/)
 {
 	switch (event->getType()) {
 	case OPENMSX_MOUSE_MOTION_EVENT: {
@@ -193,15 +194,15 @@ void Mouse::signalEvent(shared_ptr<const Event> event, const EmuTime& /*time*/)
 		const MouseButtonEvent& buttonEvent =
 			checked_cast<const MouseButtonEvent&>(*event);
 		switch (buttonEvent.getButton()) {
-			case MouseButtonEvent::LEFT:
-				status &= ~JOY_BUTTONA;
-				break;
-			case MouseButtonEvent::RIGHT:
-				status &= ~JOY_BUTTONB;
-				break;
-			default:
-				// ignore other buttons
-				break;
+		case MouseButtonEvent::LEFT:
+			status &= ~JOY_BUTTONA;
+			break;
+		case MouseButtonEvent::RIGHT:
+			status &= ~JOY_BUTTONB;
+			break;
+		default:
+			// ignore other buttons
+			break;
 		}
 		break;
 	}
@@ -209,15 +210,15 @@ void Mouse::signalEvent(shared_ptr<const Event> event, const EmuTime& /*time*/)
 		const MouseButtonEvent& buttonEvent =
 			checked_cast<const MouseButtonEvent&>(*event);
 		switch (buttonEvent.getButton()) {
-			case MouseButtonEvent::LEFT:
-				status |= JOY_BUTTONA;
-				break;
-			case MouseButtonEvent::RIGHT:
-				status |= JOY_BUTTONB;
-				break;
-			default:
-				// ignore other buttons
-				break;
+		case MouseButtonEvent::LEFT:
+			status |= JOY_BUTTONA;
+			break;
+		case MouseButtonEvent::RIGHT:
+			status |= JOY_BUTTONB;
+			break;
+		default:
+			// ignore other buttons
+			break;
 		}
 		break;
 	}
@@ -227,4 +228,20 @@ void Mouse::signalEvent(shared_ptr<const Event> event, const EmuTime& /*time*/)
 	}
 }
 
+
+template<typename Archive>
+void Mouse::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.serialize("lastTime", lastTime);
+	ar.serialize("faze", faze);
+	ar.serialize("xrel", xrel);
+	ar.serialize("yrel", yrel);
+	ar.serialize("mouseMode", mouseMode);
+
+	// Don't serialzie status, curxrel, curyrel.
+	// These are controlled via (mouse button/motion) events
+}
+INSTANTIATE_SERIALIZE_METHODS(Mouse);
+REGISTER_POLYMORPHIC_INITIALIZER(Pluggable, Mouse, "Mouse");
+
 } // namespace openmsx
