@@ -1,4 +1,4 @@
-// $Id: main.cc 7371 2007-12-08 04:52:22Z mthuurne $
+// $Id: main.cc 8262 2008-09-22 18:21:53Z m9710797 $
 
 /*
  *  openmsx - the MSX emulator that aims for perfection
@@ -6,10 +6,9 @@
  */
 
 #include "Reactor.hh"
-#include "GlobalCommandController.hh"
+#include "CommandController.hh"
 #include "CommandLineParser.hh"
 #include "AfterCommand.hh"
-#include "SettingsConfig.hh"
 #include "CliServer.hh"
 #include "Interpreter.hh"
 #include "Display.hh"
@@ -32,7 +31,7 @@ namespace openmsx {
 
 static void initializeSDL()
 {
-	int flags = SDL_INIT_TIMER;
+	int flags = 0;
 #ifndef NDEBUG
 	flags |= SDL_INIT_NOPARACHUTE;
 #endif
@@ -62,10 +61,10 @@ static int main(int argc, char **argv)
 	try {
 		Thread::setMainThread();
 		Reactor reactor;
-		reactor.getGlobalCommandController().getInterpreter().init(argv[0]);
+		reactor.getCommandController().getInterpreter().init(argv[0]);
 		AfterCommand afterCommand(reactor,
 		                          reactor.getEventDistributor(),
-		                          reactor.getGlobalCommandController());
+		                          reactor.getCommandController());
 
 
 		CommandLineParser parser(reactor);
@@ -74,13 +73,12 @@ static int main(int argc, char **argv)
 
 		if (parseStatus != CommandLineParser::EXIT) {
 			initializeSDL();
-			reactor.getIconStatus();
 			if (!parser.isHiddenStartup()) {
 				reactor.getDisplay().getRenderSettings().
 					getRenderer().restoreDefault();
 			}
 			if (parseStatus != CommandLineParser::TEST) {
-				CliServer cliServer(reactor.getGlobalCommandController(),
+				CliServer cliServer(reactor.getCommandController(),
 				                    reactor.getEventDistributor(),
 				                    reactor.getGlobalCliComm());
 				reactor.run(parser);
