@@ -1,4 +1,4 @@
-// $Id: ESE_RAM.cc 6593 2007-06-03 23:12:19Z mthuurne $
+// $Id: ESE_RAM.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 /*
  * MEGA-SCSI and ESE-RAM cartridge:
@@ -24,43 +24,46 @@
 
 #include "ESE_RAM.hh"
 #include "SRAM.hh"
-#include "MSXMotherBoard.hh"
-#include "MSXCPU.hh"
 #include "StringOp.hh"
 #include "MSXException.hh"
 #include "XMLElement.hh"
+#include "serialize.hh"
 #include <cassert>
 
 namespace openmsx {
 
-ESE_RAM::ESE_RAM(MSXMotherBoard& motherBoard, const XMLElement& config,
-                   const EmuTime& time)
-	: MSXDevice(motherBoard, config, time)
-	, cpu(motherBoard.getCPU())
+static SRAM* createSRAM(MSXMotherBoard& motherBoard, const XMLElement& config,
+                        const std::string& name)
 {
 	unsigned sramSize = config.getChildDataAsInt("sramsize", 256); // size in kb
 	if (sramSize != 1024 && sramSize != 512 && sramSize != 256 && sramSize != 128) {
-		throw MSXException("SRAM size for " + getName() +
+		throw MSXException("SRAM size for " + name +
 			" should be 128, 256, 512 or 1024kB and not " +
 			StringOp::toString(sramSize) + "kB!");
 	}
 	sramSize *= 1024; // in bytes
-	sram.reset(new SRAM(motherBoard, getName() + " SRAM", sramSize, config));
-	blockMask = (sramSize / 8192) - 1;
+	return new SRAM(motherBoard, name + " SRAM", sramSize, config);
+}
+
+ESE_RAM::ESE_RAM(MSXMotherBoard& motherBoard, const XMLElement& config)
+	: MSXDevice(motherBoard, config)
+	, sram(createSRAM(motherBoard, config, getName()))
+	, blockMask((sram->getSize() / 8192) - 1)
+{
 }
 
 ESE_RAM::~ESE_RAM()
 {
 }
 
-void ESE_RAM::reset(const EmuTime& /*time*/)
+void ESE_RAM::reset(EmuTime::param /*time*/)
 {
 	for (int i = 0; i < 4; ++i) {
 		setSRAM(i, 0);
 	}
 }
 
-byte ESE_RAM::readMem(word address, const EmuTime& /*time*/)
+byte ESE_RAM::readMem(word address, EmuTime::param /*time*/)
 {
 	byte result;
 	if ((0x4000 <= address) && (address < 0xC000)) {
@@ -84,7 +87,7 @@ const byte* ESE_RAM::getReadCacheLine(word address) const
 	}
 }
 
-void ESE_RAM::writeMem(word address, byte value, const EmuTime& /*time*/)
+void ESE_RAM::writeMem(word address, byte value, EmuTime::param /*time*/)
 {
 	if ((0x6000 <= address) && (address < 0x8000)) {
 		byte region = ((address >> 11) & 3);
@@ -113,10 +116,21 @@ byte* ESE_RAM::getWriteCacheLine(word address) const
 
 void ESE_RAM::setSRAM(unsigned region, byte block)
 {
-	cpu.invalidateMemCache(region * 0x2000 + 0x4000, 0x2000);
+	invalidateMemCache(region * 0x2000 + 0x4000, 0x2000);
 	assert(region < 4);
 	isWriteable[region] = block & 0x80;
 	mapped[region] = block & blockMask;
 }
 
+template<typename Archive>
+void ESE_RAM::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<MSXDevice>(*this);
+	ar.serialize("SRAM", *sram);
+	ar.serialize("isWriteable", isWriteable);
+	ar.serialize("mapped", mapped);
+}
+INSTANTIATE_SERIALIZE_METHODS(ESE_RAM);
+REGISTER_MSXDEVICE(ESE_RAM, "ESE_RAM");
+
 } // namespace openmsx
