@@ -1,9 +1,9 @@
-// $Id: ESE_SCC.cc 6761 2007-07-30 16:41:52Z m9710797 $
+// $Id: ESE_SCC.cc 8352 2008-11-12 18:39:08Z m9710797 $
 /*
  * 'Ese-SCC' cartride and 'MEGA-SCSI with SCC'(alias WAVE-SCSI) cartrige.
  *
  * Specification:
- *  SRAM(MegaROM) controller: KONAMI_SCC type (KONAMI5)
+ *  SRAM(MegaROM) controller: KONAMI_SCC type
  *  SRAM capacity           : 128, 256, 512kB and 1024kb(WAVE-SCSI only)
  *  SCSI Protocol controller: Fujitsu MB89352A
  *
@@ -51,23 +51,20 @@
 #include "SRAM.hh"
 #include "SCC.hh"
 #include "MB89352.hh"
-#include "MSXMotherBoard.hh"
-#include "MSXCPU.hh"
 #include "StringOp.hh"
 #include "MSXException.hh"
 #include "XMLElement.hh"
+#include "serialize.hh"
 #include <cassert>
 
 namespace openmsx {
 
-ESE_SCC::ESE_SCC(MSXMotherBoard& motherBoard, const XMLElement& config,
-                 const EmuTime& time, bool withSCSI)
-	: MSXDevice(motherBoard, config, time)
-	, cpu(motherBoard.getCPU())
+static SRAM* createSRAM(MSXMotherBoard& motherBoard, const XMLElement& config,
+                        bool withSCSI, const std::string& name)
 {
 	unsigned sramSize = config.getChildDataAsInt("sramsize", 256); // size in kb
 	if (sramSize != 1024 && sramSize != 512 && sramSize != 256 && sramSize != 128) {
-		throw MSXException("SRAM size for " + getName() +
+		throw MSXException("SRAM size for " + name +
 			" should be 128, 256, 512 or 1024kB and not " +
 			StringOp::toString(sramSize) + "kB!");
 	}
@@ -75,9 +72,17 @@ ESE_SCC::ESE_SCC(MSXMotherBoard& motherBoard, const XMLElement& config,
 		throw MSXException("1024kB SRAM is only allowed in WAVE-SCSI!");
 	}
 	sramSize *= 1024; // in bytes
-	sram.reset(new SRAM(motherBoard, getName() + " SRAM", sramSize, config));
-	mapperMask = (sramSize / 0x2000) - 1;
+	return new SRAM(motherBoard, name + " SRAM", sramSize, config);
+}
 
+ESE_SCC::ESE_SCC(MSXMotherBoard& motherBoard, const XMLElement& config,
+                 bool withSCSI)
+	: MSXDevice(motherBoard, config)
+	, sram(createSRAM(motherBoard, config, withSCSI, getName()))
+	, scc(new SCC(motherBoard, getName(), config, getCurrentTime()))
+	, spc(withSCSI ? new MB89352(motherBoard, config) : NULL)
+	, mapperMask((sram->getSize() / 0x2000) - 1)
+{
 	// initialized mapper
 	sccEnable   = false;
 	spcEnable   = false;
@@ -85,21 +90,13 @@ ESE_SCC::ESE_SCC(MSXMotherBoard& motherBoard, const XMLElement& config,
 	for (int i = 0; i < 4; ++i) {
 		mapper[i] = i;
 	}
-
-	// initialize SCC
-	scc.reset(new SCC(motherBoard, getName(), config, time));
-
-	if (withSCSI) {
-		// initialize SPC
-		spc.reset(new MB89352(motherBoard, config));
-	}
 }
 
 ESE_SCC::~ESE_SCC()
 {
 }
 
-void ESE_SCC::reset(const EmuTime& time)
+void ESE_SCC::reset(EmuTime::param time)
 {
 	setMapperHigh(0);
 	for (int i = 0; i < 4; ++i) {
@@ -130,7 +127,7 @@ void ESE_SCC::setMapperLow(unsigned page, byte value)
 		flush = true;
 	}
 	if (flush) {
-		cpu.invalidateMemCache(0x4000 + 0x2000 * page, 0x2000);
+		invalidateMemCache(0x4000 + 0x2000 * page, 0x2000);
 	}
 }
 
@@ -155,11 +152,11 @@ void ESE_SCC::setMapperHigh(byte value)
 		flush = true;
 	}
 	if (flush) {
-		cpu.invalidateMemCache(0x4000, 0x2000);
+		invalidateMemCache(0x4000, 0x2000);
 	}
 }
 
-byte ESE_SCC::readMem(word address, const EmuTime& time)
+byte ESE_SCC::readMem(word address, EmuTime::param time)
 {
 	unsigned page = address / 0x2000 - 2;
 	// SPC
@@ -179,7 +176,7 @@ byte ESE_SCC::readMem(word address, const EmuTime& time)
 	return (*sram)[mapper[page] * 0x2000 + (address & 0x1fff)];
 }
 
-byte ESE_SCC::peekMem(word address, const EmuTime& time) const
+byte ESE_SCC::peekMem(word address, EmuTime::param time) const
 {
 	unsigned page = address / 0x2000 - 2;
 	// SPC
@@ -214,7 +211,7 @@ const byte* ESE_SCC::getReadCacheLine(word address) const
 	return &(*sram)[mapper[page] * 0x2000 + (address & 0x1fff)];
 }
 
-void ESE_SCC::writeMem(word address, byte value, const EmuTime& time)
+void ESE_SCC::writeMem(word address, byte value, EmuTime::param time)
 {
 	unsigned page = address / 0x2000 - 2;
 	// SPC Write
@@ -258,4 +255,20 @@ byte* ESE_SCC::getWriteCacheLine(word /*address*/) const
 	return NULL; // not cacheable
 }
 
+
+template<typename Archive>
+void ESE_SCC::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<MSXDevice>(*this);
+	ar.serialize("sram", *sram);
+	ar.serialize("scc", *scc);
+	ar.serialize("MB89352", *spc);
+	ar.serialize("mapper", mapper);
+	ar.serialize("spcEnable", spcEnable);
+	ar.serialize("sccEnable", sccEnable);
+	ar.serialize("writeEnable", writeEnable);
+}
+INSTANTIATE_SERIALIZE_METHODS(ESE_SCC);
+REGISTER_MSXDEVICE(ESE_SCC, "ESE_SCC");
+
 } // namespace openmsx
