@@ -1,19 +1,22 @@
-// $Id: MSXRam.cc 5751 2006-10-05 19:34:54Z m9710797 $
+// $Id: MSXRam.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "MSXRam.hh"
 #include "CacheLine.hh"
 #include "CheckedRam.hh"
 #include "XMLElement.hh"
 #include "MSXException.hh"
+#include "serialize.hh"
+
+#include "Ram.hh" // because we serialize Ram instead of CheckedRam
 
 namespace openmsx {
 
-MSXRam::MSXRam(MSXMotherBoard& motherBoard, const XMLElement& config,
-               const EmuTime& time)
-	: MSXDevice(motherBoard, config, time)
+MSXRam::MSXRam(MSXMotherBoard& motherBoard, const XMLElement& config)
+	: MSXDevice(motherBoard, config)
+	, base(config.getChildDataAsInt("base", 0))
+	, size(config.getChildDataAsInt("size", 0x10000))
+	, checkedRam(new CheckedRam(motherBoard, getName(), "ram", size))
 {
-	base = config.getChildDataAsInt("base", 0);
-	size = config.getChildDataAsInt("size", 0x10000);
 	if ((size > 0x10000) || (base >= 0x10000)) {
 		throw MSXException("Invalid base/size for " + getName() +
 		                   ", must be in range [0x0000,0x10000).");
@@ -22,11 +25,9 @@ MSXRam::MSXRam(MSXMotherBoard& motherBoard, const XMLElement& config,
 		throw MSXException("Invalid base/size alignment for " +
 		                   getName());
 	}
-
-	checkedRam.reset(new CheckedRam(motherBoard, getName(), "ram", size));
 }
 
-void MSXRam::powerUp(const EmuTime& /*time*/)
+void MSXRam::powerUp(EmuTime::param /*time*/)
 {
 	checkedRam->clear();
 }
@@ -37,17 +38,17 @@ word MSXRam::translate(word address) const
 	return (tmp < size) ? tmp : tmp & (size - 1);
 }
 
-byte MSXRam::peekMem(word address, const EmuTime& /*time*/) const
+byte MSXRam::peekMem(word address, EmuTime::param /*time*/) const
 {
 	return checkedRam->peek(translate(address));
 }
 
-byte MSXRam::readMem(word address, const EmuTime& /*time*/)
+byte MSXRam::readMem(word address, EmuTime::param /*time*/)
 {
 	return checkedRam->read(translate(address));
 }
 
-void MSXRam::writeMem(word address, byte value, const EmuTime& /*time*/)
+void MSXRam::writeMem(word address, byte value, EmuTime::param /*time*/)
 {
 	checkedRam->write(translate(address), value);
 }
@@ -62,4 +63,14 @@ byte* MSXRam::getWriteCacheLine(word start) const
 	return checkedRam->getWriteCacheLine(translate(start));
 }
 
+template<typename Archive>
+void MSXRam::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<MSXDevice>(*this);
+	// TODO ar.serialize("checkedRam", checkedRam);
+	ar.serialize("ram", checkedRam->getUncheckedRam());
+}
+INSTANTIATE_SERIALIZE_METHODS(MSXRam);
+REGISTER_MSXDEVICE(MSXRam, "Ram");
+
 } // namespace openmsx
