@@ -1,7 +1,8 @@
-// $Id: Ram.cc 6463 2007-05-12 08:27:34Z m9710797 $
+// $Id: Ram.cc 7928 2008-06-28 19:43:43Z m9710797 $
 
 #include "Ram.hh"
 #include "SimpleDebuggable.hh"
+#include "serialize.hh"
 #include <cstring>
 
 namespace openmsx {
@@ -62,4 +63,12 @@ void RamDebuggable::write(unsigned address, byte value)
 	ram[address] = value;
 }
 
+
+template<typename Archive>
+void Ram::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.serialize_blob("ram", ram, size);
+}
+INSTANTIATE_SERIALIZE_METHODS(Ram);
+
 } // namespace openmsx
