@@ -1,4 +1,4 @@
-// $Id: RomAscii16kB.cc 4858 2005-11-12 21:53:21Z m9710797 $
+// $Id: RomAscii16kB.cc 8353 2008-11-13 18:38:44Z m9710797 $
 
 // ASCII 16kB cartridges
 //
@@ -12,18 +12,19 @@
 
 #include "RomAscii16kB.hh"
 #include "Rom.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
 RomAscii16kB::RomAscii16kB(
-	MSXMotherBoard& motherBoard, const XMLElement& config,
-	const EmuTime& time, std::auto_ptr<Rom> rom)
-	: Rom16kBBlocks(motherBoard, config, time, rom)
+		MSXMotherBoard& motherBoard, const XMLElement& config,
+		std::auto_ptr<Rom> rom)
+	: Rom16kBBlocks(motherBoard, config, rom)
 {
-	reset(time);
+	reset(EmuTime::dummy());
 }
 
-void RomAscii16kB::reset(const EmuTime& /*time*/)
+void RomAscii16kB::reset(EmuTime::param /*time*/)
 {
 	setBank(0, unmappedRead);
 	setRom (1, 0);
@@ -31,7 +32,7 @@ void RomAscii16kB::reset(const EmuTime& /*time*/)
 	setBank(3, unmappedRead);
 }
 
-void RomAscii16kB::writeMem(word address, byte value, const EmuTime& /*time*/)
+void RomAscii16kB::writeMem(word address, byte value, EmuTime::param /*time*/)
 {
 	if ((0x6000 <= address) && (address < 0x7800) && !(address & 0x0800)) {
 		byte region = ((address >> 12) & 1) + 1;
@@ -48,4 +49,6 @@ byte* RomAscii16kB::getWriteCacheLine(word address) const
 	}
 }
 
+REGISTER_MSXDEVICE(RomAscii16kB, "RomAscii16kB");
+
 } // namespace openmsx
