@@ -1,4 +1,4 @@
-// $Id: RomFSA1FM.cc 6593 2007-06-03 23:12:19Z mthuurne $
+// $Id: RomFSA1FM.cc 8353 2008-11-13 18:38:44Z m9710797 $
 
 // FS-A1FM mappers
 //   based on info from:
@@ -35,19 +35,19 @@
 //                0x84-0x87 0x8C-0x8F  contain (same) 8kB RAM
 
 #include "RomFSA1FM.hh"
-#include "MSXCPU.hh"
 #include "CacheLine.hh"
 #include "Rom.hh"
 #include "SRAM.hh"
 #include "MSXMotherBoard.hh"
 #include "FirmwareSwitch.hh"
 #include "XMLElement.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
 // 8kb shared sram //
 
-static SRAM* allocSRAM(MSXMotherBoard& motherBoard, const XMLElement& config)
+static SRAM& allocSRAM(MSXMotherBoard& motherBoard, const XMLElement& config)
 {
 	MSXMotherBoard::SharedStuff& info =
 		motherBoard.getSharedStuff("FSA1FM-sram");
@@ -57,7 +57,7 @@ static SRAM* allocSRAM(MSXMotherBoard& motherBoard, const XMLElement& config)
 		                      0x2000, config);
 	}
 	++info.counter;
-	return reinterpret_cast<SRAM*>(info.stuff);
+	return *reinterpret_cast<SRAM*>(info.stuff);
 }
 
 static void releaseSRAM(MSXMotherBoard& motherBoard)
@@ -78,13 +78,12 @@ static void releaseSRAM(MSXMotherBoard& motherBoard)
 // Mapper for slot 3-1 //
 
 RomFSA1FM1::RomFSA1FM1(MSXMotherBoard& motherBoard, const XMLElement& config,
-                       const EmuTime& time, std::auto_ptr<Rom> rom)
-	: MSXRom(motherBoard, config, time, rom)
+                       std::auto_ptr<Rom> rom)
+	: MSXRom(motherBoard, config, rom)
 	, firmwareSwitch(
 	      new FirmwareSwitch(motherBoard.getCommandController(), config))
+	, sram(allocSRAM(motherBoard, config))
 {
-	sram = allocSRAM(motherBoard, config);
-	reset(time);
 }
 
 RomFSA1FM1::~RomFSA1FM1()
@@ -92,21 +91,21 @@ RomFSA1FM1::~RomFSA1FM1()
 	releaseSRAM(getMotherBoard());
 }
 
-void RomFSA1FM1::reset(const EmuTime& /*time*/)
+void RomFSA1FM1::reset(EmuTime::param /*time*/)
 {
 	// initial rom bank is undefined
 }
 
-byte RomFSA1FM1::peekMem(word address, const EmuTime& /*time*/) const
+byte RomFSA1FM1::peekMem(word address, EmuTime::param /*time*/) const
 {
 	if ((0x4000 <= address) && (address < 0x6000)) {
 		// read rom
-		return (*rom)[(0x2000 * ((*sram)[0x1FC4] & 0x0F)) +
+		return (*rom)[(0x2000 * (sram[0x1FC4] & 0x0F)) +
 		              (address & 0x1FFF)];
 	} else if ((0x7FC0 <= address) && (address < 0x7FD0)) {
 		switch (address & 0x0F) {
 		case 4:
-			return (*sram)[address & 0x1FFF];
+			return sram[address & 0x1FFF];
 		case 6:
 			return firmwareSwitch->getStatus() ? 0xFB : 0xFF;
 		default:
@@ -115,13 +114,13 @@ byte RomFSA1FM1::peekMem(word address, const EmuTime& /*time*/) const
 	} else if ((0x6000 <= address) && (address < 0x8000)) {
 		// read sram
 		// TODO are there multiple sram blocks?
-		return (*sram)[address & 0x1FFF];
+		return sram[address & 0x1FFF];
 	} else {
 		return 0xFF;
 	}
 }
 
-byte RomFSA1FM1::readMem(word address, const EmuTime& time)
+byte RomFSA1FM1::readMem(word address, EmuTime::param time)
 {
 	return peekMem(address, time);
 }
@@ -133,17 +132,17 @@ const byte* RomFSA1FM1::getReadCacheLine(word address) const
 		return NULL;
 	} else if ((0x4000 <= address) && (address < 0x6000)) {
 		// read rom
-		return &(*rom)[(0x2000 * ((*sram)[0x1FC4] & 0x0F)) +
+		return &(*rom)[(0x2000 * (sram[0x1FC4] & 0x0F)) +
 		               (address & 0x1FFF)];
 	} else if ((0x6000 <= address) && (address < 0x8000)) {
 		// read sram
-		return &(*sram)[address & 0x1FFF];
+		return &sram[address & 0x1FFF];
 	} else {
 		return unmappedRead;
 	}
 }
 
-void RomFSA1FM1::writeMem(word address, byte value, const EmuTime& /*time*/)
+void RomFSA1FM1::writeMem(word address, byte value, EmuTime::param /*time*/)
 {
 	// TODO 0x7FC0 - 0x7FCF is modem IO area
 
@@ -151,9 +150,9 @@ void RomFSA1FM1::writeMem(word address, byte value, const EmuTime& /*time*/)
 	if ((0x6000 <= address) && (address < 0x8000)) {
 		if (address == 0x7FC4) {
 			// switch rom bank
-			cpu.invalidateMemCache(0x4000, 0x2000);
+			invalidateMemCache(0x4000, 0x2000);
 		}
-		sram->write(address & 0x1FFF, value);
+		sram.write(address & 0x1FFF, value);
 	}
 }
 
@@ -170,15 +169,25 @@ byte* RomFSA1FM1::getWriteCacheLine(word address) const
 	}
 }
 
+template<typename Archive>
+void RomFSA1FM1::serialize(Archive& ar, unsigned /*version*/)
+{
+	// skip MSXRom base class
+	ar.template serializeBase<MSXDevice>(*this);
+	// don't serialize (shared) sram here, rely on RomFSA1FM2 to do that
+}
+INSTANTIATE_SERIALIZE_METHODS(RomFSA1FM1);
+REGISTER_MSXDEVICE(RomFSA1FM1, "RomFSA1FM1");
+
 
 // Mapper for slot 3-3 //
 
 RomFSA1FM2::RomFSA1FM2(MSXMotherBoard& motherBoard, const XMLElement& config,
-                       const EmuTime& time, std::auto_ptr<Rom> rom)
-	: Rom8kBBlocks(motherBoard, config, time, rom)
+                       std::auto_ptr<Rom> rom)
+	: Rom8kBBlocks(motherBoard, config, rom)
+	, sram(allocSRAM(motherBoard, config))
 {
-	sram = allocSRAM(motherBoard, config);
-	reset(time);
+	reset(EmuTime::dummy());
 }
 
 RomFSA1FM2::~RomFSA1FM2()
@@ -186,7 +195,7 @@ RomFSA1FM2::~RomFSA1FM2()
 	releaseSRAM(getMotherBoard());
 }
 
-void RomFSA1FM2::reset(const EmuTime& /*time*/)
+void RomFSA1FM2::reset(EmuTime::param /*time*/)
 {
 	control = 0;
 	for (int region = 0; region < 6; ++region) {
@@ -196,7 +205,7 @@ void RomFSA1FM2::reset(const EmuTime& /*time*/)
 	changeBank(7, 0);
 }
 
-byte RomFSA1FM2::peekMem(word address, const EmuTime& time) const
+byte RomFSA1FM2::peekMem(word address, EmuTime::param time) const
 {
 	byte result;
 	if (0xC000 <= address) {
@@ -205,7 +214,7 @@ byte RomFSA1FM2::peekMem(word address, const EmuTime& time) const
 		// read mapper state
 		result = bankSelect[address & 7];
 	} else if (isRam[address >> 13]) {
-		result = (*sram)[address & 0x1FFF];
+		result = sram[address & 0x1FFF];
 	} else if (isEmpty[address >> 13]) {
 		result = 0xFF;
 	} else {
@@ -215,7 +224,7 @@ byte RomFSA1FM2::peekMem(word address, const EmuTime& time) const
 	return result;
 }
 
-byte RomFSA1FM2::readMem(word address, const EmuTime& time)
+byte RomFSA1FM2::readMem(word address, EmuTime::param time)
 {
 	return peekMem(address, time);
 }
@@ -227,7 +236,7 @@ const byte* RomFSA1FM2::getReadCacheLine(word address) const
 	} else if ((0x7FF0 & CacheLine::HIGH) == address) {
 		return NULL;
 	} else if (isRam[address >> 13]) {
-		return &(*sram)[address & 0x1FFF];
+		return &sram[address & 0x1FFF];
 	} else if (isEmpty[address >> 13]) {
 		return unmappedRead;
 	} else {
@@ -236,43 +245,43 @@ const byte* RomFSA1FM2::getReadCacheLine(word address) const
 }
 
 void RomFSA1FM2::writeMem(word address, byte value,
-                          const EmuTime& /*time*/)
+                          EmuTime::param /*time*/)
 {
 	//PRT_DEBUG("FSA1FM2: write "<<hex<<(int)address<<" "<<(int)value<<dec);
 
 	if ((0x6000 <= address) && (address < 0x7FF0)) {
 		// set mapper state
 		switch (address & 0x1C00) {
-			case 0x0000:
-				changeBank(2, value);
-				break;
-			case 0x0400:
-				changeBank(0, value);
-				break;
-			case 0x0800:
-				changeBank(3, value);
-				break;
-			case 0x0C00:
-				changeBank(1, value);
-				break;
-			case 0x1000:
-				changeBank(4, value);
-				break;
-			case 0x1400:
-				// nothing
-				break;
-			case 0x1800:
-				changeBank(5, value);
-				break;
-			case 0x1C00:
-				// nothing
-				break;
+		case 0x0000:
+			changeBank(2, value);
+			break;
+		case 0x0400:
+			changeBank(0, value);
+			break;
+		case 0x0800:
+			changeBank(3, value);
+			break;
+		case 0x0C00:
+			changeBank(1, value);
+			break;
+		case 0x1000:
+			changeBank(4, value);
+			break;
+		case 0x1400:
+			// nothing
+			break;
+		case 0x1800:
+			changeBank(5, value);
+			break;
+		case 0x1C00:
+			// nothing
+			break;
 		}
 	} else if (address == 0x7FF9) {
 		// write control byte
 		control = value;
 	} else if (isRam[address >> 13]) {
-		sram->write(address & 0x1FFF, value);
+		sram.write(address & 0x1FFF, value);
 	}
 }
 
@@ -298,7 +307,7 @@ void RomFSA1FM2::changeBank(byte region, byte bank)
 			isRam[region]   = false;
 			isEmpty[region] = true;
 		}
-		cpu.invalidateMemCache(0x2000 * region, 0x2000);
+		invalidateMemCache(0x2000 * region, 0x2000);
 	} else {
 		isRam[region]   = false;
 		isEmpty[region] = false;
@@ -306,4 +315,23 @@ void RomFSA1FM2::changeBank(byte region, byte bank)
 	}
 }
 
+template<typename Archive>
+void RomFSA1FM2::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<Rom8kBBlocks>(*this);
+	// note: SRAM can be serialized in this class (as opposed to
+	//       Rom8kBBlocks), because we don't use setBank to map it
+	ar.serialize("SRAM", sram);
+	ar.serialize("bankSelect", bankSelect);
+	ar.serialize("control", control);
+	if (ar.isLoader()) {
+		// recalculate 'isRam' and 'isEmpty' from bankSelect
+		for (int region = 0; region < 8; ++region) {
+			changeBank(region, bankSelect[region]);
+		}
+	}
+}
+INSTANTIATE_SERIALIZE_METHODS(RomFSA1FM2);
+REGISTER_MSXDEVICE(RomFSA1FM2, "RomFSA1FM2");
+
 } // namespace openmsx
