@@ -1,4 +1,4 @@
-// $Id: RomHalnote.cc 7100 2007-09-29 13:23:48Z m9710797 $
+// $Id: RomHalnote.cc 8353 2008-11-13 18:38:44Z m9710797 $
 
 /*
  * HALNOTE mapper
@@ -10,10 +10,10 @@
  * can also be mapped as 256 pages of 2kB. There is 16kB SRAM.
  *
  * Main bankswitch registers:
- *   bank 0,  region: [0x4000-0x5FFF],  switch addr: 0x4FFF 
- *   bank 1,  region: [0x6000-0x7FFF],  switch addr: 0x6FFF 
- *   bank 2,  region: [0x8000-0x9FFF],  switch addr: 0x8FFF 
- *   bank 3,  region: [0xA000-0xBFFF],  switch addr: 0xAFFF 
+ *   bank 0,  region: [0x4000-0x5FFF],  switch addr: 0x4FFF
+ *   bank 1,  region: [0x6000-0x7FFF],  switch addr: 0x6FFF
+ *   bank 2,  region: [0x8000-0x9FFF],  switch addr: 0x8FFF
+ *   bank 3,  region: [0xA000-0xBFFF],  switch addr: 0xAFFF
  * Sub-bankswitch registers:
  *   bank 0,  region: [0x7000-0x77FF],  switch addr: 0x77FF
  *   bank 1,  region: [0x7800-0x7FFF],  switch addr: 0x7FFF
@@ -27,25 +27,25 @@
 
 #include "RomHalnote.hh"
 #include "CacheLine.hh"
-#include "MSXCPU.hh"
 #include "Rom.hh"
 #include "SRAM.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
 RomHalnote::RomHalnote(MSXMotherBoard& motherBoard, const XMLElement& config,
-                       const EmuTime& time, std::auto_ptr<Rom> rom)
-	: Rom8kBBlocks(motherBoard, config, time, rom)
-	, sram(new SRAM(motherBoard, getName() + " SRAM", 0x4000, config))
+                       std::auto_ptr<Rom> rom)
+	: Rom8kBBlocks(motherBoard, config, rom)
 {
-	reset(time);
+	sram.reset(new SRAM(motherBoard, getName() + " SRAM", 0x4000, config));
+	reset(EmuTime::dummy());
 }
 
 RomHalnote::~RomHalnote()
 {
 }
 
-void RomHalnote::reset(const EmuTime& /*time*/)
+void RomHalnote::reset(EmuTime::param /*time*/)
 {
 	subBanks[0] = subBanks[1] = 0;
 	sramEnabled = false;
@@ -71,13 +71,13 @@ const byte* RomHalnote::getReadCacheLine(word address) const
 		return Rom8kBBlocks::getReadCacheLine(address);
 	}
 }
-byte RomHalnote::readMem(word address, const EmuTime& /*time*/)
+byte RomHalnote::readMem(word address, EmuTime::param /*time*/)
 {
 	// all reads are cacheable, reuse that implementation
 	return *getReadCacheLine(address);
 }
 
-void RomHalnote::writeMem(word address, byte value, const EmuTime& /*time*/)
+void RomHalnote::writeMem(word address, byte value, EmuTime::param /*time*/)
 {
 	if (address < 0x4000) {
 		// SRAM region
@@ -91,7 +91,7 @@ void RomHalnote::writeMem(word address, byte value, const EmuTime& /*time*/)
 			if (subBanks[subBank] != value) {
 				subBanks[subBank] = value;
 				if (subMapperEnabled) {
-					cpu.invalidateMemCache(
+					invalidateMemCache(
 						0x7000 + subBank * 0x800, 0x800);
 				}
 			}
@@ -117,7 +117,7 @@ void RomHalnote::writeMem(word address, byte value, const EmuTime& /*time*/)
 				bool newSubMapperEnabled = value & 0x80;
 				if (newSubMapperEnabled != subMapperEnabled) {
 					subMapperEnabled = newSubMapperEnabled;
-					cpu.invalidateMemCache(0x7000, 0x1000);
+					invalidateMemCache(0x7000, 0x1000);
 				}
 			}
 		}
@@ -145,4 +145,16 @@ byte* RomHalnote::getWriteCacheLine(word address) const
 	return unmappedWrite;
 }
 
+template<typename Archive>
+void RomHalnote::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<Rom8kBBlocks>(*this);
+	ar.serialize("subBanks", subBanks);
+	ar.serialize("sramEnabled", sramEnabled);
+	ar.serialize("subMapperEnabled", subMapperEnabled);
+
+}
+INSTANTIATE_SERIALIZE_METHODS(RomHalnote);
+REGISTER_MSXDEVICE(RomHalnote, "RomHalnote");
+
 } // namespace openmsx
