@@ -1,26 +1,27 @@
-// $Id: RomMSXDOS2.cc 6733 2007-07-17 18:33:17Z manuelbi $
+// $Id: RomMSXDOS2.cc 8353 2008-11-13 18:38:44Z m9710797 $
 
 #include "RomMSXDOS2.hh"
 #include "Rom.hh"
 #include "CacheLine.hh"
 #include "MSXException.hh"
+#include "serialize.hh"
 #include <cassert>
 
 namespace openmsx {
 
 RomMSXDOS2::RomMSXDOS2(
-	MSXMotherBoard& motherBoard, const XMLElement& config,
-	const EmuTime& time, std::auto_ptr<Rom> rom_)
-	: Rom16kBBlocks(motherBoard, config, time, rom_)
+		MSXMotherBoard& motherBoard, const XMLElement& config,
+		std::auto_ptr<Rom> rom_)
+	: Rom16kBBlocks(motherBoard, config, rom_)
+	, range((*rom)[0x94])
 {
-	range = (*rom)[0x94];
 	if ((range != 0x00) && (range != 0x60) && (range != 0x7f)) {
 		throw MSXException("Invalid rom for MSXDOS2 mapper");
 	}
-	reset(time);
+	reset(EmuTime::dummy());
 }
 
-void RomMSXDOS2::reset(const EmuTime& /*time*/)
+void RomMSXDOS2::reset(EmuTime::param /*time*/)
 {
 	setBank(0, unmappedRead);
 	setRom (1, 0);
@@ -28,7 +29,7 @@ void RomMSXDOS2::reset(const EmuTime& /*time*/)
 	setBank(3, unmappedRead);
 }
 
-void RomMSXDOS2::writeMem(word address, byte value, const EmuTime& /*time*/)
+void RomMSXDOS2::writeMem(word address, byte value, EmuTime::param /*time*/)
 {
 	switch (range) {
 	case 0x00:
@@ -64,4 +65,6 @@ byte* RomMSXDOS2::getWriteCacheLine(word address) const
 	return unmappedWrite;
 }
 
+REGISTER_MSXDEVICE(RomMSXDOS2, "RomMSXDOS2");
+
 } // namespace openmsx
