@@ -1,4 +1,4 @@
-// $Id: RomPadial8kB.cc 5954 2006-12-22 08:10:58Z mthuurne $
+// $Id: RomPadial8kB.cc 8353 2008-11-13 18:38:44Z m9710797 $
 
 // Padial 8kB
 //
@@ -10,17 +10,18 @@
 
 #include "RomPadial8kB.hh"
 #include "Rom.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
 RomPadial8kB::RomPadial8kB(MSXMotherBoard& motherBoard, const XMLElement& config,
-                           const EmuTime& time, std::auto_ptr<Rom> rom)
-	: RomAscii8kB(motherBoard, config, time, rom)
+                           std::auto_ptr<Rom> rom)
+	: RomAscii8kB(motherBoard, config, rom)
 {
-	reset(time);
+	reset(EmuTime::dummy());
 }
 
-void RomPadial8kB::reset(const EmuTime& /*time*/)
+void RomPadial8kB::reset(EmuTime::param /*time*/)
 {
 	setRom (0, 0);
 	setRom (1, 0);
@@ -31,4 +32,6 @@ void RomPadial8kB::reset(const EmuTime& /*time*/)
 	setBank(7, unmappedRead);
 }
 
+REGISTER_MSXDEVICE(RomPadial8kB, "RomPadial8kB");
+
 } // namespace openmsx
