@@ -1,4 +1,4 @@
-// $Id: RomRType.cc 4858 2005-11-12 21:53:21Z m9710797 $
+// $Id: RomRType.cc 8353 2008-11-13 18:38:44Z m9710797 $
 
 // R-Type cartridges
 //
@@ -11,17 +11,18 @@
 
 #include "RomRType.hh"
 #include "Rom.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
 RomRType::RomRType(MSXMotherBoard& motherBoard, const XMLElement& config,
-                   const EmuTime& time, std::auto_ptr<Rom> rom)
-	: Rom16kBBlocks(motherBoard, config, time, rom)
+                   std::auto_ptr<Rom> rom)
+	: Rom16kBBlocks(motherBoard, config, rom)
 {
-	reset(time);
+	reset(EmuTime::dummy());
 }
 
-void RomRType::reset(const EmuTime& /*time*/)
+void RomRType::reset(EmuTime::param /*time*/)
 {
 	setBank(0, unmappedRead);
 	setRom (1, 0x17);
@@ -29,7 +30,7 @@ void RomRType::reset(const EmuTime& /*time*/)
 	setBank(3, unmappedRead);
 }
 
-void RomRType::writeMem(word address, byte value, const EmuTime& /*time*/)
+void RomRType::writeMem(word address, byte value, EmuTime::param /*time*/)
 {
 	if ((0x7000 <= address) && (address < 0x8000)) {
 		value &= (value & 0x10) ? 0x17 : 0x1F;
@@ -46,4 +47,6 @@ byte* RomRType::getWriteCacheLine(word address) const
 	}
 }
 
+REGISTER_MSXDEVICE(RomRType, "RomRType");
+
 } // namespace openmsx
