@@ -4,17 +4,18 @@
 #include "MSXMotherBoard.hh"
 #include "MSXCPUInterface.hh"
 #include "Rom.hh"
+#include "serialize.hh"
 #include <cassert>
 
 namespace openmsx {
 
 RomSuperLodeRunner::RomSuperLodeRunner(
-	MSXMotherBoard& motherBoard, const XMLElement& config,
-	const EmuTime& time, std::auto_ptr<Rom> rom)
-	: Rom16kBBlocks(motherBoard, config, time, rom)
+		MSXMotherBoard& motherBoard, const XMLElement& config,
+		std::auto_ptr<Rom> rom)
+	: Rom16kBBlocks(motherBoard, config, rom)
 {
 	getMotherBoard().getCPUInterface().registerGlobalWrite(*this, 0x0000);
-	reset(time);
+	reset(EmuTime::dummy());
 }
 
 RomSuperLodeRunner::~RomSuperLodeRunner()
@@ -22,7 +23,7 @@ RomSuperLodeRunner::~RomSuperLodeRunner()
 	getMotherBoard().getCPUInterface().unregisterGlobalWrite(*this, 0x0000);
 }
 
-void RomSuperLodeRunner::reset(const EmuTime& /*time*/)
+void RomSuperLodeRunner::reset(EmuTime::param /*time*/)
 {
 	setBank(0, unmappedRead);
 	setBank(1, unmappedRead);
@@ -30,11 +31,13 @@ void RomSuperLodeRunner::reset(const EmuTime& /*time*/)
 	setBank(3, unmappedRead);
 }
 
-void RomSuperLodeRunner::globalWrite(word address, byte value, const EmuTime& /*time*/)
+void RomSuperLodeRunner::globalWrite(word address, byte value, EmuTime::param /*time*/)
 {
 	assert(address == 0);
 	(void)address;
 	setRom(2, value);
 }
 
+REGISTER_MSXDEVICE(RomSuperLodeRunner, "RomSuperLodeRunner");
+
 } // namespace openmsx
