@@ -1,4 +1,4 @@
-// $Id: RomSynthesizer.cc 6457 2007-05-10 20:53:43Z manuelbi $
+// $Id: RomSynthesizer.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 /* On Sat, 3 Apr 2004, Manuel Pazos wrote:
  *
@@ -23,35 +23,35 @@
 #include "DACSound8U.hh"
 #include "CacheLine.hh"
 #include "Rom.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
 RomSynthesizer::RomSynthesizer(
-	MSXMotherBoard& motherBoard, const XMLElement& config,
-	const EmuTime& time, std::auto_ptr<Rom> rom)
-	: Rom16kBBlocks(motherBoard, config, time, rom)
+		MSXMotherBoard& motherBoard, const XMLElement& config,
+		std::auto_ptr<Rom> rom)
+	: Rom16kBBlocks(motherBoard, config, rom)
+	, dac(new DACSound8U(motherBoard.getMSXMixer(), "Synthesizer-DAC",
+	                     "Konami Synthesizer's DAC", config))
 {
 	setBank(0, unmappedRead);
 	setRom (1, 0);
 	setRom (2, 1);
 	setBank(3, unmappedRead);
 
-	dac.reset(new DACSound8U(motherBoard.getMSXMixer(), "Synthesizer-DAC",
-	                         "Konami Synthesizer's DAC", config, time));
-
-	reset(time);
+	reset(getCurrentTime());
 }
 
 RomSynthesizer::~RomSynthesizer()
 {
 }
 
-void RomSynthesizer::reset(const EmuTime& time)
+void RomSynthesizer::reset(EmuTime::param time)
 {
 	dac->reset(time);
 }
 
-void RomSynthesizer::writeMem(word address, byte value, const EmuTime& time)
+void RomSynthesizer::writeMem(word address, byte value, EmuTime::param time)
 {
 	if ((address & 0xC010) == 0x4000) {
 		dac->writeDAC(value, time);
@@ -67,4 +67,13 @@ byte* RomSynthesizer::getWriteCacheLine(word address) const
 	}
 }
 
+template<typename Archive>
+void RomSynthesizer::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<Rom16kBBlocks>(*this);
+	ar.serialize("DAC", *dac);
+}
+INSTANTIATE_SERIALIZE_METHODS(RomSynthesizer);
+REGISTER_MSXDEVICE(RomSynthesizer, "RomSynthesizer");
+
 } // namespace openmsx
