@@ -1,10 +1,10 @@
-// $Id: DummyMidiInDevice.cc 4836 2005-11-11 20:53:06Z m9710797 $
+// $Id: DummyMidiInDevice.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "DummyMidiInDevice.hh"
 
 namespace openmsx {
 
-void DummyMidiInDevice::signal(const EmuTime& /*time*/)
+void DummyMidiInDevice::signal(EmuTime::param /*time*/)
 {
 	// ignore
 }
@@ -16,11 +16,11 @@ const std::string& DummyMidiInDevice::getDescription() const
 }
 
 void DummyMidiInDevice::plugHelper(Connector& /*connector*/,
-                                   const EmuTime& /*time*/)
+                                   EmuTime::param /*time*/)
 {
 }
 
-void DummyMidiInDevice::unplugHelper(const EmuTime& /*time*/)
+void DummyMidiInDevice::unplugHelper(EmuTime::param /*time*/)
 {
 }
 
