@@ -1,10 +1,10 @@
-// $Id: DummyMidiOutDevice.cc 4836 2005-11-11 20:53:06Z m9710797 $
+// $Id: DummyMidiOutDevice.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "DummyMidiOutDevice.hh"
 
 namespace openmsx {
 
-void DummyMidiOutDevice::recvByte(byte /*value*/, const EmuTime& /*time*/)
+void DummyMidiOutDevice::recvByte(byte /*value*/, EmuTime::param /*time*/)
 {
 	// ignore
 }
@@ -16,11 +16,11 @@ const std::string& DummyMidiOutDevice::getDescription() const
 }
 
 void DummyMidiOutDevice::plugHelper(
-		Connector& /*connector*/, const EmuTime& /*time*/)
+		Connector& /*connector*/, EmuTime::param /*time*/)
 {
 }
 
-void DummyMidiOutDevice::unplugHelper(const EmuTime& /*time*/)
+void DummyMidiOutDevice::unplugHelper(EmuTime::param /*time*/)
 {
 }
 
