@@ -1,6 +1,7 @@
-// $Id: I8251.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: I8251.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "I8251.hh"
+#include "serialize.hh"
 #include <cassert>
 
 using std::string;
@@ -50,21 +51,35 @@ enum SyncPointType {
 };
 
 
-I8251::I8251(Scheduler& scheduler, I8251Interface* interf_, const EmuTime& time)
-	: Schedulable(scheduler), interf(interf_), clock(scheduler), recvBuf(0)
+I8251::I8251(Scheduler& scheduler, I8251Interface& interf_, EmuTime::param time)
+	: Schedulable(scheduler), interf(interf_), clock(scheduler)
 {
 	reset(time);
 }
 
-void I8251::reset(const EmuTime& time)
+void I8251::reset(EmuTime::param time)
 {
+	// initialize these to avoid UMR on savestate
+	//   TODO investigate correct initial state after reset
+	charLength = 0;
+	recvDataBits  = SerialDataInterface::DATA_8;
+	recvStopBits  = SerialDataInterface::STOP_1;
+	recvParityBit = SerialDataInterface::EVEN;
+	recvParityEnabled = false;
+	recvBuf = 0;
+	recvReady = false;
+	sendByte = 0;
+	sendBuffer = 0;
+	mode = 0;
+	sync1 = sync2 = 0;
+
 	status = STAT_TXRDY | STAT_TXEMPTY;
 	command = 0xFF; // make sure all bits change
 	writeCommand(0, time);
 	cmdFaze = FAZE_MODE;
 }
 
-byte I8251::readIO(word port, const EmuTime& time)
+byte I8251::readIO(word port, EmuTime::param time)
 {
 	byte result;
 	switch (port & 1) {
@@ -82,7 +97,7 @@ byte I8251::readIO(word port, const EmuTime& time)
 	return result;
 }
 
-byte I8251::peekIO(word port, const EmuTime& /*time*/) const
+byte I8251::peekIO(word port, EmuTime::param /*time*/) const
 {
 	byte result;
 	switch (port & 1) {
@@ -100,7 +115,7 @@ byte I8251::peekIO(word port, const EmuTime& /*time*/) const
 }
 
 
-void I8251::writeIO(word port, byte value, const EmuTime& time)
+void I8251::writeIO(word port, byte value, EmuTime::param time)
 {
 	//PRT_DEBUG("I8251: write " << (int)port << " " << (int)value);
 	switch (port & 1) {
@@ -167,7 +182,7 @@ void I8251::setMode(byte value)
 		assert(false);
 		dataBits = SerialDataInterface::DATA_8;
 	}
-	interf->setDataBits(dataBits);
+	interf.setDataBits(dataBits);
 
 	SerialDataInterface::StopBits stopBits;
 	switch(mode & MODE_STOP_BITS) {
@@ -187,14 +202,14 @@ void I8251::setMode(byte value)
 		assert(false);
 		stopBits = SerialDataInterface::STOP_2;
 	}
-	interf->setStopBits(stopBits);
+	interf.setStopBits(stopBits);
 
 	bool parityEnable = mode & MODE_PARITYEN;
 	SerialDataInterface::ParityBit parity = (mode & MODE_PARITEVEN) ?
 		SerialDataInterface::EVEN : SerialDataInterface::ODD;
-	interf->setParityBit(parityEnable, parity);
+	interf.setParityBit(parityEnable, parity);
 
-	int baudrate;
+	unsigned baudrate;
 	switch (mode & MODE_BAUDRATE) {
 	case MODE_SYNCHRONOUS:
 		baudrate = 1;
@@ -213,19 +228,19 @@ void I8251::setMode(byte value)
 		baudrate = 1;
 	}
 
-	charLength = (((2 * (1 + int(dataBits) + (parityEnable ? 1 : 0))) +
-	               int(stopBits)) * baudrate) / 2;
+	charLength = (((2 * (1 + unsigned(dataBits) + (parityEnable ? 1 : 0))) +
+	               unsigned(stopBits)) * baudrate) / 2;
 }
 
-void I8251::writeCommand(byte value, const EmuTime& time)
+void I8251::writeCommand(byte value, EmuTime::param time)
 {
 	byte oldCommand = command;
 	command = value;
 
 	// CMD_RESET, CMD_TXEN, CMD_RXE  handled in other routines
 
-	interf->setRTS(command & CMD_RTS, time);
-	interf->setDTR(command & CMD_DTR, time);
+	interf.setRTS(command & CMD_RTS, time);
+	interf.setDTR(command & CMD_DTR, time);
 
 	if (!(command & CMD_TXEN)) {
 		// disable transmitter
@@ -253,27 +268,27 @@ void I8251::writeCommand(byte value, const EmuTime& time)
 			status &= ~(STAT_PE | STAT_OE | STAT_FE); // TODO
 			status &= ~STAT_RXRDY;
 		}
-		interf->signal(time);
+		interf.signal(time);
 	}
 }
 
-byte I8251::readStatus(const EmuTime& time)
+byte I8251::readStatus(EmuTime::param time)
 {
 	byte result = status;
-	if (interf->getDSR(time)) {
+	if (interf.getDSR(time)) {
 		result |= STAT_DSR;
 	}
 	return result;
 }
 
-byte I8251::readTrans(const EmuTime& time)
+byte I8251::readTrans(EmuTime::param time)
 {
 	status &= ~STAT_RXRDY;
-	interf->setRxRDY(false, time);
+	interf.setRxRDY(false, time);
 	return recvBuf;
 }
 
-void I8251::writeTrans(byte value, const EmuTime& time)
+void I8251::writeTrans(byte value, EmuTime::param time)
 {
 	if (!(command & CMD_TXEN)) {
 		return;
@@ -309,7 +324,7 @@ void I8251::setParityBit(bool enable, ParityBit parity)
 	recvParityBit = parity;
 }
 
-void I8251::recvByte(byte value, const EmuTime& time)
+void I8251::recvByte(byte value, EmuTime::param time)
 {
 	// TODO STAT_PE / STAT_FE / STAT_SYNBRK
 	assert(recvReady && (command & CMD_RXE));
@@ -318,7 +333,7 @@ void I8251::recvByte(byte value, const EmuTime& time)
 	} else {
 		recvBuf = value;
 		status |= STAT_RXRDY;
-		interf->setRxRDY(true, time);
+		interf.setRxRDY(true, time);
 	}
 	recvReady = false;
 	if (clock.isPeriodic()) {
@@ -336,7 +351,7 @@ bool I8251::isRecvEnabled()
 	return command & CMD_RXE;
 }
 
-void I8251::send(byte value, const EmuTime& time)
+void I8251::send(byte value, EmuTime::param time)
 {
 	status &= ~STAT_TXEMPTY;
 	sendByte = value;
@@ -346,18 +361,18 @@ void I8251::send(byte value, const EmuTime& time)
 	}
 }
 
-void I8251::executeUntil(const EmuTime& time, int userData)
+void I8251::executeUntil(EmuTime::param time, int userData)
 {
 	switch (userData) {
 	case RECV:
 		assert(command & CMD_RXE);
 		recvReady = true;
-		interf->signal(time);
+		interf.signal(time);
 		break;
 	case TRANS:
 		assert(!(status & STAT_TXEMPTY) && (command & CMD_TXEN));
 
-		interf->recvByte(sendByte, time);
+		interf.recvByte(sendByte, time);
 		if (status & STAT_TXRDY) {
 			status |= STAT_TXEMPTY;
 		} else {
@@ -376,4 +391,58 @@ const string& I8251::schedName() const
 	return I8251_NAME;
 }
 
+
+static enum_string<SerialDataInterface::DataBits> dataBitsInfo[] = {
+		{ "5", SerialDataInterface::DATA_5 },
+		{ "6", SerialDataInterface::DATA_6 },
+		{ "7", SerialDataInterface::DATA_7 },
+		{ "8", SerialDataInterface::DATA_8 }
+};
+SERIALIZE_ENUM(SerialDataInterface::DataBits, dataBitsInfo);
+
+static enum_string<SerialDataInterface::StopBits> stopBitsInfo[] = {
+	{ "INVALID", SerialDataInterface::STOP_INV },
+	{ "1",       SerialDataInterface::STOP_1   },
+	{ "1.5",     SerialDataInterface::STOP_15  },
+	{ "2",       SerialDataInterface::STOP_2   }
+};
+SERIALIZE_ENUM(SerialDataInterface::StopBits, stopBitsInfo);
+
+static enum_string<SerialDataInterface::ParityBit> parityBitInfo[] = {
+	{ "EVEN", SerialDataInterface::EVEN },
+	{ "ODD",  SerialDataInterface::ODD  }
+};
+SERIALIZE_ENUM(SerialDataInterface::ParityBit, parityBitInfo);
+
+static enum_string<I8251::CmdFaze> cmdFazeInfo[] = {
+	{ "MODE",  I8251::FAZE_MODE  },
+	{ "SYNC1", I8251::FAZE_SYNC1 },
+	{ "SYNC2", I8251::FAZE_SYNC2 },
+	{ "CMD",   I8251::FAZE_CMD   }
+};
+SERIALIZE_ENUM(I8251::CmdFaze, cmdFazeInfo);
+
+template<typename Archive>
+void I8251::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<Schedulable>(*this);
+	ar.serialize("clock", clock);
+	ar.serialize("charLength", charLength);
+	ar.serialize("recvDataBits", recvDataBits);
+	ar.serialize("recvStopBits", recvStopBits);
+	ar.serialize("recvParityBit", recvParityBit);
+	ar.serialize("recvParityEnabled", recvParityEnabled);
+	ar.serialize("recvBuf", recvBuf);
+	ar.serialize("recvReady", recvReady);
+	ar.serialize("sendByte", sendByte);
+	ar.serialize("sendBuffer", sendBuffer);
+	ar.serialize("status", status);
+	ar.serialize("command", command);
+	ar.serialize("mode", mode);
+	ar.serialize("sync1", sync1);
+	ar.serialize("sync2", sync2);
+	ar.serialize("cmdFaze", cmdFaze);
+}
+INSTANTIATE_SERIALIZE_METHODS(I8251);
+
 } // namespace openmsx
