@@ -1,11 +1,14 @@
-// $Id: MSXRS232.cc 6366 2007-04-24 19:05:58Z m9710797 $
+// $Id: MSXRS232.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "MSXRS232.hh"
+#include "RS232Device.hh"
 #include "MSXMotherBoard.hh"
 #include "I8254.hh"
+#include "I8251.hh"
 #include "Ram.hh"
 #include "Rom.hh"
 #include "XMLElement.hh"
+#include "serialize.hh"
 #include <cassert>
 
 namespace openmsx {
@@ -13,26 +16,71 @@ namespace openmsx {
 const unsigned RAM_OFFSET = 0x2000;
 const unsigned RAM_SIZE = 0x800;
 
-MSXRS232::MSXRS232(MSXMotherBoard& motherBoard, const XMLElement& config,
-                   const EmuTime& time)
-	: MSXDevice(motherBoard, config, time)
+
+class Counter0 : public ClockPinListener
+{
+public:
+	explicit Counter0(MSXRS232& rs232);
+	virtual ~Counter0();
+	virtual void signal(ClockPin& pin, EmuTime::param time);
+	virtual void signalPosEdge(ClockPin& pin, EmuTime::param time);
+private:
+	MSXRS232& rs232;
+};
+
+class Counter1 : public ClockPinListener
+{
+public:
+	explicit Counter1(MSXRS232& rs232);
+	virtual ~Counter1();
+	virtual void signal(ClockPin& pin, EmuTime::param time);
+	virtual void signalPosEdge(ClockPin& pin, EmuTime::param time);
+private:
+	MSXRS232& rs232;
+};
+
+class I8251Interf : public I8251Interface
+{
+public:
+	explicit I8251Interf(MSXRS232& rs232);
+	virtual ~I8251Interf();
+	virtual void setRxRDY(bool status, EmuTime::param time);
+	virtual void setDTR(bool status, EmuTime::param time);
+	virtual void setRTS(bool status, EmuTime::param time);
+	virtual bool getDSR(EmuTime::param time);
+	virtual bool getCTS(EmuTime::param time);
+	virtual void setDataBits(DataBits bits);
+	virtual void setStopBits(StopBits bits);
+	virtual void setParityBit(bool enable, ParityBit parity);
+	virtual void recvByte(byte value, EmuTime::param time);
+	virtual void signal(EmuTime::param time);
+private:
+	MSXRS232& rs232;
+};
+
+
+MSXRS232::MSXRS232(MSXMotherBoard& motherBoard, const XMLElement& config)
+	: MSXDevice(motherBoard, config)
 	, RS232Connector(motherBoard.getPluggingController(), "msx-rs232")
-	, cntr0(*this), cntr1(*this)
-	, i8254(new I8254(motherBoard.getScheduler(), &cntr0, &cntr1, NULL, time))
-	, interf(*this)
-	, i8251(new I8251(motherBoard.getScheduler(), &interf, time))
+	, cntr0(new Counter0(*this))
+	, cntr1(new Counter1(*this))
+	, i8254(new I8254(motherBoard.getScheduler(),
+	                  cntr0.get(), cntr1.get(), NULL, getCurrentTime()))
+	, interf(new I8251Interf(*this))
+	, i8251(new I8251(motherBoard.getScheduler(), *interf,
+	                  getCurrentTime()))
 	, rom(new Rom(motherBoard, MSXDevice::getName() + " ROM", "rom", config))
-	, rxrdyIRQ(getMotherBoard().getCPU())
+	, ram(config.getChildDataAsBool("ram", false)
+	      ? new Ram(motherBoard, MSXDevice::getName() + " RAM",
+	                "RS232 RAM", RAM_SIZE)
+	      : NULL)
+	, rxrdyIRQ(getMotherBoard(), MSXDevice::getName() + ".IRQrxrdy")
 	, rxrdyIRQlatch(false)
 	, rxrdyIRQenabled(false)
 {
-	if (config.getChildDataAsBool("ram", false)) {
-		ram.reset(new Ram(motherBoard, MSXDevice::getName() + " RAM",
-		                  "RS232 RAM", RAM_SIZE));
-	}
-
 	EmuDuration total(1.0 / 1.8432e6); // 1.8432MHz
 	EmuDuration hi   (1.0 / 3.6864e6); //   half clock period
+	EmuTime::param time = getCurrentTime();
 	i8254->getClockPin(0).setPeriodicState(total, hi, time);
 	i8254->getClockPin(1).setPeriodicState(total, hi, time);
 	i8254->getClockPin(2).setPeriodicState(total, hi, time);
@@ -44,7 +92,7 @@ MSXRS232::~MSXRS232()
 {
 }
 
-void MSXRS232::reset(const EmuTime& /*time*/)
+void MSXRS232::reset(EmuTime::param /*time*/)
 {
 	rxrdyIRQlatch = false;
 	rxrdyIRQenabled = false;
@@ -55,7 +103,7 @@ void MSXRS232::reset(const EmuTime& /*time*/)
 	}
 }
 
-byte MSXRS232::readMem(word address, const EmuTime& /*time*/)
+byte MSXRS232::readMem(word address, EmuTime::param /*time*/)
 {
 	word addr = address & 0x3FFF;
 	if (ram.get() && ((RAM_OFFSET <= addr) && (addr < (RAM_OFFSET + RAM_SIZE)))) {
@@ -79,7 +127,7 @@ const byte* MSXRS232::getReadCacheLine(word start) const
 	}
 }
 
-void MSXRS232::writeMem(word address, byte value, const EmuTime& /*time*/)
+void MSXRS232::writeMem(word address, byte value, EmuTime::param /*time*/)
 {
 	word addr = address & 0x3FFF;
 	if (ram.get() && ((RAM_OFFSET <= addr) && (addr < (RAM_OFFSET + RAM_SIZE)))) {
@@ -98,7 +146,7 @@ byte* MSXRS232::getWriteCacheLine(word start) const
 	}
 }
 
-byte MSXRS232::readIO(word port, const EmuTime& time)
+byte MSXRS232::readIO(word port, EmuTime::param time)
 {
 	byte result;
 	port &= 0x07;
@@ -121,13 +169,13 @@ byte MSXRS232::readIO(word port, const EmuTime& time)
 			break;
 		default:
 			assert(false);
-			result = 0xFF;	// avoid warning
+			result = 0xFF; // avoid warning
 	}
 	//PRT_DEBUG("MSXRS232 read " << (int)port << " " << (int)result);
 	return result;
 }
 
-byte MSXRS232::peekIO(word port, const EmuTime& time) const
+byte MSXRS232::peekIO(word port, EmuTime::param time) const
 {
 	byte result;
 	port &= 0x07;
@@ -150,12 +198,12 @@ byte MSXRS232::peekIO(word port, const EmuTime& time) const
 			break;
 		default:
 			assert(false);
-			result = 0xFF;	// avoid warning
+			result = 0xFF; // avoid warning
 	}
 	return result;
 }
 
-void MSXRS232::writeIO(word port, byte value, const EmuTime& time)
+void MSXRS232::writeIO(word port, byte value, EmuTime::param time)
 {
 	//PRT_DEBUG("MSXRS232 write " << (int)port << " " << (int)value);
 	port &= 0x07;
@@ -178,10 +226,10 @@ void MSXRS232::writeIO(word port, byte value, const EmuTime& time)
 	}
 }
 
-byte MSXRS232::readStatus(const EmuTime& time)
+byte MSXRS232::readStatus(EmuTime::param time)
 {
-	byte result = 0;	// TODO check unused bits
-	if (!interf.getCTS(time)) {
+	byte result = 0; // TODO check unused bits
+	if (!interf->getCTS(time)) {
 		result |= 0x80;
 	}
 	if (i8254->getOutputPin(2).getState(time)) {
@@ -222,78 +270,78 @@ void MSXRS232::enableRxRDYIRQ(bool enabled)
 
 // I8251Interface  (pass calls from I8251 to outConnector)
 
-MSXRS232::I8251Interf::I8251Interf(MSXRS232& rs232_)
+I8251Interf::I8251Interf(MSXRS232& rs232_)
 	: rs232(rs232_)
 {
 }
 
-MSXRS232::I8251Interf::~I8251Interf()
+I8251Interf::~I8251Interf()
 {
 }
 
-void MSXRS232::I8251Interf::setRxRDY(bool status, const EmuTime& /*time*/)
+void I8251Interf::setRxRDY(bool status, EmuTime::param /*time*/)
 {
 	rs232.setRxRDYIRQ(status);
 }
 
-void MSXRS232::I8251Interf::setDTR(bool status, const EmuTime& time)
+void I8251Interf::setDTR(bool status, EmuTime::param time)
 {
-	rs232.getPlugged().setDTR(status, time);
+	rs232.getPluggedRS232Dev().setDTR(status, time);
 }
 
-void MSXRS232::I8251Interf::setRTS(bool status, const EmuTime& time)
+void I8251Interf::setRTS(bool status, EmuTime::param time)
 {
-	rs232.getPlugged().setRTS(status, time);
+	rs232.getPluggedRS232Dev().setRTS(status, time);
 }
 
-bool MSXRS232::I8251Interf::getDSR(const EmuTime& time)
+bool I8251Interf::getDSR(EmuTime::param time)
 {
-	return rs232.getPlugged().getDSR(time);
+	return rs232.getPluggedRS232Dev().getDSR(time);
 }
 
-bool MSXRS232::I8251Interf::getCTS(const EmuTime& time)
+bool I8251Interf::getCTS(EmuTime::param time)
 {
-	return rs232.getPlugged().getCTS(time);
+	return rs232.getPluggedRS232Dev().getCTS(time);
 }
 
-void MSXRS232::I8251Interf::setDataBits(DataBits bits)
+void I8251Interf::setDataBits(DataBits bits)
 {
-	rs232.getPlugged().setDataBits(bits);
+	rs232.getPluggedRS232Dev().setDataBits(bits);
 }
 
-void MSXRS232::I8251Interf::setStopBits(StopBits bits)
+void I8251Interf::setStopBits(StopBits bits)
 {
-	rs232.getPlugged().setStopBits(bits);
+	rs232.getPluggedRS232Dev().setStopBits(bits);
 }
 
-void MSXRS232::I8251Interf::setParityBit(bool enable, ParityBit parity)
+void I8251Interf::setParityBit(bool enable, ParityBit parity)
 {
-	rs232.getPlugged().setParityBit(enable, parity);
+	rs232.getPluggedRS232Dev().setParityBit(enable, parity);
 }
 
-void MSXRS232::I8251Interf::recvByte(byte value, const EmuTime& time)
+void I8251Interf::recvByte(byte value, EmuTime::param time)
 {
-	rs232.getPlugged().recvByte(value, time);
+	rs232.getPluggedRS232Dev().recvByte(value, time);
 }
 
-void MSXRS232::I8251Interf::signal(const EmuTime& time)
+void I8251Interf::signal(EmuTime::param time)
 {
-	rs232.getPlugged().signal(time); // for input
+	rs232.getPluggedRS232Dev().signal(time); // for input
 }
 
 
 // Counter 0 output
 
-MSXRS232::Counter0::Counter0(MSXRS232& rs232_)
+Counter0::Counter0(MSXRS232& rs232_)
 	: rs232(rs232_)
 {
 }
 
-MSXRS232::Counter0::~Counter0()
+Counter0::~Counter0()
 {
 }
 
-void MSXRS232::Counter0::signal(ClockPin& pin, const EmuTime& time)
+void Counter0::signal(ClockPin& pin, EmuTime::param time)
 {
 	ClockPin& clk = rs232.i8251->getClockPin();
 	if (pin.isPeriodic()) {
@@ -304,7 +352,7 @@ void MSXRS232::Counter0::signal(ClockPin& pin, const EmuTime& time)
 	}
 }
 
-void MSXRS232::Counter0::signalPosEdge(ClockPin& /*pin*/, const EmuTime& /*time*/)
+void Counter0::signalPosEdge(ClockPin& /*pin*/, EmuTime::param /*time*/)
 {
 	assert(false);
 }
@@ -312,16 +360,16 @@ void MSXRS232::Counter0::signalPosEdge(ClockPin& /*pin*/, const EmuTime& /*time*
 
 // Counter 1 output // TODO split rx tx
 
-MSXRS232::Counter1::Counter1(MSXRS232& rs232_)
+Counter1::Counter1(MSXRS232& rs232_)
 	: rs232(rs232_)
 {
 }
 
-MSXRS232::Counter1::~Counter1()
+Counter1::~Counter1()
 {
 }
 
-void MSXRS232::Counter1::signal(ClockPin& pin, const EmuTime& time)
+void Counter1::signal(ClockPin& pin, EmuTime::param time)
 {
 	ClockPin& clk = rs232.i8251->getClockPin();
 	if (pin.isPeriodic()) {
@@ -332,8 +380,7 @@ void MSXRS232::Counter1::signal(ClockPin& pin, const EmuTime& time)
 	}
 }
 
-void MSXRS232::Counter1::signalPosEdge(ClockPin& /*pin*/,
-                                       const EmuTime& /*time*/)
+void Counter1::signalPosEdge(ClockPin& /*pin*/, EmuTime::param /*time*/)
 {
 	assert(false);
 }
@@ -366,10 +413,31 @@ void MSXRS232::setParityBit(bool enable, ParityBit parity)
 	i8251->setParityBit(enable, parity);
 }
 
-void MSXRS232::recvByte(byte value, const EmuTime& time)
+void MSXRS232::recvByte(byte value, EmuTime::param time)
 {
 	i8251->recvByte(value, time);
 }
 
+
+template<typename Archive>
+void MSXRS232::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<MSXDevice>(*this);
+	ar.template serializeBase<RS232Connector>(*this);
+
+	ar.serialize("I8254", *i8254);
+	ar.serialize("I8251", *i8251);
+	if (ram.get()) {
+		ar.serialize("ram", *ram);
+	}
+	ar.serialize("rxrdyIRQ", rxrdyIRQ);
+	ar.serialize("rxrdyIRQlatch", rxrdyIRQlatch);
+	ar.serialize("rxrdyIRQenabled", rxrdyIRQenabled);
+
+	// don't serialize cntr0, cntr1, interf
+}
+INSTANTIATE_SERIALIZE_METHODS(MSXRS232);
+REGISTER_MSXDEVICE(MSXRS232, "RS232");
+
 } // namespace openmsx
 
