@@ -1,7 +1,9 @@
-// $Id: MidiOutLogger.cc 4836 2005-11-11 20:53:06Z m9710797 $
+// $Id: MidiOutLogger.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "MidiOutLogger.hh"
+#include "PlugException.hh"
 #include "FilenameSetting.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
@@ -14,7 +16,7 @@ MidiOutLogger::MidiOutLogger(CommandController& commandController)
 }
 
 void MidiOutLogger::plugHelper(Connector& /*connector*/,
-                               const EmuTime& /*time*/)
+                               EmuTime::param /*time*/)
 {
 	file.open(logFilenameSetting->getValue().c_str());
 	if (file.fail()) {
@@ -23,7 +25,7 @@ void MidiOutLogger::plugHelper(Connector& /*connector*/,
 	}
 }
 
-void MidiOutLogger::unplugHelper(const EmuTime& /*time*/)
+void MidiOutLogger::unplugHelper(EmuTime::param /*time*/)
 {
 	file.close();
 }
@@ -43,10 +45,20 @@ const std::string& MidiOutLogger::getDescription() const
 	return desc;
 }
 
-void MidiOutLogger::recvByte(byte value, const EmuTime& /*time*/)
+void MidiOutLogger::recvByte(byte value, EmuTime::param /*time*/)
 {
-	file.put(value);
-	file.flush();
+	if (file.is_open()) {
+		file.put(value);
+		file.flush();
+	}
 }
 
+template<typename Archive>
+void MidiOutLogger::serialize(Archive& /*ar*/, unsigned /*version*/)
+{
+	// don't try to resume a previous logfile (see PrinterPortLogger)
+}
+INSTANTIATE_SERIALIZE_METHODS(MidiOutLogger);
+REGISTER_POLYMORPHIC_INITIALIZER(Pluggable, MidiOutLogger, "MidiOutLogger");
+
 } // namespace openmsx
