@@ -1,10 +1,12 @@
-// $Id: RS232Tester.cc 5731 2006-10-01 19:54:47Z m9710797 $
+// $Id: RS232Tester.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "RS232Tester.hh"
 #include "RS232Connector.hh"
+#include "PlugException.hh"
 #include "EventDistributor.hh"
 #include "Scheduler.hh"
 #include "FilenameSetting.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
@@ -12,7 +14,7 @@ RS232Tester::RS232Tester(EventDistributor& eventDistributor_,
                          Scheduler& scheduler_,
                          CommandController& commandController)
 	: eventDistributor(eventDistributor_), scheduler(scheduler_)
-	, thread(this), lock(1)
+	, thread(this), inFile(NULL), lock(1)
 	, rs232InputFilenameSetting(new FilenameSetting(
 	        commandController, "rs232-inputfilename",
 	        "filename of the file where the RS232 input is read from",
@@ -31,7 +33,7 @@ RS232Tester::~RS232Tester()
 }
 
 // Pluggable
-void RS232Tester::plugHelper(Connector& connector_, const EmuTime& /*time*/)
+void RS232Tester::plugHelper(Connector& connector_, EmuTime::param /*time*/)
 {
 	// output
 	std::string outName = rs232OutputFilenameSetting->getValue();
@@ -54,12 +56,12 @@ void RS232Tester::plugHelper(Connector& connector_, const EmuTime& /*time*/)
 	rs232Connector.setStopBits(SerialDataInterface::STOP_1);	// 1 stop bit
 	rs232Connector.setParityBit(false, SerialDataInterface::EVEN); // no parity
 
-	connector = &connector_; // base class will do this in a moment,
-	                        // but thread already needs it
+	setConnector(&connector_); // base class will do this in a moment,
+	                           // but thread already needs it
 	thread.start();
 }
 
-void RS232Tester::unplugHelper(const EmuTime& /*time*/)
+void RS232Tester::unplugHelper(EmuTime::param /*time*/)
 {
 	// output
 	outFile.close();
@@ -67,7 +69,10 @@ void RS232Tester::unplugHelper(const EmuTime& /*time*/)
 	// input
 	ScopedLock l(lock);
 	thread.stop();
-	fclose(inFile);
+	if (inFile) {
+		fclose(inFile);
+		inFile = NULL;
+	}
 }
 
 const std::string& RS232Tester::getName() const
@@ -90,6 +95,7 @@ const std::string& RS232Tester::getDescription() const
 void RS232Tester::run()
 {
 	byte buf;
+	if (!inFile) return;
 	while (true) {
 		int num = fread(&buf, 1, 1, inFile);
 		if (num != 1) {
@@ -104,7 +110,7 @@ void RS232Tester::run()
 }
 
 // input
-void RS232Tester::signal(const EmuTime& time)
+void RS232Tester::signal(EmuTime::param time)
 {
 	RS232Connector* connector = static_cast<RS232Connector*>(getConnector());
 	if (!connector->acceptsData()) {
@@ -134,10 +140,21 @@ bool RS232Tester::signalEvent(shared_ptr<const Event> /*event*/)
 
 
 // output
-void RS232Tester::recvByte(byte value, const EmuTime& /*time*/)
+void RS232Tester::recvByte(byte value, EmuTime::param /*time*/)
 {
-	outFile.put(value);
-	outFile.flush();
+	if (outFile.is_open()) {
+		outFile.put(value);
+		outFile.flush();
+	}
 }
 
+
+template<typename Archive>
+void RS232Tester::serialize(Archive& /*ar*/, unsigned /*version*/)
+{
+	// don't try to resume a previous logfile (see PrinterPortLogger)
+}
+INSTANTIATE_SERIALIZE_METHODS(RS232Tester);
+REGISTER_POLYMORPHIC_INITIALIZER(Pluggable, RS232Tester, "RS232Tester");
+
 } // namespace openmsx
