@@ -1,4 +1,4 @@
-// $Id: SettingsManager.cc 6593 2007-06-03 23:12:19Z mthuurne $
+// $Id: SettingsManager.cc 8209 2008-09-09 20:48:44Z m9710797 $
 
 #include "SettingsManager.hh"
 #include "GlobalCommandController.hh"
@@ -81,6 +81,12 @@ void SettingsManager::unregisterSetting(Setting& /*setting*/, const string& name
 	settingsMap.erase(name);
 }
 
+Setting* SettingsManager::findSetting(const std::string& name) const
+{
+	SettingsMap::const_iterator it = settingsMap.find(name);
+	return (it != settingsMap.end()) ? it->second : NULL;
+}
+
 // Helper functions for setting commands
 
 void SettingsManager::getSettingNames(set<string>& result) const
@@ -140,7 +146,7 @@ void SettingsManager::loadSettings(const XMLElement& config)
 			"setting", "id", name);
 		if (elem) {
 			try {
-				setting.setValueString(elem->getData());
+				setting.changeValueString(elem->getData());
 			} catch (MSXException& e) {
 				// ignore, keep default value
 			}
@@ -202,12 +208,12 @@ string SettingInfo::help(const vector<string>& /*tokens*/) const
 void SettingInfo::tabCompletion(vector<string>& tokens) const
 {
 	switch (tokens.size()) {
-		case 3: { // complete setting name
-			set<string> settings;
-			manager.getSettingNames(settings);
-			completeString(tokens, settings);
-			break;
-		}
+	case 3: { // complete setting name
+		set<string> settings;
+		manager.getSettingNames(settings);
+		completeString(tokens, settings);
+		break;
+	}
 	}
 }
 
@@ -225,7 +231,7 @@ string SetCompleter::help(const vector<string>& tokens) const
 	if (tokens.size() == 2) {
 		return manager.getByName("set", tokens[1]).getDescription();
 	}
-	return "Set or query the value of a openMSX setting or TCL variable\n"
+	return "Set or query the value of a openMSX setting or Tcl variable\n"
 	       "  set <setting>          shows current value\n"
 	       "  set <setting> <value>  set a new value\n"
 	       "Use 'help set <setting>' to get more info on a specific\n"
@@ -235,22 +241,22 @@ string SetCompleter::help(const vector<string>& tokens) const
 void SetCompleter::tabCompletion(vector<string>& tokens) const
 {
 	switch (tokens.size()) {
-		case 2: {
-			// complete setting name
-			set<string> settings;
-			manager.getSettingNames(settings);
-			completeString(tokens, settings);
-			break;
-		}
-		case 3: {
-			// complete setting value
-			SettingsManager::SettingsMap::iterator it =
-				manager.settingsMap.find(tokens[1]);
-			if (it != manager.settingsMap.end()) {
-				it->second->tabCompletion(tokens);
-			}
-			break;
+	case 2: {
+		// complete setting name
+		set<string> settings;
+		manager.getSettingNames(settings);
+		completeString(tokens, settings, false); // case insensitive
+		break;
+	}
+	case 3: {
+		// complete setting value
+		SettingsManager::SettingsMap::iterator it =
+			manager.settingsMap.find(tokens[1]);
+		if (it != manager.settingsMap.end()) {
+			it->second->tabCompletion(tokens);
 		}
+		break;
+	}
 	}
 }
 
@@ -273,13 +279,13 @@ string SettingCompleter::help(const vector<string>& /*tokens*/) const
 void SettingCompleter::tabCompletion(vector<string>& tokens) const
 {
 	switch (tokens.size()) {
-		case 2: {
-			// complete setting name
-			set<string> settings;
-			manager.getSettingNames(settings);
-			completeString(tokens, settings);
-			break;
-		}
+	case 2: {
+		// complete setting name
+		set<string> settings;
+		manager.getSettingNames(settings);
+		completeString(tokens, settings);
+		break;
+	}
 	}
 }
 
