@@ -1,11 +1,11 @@
-// $Id: DirectXSoundDriver.cc 6798 2007-08-04 15:35:56Z m9710797 $
+// $Id: DirectXSoundDriver.cc 8201 2008-09-04 17:06:27Z m9710797 $
 
 #ifdef _WIN32
 
 #include "DirectXSoundDriver.hh"
 #include "MSXException.hh"
 #include "openmsx.hh"
-#include <string.h>
+#include <cstring>
 #include <SDL.h>       //
 #include <SDL_syswm.h> //
 
