@@ -1,10 +1,9 @@
-// $Id: MSXMixer.cc 7311 2007-11-25 09:27:47Z m9710797 $
+// $Id: MSXMixer.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "MSXMixer.hh"
 #include "Mixer.hh"
 #include "SoundDevice.hh"
 #include "MSXCommandController.hh"
-#include "Scheduler.hh"
 #include "InfoTopic.hh"
 #include "TclObject.hh"
 #include "ThrottleManager.hh"
@@ -12,9 +11,12 @@
 #include "IntegerSetting.hh"
 #include "StringSetting.hh"
 #include "BooleanSetting.hh"
+#include "CommandException.hh"
 #include "AviRecorder.hh"
-#include "Math.hh"
+#include "Filename.hh"
 #include "CliComm.hh"
+#include "Math.hh"
+#include "StringOp.hh"
 #include <algorithm>
 #include <cmath>
 #include <cstring>
@@ -44,15 +46,15 @@ MSXMixer::MSXMixer(Mixer& mixer_, Scheduler& scheduler,
                    MSXCommandController& msxCommandController_)
 	: Schedulable(scheduler)
 	, mixer(mixer_)
-	, msxCommandController(msxCommandController_)
+	, commandController(msxCommandController_)
 	, masterVolume(mixer.getMasterVolume())
 	, speedSetting(
-		msxCommandController.getGlobalSettings().getSpeedSetting())
+		commandController.getGlobalSettings().getSpeedSetting())
 	, throttleManager(
-		msxCommandController.getGlobalSettings().getThrottleManager())
+		commandController.getGlobalSettings().getThrottleManager())
 	, prevTime(EmuTime::zero)
 	, soundDeviceInfo(new SoundDeviceInfoTopic(
-	              msxCommandController.getMachineInfoCommand(), *this))
+	              msxCommandController_.getMachineInfoCommand(), *this))
 	, recorder(0)
 	, synchronousCounter(0)
 {
@@ -90,9 +92,9 @@ void MSXMixer::registerSound(SoundDevice& device, double volume,
 	const string& name = device.getName();
 	SoundDeviceInfo info;
 	info.defaultVolume = volume;
-	info.volumeSetting = new IntegerSetting(msxCommandController,
+	info.volumeSetting = new IntegerSetting(commandController,
 		name + "_volume", "the volume of this sound chip", 75, 0, 100);
-	info.balanceSetting = new IntegerSetting(msxCommandController,
+	info.balanceSetting = new IntegerSetting(commandController,
 		name + "_balance", "the balance of this sound chip",
 		balance, -100, 100);
 
@@ -104,13 +106,13 @@ void MSXMixer::registerSound(SoundDevice& device, double volume,
 		string ch_name = name + "_ch" + StringOp::toString(i + 1);
 
 		channelSettings.recordSetting = new StringSetting(
-			msxCommandController, ch_name + "_record",
+			commandController, ch_name + "_record",
 			"filename to record this channel to",
 			"", Setting::DONT_SAVE);
 		channelSettings.recordSetting->attach(*this);
 
 		channelSettings.muteSetting = new BooleanSetting(
-			msxCommandController, ch_name + "_mute",
+			commandController, ch_name + "_mute",
 			"sets mute-status of individual sound channels",
 			false, Setting::DONT_SAVE);
 		channelSettings.muteSetting->attach(*this);
@@ -125,7 +127,7 @@ void MSXMixer::registerSound(SoundDevice& device, double volume,
 	assert(it != infos.end());
 	updateVolumeParams(it);
 
-	msxCommandController.getCliComm().update(CliComm::SOUNDDEVICE, device.getName(), "add");
+	commandController.getCliComm().update(CliComm::SOUNDDEVICE, device.getName(), "add");
 }
 
 void MSXMixer::unregisterSound(SoundDevice& device)
@@ -145,7 +147,7 @@ void MSXMixer::unregisterSound(SoundDevice& device)
 		delete it2->muteSetting;
 	}
 	infos.erase(it);
-	msxCommandController.getCliComm().update(CliComm::SOUNDDEVICE, device.getName(), "remove");
+	commandController.getCliComm().update(CliComm::SOUNDDEVICE, device.getName(), "remove");
 }
 
 void MSXMixer::setSynchronousMode(bool synchronous)
@@ -158,14 +160,14 @@ void MSXMixer::setSynchronousMode(bool synchronous)
 	}
 }
 
-void MSXMixer::updateStream(const EmuTime& time)
+void MSXMixer::updateStream(EmuTime::param time)
 {
 	if ((!muteCount && fragmentSize) || synchronousCounter) {
 		updateStream2(time);
 	}
 }
 
-void MSXMixer::updateStream2(const EmuTime& time)
+void MSXMixer::updateStream2(EmuTime::param time)
 {
 	assert(prevTime <= time);
 	EmuDuration duration = time - prevTime;
@@ -198,14 +200,27 @@ void MSXMixer::updateStream2(const EmuTime& time)
 }
 
 void MSXMixer::generate(short* output, unsigned samples,
-	const EmuTime& start, const EmuDuration& sampDur)
+	EmuTime::param start, EmuDuration::param sampDur)
 {
+	// The code below is specialized for a lot of cases (before this
+	// routine was _much_ shorter). This is done because this routine
+	// ends up relatively high (top 5) in a profile run.
+	// After these specialization this routine runs about two times
+	// faster for the common cases (mono output or no sound at all).
+	// In total emulation time this gave a speedup of about 2%.
+
 	assert((!muteCount && fragmentSize) || synchronousCounter);
 	assert(samples <= 8192);
 
-	int mixBuf[2 * samples];
-	int tmpBuf[2 * samples];
-	memset(mixBuf, 0, 2 * samples * sizeof(int));
+	if (samples == 0) return;
+
+	int stereoBuf[2 * samples + 3];
+	int monoBuf[samples + 3];
+	int tmpBuf[2 * samples + 3] __attribute__((aligned(16)));
+
+	static const unsigned HAS_MONO_FLAG = 1;
+	static const unsigned HAS_STEREO_FLAG = 2;
+	unsigned usedBuffers = 0;
 
 	for (Infos::const_iterator it = infos.begin();
 	     it != infos.end(); ++it) {
@@ -215,42 +230,248 @@ void MSXMixer::generate(short* output, unsigned samples,
 			if (!device.isStereo()) {
 				int l1 = it->second.left1;
 				int r1 = it->second.right1;
-				for (unsigned i = 0; i < samples; ++i) {
-					mixBuf[2 * i + 0] += (l1 * tmpBuf[i]) >> 8;
-					mixBuf[2 * i + 1] += (r1 * tmpBuf[i]) >> 8;
+				if (l1 == r1) {
+					if (!(usedBuffers & HAS_MONO_FLAG)) {
+						usedBuffers |= HAS_MONO_FLAG;
+#ifdef __arm__
+						asm volatile (
+						"0:\n\t"
+							"ldmia	%[in]!,{r4-r7}\n\t"
+							"mul	r4,%[f],r4\n\t"
+							"mul	r5,%[f],r5\n\t"
+							"mul	r6,%[f],r6\n\t"
+							"mul	r7,%[f],r7\n\t"
+							"stmia	%[out]!,{r4-r7}\n\t"
+							"subs	%[n],%[n],#4\n\t"
+							"bgt	0b\n\t"
+							: // no output
+							: [in]  "r" (tmpBuf)
+							, [out] "r" (monoBuf)
+							, [f]   "r" (l1)
+							, [n]   "r" (samples)
+							: "r4","r5","r6","r7"
+						);
+#else
+						for (unsigned i = 0; i < samples; ++i) {
+							int tmp = l1 * tmpBuf[i];
+							monoBuf[i] = tmp;
+						}
+#endif
+					} else {
+#ifdef __arm__
+						asm volatile (
+						"0:\n\t"
+							"ldmia	%[in]!,{r3-r6}\n\t"
+							"ldmia	%[out],{r7-r10}\n\t"
+							"mla	r3,%[f],r3,r7\n\t"
+							"mla	r4,%[f],r4,r8\n\t"
+							"mla	r5,%[f],r5,r9\n\t"
+							"mla	r6,%[f],r6,r10\n\t"
+							"stmia	%[out]!,{r3-r6}\n\t"
+							"subs	%[n],%[n],#4\n\t"
+							"bgt	0b\n\t"
+							: // no output
+							: [in]  "r" (tmpBuf)
+							, [out] "r" (monoBuf)
+							, [f]   "r" (l1)
+							, [n]   "r" (samples)
+							: "r3","r4","r5","r6"
+							, "r7","r8","r9","r10"
+						);
+#else
+						for (unsigned i = 0; i < samples; ++i) {
+							int tmp = l1 * tmpBuf[i];
+							monoBuf[i] += tmp;
+						}
+#endif
+					}
+				} else {
+					if (!(usedBuffers & HAS_STEREO_FLAG)) {
+						usedBuffers |= HAS_STEREO_FLAG;
+						for (unsigned i = 0; i < samples; ++i) {
+							int l = l1 * tmpBuf[i];
+							int r = r1 * tmpBuf[i];
+							stereoBuf[2 * i + 0] = l;
+							stereoBuf[2 * i + 1] = r;
+						}
+					} else {
+						for (unsigned i = 0; i < samples; ++i) {
+							int l = l1 * tmpBuf[i];
+							int r = r1 * tmpBuf[i];
+							stereoBuf[2 * i + 0] += l;
+							stereoBuf[2 * i + 1] += r;
+						}
+					}
 				}
 			} else {
 				int l1 = it->second.left1;
 				int r1 = it->second.right1;
 				int l2 = it->second.left2;
 				int r2 = it->second.right2;
-				for (unsigned i = 0; i < samples; ++i) {
-					int in1 = tmpBuf[2 * i + 0];
-					int in2 = tmpBuf[2 * i + 1];
-					mixBuf[2 * i + 0] += (l1 * in1 + l2 * in2) >> 8;
-					mixBuf[2 * i + 1] += (r1 * in1 + r2 * in2) >> 8;
+				if (!(usedBuffers & HAS_STEREO_FLAG)) {
+					usedBuffers |= HAS_STEREO_FLAG;
+					for (unsigned i = 0; i < samples; ++i) {
+						int in1 = tmpBuf[2 * i + 0];
+						int in2 = tmpBuf[2 * i + 1];
+						int l = l1 * in1 + l2 * in2;
+						int r = r1 * in1 + r2 * in2;
+						stereoBuf[2 * i + 0] = l;
+						stereoBuf[2 * i + 1] = r;
+					}
+				} else {
+					for (unsigned i = 0; i < samples; ++i) {
+						int in1 = tmpBuf[2 * i + 0];
+						int in2 = tmpBuf[2 * i + 1];
+						int l = l1 * in1 + l2 * in2;
+						int r = r1 * in1 + r2 * in2;
+						stereoBuf[2 * i + 0] += l;
+						stereoBuf[2 * i + 1] += r;
+					}
 				}
 			}
 		}
 	}
 
-	for (unsigned j = 0; j < samples; ++j) {
-		int left  = mixBuf[2 * j + 0];
-		int right = mixBuf[2 * j + 1];
+	// DC removal filter
+	//   y(n) = x(n) - x(n-1) + R * y(n-1)
+	//   R = 1 - (pi*2 * cut-off-frequency / samplerate)
+	// take R = 511/512
+	//   44100Hz --> cutt-off freq = 14Hz
+	//   22050Hz                     7Hz
+	// Note: we divide by 512 iso shift-right by 9 because we want
+	//       to round towards zero.
+	switch (usedBuffers) {
+	case 0:
+		// no new input
+		if ((outLeft == outRight) && (prevLeft == prevRight)) {
+			if ((outLeft == 0) && (prevLeft == 0)) {
+				// output was already zero, after DC-filter
+				// it will still be zero
+				memset(output, 0, 2 * samples * sizeof(short));
+			} else {
+				// output was not zero, but it was the same
+				// left and right
+				assert(samples > 0);
+				outLeft  = -prevLeft + ((511 * outLeft) / 512);
+				prevLeft = 0;
+				short out = Math::clipIntToShort(outLeft);
+				output[0] = out;
+				output[1] = out;
+				for (unsigned j = 1; j < samples; ++j) {
+					outLeft = ((511 * outLeft) / 512);
+					out = Math::clipIntToShort(outLeft);
+					output[2 * j + 0] = out;
+					output[2 * j + 1] = out;
+				}
+			}
+			outRight = outLeft;
+			prevRight = prevLeft;
+		} else {
+			assert(samples > 0);
+			outLeft   = -prevLeft  + ((511 * outLeft ) / 512);
+			outRight  = -prevRight + ((511 * outRight) / 512);
+			prevLeft  = 0;
+			prevRight = 0;
+			output[0] = Math::clipIntToShort(outLeft);
+			output[1] = Math::clipIntToShort(outRight);
+			for (unsigned j = 1; j < samples; ++j) {
+				outLeft   = ((511 *  outLeft) / 512);
+				outRight  = ((511 * outRight) / 512);
+				output[2 * j + 0] = Math::clipIntToShort(outLeft);
+				output[2 * j + 1] = Math::clipIntToShort(outRight);
+			}
+		}
+		break;
 
-		// DC removal filter
-		//   y(n) = x(n) - x(n-1) + R * y(n-1)
-		//   R = 1 - (pi*2 * cut-off-frequency / samplerate)
-		// take R = 1022/1024
-		//   44100Hz --> cutt-off freq = 14Hz
-		//   22050Hz                     7Hz
-		outLeft   =  left -  prevLeft + ((1022 *  outLeft) >> 10);
-		prevLeft  =  left;
-		outRight  = right - prevRight + ((1022 * outRight) >> 10);
-		prevRight = right;
+	case HAS_MONO_FLAG:
+		// only mono
+		if ((outLeft == outRight) && (prevLeft == prevRight)) {
+			// previous output was also mono
+#ifdef __arm__
+			// Note: there are two functional differences in the
+			//       asm and c++ code below:
+			//  - divide by 512 is replaced by ASR #9
+			//    (different for negative numbers)
+			//  - the outLeft variable is set to the clipped value
+			// Though this difference is very small, and we need
+			// the extra speed.
+			unsigned dummy;
+			asm volatile (
+			"0:\n\t"
+				"rsb	%[o],%[o],%[o],LSL #9\n\t"
+				"rsb	%[o],%[p],%[o],ASR #9\n\t"
+				"ldr	%[p],[%[in]],#4\n\t"
+				"mov	%[p],%[p],ASR #8\n\t"
+				"add	%[o],%[o],%[p]\n\t"
+				"mov	%[t],%[o],LSL #16\n\t"
+				"cmp	%[o],%[t],ASR #16\n\t"
+				"subne	%[o],%[m],%[o],ASR #31\n\t"
+				"strh	%[o],[%[out]],#2\n\t"
+				"strh	%[o],[%[out]],#2\n\t"
+				"subs	%[n],%[n],#1\n\t"
+				"bne	0b\n\t"
+				: [o]   "=r"  (outLeft)
+				, [p]   "=r"  (prevLeft)
+				, [t]   "=&r" (dummy)
+				:       "[o]" (outLeft)
+				,       "[p]" (prevLeft)
+				, [in]  "r"   (monoBuf)
+				, [out] "r"   (output)
+				, [m]   "r"   (0x7FFF)
+				, [n]   "r"   (samples)
+			);
+#else
+			for (unsigned j = 0; j < samples; ++j) {
+				int mono = monoBuf[j] >> 8;
+				outLeft   = mono -  prevLeft + ((511 *  outLeft) / 512);
+				prevLeft  = mono;
+				short out = Math::clipIntToShort(outLeft);
+				output[2 * j + 0] = out;
+				output[2 * j + 1] = out;
+			}
+#endif
+			outRight = outLeft;
+			prevRight = prevLeft;
+		} else {
+			for (unsigned j = 0; j < samples; ++j) {
+				int mono = monoBuf[j] >> 8;
+				outLeft   = mono -  prevLeft + ((511 *  outLeft) / 512);
+				prevLeft  = mono;
+				outRight  = mono - prevRight + ((511 * outRight) / 512);
+				prevRight = mono;
+				output[2 * j + 0] = Math::clipIntToShort(outLeft);
+				output[2 * j + 1] = Math::clipIntToShort(outRight);
+			}
+		}
+		break;
 
-		output[2 * j + 0] = Math::clipIntToShort(outLeft);
-		output[2 * j + 1] = Math::clipIntToShort(outRight);
+	case HAS_STEREO_FLAG:
+		// only stereo
+		for (unsigned j = 0; j < samples; ++j) {
+			int left  = stereoBuf[2 * j + 0] >> 8;
+			int right = stereoBuf[2 * j + 1] >> 8;
+			outLeft   =  left -  prevLeft + ((511 *  outLeft) / 512);
+			prevLeft  =  left;
+			outRight  = right - prevRight + ((511 * outRight) / 512);
+			prevRight = right;
+			output[2 * j + 0] = Math::clipIntToShort(outLeft);
+			output[2 * j + 1] = Math::clipIntToShort(outRight);
+		}
+		break;
+
+	default:
+		// mono + stereo
+		for (unsigned j = 0; j < samples; ++j) {
+			int mono = monoBuf[j] >> 8;
+			int left  = (stereoBuf[2 * j + 0] >> 8) + mono;
+			int right = (stereoBuf[2 * j + 1] >> 8) + mono;
+			outLeft   =  left -  prevLeft + ((511 *  outLeft) / 512);
+			prevLeft  =  left;
+			outRight  = right - prevRight + ((511 * outRight) / 512);
+			prevRight = right;
+			output[2 * j + 0] = Math::clipIntToShort(outLeft);
+			output[2 * j + 1] = Math::clipIntToShort(outRight);
+		}
 	}
 }
 
@@ -274,6 +495,11 @@ void MSXMixer::unmute()
 	}
 }
 
+void MSXMixer::reschedule()
+{
+	reInit();
+}
+
 void MSXMixer::reInit()
 {
 	if (synchronousCounter) {
@@ -288,11 +514,11 @@ void MSXMixer::reInit()
 
 	removeSyncPoints();
 	if (fragmentSize || !muteCount) {
-		prevTime = getScheduler().getCurrentTime();
+		prevTime = getCurrentTime();
 		EmuDuration interval2 = interval1 * fragmentSize;
 		setSyncPoint(prevTime + interval2);
 	} else if (synchronousCounter) {
-		prevTime = getScheduler().getCurrentTime();
+		prevTime = getCurrentTime();
 		EmuDuration interval2 = interval1 * 512;
 		setSyncPoint(prevTime + interval2);
 	}
@@ -367,7 +593,8 @@ void MSXMixer::changeRecordSetting(const Setting& setting)
 		     ++it2, ++channel) {
 			if (it2->recordSetting == &setting) {
 				it->first->recordChannel(
-					channel, it2->recordSetting->getValue());
+					channel,
+					Filename(it2->recordSetting->getValue()));
 				return;
 			}
 		}
@@ -428,10 +655,11 @@ void MSXMixer::updateVolumeParams(Infos::iterator it)
 		r1 = volume * sqrt(std::max(0.0,       b));
 		l2 = r2 = 0.0; // dummy
 	}
-	info.left1  = int(l1 * 256);
-	info.right1 = int(r1 * 256);
-	info.left2  = int(l2 * 256);
-	info.right2 = int(r2 * 256);
+	int amp = 256 * it->first->getAmplificationFactor();
+	info.left1  = int(l1 * amp);
+	info.right1 = int(r1 * amp);
+	info.left2  = int(l2 * amp);
+	info.right2 = int(r2 * amp);
 }
 
 void MSXMixer::updateMasterVolume()
@@ -441,7 +669,7 @@ void MSXMixer::updateMasterVolume()
 	}
 }
 
-void MSXMixer::executeUntil(const EmuTime& time, int /*userData*/)
+void MSXMixer::executeUntil(EmuTime::param time, int /*userData*/)
 {
 	if (!muteCount && fragmentSize) {
 		updateStream2(time);
