@@ -1,4 +1,4 @@
-// $Id: MSXSCCPlusCart.cc 6622 2007-06-15 10:02:12Z m9710797 $
+// $Id: MSXSCCPlusCart.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 // Note: this device is actually called SCC-I. But this would take a lot of
 // renaming, which isn't worth it right now. TODO rename this :)
@@ -9,24 +9,26 @@
 #include "File.hh"
 #include "FileContext.hh"
 #include "FileException.hh"
-#include "MSXCPU.hh"
 #include "MSXMotherBoard.hh"
 #include "CacheLine.hh"
 #include "XMLElement.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
 MSXSCCPlusCart::MSXSCCPlusCart(MSXMotherBoard& motherBoard,
-                               const XMLElement& config, const EmuTime& time)
-	: MSXDevice(motherBoard, config, time)
+                               const XMLElement& config)
+	: MSXDevice(motherBoard, config)
 	, ram(new Ram(motherBoard, getName() + " RAM", "SCC+ RAM", 0x20000))
+	, scc(new SCC(motherBoard, getName(), config, getCurrentTime(),
+	              SCC::SCC_Compatible))
 {
-	const XMLElement* fileElem = config.findChild("filename");
-	if (fileElem) {
+	if (const XMLElement* fileElem = config.findChild("filename")) {
 		// read the rom file
 		const std::string& filename = fileElem->getData();
 		try {
-			File file(config.getFileContext().resolve(filename));
+			File file(config.getFileContext().resolve(
+				motherBoard.getCommandController(), filename));
 			int romSize = file.getSize();
 			file.read(&(*ram)[0], romSize);
 		} catch (FileException& e) {
@@ -59,17 +61,14 @@ MSXSCCPlusCart::MSXSCCPlusCart(MSXMotherBoard& motherBoard,
 		mapper[i] = 0;
 	}
 
-	scc.reset(new SCC(motherBoard, getName(), config, time,
-	                  SCC::SCC_Compatible));
-
-	reset(time);
+	reset(getCurrentTime());
 }
 
 MSXSCCPlusCart::~MSXSCCPlusCart()
 {
 }
 
-void MSXSCCPlusCart::reset(const EmuTime& time)
+void MSXSCCPlusCart::reset(EmuTime::param time)
 {
 	setModeRegister(0);
 	setMapper(0, 0);
@@ -80,20 +79,20 @@ void MSXSCCPlusCart::reset(const EmuTime& time)
 }
 
 
-byte MSXSCCPlusCart::readMem(word addr, const EmuTime& time)
+byte MSXSCCPlusCart::readMem(word addr, EmuTime::param time)
 {
 	byte result;
 	if (((enable == EN_SCC)     && (0x9800 <= addr) && (addr < 0xA000)) ||
 	    ((enable == EN_SCCPLUS) && (0xB800 <= addr) && (addr < 0xC000))) {
-                result = scc->readMem(addr & 0xFF, time);
-        } else {
-                result = peekMem(addr, time);
-        }
+		result = scc->readMem(addr & 0xFF, time);
+	} else {
+		result = peekMem(addr, time);
+	}
 	//PRT_DEBUG("SCC+ read "<< hex << (int)addr << " " << (int)result << dec);
 	return result;
 }
 
-byte MSXSCCPlusCart::peekMem(word addr, const EmuTime& time) const
+byte MSXSCCPlusCart::peekMem(word addr, EmuTime::param time) const
 {
 	// modeRegister can not be read!
 	if (((enable == EN_SCC)     && (0x9800 <= addr) && (addr < 0xA000)) ||
@@ -127,7 +126,7 @@ const byte* MSXSCCPlusCart::getReadCacheLine(word start) const
 }
 
 
-void MSXSCCPlusCart::writeMem(word address, byte value, const EmuTime& time)
+void MSXSCCPlusCart::writeMem(word address, byte value, EmuTime::param time)
 {
 	//PRT_DEBUG("SCC+ write "<< hex << address << " " << (int)value << dec);
 
@@ -145,7 +144,7 @@ void MSXSCCPlusCart::writeMem(word address, byte value, const EmuTime& time)
 	// Write to RAM
 	int regio = (address >> 13) - 2;
 	if (isRamSegment[regio]) {
-		//According to Sean Young
+		// According to Sean Young
 		// when the regio's are in RAM mode you can read from
 		// the SCC(+) but not write to them
 		// => we assume a write to the memory but maybe
@@ -189,7 +188,6 @@ void MSXSCCPlusCart::writeMem(word address, byte value, const EmuTime& time)
 
 byte* MSXSCCPlusCart::getWriteCacheLine(word start) const
 {
-	//return NULL;
 	if ((0x4000 <= start) && (start < 0xC000)) {
 		if (start == (0xBFFF & CacheLine::HIGH)) {
 			return NULL;
@@ -221,8 +219,7 @@ void MSXSCCPlusCart::setMapper(int regio, byte value)
 
 	checkEnable();
 	internalMemoryBank[regio] = block;
-	getMotherBoard().getCPU().invalidateMemCache(
-		0x4000 + regio * 0x2000, 0x2000);
+	invalidateMemCache(0x4000 + regio * 0x2000, 0x2000);
 }
 
 void MSXSCCPlusCart::setModeRegister(byte value)
@@ -242,12 +239,11 @@ void MSXSCCPlusCart::setModeRegister(byte value)
 		isRamSegment[2] = true;
 		isRamSegment[3] = true;
 	} else {
-		MSXCPU& cpu = getMotherBoard().getCPU();
 		if (modeRegister & 0x01) {
 			isRamSegment[0] = true;
 		} else {
 			if (isRamSegment[0]) {
-				cpu.invalidateMemCache(0x4000, 0x2000);
+				invalidateMemCache(0x4000, 0x2000);
 				isRamSegment[0] = false;
 			}
 		}
@@ -255,7 +251,7 @@ void MSXSCCPlusCart::setModeRegister(byte value)
 			isRamSegment[1] = true;
 		} else {
 			if (isRamSegment[1]) {
-				cpu.invalidateMemCache(0x6000, 0x2000);
+				invalidateMemCache(0x6000, 0x2000);
 				isRamSegment[1] = false;
 			}
 		}
@@ -264,12 +260,12 @@ void MSXSCCPlusCart::setModeRegister(byte value)
 			isRamSegment[2] = true;
 		} else {
 			if (isRamSegment[2]) {
-				cpu.invalidateMemCache(0x8000, 0x2000);
+				invalidateMemCache(0x8000, 0x2000);
 				isRamSegment[2] = false;
 			}
 		}
 		if (isRamSegment[3]) {
-			cpu.invalidateMemCache(0xA000, 0x2000);
+			invalidateMemCache(0xA000, 0x2000);
 			isRamSegment[3] = false;
 		}
 	}
@@ -286,4 +282,34 @@ void MSXSCCPlusCart::checkEnable()
 	}
 }
 
+
+template<typename Archive>
+void MSXSCCPlusCart::serialize(Archive& ar, unsigned /*version*/)
+{
+	// These are constants:
+	//   mapperMask, lowRAM, highRAM
+
+	// only serialize that part of the Ram object that's actually
+	// present in the cartridge
+	unsigned ramSize = (lowRAM && highRAM && (mapperMask == 0xF))
+	                 ? 0x20000 : 0x10000;
+	unsigned ramBase = lowRAM ? 0x00000 : 0x10000;
+	ar.serialize_blob("ram", &(*ram)[ramBase], ramSize);
+
+	ar.serialize("scc", *scc);
+	ar.serialize("mapper", mapper);
+	ar.serialize("mode", modeRegister);
+
+	if (ar.isLoader()) {
+		// recalculate: isMapped[4], internalMemoryBank[4]
+		for (int i = 0; i < 4; ++i) {
+			setMapper(i, mapper[i]);
+		}
+		// recalculate: enable, isRamSegment[4]
+		setModeRegister(modeRegister);
+	}
+}
+INSTANTIATE_SERIALIZE_METHODS(MSXSCCPlusCart);
+REGISTER_MSXDEVICE(MSXSCCPlusCart, "SCCPlus");
+
 } // namespace openmsx
