@@ -1,24 +1,25 @@
-// $Id: MSXTurboRPCM.cc 6623 2007-06-15 11:27:19Z m9710797 $
+// $Id: MSXTurboRPCM.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "MSXTurboRPCM.hh"
 #include "MSXMotherBoard.hh"
 #include "AudioInputConnector.hh"
 #include "DACSound8U.hh"
 #include "MSXMixer.hh"
+#include "serialize.hh"
 
 namespace openmsx {
 
 MSXTurboRPCM::MSXTurboRPCM(MSXMotherBoard& motherBoard,
-                           const XMLElement& config, const EmuTime& time)
-	: MSXDevice(motherBoard, config, time)
+                           const XMLElement& config)
+	: MSXDevice(motherBoard, config)
 	, mixer(motherBoard.getMSXMixer())
 	, connector(new AudioInputConnector(
               motherBoard.getPluggingController(), "pcminput"))
-	, dac(new DACSound8U(mixer, "PCM", "Turbo-R PCM", config, time))
-	, reference(time)
+	, dac(new DACSound8U(mixer, "PCM", "Turbo-R PCM", config))
+	, reference(getCurrentTime())
 	, hwMute(false)
 {
-	reset(time);
+	reset(getCurrentTime());
 }
 
 MSXTurboRPCM::~MSXTurboRPCM()
@@ -26,20 +27,22 @@ MSXTurboRPCM::~MSXTurboRPCM()
 	hardwareMute(false);
 }
 
-void MSXTurboRPCM::reset(const EmuTime& time)
+void MSXTurboRPCM::reset(EmuTime::param time)
 {
 	reference.advance(time);
 	status = 0;
+	DValue = 0x80; // TODO correct initial value?
+	hold = 0x80; // avoid UMR
 	dac->reset(time);
 	hardwareMute(false);
 }
 
-byte MSXTurboRPCM::readIO(word port, const EmuTime& time)
+byte MSXTurboRPCM::readIO(word port, EmuTime::param time)
 {
 	return peekIO(port, time);
 }
 
-byte MSXTurboRPCM::peekIO(word port, const EmuTime& time) const
+byte MSXTurboRPCM::peekIO(word port, EmuTime::param time) const
 {
 	byte result;
 	switch (port & 0x01) {
@@ -71,7 +74,7 @@ byte MSXTurboRPCM::peekIO(word port, const EmuTime& time) const
 	return result;
 }
 
-void MSXTurboRPCM::writeIO(word port, byte value, const EmuTime& time)
+void MSXTurboRPCM::writeIO(word port, byte value, EmuTime::param time)
 {
 	//PRT_DEBUG("PCM: write " << hex << (int)port << " " << (int)value << dec);
 	switch (port & 0x01) {
@@ -108,19 +111,19 @@ void MSXTurboRPCM::writeIO(word port, byte value, const EmuTime& time)
 	}
 }
 
-byte MSXTurboRPCM::getSample(const EmuTime& time) const
+byte MSXTurboRPCM::getSample(EmuTime::param time) const
 {
 	byte result;
 	if (status & 0x04) {
 		result = (connector->readSample(time) / 256) + 0x80;
 	} else {
-		result = 0x80;	// TODO check
+		result = 0x80; // TODO check
 	}
 	//PRT_DEBUG("PCM: read " << (int)result);
 	return result;
 }
 
-bool MSXTurboRPCM::getComp(const EmuTime& time) const
+bool MSXTurboRPCM::getComp(EmuTime::param time) const
 {
 	// TODO also when D/A ??
 	byte sample = (status & 0x10) ? hold : getSample(time);
@@ -139,4 +142,22 @@ void MSXTurboRPCM::hardwareMute(bool mute)
 	}
 }
 
+
+template<typename Archive>
+void MSXTurboRPCM::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<MSXDevice>(*this);
+
+	ar.serialize("audioConnector", *connector);
+	ar.serialize("reference", reference);
+	ar.serialize("status", status);
+	ar.serialize("DValue", DValue);
+	ar.serialize("hold", hold);
+	ar.serialize("DAC", *dac);
+
+	hardwareMute(!(status & 0x02));  // restore hwMute
+}
+INSTANTIATE_SERIALIZE_METHODS(MSXTurboRPCM);
+REGISTER_MSXDEVICE(MSXTurboRPCM, "TurboR-PCM");
+
 } // namespace openmsx
