@@ -2,6 +2,7 @@
 
 #include "ResampleTrivial.hh"
 #include "Resample.hh"
+#include <cassert>
 
 namespace openmsx {
 
@@ -12,6 +13,7 @@ ResampleTrivial::ResampleTrivial(Resample& input_)
 
 bool ResampleTrivial::generateOutput(int* dataOut, unsigned num)
 {
+	assert((long(dataOut) & 15) == 0); // must be 16-byte aligned
 	return input.generateInput(dataOut, num);
 }
 
