@@ -1,4 +1,4 @@
-// $Id: SCC.cc 7197 2007-10-23 18:35:20Z m9710797 $
+// $Id: SCC.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 //-----------------------------------------------------------------------------
 //
@@ -101,6 +101,8 @@
 #include "SCC.hh"
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
+#include "serialize.hh"
+#include "likely.hh"
 
 using std::string;
 
@@ -110,8 +112,8 @@ class SCCDebuggable : public SimpleDebuggable
 {
 public:
 	SCCDebuggable(MSXMotherBoard& motherBoard, SCC& scc);
-	virtual byte read(unsigned address, const EmuTime& time);
-	virtual void write(unsigned address, byte value, const EmuTime& time);
+	virtual byte read(unsigned address, EmuTime::param time);
+	virtual void write(unsigned address, byte value, EmuTime::param time);
 private:
 	SCC& scc;
 };
@@ -123,29 +125,48 @@ static string calcDescription(SCC::ChipMode mode)
 }
 
 SCC::SCC(MSXMotherBoard& motherBoard, const string& name,
-         const XMLElement& config, const EmuTime& time, ChipMode mode)
+         const XMLElement& config, EmuTime::param time, ChipMode mode)
 	: SoundDevice(motherBoard.getMSXMixer(), name, calcDescription(mode), 5)
 	, Resample(motherBoard.getGlobalSettings(), 1)
 	, debuggable(new SCCDebuggable(motherBoard, *this))
 	, deformTimer(time)
 	, currentChipMode(mode)
 {
-	// Clear all state that is unaffected by reset.
+	// Power on values, tested by enen (log from IRC #openmsx):
+	//
+	//  <enen>    wouter_: i did an scc poweron values test, deform=0,
+	//            amplitude=full, channelenable=0, period=under 8
+	//    ...
+	//  <wouter_> did you test the value of the waveforms as well?
+	//    ...
+	//  <enen>    filled with $FF, some bits cleared but that seems random
+
+	// Make valgrind happy
+	for (int i = 0; i < 5; ++i) {
+		orgPeriod[i] = 0;
+	}
+	// Initialize ch_enable, deform (initialize this before period)
+	reset(time);
+	// Initialize waveform (initialize before volumes)
 	for (unsigned i = 0; i < 5; ++i) {
 		for (unsigned j = 0; j < 32; ++j) {
-			wave[i][j] = 0;
-			volAdjustedWave[i][j] = 0;
+			wave[i][j] = 0xFF;
 		}
-		out[i] = 0;
-		count[i] = 0;
-		incr[i] = 0;
+	}
+	// Initialize volume (initialize this before period)
+	for (int i = 0; i < 5; ++i) {
+		setFreqVol(i + 10, 15, time);
+	}
+	// Actual initial value is difficult to measure, assume zero
+	// (initialize before period)
+	for (unsigned i = 0; i < 5; ++i) {
 		pos[i] = 0;
-		period[i] = 0;
-		orgPeriod[i] = 0;
-		volume[i] = 0;
+	}
+	// Initialize period (sets members orgPeriod, period, incr, count, out)
+	for (int i = 0; i < 2 * 5; ++i) {
+		setFreqVol(i, 0, time);
 	}
 
-	reset(time);
 	registerSound(config);
 }
 
@@ -154,17 +175,13 @@ SCC::~SCC()
 	unregisterSound();
 }
 
-void SCC::reset(const EmuTime& /*time*/)
+void SCC::reset(EmuTime::param /*time*/)
 {
 	if (currentChipMode != SCC_Real) {
 		setChipMode(SCC_Compatible);
 	}
 
-	for (unsigned i = 0; i < 5; ++i) {
-		rotate[i] = false;
-		readOnly[i] = false;
-	}
-	deformValue = 0;
+	setDeformRegHelper(0);
 	ch_enable = 0;
 }
 
@@ -185,7 +202,7 @@ void SCC::setChipMode(ChipMode newMode)
 	currentChipMode = newMode;
 }
 
-byte SCC::readMem(byte addr, const EmuTime& time)
+byte SCC::readMem(byte addr, EmuTime::param time)
 {
 	// Deform-register locations:
 	//   SCC_Real:       0xE0..0xFF
@@ -198,7 +215,7 @@ byte SCC::readMem(byte addr, const EmuTime& time)
 	return peekMem(addr, time);
 }
 
-byte SCC::peekMem(byte address, const EmuTime& time) const
+byte SCC::peekMem(byte address, EmuTime::param time) const
 {
 	byte result;
 	switch (currentChipMode) {
@@ -248,7 +265,7 @@ byte SCC::peekMem(byte address, const EmuTime& time) const
 	return result;
 }
 
-byte SCC::readWave(byte channel, byte address, const EmuTime& time) const
+byte SCC::readWave(unsigned channel, unsigned address, EmuTime::param time) const
 {
 	if (!rotate[channel]) {
 		return wave[channel][address & 0x1F];
@@ -264,12 +281,12 @@ byte SCC::readWave(byte channel, byte address, const EmuTime& time) const
 }
 
 
-byte SCC::getFreqVol(byte address) const
+byte SCC::getFreqVol(unsigned address) const
 {
 	address &= 0x0F;
 	if (address < 0x0A) {
 		// get frequency
-		byte channel = address / 2;
+		unsigned channel = address / 2;
 		if (address & 1) {
 			return orgPeriod[channel] >> 8;
 		} else {
@@ -284,7 +301,7 @@ byte SCC::getFreqVol(byte address) const
 	}
 }
 
-void SCC::writeMem(byte address, byte value, const EmuTime& time)
+void SCC::writeMem(byte address, byte value, EmuTime::param time)
 {
 	updateStream(time);
 
@@ -295,7 +312,7 @@ void SCC::writeMem(byte address, byte value, const EmuTime& time)
 			writeWave(address >> 5, address, value);
 		} else if (address < 0xA0) {
 			// 0x80..0x9F : freq volume block
-			setFreqVol(address, value);
+			setFreqVol(address, value, time);
 		} else if (address < 0xE0) {
 			// 0xA0..0xDF : no function
 		} else {
@@ -309,7 +326,7 @@ void SCC::writeMem(byte address, byte value, const EmuTime& time)
 			writeWave(address >> 5, address, value);
 		} else if (address < 0xA0) {
 			// 0x80..0x9F : freq volume block
-			setFreqVol(address, value);
+			setFreqVol(address, value, time);
 		} else if (address < 0xC0) {
 			// 0xA0..0xBF : ignore write wave form 5
 		} else if (address < 0xE0) {
@@ -325,7 +342,7 @@ void SCC::writeMem(byte address, byte value, const EmuTime& time)
 			writeWave(address >> 5, address, value);
 		} else if (address < 0xC0) {
 			// 0xA0..0xBF : freq volume block
-			setFreqVol(address, value);
+			setFreqVol(address, value, time);
 		} else if (address < 0xE0) {
 			// 0xC0..0xDF : deformation register
 			setDeformReg(value, time);
@@ -338,20 +355,24 @@ void SCC::writeMem(byte address, byte value, const EmuTime& time)
 	}
 }
 
-inline int SCC::adjust(signed char wav, byte vol)
+int SCC::getAmplificationFactor() const
 {
-	int tmp = (int(wav) * vol) / 16;
-	return tmp * 256;
+	return 256;
 }
 
-void SCC::writeWave(byte channel, byte address, byte value)
+inline int SCC::adjust(signed char wav, byte vol)
+{
+	return (int(wav) * vol) >> 4;
+}
+
+void SCC::writeWave(unsigned channel, unsigned address, byte value)
 {
 	// write to channel 5 only possible in SCC+ mode
 	assert(channel < 5);
 	assert((channel != 4) || (currentChipMode == SCC_plusmode));
 
 	if (!readOnly[channel]) {
-		byte pos = address & 0x1F;
+		unsigned pos = address & 0x1F;
 		wave[channel][pos] = value;
 		volAdjustedWave[channel][pos] = adjust(value, volume[channel]);
 		if ((currentChipMode != SCC_plusmode) && (channel == 3)) {
@@ -362,12 +383,12 @@ void SCC::writeWave(byte channel, byte address, byte value)
 	}
 }
 
-void SCC::setFreqVol(byte address, byte value)
+void SCC::setFreqVol(unsigned address, byte value, EmuTime::param time)
 {
 	address &= 0x0F; // region is visible twice
 	if (address < 0x0A) {
 		// change frequency
-		byte channel = address / 2;
+		unsigned channel = address / 2;
 		unsigned per =
 			  (address & 1)
 			? ((value & 0xF) << 8) | (orgPeriod[channel] & 0xFF)
@@ -385,14 +406,17 @@ void SCC::setFreqVol(byte address, byte value)
 		count[channel] = 0; // reset to begin of byte
 		if (deformValue & 0x20) {
 			pos[channel] = 0; // reset to begin of waveform
+			// also 'rotation' mode (confirmed by test based on
+			// Artag's SCC sample player)
+			deformTimer.advance(time);
 		}
 		// after a freq change, update the output
 		out[channel] = volAdjustedWave[channel][pos[channel]];
 	} else if (address < 0x0F) {
 		// change volume
-		byte channel = address - 0x0A;
+		unsigned channel = address - 0x0A;
 		volume[channel] = value & 0xF;
-		for (int i = 0; i < 32; ++i) {
+		for (unsigned i = 0; i < 32; ++i) {
 			volAdjustedWave[channel][i] =
 				adjust(wave[channel][i], volume[channel]);
 		}
@@ -402,14 +426,18 @@ void SCC::setFreqVol(byte address, byte value)
 	}
 }
 
-void SCC::setDeformReg(byte value, const EmuTime& time)
+void SCC::setDeformReg(byte value, EmuTime::param time)
 {
 	if (value == deformValue) {
 		return;
 	}
-	deformValue = value;
 	deformTimer.advance(time);
+	setDeformRegHelper(value);
+}
 
+void SCC::setDeformRegHelper(byte value)
+{
+	deformValue = value;
 	if (currentChipMode != SCC_Real) {
 		value &= ~0x80;
 	}
@@ -453,20 +481,70 @@ void SCC::setDeformReg(byte value, const EmuTime& time)
 
 void SCC::generateChannels(int** bufs, unsigned num)
 {
-	byte enable = ch_enable;
-	for (int i = 0; i < 5; ++i, enable >>= 1) {
+	unsigned enable = ch_enable;
+	for (unsigned i = 0; i < 5; ++i, enable >>= 1) {
 		if ((enable & 1) && (volume[i] || out[i])) {
+#ifdef __arm__
+			unsigned dummy;
+			int* buf = bufs[i];
+			asm (
+			"0:\n\t"
+				"ldr	%[T],[%[B]]\n\t"
+				"add	%[T],%[T],%[O]\n\t"
+				"add	%[C],%[C],%[I]\n\t"
+				"str	%[T],[%[B]],#4\n\t"
+				"subs	%[T],%[C],%[PE]\n\t"
+				"bpl	2f\n"
+			"1:\n\t"
+				"cmp	%[B],%[E]\n\t"
+				"bne	0b\n\t"
+				"b	3f\n"
+			"2:\n\t"
+				"subs	%[T],%[T],%[PE]\n\t"
+				"add	%[PO],%[PO],#1\n\t"
+				"bpl	2b\n\t"
+				"and	%[PO],%[PO],#31\n\t"
+				"add	%[C],%[T],%[PE]\n\t"
+				"ldr	%[O],[%[V],%[PO],LSL #2]\n\t"
+				"b	1b\n"
+			"3:\n\t"
+
+				: [T]  "=&r"  (dummy)
+				, [B]  "=r"   (buf)
+				, [O]  "=r"   (out[i])
+				, [C]  "=r"   (count[i])
+				, [PO] "=r"   (pos[i])
+				:      "[B]"  (buf)
+				,      "[O]"  (out[i])
+				,      "[C]"  (count[i])
+				, [I]  "r"    (incr[i])
+				, [PE] "r"    (period[i] + 1)
+				, [E]  "r"    (&buf[num])
+				,      "[PO]" (pos[i])
+				, [V]  "r"    (volAdjustedWave[i])
+				: "cc"
+			);
+#else
+			int out2 = out[i];
+			unsigned count2 = count[i];
+			unsigned pos2 = pos[i];
+			unsigned incr2 = incr[i];
+			unsigned period2 = period[i] + 1;
 			for (unsigned j = 0; j < num; ++j) {
-				bufs[i][j] = out[i];
-				count[i] += incr[i];
-				// Note: Only for very small periods this loop will take
-				//       more than a single iteration.
-				while (count[i] > period[i]) {
-					pos[i] = (pos[i] + 1) % 32;
-					count[i] -= period[i] + 1;
-					out[i] = volAdjustedWave[i][pos[i]];
+				bufs[i][j] += out2;
+				count2 += incr2;
+				// Note: only for very small periods
+				//       this will take more than 1 iteration
+				while (unlikely(count2 >= period2)) {
+					count2 -= period2;
+					pos2 = (pos2 + 1) % 32;
+					out2 = volAdjustedWave[i][pos2];
 				}
 			}
+			out[i] = out2;
+			count[i] = count2;
+			pos[i] = pos2;
+#endif
 		} else {
 			bufs[i] = 0; // channel muted
 			// Update phase counter.
@@ -485,7 +563,7 @@ bool SCC::generateInput(int* buffer, unsigned num)
 }
 
 bool SCC::updateBuffer(unsigned length, int* buffer,
-                       const EmuTime& /*time*/, const EmuDuration& /*sampDur*/)
+                       EmuTime::param /*time*/, EmuDuration::param /*sampDur*/)
 {
 	return generateOutput(buffer, length);
 }
@@ -500,7 +578,7 @@ SCCDebuggable::SCCDebuggable(MSXMotherBoard& motherBoard, SCC& scc_)
 {
 }
 
-byte SCCDebuggable::read(unsigned address, const EmuTime& time)
+byte SCCDebuggable::read(unsigned address, EmuTime::param time)
 {
 	if (address < 0xA0) {
 		// read wave form 1..5
@@ -516,14 +594,14 @@ byte SCCDebuggable::read(unsigned address, const EmuTime& time)
 	}
 }
 
-void SCCDebuggable::write(unsigned address, byte value, const EmuTime& time)
+void SCCDebuggable::write(unsigned address, byte value, EmuTime::param time)
 {
 	if (address < 0xA0) {
 		// read wave form 1..5
 		scc.writeWave(address >> 5, address, value);
 	} else if (address < 0xC0) {
 		// freq volume block
-		scc.setFreqVol(address, value);
+		scc.setFreqVol(address, value, time);
 	} else if (address < 0xE0) {
 		// deformation register
 		scc.setDeformReg(value, time);
@@ -532,4 +610,61 @@ void SCCDebuggable::write(unsigned address, byte value, const EmuTime& time)
 	}
 }
 
+
+static enum_string<SCC::ChipMode> chipModeInfo[] = {
+	{ "Real",       SCC::SCC_Real       },
+	{ "Compatible", SCC::SCC_Compatible },
+	{ "Plus",       SCC::SCC_plusmode   },
+};
+SERIALIZE_ENUM(SCC::ChipMode, chipModeInfo);
+
+template<typename Archive>
+void SCC::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.serialize("mode", currentChipMode);
+	ar.serialize("period", orgPeriod);
+	ar.serialize("volume", volume);
+	ar.serialize("ch_enable", ch_enable);
+	ar.serialize("deformTimer", deformTimer);
+	ar.serialize("deform", deformValue);
+	// multi-dimensional arrays are not directly support by the
+	// serialization framework, maybe in the future. So for now
+	// manually loop over the channels.
+	for (int channel = 0; channel < 5; ++channel) {
+		string tag = string("wave") + char('1' + channel);
+		ar.serialize(tag.c_str(), wave[channel]); // signed char
+	}
+
+	if (ar.isLoader()) {
+		// recalculate volAdjustedWave
+		for (int channel = 0; channel < 5; ++channel) {
+			for (int pos = 0; pos < 32; ++pos) {
+				volAdjustedWave[channel][pos] =
+					adjust(wave[channel][pos], volume[channel]);
+			}
+		}
+
+		// recalculate rotate[5] and readOnly[5]
+		setDeformRegHelper(deformValue);
+
+		// recalculate incr[5] and period[5]
+		//  this also (possibly) changes count[5], pos[5] and out[5]
+		//  as an unwanted side-effect, so (de)serialize those later
+		// Don't use current time, but instead use deformTimer, to
+		// avoid changing the value of deformTimer.
+		EmuTime::param time = deformTimer.getTime();
+		for (int channel = 0; channel < 5; ++channel) {
+			unsigned per = orgPeriod[channel];
+			setFreqVol(2 * channel + 0, (per & 0x0FF) >> 0, time);
+			setFreqVol(2 * channel + 1, (per & 0xF00) >> 8, time);
+		}
+	}
+
+	// call to setFreqVol() modifies these variables, see above
+	ar.serialize("count", count);
+	ar.serialize("pos", pos);
+	ar.serialize("out", out);
+}
+INSTANTIATE_SERIALIZE_METHODS(SCC);
+
 } // namespace openmsx
