@@ -1,4 +1,4 @@
-// $Id: VLM5030.cc 7188 2007-10-22 08:11:25Z m9710797 $
+// $Id: VLM5030.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 /*
     vlm5030.c
@@ -77,15 +77,93 @@ chirp 12-..: vokume   0   : silent
 */
 
 #include "VLM5030.hh"
+#include "SoundDevice.hh"
+#include "Resample.hh"
 #include "Rom.hh"
 #include "MSXMotherBoard.hh"
 #include "XMLElement.hh"
 #include "FileContext.hh"
+#include "serialize.hh"
 #include <cstring>
 #include <cstdlib>
 
 namespace openmsx {
 
+class VLM5030Impl : public SoundDevice, private Resample
+{
+public:
+	VLM5030Impl(MSXMotherBoard& motherBoard, const std::string& name,
+	            const std::string& desc, const XMLElement& config);
+	~VLM5030Impl();
+
+	void reset();
+	void writeData(byte data);
+	void writeControl(byte data, EmuTime::param time);
+	bool getBSY(EmuTime::param time);
+
+	template<typename Archive>
+	void serialize(Archive& ar, unsigned version);
+
+private:
+	void setRST(bool pin);
+	void setVCU(bool pin);
+	void setST (bool pin);
+
+	// SoundDevice
+	virtual void setOutputRate(unsigned sampleRate);
+	virtual void generateChannels(int** bufs, unsigned num);
+	virtual bool updateBuffer(
+		unsigned length, int* buffer, EmuTime::param start,
+		EmuDuration::param sampDur);
+
+	// Resample
+	virtual bool generateInput(int* buffer, unsigned num);
+
+	void setupParameter(byte param);
+	int getBits(unsigned sbit, unsigned bits);
+	int parseFrame();
+
+	std::auto_ptr<Rom> rom;
+	int address_mask;
+
+	// state of option paramter
+	int frame_size;
+	int pitch_offset;
+
+	// these contain data describing the current and previous voice frames
+	// these are all used to contain the current state of the sound generation
+	unsigned current_energy;
+	unsigned current_pitch;
+	int current_k[10];
+	int x[10];
+
+	word address;
+	word vcu_addr_h;
+
+	signed_word old_k[10];
+	signed_word new_k[10];
+	signed_word target_k[10];
+	word old_energy;
+	word new_energy;
+	word target_energy;
+	byte old_pitch;
+	byte new_pitch;
+	byte target_pitch;
+
+	byte interp_step;
+	byte interp_count; // number of interp periods
+	byte sample_count; // sample number within interp
+	byte pitch_count;
+
+	byte latch_data;
+	byte parameter;
+	byte phase;
+	bool pin_BSY;
+	bool pin_ST;
+	bool pin_VCU;
+	bool pin_RST;
+};
+
 // interpolator per frame
 static const int FR_SIZE = 4;
 // samples per interpolator
@@ -106,23 +184,23 @@ enum {
 	PH_END
 };
 
-//  speed parameter
-//SPC SPB SPA
-// 1   0   1  more slow (05h)     : 42ms   (150%) : 60sample
-// 1   1   x  slow      (06h,07h) : 34ms   (125%) : 50sample
-// x   0   0  normal    (00h,04h) : 25.6ms (100%) : 40samplme
-// 0   0   1  fast      (01h)     : 20.2ms  (75%) : 30sample
-// 0   1   x  more fast (02h,03h) : 12.2ms  (50%) : 20sample
+// speed parameter
+// SPC SPB SPA
+//  1   0   1  more slow (05h)     : 42ms   (150%) : 60sample
+//  1   1   x  slow      (06h,07h) : 34ms   (125%) : 50sample
+//  x   0   0  normal    (00h,04h) : 25.6ms (100%) : 40samplme
+//  0   0   1  fast      (01h)     : 20.2ms  (75%) : 30sample
+//  0   1   x  more fast (02h,03h) : 12.2ms  (50%) : 20sample
 static const int VLM5030_speed_table[8] =
 {
- IP_SIZE_NORMAL,
- IP_SIZE_FAST,
- IP_SIZE_FASTER,
- IP_SIZE_FASTER,
- IP_SIZE_NORMAL,
- IP_SIZE_SLOWER,
- IP_SIZE_SLOW,
- IP_SIZE_SLOW
+	IP_SIZE_NORMAL,
+	IP_SIZE_FAST,
+	IP_SIZE_FASTER,
+	IP_SIZE_FASTER,
+	IP_SIZE_NORMAL,
+	IP_SIZE_SLOWER,
+	IP_SIZE_SLOW,
+	IP_SIZE_SLOW
 };
 
 // ROM Tables
@@ -139,52 +217,52 @@ static word energytable[0x20] =
 };
 
 // This is the pitch lookup table
-static const byte pitchtable [0x20]=
+static const byte pitchtable [0x20] =
 {
-   1,                               // 0     : random mode
-   22,                              // 1     : start=22
-   23, 24, 25, 26, 27, 28, 29, 30,  //  2- 9 : 1step
-   32, 34, 36, 38, 40, 42, 44, 46,  // 10-17 : 2step
-   50, 54, 58, 62, 66, 70, 74, 78,  // 18-25 : 4step
-   86, 94, 102,110,118,126          // 26-31 : 8step
+	1,                               // 0     : random mode
+	22,                              // 1     : start=22
+	23, 24, 25, 26, 27, 28, 29, 30,  //  2- 9 : 1step
+	32, 34, 36, 38, 40, 42, 44, 46,  // 10-17 : 2step
+	50, 54, 58, 62, 66, 70, 74, 78,  // 18-25 : 4step
+	86, 94, 102,110,118,126          // 26-31 : 8step
 };
 
 static const signed_word K1_table[] = {
-  -24898,  -25672,  -26446,  -27091,  -27736,  -28252,  -28768,  -29155,
-  -29542,  -29929,  -30316,  -30574,  -30832,  -30961,  -31219,  -31348,
-  -31606,  -31735,  -31864,  -31864,  -31993,  -32122,  -32122,  -32251,
-  -32251,  -32380,  -32380,  -32380,  -32509,  -32509,  -32509,  -32509,
-   24898,   23995,   22963,   21931,   20770,   19480,   18061,   16642,
-   15093,   13416,   11610,    9804,    7998,    6063,    3999,    1935,
-       0,   -1935,   -3999,   -6063,   -7998,   -9804,  -11610,  -13416,
-  -15093,  -16642,  -18061,  -19480,  -20770,  -21931,  -22963,  -23995
+	-24898,  -25672,  -26446,  -27091,  -27736,  -28252,  -28768,  -29155,
+	-29542,  -29929,  -30316,  -30574,  -30832,  -30961,  -31219,  -31348,
+	-31606,  -31735,  -31864,  -31864,  -31993,  -32122,  -32122,  -32251,
+	-32251,  -32380,  -32380,  -32380,  -32509,  -32509,  -32509,  -32509,
+	 24898,   23995,   22963,   21931,   20770,   19480,   18061,   16642,
+	 15093,   13416,   11610,    9804,    7998,    6063,    3999,    1935,
+	     0,   -1935,   -3999,   -6063,   -7998,   -9804,  -11610,  -13416,
+	-15093,  -16642,  -18061,  -19480,  -20770,  -21931,  -22963,  -23995
 };
 static const signed_word K2_table[] = {
-       0,   -3096,   -6321,   -9417,  -12513,  -15351,  -18061,  -20770,
-  -23092,  -25285,  -27220,  -28897,  -30187,  -31348,  -32122,  -32638,
-       0,   32638,   32122,   31348,   30187,   28897,   27220,   25285,
-   23092,   20770,   18061,   15351,   12513,    9417,    6321,    3096
+	     0,   -3096,   -6321,   -9417,  -12513,  -15351,  -18061,  -20770,
+	-23092,  -25285,  -27220,  -28897,  -30187,  -31348,  -32122,  -32638,
+	     0,   32638,   32122,   31348,   30187,   28897,   27220,   25285,
+	 23092,   20770,   18061,   15351,   12513,    9417,    6321,    3096
 };
 static const signed_word K3_table[] = {
-       0,   -3999,   -8127,  -12255,  -16384,  -20383,  -24511,  -28639,
-   32638,   28639,   24511,   20383,   16254,   12255,    8127,    3999
+	    0,   -3999,   -8127,  -12255,  -16384,  -20383,  -24511,  -28639,
+	32638,   28639,   24511,   20383,   16254,   12255,    8127,    3999
 };
 static const signed_word K5_table[] = {
-       0,   -8127,  -16384,  -24511,   32638,   24511,   16254,    8127
+	0,   -8127,  -16384,  -24511,   32638,   24511,   16254,    8127
 };
 
-int VLM5030::getBits(int sbit, int bits)
+int VLM5030Impl::getBits(unsigned sbit, unsigned bits)
 {
 	unsigned offset = address + (sbit / 8);
-	int data = (*rom)[(offset + 0) & address_mask] +
-	           (*rom)[(offset + 1) & address_mask] * 256;
+	unsigned data = (*rom)[(offset + 0) & address_mask] +
+	                (*rom)[(offset + 1) & address_mask] * 256;
 	data >>= (sbit & 7);
 	data &= (0xFF >> (8 - bits));
 	return data;
 }
 
 // get next frame
-int VLM5030::parseFrame()
+int VLM5030Impl::parseFrame()
 {
 	// remember previous frame
 	old_energy = new_energy;
@@ -232,7 +310,7 @@ int VLM5030::parseFrame()
 }
 
 // decode and buffering data
-void VLM5030::generateChannels(int** bufs, unsigned length)
+void VLM5030Impl::generateChannels(int** bufs, unsigned length)
 {
 	if (phase == PH_IDLE) {
 		bufs[0] = 0;
@@ -324,11 +402,11 @@ void VLM5030::generateChannels(int** bufs, unsigned length)
 
 			// clipping, buffering
 			if (u[0] > 511) {
-				bufs[0][buf_count] = 511 << 6;
+				bufs[0][buf_count] += 511 << 6;
 			} else if (u[0] < -511) {
-				bufs[0][buf_count] = -511 << 6;
+				bufs[0][buf_count] += -511 << 6;
 			} else {
-				bufs[0][buf_count] = (u[0] << 6);
+				bufs[0][buf_count] += (u[0] << 6);
 			}
 			++buf_count;
 			--sample_count;
@@ -361,14 +439,14 @@ phase_stop:
 		}
 	}
 	// silent buffering
-	while (length > 0) {
-		bufs[0][buf_count++] = 0;
-		--length;
-	}
+	//while (length > 0) {
+	//	bufs[0][buf_count++] += 0;
+	//	--length;
+	//}
 }
 
 // setup parameteroption when RST=H
-void VLM5030::setupParameter(byte param)
+void VLM5030Impl::setupParameter(byte param)
 {
 	// latch parameter value
 	parameter = param;
@@ -395,7 +473,7 @@ void VLM5030::setupParameter(byte param)
 	}
 }
 
-void VLM5030::reset(const EmuTime& /*time*/)
+void VLM5030Impl::reset()
 {
 	phase = PH_RESET;
 	address = 0;
@@ -417,20 +495,28 @@ void VLM5030::reset(const EmuTime& /*time*/)
 }
 
 // get BSY pin level
-bool VLM5030::getBSY(const EmuTime& time)
+bool VLM5030Impl::getBSY(EmuTime::param time)
 {
 	updateStream(time);
 	return pin_BSY;
 }
 
 // latch control data
-void VLM5030::writeData(byte data)
+void VLM5030Impl::writeData(byte data)
 {
 	latch_data = data;
 }
 
+void VLM5030Impl::writeControl(byte data, EmuTime::param time)
+{
+	updateStream(time);
+	setRST(data & 0x01);
+	setVCU(data & 0x04);
+	setST (data & 0x02);
+}
+
 // set RST pin level : reset / set table address A8-A15
-void VLM5030::setRST (bool pin, const EmuTime& time)
+void VLM5030Impl::setRST(bool pin)
 {
 	if (pin_RST) {
 		if (!pin) { // H -> L : latch parameters
@@ -441,21 +527,21 @@ void VLM5030::setRST (bool pin, const EmuTime& time)
 		if (pin) { // L -> H : reset chip
 			pin_RST = true;
 			if (pin_BSY) {
-				reset(time);
+				reset();
 			}
 		}
 	}
 }
 
 // set VCU pin level : ?? unknown
-void VLM5030::setVCU(bool pin, const EmuTime& /*time*/)
+void VLM5030Impl::setVCU(bool pin)
 {
 	// direct mode / indirect mode
 	pin_VCU = pin;
 }
 
 // set ST pin level  : set table address A0-A7 / start speech
-void VLM5030::setST(bool pin, const EmuTime& time)
+void VLM5030Impl::setST(bool pin)
 {
 	if (pin_ST == pin) {
 		// pin level unchanged
@@ -479,7 +565,6 @@ void VLM5030::setST(bool pin, const EmuTime& time)
 				address = (((*rom)[(table + 0) & address_mask]) << 8) |
 				            (*rom)[(table + 1) & address_mask];
 			}
-			updateStream(time);
 			// reset process status
 			sample_count = frame_size;
 			interp_count = FR_SIZE;
@@ -497,9 +582,8 @@ void VLM5030::setST(bool pin, const EmuTime& time)
 	}
 }
 
-VLM5030::VLM5030(MSXMotherBoard& motherBoard, const std::string& name,
-                 const std::string& desc, const XMLElement& config,
-                 const EmuTime& time)
+VLM5030Impl::VLM5030Impl(MSXMotherBoard& motherBoard, const std::string& name,
+                 const std::string& desc, const XMLElement& config)
 	: SoundDevice(motherBoard.getMSXMixer(), name, desc, 1)
 	, Resample(motherBoard.getGlobalSettings(), 1)
 {
@@ -520,7 +604,7 @@ VLM5030::VLM5030(MSXMotherBoard& motherBoard, const std::string& name,
 	pin_RST = pin_ST = pin_VCU = false;
 	latch_data = 0;
 
-	reset(time);
+	reset();
 	phase = PH_IDLE;
 
 	address_mask = rom->getSize() - 1;
@@ -528,12 +612,12 @@ VLM5030::VLM5030(MSXMotherBoard& motherBoard, const std::string& name,
 	registerSound(config);
 }
 
-VLM5030::~VLM5030()
+VLM5030Impl::~VLM5030Impl()
 {
 	unregisterSound();
 }
 
-void VLM5030::setOutputRate(unsigned sampleRate)
+void VLM5030Impl::setOutputRate(unsigned sampleRate)
 {
        const int CLOCK_FREQ = 3579545;
        double input = CLOCK_FREQ / 440.0;
@@ -541,15 +625,89 @@ void VLM5030::setOutputRate(unsigned sampleRate)
        setResampleRatio(input, sampleRate);
 }
 
-bool VLM5030::generateInput(int* buffer, unsigned length)
+bool VLM5030Impl::generateInput(int* buffer, unsigned length)
 {
 	return mixChannels(buffer, length);
 }
 
-bool VLM5030::updateBuffer(unsigned length, int* buffer,
-                           const EmuTime& /*start*/, const EmuDuration& /*sampDur*/)
+bool VLM5030Impl::updateBuffer(unsigned length, int* buffer,
+                           EmuTime::param /*start*/, EmuDuration::param /*sampDur*/)
 {
 	return generateOutput(buffer, length);
 }
 
+template<typename Archive>
+void VLM5030Impl::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.serialize("address_mask", address_mask);
+	ar.serialize("frame_size", frame_size);
+	ar.serialize("pitch_offset", pitch_offset);
+	ar.serialize("current_energy", current_energy);
+	ar.serialize("current_pitch", current_pitch);
+	ar.serialize("current_k", current_k);
+	ar.serialize("x", x);
+	ar.serialize("address", address);
+	ar.serialize("vcu_addr_h", vcu_addr_h);
+	ar.serialize("old_k", old_k);
+	ar.serialize("new_k", new_k);
+	ar.serialize("target_k", target_k);
+	ar.serialize("old_energy", old_energy);
+	ar.serialize("new_energy", new_energy);
+	ar.serialize("target_energy", target_energy);
+	ar.serialize("old_pitch", old_pitch);
+	ar.serialize("new_pitch", new_pitch);
+	ar.serialize("target_pitch", target_pitch);
+	ar.serialize("interp_step", interp_step);
+	ar.serialize("interp_count", interp_count);
+	ar.serialize("sample_count", sample_count);
+	ar.serialize("pitch_count", pitch_count);
+	ar.serialize("latch_data", latch_data);
+	ar.serialize("parameter", parameter);
+	ar.serialize("phase", phase);
+	ar.serialize("pin_BSY", pin_BSY);
+	ar.serialize("pin_ST", pin_ST);
+	ar.serialize("pin_VCU", pin_VCU);
+	ar.serialize("pin_RST", pin_RST);
+}
+
+
+// class VLM5030
+
+VLM5030::VLM5030(MSXMotherBoard& motherBoard, const std::string& name,
+                 const std::string& desc, const XMLElement& config)
+	: pimple(new VLM5030Impl(motherBoard, name, desc, config))
+{
+}
+
+VLM5030::~VLM5030()
+{
+}
+
+void VLM5030::reset()
+{
+	pimple->reset();
+}
+
+void VLM5030::writeData(byte data)
+{
+	pimple->writeData(data);
+}
+
+void VLM5030::writeControl(byte data, EmuTime::param time)
+{
+	pimple->writeControl(data, time);
+}
+
+bool VLM5030::getBSY(EmuTime::param time)
+{
+	return pimple->getBSY(time);
+}
+
+template<typename Archive>
+void VLM5030::serialize(Archive& ar, unsigned version)
+{
+	pimple->serialize(ar, version);
+}
+INSTANTIATE_SERIALIZE_METHODS(VLM5030);
+
 } // namespace openmsx
