@@ -3,7 +3,6 @@
 #include "WavData.hh"
 #include "MSXException.hh"
 #include <SDL.h>
-#include <cstdlib>
 #include <cassert>
 
 using std::string;
@@ -15,14 +14,13 @@ bool is8Bit(Uint16 format)
 	return (format == AUDIO_U8) || (format == AUDIO_S8);
 }
 
-WavData::WavData(const string& filename, int wantedBits, int wantedFreq)
+WavData::WavData(const string& filename, unsigned wantedBits, unsigned wantedFreq)
 {
 	SDL_AudioSpec wavSpec;
 	Uint8* wavBuf;
 	Uint32 wavLen;
 	if (SDL_LoadWAV(filename.c_str(), &wavSpec, &wavBuf, &wavLen) == NULL) {
-		throw MSXException(string("WavData error: ") +
-		                   SDL_GetError());
+		throw MSXException(string("WavData error: ") + SDL_GetError());
 	}
 
 	freq = (wantedFreq == 0) ? wavSpec.freq : wantedFreq;
@@ -38,22 +36,16 @@ WavData::WavData(const string& filename, int wantedBits, int wantedFreq)
 		throw MSXException("Couldn't build wav converter");
 	}
 
-	buffer = malloc(wavLen * audioCVT.len_mult);
-	audioCVT.buf = static_cast<Uint8*>(buffer);
-	audioCVT.len = wavLen;
-	memcpy(buffer, wavBuf, wavLen);
+	buffer.assign(wavBuf, wavBuf + wavLen);
 	SDL_FreeWAV(wavBuf);
+	buffer.resize(wavLen * audioCVT.len_mult); // possibly we need more space
+	audioCVT.buf = &buffer[0];
+	audioCVT.len = wavLen;
 
 	if (SDL_ConvertAudio(&audioCVT) == -1) {
-		free(buffer);
 		throw MSXException("Couldn't convert wav file to internal format");
 	}
-	length = int(audioCVT.len * audioCVT.len_ratio) / 2;
-}
-
-WavData::~WavData()
-{
-	free(buffer);
+	length = unsigned(audioCVT.len * audioCVT.len_ratio) / 2;
 }
 
 unsigned WavData::getFreq() const
@@ -73,7 +65,7 @@ unsigned WavData::getSize() const
 
 const void* WavData::getData() const
 {
-	return buffer;
+	return &buffer[0];
 }
 
 } // namespace openmsx
