@@ -1,4 +1,4 @@
-// $Id: Y8950Adpcm.cc 6933 2007-09-08 21:26:27Z m9710797 $
+// $Id: Y8950Adpcm.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "Y8950Adpcm.hh"
 #include "Y8950.hh"
@@ -6,6 +6,7 @@
 #include "Ram.hh"
 #include "MSXMotherBoard.hh"
 #include "Math.hh"
+#include "serialize.hh"
 #include <cstring>
 
 namespace openmsx {
@@ -16,8 +17,6 @@ static const int ADPCM_VOLUME = 356;
 
 // Bitmask for register 0x07
 static const int R07_RESET        = 0x01;
-//static const int R07            = 0x02;.      // not used
-//static const int R07            = 0x04;.      // not used
 static const int R07_SP_OFF       = 0x08;
 static const int R07_REPEAT       = 0x10;
 static const int R07_MEMORY_DATA  = 0x20;
@@ -25,13 +24,11 @@ static const int R07_REC          = 0x40;
 static const int R07_START        = 0x80;
 static const int R07_MODE         = 0xE0;
 
-//Bitmask for register 0x08
+// Bitmask for register 0x08
 static const int R08_ROM          = 0x01;
 static const int R08_64K          = 0x02;
 static const int R08_DA_AD        = 0x04;
 static const int R08_SAMPL        = 0x08;
-//static const int R08            = 0x10;.      // not used
-//static const int R08            = 0x20;.      // not used
 static const int R08_NOTE_SET     = 0x40;
 static const int R08_CSM          = 0x80;
 
@@ -51,13 +48,16 @@ Y8950Adpcm::Y8950Adpcm(Y8950& y8950_, MSXMotherBoard& motherBoard,
 	, volume(0)
 {
 	memset(&(*ram)[0], 0xFF, ram->getSize());
+
+	// avoid (harmless) UMR in serialize()
+	adpcm_data = 0;
 }
 
 Y8950Adpcm::~Y8950Adpcm()
 {
 }
 
-void Y8950Adpcm::reset(const EmuTime &time)
+void Y8950Adpcm::reset(EmuTime::param time)
 {
 	startAddr = 0;
 	stopAddr = 7;
@@ -92,7 +92,7 @@ void Y8950Adpcm::restart()
 	volumeWStep = (volume * delta) >> STEP_BITS;
 }
 
-void Y8950Adpcm::schedule(const EmuTime& time)
+void Y8950Adpcm::schedule(EmuTime::param time)
 {
 	if ((stopAddr > startAddr) && (delta != 0)) {
 		uint64 samples = stopAddr - memPntr + 1;
@@ -102,7 +102,7 @@ void Y8950Adpcm::schedule(const EmuTime& time)
 	}
 }
 
-void Y8950Adpcm::executeUntil(const EmuTime& time, int /*userData*/)
+void Y8950Adpcm::executeUntil(EmuTime::param time, int /*userData*/)
 {
 	y8950.setStatus(Y8950::STATUS_EOS);
 	if (reg7 & R07_REPEAT) {
@@ -116,7 +116,7 @@ const std::string& Y8950Adpcm::schedName() const
 	return name;
 }
 
-void Y8950Adpcm::writeReg(byte rg, byte data, const EmuTime &time)
+void Y8950Adpcm::writeReg(byte rg, byte data, EmuTime::param time)
 {
 	//PRT_DEBUG("Y8950Adpcm: write "<<(int)rg<<" "<<(int)data);
 	switch (rg) {
@@ -252,14 +252,14 @@ byte Y8950Adpcm::readReg(byte rg)
 byte Y8950Adpcm::peekReg(byte rg) const
 {
 	switch (rg) {
-		case 0x0F: // ADPCM-DATA
-			return peekData();
-		case 0x13: // TODO check
-			return out & 0xFF;
-		case 0x14: // TODO check
-			return out / 256;
-		default:
-			return 255;
+	case 0x0F: // ADPCM-DATA
+		return peekData();
+	case 0x13: // TODO check
+		return out & 0xFF;
+	case 0x14: // TODO check
+		return out >> 8;
+	default:
+		return 255;
 	}
 }
 
@@ -388,4 +388,31 @@ int Y8950Adpcm::calcSample()
 	return output >> 12;
 }
 
+
+template<typename Archive>
+void Y8950Adpcm::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<Schedulable>(*this);
+	ar.serialize("ram", *ram);
+	ar.serialize("startAddr", startAddr);
+	ar.serialize("stopAddr", stopAddr);
+	ar.serialize("addrMask", addrMask);
+	ar.serialize("memPntr", memPntr);
+	ar.serialize("nowStep", nowStep);
+	ar.serialize("volume", volume);
+	ar.serialize("out", out);
+	ar.serialize("output", output);
+	ar.serialize("diff", diff);
+	ar.serialize("nextLeveling", nextLeveling);
+	ar.serialize("sampleStep", sampleStep);
+	ar.serialize("volumeWStep", volumeWStep);
+	ar.serialize("readDelay", readDelay);
+	ar.serialize("delta", delta);
+	ar.serialize("reg7", reg7);
+	ar.serialize("reg15", reg15);
+	ar.serialize("adpcm_data", adpcm_data);
+	ar.serialize("romBank", romBank);
+}
+INSTANTIATE_SERIALIZE_METHODS(Y8950Adpcm);
+
 } // namespace openmsx
