@@ -1,4 +1,4 @@
-// $Id: YMF278.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: YMF278.cc 8357 2008-11-19 20:40:49Z m9710797 $
 
 // Based on ymf278b.c written by R. Belmont and O. Galibert
 
@@ -9,8 +9,10 @@
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
 #include "Clock.hh"
+#include "serialize.hh"
 #include <algorithm>
 #include <cmath>
+#include <cstring>
 
 namespace openmsx {
 
@@ -19,7 +21,7 @@ class DebugRegisters : public SimpleDebuggable
 public:
 	DebugRegisters(YMF278Impl& ymf278, MSXMotherBoard& motherBoard);
 	virtual byte read(unsigned address);
-	virtual void write(unsigned address, byte value, const EmuTime& time);
+	virtual void write(unsigned address, byte value, EmuTime::param time);
 private:
 	YMF278Impl& ymf278;
 };
@@ -46,6 +48,9 @@ public:
 	inline int compute_am();
 	void set_lfo(int newlfo);
 
+	template<typename Archive>
+	void serialize(Archive& ar, unsigned version);
+
 	unsigned startaddr;
 	unsigned loopaddr;
 	unsigned endaddr;
@@ -55,8 +60,6 @@ public:
 	short sample1, sample2;
 
 	int env_vol;
-	unsigned env_vol_step;
-	unsigned env_vol_lim;
 
 	int lfo_cnt;
 	int lfo_step;
@@ -90,26 +93,29 @@ class YMF278Impl : public SoundDevice, private Resample
 {
 public:
 	YMF278Impl(MSXMotherBoard& motherBoard, const std::string& name,
-	       int ramSize, const XMLElement& config, const EmuTime& time);
+	       int ramSize, const XMLElement& config);
 	virtual ~YMF278Impl();
-	void reset(const EmuTime& time);
-	void writeRegOPL4(byte reg, byte data, const EmuTime& time);
-	byte readReg(byte reg, const EmuTime& time);
+	void reset(EmuTime::param time);
+	void writeRegOPL4(byte reg, byte data, EmuTime::param time);
+	byte readReg(byte reg, EmuTime::param time);
 	byte peekReg(byte reg) const;
-	byte readStatus(const EmuTime& time);
-	byte peekStatus(const EmuTime& time) const;
+	byte readStatus(EmuTime::param time);
+	byte peekStatus(EmuTime::param time) const;
+
+	template<typename Archive>
+	void serialize(Archive& ar, unsigned version);
 
 private:
 	// SoundDevice
 	virtual void setOutputRate(unsigned sampleRate);
 	virtual void generateChannels(int** bufs, unsigned num);
 	virtual bool updateBuffer(unsigned length, int* buffer,
-		const EmuTime& time, const EmuDuration& sampDur);
+		EmuTime::param time, EmuDuration::param sampDur);
 
 	// Resample
 	virtual bool generateInput(int* buffer, unsigned num);
 
-	void writeReg(byte reg, byte data, const EmuTime& time);
+	void writeReg(byte reg, byte data, EmuTime::param time);
 	byte readMem(unsigned address) const;
 	void writeMem(unsigned address, byte value);
 	short getSample(YMF278Slot& op);
@@ -117,6 +123,7 @@ private:
 	bool anyActive();
 	void keyOnHelper(YMF278Slot& slot);
 
+	MSXMotherBoard& motherBoard;
 	friend class DebugRegisters;
 	friend class DebugMemory;
 	const std::auto_ptr<DebugRegisters> debugRegisters;
@@ -176,14 +183,14 @@ const EmuDuration YMF278Impl::MEM_WRITE_DELAY = MasterClock::duration(28);
 // 10000 ticks is approx 300us; exact delay is unknown.
 const EmuDuration YMF278Impl::LOAD_DELAY = MasterClock::duration(10000);
 
-const int EG_SH = 16;	// 16.16 fixed point (EG timing)
+const int EG_SH = 16; // 16.16 fixed point (EG timing)
 const unsigned EG_TIMER_OVERFLOW = 1 << EG_SH;
 
 // envelope output entries
 const int ENV_BITS      = 10;
 const int ENV_LEN       = 1 << ENV_BITS;
 const double ENV_STEP   = 128.0 / ENV_LEN;
-const int MAX_ATT_INDEX = (1 << (ENV_BITS - 1)) - 1; //511
+const int MAX_ATT_INDEX = (1 << (ENV_BITS - 1)) - 1; // 511
 const int MIN_ATT_INDEX = 0;
 
 // Envelope Generator phases
@@ -193,8 +200,8 @@ const int EG_SUS = 2;
 const int EG_REL = 1;
 const int EG_OFF = 0;
 
-const int EG_REV = 5;	//pseudo reverb
-const int EG_DMP = 6;	//damp
+const int EG_REV = 5; // pseudo reverb
+const int EG_DMP = 6; // damp
 
 // Pan values, units are -3dB, i.e. 8.
 const int pan_left[16]  = {
@@ -220,7 +227,7 @@ const unsigned dl_tab[16] = {
 
 const byte RATE_STEPS = 8;
 const byte eg_inc[15 * RATE_STEPS] = {
-//cycle:0 1  2 3  4 5  6 7
+//cycle:0  1   2  3   4  5   6  7
 	0, 1,  0, 1,  0, 1,  0, 1, //  0  rates 00..12 0 (increment by 0 or 1)
 	0, 1,  0, 1,  1, 1,  0, 1, //  1  rates 00..12 1
 	0, 1,  1, 1,  0, 1,  1, 1, //  2  rates 00..12 2
@@ -262,9 +269,9 @@ const byte eg_rate_select[64] = {
 };
 #undef O
 
-//rate  0,    1,    2,    3,   4,   5,   6,  7,  8,  9,  10, 11, 12, 13, 14, 15
-//shift 12,   11,   10,   9,   8,   7,   6,  5,  4,  3,  2,  1,  0,  0,  0,  0
-//mask  4095, 2047, 1023, 511, 255, 127, 63, 31, 15, 7,  3,  1,  0,  0,  0,  0
+// rate  0,    1,    2,    3,   4,   5,   6,  7,  8,  9,  10, 11, 12, 13, 14, 15
+// shift 12,   11,   10,   9,   8,   7,   6,  5,  4,  3,  2,  1,  0,  0,  0,  0
+// mask  4095, 2047, 1023, 511, 255, 127, 63, 31, 15, 7,  3,  1,  0,  0,  0,  0
 #define O(a) (a)
 const byte eg_rate_shift[64] = {
 	O(12),O(12),O(12),O(12),
@@ -287,8 +294,8 @@ const byte eg_rate_shift[64] = {
 #undef O
 
 
-//number of steps to take in quarter of lfo frequency
-//TODO check if frequency matches real chip
+// number of steps to take in quarter of lfo frequency
+// TODO check if frequency matches real chip
 #define O(a) int((EG_TIMER_OVERFLOW / a) / 6)
 const int lfo_period[8] = {
 	O(0.168), O(2.019), O(3.196), O(4.206),
@@ -325,7 +332,6 @@ void YMF278Slot::reset()
 	step = stepptr = 0;
 	bits = startaddr = loopaddr = endaddr = 0;
 	env_vol = MAX_ATT_INDEX;
-	//env_vol_step = env_vol_lim = 0;
 
 	lfo_active = false;
 	lfo_cnt = lfo_step = 0;
@@ -333,6 +339,9 @@ void YMF278Slot::reset()
 
 	state = EG_OFF;
 	active = false;
+
+	// not strictly needed, but avoid UMR on savestate
+	pos = sample1 = sample2 = 0;
 }
 
 int YMF278Slot::compute_rate(int val)
@@ -389,7 +398,7 @@ void YMF278Impl::advance()
 {
 	eg_cnt++;
 	for (int i = 0; i < 24; i++) {
-		YMF278Slot &op = slots[i];
+		YMF278Slot& op = slots[i];
 
 		if (op.lfo_active) {
 			op.lfo_cnt++;
@@ -407,7 +416,7 @@ void YMF278Impl::advance()
 
 		// Envelope Generator
 		switch(op.state) {
-		case EG_ATT: {	// attack phase
+		case EG_ATT: { // attack phase
 			byte rate = op.compute_rate(op.AR);
 			if (rate < 4) {
 				break;
@@ -427,7 +436,7 @@ void YMF278Impl::advance()
 			}
 			break;
 		}
-		case EG_DEC: {	// decay phase
+		case EG_DEC: { // decay phase
 			byte rate = op.compute_rate(op.D1R);
 			if (rate < 4) {
 				break;
@@ -447,7 +456,7 @@ void YMF278Impl::advance()
 			}
 			break;
 		}
-		case EG_SUS: {	// sustain phase
+		case EG_SUS: { // sustain phase
 			byte rate = op.compute_rate(op.D2R);
 			if (rate < 4) {
 				break;
@@ -468,7 +477,7 @@ void YMF278Impl::advance()
 			}
 			break;
 		}
-		case EG_REL: {	// release phase
+		case EG_REL: { // release phase
 			byte rate = op.compute_rate(op.RR);
 			if (rate < 4) {
 				break;
@@ -489,8 +498,8 @@ void YMF278Impl::advance()
 			}
 			break;
 		}
-		case EG_REV: {	//pseudo reverb
-			//TODO improve env_vol update
+		case EG_REV: { // pseudo reverb
+			// TODO improve env_vol update
 			byte rate = op.compute_rate(5);
 			//if (rate < 4) {
 			//	break;
@@ -507,8 +516,8 @@ void YMF278Impl::advance()
 			}
 			break;
 		}
-		case EG_DMP: {	//damping
-			//TODO improve env_vol update, damp is just fastest decay now
+		case EG_DMP: { // damping
+			// TODO improve env_vol update, damp is just fastest decay now
 			byte rate = 56;
 			byte shift = eg_rate_shift[rate];
 			if (!(eg_cnt & ((1 << shift) - 1))) {
@@ -533,7 +542,7 @@ void YMF278Impl::advance()
 	}
 }
 
-short YMF278Impl::getSample(YMF278Slot &op)
+short YMF278Impl::getSample(YMF278Slot& op)
 {
 	short sample;
 	switch (op.bits) {
@@ -544,7 +553,7 @@ short YMF278Impl::getSample(YMF278Slot &op)
 	}
 	case 1: {
 		// 12 bit
-		int addr = op.startaddr + ((op.pos / 2) * 3);
+		unsigned addr = op.startaddr + ((op.pos / 2) * 3);
 		if (op.pos & 1) {
 			sample = readMem(addr + 2) << 8 |
 				 ((readMem(addr + 1) << 4) & 0xF0);
@@ -556,7 +565,7 @@ short YMF278Impl::getSample(YMF278Slot &op)
 	}
 	case 2: {
 		// 16 bit
-		int addr = op.startaddr + (op.pos * 2);
+		unsigned addr = op.startaddr + (op.pos * 2);
 		sample = (readMem(addr + 0) << 8) |
 			 (readMem(addr + 1));
 		break;
@@ -595,8 +604,8 @@ void YMF278Impl::generateChannels(int** bufs, unsigned num)
 		for (int i = 0; i < 24; i++) {
 			YMF278Slot& sl = slots[i];
 			if (!sl.active) {
-				bufs[i][2 * j + 0] = 0;
-				bufs[i][2 * j + 1] = 0;
+				//bufs[i][2 * j + 0] += 0;
+				//bufs[i][2 * j + 1] += 0;
 				continue;
 			}
 
@@ -610,8 +619,8 @@ void YMF278Impl::generateChannels(int** bufs, unsigned num)
 			volLeft  = std::max(0, volLeft);
 			volRight = std::max(0, volRight);
 
-			bufs[i][2 * j + 0] = (sample * volume[volLeft] ) >> 14;
-			bufs[i][2 * j + 1] = (sample * volume[volRight]) >> 14;
+			bufs[i][2 * j + 0] += (sample * volume[volLeft] ) >> 14;
+			bufs[i][2 * j + 1] += (sample * volume[volRight]) >> 14;
 
 			if (sl.lfo_active && sl.vib) {
 				int oct = sl.OCT;
@@ -647,7 +656,7 @@ bool YMF278Impl::generateInput(int* buffer, unsigned num)
 }
 
 bool YMF278Impl::updateBuffer(unsigned length, int* buffer,
-     const EmuTime& /*time*/, const EmuDuration& /*sampDur*/)
+     EmuTime::param /*time*/, EmuDuration::param /*sampDur*/)
 {
 	return generateOutput(buffer, length);
 }
@@ -673,13 +682,13 @@ void YMF278Impl::keyOnHelper(YMF278Slot& slot)
 	slot.sample2 = getSample(slot);
 }
 
-void YMF278Impl::writeRegOPL4(byte reg, byte data, const EmuTime& time)
+void YMF278Impl::writeRegOPL4(byte reg, byte data, EmuTime::param time)
 {
 	busyTime = time + REG_WRITE_DELAY;
 	writeReg(reg, data, time);
 }
 
-void YMF278Impl::writeReg(byte reg, byte data, const EmuTime& time)
+void YMF278Impl::writeReg(byte reg, byte data, EmuTime::param time)
 {
 	updateStream(time); // TODO optimize only for regs that directly influence sound
 	// Handle slot registers specifically
@@ -779,18 +788,18 @@ void YMF278Impl::writeReg(byte reg, byte data, const EmuTime& time)
 			}
 
 			switch (data >> 6) {
-			case 0:	//tone off, no damp
+			case 0: // tone off, no damp
 				if (slot.active && (slot.state != EG_REV) ) {
 					slot.state = EG_REL;
 				}
 				break;
-			case 2:	//tone on, no damp
+			case 2: // tone on, no damp
 				if (!(regs[reg] & 0x080)) {
 					keyOnHelper(slot);
 				}
 				break;
-			case 1:	//tone off, damp
-			case 3:	//tone on,  damp
+			case 1: // tone off, damp
+			case 3: // tone on,  damp
 				slot.state = EG_DMP;
 				break;
 			}
@@ -818,7 +827,7 @@ void YMF278Impl::writeReg(byte reg, byte data, const EmuTime& time)
 	} else {
 		// All non-slot registers
 		switch (reg) {
-		case 0x00:    	// TEST
+		case 0x00: // TEST
 		case 0x01:
 			break;
 
@@ -861,7 +870,7 @@ void YMF278Impl::writeReg(byte reg, byte data, const EmuTime& time)
 	regs[reg] = data;
 }
 
-byte YMF278Impl::readReg(byte reg, const EmuTime& time)
+byte YMF278Impl::readReg(byte reg, EmuTime::param time)
 {
 	// no need to call updateStream(time)
 	byte result;
@@ -902,13 +911,13 @@ byte YMF278Impl::peekReg(byte reg) const
 	return result;
 }
 
-byte YMF278Impl::readStatus(const EmuTime& time)
+byte YMF278Impl::readStatus(EmuTime::param time)
 {
 	// no need to call updateStream(time)
 	return peekStatus(time);
 }
 
-byte YMF278Impl::peekStatus(const EmuTime& time) const
+byte YMF278Impl::peekStatus(EmuTime::param time) const
 {
 	byte result = 0;
 	if (time < busyTime) result |= 0x01;
@@ -916,25 +925,28 @@ byte YMF278Impl::peekStatus(const EmuTime& time) const
 	return result;
 }
 
-YMF278Impl::YMF278Impl(MSXMotherBoard& motherBoard, const std::string& name,
-                       int ramSize, const XMLElement& config, const EmuTime& time)
-	: SoundDevice(motherBoard.getMSXMixer(), name, "MoonSound wave-part",
+YMF278Impl::YMF278Impl(MSXMotherBoard& motherBoard_, const std::string& name,
+                       int ramSize, const XMLElement& config)
+	: SoundDevice(motherBoard_.getMSXMixer(), name, "MoonSound wave-part",
 	              24, true)
-	, Resample(motherBoard.getGlobalSettings(), 2)
+	, Resample(motherBoard_.getGlobalSettings(), 2)
+	, motherBoard(motherBoard_)
 	, debugRegisters(new DebugRegisters(*this, motherBoard))
 	, debugMemory   (new DebugMemory   (*this, motherBoard))
-	, loadTime(time), busyTime(time)
+	, loadTime(motherBoard.getCurrentTime())
+	, busyTime(motherBoard.getCurrentTime())
 	, rom(new Rom(motherBoard, name + " ROM", "rom", config))
 {
-	memadr = 0;	// avoid UMR
+	memadr = 0; // avoid UMR
 	setOutputRate(44100);	// make valgrind happy
 
 	endRom = rom->getSize();
-	ramSize *= 1024;	// in kb
+	ramSize *= 1024; // in kb
 	ram = new byte[ramSize];
 	endRam = endRom + ramSize;
+	memset(ram, 0, ramSize);
 
-	reset(time);
+	reset(motherBoard.getCurrentTime());
 	registerSound(config);
 
 	// Volume table, 1 = -0.375dB, 8 = -3dB, 256 = -96dB
@@ -952,7 +964,7 @@ YMF278Impl::~YMF278Impl()
 	delete[] ram;
 }
 
-void YMF278Impl::reset(const EmuTime& time)
+void YMF278Impl::reset(EmuTime::param time)
 {
 	eg_cnt   = 0;
 
@@ -981,7 +993,7 @@ byte YMF278Impl::readMem(unsigned address) const
 	} else if (address < endRam) {
 		return ram[address - endRom];
 	} else {
-		return 255;	// TODO check
+		return 255; // TODO check
 	}
 }
 
@@ -997,6 +1009,76 @@ void YMF278Impl::writeMem(unsigned address, byte value)
 }
 
 
+template<typename Archive>
+void YMF278Slot::serialize(Archive& ar, unsigned /*version*/)
+{
+	// TODO restore more state from registers
+	ar.serialize("startaddr", startaddr);
+	ar.serialize("loopaddr", loopaddr);
+	ar.serialize("endaddr", endaddr);
+	ar.serialize("step", step);
+	ar.serialize("stepptr", stepptr);
+	ar.serialize("pos", pos);
+	ar.serialize("sample1", sample1);
+	ar.serialize("sample2", sample2);
+	ar.serialize("env_vol", env_vol);
+	ar.serialize("lfo_cnt", lfo_cnt);
+	ar.serialize("lfo_step", lfo_step);
+	ar.serialize("lfo_max", lfo_max);
+	ar.serialize("DL", DL);
+	ar.serialize("wave", wave);
+	ar.serialize("FN", FN);
+	ar.serialize("OCT", OCT);
+	ar.serialize("PRVB", PRVB);
+	ar.serialize("LD", LD);
+	ar.serialize("TL", TL);
+	ar.serialize("pan", pan);
+	ar.serialize("lfo", lfo);
+	ar.serialize("vib", vib);
+	ar.serialize("AM", AM);
+	ar.serialize("AR", AR);
+	ar.serialize("D1R", D1R);
+	ar.serialize("D2R", D2R);
+	ar.serialize("RC", RC);
+	ar.serialize("RR", RR);
+	ar.serialize("bits", bits);
+	ar.serialize("active", active);
+	ar.serialize("state", state);
+	ar.serialize("lfo_active", lfo_active);
+
+	// Older version also had "env_vol_step" and "env_vol_lim"
+	// but those members were nowhere used, so removed those
+	// in the current version (it's ok to remove members from the
+	// savestate without updating the version number).
+}
+
+template<typename Archive>
+void YMF278Impl::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.serialize("loadTime", loadTime);
+	ar.serialize("busyTime", busyTime);
+	ar.serialize("slots", slots);
+	ar.serialize("eg_cnt", eg_cnt);
+	ar.serialize_blob("ram", ram, endRam - endRom);
+	ar.serialize_blob("registers", regs, sizeof(regs));
+
+	// TODO restore more state from registers
+	static const byte rewriteRegs[] = {
+		2,       // wavetblhdr, memmode
+		3, 4, 5, // memadr
+		0xf8,    // fm_l, fm_r
+		0xf9,    // pcm_l, pcm_r
+	};
+	if (ar.isLoader()) {
+		EmuTime::param time = motherBoard.getCurrentTime();
+		for (unsigned i = 0; i < sizeof(rewriteRegs); ++i) {
+			byte reg = rewriteRegs[i];
+			writeReg(reg, regs[reg], time);
+		}
+	}
+}
+
+
 // class DebugRegisters
 
 DebugRegisters::DebugRegisters(YMF278Impl& ymf278_,
@@ -1012,7 +1094,7 @@ byte DebugRegisters::read(unsigned address)
 	return ymf278.peekReg(address);
 }
 
-void DebugRegisters::write(unsigned address, byte value, const EmuTime& time)
+void DebugRegisters::write(unsigned address, byte value, EmuTime::param time)
 {
 	ymf278.writeReg(address, value, time);
 }
@@ -1041,8 +1123,8 @@ void DebugMemory::write(unsigned address, byte value)
 // class YMF278
 
 YMF278::YMF278(MSXMotherBoard& motherBoard, const std::string& name,
-       int ramSize, const XMLElement& config, const EmuTime& time)
-	: pimple(new YMF278Impl(motherBoard, name, ramSize, config, time))
+               int ramSize, const XMLElement& config)
+	: pimple(new YMF278Impl(motherBoard, name, ramSize, config))
 {
 }
 
@@ -1050,17 +1132,17 @@ YMF278::~YMF278()
 {
 }
 
-void YMF278::reset(const EmuTime& time)
+void YMF278::reset(EmuTime::param time)
 {
 	pimple->reset(time);
 }
 
-void YMF278::writeRegOPL4(byte reg, byte data, const EmuTime& time)
+void YMF278::writeRegOPL4(byte reg, byte data, EmuTime::param time)
 {
 	pimple->writeRegOPL4(reg, data, time);
 }
 
-byte YMF278::readReg(byte reg, const EmuTime& time)
+byte YMF278::readReg(byte reg, EmuTime::param time)
 {
 	return pimple->readReg(reg, time);
 }
@@ -1070,14 +1152,21 @@ byte YMF278::peekReg(byte reg) const
 	return pimple->peekReg(reg);
 }
 
-byte YMF278::readStatus(const EmuTime& time)
+byte YMF278::readStatus(EmuTime::param time)
 {
 	return pimple->readStatus(time);
 }
 
-byte YMF278::peekStatus(const EmuTime& time) const
+byte YMF278::peekStatus(EmuTime::param time) const
 {
 	return pimple->peekStatus(time);
 }
 
+template<typename Archive>
+void YMF278::serialize(Archive& ar, unsigned version)
+{
+	pimple->serialize(ar, version);
+}
+INSTANTIATE_SERIALIZE_METHODS(YMF278);
+
 } // namespace openmsx
