@@ -1,10 +1,10 @@
-// $Id: Timer.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: Timer.cc 8205 2008-09-08 07:49:38Z m9710797 $
 
 #include "Timer.hh"
 #include "probed_defs.hh"
 #ifdef HAVE_GETTIMEOFDAY
 #include <sys/time.h>
-#include <time.h>
+#include <ctime>
 #endif
 #ifdef HAVE_USLEEP
 #include <unistd.h>
