@@ -1,4 +1,4 @@
-//$Id: StringOp.cc 6672 2007-06-27 14:31:15Z m9710797 $
+// $Id: StringOp.cc 8351 2008-11-12 18:20:29Z m9710797 $
 
 #include <cstdlib>
 #include <algorithm>
@@ -10,8 +10,6 @@ using std::string;
 using std::transform;
 using std::vector;
 
-namespace openmsx {
-
 namespace StringOp {
 
 int stringToInt(const string& str)
@@ -19,7 +17,12 @@ int stringToInt(const string& str)
 	return strtol(str.c_str(), NULL, 0);
 }
 
-uint64 stringToUint64(const string& str)
+unsigned stringToUint(const string& str)
+{
+	return strtoul(str.c_str(), NULL, 0);
+}
+
+unsigned long long stringToUint64(const string& str)
 {
        return strtoull(str.c_str(), NULL, 0);
 }
@@ -107,5 +110,3 @@ void split(const string& str, const string& chars, vector<string>& result)
 }
 
 } // namespace StringOp
-
-} // namespace openmsx
