@@ -1,4 +1,4 @@
-// $Id: AviRecorder.cc 6689 2007-07-01 15:58:02Z mthuurne $
+// $Id: AviRecorder.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "AviRecorder.hh"
 #include "AviWriter.hh"
@@ -12,8 +12,8 @@
 #include "VideoSourceSetting.hh"
 #include "PostProcessor.hh"
 #include "MSXMixer.hh"
-#include "Scheduler.hh"
-#include "GlobalCliComm.hh"
+#include "Filename.hh"
+#include "CliComm.hh"
 #include "FileOperations.hh"
 #include "StringOp.hh"
 #include <cassert>
@@ -41,9 +41,9 @@ AviRecorder::AviRecorder(Reactor& reactor_)
 	, postProcessor1(0)
 	, postProcessor2(0)
 	, mixer(0)
-	, scheduler(0)
 	, duration(EmuDuration::infinity)
 	, prevTime(EmuTime::infinity)
+	, frameHeight(0)
 {
 }
 
@@ -54,7 +54,7 @@ AviRecorder::~AviRecorder()
 }
 
 void AviRecorder::start(bool recordAudio, bool recordVideo,
-                        const string& filename)
+                        const Filename& filename)
 {
 	stop();
 	MSXMotherBoard* motherBoard = reactor.getMotherBoard();
@@ -68,7 +68,6 @@ void AviRecorder::start(bool recordAudio, bool recordVideo,
 	}
 	if (recordVideo) {
 		Display& display = reactor.getDisplay();
-		scheduler = &motherBoard->getScheduler();
 		VideoSourceSetting* videoSource =
 			&display.getRenderSettings().getVideoSource();
 		Layer* layer1 = display.findLayer("V99x8 PostProcessor");
@@ -86,8 +85,13 @@ void AviRecorder::start(bool recordAudio, bool recordVideo,
 		duration = EmuDuration::infinity;
 		prevTime = EmuTime::infinity;
 
-		aviWriter.reset(new AviWriter(filename, 320, 240, bpp,
-		                              sampleRate));
+		try {
+			aviWriter.reset(new AviWriter(filename, frameWidth, frameHeight,
+						      bpp, sampleRate));
+		} catch (MSXException& e) {
+			throw CommandException("Can't start recording: " +
+			                       e.getMessage());
+		}
 	} else {
 		assert(recordAudio);
 		wavWriter.reset(new WavWriter(filename, 2, 16, sampleRate));
@@ -118,7 +122,6 @@ void AviRecorder::stop()
 		mixer->setRecorder(0);
 		mixer = 0;
 	}
-	scheduler = 0;
 	sampleRate = 0;
 	aviWriter.reset();
 	wavWriter.reset();
@@ -128,7 +131,7 @@ void AviRecorder::addWave(unsigned num, short* data)
 {
 	if (!warnedSampleRate && (mixer->getSampleRate() != sampleRate)) {
 		warnedSampleRate = true;
-		reactor.getGlobalCliComm().printWarning(
+		reactor.getCliComm().printWarning(
 			"Detected audio sample frequency change during "
 			"avi recording. Audio/video might get out of sync "
 			"because of this.");
@@ -141,13 +144,13 @@ void AviRecorder::addWave(unsigned num, short* data)
 	}
 }
 
-void AviRecorder::addImage(const void** lines, const EmuTime& time)
+void AviRecorder::addImage(const void** lines, EmuTime::param time)
 {
 	assert(!wavWriter.get());
 	if (duration != EmuDuration::infinity) {
 		if (!warnedFps && ((time - prevTime) != duration)) {
 			warnedFps = true;
-			reactor.getGlobalCliComm().printWarning(
+			reactor.getCliComm().printWarning(
 				"Detected frame rate change (PAL/NTSC or frameskip) "
 				"during avi recording. Audio/video might get out of "
 				"sync because of this.");
@@ -159,12 +162,18 @@ void AviRecorder::addImage(const void** lines, const EmuTime& time)
 	prevTime = time;
 
 	if (mixer) {
-		mixer->updateStream(scheduler->getCurrentTime());
+		MSXMotherBoard* motherBoard = reactor.getMotherBoard();
+		assert(motherBoard);
+		mixer->updateStream(motherBoard->getCurrentTime());
 	}
 	aviWriter->addFrame(lines, audioBuf.size() / 2, &audioBuf[0]);
 	audioBuf.clear();
 }
 
+unsigned AviRecorder::getFrameHeight() const {
+	assert (frameHeight != 0); // someone uses the getter too early?
+	return frameHeight;
+}
 
 string AviRecorder::processStart(const vector<string>& tokens)
 {
@@ -172,6 +181,8 @@ string AviRecorder::processStart(const vector<string>& tokens)
 	string prefix = "openmsx";
 	bool recordAudio = true;
 	bool recordVideo = true;
+	frameWidth = 320;
+	frameHeight = 240;
 
 	vector<string> arguments;
 	for (unsigned i = 2; i < tokens.size(); ++i) {
@@ -190,8 +201,11 @@ string AviRecorder::processStart(const vector<string>& tokens)
 				recordVideo = false;
 			} else if (tokens[i] == "-videoonly") {
 				recordAudio = false;
+			} else if (tokens[i] == "-doublesize") {
+				frameWidth = 640;
+				frameHeight = 480;
 			} else {
-				throw CommandException("Invalid option");
+				throw CommandException("Invalid option: " + tokens[i]);
 			}
 		} else {
 			arguments.push_back(tokens[i]);
@@ -221,7 +235,7 @@ string AviRecorder::processStart(const vector<string>& tokens)
 	if (aviWriter.get() || wavWriter.get()) {
 		return "Already recording.";
 	}
-	start(recordAudio, recordVideo, filename);
+	start(recordAudio, recordVideo, Filename(filename));
 	return "Recording to " + filename;
 }
 
@@ -279,7 +293,10 @@ string RecordCommand::help(const vector<string>& /*tokens*/) const
 	       "record stop               Stop recording\n"
 	       "record toggle             Toggle recording (useful as keybinding)\n"
 	       "\n"
-	       "The start subcommand also accepts an optional -audioonly or -videoonly flag.\n";
+	       "The start subcommand also accepts an optional -audioonly, -videoonly and "
+	       "a -doublesize flag.\n"
+	       "Videos are recorded in a 320x240 size by default and at 640x480 when the "
+		   "-doublesize flag is used.";
 }
 
 void RecordCommand::tabCompletion(vector<string>& tokens) const
@@ -289,8 +306,9 @@ void RecordCommand::tabCompletion(vector<string>& tokens) const
 		std::set<string> cmds(str, str + 3);
 		completeString(tokens, cmds);
 	} else if ((tokens.size() >= 3) && (tokens[1] == "start")) {
-		const char* const str[3] = { "-prefix", "-videoonly", "-audioonly" };
-		std::set<string> cmds(str, str + 3);
+		const char* const str[4] = { "-prefix", "-videoonly", "-audioonly",
+						"-doublesize"};
+		std::set<string> cmds(str, str + 4);
 		completeString(tokens, cmds);
 	}
 }
