@@ -1,15 +1,17 @@
-// $Id: AviWriter.cc 6807 2007-08-06 17:08:53Z m9710797 $
+// $Id: AviWriter.cc 8232 2008-09-16 07:36:50Z m9710797 $
 
 // Code based on DOSBox-0.65
 
 #include "AviWriter.hh"
 #include "ZMBVEncoder.hh"
-#include "CommandException.hh"
+#include "File.hh"
+#include "MSXException.hh"
 #include "build-info.hh"
 #include "Version.hh"
 #include <cstring>
 #include <cstdlib>
 #include <ctime>
+#include <cstdio> // for snprintf
 #include <cassert>
 
 namespace openmsx {
@@ -29,23 +31,19 @@ static inline void writeLE4(unsigned char* p, unsigned x)
 	p[3] = (x >> 24) & 255;
 }
 
-AviWriter::AviWriter(const std::string& filename, unsigned width_,
+AviWriter::AviWriter(const Filename& filename, unsigned width_,
                      unsigned height_, unsigned bpp, unsigned freq_)
-	: fps(50.0)
+	: file(new File(filename, "wb"))
+	, codec(new ZMBVEncoder(width_, height_, bpp))
+	, fps(50.0)
 	, width(width_)
 	, height(height_)
 	, audiorate(freq_)
 {
-	file = fopen(filename.c_str(), "wb");
-	if (!file) {
-		throw CommandException("Couldn't open " + filename +
-		                       " for writing.");
-	}
 	char dummy[AVI_HEADER_SIZE];
 	memset(dummy, 0, AVI_HEADER_SIZE);
-	fwrite(dummy, 1, AVI_HEADER_SIZE, file);
+	file->write(dummy, AVI_HEADER_SIZE);
 
-	codec.reset(new ZMBVEncoder(width, height, bpp));
 	index.resize(8);
 
 	frames = 0;
@@ -172,10 +170,10 @@ AviWriter::~AviWriter()
 		+ (4 + 4 + ((strlen(dateStr   ) + 1 + 1) & ~1)) // 2nd chunk
 		); // size of the list
 	AVIOUT4("INFO");
-        AVIOUT4("ISFT");
+	AVIOUT4("ISFT");
 	AVIOUTd(strlen(versionStr) + 1); // # of bytes to follow
 	AVIOUTs(versionStr);
-        AVIOUT4("ICRD");
+	AVIOUT4("ICRD");
 	AVIOUTd(strlen(dateStr) + 1); // # of bytes to follow
 	AVIOUTs(dateStr);
 	// TODO: add artist (IART), comments (ICMT), name (INAM), etc.
@@ -192,19 +190,21 @@ AviWriter::~AviWriter()
 	header_pos = main_list;
 	AVIOUTd(nmain);
 	header_pos = AVI_HEADER_SIZE - 12;
-	
+
 	AVIOUT4("LIST");
 	AVIOUTd(written + 4);               // Length of list in bytes
 	AVIOUT4("movi");
 
-	// First add the index table to the end
-	memcpy(&index[0], "idx1", 4);
-	writeLE4(&index[4], index.size() - 8);
-	fwrite(&index[0], 1, index.size(), file);
-
-	fseek(file, 0, SEEK_SET);
-	fwrite(&avi_header, 1, AVI_HEADER_SIZE, file);
-	fclose(file);
+	try {
+		// First add the index table to the end
+		memcpy(&index[0], "idx1", 4);
+		writeLE4(&index[4], index.size() - 8);
+		file->write(&index[0], index.size());
+		file->seek(0);
+		file->write(&avi_header, AVI_HEADER_SIZE);
+	} catch (MSXException& e) {
+		// can't throw from destructor
+	}
 }
 
 void AviWriter::setFps(double fps_)
@@ -220,10 +220,10 @@ void AviWriter::addAviChunk(const char* tag, unsigned size, void* data, unsigned
 	chunk[2] = tag[2];
 	chunk[3] = tag[3];
 	writeLE4(&chunk[4], size);
-	fwrite(chunk, 1, 8, file);
+	file->write(chunk, 8);
 
 	unsigned writesize = (size + 1) & ~1;
-	fwrite(data, 1, writesize, file);
+	file->write(data, writesize);
 	unsigned pos = written + 4;
 	written += writesize + 8;
 
