@@ -1,4 +1,4 @@
-// $Id: FBPostProcessor.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: FBPostProcessor.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "FBPostProcessor.hh"
 #include "RenderSettings.hh"
@@ -10,6 +10,7 @@
 #include "VisibleSurface.hh"
 #include "HostCPU.hh"
 #include "Math.hh"
+#include "build-info.hh"
 #include <algorithm>
 #include <cassert>
 
@@ -230,9 +231,9 @@ void FBPostProcessor<Pixel>::drawNoise()
 
 	unsigned height = screen.getHeight();
 	unsigned width = screen.getWidth();
+	screen.lock();
 	for (unsigned y = 0; y < height; ++y) {
-		Pixel* dummy = 0;
-		Pixel* buf = screen.getLinePtr(y, dummy);
+		Pixel* buf = screen.getLinePtrDirect<Pixel>(y);
 		drawNoiseLine(buf, buf, &noiseBuf[noiseShift[y]], width);
 	}
 }
@@ -249,13 +250,13 @@ void FBPostProcessor<Pixel>::update(const Setting& setting)
 
 
 template <class Pixel>
-FBPostProcessor<Pixel>::FBPostProcessor(CommandController& commandController,
+FBPostProcessor<Pixel>::FBPostProcessor(MSXMotherBoard& motherBoard,
 	Display& display, VisibleSurface& screen_, VideoSource videoSource,
 	unsigned maxWidth, unsigned height)
 	: PostProcessor(
-		commandController, display, screen_, videoSource, maxWidth, height)
+		motherBoard, display, screen_, videoSource, maxWidth, height)
 	, noiseShift(screen.getHeight())
-	, pixelOps(screen.getFormat())
+	, pixelOps(screen.getSDLFormat())
 {
 	scaleAlgorithm = static_cast<RenderSettings::ScaleAlgorithm>(-1); // not a valid scaler
 	scaleFactor = unsigned(-1);
@@ -286,7 +287,7 @@ void FBPostProcessor<Pixel>::paint()
 		scaleAlgorithm = algo;
 		scaleFactor = factor;
 		currScaler = ScalerFactory<Pixel>::createScaler(
-			PixelOperations<Pixel>(screen.getFormat()), renderSettings);
+			PixelOperations<Pixel>(screen.getSDLFormat()), renderSettings);
 	}
 
 	// Scale image.
@@ -339,7 +340,7 @@ void FBPostProcessor<Pixel>::paint()
 template <class Pixel>
 RawFrame* FBPostProcessor<Pixel>::rotateFrames(
 	RawFrame* finishedFrame, FrameSource::FieldType field,
-	const EmuTime& time)
+	EmuTime::param time)
 {
 	for (unsigned y = 0; y < screen.getHeight(); ++y) {
 		noiseShift[y] = rand() & (NOISE_SHIFT - 1) & ~15;
@@ -350,7 +351,11 @@ RawFrame* FBPostProcessor<Pixel>::rotateFrames(
 
 
 // Force template instantiation.
+#if HAVE_16BPP
 template class FBPostProcessor<word>;
+#endif
+#if HAVE_32BPP
 template class FBPostProcessor<unsigned>;
+#endif
 
 } // namespace openmsx
