@@ -1,4 +1,4 @@
-// $Id: GLPostProcessor.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: GLPostProcessor.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "GLPostProcessor.hh"
 #include "GLScaler.hh"
@@ -16,11 +16,13 @@
 namespace openmsx {
 
 GLPostProcessor::GLPostProcessor(
-	CommandController& commandController, Display& display,
+	MSXMotherBoard& motherBoard, Display& display,
 	VisibleSurface& screen, VideoSource videoSource,
 	unsigned maxWidth, unsigned height_)
-	: PostProcessor(commandController, display, screen, videoSource,
-	                maxWidth, height_)
+	: PostProcessor(motherBoard, display, screen,
+	                videoSource, maxWidth, height_)
+	, noiseTextureA(256, 256)
+	, noiseTextureB(256, 256)
 	, height(height_)
 {
 	if (!GLEW_VERSION_2_0) {
@@ -40,8 +42,6 @@ GLPostProcessor::GLPostProcessor(
 
 	scaleAlgorithm = static_cast<RenderSettings::ScaleAlgorithm>(-1); // not a valid scaler
 
-	noiseTextureA.setImage(256, 256);
-	noiseTextureB.setImage(256, 256);
 	frameCounter = 0;
 	noiseX = 0.0;
 	noiseY = 0.0;
@@ -65,81 +65,16 @@ GLPostProcessor::GLPostProcessor(
 		fbo[i].reset(new FrameBufferObject(*colorTex[i]));
 	}
 
-	// generate display list for 3d deform
-	static const int GRID_SIZE = 16;
-	struct Point {
-		GLfloat vx, vy, vz;
-		GLfloat nx, ny, nz;
-		GLfloat tx, ty;
-	} points[GRID_SIZE + 1][GRID_SIZE + 1];
-	const int GRID_SIZE2 = GRID_SIZE / 2;
-	GLfloat s = 284.0f / 320.0f;
-	GLfloat b = (320.0f - 284.0f) / (2.0f * 320.0f);
-
-	for (int sx = 0; sx <= GRID_SIZE; ++sx) {
-		for (int sy = 0; sy <= GRID_SIZE; ++sy) {
-			Point& p = points[sx][sy];
-			GLfloat x = GLfloat(sx - GRID_SIZE2) / GRID_SIZE2;
-			GLfloat y = GLfloat(sy - GRID_SIZE2) / GRID_SIZE2;
-
-			p.vx = x;
-			p.vy = y;
-			p.vz = (x * x + y * y) / -12.0f;
-
-			p.nx = x / 6.0f;
-			p.ny = y / 6.0f;
-			p.nz = 1.0f;      // note: not normalized
-
-			p.tx = (GLfloat(sx) / GRID_SIZE) * s + b;
-			p.ty = GLfloat(sy) / GRID_SIZE;
-		}
-	}
-
-	GLfloat LightDiffuse[]= { 1.2f, 1.2f, 1.2f, 1.2f };
-	glLightfv(GL_LIGHT0, GL_DIFFUSE, LightDiffuse);
-	glEnable(GL_LIGHT0);
-	glEnable(GL_NORMALIZE);
-
 	monitor3DList = glGenLists(1);
-	glNewList(monitor3DList, GL_COMPILE);
-	glEnable(GL_LIGHTING);
-	glMatrixMode(GL_PROJECTION);
-	glPushMatrix();
-	glLoadIdentity();
-	glFrustum(-1, 1, -1, 1, 1, 10);
-	glMatrixMode(GL_MODELVIEW);
-	glPushMatrix();
-	glLoadIdentity();
-	glTranslatef(0.0f, 0.4f, -2.0f);
-	glRotatef(-10.0f, 1.0f, 0.0f, 0.0f);
-	glScalef(2.2f, 2.2f, 2.2f);
-	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-	for (int y = 0; y < GRID_SIZE; ++y) {
-		glBegin(GL_TRIANGLE_STRIP);
-		for (int x = 0; x < (GRID_SIZE + 1); ++x) {
-			Point& p1 = points[x][y + 0];
-			Point& p2 = points[x][y + 1];
-			glTexCoord2f(p1.tx, p1.ty);
-			glNormal3f  (p1.nx, p1.ny, p1.nz);
-			glVertex3f  (p1.vx, p1.vy, p1.vz);
-			glTexCoord2f(p2.tx, p2.ty);
-			glNormal3f  (p2.nx, p2.ny, p2.nz);
-			glVertex3f  (p2.vx, p2.vy, p2.vz);
-		}
-		glEnd();
-	}
-	glMatrixMode(GL_PROJECTION);
-	glPopMatrix();
-	glMatrixMode(GL_MODELVIEW);
-	glPopMatrix();
-	glDisable(GL_LIGHTING);
-	glEndList();
+	preCalc3DDisplayList(renderSettings.getHorizontalStretch().getValue());
 
 	renderSettings.getNoise().attach(*this);
+	renderSettings.getHorizontalStretch().attach(*this);
 }
 
 GLPostProcessor::~GLPostProcessor()
 {
+	renderSettings.getHorizontalStretch().detach(*this);
 	renderSettings.getNoise().detach(*this);
 
 	glDeleteLists(monitor3DList, 1);
@@ -196,8 +131,10 @@ void GLPostProcessor::paint()
 {
 	RenderSettings::DisplayDeform deform =
 		renderSettings.getDisplayDeform().getValue();
+	double horStretch = renderSettings.getHorizontalStretch().getValue();
 	int glow = renderSettings.getGlow().getValue();
 	bool renderToTexture = (deform != RenderSettings::DEFORM_NORMAL) ||
+	                       (horStretch != 320.0) ||
 	                       (glow != 0);
 
 	if ((deform == RenderSettings::DEFORM_3D) || !paintFrame) {
@@ -252,9 +189,7 @@ void GLPostProcessor::paint()
 			glBegin(GL_QUADS);
 			int w = screen.getWidth();
 			int h = screen.getHeight();
-			GLfloat x1 = (deform == RenderSettings::DEFORM_HOR_STRETCH)
-			           ? (320.0f - 284.0f) / (2.0f * 320.0f)
-				   : 0.0f;
+			GLfloat x1 = (320.0f - horStretch) / (2.0f * 320.0f);
 			GLfloat x2 = 1.0f - x1;
 			glTexCoord2f(x1, 0.0f); glVertex2i(0, h);
 			glTexCoord2f(x1, 1.0f); glVertex2i(0, 0);
@@ -271,7 +206,7 @@ void GLPostProcessor::paint()
 
 RawFrame* GLPostProcessor::rotateFrames(
 	RawFrame* finishedFrame, FrameSource::FieldType field,
-	const EmuTime& time)
+	EmuTime::param time)
 {
 	RawFrame* reuseFrame =
 		PostProcessor::rotateFrames(finishedFrame, field, time);
@@ -286,8 +221,11 @@ void GLPostProcessor::update(const Setting& setting)
 {
 	VideoLayer::update(setting);
 	FloatSetting& noiseSetting = renderSettings.getNoise();
+	FloatSetting& horizontalStretch = renderSettings.getHorizontalStretch();
 	if (&setting == &noiseSetting) {
 		preCalcNoise(noiseSetting.getValue());
+	} else if (&setting == &horizontalStretch) {
+		preCalc3DDisplayList(horizontalStretch.getValue());
 	}
 }
 
@@ -313,15 +251,14 @@ void GLPostProcessor::uploadBlock(
 {
 	// create texture/pbo if needed
 	Textures::iterator it = textures.find(lineWidth);
-	ColourTexture* tex;
+	ColorTexture* tex;
 	PixelBuffer<unsigned>* pbo;
 	if (it != textures.end()) {
 		tex = it->second.tex;
 		pbo = it->second.pbo;
 	} else {
-		tex = new ColourTexture();
+		tex = new ColorTexture(lineWidth, height * 2); // *2 for interlace
 		tex->setWrapMode(false);
-		tex->setImage(lineWidth, height * 2); // *2 for interlace
 
 		pbo = new PixelBuffer<unsigned>();
 		if (pbo->openGLSupported()) {
@@ -345,9 +282,8 @@ void GLPostProcessor::uploadBlock(
 		pbo->bind();
 		unsigned* mapped = pbo->mapWrite();
 		for (unsigned y = srcStartY; y < srcEndY; ++y) {
-			unsigned* dummy = 0;
 			const unsigned* data =
-				paintFrame->getLinePtr(y, lineWidth, dummy);
+				paintFrame->getLinePtr<unsigned>(y, lineWidth);
 			MemoryOps::stream_memcpy(mapped + y * lineWidth, data, lineWidth);
 			paintFrame->freeLineBuffers(); // ASAP to keep cache warm
 		}
@@ -369,9 +305,8 @@ void GLPostProcessor::uploadBlock(
 		unsigned remainingLines = srcEndY - srcStartY;
 		while (remainingLines) {
 			unsigned lines;
-			unsigned* dummy = 0;
-			const unsigned* data = paintFrame->getMultiLinePtr(
-				y, remainingLines, lines, lineWidth, dummy);
+			const unsigned* data = paintFrame->getMultiLinePtr<unsigned>(
+				y, remainingLines, lines, lineWidth);
 			glTexSubImage2D(
 				GL_TEXTURE_2D,     // target
 				0,                 // level
@@ -490,4 +425,76 @@ void GLPostProcessor::drawNoise()
 	if (glBlendEquation) glBlendEquation(GL_FUNC_ADD);
 }
 
+void GLPostProcessor::preCalc3DDisplayList(double width)
+{
+	// generate display list for 3d deform
+	static const int GRID_SIZE = 16;
+	struct Point {
+		GLfloat vx, vy, vz;
+		GLfloat nx, ny, nz;
+		GLfloat tx, ty;
+	} points[GRID_SIZE + 1][GRID_SIZE + 1];
+	const int GRID_SIZE2 = GRID_SIZE / 2;
+	GLfloat s = width / 320.0f;
+	GLfloat b = (320.0f - width) / (2.0f * 320.0f);
+
+	for (int sx = 0; sx <= GRID_SIZE; ++sx) {
+		for (int sy = 0; sy <= GRID_SIZE; ++sy) {
+			Point& p = points[sx][sy];
+			GLfloat x = GLfloat(sx - GRID_SIZE2) / GRID_SIZE2;
+			GLfloat y = GLfloat(sy - GRID_SIZE2) / GRID_SIZE2;
+
+			p.vx = x;
+			p.vy = y;
+			p.vz = (x * x + y * y) / -12.0f;
+
+			p.nx = x / 6.0f;
+			p.ny = y / 6.0f;
+			p.nz = 1.0f;      // note: not normalized
+
+			p.tx = (GLfloat(sx) / GRID_SIZE) * s + b;
+			p.ty = GLfloat(sy) / GRID_SIZE;
+		}
+	}
+
+	GLfloat LightDiffuse[]= { 1.2f, 1.2f, 1.2f, 1.2f };
+	glLightfv(GL_LIGHT0, GL_DIFFUSE, LightDiffuse);
+	glEnable(GL_LIGHT0);
+	glEnable(GL_NORMALIZE);
+
+	glNewList(monitor3DList, GL_COMPILE);
+	glEnable(GL_LIGHTING);
+	glMatrixMode(GL_PROJECTION);
+	glPushMatrix();
+	glLoadIdentity();
+	glFrustum(-1, 1, -1, 1, 1, 10);
+	glMatrixMode(GL_MODELVIEW);
+	glPushMatrix();
+	glLoadIdentity();
+	glTranslatef(0.0f, 0.4f, -2.0f);
+	glRotatef(-10.0f, 1.0f, 0.0f, 0.0f);
+	glScalef(2.2f, 2.2f, 2.2f);
+	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+	for (int y = 0; y < GRID_SIZE; ++y) {
+		glBegin(GL_TRIANGLE_STRIP);
+		for (int x = 0; x < (GRID_SIZE + 1); ++x) {
+			Point& p1 = points[x][y + 0];
+			Point& p2 = points[x][y + 1];
+			glTexCoord2f(p1.tx, p1.ty);
+			glNormal3f  (p1.nx, p1.ny, p1.nz);
+			glVertex3f  (p1.vx, p1.vy, p1.vz);
+			glTexCoord2f(p2.tx, p2.ty);
+			glNormal3f  (p2.nx, p2.ny, p2.nz);
+			glVertex3f  (p2.vx, p2.vy, p2.vz);
+		}
+		glEnd();
+	}
+	glMatrixMode(GL_PROJECTION);
+	glPopMatrix();
+	glMatrixMode(GL_MODELVIEW);
+	glPopMatrix();
+	glDisable(GL_LIGHTING);
+	glEndList();
+}
+
 } // namespace openmsx
