@@ -1,4 +1,4 @@
-// $Id: GLRasterizer.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: GLRasterizer.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 /*
 TODO:
@@ -30,6 +30,7 @@ TODO:
 #include "Math.hh"
 #include "build-info.hh"
 #include <algorithm>
+#include <cstring>
 
 using std::string;
 
@@ -48,49 +49,50 @@ static const int TICKS_VISIBLE_MIDDLE =
 
 inline static GLRasterizer::Pixel GLMapRGB(double dr, double dg, double db)
 {
+	// convert to BGR packed pixel
 	int r = int(dr * 255.0);
 	int g = int(dg * 255.0);
 	int b = int(db * 255.0);
 	if (OPENMSX_BIGENDIAN) {
-		return (r << 24) | (g << 16) | (b <<  8) | 0x000000FF;
+		return (r <<  8) | (g << 16) | (b << 24) | 0x000000FF;
 	} else {
-		return (r <<  0) | (g <<  8) | (b << 16) | 0xFF000000;
+		return (r << 16) | (g <<  8) | (b <<  0) | 0xFF000000;
 	}
 }
 
-inline static void GLSetColour(GLRasterizer::Pixel colour)
+inline static void GLSetColor(GLRasterizer::Pixel colorBGR)
 {
 	if (OPENMSX_BIGENDIAN) {
-		glColor3ub((colour >> 24) & 0xFF,
-		           (colour >> 16) & 0xFF,
-		           (colour >>  8) & 0xFF);
+		glColor3ub((colorBGR >>  8) & 0xFF,
+		           (colorBGR >> 16) & 0xFF,
+		           (colorBGR >> 24) & 0xFF);
 	} else {
-		glColor3ub((colour >>  0) & 0xFF,
-		           (colour >>  8) & 0xFF,
-		           (colour >> 16) & 0xFF);
+		glColor3ub((colorBGR >> 16) & 0xFF,
+		           (colorBGR >>  8) & 0xFF,
+		           (colorBGR >>  0) & 0xFF);
 	}
 }
 
-inline static void GLSetTexEnvCol(GLRasterizer::Pixel colour)
+inline static void GLSetTexEnvCol(GLRasterizer::Pixel colorBGR)
 {
 	int r, g, b;
 	if (OPENMSX_BIGENDIAN) {
-		r = (colour >> 24) & 0xFF;
-		g = (colour >> 16) & 0xFF;
-		b = (colour >>  8) & 0xFF;
+		r = (colorBGR >>  8) & 0xFF;
+		g = (colorBGR >> 16) & 0xFF;
+		b = (colorBGR >> 24) & 0xFF;
 	} else {
-		r = (colour >>  0) & 0xFF;
-		g = (colour >>  8) & 0xFF;
-		b = (colour >> 16) & 0xFF;
+		r = (colorBGR >> 16) & 0xFF;
+		g = (colorBGR >>  8) & 0xFF;
+		b = (colorBGR >>  0) & 0xFF;
 	}
-	GLfloat colourVec[4] = { r / 255.0f, g / 255.0f, b / 255.0f, 1.0f };
-	glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, colourVec);
+	GLfloat colorVec[4] = { r / 255.0f, g / 255.0f, b / 255.0f, 1.0f };
+	glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, colorVec);
 }
 
 inline static void GLFillBlock(int x1, int y1, int x2, int y2,
-                               GLRasterizer::Pixel colour)
+                               GLRasterizer::Pixel colorBGR)
 {
-	GLSetColour(colour);
+	GLSetColor(colorBGR);
 	glRecti(x1, y1, x2, y2);
 }
 
@@ -108,8 +110,8 @@ inline static void GLBindMonoBlock(GLuint textureId, const byte* pixels)
 	                pixels);          // data
 }
 
-inline static void GLBindColourBlock(
-	GLuint textureId, const GLRasterizer::Pixel *pixels)
+inline static void GLBindColorBlock(
+	GLuint textureId, const GLRasterizer::Pixel* pixels)
 {
 	glBindTexture(GL_TEXTURE_2D, textureId);
 	glTexSubImage2D(GL_TEXTURE_2D,    // target
@@ -118,7 +120,7 @@ inline static void GLBindColourBlock(
 	                0,                // y-offset
 	                8,                // width
 	                8,                // height
-	                GL_RGBA,          // format
+	                GL_BGRA,          // format
 	                GL_UNSIGNED_BYTE, // type
 	                pixels);          // data
 }
@@ -128,7 +130,7 @@ inline static void GLDrawMonoBlock(
 	GLRasterizer::Pixel bg, int verticalScroll)
 {
 	glBindTexture(GL_TEXTURE_2D, textureId);
-	GLSetColour(bg);
+	GLSetColor(bg);
 	glBegin(GL_QUADS);
 	int x1 = x + width * zoom;
 	int y1 = y + 16;
@@ -141,7 +143,7 @@ inline static void GLDrawMonoBlock(
 	glEnd();
 }
 
-inline static void GLDrawColourBlock(GLuint textureId, int x, int y)
+inline static void GLDrawColorBlock(GLuint textureId, int x, int y)
 {
 	glBindTexture(GL_TEXTURE_2D, textureId);
 	glBegin(GL_QUADS);
@@ -163,7 +165,7 @@ inline static void drawStripes(
 	glBindTexture(GL_TEXTURE_2D, textureId);
 	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
 	GLSetTexEnvCol(col0);
-	GLSetColour(col1);
+	GLSetColor(col1);
 	glBegin(GL_QUADS);
 	glTexCoord2f(x1 / 2.0f, 1.0f); glVertex2i(x1, y2); // Bottom Left
 	glTexCoord2f(x2 / 2.0f, 1.0f); glVertex2i(x2, y2); // Bottom Right
@@ -194,6 +196,7 @@ inline static int translateX(int absoluteX)
 
 inline void GLRasterizer::renderBitmapLine(byte mode, int vramLine)
 {
+	Pixel lineBuffer[512];
 	if (lineValidInMode[vramLine] != mode) {
 		const byte* vramPtr =
 			vram.bitmapCacheWindow.getReadArea(vramLine * 128, 128);
@@ -222,6 +225,7 @@ inline void GLRasterizer::renderBitmapLines(byte line, int count)
 
 inline void GLRasterizer::renderPlanarBitmapLine(byte mode, int vramLine)
 {
+	Pixel lineBuffer[512];
 	if ((lineValidInMode[vramLine |   0] != mode) ||
 	    (lineValidInMode[vramLine | 512] != mode)) {
 		const byte* vramPtr0;
@@ -252,28 +256,26 @@ inline void GLRasterizer::renderPlanarBitmapLines(byte line, int count)
 	}
 }
 
-GLRasterizer::GLRasterizer(
-		CommandController& commandController,
-		VDP& vdp_, Display& display, OutputSurface& screen_)
-	: VideoLayer(VIDEO_MSX, commandController, display)
+GLRasterizer::GLRasterizer(VDP& vdp_, Display& display, OutputSurface& screen_)
+	: VideoLayer(vdp_.getMotherBoard(), VIDEO_MSX, display)
 	, renderSettings(display.getRenderSettings())
-	, vdp(vdp_), vram(vdp.getVRAM())
+	, vdp(vdp_), vram(vdp_.getVRAM())
 	, screen(screen_)
-	, characterConverter(new CharacterConverter<Pixel>(vdp, palFg, palBg))
+	, characterConverter(new CharacterConverter<Pixel>(vdp_, palFg, palBg))
 	, bitmapConverter(new BitmapConverter<Pixel>(
 	                                    palFg, PALETTE256, V9958_COLOURS))
-	, spriteConverter(new SpriteConverter<Pixel>(vdp.getSpriteChecker()))
+	, spriteConverter(new SpriteConverter<Pixel>(vdp_.getSpriteChecker()))
 {
 	GLint size;
 	glGetIntegerv(GL_MAX_TEXTURE_SIZE, &size);
 	//printf("Max texture size: %d\n", size);
 	glTexImage2D(GL_PROXY_TEXTURE_2D,
 	             0,
-	             GL_RGBA,
+	             GL_RGBA8,
 	             512,
 	             1,
 	             0,
-	             GL_RGBA,
+	             GL_BGRA,
 	             GL_UNSIGNED_BYTE,
 	             NULL);
 	size = -1;
@@ -295,23 +297,21 @@ GLRasterizer::GLRasterizer(
 	// Block based:
 	glGenTextures(4 * 256, colorChrTex);
 	for (int i = 0; i < 4 * 256; ++i) {
-		byte dummy[8 * 8 * 4];
 		glBindTexture(GL_TEXTURE_2D, colorChrTex[i]);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 		glTexImage2D(GL_TEXTURE_2D,    // target
 			     0,                // level
-			     GL_RGBA,          // internal format
+			     GL_RGBA8,          // internal format
 			     8,                // width
 			     8,                // height
 			     0,                // border
-			     GL_RGBA,          // format
+			     GL_BGRA,          // format
 			     GL_UNSIGNED_BYTE, // type
-			     dummy);           // data
+			     NULL);            // data
 	}
 	glGenTextures(256, monoChrTex);
 	for (int i = 0; i < 256; ++i) {
-		byte dummy[8 * 8];
 		glBindTexture(GL_TEXTURE_2D, monoChrTex[i]);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
@@ -323,7 +323,7 @@ GLRasterizer::GLRasterizer(
 			     0,                // border
 			     GL_LUMINANCE,     // format
 			     GL_UNSIGNED_BYTE, // type
-			     dummy);           // data
+			     NULL);            // data
 	}
 
 	// texture for drawing stripes
@@ -450,7 +450,7 @@ void GLRasterizer::resetPalette()
 	}
 }
 
-void GLRasterizer::frameStart(const EmuTime& /*time*/)
+void GLRasterizer::frameStart(EmuTime::param /*time*/)
 {
 	// Calculate line to render at top of screen.
 	// Make sure the display area is centered.
@@ -576,6 +576,7 @@ void GLRasterizer::setPalette(int index, int grb)
 		// so we have to repaint them all.
 		dirtyColour.flush();
 		memset(lineValidInMode, 0xFF, sizeof(lineValidInMode));
+		bitmapConverter->palette16Changed();
 	}
 
 	precalcColourIndex0(vdp.getDisplayMode(), vdp.getTransparency(),
@@ -669,6 +670,7 @@ void GLRasterizer::precalcColourIndex0(
 			// so we have to repaint them all.
 			dirtyColour.flush();
 			memset(lineValidInMode, 0xFF, sizeof(lineValidInMode));
+			bitmapConverter->palette16Changed();
 		}
 	} else {
 		if ((palFg[ 0] != palBg[tpIndex >> 2]) ||
@@ -681,6 +683,7 @@ void GLRasterizer::precalcColourIndex0(
 			// so we have to repaint them all.
 			dirtyColour.flush();
 			memset(lineValidInMode, 0xFF, sizeof(lineValidInMode));
+			bitmapConverter->palette16Changed();
 		}
 	}
 }
@@ -850,7 +853,7 @@ void GLRasterizer::renderText2(
 	Pixel plainFg = palFg[vdp.getForegroundColour()];
 	Pixel plainBg = palBg[vdp.getBackgroundColour()];
 	Pixel blinkFg, blinkBg;
- 	if (vdp.getBlinkState()) {
+	if (vdp.getBlinkState()) {
 		int fg = vdp.getBlinkForegroundColour();
 		blinkFg = palBg[fg ? fg : vdp.getBlinkBackgroundColour()];
 		blinkBg = palBg[vdp.getBlinkBackgroundColour()];
@@ -1026,9 +1029,9 @@ void GLRasterizer::renderGraphic2Row(
 				charPixels,
 				vram.patternTable.getReadArea(char8, 8),
 				vram.colourTable. getReadArea(char8, 8));
-			GLBindColourBlock(textureId, charPixels);
+			GLBindColorBlock(textureId, charPixels);
 		}
-		GLDrawColourBlock(textureId, x, screenLine);
+		GLDrawColorBlock(textureId, x, screenLine);
 		x += 16;
 	}
 }
@@ -1177,8 +1180,7 @@ void GLRasterizer::drawDisplay(
 				bitmapTexture->draw(
 					srcL, srcT, srcR, srcB,
 					screenX, screenY,
-					screenX + firstPageWidth, screenEndY
-					);
+					screenX + firstPageWidth, screenEndY);
 			} else {
 				firstPageWidth = 0;
 			}
@@ -1191,8 +1193,7 @@ void GLRasterizer::drawDisplay(
 				bitmapTexture->draw(
 					srcL, srcT, srcR, srcB,
 					screenX + firstPageWidth, screenY,
-					screenX + displayWidth, screenEndY
-					);
+					screenX + displayWidth, screenEndY);
 			}
 			screenY = screenEndY;
 			displayY = displayEndY & 255;
@@ -1225,12 +1226,12 @@ void GLRasterizer::drawDisplay(
 			break;
 		default:
 			// TEXT1Q / MULTIQ / BOGUS
-			LineTexture charTexture;
+			Pixel lineBuffer[512];
 			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 			for (int y = screenY; y < screenLimitY; y += 2) {
 				characterConverter->convertLine(lineBuffer, displayY);
-				charTexture.update(lineBuffer, lineWidth);
-				charTexture.draw(displayX + hScroll, screenX,
+				lineTexture.update(lineBuffer, lineWidth);
+				lineTexture.draw(displayX + hScroll, screenX,
 				                 y, displayWidth, 2);
 				displayY++; // is a byte, so wraps at 256
 			}
@@ -1262,6 +1263,7 @@ void GLRasterizer::drawSprites(
 		(mode == DisplayMode::GRAPHIC5 || mode == DisplayMode::GRAPHIC6)
 		? 2 : 1;
 	const SpriteChecker& spriteChecker = vdp.getSpriteChecker();
+	Pixel lineBuffer[512];
 	for (int y = fromY; y < limitY; y++, screenY += 2) {
 		// Optimisation: only draw sprites if there are actually sprites on
 		// this line.
@@ -1276,15 +1278,23 @@ void GLRasterizer::drawSprites(
 		if (spriteMode == 1) {
 			spriteConverter->drawMode1(y, displayX, displayLimitX, lineBuffer);
 		} else {
-			spriteConverter->drawMode2(y, displayX, displayLimitX, lineBuffer);
+			if (mode == DisplayMode::GRAPHIC5) {
+				spriteConverter->drawMode2<DisplayMode::GRAPHIC5>(
+					y, displayX, displayLimitX, lineBuffer);
+			} else if (mode == DisplayMode::GRAPHIC6) {
+				spriteConverter->drawMode2<DisplayMode::GRAPHIC6>(
+					y, displayX, displayLimitX, lineBuffer);
+			} else {
+				spriteConverter->drawMode2<DisplayMode::GRAPHIC4>(
+					y, displayX, displayLimitX, lineBuffer);
+			}
 		}
 
 		// Make line buffer into a texture and draw it.
 		// TODO: Make a texture of only the portion that will be drawn.
 		//       Or skip that and use block sprites instead.
-		LineTexture &texture = spriteTextures[y];
-		texture.update(lineBuffer, pixelZoom * 256);
-		texture.draw(displayX * 2, screenX, screenY, displayWidth * 2, 2);
+		lineTexture.update(lineBuffer, pixelZoom * 256);
+		lineTexture.draw(displayX * 2, screenX, screenY, displayWidth * 2, 2);
 	}
 
 	glDisable(GL_BLEND);
@@ -1317,4 +1327,3 @@ void GLRasterizer::update(const Setting& setting)
 }
 
 } // namespace openmsx
-
