@@ -23,7 +23,7 @@ GLSaIScaler::GLSaIScaler()
 }
 
 void GLSaIScaler::scaleImage(
-	ColourTexture& src,
+	ColorTexture& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	unsigned dstStartY, unsigned dstEndY, unsigned dstWidth)
 {
