@@ -1,4 +1,4 @@
-// $Id: GLScaleNxScaler.cc 5474 2006-06-20 20:02:56Z m9710797 $
+// $Id: GLScaleNxScaler.cc 8333 2008-11-01 08:29:32Z m9710797 $
 
 #include "GLScaleNxScaler.hh"
 
@@ -23,7 +23,7 @@ GLScaleNxScaler::GLScaleNxScaler()
 }
 
 void GLScaleNxScaler::scaleImage(
-	ColourTexture& src,
+	ColorTexture& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	unsigned dstStartY, unsigned dstEndY, unsigned dstWidth)
 {
