@@ -1,4 +1,4 @@
-// $Id: GLSimpleScaler.cc 6353 2007-04-23 18:28:00Z m9710797 $
+// $Id: GLSimpleScaler.cc 8333 2008-11-01 08:29:32Z m9710797 $
 
 #include "GLSimpleScaler.hh"
 #include "GLUtil.hh"
@@ -28,7 +28,7 @@ GLSimpleScaler::GLSimpleScaler(RenderSettings& renderSettings_)
 }
 
 void GLSimpleScaler::scaleImage(
-	ColourTexture& src,
+	ColorTexture& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	unsigned dstStartY, unsigned dstEndY, unsigned dstWidth)
 {
