@@ -1,4 +1,4 @@
-// $Id: GLTVScaler.cc 5474 2006-06-20 20:02:56Z m9710797 $
+// $Id: GLTVScaler.cc 8333 2008-11-01 08:29:32Z m9710797 $
 
 #include "GLTVScaler.hh"
 
@@ -28,7 +28,7 @@ GLTVScaler::~GLTVScaler()
 }
 
 void GLTVScaler::scaleImage(
-	ColourTexture& src,
+	ColorTexture& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned /*srcWidth*/,
 	unsigned dstStartY, unsigned dstEndY, unsigned dstWidth)
 {
