@@ -1,4 +1,4 @@
-// $Id: GLUtil.cc 6826 2007-08-09 16:50:00Z m9710797 $
+// $Id: GLUtil.cc 8334 2008-11-01 08:29:45Z m9710797 $
 
 #include "GLUtil.hh"
 #include "File.hh"
@@ -9,8 +9,8 @@
 #include "Version.hh"
 #include <iostream>
 #include <memory>
-#include <cstdlib>
 #include <cstring>
+#include <cstdio>
 
 #ifdef GL_VERSION_2_0
 #ifndef glGetShaderiv
@@ -97,16 +97,11 @@ void Texture::drawRect(GLfloat tx, GLfloat ty, GLfloat twidth, GLfloat theight,
 }
 
 
-// class ColourTexture
+// class ColorTexture
 
-ColourTexture::ColourTexture()
+ColorTexture::ColorTexture(GLsizei width_, GLsizei height_)
 	: Texture(GL_TEXTURE_2D)
 {
-}
-
-void ColourTexture::setImage(GLsizei width_, GLsizei height_, GLuint* data)
-{
-	bind();
 	width = width_;
 	height = height_;
 	glTexImage2D(
@@ -116,39 +111,17 @@ void ColourTexture::setImage(GLsizei width_, GLsizei height_, GLuint* data)
 		width,            // width
 		height,           // height
 		0,                // border
-		GL_RGBA,          // format
+		GL_BGRA,          // format
 		GL_UNSIGNED_BYTE, // type
-		data              // data
-		);
-}
-
-void ColourTexture::updateImage(
-	GLint x, GLint y, GLsizei width, GLsizei height, GLuint* data)
-{
-	bind();
-	glTexSubImage2D(
-		GL_TEXTURE_2D,    // target
-		0,                // level
-		x,                // offset x
-		y,                // offset y
-		width,            // width
-		height,           // height
-		GL_RGBA,          // format
-		GL_UNSIGNED_BYTE, // type
-		data);            // data
+		NULL);            // data
 }
 
 
 // class LuminanceTexture
 
-LuminanceTexture::LuminanceTexture()
+LuminanceTexture::LuminanceTexture(GLsizei width, GLsizei height)
 	: Texture(GL_TEXTURE_2D)
 {
-}
-
-void LuminanceTexture::setImage(GLsizei width, GLsizei height, GLbyte* data)
-{
-	bind();
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 	glTexImage2D(
@@ -160,7 +133,7 @@ void LuminanceTexture::setImage(GLsizei width, GLsizei height, GLbyte* data)
 		0,                // border
 		GL_LUMINANCE,     // format
 		GL_UNSIGNED_BYTE, // type
-		data);            // data
+		NULL);            // data
 }
 
 void LuminanceTexture::updateImage(
@@ -186,23 +159,21 @@ BitmapTexture::BitmapTexture()
 	: Texture(GL_TEXTURE_2D)
 {
 	static const unsigned SIZE = WIDTH * HEIGHT * 4;
-	void* blackness = malloc(SIZE);
-	memset(blackness, 0, SIZE);
+	std::vector<byte> blackness(SIZE); // zero-initialized
 	bind();
 	glTexImage2D(
 		GL_TEXTURE_2D,    // target
 		0,                // level
-		GL_RGBA,          // internal format
+		GL_RGBA8,         // internal format
 		512,              // width
 		1024,             // height
 		0,                // border
-		GL_RGBA,          // format
+		GL_BGRA,          // format
 		GL_UNSIGNED_BYTE, // type
-		blackness);       // data
-	free(blackness);
+		&blackness[0]);   // data
 }
 
-void BitmapTexture::update(int y, const GLuint* data, int lineWidth)
+void BitmapTexture::update(int y, const GLuint* dataBGR, int lineWidth)
 {
 	assert(0 <= y && y < HEIGHT);
 	assert(lineWidth <= WIDTH);
@@ -215,9 +186,9 @@ void BitmapTexture::update(int y, const GLuint* data, int lineWidth)
 		y,                // y-offset
 		lineWidth,        // width
 		1,                // height
-		GL_RGBA,          // format
+		GL_BGRA,          // format
 		GL_UNSIGNED_BYTE, // type
-		data);            // data
+		dataBGR);         // data
 }
 
 void BitmapTexture::draw(
@@ -248,7 +219,7 @@ LineTexture::LineTexture()
 {
 }
 
-void LineTexture::update(const GLuint* data, int lineWidth)
+void LineTexture::update(const GLuint* dataBGR, int lineWidth)
 {
 	bind();
 	if (prevLineWidth == lineWidth) {
@@ -260,24 +231,22 @@ void LineTexture::update(const GLuint* data, int lineWidth)
 			0,                // y-offset
 			lineWidth,        // width
 			1,                // height
-			GL_RGBA,          // format
+			GL_BGRA,          // format
 			GL_UNSIGNED_BYTE, // type
-			data              // data
-			);
+			dataBGR);         // data
 	} else {
 		// create new texture
 		prevLineWidth = lineWidth;
 		glTexImage2D(
 			GL_TEXTURE_2D,    // target
 			0,                // level
-			GL_RGBA,          // internal format
+			GL_RGBA8,         // internal format
 			lineWidth,        // width
 			1,                // height
 			0,                // border
-			GL_RGBA,          // format
+			GL_BGRA,          // format
 			GL_UNSIGNED_BYTE, // type
-			data              // data
-			);
+			dataBGR);         // data
 	}
 }
 
@@ -436,7 +405,8 @@ bool PixelBuffers::enabled = true;
 static string readTextFile(const string& filename)
 {
 	SystemFileContext context;
-	File file(context.resolve(filename));
+	CommandController* controller = NULL; // ok for SystemFileContext
+	File file(context.resolve(*controller, filename));
 	return string(reinterpret_cast<char*>(file.mmap()), file.getSize());
 }
 #endif
