@@ -1,4 +1,4 @@
-// $Id: HQ3xScaler.cc 5886 2006-11-18 12:03:15Z m9710797 $
+// $Id: HQ3xScaler.cc 7569 2008-01-23 22:26:20Z m9710797 $
 
 /*
 Original code: Copyright (C) 2003 MaxSt ( maxst@hiend3d.com )
@@ -14,6 +14,7 @@ Visit the HiEnd3D site for info:
 #include "HQCommon.hh"
 #include "LineScalers.hh"
 #include "openmsx.hh"
+#include "build-info.hh"
 
 namespace openmsx {
 
@@ -158,7 +159,11 @@ void HQ3xScaler<Pixel>::scale4x1to3x3(FrameSource& src,
 }
 
 // Force template instantiation.
+#if HAVE_16BPP
 template class HQ3xScaler<word>;
+#endif
+#if HAVE_32BPP
 template class HQ3xScaler<unsigned>;
+#endif
 
 } // namespace openmsx
