@@ -1,11 +1,13 @@
-// $Id: LowScaler.cc 5907 2006-11-29 17:23:52Z m9710797 $
+// $Id: LowScaler.cc 7904 2008-06-17 16:51:18Z m9710797 $
 
 #include "LowScaler.hh"
 #include "LineScalers.hh"
 #include "FrameSource.hh"
+#include "RawFrame.hh"
 #include "OutputSurface.hh"
 #include "MemoryOps.hh"
 #include "openmsx.hh"
+#include "build-info.hh"
 #include <cassert>
 
 namespace openmsx {
@@ -30,16 +32,43 @@ void LowScaler<Pixel>::averageHalve(const Pixel* pIn0, const Pixel* pIn1, Pixel*
 
 template <class Pixel>
 void LowScaler<Pixel>::scaleBlank1to1(
-		FrameSource& src, unsigned srcStartY, unsigned /*srcEndY*/,
+		FrameSource& src, unsigned srcStartY, unsigned srcEndY,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	MemoryOps::MemSet<Pixel, MemoryOps::STREAMING> memset;
-	for (unsigned srcY = srcStartY, dstY = dstStartY;
-	     dstY < dstEndY; srcY += 1, dstY += 1) {
-		Pixel* dummy = 0;
-		Pixel color = src.getLinePtr(srcY, dummy)[0];
-		Pixel* dstLine = dst.getLinePtr(dstY, dummy);
-		memset(dstLine, dst.getWidth(), color);
+	if (PLATFORM_GP2X) {
+		// note: src.getLinePtr() internally does a src.lock(). In a
+		//       profile the Lock function is relatively high. Though
+		//       if we put the blank line color info in some other
+		//       data structure (no need to lock()), the FillRect
+		//       function takes the place of Lock. So it seems we are
+		//       simply waiting for the previous blit command to
+		//       finish.
+		dst.unlock();
+		SDL_Rect dstRect;
+		dstRect.x = 0;
+		dstRect.w = dst.getWidth();
+		for (unsigned srcY = srcStartY, dstY = dstStartY; dstY < dstEndY; /**/) {
+			dstRect.y = dstY;
+			Pixel color = src.getLinePtr<Pixel>(srcY)[0];
+			unsigned start = srcY;
+			do {
+				srcY += 1;
+			} while ((src.getLinePtr<Pixel>(srcY)[0] == color) &&
+				 (srcY < srcEndY));
+			unsigned height = srcY - start;
+			dstY += height;
+			dstRect.h = height;
+			SDL_FillRect(dst.getSDLWorkSurface(), &dstRect, color);
+		}
+	} else {
+		dst.lock();
+		MemoryOps::MemSet<Pixel, MemoryOps::STREAMING> memset;
+		for (unsigned srcY = srcStartY, dstY = dstStartY;
+		     dstY < dstEndY; srcY += 1, dstY += 1) {
+			Pixel color = src.getLinePtr<Pixel>(srcY)[0];
+			Pixel* dstLine = dst.getLinePtrDirect<Pixel>(dstY);
+			memset(dstLine, dst.getWidth(), color);
+		}
 	}
 }
 
@@ -48,14 +77,14 @@ void LowScaler<Pixel>::scaleBlank2to1(
 		FrameSource& src, unsigned srcStartY, unsigned /*srcEndY*/,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
+	dst.lock();
 	MemoryOps::MemSet<Pixel, MemoryOps::STREAMING> memset;
 	for (unsigned srcY = srcStartY, dstY = dstStartY;
 	     dstY < dstEndY; srcY += 2, dstY += 1) {
-		Pixel* dummy = 0;
-		Pixel color0 = src.getLinePtr(srcY + 0, dummy)[0];
-		Pixel color1 = src.getLinePtr(srcY + 1, dummy)[0];
+		Pixel color0 = src.getLinePtr<Pixel>(srcY + 0)[0];
+		Pixel color1 = src.getLinePtr<Pixel>(srcY + 1)[0];
 		Pixel color01 = pixelOps.template blend<1, 1>(color0, color1);
-		Pixel* dstLine = dst.getLinePtr(dstY, dummy);
+		Pixel* dstLine = dst.getLinePtrDirect<Pixel>(dstY);
 		memset(dstLine, dst.getWidth(), color01);
 	}
 }
@@ -66,11 +95,11 @@ static void doScale1(FrameSource& src,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY,
 	ScaleOp scale)
 {
+	dst.lock();
 	for (unsigned srcY = srcStartY, dstY = dstStartY;
 	     dstY < dstEndY; ++srcY, ++dstY) {
-		Pixel* dummy = 0;
-		const Pixel* srcLine = src.getLinePtr(srcY, srcWidth, dummy);
-		Pixel* dstLine = dst.getLinePtr(dstY, dummy);
+		const Pixel* srcLine = src.getLinePtr<Pixel>(srcY, srcWidth);
+		Pixel* dstLine = dst.getLinePtrDirect<Pixel>(dstY);
 		scale(srcLine, dstLine, dst.getWidth());
 	}
 }
@@ -81,16 +110,16 @@ static void doScaleDV(FrameSource& src,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY,
 	PixelOperations<Pixel> ops, ScaleOp scale)
 {
+	dst.lock();
 	BlendLines<Pixel> blend(ops);
 	for (unsigned srcY = srcStartY, dstY = dstStartY;
 	     dstY < dstEndY; srcY += 2, dstY += 1) {
-		Pixel* dummy = 0;
-		const Pixel* srcLine0 = src.getLinePtr(srcY + 0, srcWidth, dummy);
-		const Pixel* srcLine1 = src.getLinePtr(srcY + 1, srcWidth, dummy);
+		const Pixel* srcLine0 = src.getLinePtr<Pixel>(srcY + 0, srcWidth);
+		const Pixel* srcLine1 = src.getLinePtr<Pixel>(srcY + 1, srcWidth);
 		Pixel buf0[dst.getWidth()], buf1[dst.getWidth()];
 		scale(srcLine0, buf0, dst.getWidth());
 		scale(srcLine1, buf1, dst.getWidth());
-		Pixel* dstLine = dst.getLinePtr(dstY, dummy);
+		Pixel* dstLine = dst.getLinePtrDirect<Pixel>(dstY);
 		blend(buf0, buf1, dstLine, dst.getWidth());
 	}
 }
@@ -119,6 +148,20 @@ void LowScaler<Pixel>::scale1x1to1x1(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
+	if (PLATFORM_GP2X) {
+		if (RawFrame* raw = dynamic_cast<RawFrame*>(&src)) {
+			raw->unlock();
+			dst.unlock();
+			unsigned height = dstEndY - dstStartY;
+			SDL_Rect srcRect, dstRect;
+			srcRect.x = 0;        srcRect.y = srcStartY;
+			srcRect.w = srcWidth; srcRect.h = height;
+			dstRect.x = 0;        dstRect.y = dstStartY;
+			SDL_BlitSurface(raw->getSDLSurface(), &srcRect,
+			                dst.getSDLWorkSurface(),  &dstRect);
+			return;
+		}
+	}
 	doScale1<Pixel>(src, srcStartY, srcEndY, srcWidth, dst, dstStartY, dstEndY,
 	                Scale_1on1<Pixel>());
 }
@@ -131,12 +174,12 @@ void LowScaler<Pixel>::scale1x2to1x1(FrameSource& src,
 	// No need to scale to local buffer first, like doScaleDV does.
 	// TODO: Even so, this scale mode is so rare, is it really worth having
 	//       optimized code for?
+	dst.lock();
 	BlendLines<Pixel> blend(pixelOps);
 	while (dstStartY < dstEndY) {
-		Pixel* dummy = 0;
-		const Pixel* srcLine0 = src.getLinePtr(srcStartY++, srcWidth, dummy);
-		const Pixel* srcLine1 = src.getLinePtr(srcStartY++, srcWidth, dummy);
-		Pixel* dstLine = dst.getLinePtr(dstStartY++, dummy);
+		const Pixel* srcLine0 = src.getLinePtr<Pixel>(srcStartY++, srcWidth);
+		const Pixel* srcLine1 = src.getLinePtr<Pixel>(srcStartY++, srcWidth);
+		Pixel* dstLine = dst.getLinePtrDirect<Pixel>(dstStartY++);
 		blend(srcLine0, srcLine1, dstLine, dst.getWidth());
 	}
 }
@@ -294,7 +337,11 @@ void LowScaler<Pixel>::scaleImage(FrameSource& src,
 }
 
 // Force template instantiation.
+#if HAVE_16BPP
 template class LowScaler<word>;
-template class LowScaler<unsigned int>;
+#endif
+#if HAVE_32BPP
+template class LowScaler<unsigned>;
+#endif
 
 } // namespace openmsx
