@@ -1,41 +1,43 @@
-// $Id: PostProcessor.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: PostProcessor.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "PostProcessor.hh"
 #include "Display.hh"
 #include "VisibleSurface.hh"
 #include "DeinterlacedFrame.hh"
 #include "DoubledFrame.hh"
+#include "ScreenShotSaver.hh"
 #include "RenderSettings.hh"
 #include "BooleanSetting.hh"
 #include "RawFrame.hh"
 #include "AviRecorder.hh"
 #include "CliComm.hh"
+#include "CommandException.hh"
+#include "build-info.hh"
 #include <algorithm>
 #include <cassert>
 
 namespace openmsx {
 
-PostProcessor::PostProcessor(CommandController& commandController_,
-	Display& display, VisibleSurface& screen_, VideoSource videoSource,
+PostProcessor::PostProcessor(MSXMotherBoard& motherBoard,
+	Display& display_, VisibleSurface& screen_, VideoSource videoSource,
 	unsigned maxWidth, unsigned height)
-	: VideoLayer(videoSource, commandController_, display)
-	, renderSettings(display.getRenderSettings())
+	: VideoLayer(motherBoard, videoSource, display_)
+	, renderSettings(display_.getRenderSettings())
 	, screen(screen_)
 	, paintFrame(0)
 	, recorder(0)
-	, prevTime(EmuTime::zero)
-	, commandController(commandController_)
+	, display(display_)
 {
-	currFrame = new RawFrame(screen.getFormat(), maxWidth, height);
-	prevFrame = new RawFrame(screen.getFormat(), maxWidth, height);
-	deinterlacedFrame = new DeinterlacedFrame(screen.getFormat());
-	interlacedFrame   = new DoubledFrame     (screen.getFormat());
+	currFrame = new RawFrame(screen.getSDLFormat(), maxWidth, height);
+	prevFrame = new RawFrame(screen.getSDLFormat(), maxWidth, height);
+	deinterlacedFrame = new DeinterlacedFrame(screen.getSDLFormat());
+	interlacedFrame   = new DoubledFrame     (screen.getSDLFormat());
 }
 
 PostProcessor::~PostProcessor()
 {
 	if (recorder) {
-		commandController.getCliComm().printWarning(
+		display.getCliComm().printWarning(
 			"Videorecording stopped, because you quit openMSX, "
 			"changed machine, or changed a video setting "
 			"during recording.");
@@ -67,10 +69,8 @@ const std::string& PostProcessor::getName()
 
 RawFrame* PostProcessor::rotateFrames(
 	RawFrame* finishedFrame, FrameSource::FieldType field,
-	const EmuTime& time)
+	EmuTime::param time)
 {
-	prevTime = time;
-
 	RawFrame* reuseFrame = prevFrame;
 	prevFrame = currFrame;
 	currFrame = finishedFrame;
@@ -103,23 +103,80 @@ RawFrame* PostProcessor::rotateFrames(
 	}
 
 	if (recorder) {
-		const void* lines[240];
-		for (int i = 0; i < 240; ++i) {
+		const unsigned height = recorder->getFrameHeight();
+		const void* lines[height];
+		getScaledFrame(height, lines);
+		/*for (unsigned i = 0; i < height; ++i) {
 			if (getBpp() == 32) {
-				unsigned* dummy = 0;
-				lines[i] = paintFrame->getLinePtr320_240(i, dummy);
+#if HAVE_32BPP
+				// 32bpp
+				if (height == 240) {
+					lines[i] = paintFrame->getLinePtr320_240<unsigned>(i);
+				} else {
+					assert (height == 480);
+					lines[i] = paintFrame->getLinePtr640_480<unsigned>(i);
+				}
+#endif
 			} else {
-				word* dummy = 0;
-				lines[i] = paintFrame->getLinePtr320_240(i, dummy);
+#if HAVE_16BPP
+				// 15bpp or 16bpp
+				if (height == 240) {
+					lines[i] = paintFrame->getLinePtr320_240<word>(i);
+				} else {
+					assert (height == 480);
+					lines[i] = paintFrame->getLinePtr640_480<word>(i);
+				}
+#endif
 			}
-		}
+		}*/
 		recorder->addImage(lines, time);
-		finishedFrame->freeLineBuffers();
+		paintFrame->freeLineBuffers();
 	}
 
 	return reuseFrame;
 }
 
+void PostProcessor::getScaledFrame(unsigned height, const void** lines)
+{
+	for (unsigned i = 0; i < height; ++i) {
+		if (getBpp() == 32) {
+#if HAVE_32BPP
+			// 32bpp
+			if (height == 240) {
+				lines[i] = paintFrame->getLinePtr320_240<unsigned>(i);
+			} else {
+				assert (height == 480);
+				lines[i] = paintFrame->getLinePtr640_480<unsigned>(i);
+			}
+#endif
+		} else {
+#if HAVE_16BPP
+			// 15bpp or 16bpp
+			if (height == 240) {
+				lines[i] = paintFrame->getLinePtr320_240<word>(i);
+			} else {
+				assert (height == 480);
+				lines[i] = paintFrame->getLinePtr640_480<word>(i);
+			}
+#endif
+		}
+	}
+}
+
+void PostProcessor::takeScreenShot(unsigned height, const std::string& filename)
+{
+	if (!paintFrame) {
+		throw CommandException("TODO");
+	}
+	const void* lines[height];
+	getScaledFrame(height, lines);
+
+	unsigned width = (height == 240) ? 320 : 640;
+	ScreenShotSaver::save(width, height, lines,
+	                      paintFrame->getSDLPixelFormat(), filename);
+	paintFrame->freeLineBuffers();
+}
+
 void PostProcessor::setRecorder(AviRecorder* recorder_)
 {
 	recorder = recorder_;
@@ -132,7 +189,7 @@ bool PostProcessor::isRecording() const
 
 unsigned PostProcessor::getBpp() const
 {
-	return screen.getFormat()->BitsPerPixel;
+	return screen.getSDLFormat().BitsPerPixel;
 }
 
 } // namespace openmsx
