@@ -1,10 +1,11 @@
-// $Id: RGBTriplet3xScaler.cc 5711 2006-09-29 11:53:12Z m9710797 $
+// $Id: RGBTriplet3xScaler.cc 7569 2008-01-23 22:26:20Z m9710797 $
 
 #include "RGBTriplet3xScaler.hh"
 #include "LineScalers.hh"
 #include "FrameSource.hh"
 #include "OutputSurface.hh"
 #include "RenderSettings.hh"
+#include "build-info.hh"
 
 namespace openmsx {
 
@@ -93,39 +94,39 @@ void RGBTriplet3xScaler<Pixel>::doScale1(FrameSource& src,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY,
 	ScaleOp scale)
 {
+	dst.lock();
 	recalcBlur();
 
 	unsigned tmpWidth = dst.getWidth() / 3;
 	int scanlineFactor = settings.getScanlineFactor();
 	unsigned y = dstStartY;
-	Pixel* dummy = 0;
-	const Pixel* srcLine = src.getLinePtr(srcStartY++, srcWidth, dummy);
-	Pixel* prevDstLine0 = dst.getLinePtr(y + 0, dummy);
+	const Pixel* srcLine = src.getLinePtr<Pixel>(srcStartY++, srcWidth);
+	Pixel* prevDstLine0 = dst.getLinePtrDirect<Pixel>(y + 0);
 	scaleLine(srcLine, prevDstLine0, scale, tmpWidth);
 
 	Scale_1on1<Pixel> copy;
-	Pixel* dstLine1 = dst.getLinePtr(y + 1, dummy);
+	Pixel* dstLine1 = dst.getLinePtrDirect<Pixel>(y + 1);
 	copy(prevDstLine0, dstLine1, dst.getWidth());
 
 	for (/* */; (y + 4) < dstEndY; y += 3, srcStartY += 1) {
-		srcLine = src.getLinePtr(srcStartY, srcWidth, dummy);
-		Pixel* dstLine0 = dst.getLinePtr(y + 3, dummy);
+		srcLine = src.getLinePtr<Pixel>(srcStartY, srcWidth);
+		Pixel* dstLine0 = dst.getLinePtrDirect<Pixel>(y + 3);
 		scaleLine(srcLine, dstLine0, scale, tmpWidth);
 
-		dstLine1 = dst.getLinePtr(y + 4, dummy);
+		dstLine1 = dst.getLinePtrDirect<Pixel>(y + 4);
 		copy(dstLine0, dstLine1, dst.getWidth());
 
-		Pixel* dstLine2 = dst.getLinePtr(y + 2, dummy);
+		Pixel* dstLine2 = dst.getLinePtrDirect<Pixel>(y + 2);
 		scanline.draw(prevDstLine0, dstLine0, dstLine2,
 		              scanlineFactor, dst.getWidth());
 
 		prevDstLine0 = dstLine0;
 	}
 
-	srcLine = src.getLinePtr(srcStartY, srcWidth, dummy);
+	srcLine = src.getLinePtr<Pixel>(srcStartY, srcWidth);
 	Pixel buf[dst.getWidth()];
 	scaleLine(srcLine, buf, scale, tmpWidth);
-	Pixel* dstLine2 = dst.getLinePtr(y + 2, dummy);
+	Pixel* dstLine2 = dst.getLinePtrDirect<Pixel>(y + 2);
 	scanline.draw(prevDstLine0, buf, dstLine2,
 	              scanlineFactor, dst.getWidth());
 }
@@ -137,22 +138,22 @@ void RGBTriplet3xScaler<Pixel>::doScale2(FrameSource& src,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY,
 	ScaleOp scale)
 {
+	dst.lock();
 	recalcBlur();
 
 	unsigned tmpWidth = dst.getWidth() / 3;
-	Pixel* dummy = 0;
 	int scanlineFactor = settings.getScanlineFactor();
 	for (unsigned srcY = srcStartY, dstY = dstStartY; dstY < dstEndY;
 	     srcY += 2, dstY += 3) {
-		const Pixel* srcLine0 = src.getLinePtr(srcY + 0, srcWidth, dummy);
-		Pixel* dstLine0 = dst.getLinePtr(dstY + 0, dummy);
+		const Pixel* srcLine0 = src.getLinePtr<Pixel>(srcY + 0, srcWidth);
+		Pixel* dstLine0 = dst.getLinePtrDirect<Pixel>(dstY + 0);
 		scaleLine(srcLine0, dstLine0, scale, tmpWidth);
 
-		const Pixel* srcLine1 = src.getLinePtr(srcY + 1, srcWidth, dummy);
-		Pixel* dstLine2 = dst.getLinePtr(dstY + 2, dummy);
+		const Pixel* srcLine1 = src.getLinePtr<Pixel>(srcY + 1, srcWidth);
+		Pixel* dstLine2 = dst.getLinePtrDirect<Pixel>(dstY + 2);
 		scaleLine(srcLine1, dstLine2, scale, tmpWidth);
 
-		Pixel* dstLine1 = dst.getLinePtr(dstY + 1, dummy);
+		Pixel* dstLine1 = dst.getLinePtrDirect<Pixel>(dstY + 1);
 		scanline.draw(dstLine0, dstLine2, dstLine1,
 		              scanlineFactor, dst.getWidth());
 	}
@@ -287,6 +288,7 @@ void RGBTriplet3xScaler<Pixel>::scaleBlank1to3(
 		FrameSource& src, unsigned srcStartY, unsigned srcEndY,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
+	dst.lock();
 	recalcBlur();
 	int scanlineFactor = settings.getScanlineFactor();
 
@@ -294,8 +296,7 @@ void RGBTriplet3xScaler<Pixel>::scaleBlank1to3(
 	                  ? dstEndY : dstEndY - 3;
 	unsigned srcY = srcStartY, dstY = dstStartY;
 	for (/* */; dstY < stopDstY; srcY += 1, dstY += 3) {
-		Pixel* dummy = 0;
-		Pixel color = src.getLinePtr(srcY, dummy)[0];
+		Pixel color = src.getLinePtr<Pixel>(srcY)[0];
 
 		Pixel inNormal [3];
 		Pixel outNormal[3 * 3];
@@ -307,11 +308,11 @@ void RGBTriplet3xScaler<Pixel>::scaleBlank1to3(
 					outNormal[i], scanlineFactor);
 		}
 
-		fillLoop(outNormal,   dst.getLinePtr(dstY + 0, dummy),
+		fillLoop(outNormal,   dst.getLinePtrDirect<Pixel>(dstY + 0),
 		         dst.getWidth());
-		fillLoop(outNormal,   dst.getLinePtr(dstY + 1, dummy),
+		fillLoop(outNormal,   dst.getLinePtrDirect<Pixel>(dstY + 1),
 		         dst.getWidth());
-		fillLoop(outScanline, dst.getLinePtr(dstY + 2, dummy),
+		fillLoop(outScanline, dst.getLinePtrDirect<Pixel>(dstY + 2),
 		         dst.getWidth());
 	}
 	if (dstY != dst.getHeight()) {
@@ -328,13 +329,13 @@ void RGBTriplet3xScaler<Pixel>::scaleBlank2to3(
 		FrameSource& src, unsigned srcStartY, unsigned /*srcEndY*/,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
+	dst.lock();
 	recalcBlur();
 	int scanlineFactor = settings.getScanlineFactor();
 	for (unsigned srcY = srcStartY, dstY = dstStartY;
 	     dstY < dstEndY; srcY += 2, dstY += 3) {
-		Pixel* dummy = 0;
-		Pixel color0 = src.getLinePtr(srcY + 0, dummy)[0];
-		Pixel color1 = src.getLinePtr(srcY + 1, dummy)[0];
+		Pixel color0 = src.getLinePtr<Pixel>(srcY + 0)[0];
+		Pixel color1 = src.getLinePtr<Pixel>(srcY + 1)[0];
 
 		Pixel inNormal [3];
 		Pixel out0Normal[3 * 3];
@@ -352,17 +353,21 @@ void RGBTriplet3xScaler<Pixel>::scaleBlank2to3(
 				scanlineFactor);
 		}
 
-		fillLoop(out0Normal,  dst.getLinePtr(dstY + 0, dummy),
+		fillLoop(out0Normal,  dst.getLinePtrDirect<Pixel>(dstY + 0),
 		         dst.getWidth());
-		fillLoop(outScanline, dst.getLinePtr(dstY + 1, dummy),
+		fillLoop(outScanline, dst.getLinePtrDirect<Pixel>(dstY + 1),
 		         dst.getWidth());
-		fillLoop(out1Normal,  dst.getLinePtr(dstY + 2, dummy),
+		fillLoop(out1Normal,  dst.getLinePtrDirect<Pixel>(dstY + 2),
 		         dst.getWidth());
 	}
 }
 
 // Force template instantiation.
+#if HAVE_16BPP
 template class RGBTriplet3xScaler<word>;
+#endif
+#if HAVE_32BPP
 template class RGBTriplet3xScaler<unsigned>;
+#endif
 
 } // namespace openmsx
