@@ -1,4 +1,4 @@
-// $Id: RenderSettings.cc 6593 2007-06-03 23:12:19Z mthuurne $
+// $Id: RenderSettings.cc 8223 2008-09-14 19:17:24Z m9710797 $
 
 #include "RenderSettings.hh"
 #include "IntegerSetting.hh"
@@ -6,6 +6,9 @@
 #include "BooleanSetting.hh"
 #include "StringSetting.hh"
 #include "VideoSourceSetting.hh"
+#include "CommandController.hh"
+#include "CommandException.hh"
+#include "build-info.hh"
 #include <cmath>
 
 namespace openmsx {
@@ -82,18 +85,21 @@ RenderSettings::RenderSettings(CommandController& commandController_)
 
 	EnumSetting<ScaleAlgorithm>::Map scalerMap;
 	scalerMap["simple"] = SCALER_SIMPLE;
-	scalerMap["SaI"] = SCALER_SAI;
-	scalerMap["ScaleNx"] = SCALER_SCALE;
-	scalerMap["hq"] = SCALER_HQ;
-	scalerMap["hqlite"] = SCALER_HQLITE;
-	scalerMap["RGBtriplet"] = SCALER_RGBTRIPLET;
-	scalerMap["TV"] = SCALER_TV;
+	if (MAX_SCALE_FACTOR > 1) {
+		scalerMap["SaI"] = SCALER_SAI;
+		scalerMap["ScaleNx"] = SCALER_SCALE;
+		scalerMap["hq"] = SCALER_HQ;
+		scalerMap["hqlite"] = SCALER_HQLITE;
+		scalerMap["RGBtriplet"] = SCALER_RGBTRIPLET;
+		scalerMap["TV"] = SCALER_TV;
+	}
 	scaleAlgorithm.reset(new EnumSetting<ScaleAlgorithm>(commandController,
 		"scale_algorithm", "scale algorithm",
 		SCALER_SIMPLE, scalerMap));
 
 	scaleFactor.reset(new IntegerSetting(commandController,
-		"scale_factor", "scale factor", 2, 1, 4));
+		"scale_factor", "scale factor", 2,
+		MIN_SCALE_FACTOR, MAX_SCALE_FACTOR));
 
 	scanlineAlpha.reset(new IntegerSetting(commandController,
 		"scanline", "amount of scanline effect: 0 = none, 100 = full",
@@ -113,10 +119,19 @@ RenderSettings::RenderSettings(CommandController& commandController_)
 	EnumSetting<DisplayDeform>::Map deformMap;
 	deformMap["normal"] = DEFORM_NORMAL;
 	deformMap["3d"] = DEFORM_3D;
-	deformMap["horizontal_stretch"] = DEFORM_HOR_STRETCH;
 	displayDeform.reset(new EnumSetting<DisplayDeform>(commandController,
 		"display_deform", "Display deform (for the moment this only "
 		"works with the SDLGL-PP renderer", DEFORM_NORMAL, deformMap));
+
+	horizontalStretch.reset(new FloatSetting(commandController,
+		"horizontal_stretch",
+		"Amount of horizontal stretch: this many MSX pixels will be "
+		"stretched over the complete width of the output screen.\n"
+		"  320 = no stretch\n"
+		"  256 = max stretch (no border visible anymore)\n"
+		"  good values are 272 or 284\n"
+		"This setting has only effect when using the SDLGL-PP renderer.",
+		284.0, 256.0, 320.0));
 }
 
 RenderSettings::~RenderSettings()
@@ -143,7 +158,7 @@ static double conv2(double x, double gamma)
 	return ::pow(std::min(std::max(0.0, x), 1.0), gamma);
 }
 
-void RenderSettings::transformRGB(double& r, double& g, double& b)
+void RenderSettings::transformRGB(double& r, double& g, double& b) const
 {
 	double brightness = getBrightness().getValue() / 100.0;
 	double contrast = getContrast().getValue();
