@@ -1,4 +1,4 @@
-// $Id: RendererFactory.cc 5993 2007-01-12 22:12:12Z m9710797 $
+// $Id: RendererFactory.cc 7641 2008-03-11 20:16:50Z m9710797 $
 
 #include "RendererFactory.hh"
 #include "RenderSettings.hh"
@@ -112,12 +112,12 @@ auto_ptr<RendererFactory::RendererSetting>
 	// then change it to 'SDL'
 	setting->setDontSaveValue("none");
 	if (setting->getValue() == DUMMY) {
-		setting->setValue(SDL);
+		setting->changeValue(SDL);
 	}
 	// set saved value as default
 	setting->setRestoreValue(setting->getValue());
 
-	setting->setValue(DUMMY); // always start hidden
+	setting->changeValue(DUMMY); // always start hidden
 
 	return setting;
 }
