@@ -1,8 +1,13 @@
-// $Id: SDLImage.cc 6914 2007-09-08 21:22:05Z m9710797 $
+// $Id: SDLImage.cc 7904 2008-06-17 16:51:18Z m9710797 $
 
 #include "SDLImage.hh"
-#include "File.hh"
+#include "OutputSurface.hh"
+#include "LocalFileReference.hh"
 #include "MSXException.hh"
+#include "build-info.hh"
+#if PLATFORM_GP2X
+#include "GP2XMMUHack.hh"
+#endif
 #include <SDL_image.h>
 #include <SDL.h>
 #include <cassert>
@@ -11,84 +16,116 @@ using std::string;
 
 namespace openmsx {
 
-SDLImage::SDLImage(SDL_Surface* output, const string& filename)
-	: outputScreen(output)
+SDLImage::SDLImage(OutputSurface& output_, const string& filename)
+	: output(output_)
 {
 	image = loadImage(filename);
 	init(filename);
 }
 
-SDLImage::SDLImage(SDL_Surface* output, const std::string& filename,
+SDLImage::SDLImage(OutputSurface& output_, const std::string& filename,
                    double scaleFactor)
-	: outputScreen(output)
+	: output(output_)
 {
 	image = loadImage(filename, scaleFactor);
 	init(filename);
 }
 
-SDLImage::SDLImage(SDL_Surface* output, const string& filename,
+SDLImage::SDLImage(OutputSurface& output_, const string& filename,
 	           unsigned width, unsigned height)
-	: outputScreen(output)
+	: output(output_)
 {
 	image = loadImage(filename, width, height);
 	init(filename);
 }
 
-SDLImage::SDLImage(SDL_Surface* output, unsigned width, unsigned height,
-                   byte alpha)
-	: outputScreen(output)
+SDLImage::SDLImage(OutputSurface& output_, unsigned width, unsigned height,
+                   byte alpha, byte r, byte g, byte b)
+	: output(output_)
 {
-	SDL_PixelFormat* format = outputScreen->format;
+	const SDL_PixelFormat& format = output.getSDLFormat();
 	image = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCALPHA,
-		width, height, format->BitsPerPixel,
-		format->Rmask, format->Gmask, format->Bmask, 0);
-	if (image) {
-		SDL_SetAlpha(image, SDL_SRCALPHA, alpha);
+		width, height, format.BitsPerPixel,
+		format.Rmask, format.Gmask, format.Bmask, 0);
+	if (!image) {
+		throw MSXException("Couldn't allocate surface.");
 	}
+	if (r || g || b) {
+		SDL_FillRect(image, NULL,
+			     SDL_MapRGB(&output.getSDLFormat(), r, g, b));
+	}
+	a = (alpha == 255) ? 256 : alpha;
+	workImage = NULL;
+}
+
+SDLImage::SDLImage(OutputSurface& output_, SDL_Surface* image_)
+	: output(output_)
+	, image(image_)
+{
 	init("");
 }
 
 void SDLImage::init(const string& filename)
 {
 	assert(image);
-	SDL_PixelFormat* format = image->format;
-	workImage = SDL_CreateRGBSurface(SDL_SWSURFACE,
+	const SDL_PixelFormat* format = image->format;
+	int flags = SDL_SWSURFACE;
+	if (PLATFORM_GP2X) {
+		flags = SDL_HWSURFACE;
+	}
+	workImage = SDL_CreateRGBSurface(flags,
 		image->w, image->h, format->BitsPerPixel,
 		format->Rmask, format->Gmask, format->Bmask, 0);
 	if (!workImage) {
 		SDL_FreeSurface(image);
 		throw MSXException("Error loading image " + filename);
 	}
+#if PLATFORM_GP2X
+	GP2XMMUHack::instance().patchPageTables();
+#endif
 }
 
 SDLImage::~SDLImage()
 {
 	SDL_FreeSurface(image);
-	SDL_FreeSurface(workImage);
+	if (workImage) SDL_FreeSurface(workImage);
 }
 
 void SDLImage::draw(unsigned x, unsigned y, byte alpha)
 {
-	if (SDL_MUSTLOCK(outputScreen)) SDL_UnlockSurface(outputScreen);
+	output.unlock();
+	SDL_Surface* outputSurface = output.getSDLWorkSurface();
+	SDL_Rect rect;
+	rect.x = x;
+	rect.y = y;
 	if (alpha == 255) {
-		SDL_Rect rect;
-		rect.x = x;
-		rect.y = y;
-		SDL_BlitSurface(image, NULL, outputScreen, &rect);
+		SDL_BlitSurface(image, NULL, outputSurface, &rect);
 	} else {
-		SDL_Rect rect;
-		rect.x = x;
-		rect.y = y;
-		rect.w = image->w;
-		rect.h = image->h;
-		SDL_BlitSurface(outputScreen, &rect, workImage, NULL);
-		SDL_BlitSurface(image,        NULL,  workImage, NULL);
-		SDL_SetAlpha(workImage, SDL_SRCALPHA, alpha);
-		SDL_BlitSurface(workImage,    NULL,  outputScreen, &rect);
+		if (workImage) {
+			rect.w = image->w;
+			rect.h = image->h;
+			SDL_BlitSurface(outputSurface, &rect, workImage, NULL);
+			SDL_BlitSurface(image,         NULL,  workImage, NULL);
+			SDL_SetAlpha(workImage, SDL_SRCALPHA, alpha);
+			SDL_BlitSurface(workImage,    NULL,  outputSurface, &rect);
+		} else {
+			SDL_SetAlpha(image, SDL_SRCALPHA, (a * alpha) / 256);
+			SDL_BlitSurface(image, NULL, outputSurface, &rect);
+		}
 	}
-	if (SDL_MUSTLOCK(outputScreen)) SDL_LockSurface(outputScreen);
 }
 
+unsigned SDLImage::getWidth() const
+{
+	return image->w;
+}
+
+unsigned SDLImage::getHeight() const
+{
+	return image->h;
+}
+
+
 
 SDL_Surface* SDLImage::loadImage(const string& filename)
 {
@@ -100,6 +137,9 @@ SDL_Surface* SDLImage::loadImage(const string& filename)
 
 SDL_Surface* SDLImage::loadImage(const string& filename, double scaleFactor)
 {
+	if (scaleFactor == 1.0) {
+		return loadImage(filename);
+	}
 	SDL_Surface* picture = readImage(filename);
 	SDL_Surface* scaled = scaleImage32(picture,
 		unsigned(picture->w * scaleFactor),
@@ -138,17 +178,10 @@ SDL_Surface* SDLImage::loadImage(const string& filename,
 
 SDL_Surface* SDLImage::readImage(const string& filename)
 {
-	string filePath;
-	// Note: It is essential the File object is destroyed before the IMG_Load
-	//       function is called, because if the file is open in read/write
-	//       mode, the image loading will fail.
-	{
-		File file(filename);
-		filePath = file.getLocalName();
-	}
-	SDL_Surface* result = IMG_Load(filePath.c_str());
+	LocalFileReference file(filename);
+	SDL_Surface* result = IMG_Load(file.getFilename().c_str());
 	if (!result) {
-		throw MSXException("File \"" + filePath + "\" is not a valid image");
+		throw MSXException("File \"" + filename + "\" is not a valid image");
 	}
 	return result;
 }
