@@ -1,4 +1,4 @@
-// $Id: SDLVideoSystem.cc 6131 2007-03-02 12:00:24Z m9710797 $
+// $Id: SDLVideoSystem.cc 8377 2008-11-30 20:56:25Z manuelbi $
 
 #include "SDLVideoSystem.hh"
 #include "SDLVisibleSurface.hh"
@@ -9,9 +9,14 @@
 #include "Display.hh"
 #include "RenderSettings.hh"
 #include "BooleanSetting.hh"
+#include "EnumSetting.hh"
 #include "IntegerSetting.hh"
 #include "EventDistributor.hh"
 #include "InputEventGenerator.hh"
+#include "VDP.hh"
+#include "V9990.hh"
+#include "build-info.hh"
+#include "openmsx.hh"
 #include <cassert>
 
 #include "components.hh"
@@ -33,11 +38,10 @@ SDLVideoSystem::SDLVideoSystem(Reactor& reactor)
 
 	console = screen->createConsoleLayer(reactor);
 	snowLayer = screen->createSnowLayer();
-	iconLayer = screen->createIconLayer(
-		reactor.getCommandController(), display, reactor.getIconStatus());
+	osdGuiLayer = screen->createOSDGUILayer(display.getOSDGUI());
 	display.addLayer(*console);
 	display.addLayer(*snowLayer);
-	display.addLayer(*iconLayer);
+	display.addLayer(*osdGuiLayer);
 
 	renderSettings.getScaleFactor().attach(*this);
 
@@ -47,49 +51,60 @@ SDLVideoSystem::SDLVideoSystem(Reactor& reactor)
 
 SDLVideoSystem::~SDLVideoSystem()
 {
+	PRT_DEBUG("Destructing SDLVideoSystem... ");
+	PRT_DEBUG("Unregistering RESIZE_EVENT... ");
 	reactor.getEventDistributor().unregisterEventListener(
 		OPENMSX_RESIZE_EVENT, *this);
 
+	PRT_DEBUG("Detach from scale factor setting... ");
 	renderSettings.getScaleFactor().detach(*this);
 
-	display.removeLayer(*iconLayer);
+	PRT_DEBUG("Removing osdGuiLayer... ");
+	display.removeLayer(*osdGuiLayer);
+	PRT_DEBUG("Removing snowLayer... ");
 	display.removeLayer(*snowLayer);
+	PRT_DEBUG("Removing consoleLayer... ");
 	display.removeLayer(*console);
+	PRT_DEBUG("Destructing SDLVideoSystem... DONE!");
 }
 
 Rasterizer* SDLVideoSystem::createRasterizer(VDP& vdp)
 {
+	MSXMotherBoard& motherBoard = vdp.getMotherBoard();
 	switch (renderSettings.getRenderer().getValue()) {
 	case RendererFactory::SDL:
 	case RendererFactory::SDLGL_FB16:
 	case RendererFactory::SDLGL_FB32:
-		switch (screen->getFormat()->BytesPerPixel) {
+		switch (screen->getSDLFormat().BytesPerPixel) {
+#if HAVE_16BPP
 		case 2:
-			return new SDLRasterizer<Uint16>(
+			return new SDLRasterizer<word>(
 				vdp, display, *screen,
-				std::auto_ptr<PostProcessor>(new FBPostProcessor<Uint16>(
-					reactor.getCommandController(),
-					display, *screen, VIDEO_MSX, 640, 240)));
+				std::auto_ptr<PostProcessor>(new FBPostProcessor<word>(
+					motherBoard, display, *screen,
+					VIDEO_MSX, 640, 240)));
+#endif
+#if HAVE_32BPP
 		case 4:
-			return new SDLRasterizer<Uint32>(
+			return new SDLRasterizer<unsigned>(
 				vdp, display, *screen,
-				std::auto_ptr<PostProcessor>(new FBPostProcessor<Uint32>(
-					reactor.getCommandController(),
-					display, *screen, VIDEO_MSX, 640, 240)));
+				std::auto_ptr<PostProcessor>(new FBPostProcessor<unsigned>(
+					motherBoard, display, *screen,
+					VIDEO_MSX, 640, 240)));
+#endif
 		default:
 			assert(false);
 			return NULL;
 		}
 #ifdef COMPONENT_GL
 	case RendererFactory::SDLGL:
-		return new GLRasterizer(
-			reactor.getCommandController(), vdp, display, *screen);
+		return new GLRasterizer(vdp, display, *screen);
 	case RendererFactory::SDLGL_PP:
-		return new SDLRasterizer<Uint32>(
+		return new SDLRasterizer<unsigned>(
 			vdp, display, *screen,
 			std::auto_ptr<PostProcessor>(new GLPostProcessor(
-				reactor.getCommandController(),
-				display, *screen, VIDEO_MSX, 640, 240)));
+				motherBoard, display, *screen,
+				VIDEO_MSX, 640, 240)));
 #endif
 	default:
 		assert(false);
@@ -99,23 +114,28 @@ Rasterizer* SDLVideoSystem::createRasterizer(VDP& vdp)
 
 V9990Rasterizer* SDLVideoSystem::createV9990Rasterizer(V9990& vdp)
 {
+	MSXMotherBoard& motherBoard = vdp.getMotherBoard();
 	switch (renderSettings.getRenderer().getValue()) {
 	case RendererFactory::SDL:
 	case RendererFactory::SDLGL_FB16:
 	case RendererFactory::SDLGL_FB32:
-		switch (screen->getFormat()->BytesPerPixel) {
+		switch (screen->getSDLFormat().BytesPerPixel) {
+#if HAVE_16BPP
 		case 2:
-			return new V9990SDLRasterizer<Uint16>(
+			return new V9990SDLRasterizer<word>(
 				vdp, display, *screen,
-				std::auto_ptr<PostProcessor>(new FBPostProcessor<Uint16>(
-					reactor.getCommandController(),
-					display, *screen, VIDEO_GFX9000, 1280, 240)));
+				std::auto_ptr<PostProcessor>(new FBPostProcessor<word>(
+					motherBoard, display, *screen,
+					VIDEO_GFX9000, 1280, 240)));
+#endif
+#if HAVE_32BPP
 		case 4:
-			return new V9990SDLRasterizer<Uint32>(
+			return new V9990SDLRasterizer<unsigned>(
 				vdp, display, *screen,
-				std::auto_ptr<PostProcessor>(new FBPostProcessor<Uint32>(
-					reactor.getCommandController(),
-					display, *screen, VIDEO_GFX9000, 1280, 240)));
+				std::auto_ptr<PostProcessor>(new FBPostProcessor<unsigned>(
+					motherBoard, display, *screen,
+					VIDEO_GFX9000, 1280, 240)));
+#endif
 		default:
 			assert(false);
 			return NULL;
@@ -127,8 +147,8 @@ V9990Rasterizer* SDLVideoSystem::createV9990Rasterizer(V9990& vdp)
 		return new V9990SDLRasterizer<unsigned>(
 			vdp, display, *screen,
 			std::auto_ptr<PostProcessor>(new GLPostProcessor(
-				reactor.getCommandController(),
-				display, *screen, VIDEO_GFX9000, 1280, 240)));
+				motherBoard, display, *screen,
+				VIDEO_GFX9000, 1280, 240)));
 #endif
 	default:
 		assert(false);
@@ -179,9 +199,9 @@ bool SDLVideoSystem::checkSettings()
 	return screen->setFullScreen(fullScreenTarget);
 }
 
-bool SDLVideoSystem::prepare()
+void SDLVideoSystem::prepare()
 {
-	return screen->init();
+	screen->init();
 }
 
 void SDLVideoSystem::flush()
@@ -199,6 +219,11 @@ void SDLVideoSystem::setWindowTitle(const std::string& title)
 	screen->setWindowTitle(title);
 }
 
+OutputSurface* SDLVideoSystem::getOutputSurface()
+{
+	return screen.get();
+}
+
 void SDLVideoSystem::resize()
 {
 	unsigned width, height;
