@@ -1,4 +1,4 @@
-// $Id: SaI3xScaler.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: SaI3xScaler.cc 8351 2008-11-12 18:20:29Z m9710797 $
 
 // 2xSaI is Copyright (c) 1999-2001 by Derek Liauw Kie Fa.
 //   http://elektron.its.tudelft.nl/~dalikifa/
@@ -11,6 +11,7 @@
 #include "OutputSurface.hh"
 #include "MemoryOps.hh"
 #include "openmsx.hh"
+#include "build-info.hh"
 #include <cassert>
 
 namespace openmsx {
@@ -27,18 +28,18 @@ void SaI3xScaler<Pixel>::scaleBlank1to3(
 		FrameSource& src, unsigned srcStartY, unsigned srcEndY,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
+	dst.lock();
 	unsigned stopDstY = (dstEndY == dst.getHeight())
 	                  ? dstEndY : dstEndY - 3;
 	unsigned srcY = srcStartY, dstY = dstStartY;
 	MemoryOps::MemSet<Pixel, MemoryOps::STREAMING> memset;
 	for (/* */; dstY < stopDstY; srcY += 1, dstY += 3) {
-		Pixel* dummy = 0;
-		Pixel color = src.getLinePtr(srcY, dummy)[0];
-		Pixel* dstLine0 = dst.getLinePtr(dstY + 0, dummy);
+		Pixel color = src.getLinePtr<Pixel>(srcY)[0];
+		Pixel* dstLine0 = dst.getLinePtrDirect<Pixel>(dstY + 0);
 		memset(dstLine0, dst.getWidth(), color);
-		Pixel* dstLine1 = dst.getLinePtr(dstY + 1, dummy);
+		Pixel* dstLine1 = dst.getLinePtrDirect<Pixel>(dstY + 1);
 		memset(dstLine1, dst.getWidth(), color);
-		Pixel* dstLine2 = dst.getLinePtr(dstY + 2, dummy);
+		Pixel* dstLine2 = dst.getLinePtrDirect<Pixel>(dstY + 2);
 		memset(dstLine2, dst.getWidth(), color);
 	}
 	if (dstY != dst.getHeight()) {
@@ -67,7 +68,7 @@ word bilinear<word>(unsigned a, unsigned b, unsigned x)
 {
 	if (a == b) return a;
 
-	const unsigned areaB = x >> 11; //reduce 16 bit fraction to 5 bits
+	const unsigned areaB = x >> 11; // reduce 16 bit fraction to 5 bits
 	const unsigned areaA = 0x20 - areaB;
 
 	a = (a & redblueMask) | ((a & greenMask) << 16);
@@ -81,7 +82,7 @@ unsigned bilinear<unsigned>(unsigned a, unsigned b, unsigned x)
 {
 	if (a == b) return a;
 
-	const unsigned areaB = x >> 8; //reduce 16 bit fraction to 8 bits
+	const unsigned areaB = x >> 8; // reduce 16 bit fraction to 8 bits
 	const unsigned areaA = 0x100 - areaB;
 
 	const unsigned result0 =
@@ -221,14 +222,14 @@ class PixelStripRepeater
 {
 public:
 	template <typename Pixel>
-	inline static void fill(Pixel* &dp, unsigned sa) {
+	inline static void fill(Pixel*& dp, unsigned sa) {
 		*dp++ = sa;
 		PixelStripRepeater<i - 1>::template fill<Pixel>(dp, sa);
 	}
 
 	template <unsigned NX, unsigned y, typename Pixel>
 	inline static void blendBackslash(
-		Pixel* &dp,
+		Pixel*& dp,
 		unsigned sa, unsigned sb, unsigned sc, unsigned sd,
 		unsigned se, unsigned sg, unsigned sj, unsigned sl)
 	{
@@ -256,7 +257,7 @@ public:
 
 	template <unsigned NX, unsigned y, typename Pixel>
 	inline static void blendSlash(
-		Pixel* &dp,
+		Pixel*& dp,
 		unsigned sa, unsigned sb, unsigned sc, unsigned sd,
 		unsigned sf, unsigned sh, unsigned si, unsigned sk)
 	{
@@ -286,7 +287,7 @@ public:
 
 	template <unsigned NX, unsigned y, typename Pixel>
 	inline static void blend4(
-		Pixel* &dp, unsigned sa, unsigned sb, unsigned sc, unsigned sd)
+		Pixel*& dp, unsigned sa, unsigned sb, unsigned sc, unsigned sd)
 	{
 		const unsigned x = ((NX - i) << 16) / NX;
 		*dp++ = Blender<Pixel>::template blend<x, y>(sa, sb, sc, sd);
@@ -327,8 +328,7 @@ public:
 		const Pixel* src3, unsigned srcWidth,
 		OutputSurface& dst, unsigned& dstY)
 	{
-		Pixel* dummy = 0;
-		Pixel* dp = dst.getLinePtr(dstY++, dummy);
+		Pixel* dp = dst.getLinePtrDirect<Pixel>(dstY++);
 		// Calculate fixed point coordinate.
 		const unsigned y1 = ((NY - i) << 16) / NY;
 
@@ -395,13 +395,13 @@ void SaI3xScaler<Pixel>::scaleFixed(FrameSource& src,
 	assert(dst.getWidth() == srcWidth * NX);
 	assert(dst.getHeight() == src.getHeight() * NY);
 
-	Pixel* const dummy = 0;
+	dst.lock();
 	int srcY = srcStartY;
-	const Pixel* src0 = src.getLinePtr(srcY - 1, srcWidth, dummy);
-	const Pixel* src1 = src.getLinePtr(srcY + 0, srcWidth, dummy);
-	const Pixel* src2 = src.getLinePtr(srcY + 1, srcWidth, dummy);
+	const Pixel* src0 = src.getLinePtr<Pixel>(srcY - 1, srcWidth);
+	const Pixel* src1 = src.getLinePtr<Pixel>(srcY + 0, srcWidth);
+	const Pixel* src2 = src.getLinePtr<Pixel>(srcY + 1, srcWidth);
 	for (unsigned dstY = dstStartY; dstY < dstEndY; srcY++) {
-		const Pixel* src3 = src.getLinePtr(srcY + 2, srcWidth, dummy);
+		const Pixel* src3 = src.getLinePtr<Pixel>(srcY + 2, srcWidth);
 		LineRepeater<NY>::template scaleFixedLine<NX, NY, Pixel>(
 			src0, src1, src2, src3, srcWidth, dst, dstY);
 		src0 = src1;
@@ -415,6 +415,8 @@ void SaI3xScaler<Pixel>::scaleAny(FrameSource& src,
 	unsigned srcStartY, unsigned /*srcEndY*/, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
+	dst.lock();
+
 	// Calculate fixed point end coordinates and deltas.
 	const unsigned wfinish = (srcWidth - 1) << 16;
 	const unsigned dw = wfinish / (dst.getWidth() - 1);
@@ -425,15 +427,14 @@ void SaI3xScaler<Pixel>::scaleAny(FrameSource& src,
 	for (unsigned dstY = dstStartY; dstY < dstEndY; dstY++) {
 		// Get source line pointers.
 		int line = srcStartY + (h >> 16);
-		Pixel* const dummy = 0;
 		// TODO possible optimization: reuse srcN from previous step
-		const Pixel* src0 = src.getLinePtr(line - 1, srcWidth, dummy);
-		const Pixel* src1 = src.getLinePtr(line + 0, srcWidth, dummy);
-		const Pixel* src2 = src.getLinePtr(line + 1, srcWidth, dummy);
-		const Pixel* src3 = src.getLinePtr(line + 2, srcWidth, dummy);
+		const Pixel* src0 = src.getLinePtr<Pixel>(line - 1, srcWidth);
+		const Pixel* src1 = src.getLinePtr<Pixel>(line + 0, srcWidth);
+		const Pixel* src2 = src.getLinePtr<Pixel>(line + 1, srcWidth);
+		const Pixel* src3 = src.getLinePtr<Pixel>(line + 2, srcWidth);
 
 		// Get destination line pointer.
-		Pixel* dp = dst.getLinePtr(dstY, dummy);
+		Pixel* dp = dst.getLinePtrDirect<Pixel>(dstY);
 
 		// Fractional parts of the fixed point Y coordinates.
 		const unsigned y1 = h & 0xffff;
@@ -532,8 +533,11 @@ void SaI3xScaler<Pixel>::scale1x1to3x3(FrameSource& src,
 
 
 // Force template instantiation.
+#if HAVE_16BPP
 template class SaI3xScaler<word>;
-template class SaI3xScaler<unsigned int>;
+#endif
+#if HAVE_32BPP
+template class SaI3xScaler<unsigned>;
+#endif
 
 } // namespace openmsx
-
