@@ -1,4 +1,4 @@
-// $Id: Scale2xScaler.cc 6439 2007-05-05 14:54:01Z m9710797 $
+// $Id: Scale2xScaler.cc 8351 2008-11-12 18:20:29Z m9710797 $
 
 /*
 Original code: Copyright (C) 2001-2003 Andrea Mazzoleni
@@ -17,6 +17,7 @@ Visit the Scale2x site for info:
 #include "OutputSurface.hh"
 #include "HostCPU.hh"
 #include "openmsx.hh"
+#include "build-info.hh"
 
 namespace openmsx {
 
@@ -207,7 +208,7 @@ void Scale2xScaler<Pixel>::scaleLineHalf_1on2(Pixel* dst,
 			"movq	%%mm2, %%mm5;"       // n1 | xx
 			"pcmpeqd (%2), %%mm3;"       // n1 = s1 | n2 = s2
 			"punpckldq %%mm5, %%mm5;"    // n1 | n1
-			"movq	%%mm3, %%mm4;"	     // n1 = s1 | n2 = s2
+			"movq	%%mm3, %%mm4;"       // n1 = s1 | n2 = s2
 			"movq	%%mm0, %%mm7;"       // xx | w1
 			"punpckldq %%mm4, %%mm4;"    // n1 = s1 | n1 = s1
 			"punpckhdq %%mm1, %%mm7;"    // w1 | e1
@@ -1079,15 +1080,15 @@ void Scale2xScaler<Pixel>::scale1x1to2x2(FrameSource& src,
 	unsigned srcStartY, unsigned /*srcEndY*/, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	Pixel* const dummy = 0;
+	dst.lock();
 	int srcY = srcStartY;
-	const Pixel* srcPrev = src.getLinePtr(srcY - 1, srcWidth, dummy);
-	const Pixel* srcCurr = src.getLinePtr(srcY + 0, srcWidth, dummy);
+	const Pixel* srcPrev = src.getLinePtr<Pixel>(srcY - 1, srcWidth);
+	const Pixel* srcCurr = src.getLinePtr<Pixel>(srcY + 0, srcWidth);
 	for (unsigned dstY = dstStartY; dstY < dstEndY; srcY += 1, dstY += 2) {
-		const Pixel* srcNext = src.getLinePtr(srcY + 1, srcWidth, dummy);
-		Pixel* dstUpper = dst.getLinePtr(dstY + 0, dummy);
+		const Pixel* srcNext = src.getLinePtr<Pixel>(srcY + 1, srcWidth);
+		Pixel* dstUpper = dst.getLinePtrDirect<Pixel>(dstY + 0);
 		scaleLineHalf_1on2(dstUpper, srcPrev, srcCurr, srcNext, srcWidth);
-		Pixel* dstLower = dst.getLinePtr(dstY + 1, dummy);
+		Pixel* dstLower = dst.getLinePtrDirect<Pixel>(dstY + 1);
 		scaleLineHalf_1on2(dstLower, srcNext, srcCurr, srcPrev, srcWidth);
 		srcPrev = srcCurr;
 		srcCurr = srcNext;
@@ -1099,24 +1100,27 @@ void Scale2xScaler<Pixel>::scale1x1to1x2(FrameSource& src,
 	unsigned srcStartY, unsigned /*srcEndY*/, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	Pixel* const dummy = 0;
+	dst.lock();
 	int srcY = srcStartY;
-	const Pixel* srcPrev = src.getLinePtr(srcY - 1, srcWidth, dummy);
-	const Pixel* srcCurr = src.getLinePtr(srcY + 0, srcWidth, dummy);
+	const Pixel* srcPrev = src.getLinePtr<Pixel>(srcY - 1, srcWidth);
+	const Pixel* srcCurr = src.getLinePtr<Pixel>(srcY + 0, srcWidth);
 	for (unsigned dstY = dstStartY; dstY < dstEndY; srcY += 1, dstY += 2) {
-		const Pixel* srcNext = src.getLinePtr(srcY + 1, srcWidth, dummy);
-		Pixel* dstUpper = dst.getLinePtr(dstY + 0, dummy);
+		const Pixel* srcNext = src.getLinePtr<Pixel>(srcY + 1, srcWidth);
+		Pixel* dstUpper = dst.getLinePtrDirect<Pixel>(dstY + 0);
 		scaleLineHalf_1on1(dstUpper, srcPrev, srcCurr, srcNext, srcWidth);
-		Pixel* dstLower = dst.getLinePtr(dstY + 1, dummy);
+		Pixel* dstLower = dst.getLinePtrDirect<Pixel>(dstY + 1);
 		scaleLineHalf_1on1(dstLower, srcNext, srcCurr, srcPrev, srcWidth);
 		srcPrev = srcCurr;
 		srcCurr = srcNext;
 	}
 }
 
-
 // Force template instantiation.
+#if HAVE_16BPP
 template class Scale2xScaler<word>;
-template class Scale2xScaler<unsigned int>;
+#endif
+#if HAVE_32BPP
+template class Scale2xScaler<unsigned>;
+#endif
 
 } // namespace openmsx
