@@ -1,4 +1,4 @@
-// $Id: Simple3xScaler.cc 6598 2007-06-05 23:02:27Z mthuurne $
+// $Id: Simple3xScaler.cc 7569 2008-01-23 22:26:20Z m9710797 $
 
 #include "Simple3xScaler.hh"
 #include "LineScalers.hh"
@@ -8,6 +8,7 @@
 #include "MemoryOps.hh"
 #include "Multiply32.hh"
 #include "HostCPU.hh"
+#include "build-info.hh"
 
 namespace openmsx {
 
@@ -45,36 +46,36 @@ void Simple3xScaler<Pixel>::doScale1(FrameSource& src,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY,
 	ScaleOp& scale)
 {
+	dst.lock();
 	int scanlineFactor = settings.getScanlineFactor();
 	unsigned y = dstStartY;
-	Pixel* dummy = 0;
-	const Pixel* srcLine = src.getLinePtr(srcStartY++, srcWidth, dummy);
-	Pixel* prevDstLine0 = dst.getLinePtr(y + 0, dummy);
+	const Pixel* srcLine = src.getLinePtr<Pixel>(srcStartY++, srcWidth);
+	Pixel* prevDstLine0 = dst.getLinePtrDirect<Pixel>(y + 0);
 	scale(srcLine, prevDstLine0, dst.getWidth());
 
 	Scale_1on1<Pixel> copy;
-	Pixel* dstLine1     = dst.getLinePtr(y + 1, dummy);
+	Pixel* dstLine1     = dst.getLinePtrDirect<Pixel>(y + 1);
 	copy(prevDstLine0, dstLine1, dst.getWidth());
 
 	for (/* */; (y + 4) < dstEndY; y += 3, srcStartY += 1) {
-		srcLine = src.getLinePtr(srcStartY, srcWidth, dummy);
-		Pixel* dstLine0 = dst.getLinePtr(y + 3, dummy);
+		srcLine = src.getLinePtr<Pixel>(srcStartY, srcWidth);
+		Pixel* dstLine0 = dst.getLinePtrDirect<Pixel>(y + 3);
 		scale(srcLine, dstLine0, dst.getWidth());
 
-		Pixel* dstLine1 = dst.getLinePtr(y + 4, dummy);
+		Pixel* dstLine1 = dst.getLinePtrDirect<Pixel>(y + 4);
 		copy(dstLine0, dstLine1, dst.getWidth());
 
-		Pixel* dstLine2 = dst.getLinePtr(y + 2, dummy);
+		Pixel* dstLine2 = dst.getLinePtrDirect<Pixel>(y + 2);
 		scanline.draw(prevDstLine0, dstLine0, dstLine2,
 		              scanlineFactor, dst.getWidth());
 
 		prevDstLine0 = dstLine0;
 	}
-	srcLine = src.getLinePtr(srcStartY, srcWidth, dummy);
+	srcLine = src.getLinePtr<Pixel>(srcStartY, srcWidth);
 	Pixel buf[dst.getWidth()];
 	scale(srcLine, buf, dst.getWidth());
 
-	Pixel* dstLine2 = dst.getLinePtr(y + 2, dummy);
+	Pixel* dstLine2 = dst.getLinePtrDirect<Pixel>(y + 2);
 	scanline.draw(prevDstLine0, buf, dstLine2,
 		      scanlineFactor, dst.getWidth());
 }
@@ -86,19 +87,19 @@ void Simple3xScaler<Pixel>::doScale2(FrameSource& src,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY,
 	ScaleOp& scale)
 {
-	Pixel* dummy = 0;
+	dst.lock();
 	int scanlineFactor = settings.getScanlineFactor();
 	for (unsigned srcY = srcStartY, dstY = dstStartY; dstY < dstEndY;
 	     srcY += 2, dstY += 3) {
-		const Pixel* srcLine0 = src.getLinePtr(srcY + 0, srcWidth, dummy);
-		Pixel* dstLine0 = dst.getLinePtr(dstY + 0, dummy);
+		const Pixel* srcLine0 = src.getLinePtr<Pixel>(srcY + 0, srcWidth);
+		Pixel* dstLine0 = dst.getLinePtrDirect<Pixel>(dstY + 0);
 		scale(srcLine0, dstLine0, dst.getWidth());
 
-		const Pixel* srcLine1 = src.getLinePtr(srcY + 1, srcWidth, dummy);
-		Pixel* dstLine2 = dst.getLinePtr(dstY + 2, dummy);
+		const Pixel* srcLine1 = src.getLinePtr<Pixel>(srcY + 1, srcWidth);
+		Pixel* dstLine2 = dst.getLinePtrDirect<Pixel>(dstY + 2);
 		scale(srcLine1, dstLine2, dst.getWidth());
 
-		Pixel* dstLine1 = dst.getLinePtr(dstY + 1, dummy);
+		Pixel* dstLine1 = dst.getLinePtrDirect<Pixel>(dstY + 1);
 		scanline.draw(dstLine0, dstLine2, dstLine1,
 		              scanlineFactor, dst.getWidth());
 	}
@@ -223,6 +224,7 @@ void Simple3xScaler<Pixel>::scaleBlank1to3(
 		FrameSource& src, unsigned srcStartY, unsigned srcEndY,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
+	dst.lock();
 	int scanlineFactor = settings.getScanlineFactor();
 
 	unsigned stopDstY = (dstEndY == dst.getHeight())
@@ -230,14 +232,13 @@ void Simple3xScaler<Pixel>::scaleBlank1to3(
 	unsigned srcY = srcStartY, dstY = dstStartY;
 	MemoryOps::MemSet<Pixel, MemoryOps::STREAMING> memset;
 	for (/* */; dstY < stopDstY; srcY += 1, dstY += 3) {
-		Pixel* dummy = 0;
-		Pixel color0 = src.getLinePtr(srcY, dummy)[0];
-		Pixel* dstLine0 = dst.getLinePtr(dstY + 0, dummy);
+		Pixel color0 = src.getLinePtr<Pixel>(srcY)[0];
+		Pixel* dstLine0 = dst.getLinePtrDirect<Pixel>(dstY + 0);
 		memset(dstLine0, dst.getWidth(), color0);
-		Pixel* dstLine1 = dst.getLinePtr(dstY + 1, dummy);
+		Pixel* dstLine1 = dst.getLinePtrDirect<Pixel>(dstY + 1);
 		memset(dstLine1, dst.getWidth(), color0);
 		Pixel color1 = scanline.darken(color0, scanlineFactor);
-		Pixel* dstLine2 = dst.getLinePtr(dstY + 2, dummy);
+		Pixel* dstLine2 = dst.getLinePtrDirect<Pixel>(dstY + 2);
 		memset(dstLine2, dst.getWidth(), color1);
 	}
 	if (dstY != dst.getHeight()) {
@@ -254,19 +255,19 @@ void Simple3xScaler<Pixel>::scaleBlank2to3(
 		FrameSource& src, unsigned srcStartY, unsigned /*srcEndY*/,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
+	dst.lock();
 	int scanlineFactor = settings.getScanlineFactor();
 	MemoryOps::MemSet<Pixel, MemoryOps::STREAMING> memset;
 	for (unsigned srcY = srcStartY, dstY = dstStartY;
 	     dstY < dstEndY; srcY += 2, dstY += 3) {
-		Pixel* dummy = 0;
-		Pixel color0 = src.getLinePtr(srcY + 0, dummy)[0];
-		Pixel color1 = src.getLinePtr(srcY + 1, dummy)[0];
+		Pixel color0 = src.getLinePtr<Pixel>(srcY + 0)[0];
+		Pixel color1 = src.getLinePtr<Pixel>(srcY + 1)[0];
 		Pixel color01 = scanline.darken(color0, color1, scanlineFactor);
-		Pixel* dstLine0 = dst.getLinePtr(dstY + 0, dummy);
+		Pixel* dstLine0 = dst.getLinePtrDirect<Pixel>(dstY + 0);
 		memset(dstLine0, dst.getWidth(), color0);
-		Pixel* dstLine1 = dst.getLinePtr(dstY + 1, dummy);
+		Pixel* dstLine1 = dst.getLinePtrDirect<Pixel>(dstY + 1);
 		memset(dstLine1, dst.getWidth(), color01);
-		Pixel* dstLine2 = dst.getLinePtr(dstY + 2, dummy);
+		Pixel* dstLine2 = dst.getLinePtrDirect<Pixel>(dstY + 2);
 		memset(dstLine2, dst.getWidth(), color1);
 	}
 }
@@ -491,7 +492,11 @@ void Blur_1on3<Pixel>::operator()(const Pixel* in, Pixel* out, unsigned long dst
 }
 
 // Force template instantiation.
+#if HAVE_16BPP
 template class Simple3xScaler<word>;
+#endif
+#if HAVE_32BPP
 template class Simple3xScaler<unsigned>;
+#endif
 
 } // namespace openmsx
