@@ -1,4 +1,4 @@
-// $Id: SimpleScaler.cc 6131 2007-03-02 12:00:24Z m9710797 $
+// $Id: SimpleScaler.cc 8223 2008-09-14 19:17:24Z m9710797 $
 
 #include "SimpleScaler.hh"
 #include "LineScalers.hh"
@@ -8,6 +8,7 @@
 #include "MemoryOps.hh"
 #include "HostCPU.hh"
 #include "openmsx.hh"
+#include "build-info.hh"
 #include <cassert>
 
 namespace openmsx {
@@ -32,6 +33,7 @@ void SimpleScaler<Pixel>::scaleBlank1to2(
 		FrameSource& src, unsigned srcStartY, unsigned srcEndY,
 		OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
+	dst.lock();
 	int scanlineFactor = settings.getScanlineFactor();
 
 	unsigned stopDstY = (dstEndY == dst.getHeight())
@@ -39,12 +41,11 @@ void SimpleScaler<Pixel>::scaleBlank1to2(
 	unsigned srcY = srcStartY, dstY = dstStartY;
 	MemoryOps::MemSet<Pixel, MemoryOps::STREAMING> memset;
 	for (/* */; dstY < stopDstY; srcY += 1, dstY += 2) {
-		Pixel* dummy = 0;
-		Pixel color0 = src.getLinePtr(srcY, dummy)[0];
-		Pixel* dstLine0 = dst.getLinePtr(dstY + 0, dummy);
+		Pixel color0 = src.getLinePtr<Pixel>(srcY)[0];
+		Pixel* dstLine0 = dst.getLinePtrDirect<Pixel>(dstY + 0);
 		memset(dstLine0, dst.getWidth(), color0);
 		Pixel color1 = scanline.darken(color0, scanlineFactor);
-		Pixel* dstLine1 = dst.getLinePtr(dstY + 1, dummy);
+		Pixel* dstLine1 = dst.getLinePtrDirect<Pixel>(dstY + 1);
 		memset(dstLine1, dst.getWidth(), color1);
 	}
 	if (dstY != dst.getHeight()) {
@@ -167,7 +168,7 @@ void SimpleScaler<Pixel>::blur1on2(const Pixel* pIn, Pixel* pOut, unsigned alpha
 			"movq	%%mm0, (%1);"		// pOut[2*x+0] = ..  pOut[2*x+1] = ..
 
 			"movq	%%mm1, %%mm4;"
-			"pmullw	%%mm6, %%mm1;" 		// tmp = multiply(p1, c2)
+			"pmullw	%%mm6, %%mm1;"		// tmp = multiply(p1, c2)
 			"paddw	%%mm2, %%mm1;"
 			"psrlw	$8, %%mm1;"		// f0 + tmp
 
@@ -400,21 +401,21 @@ void SimpleScaler<Pixel>::scale1x1to2x2(FrameSource& src,
 	unsigned srcStartY, unsigned /*srcEndY*/, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
+	dst.lock();
 	int blur = settings.getBlurFactor();
 	int scanlineFactor = settings.getScanlineFactor();
 
 	unsigned dstY = dstStartY;
-	Pixel* dummy = 0;
-	const Pixel* srcLine = src.getLinePtr(srcStartY++, srcWidth, dummy);
-	Pixel* prevDstLine0 = dst.getLinePtr(dstY++, dummy);
+	const Pixel* srcLine = src.getLinePtr<Pixel>(srcStartY++, srcWidth);
+	Pixel* prevDstLine0 = dst.getLinePtrDirect<Pixel>(dstY++);
 	blur1on2(srcLine, prevDstLine0, blur, srcWidth);
 
 	while (dstY < dstEndY - 1) {
-		srcLine = src.getLinePtr(srcStartY++, srcWidth, dummy);
-		Pixel* dstLine0 = dst.getLinePtr(dstY + 1, dummy);
+		srcLine = src.getLinePtr<Pixel>(srcStartY++, srcWidth);
+		Pixel* dstLine0 = dst.getLinePtrDirect<Pixel>(dstY + 1);
 		blur1on2(srcLine, dstLine0, blur, srcWidth);
 
-		Pixel* dstLine1 = dst.getLinePtr(dstY, dummy);
+		Pixel* dstLine1 = dst.getLinePtrDirect<Pixel>(dstY);
 		drawScanline(prevDstLine0, dstLine0, dstLine1, scanlineFactor,
 		             2 * srcWidth);
 
@@ -422,11 +423,11 @@ void SimpleScaler<Pixel>::scale1x1to2x2(FrameSource& src,
 		dstY += 2;
 	}
 
-	srcLine = src.getLinePtr(srcStartY++, srcWidth, dummy);
+	srcLine = src.getLinePtr<Pixel>(srcStartY++, srcWidth);
 	Pixel buf[2 * srcWidth];
 	blur1on2(srcLine, buf, blur, srcWidth);
 
-	Pixel* dstLine1 = dst.getLinePtr(dstY, dummy);
+	Pixel* dstLine1 = dst.getLinePtrDirect<Pixel>(dstY);
 	drawScanline(prevDstLine0, buf, dstLine1, scanlineFactor, 2 * srcWidth);
 }
 
@@ -435,21 +436,21 @@ void SimpleScaler<Pixel>::scale1x1to1x2(FrameSource& src,
 	unsigned srcStartY, unsigned /*srcEndY*/, unsigned srcWidth,
 	OutputSurface& dst, unsigned dstStartY, unsigned dstEndY)
 {
+	dst.lock();
 	int blur = settings.getBlurFactor();
 	int scanlineFactor = settings.getScanlineFactor();
 
 	unsigned dstY = dstStartY;
-	Pixel* dummy = 0;
-	const Pixel* srcLine = src.getLinePtr(srcStartY++, srcWidth, dummy);
-	Pixel* prevDstLine0 = dst.getLinePtr(dstY++, dummy);
+	const Pixel* srcLine = src.getLinePtr<Pixel>(srcStartY++, srcWidth);
+	Pixel* prevDstLine0 = dst.getLinePtrDirect<Pixel>(dstY++);
 	blur1on1(srcLine, prevDstLine0, blur, srcWidth);
 
 	while (dstY < dstEndY - 1) {
-		srcLine = src.getLinePtr(srcStartY++, srcWidth, dummy);
-		Pixel* dstLine0 = dst.getLinePtr(dstY + 1, dummy);
+		srcLine = src.getLinePtr<Pixel>(srcStartY++, srcWidth);
+		Pixel* dstLine0 = dst.getLinePtrDirect<Pixel>(dstY + 1);
 		blur1on1(srcLine, dstLine0, blur, srcWidth);
 
-		Pixel* dstLine1 = dst.getLinePtr(dstY + 0, dummy);
+		Pixel* dstLine1 = dst.getLinePtrDirect<Pixel>(dstY + 0);
 		drawScanline(prevDstLine0, dstLine0, dstLine1, scanlineFactor,
 		             srcWidth);
 
@@ -457,17 +458,20 @@ void SimpleScaler<Pixel>::scale1x1to1x2(FrameSource& src,
 		dstY += 2;
 	}
 
-	srcLine = src.getLinePtr(srcStartY++, srcWidth, dummy);
+	srcLine = src.getLinePtr<Pixel>(srcStartY++, srcWidth);
 	Pixel buf[srcWidth];
 	blur1on1(srcLine, buf, blur, srcWidth);
 
-	Pixel* dstLine1 = dst.getLinePtr(dstY, dummy);
+	Pixel* dstLine1 = dst.getLinePtrDirect<Pixel>(dstY);
 	drawScanline(prevDstLine0, buf, dstLine1, scanlineFactor, srcWidth);
 }
 
-
 // Force template instantiation.
+#if HAVE_16BPP
 template class SimpleScaler<word>;
-template class SimpleScaler<unsigned int>;
+#endif
+#if HAVE_32BPP
+template class SimpleScaler<unsigned>;
+#endif
 
 } // namespace openmsx
