@@ -1,4 +1,4 @@
-// $Id: VDP.cc 7199 2007-10-23 18:59:10Z m9710797 $
+// $Id: VDP.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 /*
 TODO:
@@ -30,7 +30,7 @@ TODO:
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
 #include "MSXException.hh"
-#include "MSXCliComm.hh"
+#include "CliComm.hh"
 #include <sstream>
 #include <cstring>
 #include <cassert>
@@ -42,7 +42,7 @@ class VDPRegDebug : public SimpleDebuggable
 public:
 	explicit VDPRegDebug(VDP& vdp);
 	virtual byte read(unsigned address);
-	virtual void write(unsigned address, byte value, const EmuTime& time);
+	virtual void write(unsigned address, byte value, EmuTime::param time);
 private:
 	VDP& vdp;
 };
@@ -51,7 +51,7 @@ class VDPStatusRegDebug : public SimpleDebuggable
 {
 public:
 	explicit VDPStatusRegDebug(VDP& vdp);
-	virtual byte read(unsigned address, const EmuTime& time);
+	virtual byte read(unsigned address, EmuTime::param time);
 private:
 	VDP& vdp;
 };
@@ -61,31 +61,41 @@ class VDPPaletteDebug : public SimpleDebuggable
 public:
 	explicit VDPPaletteDebug(VDP& vdp);
 	virtual byte read(unsigned address);
-	virtual void write(unsigned address, byte value, const EmuTime& time);
+	virtual void write(unsigned address, byte value, EmuTime::param time);
+private:
+	VDP& vdp;
+};
+
+class VRAMPointerDebug : public SimpleDebuggable
+{
+public:
+	explicit VRAMPointerDebug(VDP& vdp);
+	virtual byte read(unsigned address);
+	virtual void write(unsigned address, byte value, EmuTime::param time);
 private:
 	VDP& vdp;
 };
 
 
-VDP::VDP(MSXMotherBoard& motherBoard, const XMLElement& config,
-         const EmuTime& time)
-	: MSXDevice(motherBoard, config, time)
+VDP::VDP(MSXMotherBoard& motherBoard, const XMLElement& config)
+	: MSXDevice(motherBoard, config)
 	, Schedulable(motherBoard.getScheduler())
 	, cliComm(motherBoard.getMSXCliComm())
 	, vdpRegDebug      (new VDPRegDebug      (*this))
 	, vdpStatusRegDebug(new VDPStatusRegDebug(*this))
 	, vdpPaletteDebug  (new VDPPaletteDebug  (*this))
-	, frameStartTime(time)
-	, irqVertical(motherBoard.getCPU())
-	, irqHorizontal(motherBoard.getCPU())
-	, displayStartSyncTime(time)
-	, vScanSyncTime(time)
-	, hScanSyncTime(time)
+	, vramPointerDebug (new VRAMPointerDebug (*this))
+	, frameStartTime(Schedulable::getCurrentTime())
+	, irqVertical  (motherBoard, getName() + ".IRQvertical")
+	, irqHorizontal(motherBoard, getName() + ".IRQhorizontal")
+	, displayStartSyncTime(Schedulable::getCurrentTime())
+	, vScanSyncTime(Schedulable::getCurrentTime())
+	, hScanSyncTime(Schedulable::getCurrentTime())
 	, warningPrinted(false)
 {
 	interlaced = false;
 
-	std::string versionString = deviceConfig.getChildData("version");
+	std::string versionString = config.getChildData("version");
 	if (versionString == "TMS99X8A") version = TMS99X8A;
 	else if (versionString == "TMS9929A") version = TMS9929A;
 	else if (versionString == "V9938") version = V9938;
@@ -113,8 +123,9 @@ VDP::VDP(MSXMotherBoard& motherBoard, const XMLElement& config,
 	}
 
 	// Video RAM.
+	EmuTime::param time = Schedulable::getCurrentTime();
 	unsigned vramSize =
-		(isMSX1VDP() ? 16 : deviceConfig.getChildDataAsInt("vram"));
+		(isMSX1VDP() ? 16 : config.getChildDataAsInt("vram"));
 	if ((vramSize !=  16) && (vramSize !=  64) &&
 	    (vramSize != 128) && (vramSize != 192)) {
 		std::ostringstream out;
@@ -171,7 +182,7 @@ void VDP::createRenderer()
 	vram->setRenderer(renderer.get(), frameStartTime.getTime());
 }
 
-void VDP::resetInit(const EmuTime& /*time*/)
+void VDP::resetInit(EmuTime::param /*time*/)
 {
 	for (int i = 0; i < 32; i++) {
 		controlRegs[i] = 0;
@@ -187,7 +198,7 @@ void VDP::resetInit(const EmuTime& /*time*/)
 	// Note: frameStart is the actual place palTiming is written, but it
 	//       can be read before frameStart is called.
 	//       TODO: Clean up initialisation sequence.
-	palTiming = true; //controlRegs[9] & 0x02;
+	palTiming = true; // controlRegs[9] & 0x02;
 	displayMode.reset();
 	vramPointer = 0;
 	readAhead = 0;
@@ -222,7 +233,7 @@ void VDP::resetInit(const EmuTime& /*time*/)
 	memcpy(palette, V9938_PALETTE, 16 * sizeof(word));
 }
 
-void VDP::resetMasks(const EmuTime& time)
+void VDP::resetMasks(EmuTime::param time)
 {
 	// TODO: Use the updateNameBase method instead of duplicating the effort
 	//       here for the initial state.
@@ -236,7 +247,7 @@ void VDP::resetMasks(const EmuTime& time)
 	//vram->bitmapWindow.setMask(~(-1 << 17), -1 << 17, time);
 }
 
-void VDP::reset(const EmuTime& time)
+void VDP::reset(EmuTime::param time)
 {
 	removeSyncPoint(VSYNC);
 	removeSyncPoint(DISPLAY_START);
@@ -260,7 +271,7 @@ void VDP::reset(const EmuTime& time)
 	frameStart(time);
 }
 
-void VDP::executeUntil(const EmuTime& time, int userData)
+void VDP::executeUntil(EmuTime::param time, int userData)
 {
 	/*
 	PRT_DEBUG("Executing VDP at time " << time
@@ -355,7 +366,7 @@ const std::string& VDP::schedName() const
 // TODO: This approach assumes that an overscan-like approach can be used
 //       skip display start, so that the border is rendered instead.
 //       This makes sense, but it has not been tested on real MSX yet.
-void VDP::scheduleDisplayStart(const EmuTime& time)
+void VDP::scheduleDisplayStart(EmuTime::param time)
 {
 	// Remove pending DISPLAY_START sync point, if any.
 	if (displayStartSyncTime > time) {
@@ -389,7 +400,7 @@ void VDP::scheduleDisplayStart(const EmuTime& time)
 	scheduleVScan(time);
 }
 
-void VDP::scheduleVScan(const EmuTime& time)
+void VDP::scheduleVScan(EmuTime::param time)
 {
 	/*
 	cerr << "scheduleVScan @ " << (getTicksThisFrame(time) / TICKS_PER_LINE) << "\n";
@@ -418,7 +429,7 @@ void VDP::scheduleVScan(const EmuTime& time)
 	}
 }
 
-void VDP::scheduleHScan(const EmuTime& time)
+void VDP::scheduleHScan(EmuTime::param time)
 {
 	// Remove pending HSCAN sync point, if any.
 	if (hScanSyncTime > time) {
@@ -469,7 +480,7 @@ void VDP::scheduleHScan(const EmuTime& time)
 //       frame when their callback occurs.
 //       But I'm not sure how to handle the PAL/NTSC setting (which also
 //       influences the frequency at which E/O toggles).
-void VDP::frameStart(const EmuTime& time)
+void VDP::frameStart(EmuTime::param time)
 {
 	//cerr << "VDP::frameStart @ " << time << "\n";
 
@@ -501,7 +512,7 @@ void VDP::frameStart(const EmuTime& time)
 	}
 
 	// Schedule next VSYNC.
-	frameStartTime.advance(time);
+	frameStartTime.advance_fast(time);
 	setSyncPoint(frameStartTime + getTicksPerFrame(), VSYNC);
 	// Schedule DISPLAY_START, VSCAN and HSCAN.
 	scheduleDisplayStart(time);
@@ -523,7 +534,7 @@ void VDP::frameStart(const EmuTime& time)
 
 // The I/O functions.
 
-void VDP::writeIO(word port, byte value, const EmuTime& time)
+void VDP::writeIO(word port, byte value, EmuTime::param time)
 {
 	assert(isInsideFrame(time));
 	switch (port & 0x03) {
@@ -592,7 +603,7 @@ void VDP::writeIO(word port, byte value, const EmuTime& time)
 			// Thanks to hap for finding this out. :)
 			if (isMSX1VDP()) {
 				vramPointer = (vramPointer & 0x3F00) | value;
-			} 
+			}
 			dataLatch = value;
 			registerDataStored = true;
 		}
@@ -624,7 +635,7 @@ void VDP::writeIO(word port, byte value, const EmuTime& time)
 	}
 }
 
-void VDP::setPalette(int index, word grb, const EmuTime& time)
+void VDP::setPalette(int index, word grb, EmuTime::param time)
 {
 	if (palette[index] != grb) {
 		renderer->updatePalette(index, grb, time);
@@ -632,7 +643,7 @@ void VDP::setPalette(int index, word grb, const EmuTime& time)
 	}
 }
 
-byte VDP::vramRead(const EmuTime& time)
+byte VDP::vramRead(EmuTime::param time)
 {
 	byte result = readAhead;
 	int addr = (controlRegs[14] << 14) | vramPointer;
@@ -655,7 +666,7 @@ byte VDP::vramRead(const EmuTime& time)
 	return result;
 }
 
-byte VDP::peekStatusReg(byte reg, const EmuTime& time) const
+byte VDP::peekStatusReg(byte reg, EmuTime::param time) const
 {
 	switch (reg) {
 	case 0:
@@ -703,7 +714,7 @@ byte VDP::peekStatusReg(byte reg, const EmuTime& time) const
 	case 6:
 		return byte(spriteChecker->getCollisionY(time) >> 8) | 0xFC;
 	case 7:
-		return cmdEngine->readColour(time);
+		return cmdEngine->readColor(time);
 	case 8:
 		return byte(cmdEngine->getBorderX(time));
 	case 9:
@@ -713,7 +724,7 @@ byte VDP::peekStatusReg(byte reg, const EmuTime& time) const
 	}
 }
 
-byte VDP::readStatusReg(byte reg, const EmuTime& time)
+byte VDP::readStatusReg(byte reg, EmuTime::param time)
 {
 	byte ret = peekStatusReg(reg, time);
 	switch (reg) {
@@ -731,13 +742,13 @@ byte VDP::readStatusReg(byte reg, const EmuTime& time)
 		spriteChecker->resetCollision();
 		break;
 	case 7:
-		cmdEngine->resetColour();
+		cmdEngine->resetColor();
 		break;
 	}
 	return ret;
 }
 
-byte VDP::readIO(word port, const EmuTime& time)
+byte VDP::readIO(word port, EmuTime::param time)
 {
 	assert(isInsideFrame(time));
 	switch (port & 0x03) {
@@ -757,13 +768,13 @@ byte VDP::readIO(word port, const EmuTime& time)
 	}
 }
 
-byte VDP::peekIO(word /*port*/, const EmuTime& /*time*/) const
+byte VDP::peekIO(word /*port*/, EmuTime::param /*time*/) const
 {
 	// TODO not implemented
 	return 0xFF;
 }
 
-void VDP::changeRegister(byte reg, byte val, const EmuTime& time)
+void VDP::changeRegister(byte reg, byte val, EmuTime::param time)
 {
 	//PRT_DEBUG("VDP[" << (int)reg << "] = " << hex << (int)val << dec);
 
@@ -926,6 +937,11 @@ void VDP::changeRegister(byte reg, byte val, const EmuTime& time)
 		if (change & 0x20) { // IE0
 			if (!(val & 0x20)) irqVertical.reset();
 		}
+		if ((change & 0x80) && isMSX1VDP()) {
+			// confirmed: VRAM remapping does not happen on a V99x8
+			// see VDPVRAM for details on the remapping itself
+			vram->change4k8kMapping(val & 0x80);
+		}
 		break;
 	case 2:
 		updateNameBase(time);
@@ -946,7 +962,7 @@ void VDP::changeRegister(byte reg, byte val, const EmuTime& time)
 		break;
 	case 9:
 		if ((val & 1) && ! warningPrinted) {
-			warningPrinted=true;
+			warningPrinted = true;
 			cliComm.printWarning
 				("The running MSX software has set bit 0 of VDP register 9 "
 				 "(dot clock direction) to one. In an ordinary MSX, "
@@ -984,7 +1000,7 @@ void VDP::changeRegister(byte reg, byte val, const EmuTime& time)
 	}
 }
 
-void VDP::syncAtNextLine(SyncType type, const EmuTime& time)
+void VDP::syncAtNextLine(SyncType type, EmuTime::param time)
 {
 	int line = getTicksThisFrame(time) / TICKS_PER_LINE;
 	int ticks = (line + 1) * TICKS_PER_LINE;
@@ -992,7 +1008,7 @@ void VDP::syncAtNextLine(SyncType type, const EmuTime& time)
 	setSyncPoint(nextTime, type);
 }
 
-void VDP::updateNameBase(const EmuTime& time)
+void VDP::updateNameBase(EmuTime::param time)
 {
 	int base = (controlRegs[2] << 10) | ~(-1 << 10);
 	// TODO:
@@ -1020,7 +1036,7 @@ void VDP::updateNameBase(const EmuTime& time)
 	vram->nameTable.setMask(base, indexMask, time);
 }
 
-void VDP::updateColourBase(const EmuTime& time)
+void VDP::updateColourBase(EmuTime::param time)
 {
 	int base = (controlRegs[10] << 14) | (controlRegs[3] << 6) | ~(-1 << 6);
 	renderer->updateColourBase(base, time);
@@ -1042,7 +1058,7 @@ void VDP::updateColourBase(const EmuTime& time)
 	}
 }
 
-void VDP::updatePatternBase(const EmuTime& time)
+void VDP::updatePatternBase(EmuTime::param time)
 {
 	int base = (controlRegs[4] << 11) | ~(-1 << 11);
 	renderer->updatePatternBase(base, time);
@@ -1065,7 +1081,7 @@ void VDP::updatePatternBase(const EmuTime& time)
 	}
 }
 
-void VDP::updateSpriteAttributeBase(const EmuTime& time)
+void VDP::updateSpriteAttributeBase(EmuTime::param time)
 {
 	int mode = displayMode.getSpriteMode();
 	if (mode == 0) {
@@ -1080,7 +1096,7 @@ void VDP::updateSpriteAttributeBase(const EmuTime& time)
 	}
 }
 
-void VDP::updateSpritePatternBase(const EmuTime& time)
+void VDP::updateSpritePatternBase(EmuTime::param time)
 {
 	if (displayMode.getSpriteMode() == 0) {
 		vram->spritePatternTable.disable(time);
@@ -1090,7 +1106,7 @@ void VDP::updateSpritePatternBase(const EmuTime& time)
 	vram->spritePatternTable.setMask(base, -1 << 11, time);
 }
 
-void VDP::updateDisplayMode(DisplayMode newMode, const EmuTime& time)
+void VDP::updateDisplayMode(DisplayMode newMode, EmuTime::param time)
 {
 	//PRT_DEBUG("VDP: mode " << newMode);
 
@@ -1150,7 +1166,7 @@ byte VDPRegDebug::read(unsigned address)
 	}
 }
 
-void VDPRegDebug::write(unsigned address, byte value, const EmuTime& time)
+void VDPRegDebug::write(unsigned address, byte value, EmuTime::param time)
 {
 	vdp.changeRegister(address, value, time);
 }
@@ -1165,11 +1181,12 @@ VDPStatusRegDebug::VDPStatusRegDebug(VDP& vdp_)
 {
 }
 
-byte VDPStatusRegDebug::read(unsigned address, const EmuTime& time)
+byte VDPStatusRegDebug::read(unsigned address, EmuTime::param time)
 {
 	return vdp.peekStatusReg(address, time);
 }
 
+
 // VDPPaletteDebug
 
 VDPPaletteDebug::VDPPaletteDebug(VDP& vdp_)
@@ -1185,7 +1202,7 @@ byte VDPPaletteDebug::read(unsigned address)
 	return (address & 1) ? (grb >> 8) : (grb & 0xff);
 }
 
-void VDPPaletteDebug::write(unsigned address, byte value, const EmuTime& time)
+void VDPPaletteDebug::write(unsigned address, byte value, EmuTime::param time)
 {
 	int index = address / 2;
 	word grb = vdp.getPalette(index);
@@ -1195,5 +1212,89 @@ void VDPPaletteDebug::write(unsigned address, byte value, const EmuTime& time)
 	vdp.setPalette(index, grb, time);
 }
 
-} // namespace openmsx
 
+// class VRAMPointerDebug
+
+VRAMPointerDebug::VRAMPointerDebug(VDP& vdp_)
+	: SimpleDebuggable(vdp_.getMotherBoard(), "VRAM pointer",
+	                   "VDP VRAM pointer (14 lower bits)", 2)
+	, vdp(vdp_)
+{
+}
+
+byte VRAMPointerDebug::read(unsigned address)
+{
+	if (address & 1) {
+		return vdp.vramPointer >> 8;  // TODO add read/write mode?
+	} else {
+		return vdp.vramPointer & 0xFF;
+	}
+}
+
+void VRAMPointerDebug::write(unsigned address, byte value, EmuTime::param /*time*/)
+{
+	int& ptr = vdp.vramPointer;
+	if (address & 1) {
+		ptr = (ptr & 0x00FF) | ((value & 0x3F) << 8);
+	} else {
+		ptr = (ptr & 0xFF00) | value;
+	}
+}
+
+
+template<typename Archive>
+void VDP::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<MSXDevice>(*this);
+	ar.template serializeBase<Schedulable>(*this);
+
+	// not serialized
+	//    std::auto_ptr<Renderer> renderer;
+	//    VdpVersion version;
+	//    int controlRegMask;
+	//    byte controlValueMasks[32];
+	//    bool warningPrinted;
+
+	ar.serialize("irqVertical", irqVertical);
+	ar.serialize("irqHorizontal", irqHorizontal);
+	ar.serialize("frameStartTime", frameStartTime);
+	ar.serialize("displayStartSyncTime", displayStartSyncTime);
+	ar.serialize("vScanSyncTime", vScanSyncTime);
+	ar.serialize("hScanSyncTime", hScanSyncTime);
+	ar.serialize("displayStart", displayStart);
+	ar.serialize("horizontalScanOffset", horizontalScanOffset);
+	ar.serialize("lineZero", lineZero);
+	ar.serialize("horizontalAdjust", horizontalAdjust);
+	ar.serialize("verticalAdjust", verticalAdjust);
+	ar.serialize("registers", controlRegs);
+	ar.serialize("blinkCount", blinkCount);
+	ar.serialize("vramPointer", vramPointer);
+	ar.serialize("palette", palette);
+	ar.serialize("isDisplayArea", isDisplayArea);
+	ar.serialize("palTiming", palTiming);
+	ar.serialize("interlaced", interlaced);
+	ar.serialize("statusReg0", statusReg0);
+	ar.serialize("statusReg1", statusReg1);
+	ar.serialize("statusReg2", statusReg2);
+	ar.serialize("blinkState", blinkState);
+	ar.serialize("dataLatch", dataLatch);
+	ar.serialize("registerDataStored", registerDataStored);
+	ar.serialize("paletteDataStored", paletteDataStored);
+	ar.serialize("readAhead", readAhead);
+	ar.serialize("cpuExtendedVram", cpuExtendedVram);
+	ar.serialize("displayEnabled", displayEnabled);
+	byte mode = displayMode.getByte();
+	ar.serialize("displayMode", mode);
+	displayMode.setByte(mode);
+
+	ar.serialize("cmdEngine", *cmdEngine);
+	ar.serialize("vram", *vram);
+	ar.serialize("spriteChecker", *spriteChecker); // must come after displayMode
+	if (ar.isLoader()) {
+		renderer->reset(Schedulable::getCurrentTime());
+	}
+}
+INSTANTIATE_SERIALIZE_METHODS(VDP);
+REGISTER_MSXDEVICE(VDP, "VDP");
+
+} // namespace openmsx
