@@ -1,13 +1,13 @@
-// $Id: VideoSourceSetting.cc 5618 2006-09-10 12:42:47Z manuelbi $
+// $Id: VideoSourceSetting.cc 7698 2008-03-20 20:43:12Z m9710797 $
 
 #include "VideoSourceSetting.hh"
+#include "CommandException.hh"
 
 namespace openmsx {
 
 VideoSourceSettingPolicy::VideoSourceSettingPolicy(
-		CommandController& commandController,
-		const std::string& name, const Map& map)
-	: EnumSettingPolicy<VideoSource>(commandController, name, map)
+		CommandController& commandController, const Map& map)
+	: EnumSettingPolicy<VideoSource>(commandController, map)
 {
 }
 
@@ -45,7 +45,7 @@ const char* const VIDEOSOURCE = "videosource";
 VideoSourceSetting::VideoSourceSetting(CommandController& commandController)
 	: SettingImpl<VideoSourceSettingPolicy>(commandController,
 		VIDEOSOURCE, "selects the video source to display on the screen",
-		VIDEO_MSX, Setting::DONT_SAVE, VIDEOSOURCE, getVideoSourceMap())
+		VIDEO_MSX, Setting::DONT_SAVE, getVideoSourceMap())
 {
 }
 
@@ -53,12 +53,16 @@ void VideoSourceSetting::registerVideoSource(VideoSource source)
 {
 	activeSources.insert(source);
 	notify();
+	notifyPropertyChange();
 }
 
 void VideoSourceSetting::unregisterVideoSource(VideoSource source)
 {
-	activeSources.erase(source);
+	ActiveSources::iterator it = activeSources.find(source);
+	assert(it != activeSources.end());
+	activeSources.erase(it);
 	notify();
+	notifyPropertyChange();
 }
 
 
