@@ -1,4 +1,4 @@
-// $Id: VideoSystem.cc 4855 2005-11-12 19:29:04Z m9710797 $
+// $Id: VideoSystem.cc 8278 2008-10-01 17:10:24Z m9710797 $
 
 #include "VideoSystem.hh"
 #include "CommandException.hh"
@@ -10,26 +10,20 @@ VideoSystem::~VideoSystem()
 {
 }
 
-Rasterizer* VideoSystem::createRasterizer(VDP& /*vdp*/)
-{
-	assert(false);
-	return 0;
-}
-
 bool VideoSystem::checkSettings()
 {
 	return true;
 }
 
-bool VideoSystem::prepare()
+void VideoSystem::prepare()
 {
-	return true;
+	// nothing
 }
 
 void VideoSystem::takeScreenShot(const std::string& /*filename*/)
 {
 	throw CommandException(
-		"Taking screenshot not possible with current renderer." );
+		"Taking screenshot not possible with current renderer.");
 }
 
 void VideoSystem::setWindowTitle(const std::string& /*title*/)
@@ -38,4 +32,3 @@ void VideoSystem::setWindowTitle(const std::string& /*title*/)
 }
 
 } // namespace openmsx
-
