@@ -1,7 +1,6 @@
-// $Id: V9990.cc 6463 2007-05-12 08:27:34Z m9710797 $
+// $Id: V9990.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "V9990.hh"
-#include "Scheduler.hh"
 #include "Display.hh"
 #include "RendererFactory.hh"
 #include "V9990VRAM.hh"
@@ -9,6 +8,7 @@
 #include "V9990Renderer.hh"
 #include "SimpleDebuggable.hh"
 #include "MSXMotherBoard.hh"
+#include "serialize.hh"
 #include <cassert>
 #include <cstring>
 
@@ -19,7 +19,7 @@ class V9990RegDebug : public SimpleDebuggable
 public:
 	explicit V9990RegDebug(V9990& v9990);
 	virtual byte read(unsigned address);
-	virtual void write(unsigned address, byte value, const EmuTime& time);
+	virtual void write(unsigned address, byte value, EmuTime::param time);
 private:
 	V9990& v9990;
 };
@@ -29,7 +29,7 @@ class V9990PalDebug : public SimpleDebuggable
 public:
 	explicit V9990PalDebug(V9990& v9990);
 	virtual byte read(unsigned address);
-	virtual void write(unsigned address, byte value, const EmuTime& time);
+	virtual void write(unsigned address, byte value, EmuTime::param time);
 private:
 	V9990& v9990;
 };
@@ -68,19 +68,18 @@ static const RegisterAccess regAccess[64] = {
 // Constructor & Destructor
 // -------------------------------------------------------------------------
 
-V9990::V9990(MSXMotherBoard& motherBoard, const XMLElement& config,
-             const EmuTime& time)
-	: MSXDevice(motherBoard, config, time)
+V9990::V9990(MSXMotherBoard& motherBoard, const XMLElement& config)
+	: MSXDevice(motherBoard, config)
 	, Schedulable(motherBoard.getScheduler())
 	, v9990RegDebug(new V9990RegDebug(*this))
 	, v9990PalDebug(new V9990PalDebug(*this))
-	, irq(motherBoard.getCPU())
-	, frameStartTime(time)
-	, hScanSyncTime(time)
+	, irq(motherBoard, getName() + ".IRQ")
+	, frameStartTime(Schedulable::getCurrentTime())
+	, hScanSyncTime(Schedulable::getCurrentTime())
 	, pendingIRQs(0)
 {
-        // clear regs TODO find realistic init values
-        memset(regs, 0, sizeof(regs));
+	// clear regs TODO find realistic init values
+	memset(regs, 0, sizeof(regs));
 	calcDisplayMode();
 
 	// initialize palette
@@ -92,6 +91,7 @@ V9990::V9990(MSXMotherBoard& motherBoard, const XMLElement& config,
 	}
 
 	// create VRAM
+	EmuTime::param time = Schedulable::getCurrentTime();
 	vram.reset(new V9990VRAM(*this, time));
 
 	// create Command Engine
@@ -105,6 +105,8 @@ V9990::V9990(MSXMotherBoard& motherBoard, const XMLElement& config,
 	setVerticalTiming();
 
 	// Initialise rendering system
+	isDisplayArea = false;
+	displayEnabled = false; // avoid UMR (used by createRenderer())
 	createRenderer(time);
 
 	reset(time);
@@ -120,14 +122,13 @@ V9990::~V9990()
 // MSXDevice
 // -------------------------------------------------------------------------
 
-void V9990::reset(const EmuTime& time)
+void V9990::reset(EmuTime::param time)
 {
 	removeSyncPoint(V9990_VSYNC);
 	removeSyncPoint(V9990_DISPLAY_START);
 	removeSyncPoint(V9990_VSCAN);
 	removeSyncPoint(V9990_HSCAN);
 	removeSyncPoint(V9990_SET_MODE);
-	removeSyncPoint(V9990_SET_BLANK);
 
 	// Clear registers / ports
 	memset(regs, 0, sizeof(regs));
@@ -151,102 +152,127 @@ void V9990::reset(const EmuTime& time)
 	frameStart(time);
 }
 
-byte V9990::readIO(word port, const EmuTime& time)
+byte V9990::readIO(word port, EmuTime::param time)
 {
 	port &= 0x0F;
 
-	byte result = 0;
+	// calculate return value (mostly uses peekIO)
+	byte result;
 	switch (port) {
-		case VRAM_DATA: {
-			// read from VRAM
-			unsigned addr = getVRAMAddr(VRAM_READ_ADDRESS_0);
-			result = vram->readVRAMCPU(addr, time);
-			if (!(regs[VRAM_READ_ADDRESS_2] & 0x80)) {
-				setVRAMAddr(VRAM_READ_ADDRESS_0, addr + 1);
-			}
-			break;
-		}
-		case PALETTE_DATA: {
-			byte& palPtr = regs[PALETTE_POINTER];
-			result = palette[palPtr];
-			if (!(regs[PALETTE_CONTROL] & 0x10)) {
-				switch (palPtr & 3) {
-				case 0:  palPtr += 1; break; // red
-				case 1:  palPtr += 1; break; // green
-				case 2:  palPtr += 2; break; // blue
-				default: palPtr -= 3; break; // checked on real V9990
-				}
-			}
-			break;
-		}
-		case COMMAND_DATA:
-			// TODO
-			//assert(cmdEngine != NULL);
-			result = cmdEngine->getCmdData(time);
-			break;
+	case COMMAND_DATA:
+		result = cmdEngine->getCmdData(time);
+		break;
 
-		case REGISTER_DATA: {
-			// read register
-			result = readRegister(regSelect & 0x3F, time);
-			if (!(regSelect & 0x40)) {
-				regSelect = ( regSelect      & 0xC0) |
-				            ((regSelect + 1) & 0x3F);
-			}
-			break;
-		}
-		case INTERRUPT_FLAG:
-			result = pendingIRQs;
-			break;
-
-		case STATUS: {
-			const V9990DisplayPeriod& hor = getHorizontalTiming();
-			const V9990DisplayPeriod& ver = getVerticalTiming();
-			unsigned left   = hor.blank + hor.border1;
-			unsigned right  = left + hor.display;
-			unsigned top    = ver.blank + ver.border1;
-			unsigned bottom = top + ver.display;
-			unsigned ticks = getUCTicksThisFrame(time);
-			unsigned x = ticks % V9990DisplayTiming::UC_TICKS_PER_LINE;
-			unsigned y = ticks / V9990DisplayTiming::UC_TICKS_PER_LINE;
-			bool hr = (x < left) || (right  <= x);
-			bool vr = (y < top)  || (bottom <= y);
-
-			result = cmdEngine->getStatus(time) |
-				 (vr ? 0x40 : 0x00) |
-				 (hr ? 0x20 : 0x00) |
-				 (status & 0x06);
-			break;
-		}
-		case KANJI_ROM_1:
-		case KANJI_ROM_3:
-			// not used in Gfx9000
-			result = 0xFF;	// TODO check
-			break;
-
-		case REGISTER_SELECT:
-		case SYSTEM_CONTROL:
-		case KANJI_ROM_0:
-		case KANJI_ROM_2:
-		default:
-			// write-only
-			result = 0xFF;
-			break;
+	case VRAM_DATA:
+	case PALETTE_DATA:
+	case REGISTER_DATA:
+	case INTERRUPT_FLAG:
+	case STATUS:
+	case KANJI_ROM_0:
+	case KANJI_ROM_1:
+	case KANJI_ROM_2:
+	case KANJI_ROM_3:
+	case REGISTER_SELECT:
+	case SYSTEM_CONTROL:
+	default:
+		result = peekIO(port, time);
 	}
 
-	//PRT_DEBUG("[" << time << "] "
-	//	  "V9990::readIO - port=0x" << std::hex << (int)port <<
-	//	                  " val=0x" << std::hex << (int)result);
+	// execute side-effects
+	switch (port) {
+	case VRAM_DATA:
+		if (!(regs[VRAM_READ_ADDRESS_2] & 0x80)) {
+			unsigned addr = getVRAMAddr(VRAM_READ_ADDRESS_0);
+			setVRAMAddr(VRAM_READ_ADDRESS_0, addr + 1);
+		}
+		break;
 
+	case PALETTE_DATA:
+		if (!(regs[PALETTE_CONTROL] & 0x10)) {
+			byte& palPtr = regs[PALETTE_POINTER];
+			switch (palPtr & 3) {
+			case 0:  palPtr += 1; break; // red
+			case 1:  palPtr += 1; break; // green
+			case 2:  palPtr += 2; break; // blue
+			default: palPtr -= 3; break; // checked on real V9990
+			}
+		}
+		break;
+
+	case REGISTER_DATA:
+		if (!(regSelect & 0x40)) {
+			//regSelect = ( regSelect      & 0xC0) |
+			//            ((regSelect + 1) & 0x3F);
+			regSelect = (regSelect + 1) & ~0x40;
+		}
+		break;
+	}
 	return result;
 }
 
-byte V9990::peekIO(word /*port*/, const EmuTime& /*time*/) const
+byte V9990::peekIO(word port, EmuTime::param time) const
 {
-	// TODO not implemented
-	return 0xFF;
+	byte result;
+	switch (port & 0x0F) {
+	case VRAM_DATA: {
+		unsigned addr = getVRAMAddr(VRAM_READ_ADDRESS_0);
+		result = vram->readVRAMCPU(addr, time);
+		break;
+	}
+	case PALETTE_DATA:
+		result = palette[regs[PALETTE_POINTER]];
+		break;
+
+	case COMMAND_DATA:
+		result = cmdEngine->peekCmdData(time);
+		break;
+
+	case REGISTER_DATA:
+		result = readRegister(regSelect & 0x3F, time);
+		break;
+
+	case INTERRUPT_FLAG:
+		result = pendingIRQs;
+		break;
+
+	case STATUS: {
+		const V9990DisplayPeriod& hor = getHorizontalTiming();
+		const V9990DisplayPeriod& ver = getVerticalTiming();
+		unsigned left   = hor.blank + hor.border1;
+		unsigned right  = left + hor.display;
+		unsigned top    = ver.blank + ver.border1;
+		unsigned bottom = top + ver.display;
+		unsigned ticks = getUCTicksThisFrame(time);
+		unsigned x = ticks % V9990DisplayTiming::UC_TICKS_PER_LINE;
+		unsigned y = ticks / V9990DisplayTiming::UC_TICKS_PER_LINE;
+		bool hr = (x < left) || (right  <= x);
+		bool vr = (y < top)  || (bottom <= y);
+
+		result = cmdEngine->getStatus(time) |
+		         (vr ? 0x40 : 0x00) |
+		         (hr ? 0x20 : 0x00) |
+		         (status & 0x06);
+		break;
+	}
+	case KANJI_ROM_1:
+	case KANJI_ROM_3:
+		// not used in Gfx9000
+		result = 0xFF; // TODO check
+		break;
+
+	case REGISTER_SELECT:
+	case SYSTEM_CONTROL:
+	case KANJI_ROM_0:
+	case KANJI_ROM_2:
+	default:
+		// write-only
+		result = 0xFF;
+		break;
+	}
+	return result;
 }
 
-void V9990::writeIO(word port, byte val, const EmuTime& time)
+void V9990::writeIO(word port, byte val, EmuTime::param time)
 {
 	port &= 0x0F;
 
@@ -333,7 +359,7 @@ void V9990::writeIO(word port, byte val, const EmuTime& time)
 // Schedulable
 // -------------------------------------------------------------------------
 
-void V9990::executeUntil(const EmuTime& time, int userData)
+void V9990::executeUntil(EmuTime::param time, int userData)
 {
 	//PRT_DEBUG("[" << time << "] "
 	//          "V9990::executeUntil - data=0x" << std::hex << userData);
@@ -369,14 +395,6 @@ void V9990::executeUntil(const EmuTime& time, int userData)
 		renderer->setColorMode(getColorMode(), time);
 		break;
 
-	case V9990_SET_BLANK: {
-		bool newDisplayEnabled = regs[CONTROL] & 0x80;
-		if (isDisplayArea) {
-			renderer->updateDisplayEnabled(newDisplayEnabled, time);
-		}
-		displayEnabled = newDisplayEnabled;
-		break;
-	}
 	default:
 		assert(false);
 	}
@@ -399,7 +417,7 @@ void V9990::preVideoSystemChange()
 
 void V9990::postVideoSystemChange()
 {
-	const EmuTime& time = getMotherBoard().getScheduler().getCurrentTime();
+	EmuTime::param time = Schedulable::getCurrentTime();
 	createRenderer(time);
 	renderer->frameStart(time);
 }
@@ -420,7 +438,7 @@ byte V9990RegDebug::read(unsigned address)
 	return v9990.regs[address];
 }
 
-void V9990RegDebug::write(unsigned address, byte value, const EmuTime& time)
+void V9990RegDebug::write(unsigned address, byte value, EmuTime::param time)
 {
 	v9990.writeRegister(address, value, time);
 }
@@ -442,7 +460,7 @@ byte V9990PalDebug::read(unsigned address)
 	return v9990.palette[address];
 }
 
-void V9990PalDebug::write(unsigned address, byte value, const EmuTime& time)
+void V9990PalDebug::write(unsigned address, byte value, EmuTime::param time)
 {
 	v9990.writePaletteRegister(address, value, time);
 }
@@ -466,7 +484,7 @@ inline void V9990::setVRAMAddr(RegisterId base, unsigned addr)
 	// TODO check
 }
 
-byte V9990::readRegister(byte reg, const EmuTime& time)
+byte V9990::readRegister(byte reg, EmuTime::param time) const
 {
 	// TODO sync(time) (if needed at all)
 
@@ -477,7 +495,7 @@ byte V9990::readRegister(byte reg, const EmuTime& time)
 			result = regs[reg];
 		} else {
 			word borderX = cmdEngine->getBorderX(time);
-			return (reg == CMD_PARAM_BORDER_X_0)
+			result = (reg == CMD_PARAM_BORDER_X_0)
 			       ? (borderX & 0xFF) : (borderX >> 8);
 		}
 	} else {
@@ -490,7 +508,7 @@ byte V9990::readRegister(byte reg, const EmuTime& time)
 	return result;
 }
 
-void V9990::syncAtNextLine(V9990SyncType type, const EmuTime& time)
+void V9990::syncAtNextLine(V9990SyncType type, EmuTime::param time)
 {
 	int line = getUCTicksThisFrame(time) / V9990DisplayTiming::UC_TICKS_PER_LINE;
 	int ticks = (line + 1) * V9990DisplayTiming::UC_TICKS_PER_LINE;
@@ -498,7 +516,7 @@ void V9990::syncAtNextLine(V9990SyncType type, const EmuTime& time)
 	setSyncPoint(nextTime, type);
 }
 
-void V9990::writeRegister(byte reg, byte val, const EmuTime& time)
+void V9990::writeRegister(byte reg, byte val, EmuTime::param time)
 {
 	// Found this table by writing 0xFF to a register and reading
 	// back the value (only works for read/write registers)
@@ -525,23 +543,18 @@ void V9990::writeRegister(byte reg, byte val, const EmuTime& time)
 
 	val &= regWriteMask[reg];
 
-	byte change = regs[reg] ^ val;
 	// This optimization is not valid for the vertical scroll registers
 	// TODO is this optimization still useful for other registers?
 	//if (!change) return;
 
 	// Perform additional tasks before new value becomes active
+	// note: no update for SCROLL_CONTROL_AY1, SCROLL_CONTROL_BY1
 	switch (reg) {
 		case SCREEN_MODE_0:
 		case SCREEN_MODE_1:
 			// TODO verify this on real V9990
 			syncAtNextLine(V9990_SET_MODE, time);
 			break;
-		case CONTROL:
-			if (change & 0x80) {
-				syncAtNextLine(V9990_SET_BLANK, time);
-			}
-			break;
 		case PALETTE_CONTROL:
 			renderer->setColorMode(getColorMode(val), time);
 			break;
@@ -551,17 +564,13 @@ void V9990::writeRegister(byte reg, byte val, const EmuTime& time)
 		case SCROLL_CONTROL_AY0:
 			renderer->updateScrollAYLow(time);
 			break;
-		case SCROLL_CONTROL_AY1:
-			renderer->updateScrollAYHigh(time);
+		case SCROLL_CONTROL_BY0:
+			renderer->updateScrollBYLow(time);
 			break;
 		case SCROLL_CONTROL_AX0:
 		case SCROLL_CONTROL_AX1:
 			renderer->updateScrollAX(time);
 			break;
-		case SCROLL_CONTROL_BY0:
-		case SCROLL_CONTROL_BY1:
-			renderer->updateScrollBY(time);
-			break;
 		case SCROLL_CONTROL_BX0:
 		case SCROLL_CONTROL_BX1:
 			renderer->updateScrollBX(time);
@@ -587,7 +596,7 @@ void V9990::writeRegister(byte reg, byte val, const EmuTime& time)
 	}
 }
 
-void V9990::writePaletteRegister(byte reg, byte val, const EmuTime& time)
+void V9990::writePaletteRegister(byte reg, byte val, EmuTime::param time)
 {
 	switch (reg & 3) {
 		case 0: val &= 0x9F; break;
@@ -605,14 +614,14 @@ void V9990::writePaletteRegister(byte reg, byte val, const EmuTime& time)
 	}
 }
 
-void V9990::getPalette(int index, byte& r, byte& g, byte& b)
+void V9990::getPalette(int index, byte& r, byte& g, byte& b) const
 {
 	r = palette[4 * index + 0];
 	g = palette[4 * index + 1];
 	b = palette[4 * index + 2];
 }
 
-void V9990::createRenderer(const EmuTime& time)
+void V9990::createRenderer(EmuTime::param time)
 {
 	assert(!renderer.get());
 	Display& display = getMotherBoard().getDisplay();
@@ -620,15 +629,18 @@ void V9990::createRenderer(const EmuTime& time)
 	renderer->reset(time);
 }
 
-void V9990::frameStart(const EmuTime& time)
+void V9990::frameStart(EmuTime::param time)
 {
 	// Update setings that are fixed at the start of a frame
-	palTiming  = regs[SCREEN_MODE_1] & 0x08;
-	interlaced = regs[SCREEN_MODE_1] & 0x02;
+	displayEnabled = regs[CONTROL]       & 0x80;
+	palTiming      = regs[SCREEN_MODE_1] & 0x08;
+	interlaced     = regs[SCREEN_MODE_1] & 0x02;
+	scrollAYHigh   = regs[SCROLL_CONTROL_AY1];
+	scrollBYHigh   = regs[SCROLL_CONTROL_BY1];
 	setVerticalTiming();
 	status ^= 0x02; // flip EO bit
 
-	frameStartTime.advance(time);
+	frameStartTime.advance_fast(time);
 
 	// schedule next VSYNC
 	setSyncPoint(
@@ -767,7 +779,7 @@ void V9990::calcDisplayMode()
 	setHorizontalTiming();
 }
 
-void V9990::scheduleHscan(const EmuTime& time)
+void V9990::scheduleHscan(EmuTime::param time)
 {
 	// remove pending HSCAN, if any
 	if (hScanSyncTime > time) {
@@ -780,7 +792,7 @@ void V9990::scheduleHscan(const EmuTime& time)
 		return;
 	}
 
-	int ticks = frameStartTime.getTicksTill(time);
+	int ticks = frameStartTime.getTicksTill_fast(time);
 	int offset;
 	if (regs[INTERRUPT_2] & 0x80) {
 		// every line
@@ -800,5 +812,50 @@ void V9990::scheduleHscan(const EmuTime& time)
 	setSyncPoint(hScanSyncTime, V9990_HSCAN);
 }
 
-} // namespace openmsx
+static enum_string<V9990DisplayMode> displayModeInfo[] = {
+	{ "INVALID", INVALID_DISPLAY_MODE },
+	{ "P1", P1 }, { "P2", P2 },
+	{ "B0", B0 }, { "B1", B1 }, { "B2", B2 }, { "B3", B3 },
+	{ "B4", B4 }, { "B5", B5 }, { "B6", B6 }, { "B7", B7 }
+};
+SERIALIZE_ENUM(V9990DisplayMode, displayModeInfo);
 
+template<typename Archive>
+void V9990::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.template serializeBase<MSXDevice>(*this);
+	ar.template serializeBase<Schedulable>(*this);
+
+	ar.serialize("vram", *vram);
+	ar.serialize("cmdEngine", *cmdEngine);
+	ar.serialize("irq", irq);
+	ar.serialize("frameStartTime", frameStartTime);
+	ar.serialize("hScanSyncTime", hScanSyncTime);
+	ar.serialize("displayMode", mode);
+	ar.serialize_blob("palette", palette, sizeof(palette));
+	ar.serialize("status", status);
+	ar.serialize("pendingIRQs", pendingIRQs);
+	ar.serialize_blob("registers", regs, sizeof(regs));
+	ar.serialize("regSelect", regSelect);
+	ar.serialize("palTiming", palTiming);
+	ar.serialize("interlaced", interlaced);
+	ar.serialize("isDisplayArea", isDisplayArea);
+	ar.serialize("displayEnabled", displayEnabled);
+	ar.serialize("scrollAYHigh", scrollAYHigh);
+	ar.serialize("scrollBYHigh", scrollBYHigh);
+
+	if (ar.isLoader()) {
+		// TODO This uses 'mode' to calculate 'horTiming' and
+		//      'verTiming'. Are these always in sync? Or can for
+		//      example one change at any time and the other only
+		//      at start of frame (or next line)? Does this matter?
+		setHorizontalTiming();
+		setVerticalTiming();
+
+		renderer->reset(Schedulable::getCurrentTime());
+	}
+}
+INSTANTIATE_SERIALIZE_METHODS(V9990);
+REGISTER_MSXDEVICE(V9990, "V9990");
+
+} // namespace openmsx
