@@ -1,26 +1,26 @@
-// $Id: V9990BitmapConverter.cc 6619 2007-06-11 19:26:49Z m9710797 $
+// $Id: V9990BitmapConverter.cc 8223 2008-09-14 19:17:24Z m9710797 $
 
 #include "V9990BitmapConverter.hh"
 #include "V9990VRAM.hh"
 #include "V9990.hh"
 #include "GLUtil.hh"
 #include "Math.hh"
+#include "build-info.hh"
 #include <cassert>
 
 namespace openmsx {
 
 template <class Pixel>
 V9990BitmapConverter<Pixel>::V9990BitmapConverter(
-	V9990& vdp_, Pixel* palette64_, Pixel* palette256_, Pixel* palette32768_)
+		V9990& vdp_, const Pixel* palette64_,
+		const Pixel* palette256_, const Pixel* palette32768_)
 	: vdp(vdp_), vram(vdp.getVRAM())
-        , palette64(palette64_), palette256(palette256_), palette32768(palette32768_)
+	, palette64(palette64_), palette256(palette256_), palette32768(palette32768_)
 {
 	// make sure function pointers have valid values
 	setColorMode(PP);
 }
 
-// - Rasterizers -------------------------------------------------------
-
 template <class Pixel>
 void V9990BitmapConverter<Pixel>::rasterBYUV(
 	Pixel* pixelPtr, unsigned address, int nrPixels)
@@ -186,17 +186,17 @@ template <class Pixel>
 void V9990BitmapConverter<Pixel>::setColorMode(V9990ColorMode mode)
 {
 	switch (mode) {
-		case PP:    rasterMethod = &V9990BitmapConverter::rasterP;     break;
-		case BYUV:  rasterMethod = &V9990BitmapConverter::rasterBYUV;  break;
-		case BYUVP: rasterMethod = &V9990BitmapConverter::rasterBYUVP; break;
-		case BYJK:  rasterMethod = &V9990BitmapConverter::rasterBYJK;  break;
-		case BYJKP: rasterMethod = &V9990BitmapConverter::rasterBYJKP; break;
-		case BD16:  rasterMethod = &V9990BitmapConverter::rasterBD16;  break;
-		case BD8:   rasterMethod = &V9990BitmapConverter::rasterBD8;   break;
-		case BP6:   rasterMethod = &V9990BitmapConverter::rasterBP6;   break;
-		case BP4:   rasterMethod = &V9990BitmapConverter::rasterBP4;   break;
-		case BP2:   rasterMethod = &V9990BitmapConverter::rasterBP2;   break;
-		default:    assert (false);
+	case PP:    rasterMethod = &V9990BitmapConverter::rasterP;     break;
+	case BYUV:  rasterMethod = &V9990BitmapConverter::rasterBYUV;  break;
+	case BYUVP: rasterMethod = &V9990BitmapConverter::rasterBYUVP; break;
+	case BYJK:  rasterMethod = &V9990BitmapConverter::rasterBYJK;  break;
+	case BYJKP: rasterMethod = &V9990BitmapConverter::rasterBYJKP; break;
+	case BD16:  rasterMethod = &V9990BitmapConverter::rasterBD16;  break;
+	case BD8:   rasterMethod = &V9990BitmapConverter::rasterBD8;   break;
+	case BP6:   rasterMethod = &V9990BitmapConverter::rasterBP6;   break;
+	case BP4:   rasterMethod = &V9990BitmapConverter::rasterBP4;   break;
+	case BP2:   rasterMethod = &V9990BitmapConverter::rasterBP2;   break;
+	default:    assert (false);
 	}
 }
 
@@ -259,12 +259,15 @@ void V9990BitmapConverter<Pixel>::convertLine(
 }
 
 // Force template instantiation
+#if HAVE_16BPP
 template class V9990BitmapConverter<word>;
+#endif
+#if HAVE_32BPP
 template class V9990BitmapConverter<unsigned>;
+#endif
 #ifdef COMPONENT_GL
 template <> class V9990BitmapConverter<GLUtil::NoExpansion> {};
 template class V9990BitmapConverter<GLUtil::ExpandGL>;
 #endif // COMPONENT_GL
 
 } // namespace openmsx
-
