@@ -1,4 +1,4 @@
-// $Id: V9990CmdEngine.cc 7040 2007-09-18 15:30:36Z m9710797 $
+// $Id: V9990CmdEngine.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "V9990CmdEngine.hh"
 #include "V9990.hh"
@@ -6,19 +6,67 @@
 #include "MSXMotherBoard.hh"
 #include "RenderSettings.hh"
 #include "BooleanSetting.hh"
+#include "EnumSetting.hh"
+#include "serialize.hh"
+#include "likely.hh"
 #include <iostream>
 
 namespace openmsx {
 
-// Timing tables   TODO
-const unsigned LMMV_TIMING[2] = { 24, 0 };
-const unsigned LMMM_TIMING[2] = { 24, 0 };
-const unsigned CMMM_TIMING[2] = { 24, 0 };
-const unsigned BMXL_TIMING[2] = { 24, 0 };
-const unsigned BMLX_TIMING[2] = { 24, 0 };
-const unsigned BMLL_TIMING[2] = { 24, 0 };
-const unsigned LINE_TIMING[2] = { 24, 0 };
-const unsigned SRCH_TIMING[2] = { 24, 0 };
+// 1st index  B0/2/4, B1/3/7, P1, P2
+// 2nd index  sprites-ON, sprites-OFF, display-OFF
+// 3th index  2bpp, 4bpp, 8bpp, 16bpp
+//            (for P1/P2 fill in the same value 4 times)
+const unsigned LMMV_TIMING[4][3][4] = {
+	{ {  8, 11, 15, 30}, { 7, 10, 13, 26}, { 7, 10, 13, 25} },
+	{ {  5,  7,  9, 18}, { 5,  6,  8, 17}, { 5,  6,  8, 17} },
+	{ { 56, 56, 56, 56}, {25, 25, 25, 25}, { 9,  9,  9,  9} },
+	{ { 28, 28, 28, 28}, {15, 15, 15, 15}, { 6,  6,  6,  6} }
+};
+const unsigned LMMM_TIMING[4][3][4] = {
+	{ { 10, 16, 32, 66}, { 8, 14, 28, 57}, { 8, 13, 27, 54} },
+	{ {  6, 10, 20, 39}, { 5,  9, 18, 35}, { 5,  9, 17, 35} },
+	{ {115,115,115,115}, {52, 52, 52, 52}, {18, 18, 18, 18} },
+	{ { 57, 57, 57, 57}, {25, 25, 25, 25}, { 9,  9,  9,  9} }
+};
+const unsigned BMXL_TIMING[4][3][4] = { // NOTE: values are BYTE based here!
+	{ { 38, 33, 32, 33}, {33, 28, 28, 28}, {33, 27, 27, 27} }, // identical to LMMM (b)
+	{ { 24, 20, 20, 19}, {22, 18, 18, 18}, {21, 17, 17, 17} }, // identical to LMMM (b)
+	{ {171,171,171,171}, {82, 82, 82, 82}, {29, 29, 29, 29} },
+	{ {114,114,114,114}, {50, 50, 50, 50}, {18, 18, 18, 18} }
+};
+const unsigned BMLX_TIMING[4][3][4] = {
+	{ { 10, 16, 32, 66}, { 8, 14, 28, 57}, { 8, 13, 27, 54} }, // identical to LMMM
+	{ {  6, 10, 20, 39}, { 5,  9, 18, 35}, { 5,  9, 17, 35} }, // identical to LMMM
+	{ { 84, 84, 84, 84}, {44, 44, 44, 44}, {17, 17, 17, 17} },
+	{ { 57, 57, 57, 57}, {25, 25, 25, 25}, { 9,  9,  9,  9} }
+};
+const unsigned BMLL_TIMING[4][3][4] = {
+	{ { 33, 33, 33, 33}, {28, 28, 28, 28}, {27, 27, 27, 27} },
+	{ { 20, 20, 20, 20}, {18, 18, 18, 18}, {18, 18, 18, 18} },
+	{ {118,118,118,118}, {52, 52, 52, 52}, {18, 18, 18, 18} },
+	{ {118,118,118,118}, {52, 52, 52, 52}, {18, 18, 18, 18} }
+};
+const unsigned CMMM_TIMING[4][3][4] = {
+	{ { 24, 24, 24, 24}, {24, 24, 24, 24}, {24, 24, 24, 24} }, // TODO
+	{ { 24, 24, 24, 24}, {24, 24, 24, 24}, {24, 24, 24, 24} }, // TODO
+	{ { 24, 24, 24, 24}, {24, 24, 24, 24}, {24, 24, 24, 24} }, // TODO
+	{ { 24, 24, 24, 24}, {24, 24, 24, 24}, {24, 24, 24, 24} }  // TODO
+};
+const unsigned LINE_TIMING[4][3][4] = {
+	{ { 24, 24, 24, 24}, {24, 24, 24, 24}, {24, 24, 24, 24} }, // TODO
+	{ { 24, 24, 24, 24}, {24, 24, 24, 24}, {24, 24, 24, 24} }, // TODO
+	{ { 24, 24, 24, 24}, {24, 24, 24, 24}, {24, 24, 24, 24} }, // TODO
+	{ { 24, 24, 24, 24}, {24, 24, 24, 24}, {24, 24, 24, 24} }  // TODO
+};
+const unsigned SRCH_TIMING[4][3][4] = {
+	{ { 24, 24, 24, 24}, {24, 24, 24, 24}, {24, 24, 24, 24} }, // TODO
+	{ { 24, 24, 24, 24}, {24, 24, 24, 24}, {24, 24, 24, 24} }, // TODO
+	{ { 24, 24, 24, 24}, {24, 24, 24, 24}, {24, 24, 24, 24} }, // TODO
+	{ { 24, 24, 24, 24}, {24, 24, 24, 24}, {24, 24, 24, 24} }  // TODO
+};
+
+
 
 static byte bitLUT[8][16][2][2];
 static byte logOpLUT[4][16][0x100][0x100]; // 4MB !!  optimize if needed
@@ -589,9 +637,9 @@ inline void V9990CmdEngine::V9990Bpp16::psetColor(
 // ====================================================================
 /** Constructor
   */
-V9990CmdEngine::V9990CmdEngine(V9990& vdp_, const EmuTime& time,
+V9990CmdEngine::V9990CmdEngine(V9990& vdp_, EmuTime::param time,
                                RenderSettings& settings_)
-	: settings(settings_), vdp(vdp_)
+	: settings(settings_), vdp(vdp_), clock(EmuTime::zero)
 {
 	MSXMotherBoard::SharedStuff& info =
 		vdp.getMotherBoard().getSharedStuff("v9990cmdtrace");
@@ -631,6 +679,14 @@ V9990CmdEngine::V9990CmdEngine(V9990& vdp_, const EmuTime& time,
 	settings.getCmdTiming().attach(*this);
 
 	reset(time);
+
+	// avoid UMR on savestate
+	srcAddress = dstAddress = nbBytes = 0;
+	ASX = ADX = ANX = ANY = 0;
+	SX = SY = DX = DY = NX = NY = 0;
+	WM = fgCol = bgCol = 0;
+	ARG = LOG = 0;
+	data = bitsLeft = partial = 0;
 }
 
 template <template <class Mode> class Command>
@@ -669,15 +725,16 @@ V9990CmdEngine::~V9990CmdEngine()
 	}
 }
 
-void V9990CmdEngine::reset(const EmuTime& /*time*/)
+void V9990CmdEngine::reset(EmuTime::param /*time*/)
 {
 	currentCommand = NULL;
+	CMD = 0;
 	status = 0;
 	borderX = 0;
 	endAfterRead = false;
 }
 
-void V9990CmdEngine::setCmdReg(byte reg, byte value, const EmuTime& time)
+void V9990CmdEngine::setCmdReg(byte reg, byte value, EmuTime::param time)
 {
 	//PRT_DEBUG("[" << time << "] V9990CmdEngine::setCmdReg("
 	//          << std::dec << (int) reg << "," << (int) value << ")");
@@ -750,37 +807,7 @@ void V9990CmdEngine::setCmdReg(byte reg, byte value, const EmuTime& time)
 		}
 		status |= CE;
 
-		CommandMode cmdMode;
-		V9990DisplayMode dispMode = vdp.getDisplayMode();
-		if (dispMode == P1) {
-			cmdMode = CMD_P1;
-		} else if (dispMode == P2) {
-			cmdMode = CMD_P2;
-		} else { // Bx
-			switch (vdp.getColorMode()) {
-				default:
-					assert(false);
-				case BP2:
-					cmdMode = CMD_BPP2;
-					break;
-				case PP:
-				case BP4:
-					cmdMode = CMD_BPP4;
-					break;
-				case BYUV:
-				case BYUVP:
-				case BYJK:
-				case BYJKP:
-				case BD8:
-				case BP6:
-					cmdMode = CMD_BPP8;
-					break;
-				case BD16:
-					cmdMode = CMD_BPP16;
-					break;
-			}
-		}
-		currentCommand = commands[CMD >> 4][cmdMode];
+		setCurrentCommand();
 		currentCommand->start(time);
 
 		// Finish command now if instantaneous command timing is active.
@@ -794,6 +821,41 @@ void V9990CmdEngine::setCmdReg(byte reg, byte value, const EmuTime& time)
 	}
 }
 
+void V9990CmdEngine::setCurrentCommand()
+{
+	CommandMode cmdMode;
+	V9990DisplayMode dispMode = vdp.getDisplayMode();
+	if (dispMode == P1) {
+		cmdMode = CMD_P1;
+	} else if (dispMode == P2) {
+		cmdMode = CMD_P2;
+	} else { // Bx
+		switch (vdp.getColorMode()) {
+			default:
+				assert(false);
+			case BP2:
+				cmdMode = CMD_BPP2;
+				break;
+			case PP:
+			case BP4:
+				cmdMode = CMD_BPP4;
+				break;
+			case BYUV:
+			case BYUVP:
+			case BYJK:
+			case BYJKP:
+			case BD8:
+			case BP6:
+				cmdMode = CMD_BPP8;
+				break;
+			case BD16:
+				cmdMode = CMD_BPP16;
+				break;
+		}
+	}
+	currentCommand = commands[CMD >> 4][cmdMode];
+}
+
 void V9990CmdEngine::reportV9990Command()
 {
 	const char* const COMMANDS[16] = {
@@ -823,10 +885,18 @@ void V9990CmdEngine::update(const Setting& setting)
 	brokenTiming = static_cast<const EnumSetting<bool>&>(setting).getValue();
 }
 
-unsigned V9990CmdEngine::getTiming() const
+unsigned V9990CmdEngine::getTiming(const unsigned table[4][3][4]) const
 {
-	// TODO
-	return brokenTiming ? 1 : 0;
+	if (unlikely(brokenTiming)) return 0;
+
+	V9990DisplayMode mode = vdp.getDisplayMode();
+	unsigned idx1 = (mode == P1) ? 2 :
+	                (mode == P2) ? 3 :
+	                (vdp.isOverScan()) ? 0 : 1;
+	unsigned idx2 = vdp.isDisplayEnabled() ? (vdp.spritesEnabled() ? 0 : 1)
+	                                       : 2;
+	unsigned idx3 = vdp.getColorDepth();
+	return table[idx1][idx2][idx3];
 }
 
 // ====================================================================
@@ -834,7 +904,7 @@ unsigned V9990CmdEngine::getTiming() const
 
 V9990CmdEngine::V9990Cmd::V9990Cmd(V9990CmdEngine& engine_,
                                    V9990VRAM& vram_)
-	: engine(engine_), vram(vram_), clock(EmuTime::zero)
+	: engine(engine_), vram(vram_)
 {
 }
 
@@ -851,12 +921,12 @@ V9990CmdEngine::CmdSTOP::CmdSTOP(V9990CmdEngine& engine_,
 {
 }
 
-void V9990CmdEngine::CmdSTOP::start(const EmuTime& time)
+void V9990CmdEngine::CmdSTOP::start(EmuTime::param time)
 {
 	engine.cmdReady(time);
 }
 
-void V9990CmdEngine::CmdSTOP::execute(const EmuTime& /*time*/)
+void V9990CmdEngine::CmdSTOP::execute(EmuTime::param /*time*/)
 {
 }
 
@@ -871,7 +941,7 @@ V9990CmdEngine::CmdLMMC<Mode>::CmdLMMC(V9990CmdEngine& engine,
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdLMMC<Mode>::start(const EmuTime& /*time*/)
+void V9990CmdEngine::CmdLMMC<Mode>::start(EmuTime::param /*time*/)
 {
 	if (Mode::BITS_PER_PIXEL == 16) {
 		engine.bitsLeft = 1;
@@ -883,7 +953,7 @@ void V9990CmdEngine::CmdLMMC<Mode>::start(const EmuTime& /*time*/)
 
 template <>
 void V9990CmdEngine::CmdLMMC<V9990CmdEngine::V9990Bpp16>::execute(
-	const EmuTime& time)
+	EmuTime::param time)
 {
 	if (!(engine.status & TR)) {
 		engine.status |= TR;
@@ -914,7 +984,7 @@ void V9990CmdEngine::CmdLMMC<V9990CmdEngine::V9990Bpp16>::execute(
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdLMMC<Mode>::execute(const EmuTime& time)
+void V9990CmdEngine::CmdLMMC<Mode>::execute(EmuTime::param time)
 {
 	if (!(engine.status & TR)) {
 		engine.status |= TR;
@@ -952,25 +1022,25 @@ V9990CmdEngine::CmdLMMV<Mode>::CmdLMMV(V9990CmdEngine& engine,
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdLMMV<Mode>::start(const EmuTime& time)
+void V9990CmdEngine::CmdLMMV<Mode>::start(EmuTime::param time)
 {
-	clock.reset(time);
+	engine.clock.reset(time);
 	engine.ANX = engine.NX;
 	engine.ANY = engine.NY;
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdLMMV<Mode>::execute(const EmuTime& time)
+void V9990CmdEngine::CmdLMMV<Mode>::execute(EmuTime::param time)
 {
 	// TODO can be optimized a lot
 
-	unsigned delta = LMMV_TIMING[engine.getTiming()];
+	unsigned delta = engine.getTiming(LMMV_TIMING);
 	unsigned pitch = Mode::getPitch(engine.vdp.getImageWidth());
 	int dx = (engine.ARG & DIX) ? -1 : 1;
 	int dy = (engine.ARG & DIY) ? -1 : 1;
 	const byte* lut = Mode::getLogOpLUT(engine.LOG);
-	while (clock.before(time)) {
-		clock += delta;
+	while (engine.clock.before(time)) {
+		engine.clock += delta;
 		Mode::psetColor(vram, engine.DX, engine.DY, pitch,
 		                engine.fgCol, engine.WM, lut, engine.LOG);
 
@@ -979,7 +1049,7 @@ void V9990CmdEngine::CmdLMMV<Mode>::execute(const EmuTime& time)
 			engine.DX -= (engine.NX * dx);
 			engine.DY += dy;
 			if (!--(engine.ANY)) {
-				engine.cmdReady(clock.getTime());
+				engine.cmdReady(engine.clock.getTime());
 				return;
 			} else {
 				engine.ANX = engine.NX;
@@ -999,7 +1069,7 @@ V9990CmdEngine::CmdLMCM<Mode>::CmdLMCM(V9990CmdEngine& engine,
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdLMCM<Mode>::start(const EmuTime& /*time*/)
+void V9990CmdEngine::CmdLMCM<Mode>::start(EmuTime::param /*time*/)
 {
 	if (Mode::BITS_PER_PIXEL == 16) {
 		engine.bitsLeft = 0;
@@ -1011,7 +1081,7 @@ void V9990CmdEngine::CmdLMCM<Mode>::start(const EmuTime& /*time*/)
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdLMCM<Mode>::execute(const EmuTime& /*time*/)
+void V9990CmdEngine::CmdLMCM<Mode>::execute(EmuTime::param /*time*/)
 {
 	if (!(engine.status & TR)) {
 		engine.status |= TR;
@@ -1060,25 +1130,25 @@ V9990CmdEngine::CmdLMMM<Mode>::CmdLMMM(V9990CmdEngine& engine,
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdLMMM<Mode>::start(const EmuTime& time)
+void V9990CmdEngine::CmdLMMM<Mode>::start(EmuTime::param time)
 {
-	clock.reset(time);
+	engine.clock.reset(time);
 	engine.ANX = engine.NX;
 	engine.ANY = engine.NY;
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdLMMM<Mode>::execute(const EmuTime& time)
+void V9990CmdEngine::CmdLMMM<Mode>::execute(EmuTime::param time)
 {
 	// TODO can be optimized a lot
 
-	unsigned delta = LMMM_TIMING[engine.getTiming()];
+	unsigned delta = engine.getTiming(LMMM_TIMING);
 	unsigned pitch = Mode::getPitch(engine.vdp.getImageWidth());
 	int dx = (engine.ARG & DIX) ? -1 : 1;
 	int dy = (engine.ARG & DIY) ? -1 : 1;
 	const byte* lut = Mode::getLogOpLUT(engine.LOG);
-	while (clock.before(time)) {
-		clock += delta;
+	while (engine.clock.before(time)) {
+		engine.clock += delta;
 		typename Mode::Type src = Mode::point(vram, engine.SX, engine.SY, pitch);
 		src = Mode::shift(src, engine.SX, engine.DX);
 		Mode::pset(vram, engine.DX, engine.DY, pitch,
@@ -1092,7 +1162,7 @@ void V9990CmdEngine::CmdLMMM<Mode>::execute(const EmuTime& time)
 			engine.DY += dy;
 			engine.SY += dy;
 			if (!--(engine.ANY)) {
-				engine.cmdReady(clock.getTime());
+				engine.cmdReady(engine.clock.getTime());
 				return;
 			} else {
 				engine.ANX = engine.NX;
@@ -1112,7 +1182,7 @@ V9990CmdEngine::CmdCMMC<Mode>::CmdCMMC(V9990CmdEngine& engine,
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdCMMC<Mode>::start(const EmuTime& /*time*/)
+void V9990CmdEngine::CmdCMMC<Mode>::start(EmuTime::param /*time*/)
 {
 	engine.ANX = engine.NX;
 	engine.ANY = engine.NY;
@@ -1120,7 +1190,7 @@ void V9990CmdEngine::CmdCMMC<Mode>::start(const EmuTime& /*time*/)
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdCMMC<Mode>::execute(const EmuTime& time)
+void V9990CmdEngine::CmdCMMC<Mode>::execute(EmuTime::param time)
 {
 	if (!(engine.status & TR)) {
 		engine.status |= TR;
@@ -1163,14 +1233,14 @@ V9990CmdEngine::CmdCMMK<Mode>::CmdCMMK(V9990CmdEngine& engine,
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdCMMK<Mode>::start(const EmuTime& time)
+void V9990CmdEngine::CmdCMMK<Mode>::start(EmuTime::param time)
 {
 	std::cout << "V9990: CMMK not yet implemented" << std::endl;
 	engine.cmdReady(time); // TODO dummy implementation
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdCMMK<Mode>::execute(const EmuTime& /*time*/)
+void V9990CmdEngine::CmdCMMK<Mode>::execute(EmuTime::param /*time*/)
 {
 }
 
@@ -1185,9 +1255,9 @@ V9990CmdEngine::CmdCMMM<Mode>::CmdCMMM(V9990CmdEngine& engine,
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdCMMM<Mode>::start(const EmuTime& time)
+void V9990CmdEngine::CmdCMMM<Mode>::start(EmuTime::param time)
 {
-	clock.reset(time);
+	engine.clock.reset(time);
 	engine.srcAddress = (engine.SX & 0xFF) + ((engine.SY & 0x7FF) << 8);
 	engine.ANX = engine.NX;
 	engine.ANY = engine.NY;
@@ -1195,17 +1265,17 @@ void V9990CmdEngine::CmdCMMM<Mode>::start(const EmuTime& time)
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdCMMM<Mode>::execute(const EmuTime& time)
+void V9990CmdEngine::CmdCMMM<Mode>::execute(EmuTime::param time)
 {
 	// TODO can be optimized a lot
 
-	unsigned delta = CMMM_TIMING[engine.getTiming()];
+	unsigned delta = engine.getTiming(CMMM_TIMING);
 	unsigned pitch = Mode::getPitch(engine.vdp.getImageWidth());
 	int dx = (engine.ARG & DIX) ? -1 : 1;
 	int dy = (engine.ARG & DIY) ? -1 : 1;
 	const byte* lut = Mode::getLogOpLUT(engine.LOG);
-	while (clock.before(time)) {
-		clock += delta;
+	while (engine.clock.before(time)) {
+		engine.clock += delta;
 		if (!engine.bitsLeft) {
 			engine.data = vram.readVRAMBx(engine.srcAddress++);
 			engine.bitsLeft = 8;
@@ -1223,7 +1293,7 @@ void V9990CmdEngine::CmdCMMM<Mode>::execute(const EmuTime& time)
 			engine.DX -= (engine.NX * dx);
 			engine.DY += dy;
 			if (!--(engine.ANY)) {
-				engine.cmdReady(clock.getTime());
+				engine.cmdReady(engine.clock.getTime());
 				return;
 			} else {
 				engine.ANX = engine.NX;
@@ -1243,9 +1313,9 @@ V9990CmdEngine::CmdBMXL<Mode>::CmdBMXL(V9990CmdEngine& engine,
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdBMXL<Mode>::start(const EmuTime& time)
+void V9990CmdEngine::CmdBMXL<Mode>::start(EmuTime::param time)
 {
-	clock.reset(time);
+	engine.clock.reset(time);
 	engine.srcAddress = (engine.SX & 0xFF) + ((engine.SY & 0x7FF) << 8);
 	engine.ANX = engine.NX;
 	engine.ANY = engine.NY;
@@ -1253,16 +1323,17 @@ void V9990CmdEngine::CmdBMXL<Mode>::start(const EmuTime& time)
 
 template <>
 void V9990CmdEngine::CmdBMXL<V9990CmdEngine::V9990Bpp16>::execute(
-	const EmuTime& time)
+	EmuTime::param time)
 {
-	unsigned delta = BMXL_TIMING[engine.getTiming()];
+	// timing value is times 2, because it does 2 bytes per iteration:
+	unsigned delta = engine.getTiming(BMXL_TIMING) * 2;
 	unsigned pitch = V9990Bpp16::getPitch(engine.vdp.getImageWidth());
 	int dx = (engine.ARG & DIX) ? -1 : 1;
 	int dy = (engine.ARG & DIY) ? -1 : 1;
 	const byte* lut = V9990Bpp16::getLogOpLUT(engine.LOG);
 
-	while (clock.before(time)) {
-		clock += delta;
+	while (engine.clock.before(time)) {
+		engine.clock += delta;
 		word src = vram.readVRAMBx(engine.srcAddress + 0) +
 		           vram.readVRAMBx(engine.srcAddress + 1) * 256;
 		engine.srcAddress += 2;
@@ -1273,7 +1344,7 @@ void V9990CmdEngine::CmdBMXL<V9990CmdEngine::V9990Bpp16>::execute(
 			engine.DX -= (engine.NX * dx);
 			engine.DY += dy;
 			if (!--(engine.ANY)) {
-				engine.cmdReady(clock.getTime());
+				engine.cmdReady(engine.clock.getTime());
 				return;
 			} else {
 				engine.ANX = engine.NX;
@@ -1283,16 +1354,16 @@ void V9990CmdEngine::CmdBMXL<V9990CmdEngine::V9990Bpp16>::execute(
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdBMXL<Mode>::execute(const EmuTime& time)
+void V9990CmdEngine::CmdBMXL<Mode>::execute(EmuTime::param time)
 {
-	unsigned delta = BMXL_TIMING[engine.getTiming()];
+	unsigned delta = engine.getTiming(BMXL_TIMING);
 	unsigned pitch = Mode::getPitch(engine.vdp.getImageWidth());
 	int dx = (engine.ARG & DIX) ? -1 : 1;
 	int dy = (engine.ARG & DIY) ? -1 : 1;
 	const byte* lut = Mode::getLogOpLUT(engine.LOG);
 
-	while (clock.before(time)) {
-		clock += delta;
+	while (engine.clock.before(time)) {
+		engine.clock += delta;
 		byte data = vram.readVRAMBx(engine.srcAddress++);
 		for (int i = 0; (engine.ANY > 0) && (i < Mode::PIXELS_PER_BYTE); ++i) {
 			Mode::pset(vram, engine.DX, engine.DY, pitch,
@@ -1302,7 +1373,7 @@ void V9990CmdEngine::CmdBMXL<Mode>::execute(const EmuTime& time)
 				engine.DX -= (engine.NX * dx);
 				engine.DY += dy;
 				if (!--(engine.ANY)) {
-					engine.cmdReady(clock.getTime());
+					engine.cmdReady(engine.clock.getTime());
 					return;
 				} else {
 					engine.ANX = engine.NX;
@@ -1323,28 +1394,28 @@ V9990CmdEngine::CmdBMLX<Mode>::CmdBMLX(V9990CmdEngine& engine,
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdBMLX<Mode>::start(const EmuTime& time)
+void V9990CmdEngine::CmdBMLX<Mode>::start(EmuTime::param time)
 {
-	clock.reset(time);
+	engine.clock.reset(time);
 	engine.dstAddress = (engine.DX & 0xFF) + ((engine.DY & 0x7FF) << 8);
 	engine.ANX = engine.NX;
 	engine.ANY = engine.NY;
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdBMLX<Mode>::execute(const EmuTime& time)
+void V9990CmdEngine::CmdBMLX<Mode>::execute(EmuTime::param time)
 {
 	// TODO lots of corner cases still go wrong
 	//      very dumb implementation, can be made much faster
-	unsigned delta = BMLX_TIMING[engine.getTiming()];
+	unsigned delta = engine.getTiming(BMLX_TIMING);
 	unsigned pitch = Mode::getPitch(engine.vdp.getImageWidth());
 	int dx = (engine.ARG & DIX) ? -1 : 1;
 	int dy = (engine.ARG & DIY) ? -1 : 1;
 
 	word tmp = 0;
 	engine.bitsLeft = 16;
-	while (clock.before(time)) {
-		clock += delta;
+	while (engine.clock.before(time)) {
+		engine.clock += delta;
 		typename Mode::Type src = Mode::point(vram, engine.SX, engine.SY, pitch);
 		src = Mode::shift(src, engine.SX, 0); // TODO optimize
 		tmp <<= Mode::BITS_PER_PIXEL;
@@ -1365,7 +1436,7 @@ void V9990CmdEngine::CmdBMLX<Mode>::execute(const EmuTime& time)
 			engine.DY += dy;
 			engine.SY += dy;
 			if (!--(engine.ANY)) {
-				engine.cmdReady(clock.getTime());
+				engine.cmdReady(engine.clock.getTime());
 				// TODO handle last pixels
 				return;
 			} else {
@@ -1386,9 +1457,9 @@ V9990CmdEngine::CmdBMLL<Mode>::CmdBMLL(V9990CmdEngine& engine,
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdBMLL<Mode>::start(const EmuTime& time)
+void V9990CmdEngine::CmdBMLL<Mode>::start(EmuTime::param time)
 {
-	clock.reset(time);
+	engine.clock.reset(time);
 	engine.srcAddress = (engine.SX & 0xFF) + ((engine.SY & 0x7FF) << 8);
 	engine.dstAddress = (engine.DX & 0xFF) + ((engine.DY & 0x7FF) << 8);
 	engine.nbBytes    = (engine.NX & 0xFF) + ((engine.NY & 0x7FF) << 8);
@@ -1405,14 +1476,15 @@ void V9990CmdEngine::CmdBMLL<Mode>::start(const EmuTime& time)
 }
 
 template <>
-void V9990CmdEngine::CmdBMLL<V9990CmdEngine::V9990Bpp16>::execute(const EmuTime& time)
+void V9990CmdEngine::CmdBMLL<V9990CmdEngine::V9990Bpp16>::execute(EmuTime::param time)
 {
 	// TODO DIX DIY?
-	unsigned delta = BMLL_TIMING[engine.getTiming()];
+	// timing value is times 2, because it does 2 bytes per iteration:
+	unsigned delta = engine.getTiming(BMLL_TIMING) * 2;
 	const byte* lut = V9990Bpp16::getLogOpLUT(engine.LOG);
 	bool transp = engine.LOG & 0x10;
-	while (clock.before(time)) {
-		clock += delta;
+	while (engine.clock.before(time)) {
+		engine.clock += delta;
 		// VRAM always mapped as in Bx modes
 		word srcColor = vram.readVRAMDirect(engine.srcAddress + 0x00000) +
 		                vram.readVRAMDirect(engine.srcAddress + 0x40000) * 256;
@@ -1425,20 +1497,20 @@ void V9990CmdEngine::CmdBMLL<V9990CmdEngine::V9990Bpp16>::execute(const EmuTime&
 		engine.srcAddress = (engine.srcAddress + 1) & 0x3FFFF;
 		engine.dstAddress = (engine.dstAddress + 1) & 0x3FFFF;
 		if (!--engine.nbBytes) {
-			engine.cmdReady(clock.getTime());
+			engine.cmdReady(engine.clock.getTime());
 			return;
 		}
 	}
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdBMLL<Mode>::execute(const EmuTime& time)
+void V9990CmdEngine::CmdBMLL<Mode>::execute(EmuTime::param time)
 {
 	// TODO DIX DIY?
-	unsigned delta = BMLL_TIMING[engine.getTiming()];
+	unsigned delta = engine.getTiming(BMLL_TIMING);
 	const byte* lut = Mode::getLogOpLUT(engine.LOG);
-	while (clock.before(time)) {
-		clock += delta;
+	while (engine.clock.before(time)) {
+		engine.clock += delta;
 		// VRAM always mapped as in Bx modes
 		byte srcColor = vram.readVRAMBx(engine.srcAddress);
 		unsigned addr = V9990VRAM::transformBx(engine.dstAddress);
@@ -1450,7 +1522,7 @@ void V9990CmdEngine::CmdBMLL<Mode>::execute(const EmuTime& time)
 		engine.srcAddress = (engine.srcAddress + 1) & 0x7FFFF;
 		engine.dstAddress = (engine.dstAddress + 1) & 0x7FFFF;
 		if (!--engine.nbBytes) {
-			engine.cmdReady(clock.getTime());
+			engine.cmdReady(engine.clock.getTime());
 			return;
 		}
 	}
@@ -1467,18 +1539,18 @@ V9990CmdEngine::CmdLINE<Mode>::CmdLINE(V9990CmdEngine& engine,
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdLINE<Mode>::start(const EmuTime& time)
+void V9990CmdEngine::CmdLINE<Mode>::start(EmuTime::param time)
 {
-	clock.reset(time);
+	engine.clock.reset(time);
 	engine.ASX = (engine.NX - 1) / 2;
 	engine.ADX = engine.DX;
 	engine.ANX = 0;
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdLINE<Mode>::execute(const EmuTime& time)
+void V9990CmdEngine::CmdLINE<Mode>::execute(EmuTime::param time)
 {
-	unsigned delta = LINE_TIMING[engine.getTiming()];
+	unsigned delta = engine.getTiming(LINE_TIMING);
 	unsigned width = engine.vdp.getImageWidth();
 	unsigned pitch = Mode::getPitch(width);
 
@@ -1488,8 +1560,8 @@ void V9990CmdEngine::CmdLINE<Mode>::execute(const EmuTime& time)
 
 	if ((engine.ARG & MAJ) == 0) {
 		// X-Axis is major direction.
-		while (clock.before(time)) {
-			clock += delta;
+		while (engine.clock.before(time)) {
+			engine.clock += delta;
 			Mode::psetColor(vram, engine.ADX, engine.DY, pitch,
 			                engine.fgCol, engine.WM, lut, engine.LOG);
 
@@ -1501,14 +1573,14 @@ void V9990CmdEngine::CmdLINE<Mode>::execute(const EmuTime& time)
 			engine.ASX -= engine.NY;
 			//engine.ASX &= 1023; // mask to 10 bits range
 			if (engine.ANX++ == engine.NX || (engine.ADX & width)) {
-				engine.cmdReady(clock.getTime());
+				engine.cmdReady(engine.clock.getTime());
 				break;
 			}
 		}
 	} else {
 		// Y-Axis is major direction.
-		while (clock.before(time)) {
-			clock += delta;
+		while (engine.clock.before(time)) {
+			engine.clock += delta;
 			Mode::psetColor(vram, engine.ADX, engine.DY, pitch,
 			                engine.fgCol, engine.WM, lut, engine.LOG);
 			engine.DY += TY;
@@ -1519,7 +1591,7 @@ void V9990CmdEngine::CmdLINE<Mode>::execute(const EmuTime& time)
 			engine.ASX -= engine.NY;
 			//engine.ASX &= 1023; // mask to 10 bits range
 			if (engine.ANX++ == engine.NX || (engine.ADX & width)) {
-				engine.cmdReady(clock.getTime());
+				engine.cmdReady(engine.clock.getTime());
 				break;
 			}
 		}
@@ -1537,16 +1609,16 @@ V9990CmdEngine::CmdSRCH<Mode>::CmdSRCH(V9990CmdEngine& engine,
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdSRCH<Mode>::start(const EmuTime& time)
+void V9990CmdEngine::CmdSRCH<Mode>::start(EmuTime::param time)
 {
-	clock.reset(time);
+	engine.clock.reset(time);
 	engine.ASX = engine.SX;
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdSRCH<Mode>::execute(const EmuTime& time)
+void V9990CmdEngine::CmdSRCH<Mode>::execute(EmuTime::param time)
 {
-	unsigned delta = SRCH_TIMING[engine.getTiming()];
+	unsigned delta = engine.getTiming(SRCH_TIMING);
 	unsigned width = engine.vdp.getImageWidth();
 	unsigned pitch = Mode::getPitch(width);
 	typename Mode::Type mask = (1 << Mode::BITS_PER_PIXEL) -1;
@@ -1554,8 +1626,8 @@ void V9990CmdEngine::CmdSRCH<Mode>::execute(const EmuTime& time)
 	int TX = (engine.ARG & DIX) ? -1 : 1;
 	bool AEQ = engine.ARG & NEQ;
 
-	while (clock.before(time)) {
-		clock += delta;
+	while (engine.clock.before(time)) {
+		engine.clock += delta;
 		typename Mode::Type value;
 		typename Mode::Type col;
 		typename Mode::Type mask2;
@@ -1572,13 +1644,13 @@ void V9990CmdEngine::CmdSRCH<Mode>::execute(const EmuTime& time)
 		}
 		if (((value & mask2) == (col & mask2)) ^ AEQ) {
 			engine.status |= BD; // border detected
-			engine.cmdReady(clock.getTime());
+			engine.cmdReady(engine.clock.getTime());
 			engine.borderX = engine.ASX;
 			break;
 		}
 		if ((engine.ASX += TX) & width) {
 			engine.status &= ~BD; // border not detected
-			engine.cmdReady(clock.getTime());
+			engine.cmdReady(engine.clock.getTime());
 			engine.borderX = engine.ASX;
 			break;
 		}
@@ -1596,14 +1668,14 @@ V9990CmdEngine::CmdPOINT<Mode>::CmdPOINT(V9990CmdEngine& engine,
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdPOINT<Mode>::start(const EmuTime& time)
+void V9990CmdEngine::CmdPOINT<Mode>::start(EmuTime::param time)
 {
 	std::cout << "V9990: POINT not yet implemented" << std::endl;
 	engine.cmdReady(time); // TODO dummy implementation
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdPOINT<Mode>::execute(const EmuTime& /*time*/)
+void V9990CmdEngine::CmdPOINT<Mode>::execute(EmuTime::param /*time*/)
 {
 }
 
@@ -1618,7 +1690,7 @@ V9990CmdEngine::CmdPSET<Mode>::CmdPSET(V9990CmdEngine& engine,
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdPSET<Mode>::start(const EmuTime& time)
+void V9990CmdEngine::CmdPSET<Mode>::start(EmuTime::param time)
 {
 	unsigned pitch = Mode::getPitch(engine.vdp.getImageWidth());
 	const byte* lut = Mode::getLogOpLUT(engine.LOG);
@@ -1631,7 +1703,7 @@ void V9990CmdEngine::CmdPSET<Mode>::start(const EmuTime& time)
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdPSET<Mode>::execute(const EmuTime& /*time*/)
+void V9990CmdEngine::CmdPSET<Mode>::execute(EmuTime::param /*time*/)
 {
 }
 
@@ -1646,28 +1718,28 @@ V9990CmdEngine::CmdADVN<Mode>::CmdADVN(V9990CmdEngine& engine,
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdADVN<Mode>::start(const EmuTime& time)
+void V9990CmdEngine::CmdADVN<Mode>::start(EmuTime::param time)
 {
 	std::cout << "V9990: ADVN not yet implemented" << std::endl;
 	engine.cmdReady(time); // TODO dummy implementation
 }
 
 template <class Mode>
-void V9990CmdEngine::CmdADVN<Mode>::execute(const EmuTime& /*time*/)
+void V9990CmdEngine::CmdADVN<Mode>::execute(EmuTime::param /*time*/)
 {
 }
 
 // ====================================================================
 // CmdEngine methods
 
-void V9990CmdEngine::setCmdData(byte value, const EmuTime& time)
+void V9990CmdEngine::setCmdData(byte value, EmuTime::param time)
 {
 	sync(time);
 	data = value;
 	status &= ~TR;
 }
 
-byte V9990CmdEngine::getCmdData(const EmuTime& time)
+byte V9990CmdEngine::getCmdData(EmuTime::param time)
 {
 	sync(time);
 
@@ -1683,12 +1755,59 @@ byte V9990CmdEngine::getCmdData(const EmuTime& time)
 	return value;
 }
 
-void V9990CmdEngine::cmdReady(const EmuTime& /*time*/)
+byte V9990CmdEngine::peekCmdData(EmuTime::param time)
+{
+	sync(time);
+	return (status & TR) ? data : 0xFF;
+}
+
+void V9990CmdEngine::cmdReady(EmuTime::param /*time*/)
 {
 	currentCommand = NULL;
+	CMD = 0; // for deserialize
 	status &= ~(CE | TR);
 	vdp.cmdReady();
 }
 
-} // namespace openmsx
 
+template<typename Archive>
+void V9990CmdEngine::serialize(Archive& ar, unsigned /*version*/)
+{
+	// note: V9990Cmd objects are stateless
+	ar.serialize("srcAddress", srcAddress);
+	ar.serialize("dstAddress", dstAddress);
+	ar.serialize("nbBytes", nbBytes);
+	ar.serialize("borderX", borderX);
+	ar.serialize("ASX", ASX);
+	ar.serialize("ADX", ADX);
+	ar.serialize("ANX", ANX);
+	ar.serialize("ANY", ANY);
+	ar.serialize("SX", SX);
+	ar.serialize("SY", SY);
+	ar.serialize("DX", DX);
+	ar.serialize("DY", DY);
+	ar.serialize("NX", NX);
+	ar.serialize("NY", NY);
+	ar.serialize("WM", WM);
+	ar.serialize("fgCol", fgCol);
+	ar.serialize("bgCol", bgCol);
+	ar.serialize("ARG", ARG);
+	ar.serialize("LOG", LOG);
+	ar.serialize("CMD", CMD);
+	ar.serialize("status", status);
+	ar.serialize("data", data);
+	ar.serialize("bitsLeft", bitsLeft);
+	ar.serialize("partial", partial);
+	ar.serialize("endAfterRead", endAfterRead);
+
+	if (ar.isLoader()) {
+		if (CMD >> 4) {
+			setCurrentCommand();
+		} else {
+			currentCommand = NULL;
+		}
+	}
+}
+INSTANTIATE_SERIALIZE_METHODS(V9990CmdEngine);
+
+} // namespace openmsx
