@@ -1,14 +1,10 @@
-// $Id: V9990GLRasterizer.cc 5984 2007-01-08 22:03:21Z m9710797 $
+// $Id: V9990GLRasterizer.cc 8352 2008-11-12 18:39:08Z m9710797 $
 
 #include "V9990GLRasterizer.hh"
-#include "V9990.hh"
-
-using std::string;
 
 namespace openmsx {
 
-V9990GLRasterizer::V9990GLRasterizer(V9990& vdp_)
-	: vdp(vdp_), vram(vdp.getVRAM())
+V9990GLRasterizer::V9990GLRasterizer(V9990& /*vdp*/)
 {
 }
 
@@ -29,7 +25,7 @@ void V9990GLRasterizer::frameStart()
 {
 }
 
-void V9990GLRasterizer::frameEnd(const EmuTime& /*time*/)
+void V9990GLRasterizer::frameEnd(EmuTime::param /*time*/)
 {
 }
 
@@ -50,7 +46,8 @@ void V9990GLRasterizer::drawBorder(int /*fromX*/, int /*fromY*/, int /*toX*/, in
 }
 
 void V9990GLRasterizer::drawDisplay(
-	int /*fromX*/, int /*fromY*/, int /*displayX*/, int /*displayY*/,
+	int /*fromX*/, int /*fromY*/, int /*displayX*/,
+	int /*displayY*/, int /*displayYA*/, int /*displayYB*/,
 	int /*displayWidth*/, int /*displayHeight*/)
 {
 }
@@ -61,5 +58,3 @@ bool V9990GLRasterizer::isRecording() const
 }
 
 } // namespace openmsx
-
-
