@@ -1,25 +1,27 @@
-// $Id: V9990P1Converter.cc 6786 2007-08-04 10:29:59Z m9710797 $
+// $Id: V9990P1Converter.cc 8223 2008-09-14 19:17:24Z m9710797 $
 
 #include "V9990P1Converter.hh"
 #include "V9990.hh"
 #include "V9990VRAM.hh"
 #include "MemoryOps.hh"
 #include "GLUtil.hh"
+#include "build-info.hh"
 #include <cassert>
 #include <algorithm>
 
 namespace openmsx {
 
 template <class Pixel>
-V9990P1Converter<Pixel>::V9990P1Converter(V9990& vdp_, Pixel* palette64_)
+V9990P1Converter<Pixel>::V9990P1Converter(V9990& vdp_, const Pixel* palette64_)
 	: vdp(vdp_), vram(vdp.getVRAM())
 	, palette64(palette64_)
 {
 }
 
 template <class Pixel>
-void V9990P1Converter<Pixel>::convertLine(Pixel* linePtr,
-		unsigned displayX, unsigned displayWidth, unsigned displayY)
+void V9990P1Converter<Pixel>::convertLine(
+	Pixel* linePtr, unsigned displayX, unsigned displayWidth,
+	unsigned displayY, unsigned displayYA, unsigned displayYB)
 {
 	unsigned prioX = vdp.getPriorityControlX();
 	unsigned prioY = vdp.getPriorityControlY();
@@ -34,8 +36,8 @@ void V9990P1Converter<Pixel>::convertLine(Pixel* linePtr,
 	unsigned scrollBY = vdp.getScrollBY();
 	unsigned scrollAYBase = scrollAY & ~rollMask & 0x1FF;
 	unsigned scrollBYBase = scrollBY & ~rollMask & 0x1FF;
-	unsigned displayAY = scrollAYBase + ((displayY + scrollAY) & rollMask);
-	unsigned displayBY = scrollBYBase + ((displayY + scrollBY) & rollMask);
+	unsigned displayAY = scrollAYBase + ((displayYA + scrollAY) & rollMask);
+	unsigned displayBY = scrollBYBase + ((displayYB + scrollBY) & rollMask);
 
 	unsigned displayEnd = displayX + displayWidth;
 	unsigned end1 = std::max<int>(0, std::min<int>(prioX, displayEnd) - displayX);
@@ -93,7 +95,7 @@ void V9990P1Converter<Pixel>::renderPattern2(
 	unsigned nameTable, unsigned patternTable, byte pal)
 {
 	x &= 511;
-	Pixel* palette = palette64 + pal;
+	const Pixel* palette = palette64 + pal;
 
 	unsigned nameAddr = nameTable + (((y / 8) * 64 + (x / 8)) * 2);
 	y = (y & 7) * 128;
@@ -225,7 +227,7 @@ void V9990P1Converter<Pixel>::renderSprites(
 			unsigned patAddr = spritePatternTable
 			     + (128 * ((spriteNo & 0xF0) + spriteY))
 			     + (  8 *  (spriteNo & 0x0F));
-			Pixel* palette = palette64 + ((spriteAttr >> 2) & 0x30);
+			const Pixel* palette = palette64 + ((spriteAttr >> 2) & 0x30);
 			for (int x = 0; x < 16; x +=2) {
 				byte data = vram.readVRAMP1(patAddr++);
 				int xPos = spriteX + x;
@@ -244,8 +246,12 @@ void V9990P1Converter<Pixel>::renderSprites(
 }
 
 // Force template instantiation
+#if HAVE_16BPP
 template class V9990P1Converter<word>;
+#endif
+#if HAVE_32BPP
 template class V9990P1Converter<unsigned>;
+#endif
 #ifdef COMPONENT_GL
 template <> class V9990P1Converter<GLUtil::NoExpansion> {};
 template class V9990P1Converter<GLUtil::ExpandGL>;
